////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ESessionManager.cpp: implementation of the CESessionManager class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
//#include <stdexcpt.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CESessionManager::CESessionManager() {}
CESessionManager::~CESessionManager() {}

const int CESessionManager::EXIT_WAIT_KEYHIT	= 0x01;
int CESessionManager::run(int argc, char* argv[])
{
	//	------------------------------	view LOGO/version page
	CESupport::PrintLogoPage();
	//  ------------------------------  biuld options outline
#if defined(LOG_BUILD_OPTIONS)
	printf("************************ build settings ******\n");
#if defined(ALTA_DEBUG)
	printf("*** DEBUG build\n");
#endif //ALTA_DEBUG
#if defined(D0_ONLY_CALC)
	printf("*** D0-only calc build\n");
#endif //D0_ONLY_CALC
#if defined(ARR_ADJUST_SIZE)
	printf("*** build: ARR_ADJUST_SIZE\n");
#endif //ARR_ADJUST_SIZE
#if defined(USE_PLAIN_ARRAY_FORSEQ)
	printf("*** build: USE_PLAIN_ARRAY_FORSEQ\n");
#endif //USE_PLAIN_ARRAY_FORSEQ
	printf("***************** end of build settings ******\n");
#endif //LOG_BUILD_OPTIONS
	//  ------------------------------  biuld options outline end

	//	------------------------------	init section
	CMemoryManager::init();
	initProcessor();
	CESupport::initSupport();
	CESupport::init_XML_TAG_structure();
	CScriptProcessor::init();
	//	------------------------------	init section end

	bool exit_OK = false;
	int exit_code = 0;
	try {
		if(argc >= 2) {
			CScriptValue* scr_res = CScriptProcessor::run_file_script(argv[1]);
			if (scr_res != NULL) delete scr_res;
		}
		else {
			// what will we do here?
			//std::cout<<"No command script to invoke."<<std::endl;
			CEProcessor::dbg->log(0,"No command script to invoke!");
			//CTestCalculation* session = new CTestCalculation(); session->test(); delete session;
		}
		exit_OK = true;
	}
	catch(CEException exc) {
		int ec = exc.getErrCode(); CEString* msg = exc.getMessage(); const char* _fn=exc.getFileName(); long lnum=exc.getLineNumber();
		bool bPrintAContacts=false;
		if (ec == CScriptErrors::HALT_COMMAND) { // filter halt exit
			if (CEUtil::equals(exc.ClassName(), CScriptException::CLASS_NAME)) {
				exit_OK = true; exit_code = ((CScriptException*)(&exc))->get_iParam(); msg = NULL;
			} else { printf("Internal Error: non-script parsing exception"); } //(CScriptException class expected)
		} else if (ec==0) { bPrintAContacts=true; }
		if (msg != NULL) { printf("Stopped:"); printf(msg->getData()); if (_fn!=NULL) { printf(" {%s",_fn); if(lnum >=0)printf(", line:%ld",lnum); printf("}");  } printf("\n"); }
		else { printf("Stopped"); if (_fn!=NULL) { printf(" {%s",_fn); if(lnum>=0)printf(", line:%ld",lnum); printf("}"); } printf(".\n"); }
		if (bPrintAContacts) { const char* sCont=CConst::EXPR_AUTHOR_CONTACT; if(sCont!=NULL) { printf("------\n"); printf("If no solution is obvious for this error, please contact author:\n"); printf(sCont); printf("\n"); printf("------\n"); } }
	}
	catch(CEException* exc) {
		int ec = exc->getErrCode();
		CEString* msg = exc->getMessage(); const char* _fn=exc->getFileName(); long lnum=exc->getLineNumber();
		bool bPrintAContacts=false;
		if (ec == CScriptErrors::HALT_COMMAND) { // filter halt exit
			if (CEUtil::equals(exc->ClassName(), CScriptException::CLASS_NAME)) {
				exit_OK = true; exit_code = ((CScriptException*)exc)->get_iParam(); msg = NULL;
			} else { printf("Internal Error: non-script parsing exception"); } //(CScriptException class expected)
		} else if (ec==0) { bPrintAContacts=true; }
		if (msg != NULL) { printf("Stopped:"); printf(msg->getData()); if (_fn!=NULL) { printf(" {%s",_fn); if(lnum >=0)printf(", line:%ld",lnum); printf("}");  } printf("\n"); }
		else { printf("Stopped"); if (_fn!=NULL) { printf(" {%s",_fn); if(lnum>=0)printf(", line:%ld",lnum); printf("}"); } printf(".\n"); }
		if (bPrintAContacts) { const char* sCont=CConst::EXPR_AUTHOR_CONTACT; if(sCont!=NULL) { printf("------\n"); printf("If no solution is obvious for this error, please contact author:\n"); printf(sCont); printf("\n"); printf("------\n"); } }
		delete exc;
	}
	catch(...) {
		printf("Stopped with undefined exception\n");
		const char* sCont=CConst::EXPR_AUTHOR_CONTACT;
		if(sCont!=NULL) { printf("------\n"); printf("If no solution is obvious for this error, please contact author:\n"); printf(sCont); printf("\n"); printf("------\n"); }
	}

	bool log_finalization = false;
	log_finalization = (CEProcessor::dbg->get_level() >= 100 );
#if defined(ALTA_DEBUG)
	log_finalization = true;
#endif //ALTA_DEBUG

	//	------------------------------	finalize section
	// restore path if changed (e.g. exception occured while path is changed). Nt: must be BEFORE done processor is finalized
	CEString sPathCurr;
	if(CVMFileUtil::GetCurrentPath(sPathCurr)) {
		if(!CEUtil::equals(sPathCurr.getData(),CVMFileUtil::GetInitialPath())){
			CVMFileUtil::ChDir(CVMFileUtil::GetInitialPath());
			if (log_finalization) printf("Initial path restored.\n");
		}
	}
	CScriptProcessor::finalize();
	if (log_finalization) printf("Script Processor finished.\n");
	CDiagramPhysics::finalize();
	if (log_finalization) printf("Diagram Physics finalized.\n");
	CEProcessor::finalize();
	if (log_finalization) printf("Processor Environment finalized.\n");
	CESupport::finalize();
	if (log_finalization) printf("Support Library finalized.\n");
	char* tm = CDebug::GetAbsTimeString();
	CEProcessor::dbg->log(0, "Total time elapsed: ", tm);
	//Old: printf("Total time elapsed: %s\n", tm);
	CMemoryManager::mfree(tm);

	// clear timer objects
	CDebug::ClearAllTimers();

	// memory examining & clearing
#if defined(ALTA_DEBUG)
	if (exit_OK) CMemoryManager::examine_watches();
	else CMemoryManager::dbg->log(10, "Memory watch skipped");
#endif //ALTA_DEBUG
	CMemoryManager::finalize();
	CLSymbolDataBank::finalize(); // delete bank after the lost symbols are examined
	CDebug::finalize();
	//	------------------------------	finalize section end
	if (exit_code != 0) printf("exit code: %ld\n", exit_code);
	return exit_code;
}

void CESessionManager::initProcessor()
{
	CEProcessEnvironment* env = new CEProcessEnvironment(new CEFunctionManager());
	//CKinematics* kin = new CElasticKinematics();
	//CKinematics* kin = new CInelasticKinematics();
	CEProcessor::init(env);
	CDiagramPhysics::init();
	int dbg_phys_lev = CDiagramPhysics::dbg->get_level();
	int dbg_cr_lev = CDiagCorrRules::dbg->get_level();
	if (dbg_phys_lev > 10) CDiagramPhysics::dbg->set_level(5); // truncate level
	if (dbg_cr_lev > 10) CDiagCorrRules::dbg->set_level(5); // truncate level to warning value
	CDiagramPhysics* phys = new CSMPhysics(CSMPhysics::PHYS_PURE_QED);
	CDiagramPhysics::install_diagram_physics(phys);
	if (dbg_phys_lev > 10) CDiagramPhysics::dbg->set_level(dbg_phys_lev); // restore level
	if (dbg_cr_lev > 10) CDiagCorrRules::dbg->set_level(dbg_cr_lev); // restore level
}
