////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EAbstractFunction.cpp: implementation of the CEAbstractFunction class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CEAbstractFunction::CEAbstractFunction(const char* pNameExt, const CEPtrArray* pPtrArray) : CExpression() {
	m_pName=pNameExt; m_pitems=CEPtrArray::get_copy(pPtrArray, true);
	m_fInheritedProperties=0;
}

CEAbstractFunction::CEAbstractFunction(const char* pNameExt, const CExpression* data) : CExpression() {
	m_pName=pNameExt; m_pitems=new CEPtrArray();
	if(data!=NULL){m_pitems->add(data->clone());}else{ CEBasic::dbg->log(0, "Warning: new function(NULL) constructor");}
	m_fInheritedProperties=0;
}
CEAbstractFunction::CEAbstractFunction(const char* pNameExt, const CExpression* par1, const CExpression* par2) : CExpression() {
	m_pName=pNameExt; m_pitems=new CEPtrArray();
	if(par1!=NULL){m_pitems->add(par1->clone());}else{ CEBasic::dbg->log(0, "Warning: new function(NULL,X) constructor");}
	if(par2!=NULL){m_pitems->add(par2->clone());}else{ CEBasic::dbg->log(0, "Warning: new function(X,NULL) constructor");}
	m_fInheritedProperties=0;
}
CEAbstractFunction::CEAbstractFunction(const char* pNameExt) : CExpression() {
	m_pName=pNameExt; m_pitems=new CEPtrArray();
	m_fInheritedProperties=0;
}

CEAbstractFunction::~CEAbstractFunction()
{
	delete m_pitems;
}

void CEAbstractFunction::collectProperties(CEProperties* props, unsigned set) {
	//CExpression::collectProperties(props, set);
	props->setProperty(CEProperties::PROP_MASK_IS_FUNCTION, true);

	//
	//	Inherit properties from parameters (if required)
	//	TODO: optimize props calculation here!
	//
	if(m_fInheritedProperties > 0){	
		//
		long nParams = params_num();
		bool is_commutative=true;	//used if (m_fInheritedProperties & CEProperties::PROP_MASK_COMMUTATIVE)
		bool is_self_conj=true;		//used if (m_fInheritedProperties & CEProperties::PROP_MASK_SELF_CONJUGATED)
		for(long i=0; i<nParams; i++) {
			CExpression* par=get_param(i); if(par==NULL)continue; unsigned long lPropsI=par->props_long();
			if ((lPropsI & CEProperties::PROP_MASK_COMMUTATIVE)==0) is_commutative=false;
			if ((lPropsI & CEProperties::PROP_MASK_SELF_CONJUGATED)==0) is_self_conj=false;
		}
		if(m_fInheritedProperties & CEProperties::PROP_MASK_COMMUTATIVE) {
			if (!is_commutative) props->setProperty(CEProperties::PROP_MASK_COMMUTATIVE, false);
		}
		if(m_fInheritedProperties & CEProperties::PROP_MASK_SELF_CONJUGATED) {
			if (!is_self_conj) props->setProperty(CEProperties::PROP_MASK_SELF_CONJUGATED, false);
		}
	}
}

unsigned long CEAbstractFunction::props_long() {
	unsigned long res = CExpression::props_long();
	res = CEProperties::set_property_to(res, CEProperties::PROP_MASK_IS_FUNCTION, true);
	//
	//
	//	Inherit properties from parameters (if required)
	//	TODO: optimize props calculation here!
	//
	if(m_fInheritedProperties > 0){	
		//
		long nParams = params_num();
		bool is_commutative=true;	//used if (m_fInheritedProperties & CEProperties::PROP_MASK_COMMUTATIVE)
		bool is_self_conj=true;		//used if (m_fInheritedProperties & CEProperties::PROP_MASK_SELF_CONJUGATED)
		for(long i=0; i<nParams; i++) {
			CExpression* par=get_param(i); if(par==NULL)continue; unsigned long lPropsI=par->props_long();
			if ((lPropsI & CEProperties::PROP_MASK_COMMUTATIVE)==0) is_commutative=false;
			if ((lPropsI & CEProperties::PROP_MASK_SELF_CONJUGATED)==0) is_self_conj=false;
		}
		if(m_fInheritedProperties & CEProperties::PROP_MASK_COMMUTATIVE) {
			if (!is_commutative) res=CEProperties::set_property_to(res,CEProperties::PROP_MASK_COMMUTATIVE,false);
		}
		if(m_fInheritedProperties & CEProperties::PROP_MASK_SELF_CONJUGATED) {
			if (!is_self_conj) res=CEProperties::set_property_to(res,CEProperties::PROP_MASK_SELF_CONJUGATED,false);
		}
	}
	//
	return res;
}

bool CEAbstractFunction::equals(CExpression* other) {
	if (!other->getProperty(CEProperties::PROP_MASK_IS_FUNCTION)) return false;
	//if (!CEUtil::equals(ClassName(), other->ClassName())) return false;
	if (ClassName() != other->ClassName()) return false;
	if (!attributes_equals_to(other)) return false;
	CEAbstractFunction* o = (CEAbstractFunction*)other;

	if (!CEUtil::equals(m_pName, o->getTitle())) return false;
	int s = params_num();
	if (s != o->params_num()) return false;
	for(int i=0; i<s; i++) {
		if (!get_param(i)->equals(o->get_param(i))) return false;
	}
	return true;
}

CExpression* CEAbstractFunction::Simplify(int level) { return NULL; }

CExpression* CEAbstractFunction::expand(bool recursive) {
	int s = params_num();
	for(int i=0; i<s; i++) {
		CExpression* item = get_param(i);
		CExpression* arr = item->expand(recursive);
		if((arr!=item)&&(arr!=NULL)) { set_param(i,arr); delete arr; } //TODO: avoid extra cloning here
	}
	return this;
}

CExpression* CEAbstractFunction::Arrange(int level) {
	int s = params_num();
	for(int i=0; i<s; i++) {
		CExpression* item = get_param(i);
		CExpression* arr = item->Arrange(AL_EXTRACT);
		if ((arr != item) && (arr != NULL)) { set_param(i, arr); delete arr; } //TODO: avoid extra cloning here
	}
	return this;
}

CExpression* CEAbstractFunction::evaluate(int level)
{
	//
	//Apply to parametes
	//
	int s = params_num();
	for(int i=0; i<s; i++) {
		CExpression* item = get_param(i); if(item==NULL)continue;
		CExpression* item2 = item->evaluate(level);
		if ((item2 != item) && (item2 != NULL)) { set_param(i, item2); delete item2; } //TODO: avoid extra cloning here
	}
	// always return 'this' ==> may be called in SpecificFunction::evaluate()
	return this;
}

CExpression* CEAbstractFunction::applyRule(CEAbstractRule* rule) {
	CERule* r = (CERule*)rule;
	long s = params_num();
	for(long i=0; i<s; i++) {
		CExpression* item = get_param(i);
		CExpression* it_new = r->ApplyTo(item, false);
		if ((it_new != item) && (it_new != NULL)) { set_param(i, it_new); delete it_new; }
	}
	return this;
}

// parameters: used at numerical-procedure creation
const DWORD CEAbstractFunction::PAR_DEFAULT				= 0; // same to common variables
const DWORD CEAbstractFunction::PAR_INTEGER				= 1;
const DWORD CEAbstractFunction::PAR_REAL8				= 2;
const DWORD CEAbstractFunction::PAR_REAL_ABSTRACT		= 3;
const DWORD CEAbstractFunction::PAR_COMPLEX				= 4;
const DWORD CEAbstractFunction::PAR_COMPLEX_ABSTRACT	= 5;
const DWORD CEAbstractFunction::PAR_4VEC_REAL			= 6;
DWORD CEAbstractFunction::GetParameterType(long i) { return CEAbstractFunction::PAR_DEFAULT; }
DWORD CEAbstractFunction::GetResultType() {  return CEAbstractFunction::PAR_DEFAULT; }
DWORD CEAbstractFunction::GetMorePreciseType(int t1, int t2) {
	if(t1==t2)return t1;
	//Precision order: Default < Int < Real8 < RealX < Complex < ComplexX < Vector
	//Note: we may use 'CEAbstractFunction::PAR_X' values, but it's NOT safe - they may be changed

	DWORD res;
	if(t1==CEAbstractFunction::PAR_DEFAULT) {
		res=t2;
	}else if(t1==CEAbstractFunction::PAR_INTEGER) {
		res=((t2==CEAbstractFunction::PAR_DEFAULT)?t1:t2);
	}else if(t1==CEAbstractFunction::PAR_REAL8) {
		res=(((t2==CEAbstractFunction::PAR_DEFAULT)||(t2==CEAbstractFunction::PAR_INTEGER))?t1:t2);
	}else if(t1==CEAbstractFunction::PAR_REAL_ABSTRACT) {
		if(t2==CEAbstractFunction::PAR_COMPLEX) {
			res=CEAbstractFunction::PAR_COMPLEX_ABSTRACT;
		} else {
			res=(((t2==CEAbstractFunction::PAR_DEFAULT)||(t2==CEAbstractFunction::PAR_INTEGER)||(t2==CEAbstractFunction::PAR_REAL8))?t1:t2);
		}
	}else if(t1==CEAbstractFunction::PAR_COMPLEX) {
		if(t2==CEAbstractFunction::PAR_REAL_ABSTRACT) {
			res=CEAbstractFunction::PAR_COMPLEX_ABSTRACT;
		}else{
			res=(((t2==CEAbstractFunction::PAR_DEFAULT)||(t2==CEAbstractFunction::PAR_INTEGER)||(t2==CEAbstractFunction::PAR_REAL8))?t1:t2);
		}
	}else if(t1==CEAbstractFunction::PAR_COMPLEX_ABSTRACT) {
		res=((t2==CEAbstractFunction::PAR_4VEC_REAL)?t2:t1);
	}else if(t1==CEAbstractFunction::PAR_4VEC_REAL) {
		res=t1;
	}
	return res;
}
long CEAbstractFunction::GetTypeForComplexComponent(long iCmplType) { //real type corresponding to complex components
	long res=0;
	if(iCmplType==CEAbstractFunction::PAR_COMPLEX){ res=CEAbstractFunction::PAR_REAL8; }
	else if(iCmplType==CEAbstractFunction::PAR_COMPLEX_ABSTRACT){ res=CEAbstractFunction::PAR_REAL_ABSTRACT; }
	return res;
}

CEString* CEAbstractFunction::getTitleAdv(unsigned conventions, int mode) {
	CEString* res = NULL;
	switch (mode)
	{
	case STORE_FORTRAN_SYMBOL:
	case STORE_CPP:
	default:
		res = new CEString(getTitle());
	}
	return res;
}

CEString* CEAbstractFunction::toString(int mode)
{
	CEString* res = NULL;
	long size = m_pitems->size();
	switch (mode)
	{
	case STORE_WR_MATHEMATICA:
		if (getFlag()) {
			res = new CEString("\\(");
			res->AddString(m_pName);
		} else {
			res = new CEString("-\\(");
			res->AddString(m_pName);
		}
		res->AddString("[");
		if (size>1)
		{
			CExpression* pE = (CExpression*)m_pitems->get(0);
			res->AddString(pE->toString(mode));
			long i;
			for(i=1; i<size; i++)
			{
				res->AddString(", ");
				pE = (CExpression*)m_pitems->get(i);
				res->AddString(pE->toString(mode));
			}
		}
		else
		{
			if (size == 1)
			{
				CExpression* pE = (CExpression*)m_pitems->get(0);
				res->AddString(pE->toString(mode));
			}
		}
		res->AddString("]\\)");
		break;
	default:
		res = new CEString("<Not supported data format.>");
	}
	return res;
}

void CEAbstractFunction::toStream(ostream* os, int mode)
{
}

CExpression* CEAbstractFunction::data() { return (CExpression*)m_pitems->get(0); }
const CExpression* CEAbstractFunction::dataConst() const { return (const CExpression*)m_pitems->GetAtConst(0); }

int CEAbstractFunction::params_num() { return m_pitems->size(); }
const char* CEAbstractFunction::getTitle() { return m_pName; }
CExpression* CEAbstractFunction::get_param(int i) { return (CExpression*)(m_pitems->get(i)); }

void CEAbstractFunction::set_param(int i, CExpression *_new) {
	if ((_new == NULL) || (i < 0) || (i >= params_num())) return;
	m_pitems->SetAt(i, _new->clone(), true);
	
	//CExpression* old = get_param(i);
	//if (old != _new) { m_pitems->put(i, _new_copy); delete old; }
}

void CEAbstractFunction::add_param(CExpression* par) { // unaffected input
	if (par!=NULL) m_pitems->add(par->clone());
}

bool CEAbstractFunction::contain_vector(void * obj)
{
	int s = params_num();
	int i;
	for(i=0; i<s; i++) {
		CExpression* temp = get_param(i);
		if ((temp == obj) || (temp->contain_vector(obj)))
			return true;
	}
	return false;
}
