////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EAbstractFunction.h: interface for the CEAbstractFunction class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EABSTRACTFUNCTION_H__9DB7A10D_0AA0_11D5_8A94_932004EF1C0E__INCLUDED_)
#define AFX_EABSTRACTFUNCTION_H__9DB7A10D_0AA0_11D5_8A94_932004EF1C0E__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/*
*	Important Note:
*		Functions inherited from 'CEAbstractFunction' MUST define their properties manually!
*			(e.g. 'isComplex', 'isCommutative' etc.)
*			Or set specific 'm_fInheritedProperties' flag in constructor.
*		To use auto-calculation of properties one should inherit from 'CEFunction'!
*/
class CEAbstractFunction : public CExpression  
{
private:
	CEPtrArray* m_pitems;
protected:
	unsigned long m_fInheritedProperties;	//these flags will be inherited from parameters (default: 0)

protected:
	CEAbstractFunction(const char* pNameExt, const CEPtrArray* pPtrArray);
	CEAbstractFunction(const char* pNameExt, const CExpression* data);
	CEAbstractFunction(const char* pNameExt, const CExpression* par1, const CExpression* par2);
	CEAbstractFunction(const char* pNameExt);

	const char* m_pName; // external string, not cloned or deleted!!!
	CEPtrArray* getParamsSet() { return m_pitems; }
	const CEPtrArray* GetParamsConst() const { return m_pitems; }

public:

	virtual bool contain_vector(void * obj);
	void set_param(int i, CExpression* _new);
	CExpression* get_param(int i);
	void add_param(CExpression* par); // unaffected input
	int params_num();
	const char* getTitle();
	CExpression* data();
	const CExpression* dataConst() const;
	virtual ~CEAbstractFunction();

	virtual CEString* getTitleAdv(unsigned conventions, int mode); // creates title string for Nb, F90 etc.

	virtual CExpression* applyRule(CEAbstractRule* rule);
	virtual CExpression* evaluate(int level = 0);
	virtual CExpression* Simplify(int level = 0);
	virtual CExpression* expand(bool recursive = true);
	virtual CExpression* Arrange(int level);
	virtual CEString* toString(int mode = 0);
	virtual void toStream(ostream* os, int mode = 0);
	virtual bool equals(CExpression* other);

public:
	// parameters: used at numerical-procedure creation
	static const DWORD PAR_DEFAULT; // same to common variables
	static const DWORD PAR_INTEGER;
	static const DWORD PAR_REAL8;			//REAL*8 or double
	static const DWORD PAR_REAL_ABSTRACT;	//tReal
	static const DWORD PAR_COMPLEX;				//COMPLEX*16
	static const DWORD PAR_COMPLEX_ABSTRACT;	//tComplex
	static const DWORD PAR_4VEC_REAL;
	virtual DWORD GetParameterType(long i);
	virtual DWORD GetResultType();
	static DWORD GetMorePreciseType(int t1, int t2);
	static bool IsTypeComplex(long iType){ return ((iType==PAR_COMPLEX)||(iType==PAR_COMPLEX_ABSTRACT)); }
	static long GetTypeForComplexComponent(long iCmplType); //real type corresponding to complex components

public:
	virtual unsigned long props_long();
protected:
	virtual void collectProperties(CEProperties* props, unsigned set = CEProperties::PROP_MASK_ALL);

};

#endif // !defined(AFX_EABSTRACTFUNCTION_H__9DB7A10D_0AA0_11D5_8A94_932004EF1C0E__INCLUDED_)
