////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EFactory.cpp: implementation of the CEFactory class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

#define data_end	'\0'

CEFactory::CEFactory()
{
	m_data = (char*)CMemoryManager::mmalloc(MAX_EDATA_LENGTH);
	m_fp = CEProcessor::getEnvironment()->getFunctionProvider();
	if (m_fp == NULL) log("<Warning><No function provider available>");
	//else log("Function provider found.");
}

CEFactory::~CEFactory()
{
	CMemoryManager::mfree(m_data);
}

CExpression* CEFactory::fromFile(const char* filename, int mode)
{
	log("fromFile called.", 9);
	CExpression* res = NULL;
	//CFile file(filename, CFile::modeRead);
	//ifstream is;
	switch (mode)
	{
	case STORE_WR_MATHEMATICA:
		//is.attach();
		//is.open(filename);
		//if(is.is_open()) log("File is open.");
		//else log("Open failed.");
		//is->getline(m_data, MAX_EDATA_LENGTH);
		//int j;
		//for(j=0; j<1000; j++) { char c = '.'; *(m_data+j) = c; }
		//*(m_data+1000) = '\0';
		//is->getline(m_data, 100);
		//readLine(&file, m_data, '{');
		//long len = readLine(&file, m_data, '}');
		//file.Read(m_data, 100);
		//file.Close();
		res = parseMathematicaLine(0);
		break;
	}
	return res;
}

CExpression* CEFactory::fromStream(istream* is, int mode)
{
	/*log("fromStream called.", 9);
	is->getline(m_data, MAX_EDATA_LENGTH);
	CExpression* res = NULL;
	switch (mode)
	{
	case STORE_WR_MATHEMATICA:
		res = parseMathematicaLine();
		log(m_data);
		break;
	}
	return res;*/
	return NULL;
}

CExpression* CEFactory::parseMathematicaLine(long pos)
{
	log("Parsing Wolfram Research Mathematica Issue.");
	log(m_data);
	long curr = findNextPos(pos, '[');
	curr = findNextPos((curr+1), '[');
	curr = findNextSpecPos(curr, '(');
	printf("First sym is: %d\n", curr);
	CExpression* res = parseMathematicaSubLine(curr, 0);
	log("Parsing complete.");
	CEString* expr = res->toString(STORE_WR_MATHEMATICA);
	printf("Parsed expression is (length = %ld):\n", expr->length());
	log(expr->getData());
	return res;
}

CExpression* CEFactory::parseMathematicaSubLine(long pos, int type)
{
	CExpression* res = NULL;
	long curr = pos;
	bool sign = getSign(curr);
	bool level = getLevel(curr);

	//printf("start parsing pos: %ld, s: %d, l: %d, t: %d\n", pos, sign, level, type);
	
	long l = 0;
	switch(type)
	{
	case 0:
		l = findNextProdItem(curr);
		if (l>0) {
			bool tensorProductCreated = false;
			CExpression* expr = parseMathematicaSubLine(curr, 1);
			expr->setFlag();
			CEProduct* pProd = NULL;
			if (expr->getProperties()->isTensor()) {
				pProd = new CETensorProduct();
				tensorProductCreated = true;
			} else {
				pProd = new CEProduct();
			}
			pProd->addItem(expr);
			//log(pProd->toString(STORE_WR_MATHEMATICA)->getData());
			while (l>0)
			{
				curr = l;
				//expr = parseMathematicaSubLine(curr, 2);
				expr = parseMathematicaSubLine(curr, 1);
				if (expr->getProperties()->isTensor() && (!tensorProductCreated)) {
					CEProduct* pTProd = new CETensorProduct(pProd);
					//<turtile>/To be fixed:/We cann't delete old product.
					pProd = pTProd;
					tensorProductCreated = true;
				}
				pProd->addItem(expr);
				/*log("\nLocal prod is: ");
				log(pProd->toString(STORE_WR_MATHEMATICA)->getData());
				log("\n");*/
				l = findNextProdItem(curr);
			}
			//log("New product created.");
			res = pProd;
			break;
		}
	case 1:
		l = findPower(curr);
		if (l > 0) {
			CExpression* ex = parseMathematicaSubLine(curr, 2);
			if (!sign) ex->inverseFlag();
			res = new CEPower(ex, parseMathematicaSubLine(l+2, 2));
			break;
		}
	case 2:
		long offs = 0;
		if (findNextSpecPos(curr, '(') == curr) { offs = 2; }
		else if (findNextPos(curr, '(') == curr) { offs = 1; }
		if (offs > 0)
		{//only sum will be proceed here.
			//log("findNextSpecPos == curr\n");
			curr += offs;
			curr = findItem(curr);
			long l = findNextAddItem(curr);
			//printf("curr: %d, next: %d\n", curr, l);
			if (l>0) {
				CESum* pSum = new CESum();
				pSum->addItem(parseMathematicaSubLine(curr, 0));
				while (l>0)
				{
					curr = l;
					//printf("curr: %d\n", curr);
					pSum->addItem(parseMathematicaSubLine(curr, 0));
					//log("\nLocal sum is: ");
					//log(pSum->toString(STORE_WR_MATHEMATICA)->getData());
					//log("\n");
					l = findNextAddItem(curr);
					//printf("Next add item found: %d\n", l);
				}
				//log("New sum created.");
				res = pSum;
			}
			else {
				//printf("curr: %d\n", curr);
				res = parseMathematicaSubLine(findItem(curr), 0);
				if (!sign) res->inverseFlag();
				return res;
			}
		}
		else res = getSimpleExpression(curr);
		//function here.
	}
	
	log("result: ");
	log(res->toString(STORE_WR_MATHEMATICA)->getData());

	res->putFlag(sign);
	res->setLevel(level);
	//printf("\n<parsed pos: %ld, s: %d, l: %d, t: %d, rs: %d, rl: %d>\n", pos, sign, level, type, res->getFlag(), res->getLevel());
	//log("result: ");
	//log(res->toString(STORE_WR_MATHEMATICA)->getData());
	
	//log("\n");
	return res;
}

CExpression* CEFactory::getSimpleExpression(long pos)
{
	CExpression* res = NULL;
	long curr = skipSpaces(pos);
	long len = findItemEnd(curr) - curr;
	char* data = copySubString(curr, len);
	//log(data);
	char c = *data;
	if ((c >= '0') && (c <= '9'))
	{
		long l=atol(data);
		res=CENumber::GetNumber(l);
	}
	else
	{//functions are still not supported.
		res = getTensorExpression(curr, len);
		if (res == NULL) {
			res = getFunctionExpression(curr, len);
			if (res == NULL) res = new CESymbol(data);
		}
		//if (res == NULL) res = new CESymbol(data);
	}
	CMemoryManager::mfree(data);
	return res;
}

long CEFactory::readLine(void* pfile, char* buf, char end)
{
	long res = 0;
	char c = ' ';
	//pfile->Read(&c, 1); 
	for (;c!=end;)
	{
		*(buf+res) = c;
		res++;
		//pfile->Read(&c, 1);
	}
	*(buf+res) = data_end;
	//res++;
	return res;
}

/*
*	Not used.
*/
long CEFactory::findEndPos(long pos)
{
	//Returns first-after- "\)" or ")" position.
	int brackets_count = 0;
	long res = pos;
	char c = *(m_data+res);
	switch (c)
	{
	case '/':
		res = findEndPosX1(pos);
		break;
	case '(':
		log("Error: Not supported string end searching.");
		//long l = findNextPos(res, ')');
		//res = l;
		//if (l >= MAX_EDATA_LENGTH) log("Error: incomplete string parsing.");
		break;
	default:
		log("Error: Not supported branch called while string parsing.");
	}
	return res;
}

long CEFactory::skipBracket(long pos)
{
	//Used for "...../(.../)" strings.
	//Returns first-after-"\)" position.
	long temp = pos;
	long res = 0;
	long l = findNextSpecPos(temp, '(');
	//long l0 = findNextPos(temp, '(');
	long ll = findNextSpecPos(temp, ')');
	//long ll0 = findNextPos(temp, ')');
	long l1 = findNextPos(temp, '[');
	long ll1 = findNextPos(temp, ']');
	long l0 = lmin(findNextPos(temp, '('), l1);
	long ll0 = lmin(findNextPos(temp, ')'), ll1);
	if (lmin(l,ll) <= lmin(l0, ll0)) {
		if (l < ll) res = findBracketEnd(l+2);
		else res = ll+2;
	} else {
		if (l0 < ll0) res = findBracketEnd(l0+1);
		else res = ll0+1;
	}
	return res;
}

long CEFactory::findBracketEnd(long pos)
{
	//Returns first-after-"\)" position.
	long temp = pos;
	long l, ll, l0, ll0, l1, ll1;
	while (1)
	{
		l = findNextSpecPos(temp, '(');
		ll = findNextSpecPos(temp, ')');
		l1 = findNextPos(temp, '[');
		ll1 = findNextPos(temp, ']');
		l0 = lmin(findNextPos(temp, '('), l1);
		ll0 = lmin(findNextPos(temp, ')'), ll1);
		if (lmin(l,ll) <= lmin(l0, ll0)) {
			if (l < ll) temp = findBracketEnd(l + 2);
			else return ll + 2;
		} else {
			if (l0 < ll0) temp = findBracketEnd(l0 + 1);
			else return ll0 + 1;
		}
	}
}

long CEFactory::findEndPosX1(long pos)
{
	//Returns first-after-"\)" position.
	return findBracketEnd(pos);
	/*long temp = pos;
	printf("<findEndPosX1: %d>", temp);
	long l, ll, l0, ll0;
	while (1)
	{
		l = findNextSpecPos(temp+1, '(') + 2;
		ll = findNextSpecPos(temp, ')') + 2;
		l0 = findNextPos(temp +
			((findNextSpecPos(temp, '(') == temp)?2:1), '(') + 1;
		ll0 = findNextPos(temp, ')') + 1;
		//min = lmin(lmin(l,ll), lmin(l0, ll0));
		printf("... curr: %d, l: %d, ll: %d, l0: %d, ll0: %d ...", temp, l ,ll, l0, ll0);
		if (lmin(l,ll) <= lmin(l0, ll0)) {
			if (l < ll) temp = findEndPosX1(l);
			else {
				printf("<%d's result: %d>\n", pos, ll);
				return ll;
			}
		} else {
			if (l0 < ll0) temp = findEndPosX1(l0);
			else {
				printf("<%d's result: %d>\n", pos, ll);
				return ll0;
			}
		}
	}*/
}

long CEFactory::findEndPosX2(long pos, int type)
{
	//Used for sum part strings.
	//Returns first position after text end.
	long curr = pos;
	long l, ll, l0, ll0, p_pos, temp;
	while (1)
	{
		l = findNextSpecPos(curr, '(');
		ll = findNextSpecPos(curr, ')');
		p_pos = findNextPos(curr, '+');
		temp = findNextPos(curr, '-');
		if (temp < p_pos) p_pos = temp;
		l0 = lmin(findNextPos(curr, '('), findNextPos(curr, '['));
		ll0 = lmin(findNextPos(curr, ')'), findNextPos(curr, ']'));
		ll0 = lmin(ll0, findNextPos(curr, ','));
		//printf("l: %d, ll: %d, l0: %d, ll0: %d, p_pos: %d\n", l, ll, l0, ll0, p_pos);
		if ((p_pos < l) && (p_pos < ll) && (p_pos < l0) && (p_pos < ll0))
		{
			return (type == 0)?(p_pos-1):(p_pos+2);//?+3
		}
		else
		{
			if (lmin(l, ll) < lmin(l0, ll0)) {
				if (l > ll) return (type == 0)?ll+2:0;
				else curr = findBracketEnd(l+2);
			}
			else {
				if (l0 > ll0) return (type == 0)?ll0+1:0;
				else curr = findBracketEnd(l0+1);
			}
		}
	}
}

long CEFactory::lmin(long l1, long l2) {
	return (l1<l2)?l1:l2;
}

long CEFactory::lmax(long l1, long l2) {
	return (l1>l2)?l1:l2;
}

long CEFactory::findEndPosX3(long pos, int type)
{
	//Used for product part strings.
	//Returns first position after text end.
	long curr = pos;
	long l, ll, l0, ll0, p_pos, m_pos;
	while (1)
	{
		unsigned back = 0;
		l = findNextSpecPos(curr, '(');
		ll = findNextSpecPos(curr, ')');
		p_pos = lmin(findNextPos(curr, '+'), findNextPos(curr, '-'));
		l0 = lmin(findNextPos(curr, '('), findNextPos(curr, '['));
		ll0 = lmin(findNextPos(curr, ')'), findNextPos(curr, ']'));
		ll0 = lmin(ll0, findNextPos(curr, ','));
		//if (temp < p_pos) p_pos = temp;
		m_pos = lmin(findNextSpecPos(curr, ' '), findNextSpecPos(curr, '/')) + 1;
		p_pos = lmin(p_pos, m_pos);
		//printf("<pos: %d, curr: %d, l: %d, ll: %d, l0: %d, ll0: %d, p_pos: %d, m_pos: %d>/n", pos, curr, l ,ll, l0, ll0, p_pos, m_pos);
		//if (temp < p_pos) p_pos = temp;
		//else if (type > 0) return 0;
		//to add divide supporting.
		if ((p_pos < l) && (p_pos < ll) && (p_pos < l0) && (p_pos < ll0)) {
			//if ((p_pos == m_pos) && (type > 0)) return 0;
			return ((type == 0)?(p_pos - 1):((p_pos == m_pos)?findItem(p_pos):0));
		} else {
			if (lmin(l, ll) < lmin(l0, ll0)) {
				if (l > ll) return (type == 0)?ll:0;
				else curr = findBracketEnd(l+2);
			}
			else {
				if (l0 > ll0) return (type == 0)?ll0:0;
				else curr = findBracketEnd(l0+1);
			}
		}
	}
}

long CEFactory::findPower(long pos)
{
	long curr = pos;
	long l, ll, l0, ll0, p_pos, p, min;
	//printf("<findPower: %d>\n", curr);

	while (1)
	{
		unsigned back = 0;
		p = findNextSpecPos(curr, '^');
		l = findNextSpecPos(curr, '(');
		l0 = lmin(findNextPos(curr, '('), findNextPos(curr, '['));
		ll0 = lmin(findNextPos(curr, ')'), findNextPos(curr, ']'));
		ll = findNextSpecPos(curr, ')');
		p_pos = lmin(lmin(findNextPos(curr, '+'), findNextPos(curr, '-')),
			lmin(findNextSpecPos(curr, ' '), findNextSpecPos(curr, '/')));
		min = lmin(lmin(lmin(l0, ll0), lmin(l, ll)), lmin(p_pos, p));
		long lc = findNextPos(curr, ',');
		min = lmin(min, lc);
		//printf("... curr: %d, l: %d, ll: %d, l0: %d, ll0: %d, p_pos: %d, min: %d ...\n", curr, l ,ll, l0, ll0, p_pos, min);
		if ((min == l) || (min == l0)) {
			curr = skipBracket(min);
		} else {
			if (min == p) {
				//printf("<power found: %d>\n", p);
				return p;
			}
			else {
				//printf("<power not found>\n");
				return 0;
			}
		}
	}
}

long CEFactory::findItemEnd(long pos)
{
	long curr = pos;
	long l, ll, l0, ll0, lc, p_pos, p, min;
	while (1)
	{
		unsigned back = 0;
		p = findNextSpecPos(curr, '^');
		l = findNextSpecPos(curr, '(');
		ll = findNextSpecPos(curr, ')');
		l0 = lmin(findNextPos(curr, '('), findNextPos(curr, '['));
		ll0 = lmin(findNextPos(curr, ')'), findNextPos(curr, ']'));
		lc = findNextPos(curr, ',');
		p_pos = lmin(lmin(findNextPos(curr, '+'), findNextPos(curr, '-')),
			lmin(findNextSpecPos(curr, ' '), findNextSpecPos(curr, '/')));
		min = lmin(lmin(lmin(l0, ll0), lmin(l, ll)), lmin(p_pos, p));
		min = lmin(min, lc);
		if ((min == l) || (min == l0)) {
			curr = skipBracket(min);
		} else return min;
	}
}

/*long CEFactory::findNextPos(long pos, char sym)
{
	long res = pos;
	char c = *(m_data+res);
	while (c != sym)
	{
		res++;
		if (res < MAX_EDATA_LENGTH) c = *(m_data+res);
		else
		{
			c = sym;
			res++;
		}
	}
	return res;
}*/

long CEFactory::findNextPos(long pos, char sym, bool skip_brackets)
{
	//if (!skip_brackets) return findNextPos(pos, sym);
	long res = pos;
	char c = *(m_data+res);
	while (c != sym)
	{
		if (skip_brackets & ((c == '(') || (c == '['))) {
			res = findBracketEnd(res+1);
		} else res++;
		if (res < MAX_EDATA_LENGTH) c = *(m_data+res);
		else
		{
			c = sym;
			res++;
		}
	}
	return res;
}

long CEFactory::findNextSpecPos(long pos, char sym, bool skip_brackets)
{
	long res = pos;
	char c = '?';
	if (*(m_data+res) == '\\')
	{
		res++;
		c = *(m_data+res);
		//printf("in findNextSpecPos c = %c.", c);
	}
	while (c != sym)
	{
		if (skip_brackets & ((c == '(') || (c == '['))) {
			res = findBracketEnd(res+1);
		} else res++;
		if (res < MAX_EDATA_LENGTH - 1)
		{
			if (*(m_data+res) == '\\')
			{
				res++;
				c = *(m_data+res);
			}
		}
		else
		{
			c = sym;
			res += 2;
		}
	}
	return (res-1);
}

long CEFactory::findNextItem(long pos)
{
	long res = pos;
	char c;
	long i;
	for(i=0; i<MAX_ITEM_LENGTH; i++) {
		c = *(m_data+res);
		if ((c != ' ') && (c != '-') && (c != '+')) return res;
		res++;
	}
	return pos;
}

long CEFactory::findItem(long pos)
{
	long res = pos;
	char c;
	long i;
	for(i=0; i<MAX_ITEM_LENGTH; i++) {
		c = *(m_data+res);
		if ((c != ' ') && (c != '-') && (c != '+') && (c != '/')) {
			if (((findNextSpecPos(res-1, '(') + 1) != res) &
				((findNextSpecPos(res-1, ')') + 1) != res) &
				(findNextSpecPos(res, ' ') != res) &
				(findNextSpecPos(res, '/') != res)) return res;
		}
		res++;
	}
	return pos;
}

long CEFactory::findNextAddItem(long pos)
{
	//printf("findNextAddItem called. (%ld)\n", pos);
	return findEndPosX2(pos, 1);
}

long CEFactory::findNextProdItem(long pos)
{
	//printf("findNextProdItem called. (%ld)\n", pos);
	long res = findEndPosX3(pos, 1);
	printf("\n<next item for: %d is: %d>\n", pos, res);
	return res;
}

bool CEFactory::getSign(long pos)
{
	bool res;
	//we mean pos to be > 2;
	char c = *(m_data+pos-1);
	if (c == ' ') c = *(m_data+pos-2);
	switch (c)
	{
	case '-':
		res = false;
		break;

	default:
		res = true;
	}
	return res;
}

bool CEFactory::getLevel(long pos)
{
	bool res;
	//we mean pos to be > 2;
	char c = *(m_data+pos-1);
	if (c == ' ') c = *(m_data+pos-2);
	switch (c)
	{
	case '/':
		res = false;
		break;

	default:
		res = true;
	}
	return res;
}

CExpression* CEFactory::getFunctionExpression(long pos, long len)
{
	if (findNextSpecPos(pos, '[') == pos) return NULL;
	CExpression* res = NULL;
	long l = findNextPos(pos, '[');
	if ((l< (pos+len)) && (l> pos)) {
		char* title = copySubString(pos, l-pos);
		CEPtrArray* params = new CEPtrArray();
		long curr = l+1;
		if (curr == findNextPos(curr, ']')) curr = 0;
		while (curr > 0)
		{
			params->add(parseMathematicaSubLine(curr, 0));
			curr = findNextParam(curr);
		}
		log(title);
		printf(" param mun: %ld\n", params->size());
		if (m_fp != NULL) res = m_fp->getFunction(title, params);
		else {
			delete title;
			delete params;
			log("<Error><Can't create function>");
		}
	}
	return res;
}

char* CEFactory::copySubString(long pos, long len)
{
	if (len <= 0) {
		log("<Warning><String of invalid size request.>");
		return NULL;
	}
	char* data = (char*)CMemoryManager::mmalloc(len + 1);
	int i;
	for(i=0; i<len; i++)
	{
		*(data + i) = *(m_data + pos + i);
	}
	*(data + len) = '\0';
	return data;
}

long CEFactory::findNextParam(long pos)
{
	long curr = pos;
	long l, ll, l0, ll0, lc, min;
	while (1)
	{
		unsigned back = 0;
		//p = findNextSpecPos(curr, '^');
		l = findNextSpecPos(curr, '(');
		ll = findNextSpecPos(curr, ')');
		l0 = lmin(findNextPos(curr, '('), findNextPos(curr, '['));
		ll0 = lmin(findNextPos(curr, ')'), findNextPos(curr, ']'));
		lc = findNextPos(curr, ',');
		//p_pos = lmin(lmin(findNextPos(curr, '+'), findNextPos(curr, '-')),
		//	lmin(findNextSpecPos(curr, ' '), findNextSpecPos(curr, '/')));
		min = lmin(lmin(lmin(l0, ll0), lmin(l, ll)), lc);
		if ((min == l) || (min == l0)) {
			curr = skipBracket(min);
		} else {
			if (min == lc) return min+2;
			else return 0;
		}
	}
}

/*
* Vectors gamma & vector-gamma coupling support.
*/
CExpression* CEFactory::getTensorExpression(long pos, long len)
{
	//Do skip brackets while searching!
	long d = findNextSpecPos(pos, '_', true);
	long u = findNextSpecPos(pos, '&', true);
	long min = lmin(u, d);
	if (min >= (pos + len)) return NULL;

	printf("tensor suspected at: %ld, %ld\n", pos, min);
	
	CExpression* base = NULL;
	if (findNextSpecPos(pos, '(') == pos) {
		base = parseMathematicaSubLine(pos + 2, 2);
	}
	else {
		char* base_data = copySubString(pos, (min - pos));
		base = new CESymbol(base_data);
		//log("Base is: ");
		//log(base_data);
		delete base_data;
	}
	
	CExpression* res = NULL;
	char c = getSymbolAt(min + 2);
	if (c == '^') {
		//vector-gamma coupling.
		res = new CESGammaProduct(base);
		log("vector-gamma coupling created.");
	} else {
		if ((c >= '0') && (c <= '9')) return NULL;
		char* index_data = copySubString(min+2, (pos+len-min-2));
		CExpression* index = new CESymbol(index_data);
		//log("index found: ");
		//log(index_data);
		delete index_data;
		bool covariance = (d < u);
		if (base->equals(new CESymbol("\\[Gamma]"))) {
			res = new CEGamma(index, covariance);
		} else {
			res = new CEVector(base, index, covariance);
		}
	}
	//log("tensor created.");
	return res;
}

/*
* Used to uptuning item's start.
* Maybe we should also skip '\n'-s.
*/
long CEFactory::skipSpaces(long pos)
{
	long res = pos;
	char c = *(m_data+res);
	while (c == ' ')
	{
		res++;
		if (res < MAX_EDATA_LENGTH) c = *(m_data+res);
		else
		{
			log("<Warning><Error skipping spaces>");
			return pos;
		}
	}
	return res;
}

char CEFactory::getSymbolAt(long pos)
{
	return *(m_data + pos);
}
