////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ENumber.cpp: implementation of the CENumber class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

const char*	CENumber::CLASS_NAME		= "CENumber";
const char*	CENumber::CLASS_TAG_NAME	= "n";

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CENumber::CENumber(long value):CExpression() {
	m_num=value; m_num_im=0; m_denom=1; is_exact=true;
}
//CENumber::CENumber(long value, bool is_real):CExpression() {
//	if(is_real){ m_num=value; m_num_im=0; }else{ m_num=0; m_num_im=value; } m_denom=1; is_exact=true;
//}
CENumber::CENumber(long value, long den):CExpression() {
	m_num=value; m_num_im=0; m_denom=den; is_exact=true;
}
//CENumber::CENumber(long num, long den, bool is_real):CExpression() {
//	if(is_real){ m_num=num; m_num_im=0; }else{ m_num=0; m_num_im=num; } m_denom=den; is_exact=true;
//}
CENumber::CENumber(long num, long im_num, long den):CExpression() {
	m_num=num; m_num_im=im_num; m_denom=den; is_exact=true;
}

CENumber::~CENumber() {}

void CENumber::SetValue(CENumber* num) {
	m_num = num->getNum(); m_num_im = num->getNumIm(); m_denom = num->getDen(); is_exact = num->isExact();
	num->copyAttributesTo(this, false);
}
void CENumber::SetValue(long val) {
	m_num = val; m_num_im = 0; m_denom = 1; is_exact = true;
	setFlag(); setLevel(true);
	if(m_num<0) { m_num=-m_num; inverseFlag(); }
}

CExpression* CENumber::applyRule(CEAbstractRule* rule)
{
	return this;
}

CExpression* CENumber::evaluate(int level)
{
	return this;
}

CExpression* CENumber::Simplify(int level)
{
	return this;
}

CExpression* CENumber::Arrange(int level) {
	to_num_level();
	// note:
	// m_denom may be == 0

	if((m_num==0) && (m_num_im==0)){
		// unify zero
		if (!getFlag()) setFlag();
		if (m_denom == 0) {
			// declare error
			// or cast invalid object to zero?
			//CEProcessor::dbg->log(0, "Error (CENumber): 0/0 number encountered, object casted to zero.");
			CEProcessor::dbg->log(0, "Error (CENumber): 0/0 number encountered.");
			CEException::halt();
		}
		if (m_denom != 1) m_denom = 1;
	}
	if(m_denom < 0){ inverseFlag(); m_denom=-m_denom; }
	if((m_num < 0) || ((m_num == 0) && (m_num_im < 0))){ inverseFlag(); m_num=-m_num; m_num_im=-m_num_im; }
	reduce_fractions();
	return this;
}

char* CENumber::m_buf_num	= NULL;
char* CENumber::m_buf_im	= NULL;
char* CENumber::m_buf_den	= NULL;
CEString* CENumber::toString(int mode) {
	if(m_buf_num==NULL) {
		m_buf_num=(char*)CMemoryManager::mmalloc(32);
		m_buf_im=(char*)CMemoryManager::mmalloc(32);
		m_buf_den=(char*)CMemoryManager::mmalloc(32);
		//TODO: fix UNSAFETY in next lines or LEAVE this memory leak
		//AddToDeleteList()-list is deleted TOO EARLY. E.g. before CDebug::finalize(). ==> Exception may appear.
		//CEProcessor::AddToDeleteList(m_buf_num);
		//CEProcessor::AddToDeleteList(m_buf_im);
		//CEProcessor::AddToDeleteList(m_buf_den);
	}

	// STORE_FORTRAN_SYMBOL: always append dots/D0 after numbers
	// STORE_CPP: always append dots after numbers
	bool bDenIsApplied; // is denominator already applied?, used for (STORE_FORTRAN_SYMBOL) case only
	if (mode==STORE_FORTRAN_SYMBOL) {
		bDenIsApplied=false;
		if ((m_denom==2) || (m_denom==4) || (m_denom==5)) {
			double dblNum=m_num; dblNum=dblNum/m_denom; double dblNumIm=m_num_im; dblNumIm=dblNumIm/m_denom;
			if (m_num!=0) { CEUtil::PrintExpForm(m_buf_num, dblNum, 'D'); }
			if (m_num_im!=0) { CEUtil::PrintExpForm(m_buf_im, dblNumIm, 'D'); }
			bDenIsApplied=true;
		} else {
			if (m_num!=0) { sprintf(m_buf_num, "%ldD0", m_num); }
			if (m_num_im!=0) { sprintf(m_buf_im, "%ldD0", m_num_im); }
			if (m_denom!=1) { sprintf(m_buf_den, "%ldD0", m_denom); }
		}
	} else if (mode==STORE_CPP) {
		if (m_num!=0) { sprintf(m_buf_num, "%ld.", m_num); }
		if (m_num_im!=0) { sprintf(m_buf_im, "%ld.", m_num_im); }
		if (m_denom!=1) { sprintf(m_buf_den, "%ld.", m_denom); }
	} else {
		if (m_num!=0) { sprintf(m_buf_num, "%ld", m_num); }
		if (m_num_im!=0) { sprintf(m_buf_im, "%ld", m_num_im); }
		if (m_denom!=1) { sprintf(m_buf_den, "%ld", m_denom); }
	}
	CEString* res = NULL;

	//-------------------------------------------- storing RE(numerator)
	if(m_num!=0){
		res=new CEString(m_buf_num);
	} else {
		res=new CEString((m_num_im!=0)?"":"0");
	}

	//-------------------------------------------- storing IM and numerator brackets(to avoid "--1" or "+-1")
	switch (mode) {
	case STORE_TEX:
	case STORE_FORTRAN_SYMBOL:
		if(m_num_im!=0){
			if(m_num==0)res->AddString("0"); res->insertString("DCMPLX(", 0); res->AddString(","); res->AddString(m_buf_im); res->AddString(")");
		}else{
			if(m_num<0){ res->insertString("(", 0); res->AddString(")"); }
		}
		break;
	case STORE_CPP:
		if(m_num_im!=0){
			if(m_num==0)res->AddString("0"); res->insertString("complex(",0); res->AddString(","); res->AddString(m_buf_im); res->AddString(")");
		}else{
			if(m_num<0){ res->insertString("(", 0); res->AddString(")"); }
		}
		break;
	case STORE_PLAIN_TEXT:
		if(m_num_im!=0){
			if((m_num!=0)&&(m_num_im>=0))res->AddString("+");
			if(m_num_im!=1){ res->AddString(m_buf_im);res->AddString("*"); }
			res->AddString("i");
		}
		if( ((m_num!=0)&&(m_num_im != 0)) ||
			((m_num<0) || (m_num_im<0)) ){ res->insertString("(", 0); res->AddString(")"); }
		break;
	default: //Mathematica
		if(m_num_im!=0) {
			bool bUseBrace = ((m_num != 0) || (m_num_im <= 0)); //neglecting getFlag() here: "+-i" is also wrong.
			//Note: N*i is not braced here, but must be braced in product-like containers
			//==> Co-modify 'PROP_MASK_SINGLE_ITEM' property implementation together with 'bUseBrace' definition.
			res->insertString((bUseBrace?"\\((":"\\("), 0);
			if (m_num != 0) { if (m_num_im >= 0) res->AddString(" + "); else res->AddString(" "); }
			if (m_num_im != 1) { if (m_num_im != -1) { res->AddString(m_buf_im); res->AddString("\\ "); } else {res->AddString("- ");} }
			res->AddString("\\[ImaginaryI]"); res->AddString((bUseBrace?")\\)":"\\)"));
		}else{
			if(m_num<0){
				if(m_denom==1){	//visible brackets are required to avoid --1
					res->insertString("\\((", 0); res->AddString(")\\)");
				}else{			//invisible brackets are enough
					res->insertString("\\(", 0); res->AddString("\\)");
				}
			}
		}
	}

	//-------------------------------------------- storing denominator
	if (m_denom != 1) {
		switch (mode) {
		case STORE_FORTRAN_SYMBOL:
			if (bDenIsApplied) break; // denominator may be already applied (like 1/2 -> 0.5/1)
		case STORE_CPP:
		case STORE_TEX:
		case STORE_PLAIN_TEXT:
			//Note: this mode is NOT implemented for CEProduct: invisible brackets required for x/{2/3}
			res->AddString("/");
			if(m_denom>0){ res->AddString(m_buf_den); }else{ res->AddString("("); res->AddString(m_buf_den); res->AddString(")"); }
			break;
		default: //Mathematica
			res->insertString("\\(", 0);
			res->AddString("\\/");
			if(m_denom>0){ res->AddString(m_buf_den); }else{ res->AddString("\\("); res->AddString(m_buf_den); res->AddString("\\)"); }
			res->AddString("\\)");
		}
	}

	//-------------------------------------------- storing sign
	if(!getFlag()){
		switch (mode) {
		case STORE_WR_MATHEMATICA:
			res->insertString("- ", 0);
			//Old for (m_num_im==0): if (!getFlag()) { res->insertString("- \\(", 0); res->AddString("\\)"); }
			break;
		default: res->insertString("-", 0);
		}
	}

	return res;
}

void CENumber::toStream(ostream* os, int mode)
{
	*os << toString(0)->getData();
}

const char* CENumber::ClassName() const { return CLASS_NAME; }

bool CENumber::equals(CExpression *other)
{
	bool res = false;
	if (CEUtil::equals(ClassName(), other->ClassName())) {
		CENumber* n = (CENumber*)other;
		res = ((n->getDen() == getDen())
			&& (n->getNum() == getNum())
			&& (n->getNumIm() == getNumIm()));
		if (res) {
			// note: ((-0) != 0)!!!
			// "equals()" checks up absolute coinsidence.
			res = ((getFlag() == n->getFlag()) && (getLevel() == n->getLevel()));
		}
	}
	return res;
}

bool CENumber::less_than(CExpression *other) {
	if (other == NULL) return false;
	if (!CEUtil::LEQU(getFlag(), other->getFlag())) return other->getFlag(); // compare sign: "-"sym is < than "sym"

	const char* o_cn = other->ClassName();
	if (o_cn == CENumber::CLASS_NAME) {
		CENumber* n = (CENumber*)other;
		if (n->getDen() == getDen()) {
			if (getNum() != n->getNum()) return (getNum() < n->getNum());
			return (getNumIm() < n->getNumIm());
		}
		if ((getNumIm()==0) && (n->getNumIm()==0)) {
			double n1 = getNum(); n1=n1/getDen();
			double n2 = n->getNum(); n2=n2/n->getDen();
			return (n1<n2);
		}
		return CExpression::less_than(other);
	}
	return CExpression::less_than(other);
}

bool CENumber::CompareOtherN(CENumber* other, int* cmp_res) { // -1/0/1: less/equal/greather than other
	if ((other==NULL) || (cmp_res==NULL)) return false;
	if ((getNumIm()!=0) || (other->getNumIm()!=0)) return false;

	long n=getNum(); long d=getDen();
	if (!getFlag()) n=-n;
	if (!getLevel()) { long tmp=n; n=d; d=tmp; }
	if (d<0) { n=-n; d=-d; }

	long nO=other->getNum(); long dO=other->getDen();
	if (!other->getFlag()) nO=-nO;
	if (!other->getLevel()) { long tmp=nO; nO=dO; dO=tmp; }
	if (dO<0) { nO=-nO; dO=-dO; }

	if ((d==0) || (dO==0)) return false;
	if (d!=dO) { n=n*dO; nO=nO*d; }

	int res=INT_MIN;
	if(n>nO) { res=1; } else if(n<nO) { res=-1; } else res=0;
	*cmp_res=res;
	return true;
}

long CENumber::getLong() {
	if (!is_integer()) return LONG_MIN; //CEBasic::dbg->log(0, "Warning: CENumber::getLong() non-integer number, some data have been ignored", this);
	// same to previous: if ((m_num_im != 0) || (m_den != 1)) return LONG_MIN;
	if (!getLevel()) return LONG_MIN; //CEBasic::dbg->log(0, "Warning: CENumber::getLong() for denominator, level flag have been ignored");
	long res = m_num; if(!getFlag())res=-res;
	return res;
}

CExpression* CENumber::clone() const
{
	CExpression* res = new CENumber(m_num, m_num_im, m_denom);
	copyAttributesTo(res);
	return res;
}

long CENumber::getNum()
{
	return m_num;
}

long CENumber::getDen()
{
	return m_denom;
}

bool CENumber::isExact()
{
	return is_exact;
}

CExpression* CENumber::add(CExpression *other)
{
	if (other->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) {
		CENumber* n = (CENumber*)other;
		if (n->isExact()) {
			to_num_level();
			n->to_num_level();
			long num_re = ((getFlag())?m_num:((-1)*m_num));
			long num_im = ((getFlag())?m_num_im:((-1)*m_num_im));
			long num_re_2 = ((n->getFlag())?n->getNum():((-1)*n->getNum()));
			long num_im_2 = ((n->getFlag())?n->getNumIm():((-1)*n->getNumIm()));
			long den = m_denom;
			long den_2 = n->getDen();
			if (den != den_2) {
				m_num = num_re*den_2+num_re_2*den;
				m_num_im = num_im*den_2+num_im_2*den;
				m_denom = den*den_2;
			} else {// equal denominators
				// fast sum & proper processing of (a/0+b/0)
				m_num = num_re+num_re_2;
				m_num_im = num_im+num_im_2;
				m_denom = den;
			}
			if (!getFlag()) setFlag();// we have already applied flag to data
			//delete n;
			Arrange(AL_INLINE);
		} else {
			// else not implemented.
			CEProcessor::dbg->log(0, "Error (CENumber): can't multiply numbers.");
			CEException::halt();
		}
		delete other;
		return this;
	}
	return CExpression::add(other);
}

void CENumber::addNumber(CENumber* n) { // does not affects other
	if (n->isExact()) {
		to_num_level();
		n->to_num_level();
		long num_re = ((getFlag())?m_num:((-1)*m_num));
		long num_im = ((getFlag())?m_num_im:((-1)*m_num_im));
		long num_re_2 = ((n->getFlag())?n->getNum():((-1)*n->getNum()));
		long num_im_2 = ((n->getFlag())?n->getNumIm():((-1)*n->getNumIm()));
		long den = m_denom;
		long den_2 = n->getDen();
		if (den != den_2) {
			m_num = num_re*den_2+num_re_2*den;
			m_num_im = num_im*den_2+num_im_2*den;
			m_denom = den*den_2;
		} else {// equal denominators
			// fast sum & proper processing of (a/0+b/0)
			m_num = num_re+num_re_2;
			m_num_im = num_im+num_im_2;
			m_denom = den;
		}
		if (!getFlag()) setFlag();// we have already applied flag to data
		//delete n;
		Arrange(AL_INLINE);
	} else {
		// else not implemented.
		CEProcessor::dbg->log(0, "Error (CENumber): can't multiply numbers."); CEException::halt();
	}
}

CExpression* CENumber::multiply(CExpression* other) // deletes other
{
	if (other->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) {
		CENumber* n = (CENumber*)other;
		if (n->isExact()) {
			to_num_level();
			n->to_num_level();
			long num_re = m_num;
			long num_im = m_num_im;
			long num_re_2 = n->getNum();
			long num_im_2 = n->getNumIm();
			long den = m_denom;
			long den_2 = n->getDen();
			m_num = num_re*num_re_2-num_im*num_im_2;
			m_num_im = num_im*num_re_2+num_re*num_im_2;
			m_denom = den*den_2;
			if(!n->getFlag()) inverseFlag();
			//delete n;
			Arrange(AL_INLINE);
		} else {
			// else not implemented.
			CEProcessor::dbg->log(0, "Error (CENumber): can't multiply numbers.");
			CEException::halt();
		}
		delete other;
		return this;
	}
	return CExpression::multiply(other);
}

void CENumber::multiplyNumber(CENumber* n) { // does not affects other
	if (n->isExact()) {
		to_num_level();
		n->to_num_level();
		long num_re = m_num;
		long num_im = m_num_im;
		long num_re_2 = n->getNum();
		long num_im_2 = n->getNumIm();
		long den = m_denom;
		long den_2 = n->getDen();
		m_num = num_re*num_re_2-num_im*num_im_2;
		m_num_im = num_im*num_re_2+num_re*num_im_2;
		m_denom = den*den_2;
		if(!n->getFlag()) inverseFlag();
		Arrange(AL_INLINE);
	} else {
		// else not implemented.
		CEProcessor::dbg->log(0, "Error (CENumber): can't multiply numbers."); CEException::halt();
	}
}

unsigned long CENumber::props_long() {
	unsigned long res = CExpression::props_long();
	CEProperties::SetPropertyTo(res,CEProperties::PROP_MASK_IS_NUMBER);
	if(m_num_im!=0)CEProperties::SetPropertyTo(res,CEProperties::PROP_MASK_SELF_CONJUGATED,false);

	//PROP_MASK_SINGLE_ITEM: "Are visible brackets needed to insert this item (UNSIGNED) into product?"
	//Co-modify this property together with 'toString()' implementation.
	//The brackets are required:
	//1. "N*i" with (Num>0)&&(Nim>0)&&(Den==1)
	if((m_num==0)&&(m_num_im>0)&&(m_denom==1)){ CEProperties::SetPropertyTo(res,CEProperties::PROP_MASK_SINGLE_ITEM,false); }
	//In other cases visible brackets are either already involved or not needed.

	//Old: if (((m_num_im==0)&&(m_denom>0)) || ((m_num_im==1)&&(m_num==0)&&(m_denom==1))){}else{ CEProperties::SetPropertyTo(res,CEProperties::PROP_MASK_SINGLE_ITEM,false); }

	return res;
}

void CENumber::collectProperties(CEProperties *props, unsigned int set) {
	props->setProperty(CEProperties::PROP_MASK_IS_NUMBER, true);
	if (m_num_im != 0) props->setProperty(CEProperties::PROP_MASK_SELF_CONJUGATED, false);

	//PROP_MASK_SINGLE_ITEM: "Are visible brackets needed to insert this item (UNSIGNED) into product?"
	//Co-modify this property together with 'toString()' implementation.
	//The brackets are required:
	//1. "N*i" with (Num>0)&&(Nim>0)&&(Den==1)
	if((m_num==0)&&(m_num_im>0)&&(m_denom==1)){ props->setProperty(CEProperties::PROP_MASK_SINGLE_ITEM,false); }
	//In other cases visible brackets are either already involved or not needed.
}

CExpression* CENumber::conjugated(int mode) {
	CExpression* res = new CENumber(m_num, -m_num_im, m_denom);
	copyAttributesTo(res, true);
	return res;
}

long CENumber::getNumIm()
{
	return m_num_im;
}

void CENumber::inverse()
{
	long n_old = m_num;
	long ni_old = m_num_im;
	long d_old = m_denom;
	m_denom = n_old*n_old+ni_old*ni_old;
	if (m_denom != 0) {
		m_num = n_old*d_old;
		m_num_im = -ni_old*d_old;
	} else {
		/* Old
		// Old: (0/N)^(-1) inversion should result in N/0
		m_num = d_old; m_num_im = 0;
		*/
		CEProcessor::dbg->log(0, "(CENumber) Warning: attempt of 0/N number inversion", this);
		m_denom=d_old; return; // restore value & return
	}
	reduce_fractions();
	inverseLevel();
}

void CENumber::reduce_fractions()
{
	//Reducing fractions.
	if (m_denom > 1) {
		bool r_minus = false;
		bool i_minus = false;
		if (m_num < 0) { m_num = -m_num; r_minus = true; }
		if (m_num_im < 0) { m_num_im = -m_num_im; i_minus = true; }

		long i;
		for (i=2; i<= m_denom; i++) {
			if ((m_denom%i == 0) && (m_num%i == 0) && (m_num_im%i == 0)) {
				m_num = (long)(m_num/i);
				m_num_im = (long)(m_num_im/i);
				m_denom = (long)(m_denom/i);
				i--;
			}
		}

		if (r_minus) { m_num = -m_num; }
		if (i_minus) { m_num_im = -m_num_im; }
	}
}

CExpression* CENumber::numeric()
{
	//return new CEValue(this);
	if (m_num_im == 0) {//possible gap involve
		return new CERealValue(to_double());
	}
	return new CEComplexValue(double_RE(), double_IM());
}

/*
*	Warning: ignores imaginary part.
*/
double CENumber::to_double()
{
	return double_RE();
}

double CENumber::double_RE()
{
	double res = getNum();
	res = res/getDen();
	if (!getFlag()) res = -res;
	if (!getLevel()) res = 1/res;
	return res;
}

double CENumber::double_IM()
{
	double res = getNumIm();
	res = res/getDen();
	if (!getFlag()) res = -res;
	if (!getLevel()) res = 1/res;
	return res;
}

/*void CENumber::normalize_level()
{
	if (getDen() > getNum()) inverse();
}*/

void CENumber::to_num_level()
{
	if (!getLevel()) inverse();
}

bool CENumber::is_positive() {
	if (getNumIm() != 0) return false;
	bool res=getFlag(); if(getNum()<0)res=!res; if(getDen()<0)res=!res; return res;
}
bool CENumber::is_negative() {
	if (getNumIm() != 0) return false;
	bool res=!getFlag(); if(getNum()<0)res=!res; if(getDen()<0)res=!res; return res;
}

int CENumber::cmpZero() { // 1: >0; -1: <0; 0: equals or unknown (im part exist);
	if (getNumIm() != 0) return 0;
	if (getNum() == 0) return 0;
	if (getDen() == 0) { CEBasic::dbg->log(0, "{Error} CENumber::getDen() == 0", this); return 0; }

	bool res = getFlag();
	if (getNum()<0) res = !res;
	if (getDen()<0) res = !res;
	return ((res)?1:(-1));
}

bool CENumber::is_integer() {
	bool res = ((getDen() == 1) && (getNumIm() == 0));
	return res;
}

CExpression* CENumber::Re()
{
	if (getNum() == 0) return NULL;
	if (getNumIm() == 0) return this;
	
	CExpression* res = new CENumber(getNum(), 0, getDen());
	copyAttributesTo(res);
	return res;
}

CExpression* CENumber::Im()
{
	if (getNumIm() == 0) return NULL;
	if (getNum() == 0) return this;
	
	CExpression* res = new CENumber(0, getNumIm(), getDen());
	copyAttributesTo(res);
	return res;
}

CESerializable* CENumber::restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings) {
	long end_pos = CESupport::fing_item_end_pos(data, CLASS_TAG_NAME, pos);
	if (end_pos <= 0) {
		printf("Error restoring number (1)\n");
		return NULL;//Error
	}
	end_pos -= 1;
	if (pos > end_pos) {
		printf("Error restoring number (2)\n");
		return NULL;//Error
	}

	long i_pos = CEUtil::findSymbolFromTo(data, 'i', pos, end_pos);
	long d_pos = CEUtil::findSymbolFromTo(data, '/', pos, end_pos);
	bool bReadOK=true;

	//num
	long num_end=end_pos; if(i_pos>0) num_end=i_pos-1; else if(d_pos>0) num_end=d_pos-1;
	long _num = 0;
	if (num_end >= pos) {
		char* num_str = CEUtil::copyStringFromTo(data, pos, num_end);
		if (!CEUtil::atolSafe(num_str,&_num)) { bReadOK=false; }
		CMemoryManager::mfree(num_str);
	}

	//im
	long _im_num = 0;
	if (i_pos > 0) {
		long i_end = end_pos; if(d_pos>0) i_end=d_pos-1;
		if (i_end>i_pos) {
			char* im_str = CEUtil::copyStringFromTo(data, i_pos+1, i_end);
			if (!CEUtil::atolSafe(im_str,&_im_num)) { bReadOK=false; }
			CMemoryManager::mfree(im_str);
		}
	}

	//den
	long _den = 1;
	if ((d_pos>0) && (end_pos>d_pos)) {
		char* d_str = CEUtil::copyStringFromTo(data, d_pos+1, end_pos);
		if (!CEUtil::atolSafe(d_str,&_den)) { bReadOK=false; }
		CMemoryManager::mfree(d_str);
	}

	if (!bReadOK) {
		char* debug_str = CEUtil::copyStringFromTo(data, pos, end_pos);
		CEBasic::dbg->log(0,"ERROR: CENumber::restore_from_buffer() failed. Buffer: ", debug_str);
		CMemoryManager::mfree(debug_str);
		// TODO: test this case!!!
		return NULL;
	}
	CENumber* res = new CENumber(_num, _im_num, _den);
	return res;
}

const char* CENumber::get_TAG_Name()
{
	return CLASS_TAG_NAME;
}

long CENumber::store_stream(ofstream *p_ofs, DWORD settings) {
	if(m_buf_num==NULL) {
		m_buf_num=(char*)CMemoryManager::mmalloc(32); CEProcessor::AddToDeleteList(m_buf_num);
		m_buf_im=(char*)CMemoryManager::mmalloc(32); CEProcessor::AddToDeleteList(m_buf_im);
		m_buf_den=(char*)CMemoryManager::mmalloc(32); CEProcessor::AddToDeleteList(m_buf_den);
	}

	//if (m_den != 1){}
	sprintf(m_buf_num, "%ld", m_num);
	sprintf(m_buf_im, "%ld", m_num_im);
	sprintf(m_buf_den, "%ld", m_denom);

	long res = 0;

	if ( (m_num != 0) || ((m_num_im==0)&&(m_denom==1)) ) {
		*p_ofs << m_buf_num;
		res += CEUtil::length(m_buf_num);
	}

	if (m_num_im != 0) {
		*p_ofs << "i";
		*p_ofs << m_buf_im;
		res += 1;
		res += CEUtil::length(m_buf_im);
	}

	if (m_denom != 1) {
		*p_ofs << "/";
		*p_ofs << m_buf_den;
		res += 1;
		res += CEUtil::length(m_buf_den);
	}

	return res;
}

CENumber* CENumber::StringToNumber(const char *str)
{
	// NOT IMPLEMENTED
	return NULL;
}
