////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ENumber.h: interface for the CENumber class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ENUMBER_H__E2DE8DC2_0BEA_11D5_8778_0010A7027CA2__INCLUDED_)
#define AFX_ENUMBER_H__E2DE8DC2_0BEA_11D5_8778_0010A7027CA2__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//##ModelId=3E3FF2D0030D
class CENumber : public CExpression  
{
	//##ModelId=3E3FF2D00320
	long m_num;
	//##ModelId=3E3FF2D0031F
	long m_num_im;
	//##ModelId=3E3FF2D0031E
	long m_denom;

	static char* m_buf_num;	//used by toString/Stream
	static char* m_buf_im;	//used by toString/Stream
	static char* m_buf_den;	//used by toString/Stream
public:
	static CENumber* GetZero() { return new CENumber((long)0); }
	static CENumber* GetUnit() { return new CENumber((long)1); }
	static CENumber* GetTwo() { return new CENumber((long)2); }
	static CENumber* GetThree() { return new CENumber((long)3); }
	static CENumber* GetFour() { return new CENumber((long)4); }
	static CENumber* GetNumber(long n) { return new CENumber((long)n); }
	static CENumber* GetMinusUnit() { return new CENumber((long)(-1)); }
	static CENumber* GetHalf() { return new CENumber((long)1, (long)2); }
	static CENumber* GetFraction(long n, long d) { return new CENumber((long)n,(long)d); }
	static CENumber* GetImUnit() { return new CENumber((long)0,(long)1,(long)1); }
	static CENumber* GetImNumber(long n) { return new CENumber((long)0,(long)n,(long)1); }
public:
	static CENumber* StringToNumber(const char* str); //not implemented
	virtual long store_stream(ofstream* p_ofs, DWORD settings);
	virtual const char* get_TAG_Name();
	static const char*	CLASS_TAG_NAME;
	static CESerializable* restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings);

	virtual CExpression* Im();
	virtual CExpression* Re();
	//##ModelId=3E3FF2D00353
	bool is_positive();
	bool is_negative();
	int cmpZero(); // 1: >0; -1: <0; 0: equals 0;
	bool is_integer();
	bool IsReal(){ return (getNumIm()==0); }
	//##ModelId=3E3FF2D00352
	void to_num_level();
	//void normalize_level();
	//##ModelId=3E3FF2D00351
	virtual unsigned long props_long();
	//##ModelId=3E3FF2D00350
	double to_double();
	double double_RE();
	double double_IM();
	//##ModelId=3E3FF2D0034F
	virtual CExpression* numeric();
	//##ModelId=3E3FF2D0034E
	void inverse();
	//##ModelId=3E3FF2D0034D
	virtual long getNumIm();
	//##ModelId=3E3FF2D0034B
	virtual CExpression* conjugated(int mode = 0);
	//##ModelId=3E3FF2D00349
	virtual CExpression* multiply(CExpression* other); // deletes other
	void multiplyNumber(CENumber* n); // does not affects other
	//##ModelId=3E3FF2D00347
	virtual CExpression* add(CExpression* other); // deletes other
	void addNumber(CENumber* n); // does not affects other
	//##ModelId=3E3FF2D00346
	virtual bool isExact();
	//##ModelId=3E3FF2D00345
	virtual long getDen();
	//##ModelId=3E3FF2D00344
	virtual long getNum();
	//##ModelId=3E3FF2D00343
	virtual CExpression* clone() const;
	//##ModelId=3E3FF2D00342
	long getLong();
	//##ModelId=3E3FF2D00340
	virtual bool equals(CExpression* other);
	virtual bool less_than(CExpression* other);
	bool CompareOtherN(CENumber* other, int* cmp_res); // -1/0/1: less/equal/greather than other

	static const char*	CLASS_NAME;
	virtual const char* ClassName() const;
	
	//CENumber(long value, bool is_real=true);
	//CENumber(long num, long den, bool is_real = true);
	//CENumber(long num, long im_num, long den);
	CENumber(long value);
	CENumber(long num, long den);
	CENumber(long num, long im_num, long den);
	virtual ~CENumber();

	void SetValue(CENumber* num);
	void SetValue(long val);

	//##ModelId=3E3FF2D00331
	virtual CExpression* applyRule(CEAbstractRule* rule);
	//##ModelId=3E3FF2D0032F
	virtual CExpression* evaluate(int level = 0);
	//##ModelId=3E3FF2D0032D
	virtual CExpression* Simplify(int level = 0);
	//##ModelId=3E3FF2D0032A
	virtual CExpression* Arrange(int level);
	//##ModelId=3E3FF2D00328
	virtual CEString* toString(int mode = 0);
	//##ModelId=3E3FF2D00325
	virtual void toStream(ostream* os, int mode = 0);
private:
	//##ModelId=3E3FF2D00324
	void reduce_fractions();
	//##ModelId=3E3FF2D0031D
	bool is_exact;
protected:
	//##ModelId=3E3FF2D00321
	virtual void collectProperties(CEProperties* props, unsigned set = CEProperties::PROP_MASK_ALL);
};

#endif // !defined(AFX_ENUMBER_H__E2DE8DC2_0BEA_11D5_8778_0010A7027CA2__INCLUDED_)
