////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EPowerReplRule.cpp: implementation of the CEPowerReplRule class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

const char* CEPowerReplRule::CLASS_NAME = "CEPowerReplRule";
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CEPowerReplRule::CEPowerReplRule(const CExpression* base, long nMaxPowerAllowed) : CERule()
{
	if (base == NULL) { dbg->log(0, "Error: CEPowerReplRule(NULL base) creation"); CEException::halt(); }
	m_base = base->clone();
	m_power = nMaxPowerAllowed;
}

CEPowerReplRule::~CEPowerReplRule()
{
	delete m_base;
}

const char* CEPowerReplRule::ClassName() const { return CLASS_NAME; }
CEString* CEPowerReplRule::toString(int mode) {
	CEString* res  = new CEString("\\({ ");
	res->AddString(m_base->toString(mode));
	res->AddString("^(");
	res->add_int(m_power);
	res->AddString(")-->0 }\\)");
	return res;
}

bool CEPowerReplRule::contain_vector(void * obj) {
	if (m_base == obj) return true;
	return m_base->contain_vector(obj);
}

CERule* CEPowerReplRule::GetRuleCopy() const { return new CEPowerReplRule(m_base, m_power); }

void CEPowerReplRule::ResetTimesApplied() { m_times_applied = 0; }
DWORD CEPowerReplRule::GetTimesApplied() { return m_times_applied; }

/*
*	Both m_base and src's power base
*	are supposed to have (flag == true) and (level == true).
*/
CExpression* CEPowerReplRule::ApplyTo(CExpression *src, bool del_src)
{
	if (src == NULL) return NULL;

	if (CEUtil::equals(src->ClassName(), CEPower::CLASS_NAME)) {
		CEPower* pwr = (CEPower*)src;
		CExpression* pwr_base = pwr->basis();
		if (pwr_base->equals(m_base)) {
			CExpression* pwr_order = pwr->power();
			if (pwr_order->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) {
				double order = ((CENumber*)pwr_order)->to_double();
				double gap = 0.001;
				if ((order - gap) > ((double)m_power)) {
					m_times_applied++;
					if (del_src) delete src;
					return CENumber::GetZero();
				}
			}
		}
	}
	else {
		if (m_power == 0) {	// power == 0 should also delete all non-powered items
			if (src->equal_or_opposite(m_base)) {
				m_times_applied++;
				if (del_src) delete src;
				return CENumber::GetZero();
			}
		}
	}

	//  Applying to sub-items.
	CExpression* res = src->applyRule(this);
	if ((res != src) && (res != NULL)) {
		if (del_src) delete src;
		return res;
	}
	
	//	source object not changed
	return src;
}
