////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EProcessor.cpp: implementation of the CEProcessor class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDebug* CEProcessor::dbg = new CDebug("CEProcessor");
CEProcessEnvironment* CEProcessor::environment = NULL;
CEData* CEProcessor::m_data = NULL;
CKinematics* CEProcessor::m_kin = NULL;
CEMap* CEProcessor::m_v_cache = NULL;
CEPtrArray* CEProcessor::m_active_watches = NULL;

bool CEProcessor::m_ExamLessThen = false;
bool CEProcessor::m_ExamEquals = false;

CEProcessor::CEProcessor()
{

}

CEProcessor::~CEProcessor()
{

}

CAPtrSeq* CEProcessor::m_ToDeleteList		= NULL;
bool CEProcessor::m_bToDeleteListCleared	= false;
void CEProcessor::AddToDeleteList(CEObject* obj) {
	if (obj!=NULL) {
		if(m_ToDeleteList==NULL){ if(m_bToDeleteListCleared){ std::cout<<"Internal WARNING. CEProcessor::AddToDeleteList("<<obj->ClassName()<<") after processor is finalized."; return; } m_ToDeleteList=new CAPtrSeq(); }
		m_ToDeleteList->add(obj);
	}
}

void CEProcessor::AddToDeleteList(char* buf) { // buf to be deleted by CMemoryManager::mfree(buf);
	if (buf!=NULL) {
		if(m_ToDeleteList==NULL){ if(m_bToDeleteListCleared){ std::cout<<"Internal WARNING. CEProcessor::AddToDeleteList(char*) after processor is finalized."; return; } m_ToDeleteList=new CAPtrSeq(); }
		m_ToDeleteList->add(new CENamedItem((long)0,buf,false));
	}
}

CEProcessEnvironment* CEProcessor::getEnvironment()
{
	return environment;
}

void CEProcessor::setEnvironment(CEProcessEnvironment* env)
{
	environment = env;
}

CEData* CEProcessor::getDataStorage()
{
	return m_data;
}

void CEProcessor::init(CEProcessEnvironment* env)
{
	m_kin = NULL;
	m_data = new CEData();
	init_data_storage();
	setEnvironment(env);
	CKinematics::init();
	CEAbstractTask::init();
	CVIScalarCalcLib::init();

	// init basis symbols
	CAmplCalcLib::InitBasisSyms();
	CEFuncBondarev::SetBondarevBaseSymsDefaults();

	CFFortran::init();
}

/*
* Note: m_data should be saved/restored.
* Also there should be a set of results of the same structure.
*/
void CEProcessor::init_data_storage()
{
	//Greek letters.
	m_data->pute(CConst::GL_ALPHA, new CESymbol("\\[Alpha]"));
	m_data->pute(CConst::GL_BETA, new CESymbol("\\[Beta]"));
	m_data->pute(CConst::GL_GAMMA, new CESymbol("\\[Gamma]"));
	m_data->pute(CConst::GL_DELTA, new CESymbol("\\[Delta]"));
	m_data->pute(CConst::GL_CHI, new CESymbol("\\[Chi]"));
	m_data->pute(CConst::GL_ETA, new CESymbol("\\[Eta]"));
	m_data->pute(CConst::GL_LAMBDA, new CESymbol("\\[Lambda]"));
	m_data->pute(CConst::GL_OMEGA, new CESymbol("\\[Omega]"));
	m_data->pute(CConst::GL_PHI, new CESymbol("\\[Phi]"));
	m_data->pute(CConst::GL_PSI, new CESymbol("\\[Psi]"));
	m_data->pute(CConst::GL_PI, new CESymbol("\\[Pi]"));
	//m_data->pute("gamma", new CESymbol("\\[Gamma]"));
	//m_data->pute("greek.alpha", new CESymbol("\\[Alpha]"));
	//m_data->pute("greek.mu", new CESymbol("\\[Mu]"));
	//m_data->pute("greek.nu", new CESymbol("\\[Nu]"));

	//Visualisation settings.
	m_data->pute(CConst::DS_METRIC_BASE, new CESymbol("g"));
	m_data->pute(CConst::DS_ANTISYMMETRIC_BASE, new CESymbol("e"));
	m_data->pute(CConst::DS_SPACE, new CESymbol(" "));
	m_data->pute(CConst::DS_GAMMA_SPECIAL_BASE, new CESymbol("\\[Gamma]\\+5"));

	//Numeric values.
	m_data->pute(CConst::DV_ZERO, CENumber::GetZero());
	m_data->pute(CConst::DV_UNIT, CENumber::GetUnit());
	m_data->pute(CConst::DV_TWO, CENumber::GetTwo());
	m_data->pute(CConst::DV_IMAGINARY, CENumber::GetImUnit());
	m_data->pute(CConst::DV_HALF, CENumber::GetHalf());
	m_data->pute(CConst::DV_GAMMA5, new CEGammaSpecial(0));
	m_data->pute(CConst::DV_GAMMA_P, new CEGammaSpecial(1));
	m_data->pute(CConst::DV_GAMMA_M, new CEGammaSpecial(-1));
	
	//Vector parameters.
	//Vector: p\\_1.
	//m_data->putv("p\\_1", CConst::DNV_SYMBOL, new CESymbol("p\\_1"));
	//m_data->putv("p\\_1", CConst::DNV_SQUARE, m_data->gete(CConst::DV_ZERO)->clone());
	//m_data->putv("p\\_1", CConst::DNV_MODULE, m_data->gete(CConst::DV_ZERO)->clone());
	
	//Vector: p\\_2.
	//m_data->putv("p\\_2", CConst::DNV_SYMBOL, new CESymbol("p\\_2"));
	//m_data->putv("p\\_2", CConst::DNV_SQUARE, m_data->gete(CConst::DV_ZERO)->clone());
	//m_data->putv("p\\_2", CConst::DNV_MODULE, m_data->gete(CConst::DV_ZERO)->clone());

	//Vector: p\\_11.
	//m_data->putv("p\\_11", CConst::DNV_SYMBOL, new CESymbol("p\\_11"));
	//m_data->putv("p\\_11", CConst::DNV_SQUARE, new CEPower("m", 2));
	//m_data->putv("p\\_11", CConst::DNV_MODULE, new CESymbol("m"));
	//m_data->putv("p\\_11", CConst::DNV_SQUARE, m_data->gete(CConst::DV_ZERO)->clone());//new CEPower("m", 2));
	//m_data->putv("p\\_11", CConst::DNV_MODULE, m_data->gete(CConst::DV_ZERO)->clone());//new CESymbol("m"));

	//Vector: p\\_21.
	//m_data->putv("p\\_21", CConst::DNV_SYMBOL, new CESymbol("p\\_21"));
	//m_data->putv("p\\_21", CConst::DNV_SQUARE, new CEPower("m", 2));
	//m_data->putv("p\\_21", CConst::DNV_MODULE, new CESymbol("m"));
	//m_data->putv("p\\_21", CConst::DNV_SQUARE, m_data->gete(CConst::DV_ZERO)->clone());//new CEPower("m", 2));
	//m_data->putv("p\\_21", CConst::DNV_MODULE, m_data->gete(CConst::DV_ZERO)->clone());//new CESymbol("m"));

	//Vector: k.
	//m_data->putv("k", CConst::DNV_SYMBOL, new CESymbol("k"));
	//m_data->putv("k", CConst::DNV_SQUARE, m_data->gete(CConst::DV_ZERO)->clone());
	//m_data->putv("k", CConst::DNV_MODULE, m_data->gete(CConst::DV_ZERO)->clone());

	//Vector: p\\_12. //Used for some testings.
	//m_data->putv("p\\_12", CConst::DNV_SYMBOL, new CESymbol("p\\_12"));

	//Vector: p\\_22. //Used for some testings.
	//m_data->putv("p\\_22", CConst::DNV_SYMBOL, new CESymbol("p\\_22"));

	//m_data->putv("k\\_1", CConst::DNV_SQUARE, m_data->gete(CConst::DV_ZERO)->clone());
	//m_data->putv("k\\_2", CConst::DNV_SQUARE, m_data->gete(CConst::DV_ZERO)->clone());
	//m_data->putv("k\\_3", CConst::DNV_SQUARE, new CEPower("z", 2));
	//m_data->putv("w\\_1", CConst::DNV_SQUARE, new CEPower("m", 2));
	//m_data->putv("w\\_2", CConst::DNV_SQUARE, new CEPower("m", 2));
	
	//data->pute(CEUtil::composeId("p\\_1", CConst::DNV_SYMBOL), new CESymbol("p\\_1"));
	//data->pute(CEUtil::composeId("p\\_2", CConst::DNV_SYMBOL), new CESymbol("p\\_2"));
	//data->pute(CEUtil::composeId("p\\_11", CConst::DNV_SYMBOL), new CESymbol("p\\_11"));
	//data->pute(CEUtil::composeId("p\\_21", CConst::DNV_SYMBOL), new CESymbol("p\\_21"));
	//data->pute(CEUtil::composeId("p\\_12", CConst::DNV_SYMBOL), new CESymbol("p\\_12"));
	//data->pute(CEUtil::composeId("p\\_22", CConst::DNV_SYMBOL), new CESymbol("p\\_22"));

	//data->pute(CEUtil::composeId("p\\_1", CConst::DNV_SQUARE), data->gete("zero"));
	//data->pute(CEUtil::composeId("p\\_2", CConst::DNV_SQUARE), data->gete("zero"));
	//data->pute(CEUtil::composeId("p\\_11", CConst::DNV_SQUARE), new CEPower("m", 2));
	//data->pute(CEUtil::composeId("p\\_21", CConst::DNV_SQUARE), new CEPower("m", 2));

	//data->pute(CEUtil::composeId("p\\_1", CConst::DNV_MODULE), data->gete("zero"));
	//data->pute(CEUtil::composeId("p\\_2", CConst::DNV_MODULE), data->gete("zero"));
	//data->pute(CEUtil::composeId("p\\_11", CConst::DNV_MODULE), new CESymbol("m"));
	//data->pute(CEUtil::composeId("p\\_21", CConst::DNV_MODULE), new CESymbol("m"));

	//m_data->pute("zero", CENumber::GetZero());
	m_data->putvalue(new CESymbol("a2pi"), new CERealValue((double)1/(137*2*3.14)), true);
	
	dbg->log(10, "Data storage initialized.");
}

CKinematics* CEProcessor::kinematics()
{
#ifdef ALTA_DEBUG_CHECKS
	if (m_kin == NULL) { dbg->log(0, "Warning: non-existing kinematics request"); }
#endif //ALTA_DEBUG_CHECKS
	return m_kin;
}

CKinematics* CEProcessor::GetKinIfDeclared() { // same to kinematics() but no ERRORS
	return m_kin;
}

void CEProcessor::setKinematics(CKinematics * _new, bool do_delete_previous)
{
	if (m_kin == _new) return;

	CKinematics* old = m_kin;
	m_kin = _new;
	if (do_delete_previous) { if (old != NULL) delete old; }

	OnKinematicsChanged();
}

void CEProcessor::OnKinematicsChanged() { //clear kin-dependent caches [the new kin is set / the old kin is modified]
	// some re-init procedures <BEGIN>
	CEVector::ClearVecSquareCache();
	// some re-init procedures <END>
}

void CEProcessor::finalize()
{
	dbg->log(15, "Processor finalizing.");
	if (m_kin != NULL) {
		delete m_kin;
		dbg->log(20, "Kinematic deleted.");
	}
	else dbg->log(20, "No kinematic to delete.");

	if (m_active_watches != NULL) { delete m_active_watches; m_active_watches = NULL; }

	CVirtInt0::finalize();
	CVIScalarCalcLib::finalize(); // delete cache there
	CFFortran::finalize();
	CLHEPBasics::Finalize(); // delete booked symbols list

	//CMemoryManager::mfree(m_data->toLine());
	//CESupport::Save("1.");
	if (m_v_cache != NULL) {
		delete m_v_cache;
		m_v_cache = NULL;
		dbg->log(20, "Datacache deleted.");
	}
	else dbg->log(20, "Datacache is empty.");
	delete m_data;
	dbg->log(20, "Databank deleted.");
	//CESupport::Save("2.");
	delete environment;
	if (CExpression::m_LT_temp_1 != NULL) { CMemoryManager::mfree(CExpression::m_LT_temp_1); CExpression::m_LT_temp_1=NULL; }
	if (CExpression::m_LT_temp_2 != NULL) { CMemoryManager::mfree(CExpression::m_LT_temp_2); CExpression::m_LT_temp_2=NULL; }

	// empty to-delete-objects list
	if (m_ToDeleteList != NULL) { delete m_ToDeleteList; m_bToDeleteListCleared=true; m_ToDeleteList=NULL; }

	dbg->log(10, "Finalized.");
	//CESupport::Save("Processor data erased.");
}

CExpression* CEProcessor::find_numeric(CExpression * obj)
{
	//some kind of caching can be performed here.
	if (obj == NULL) {
		dbg->log(1, "Error: NULL's value request.");
		return NULL;
	}

	if (m_v_cache != NULL) {
		CEObject* val_in_cache = m_v_cache->get(obj);
		if (val_in_cache != NULL) return ((CExpression*)val_in_cache)->clone();
	}

	if (!(obj->getFlag() && obj->getLevel())) {
		CExpression* item = obj->clone();
		item->clear_all_attributes();
		CExpression* res = find_numeric(item);
		delete item;
		if (res != NULL) {
			obj->copyAttributesTo(res, true);
			if (m_v_cache != NULL) m_v_cache->put(obj, res->clone());
		}
		return res;
	}

	//Numeric values can be both in CEData & in kinematics.
	CExpression* res = CEProcessor::getDataStorage()->getvalue(obj);
	if (res != NULL) {
		res = res->clone();
		if (m_v_cache != NULL) m_v_cache->put(obj, res->clone());
	}
	else {
		dbg->log(5, "Warning: can't find numeric", obj);
	}
	return res;
}

void CEProcessor::init_values_cache()
{
	if (m_v_cache != NULL) delete m_v_cache;
	m_v_cache = CEMap::createNewMap();
}

void CEProcessor::disable_values_cache()
{
	if (m_v_cache != NULL) delete m_v_cache;
	m_v_cache = NULL;
}

// global parameters
bool CEProcessor::m_MandelstamsEverywhere	= false;
bool CEProcessor::m_AllowClassDiagrams		= true;
int CEProcessor::m_ScreenSize = 1; //default(1), 1: 1024x768, 2: 1400x1050
bool CEProcessor::m_DropZeroMEDiagrams		= true;
bool CEProcessor::m_bAutoKinArrange			= true;

// global parameters ID
const char* CEProcessor::GPAR_MASSIVE_VECTOR_MATRIX		= "MASSIVE_VECTOR_MATRIX";
const char* CEProcessor::GPAR_EXAMINE_LESS_THEN			= "EXAMINE_LESS_THEN";
const char* CEProcessor::GPAR_EXAMINE_EQUALS			= "EXAMINE_EQUALS";
const char* CEProcessor::GPAR_MANDELSTAMS				= "PAR_MANDELSTAMS";
const char* CEProcessor::GPAR_SCREEN_SIZE				= "PAR_SCREEN_SIZE";
const char* CEProcessor::GPAR_TRACES_BONDEREV			= "PAR_TRACES_BONDEREV";
const char* CEProcessor::GPAR_TRACES_BONDEREV_F3EXPAND	= "PAR_TRACES_BONDEREV_F3EXPAND";
const char* CEProcessor::GPAR_STR_FORTRAN_TEMP			= "PAR_STR_FORTRAN_TEMP"; //deprecated!
const char* CEProcessor::GPAR_STR_CODE_TEMP				= "PAR_STR_CODE_TEMP";
const char* CEProcessor::GPAR_ALLOW_CLASS_DIAGRAMS		= "PAR_ALLOW_CLASS_DIAGRAMS";
const char* CEProcessor::GPAR_DROP_ZERO_DIAGRAMS		= "PAR_DROP_ZERO_DIAGRAMS";
const char* CEProcessor::GPAR_AUTO_KINARRANGE			= "PAR_AUTO_KINARRANGE";
const char* CEProcessor::GPAR_EVALUATE_COMPLICATED_VI	= "PAR_EVALUATE_COMPLICATED_VI";
const char* CEProcessor::GPAR_LOOPTOOLS_COMPARTIBLE		= "PAR_LOOPTOOLS_COMPARTIBLE";
const char* CEProcessor::GPAR_RUN_EVT_NUCLON_ENTRIES	= "RUN_EVT_NUCLON_ENTRIES";
const char* CEProcessor::GPAR_CODE_GEN_FAST				= "PAR_CODE_GEN_FAST"; //no 'arrange' after DiffCS factor applying (to be tested)

int CEProcessor::setGlobalParameter(char* par, int value) { // -1: invalid parameter key, -2: invalid value
	if (par == NULL) return -1;

	if (CEUtil::equals(par,CEProcessor::GPAR_MASSIVE_VECTOR_MATRIX)) {
		bool ok = CPAbstractBoson::SetMassiveVecMatrixType(value);
		if (!ok) return -2;
		return 1;
	}

	if (CEUtil::equals(par,CEProcessor::GPAR_EXAMINE_LESS_THEN)) { CEProcessor::m_ExamLessThen=(value!=0); return 1; }
	if (CEUtil::equals(par,CEProcessor::GPAR_EXAMINE_EQUALS)) { CEProcessor::m_ExamEquals=(value!=0); return 1; }
	if (CEUtil::equals(par,CEProcessor::GPAR_MANDELSTAMS)) { CEProcessor::m_MandelstamsEverywhere=(value!=0); return 1; }
	if (CEUtil::equals(par,CEProcessor::GPAR_ALLOW_CLASS_DIAGRAMS)) { CEProcessor::m_AllowClassDiagrams=(value>0); return 1; }
	if (CEUtil::equals(par,CEProcessor::GPAR_DROP_ZERO_DIAGRAMS)) { CEProcessor::m_DropZeroMEDiagrams=(value>0); return 1; }
	if (CEUtil::equals(par,CEProcessor::GPAR_AUTO_KINARRANGE)) { CEProcessor::m_bAutoKinArrange=(value>0); return 1; }
	if (CEUtil::equals(par,CEProcessor::GPAR_EVALUATE_COMPLICATED_VI)) { CVirtIntegral::EVALUATE_ALL_POSSBILE_VI=(value>0); return 1; }
	if (CEUtil::equals(par,CEProcessor::GPAR_LOOPTOOLS_COMPARTIBLE)) { CVirtIntegral::PAR_LOOPTOOLS_COMPARTIBLE=(value>0); return 1; }
	if (CEUtil::equals(par,CEProcessor::GPAR_RUN_EVT_NUCLON_ENTRIES)) { CLHEPRunProject::CLHEPRunProject_Settings::bEvtNuclonEntriesDefault=(value>0); return 1; }
	if (CEUtil::equals(par,CEProcessor::GPAR_CODE_GEN_FAST)) { CFFortran::m_PAR_CODE_GEN_FAST=(value>0); return 1; }

	if (CEUtil::equals(par,CEProcessor::GPAR_SCREEN_SIZE)) {
		if ((value <1) || (value > 2)) return -2;
		CEProcessor::m_ScreenSize = value;
		return 1;
	}
	
	if (CEUtil::equals(par,CEProcessor::GPAR_TRACES_BONDEREV)) { CFSp::m_TracesBondarev=(value>0); return 1; }

	if (CEUtil::equals(par,CEProcessor::GPAR_TRACES_BONDEREV_F3EXPAND)) {
		CEFuncBondarev::m_DoExpandF3 = (value > 0);
		if (value >0) CFSp::m_TracesBondarev = true;
		return 1;
	}

	return -1;
}

int CEProcessor::setGlobalParameter(const char* par, const char* value) { // -1: invalid parameter key, -2: invalid value
	if (par == NULL) return -1;

	if (CEUtil::equals(par,CEProcessor::GPAR_STR_FORTRAN_TEMP)) {
		dbg->log(0, "------ ERROR: deprecated parameter usage! ------");
		dbg->log(0, "The following parameter is deprecated: ", CEProcessor::GPAR_STR_FORTRAN_TEMP);
		dbg->log(0, "The correct parameter is: ", CEProcessor::GPAR_STR_CODE_TEMP);
		dbg->log(0, "------------------------------------------------");
		CFFortran::SetTEMPstr(value); return 1;
	}
	if (CEUtil::equals(par,CEProcessor::GPAR_STR_CODE_TEMP)) { CFFortran::SetTEMPstr(value); return 1; }

	return -1;
}

/*
*	Progress watches usage:
*
*	start new watch
*	//	*****************************************************	<scroll>
*	CEProgressWatch* pw = NULL;
*	if (do_watch_condition) { pw = CEProcessor::newProgressWatch("Text, %ld", some_value); }
*	//	*****************************************************	<\scroll>
*
*	update watch
*	//	*****************************************************	<scroll>
*	if (pw != NULL) pw->step(i+1, size());
*	//	*****************************************************	<\scroll>
*
*	dismiss watch
*	//	*****************************************************	<scroll>
*	if (pw != NULL) CEProcessor::dismissProgressWatch(pw);
*	//	*****************************************************	<\scroll>
*/
CEProgressWatch* CEProcessor::newProgressWatch(const char* _description) {
	if (m_active_watches == NULL) m_active_watches = new CEPtrArray();

	if (m_active_watches->size() > 0) return NULL; // no multiple watches implemented.

	CEProgressWatch* new_watch = new CEProgressWatch(_description);
	m_active_watches->add(new_watch);
	return new_watch;
}

CEProgressWatch* CEProcessor::newProgressWatch(const char* _description, long _data) {
	if (_description == NULL) _description = "Untitled process (%ld steps).";

	long title_len = CEUtil::length(_description);
	char* pw_title = (char*)CMemoryManager::mmalloc(title_len+20);
	sprintf(pw_title, _description, _data);
	CEProgressWatch* pw_res = newProgressWatch(pw_title);
	CMemoryManager::mfree(pw_title);
	return pw_res;
}

void CEProcessor::dismissProgressWatch(CEProgressWatch* pw) {
	if (pw == NULL) return;

	pw->step(1, 1);
	int s = m_active_watches->size();
	int i;
	for(i=(s-1); i>=0; i--) {
		if (m_active_watches->GetAt(i) == pw) {
			m_active_watches->RemoveAt(i, false); break;
		}
	}
	delete pw;
}
