////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EProcessor.h: interface for the CEProcessor class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EPROCESSOR_H__9DB7A104_0AA0_11D5_8A94_932004EF1C0E__INCLUDED_)
#define AFX_EPROCESSOR_H__9DB7A104_0AA0_11D5_8A94_932004EF1C0E__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//#include "EAbstractProcessor.h"

//##ModelId=3E3FF2DC0232
class CEProcessor : public CEAbstractProcessor
{
private:
	//##ModelId=3E3FF2DC0253
	static CEData* m_data;
	//##ModelId=3E3FF2DC024E
	static CEProcessEnvironment* environment;
	//##ModelId=3E3FF2DC0249
	static CKinematics* m_kin;
	//##ModelId=3E3FF2DC0244
	static CEMap* m_v_cache;

	static CEPtrArray* m_active_watches;
	//static bool env_inited = false;
public:
	static CDebug* dbg;

	static CAPtrSeq* m_ToDeleteList; // obj*
	static bool m_bToDeleteListCleared;
	static void AddToDeleteList(CEObject* obj);
	static void AddToDeleteList(char* buf); // buf to be deleted by CMemoryManager::mfree(buf);

	//##ModelId=3E3FF2DC0284
	static void disable_values_cache();
	//##ModelId=3E3FF2DC0283
	static void init_values_cache();
	//##ModelId=3E3FF2DC0281
	static CExpression* find_numeric(CExpression* obj);
	//##ModelId=3E3FF2DC0280
	static void finalize();
	//##ModelId=3E3FF2DC0275
	static void setKinematics(CKinematics* _new, bool do_delete_previous = true);
	static void OnKinematicsChanged(); //clear kin-dependent caches [the new kin is set / the old kin is modified]
	//##ModelId=3E3FF2DC0274
	static CKinematics* kinematics();
	static CKinematics* GetKinIfDeclared(); // same to kinematics() but no ERRORS
	//##ModelId=3E3FF2DC0273
	static void init_data_storage();
	//##ModelId=3E3FF2DC0271
	static void init(CEProcessEnvironment* env);
	//##ModelId=3E3FF2DC0266
	static CEData* getDataStorage();
	//##ModelId=3E3FF2DC0264
	static void setEnvironment(CEProcessEnvironment* env);
	//##ModelId=3E3FF2DC0263
	static CEProcessEnvironment* getEnvironment();

	// global parameters
	static bool m_ExamLessThen;
	static bool m_ExamEquals;
	static bool m_MandelstamsEverywhere;
	static int m_ScreenSize;
	static bool m_AllowClassDiagrams;
	static bool m_DropZeroMEDiagrams;
	static bool m_bAutoKinArrange;

	static const char* GPAR_MASSIVE_VECTOR_MATRIX;
	static const char* GPAR_EXAMINE_LESS_THEN;
	static const char* GPAR_EXAMINE_EQUALS;
	static const char* GPAR_MANDELSTAMS;
	static const char* GPAR_SCREEN_SIZE;
	static const char* GPAR_TRACES_BONDEREV;
	static const char* GPAR_TRACES_BONDEREV_F3EXPAND;
	static const char* GPAR_STR_FORTRAN_TEMP;			 //deprecated!
	static const char* GPAR_STR_CODE_TEMP;
	static const char* GPAR_ALLOW_CLASS_DIAGRAMS;
	static const char* GPAR_DROP_ZERO_DIAGRAMS;
	static const char* GPAR_AUTO_KINARRANGE;
	static const char* GPAR_EVALUATE_COMPLICATED_VI;	 // incl. C_ij-values --> may be very slow for massive vectors
	static const char* GPAR_LOOPTOOLS_COMPARTIBLE;		 // Forbid V-integrals not supported by LoopTools
	static const char* GPAR_RUN_EVT_NUCLON_ENTRIES;		 // Add nuclon entries to event in generator project
	static const char* GPAR_CODE_GEN_FAST;				 // no 'arrange' after DiffCS factor applying (to be tested)
	static int setGlobalParameter(char* par, int value); // -1: invalid parameter key, -2: invalid value
	static int setGlobalParameter(const char* par, const char* value); // -1: invalid parameter key, -2: invalid value

	static CEProgressWatch* newProgressWatch(const char* _description);
	static CEProgressWatch* newProgressWatch(const char* _description, long _data);
	static void dismissProgressWatch(CEProgressWatch* pw);

	//##ModelId=3E3FF2DC0262
	CEProcessor();
	//##ModelId=3E3FF2DC0261
	virtual ~CEProcessor();
};

#endif // !defined(AFX_EPROCESSOR_H__9DB7A104_0AA0_11D5_8A94_932004EF1C0E__INCLUDED_)
