////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EProduct.cpp: implementation of the CEProduct class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//#define PROD_LEN_TO_SCROLL_ARRANGE			1000
#define PROD_LEN_TO_SCROLL_EXPAND			2000

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

const char*	CEProduct::CLASS_TAG_NAME = "pr";
CDebug* CEProduct::dbg = new CDebug("CEProduct");

#ifdef USE_PLAIN_ARRAY_FORSEQ // biuld sequence-expressions on CAPtrArray* basis
CEProduct::CEProduct(CEPtrArray* pPtrArray):CSequence(pPtrArray) {}
#else
CEProduct::CEProduct(CAPtrSeq* pPtrArray):CSequence(pPtrArray) {}
#endif //USE_PLAIN_ARRAY_FORSEQ

CEProduct::CEProduct():CSequence()
{
}

CEProduct::~CEProduct()
{

}

CExpression* CEProduct::Simplify(int level)
{
	return this;
}

int CEProduct::m_ErrorNumInDinCntr=0;// counter for warnings log
CExpression* CEProduct::Arrange(int level)
{
	dbg->log(15, "Arrange", this);
	long s = size();
	long i;
	for(i=0; i<s; i++)
	{
		CExpression* item = GetAt(i);
		CExpression* temp = CExpression::arrge(item, false);
		if (temp != item) {
			SetAtX(i, temp);
		}
	}
	dbg->log(55, "sub-items arranged");
	inline_subproducts(false);
	arrangeFactor();
	dbg->log(55, "factor arranged", this);
	arrangeSign();
	dbg->log(55, "sign arranged", this);
	arrangeSequence();
	dbg->log(55, "arranged", this);

	if (level == AL_EXTRACT) {
		//	prosessing special cases
		//	Warning: 1/x -items should remains products
		long _s = size();
		if (_s == 0) {
			if(m_ErrorNumInDinCntr==0){m_ErrorNumInDinCntr++;dbg->log(0, "Warning: 0-sized product, treated as UNIT",this);}
			CExpression* res = CENumber::GetUnit();
			// copy level to unit? :) and arrange it further? no.
			if (!getFlag()) res->inverseFlag();//copyAttributesTo(res, true);
			return res;
		}
		else if (_s == 1) {
			CExpression* item = GetAt(0);
			if (item->getLevel()) {
				CExpression* res = item->clone();
				copyAttributesTo(res, true);
				dbg->log(45, "Single item extracted", res);
				return res;
			}
			else {
				if (item->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) {
					// Warning: numbers should (?) be already arranged.
					if(m_ErrorNumInDinCntr==0){m_ErrorNumInDinCntr++;dbg->log(0, "Internal: inversed number as single item in product",this);}
					CExpression* res = item->clone();
					copyAttributesTo(res, true);
					res = CExpression::arrge(res); // turn number to num-level
					return res;
				}
			}
		}
	}
	return this;
}

CExpression* CEProduct::evaluate(int level)
{
	long _size = size();
	CExpression* _old = NULL;
	CExpression* _new = NULL;
	long i;
	for(i=0; i<_size; i++)
	{
		_old = GetAt(i);
		_new = _old->evaluate();
		if (_old != _new) {
			if (_new != NULL) {
				SetAtX(i, _new);
			}
			else log("<Warning><NULL as evaluate()'s result.>");
		}
	}
	return this;
}

CExpression* CEProduct::applyRule(CEAbstractRule* rule)
{
	CERule* r = (CERule*)rule;
	dbg->log(15, "applyRule", this);
	//dbg->log(115, "rule", rule);

	long i;
	for (i=0; i<size(); i++) {
		CExpression* item = GetAt(i);
		//dbg->log(215, i, "item %ld ", item);
		CExpression* item_repl = r->ApplyTo(item, false);
		//dbg->log(215, "item_repl ", item_repl);
		if ((item_repl != item) && (item_repl != NULL)) {
			SetAtX(i, item_repl);
		}
	}
	return this;
}

CEString* CEProduct::toString(int mode) {
	CEString* res = NULL;
	long _size = size();
	switch (mode) {
	case STORE_WR_MATHEMATICA:
		if(_size>1) {
			bool prev_is_denom=false; bool bPrevInBrackets=false;
			CExpression* p_e=NULL;
			res=new CEString((getFlag()?"":"- "));
			p_e=GetAt(0);
			if(!p_e->getLevel()){ res->AddString("1\\/"); prev_is_denom=true; }
			else if(p_e->getProperty(CEProperties::PROP_MASK_IS_NUMBER)){ if(((CENumber*)p_e)->getDen()!=1)prev_is_denom=true; }
			bPrevInBrackets=toCEStringItem(res, p_e, mode);
			long i;
			for(i=1; i<_size; i++) {
				p_e = GetAt(i);
				if(p_e->getLevel()){ res->AddString("\\ "); prev_is_denom=false; }
				else{ if(prev_is_denom)res->AddString("\\ 1"); res->AddString("\\/"); prev_is_denom=true; }
				bPrevInBrackets=toCEStringItem(res, p_e, mode);
				if((!bPrevInBrackets) && (!prev_is_denom) && (p_e->getProperty(CEProperties::PROP_MASK_IS_NUMBER))){
					if(((CENumber*)p_e)->getDen()!=1)prev_is_denom=true;
				}
			}
		} else {
			if(_size==1) {
				CExpression* pE = GetAt(0);
				if(getFlag()) {
					if(pE->getLevel()){ res = pE->toString(mode); }else{ res=new CEString("1\\/"); toCEStringItem(res, pE, mode); }
				} else {
					res=new CEString("- "); if(!pE->getLevel())res->AddString("1\\/"); toCEStringItem(res, pE, mode);
				}
			} else {
				res=new CEString("<.>");
			}
		}
		break;
	default:
		res=new CEString("<Not supported data format.>");
	}
	return res;
}

bool CEProduct::toCEStringItem(CEString* res, CExpression* pE, int mode) { //returns 'true' if brackets were inserted
	bool bNeedBrackets = (!IsSimpleItem(pE));
	if(bNeedBrackets){
		switch(mode) {
		case STORE_WR_MATHEMATICA:
			*res<<"\\((";res->AddString(pE->toString(mode));*res<<")\\)"; break;
		default:
			*res<<"(";res->AddString(pE->toString(mode));*res<<")";
		}
	}else{
		res->AddString(pE->toString(mode));
	}
	return bNeedBrackets;
}

void CEProduct::toStream(ostream* os, int mode){
}

/*
* The collection of sub-items properties should be added.
*/
void CEProduct::collectProperties(CEProperties* props, unsigned set) {
	//dbg->log(29, "collectProperties");
	//CExpression::collectProperties(props, set);
	long s = size();
	props->setSingleItem(s <= 1);
	props->setProperty(CEProperties::PROP_MASK_IS_PRODUCT, true);
	if (getFirstMatrixPos() >= 0) props->setCommutative(false);

	for (long i=0; i<s; i++) {
		unsigned long lPropsI=GetAt(i)->props_long();
		if ((lPropsI & CEProperties::PROP_MASK_SELF_CONJUGATED)==0) props->setProperty(CEProperties::PROP_MASK_SELF_CONJUGATED, false);
	}
}

unsigned long CEProduct::props_long() {
	unsigned long res = CExpression::props_long();
	long s = size();
	res = CEProperties::set_property_to(res, CEProperties::PROP_MASK_SINGLE_ITEM, (s <= 1));
	res = CEProperties::set_property_to(res, CEProperties::PROP_MASK_IS_PRODUCT, true);
	if (getFirstMatrixPos() >= 0) res = CEProperties::set_property_to(res, CEProperties::PROP_MASK_COMMUTATIVE, false);

	for (long i=0; i<s; i++) {
		unsigned long lPropsI=GetAt(i)->props_long();
		if ((lPropsI & CEProperties::PROP_MASK_SELF_CONJUGATED)==0) res=CEProperties::set_property_to(res,CEProperties::PROP_MASK_SELF_CONJUGATED,false);
	}
	return res;
}

const char* CEProduct::CLASS_NAME = "CEProduct";
const char* CEProduct::ClassName() const { return CLASS_NAME; }

/*
* We do not care about "recursive"'s value.
*/
CExpression* CEProduct::expand(bool recursive)
{
	dbg->log(15, "expand", this);

	//first we expand and arrange (!) all the sub-items.
	long _s = size();
	long i;
	for(i=0; i<_s; i++)
	{
		CExpression* item = GetAt(i);
		if (getLevel()) {
			CExpression* temp = CExpression::reduce(item, false);
			if ((temp != item) && (temp != NULL)) {
				SetAtX(i, temp);
			}
		}
		else {
			// we work with numerators
			item->inverseLevel();
			CExpression* temp = CExpression::reduce(item, false);
			if ((temp != item) && (temp != NULL)) {
				temp->inverseLevel();
				SetAtX(i, temp);
			}
			else item->inverseLevel();
		}
	}
	dbg->log(35, "sub-items expanded", this);

	//	*****************************************************	<scroll>
	CEProgressWatch* pw1 = NULL;
	if (_s > PROD_LEN_TO_SCROLL_EXPAND) { pw1 = CEProcessor::newProgressWatch("Long product expand #1, size: %ld", _s); }
	//	*****************************************************	<\scroll>

	// inlinig sub-products
	inline_subproducts(false);

	// inlinig sequences
	if (getLevel()) {
		for(i=0; i<size(); i++) {
			CExpression* item = GetAt(i);
			if (item->getLevel()) {
				//expanding sub-sequence
				if (item->ClassName() == CSequence::CLASS_NAME) {
					dbg->log(35, "expanding sequence", item);
					CSequence* seq = (CSequence*)item;
					if (!seq->getFlag()) { inverseFlag(); seq->setFlag(); }
					CSequence* res_seq = new CSequence();
					long s_size = seq->size();
					for (long j=0; j<s_size; j++) {
						CEProduct* copy = (CEProduct*)clone();
						copy->SetAt(i, seq->GetAt(j));
						res_seq->addItem(copy);
					}
					// further expand
					res_seq->Arrange(AL_INLINE);
					CExpression* res = res_seq->expand();
					if (res == NULL) { dbg->log(0, "expand(prod of seq) -> NULL ERROR ", res_seq); CEException::halt(); }
					if (res != res_seq) { delete res_seq; }
					dbg->log(35, "sequence expanded", res);
					return res;
				}
			}
		}
	}

	// inlinig sums
	for(i=0; i<size(); i++) {
		CExpression* item = GetAt(i);
		
		if (item->getLevel()) {
			//expanding numerator
			//expanding sub-sum
			if ((getLevel()) && (item->getProperty(CEProperties::PROP_MASK_IS_SUM))) {
				dbg->log(35, "expanding sum", item);
				CESum* sum = (CESum*)item;
				if (!sum->getFlag()) {
					inverseFlag();
					sum->setFlag();
				}
				CESum* res_sum = new CESum();
				long s_size = sum->size();
				//	*****************************************************	<scroll>
				if (pw1 != NULL) CEProcessor::dismissProgressWatch(pw1);
				CEProgressWatch* pw2 = NULL;
				if (s_size*size() > PROD_LEN_TO_SCROLL_EXPAND) {
					pw2 = CEProcessor::newProgressWatch("Long product expand #2, size: %ld", s_size*size());
					//debug: save product for tests
					//debug: if(s_size*size()==23375){ dbg->store_debug_expression(0,"[debug] product saved",this); }
				}
				//	*****************************************************	<\scroll>
				long j;
				for (j=0; j<s_size; j++) {
					CEProduct* copy = (CEProduct*)clone();
					copy->SetAt(i, sum->GetAt(j));
					res_sum->addItem(copy);
					//	*****************************************************	<scroll>
					if (pw2 != NULL) pw2->step(j+1, s_size);
					//	*****************************************************	<\scroll>
				}
				//	*****************************************************	<scroll>
				if (pw2 != NULL) CEProcessor::dismissProgressWatch(pw2);
				//	*****************************************************	<\scroll>

				// Arrange to prevent (a+b)/(a+b)->a/(a+b)+b/(a+b)
				// TODO: Inprove next line to prevent (a-b)/(b-a)->a/(b-a)-b/(b-a)
				res_sum->Arrange(AL_INLINE);

				// further expand
				CExpression* res = res_sum->expand();
				if (res == NULL) { dbg->log(0, "expand(prod of sum) -> NULL ERROR", res_sum); CEException::halt(); }
				if (res != res_sum) { delete res_sum; }
				dbg->log(35, "sum expanded", res);
				return res;
			}
		}
		else {
			//expanding single denominator
			if (item->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) {

				// Warning: numbers should NOT (?) be already arranged.
				// -------- ^^^^ What should it mean ??? >:-(
				//printf("xxx :2");
				item = CExpression::arrge(item);
				//((CENumber*)item)->inverse();
			}
		}
		//	*****************************************************	<scroll>
		if (pw1 != NULL) pw1->step(i+1, size());
		//	*****************************************************	<\scroll>
	}
	//	*****************************************************	<scroll>
	if (pw1 != NULL) CEProcessor::dismissProgressWatch(pw1);
	//	*****************************************************	<\scroll>
	dbg->log(45, "expanded", this);
	return this;
}

CExpression* CEProduct::clone() const {
	CEProduct* res = new CEProduct();
	long s = size();
	long i;
	for(i=0; i<s; i++) {
		res->CSequence::addItem(GetAtConst(i)->clone());
	}
	copyAttributesTo(res);
	return res;
}

CExpression* CEProduct::removeFactor()
{
	CEProduct* res = new CEProduct();
	CExpression* temp = NULL;
	long i;
	for(i=0; i<size();)
	{
		temp = GetAt(i);
		if (temp->getProperty(CEProperties::PROP_MASK_COMMUTATIVE)) {
			res->addItem(temp);
			RemoveAt(i);
		}
		else i++;
	}

	switch (res->size()) {
	case 1:
		return res->GetAt(0);
	}
	return res;
}

long CEProduct::getFirstMatrixPos()
{
	//dbg->log(29, "getFirstMatrixPos");
	long _size = size();
	long i;
	for(i=0; i<_size; i++)
	{
		if (!GetAt(i)->getProperty(CEProperties::PROP_MASK_COMMUTATIVE)) return i;
	}
	return -1;
}

void CEProduct::arrangeSign() {
	long _size = size();
	long i;
	for (i=0; i<_size; i++) {
		CExpression* temp = GetAt(i);
		if (!temp->getFlag()) {
			temp->setFlag();
			inverseFlag();
		}
	}
	
	if (_size == 1) { // truncate sign in ZERO expression: -{0}=>{0}.
		// note: itm should be already arranged, so that no '-0' may occur here
		if ((CEBasic::isZero(GetAt(0))) && (!getFlag())) { setFlag(); }
	}
}

void CEProduct::arrangeFactor() {
	dbg->log(35, "arrangeFactor", this);
	CExpression* temp = NULL;
	CENumber* _num=NULL; bool _num_first=false;
	long i;
	long s = size();
	for (i=0; i<s;) {
		temp = GetAt(i);
		if (temp->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) {
			if (i>0) {
				if (_num == NULL) {
					_num = (CENumber*)((CENumber*)temp)->clone();
				} else {
					_num->multiplyNumber((CENumber*)temp);
				}
				RemoveAt(i); s--;
			} else { // number at i==0, (_num == NULL)
				_num = (CENumber*)temp; _num_first=true; i++;
			}
		}
		else i++;
	}
	if (_num != NULL) {
		if (_num_first) {
			if (CEBasic::isZero(_num)) {
				for (i=1; i<size();) {RemoveAt(i);} // remove all except of the first
			} else {
				if ( (CEBasic::isUnit(_num)) && (s>1)) { RemoveAt(0); }
			}
		} else {
			if (CEBasic::isZero(_num)) {
				for (i=0; i<size();) {RemoveAt(i);} // remove all
				addItem(_num);
			} else {
				if (!CEBasic::isUnit(_num)) InsertAt(0, _num);
				else if (size() == 0) addItem(_num->clone());
				delete _num;
			}
		}
	}
}

void CEProduct::arrangeSequence() {
	dbg->log(35, "arrangeSequence", this);
	CExpression* temp = NULL;

	//	First leave all the non-commutatives at the end.
	long i;
	long fmp = getFirstMatrixPos();
	if (fmp >= 0) {
		long _size = size();
		for (i=fmp; i<_size; i++) {
			temp = GetAt(i);
			if (temp->getProperty(CEProperties::PROP_MASK_COMMUTATIVE)) {
				//CSequence::move();
				move(i,fmp); //change on 09.2009
				//InsertAt(fmp, temp); RemoveAt(i+1);
				fmp++;
			}
		}
	}

	if (size() == 0) return;// nothing to arrange

	//	Next arranging powers for commutative part.
	arrangePowers();

	//	Arranging sequence in alphabetical order.
	long l = lastCommutativePos();
	long f = ((GetAt(0)->getProperty(CEProperties::PROP_MASK_IS_NUMBER))?1:0);
	for (i=f+1; i<=l; i++) {
		CExpression* item = GetAt2(i);
		bool loop = true;
		long j;
		/*for (j=f; ((j<i) && (loop)); j++) {
			if (item->less_than(GetAt(j))) {
				move(i,j); loop = false;
			}
		}*/
		for (j=i-1; ((j>=f) && (loop)); j--) { // check right order first
			if (!item->less_than(GetAt(j))) { j++; if(i>j){move(i, j);} loop=false; }
		}
		if (loop) { move(i, f); } // item is less than all previous
	}
	dbg->log(55, "Sequence arranged", this);
}

void CEProduct::arrangePowers() {
	dbg->log(35, "arrangePowers", this);
	CExpression* base = NULL;
	//CExpression* o_base = NULL;
	CENumber o_power(0);
	long last_pos = lastCommutativePos();
	long first_pos=((GetAt(0)->getProperty(CEProperties::PROP_MASK_IS_NUMBER))?1:0);
	long i;

	// arrange (1/a)^n and 1/a^{-n}
	// [New 09.2009] also uniform a^{-n} -> 1/a^n
	// Note. arrange 1/a^{(num<0)} is not required: Arrange(num<0) --> '-num'
	for (i=first_pos; i<=last_pos; i++) {
		CExpression* itm = GetAt(i);
		//if (CEUtil::equals(itm->ClassName(), "CEPower")) {
		if (itm->ClassName() == CEPower::CLASS_NAME) {
			if (!itm->getLevel()) {
				CEPower* pwr=(CEPower*)itm; base=pwr->basis(); CExpression* powerS=pwr->power(); if(powerS==NULL)continue;

				// 1. arrange (1/a)^n->1/a^n
				if (!base->getLevel()) { itm->inverseLevel(); base->inverseLevel(); }

				// 2. arrange 1/a^{-pow}
				if (!powerS->getFlag()) { itm->inverseLevel(); powerS->inverseFlag(); }
			} else {
				CEPower* pwr=(CEPower*)itm; base=pwr->basis(); CExpression* powerS=pwr->power(); if(powerS==NULL)continue;

				// 3. uniform a^{-n} -> 1/a^n
				if (!powerS->getFlag()) { itm->inverseLevel(); powerS->inverseFlag(); }
			}
		}
	}

	// arrange a^x...a^y
	if (last_pos == 0) return;
	CENumber powerX(0);
	for (i=first_pos; i<last_pos; i++) {
		CExpression* powerS = NULL; // for possible symbol powers
		bool pow_to_invert=false; //pow_to_invert: power should be inverted BEFORE examining and AFTER it
		CExpression* itm = GetAt(i);

		//Note: some items must NOT be coupled to powers, e.g.:
		// g_ab*g_ab != g_ab^2
		// But:
		//	Tr[GellMann_a GellMann_b]^2 --> g_ab^2 ==> we MUST expand it afterwards!
		if (itm->getProperty(CEProperties::PROP_MASK_IS_TENSOR)) {
			IETensor* t = IETensor::GetIETensor(itm);
			if(t->rank() > 0){
				continue;	//skip procedure for this object
			}
		}

		if (itm->ClassName() == CEPower::CLASS_NAME) {
			CEPower* pwr = (CEPower*)itm;
			base = pwr->basis(); powerS = pwr->power();
			if (!itm->getLevel()) pow_to_invert=true;
			if (powerS->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) { powerX.SetValue((CENumber*)powerS); powerS=NULL; }
		} else {
			base = itm;
			powerX.SetValue((itm->getLevel())?1:-1);
		}

		//Note the level-sign is already included into powerX.
		bool inv_base = false;
		if (!base->getLevel()) { base->inverseLevel(); inv_base=true; }
		dbg->log(75, "base", base);
		dbg->log(75, (long)(pow_to_invert?(-1):1), "power(%ld)",  &powerX);

		bool changed = false;
		CExpression* power_res = NULL;
		long j;
		for (j=i+1; j<=last_pos;) {
			itm = GetAt2(j);
			CExpression* o_base;
			CExpression* o_powerS = NULL; // for possible symbol powers
			bool powO_to_invert=false; //powO_to_invert: powerX should be inverted BEFORE examining and after it
			if (CEUtil::equals(itm->ClassName(), "CEPower")) {
				CEPower* pwr = (CEPower*)itm;
				o_base = pwr->basis(); o_powerS = pwr->power();
				if (!itm->getLevel()) powO_to_invert=true;
				if (o_powerS->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) { o_power.SetValue((CENumber*)o_powerS); o_powerS=NULL; }
			} else {
				o_base = itm;
				o_power.SetValue((itm->getLevel())?1:-1);
			}
			bool inv_o_base = false;
			if (!o_base->getLevel()) { o_base->inverseLevel(); inv_o_base=true; }
			if(base->equals(o_base)) {
				if (pow_to_invert) {
					if (powerS != NULL) { powerS->inverseFlag(); } else { powerX.inverseFlag(); }
					pow_to_invert=false; // first time only
				}
				if (powO_to_invert) { if (o_powerS != NULL) { o_powerS->inverseFlag(); } else { o_power.inverseFlag(); }}
				// Note: no back-inversion for powerS or o_powerS are required -- its sources will be deleted soon
				if ((powerS == NULL) && (o_powerS == NULL)) { // numeric powers
					powerX.addNumber(&o_power);
				} else { // unknown(symbol) powers
					power_res = ((powerS != NULL)?powerS->clone():powerX.clone());
					power_res = power_res->add((o_powerS != NULL)?o_powerS->clone():o_power.clone());
				}
				RemoveAt(j); last_pos--;
				changed=true;
			} else {
				j++;
				if (inv_o_base) { o_base->inverseLevel(); }
			}
		}

		if (!changed) {
			if (inv_base) { base->inverseLevel(); }
			continue;
		}

		bool num_power_res;
		if (power_res != NULL) {
			num_power_res=false;
		} else {
			power_res = &powerX;
			num_power_res=true;
		}

		//dbg->log(45, "changed : base (result)", base);
		//dbg->log(45, "changed : power (result)", power_res);
		if (CEBasic::isUnit(power_res)) {
			SetAtX(i, base->clone()); // clone base (for safer source deletion)
		} else if (CEBasic::isZero(power_res)) {
			RemoveAt(i); last_pos--;
			i--;
		} else {
			//This probably shold be done in CEPower::Arrange();
			CExpression* itm_new;
			bool powLT0 = (num_power_res)?(powerX.cmpZero()<0):(!power_res->getFlag());
			if (powLT0) {
				power_res->inverseFlag();
				if (num_power_res) powerX.Arrange(AL_INLINE); // to cancel -(-2) terms
				if (CEBasic::isUnit(power_res)) itm_new = base->clone();
				else itm_new = new CEPower(base, power_res);
				itm_new->inverseLevel();
			} else {
				itm_new = new CEPower(base, power_res);
			}
			SetAtX(i, itm_new);
		}
		if (!num_power_res) delete power_res;
	}

	// Expand tensor powers
	// Nt1. This case appears e.g. from Tr[GellMann_a GellMann_b]^2 = g_ab^2
	// Nt2. To avoid ethernal loops,
	//      one MUST also check for (g_ab*g_ab != g_ab^2) above (see "arrange a^x...a^y"), BEFORE power arranging!
	bool bNeedInlineResults=false;
	for (i=first_pos; i<=last_pos; i++) {
		CExpression* itm = GetAt(i);
		if (itm->ClassName() == CEPower::CLASS_NAME) {
			CExpression* base = ((CEPower*)itm)->basis();
			if (base->getProperty(CEProperties::PROP_MASK_IS_TENSOR)) {
				IETensor* t = IETensor::GetIETensor(base);
				if((t->rank() > 0) && (itm->getLevel())) {
					CExpression* pow_as_prod = ((CEPower*)itm)->as_product();
					if(pow_as_prod!=NULL){ //inline this object
						SetAtX(i,pow_as_prod);
						bNeedInlineResults=true;
					}
				}
			}
		}
	}
	if(bNeedInlineResults){ inline_subproducts(false); }

	if (size() == 0) {
		addItem(CENumber::GetUnit());
	}
	dbg->log(55, "powers (re-)arranged", this);
}

long CEProduct::lastCommutativePos()
{
	long res = getFirstMatrixPos();
	if (res < 0) res = size();
	res--;
	return res;
}

bool CEProduct::equals(CExpression *other) {
	if (other == NULL) return false;
	if (!other->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) return false;

	CEProduct* o = (CEProduct*)other;
	long s = size();

	if (s != o->size()) return false;
	if (!attributes_equals_to(o)) return false;

	long i;
	for (i=0; i<s; i++) {
		if (!GetAt(i)->equals(o->GetAt(i))) return false;
	}
	return true;
}

/*bool CEProduct::less_than(CExpression* other) {
	if (other == NULL) return false;
	if (!other->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) return false;

	long s = size();
	long o_s = ((CEProduct*)other)->size();
	if (o_s != s) return (s<o_s);
	
	for (long i=0; i<s; i++) {
		CExpression* expr = GetAt(i);
		CExpression* o_expr = ((CEProduct*)other)->GetAt(i);
		if (expr->less_than(o_expr)) return true;
		if (!expr->equal_or_opposite(o_expr)) return false;
	}
	return false;
}*/

bool CEProduct::BasesEquals(CEProduct* other) {
	//dbg->log(135, "BasesEquals()", this, other);
	long s = size();
	long i=0; if (s>0) { if (GetAt(0)->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) { i=1; } }
	long s2 = other->size();
	long i2=0; if (s2>0) { if ((other->GetAt(0))->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) { i2=1; } }

	if ((s-i) != (s2-i2)) return false;
	// Note: no sign comparison here

	for (;i<s;) {
		if (!GetAt(i)->equals(other->GetAt(i2))) return false;
		i++; i2++;
	}
	return true;
}

bool CEProduct::BasesEqualsExpr(CExpression* other) {
	if (other->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
		return BasesEquals((CEProduct*)other);
	}

	long s = size();
	long i=0; if (s>0) { if (GetAt(0)->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) { i=1; } }
	if ((s-i) != 1) return false;

	// Note: no sign comparison here
	CExpression* item = GetAt(i);
	return CEBasic::equalsSignless(item,other);
}

bool CEProduct::BasesLessOrGreatherThen(CEProduct* other, bool chk_less) { // might be replaced by lass_than implementation
	//dbg->log(135, "BasesLessOrGreatherThen()", this, other);
	long s = size();
	long i=0; if (s>0) { if (GetAt(0)->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) { i=1; } }
	long s2 = other->size();
	long i2=0; if (s2>0) { if ((other->GetAt(0))->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) { i2=1; } }

	for (;((i<s) && (i2<s2));) {
		CExpression* expr_1 = GetAt(i);
		CExpression* expr_2 = other->GetAt(i2);
		if (chk_less) {
			if (expr_1->less_than(expr_2)) return true;
		} else  {
			if (expr_2->less_than(expr_1)) return true;
		}
		if (!CEBasic::equalsSignless(expr_1,expr_2)) return false;
		i++; i2++;
	}
	if (chk_less) {
		if (i2<s2) return true;
	} else {
		if (i<s) return true;
	}
	return false;
}

bool CEProduct::BasesLessThenExpr(CExpression* other) { // might be replaced by lass_than implementation
	if (other->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) { return BasesLessOrGreatherThen((CEProduct*)other, true); }

	long s = size();
	long i=0; if (s>0) { if (GetAt(0)->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) { i=1; } }
	if (i>=s) return true;

	CExpression* a = GetAt(i);
	bool a_Msign = (!a->getFlag()); if (a_Msign) a->inverseFlag();
	bool o_Msign = (!other->getFlag()); if (o_Msign) other->inverseFlag();
	bool less_res = a->less_than(other);
	bool equ_res; if (!less_res) equ_res = a->equals(other);
	if (a_Msign) a->inverseFlag();
	if (o_Msign) other->inverseFlag();

	if (less_res) return true;
	if (!equ_res) return false;
	if ((s-i)>1) return false;
	return false;
}

bool CEProduct::BasesGreatherThenExpr(CExpression* other) { // might be replaced by lass_than implementation
	if (other->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) { return BasesLessOrGreatherThen((CEProduct*)other, false); }
	
	long s = size();
	long i=0; if (s>0) { if (GetAt(0)->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) { i=1; } }
	if (i>=s) return false;

	CExpression* a = GetAt(i);
	bool a_Msign = (!a->getFlag()); if (a_Msign) a->inverseFlag();
	bool o_Msign = (!other->getFlag()); if (o_Msign) other->inverseFlag();
	bool grea_res = other->less_than(a);
	bool equ_res; if (!grea_res) equ_res = a->equals(other);
	if (a_Msign) a->inverseFlag();
	if (o_Msign) other->inverseFlag();

	if (grea_res) return true;
	if (!equ_res) return false;
	if ((s-i)>1) return true;
	return false;
}

CExpression* CEProduct::arrangeTensors()
{
	dbg->log(15, "arrangeTensors", this);
	arrangeFactor();
	arrangeSign();
	arrangeSequence();

	CExpression* res = this;
	bool changed = false;
	bool need_expand = false;
	long i;
	for(i=0; i<(size()-1); i++) {
		CExpression* temp = GetAt2(i);
		if (temp->getProperty(CEProperties::PROP_MASK_IS_TENSOR)) {
			IETensor* t = IETensor::GetIETensor(temp);
			bool t_comm = temp->getProperty(CEProperties::PROP_MASK_COMMUTATIVE);
			bool loop = true;
			//[debug] dbg->log(205, "trying to unite:", temp);
			for(long j=i+1; (loop & (j<size())); j++) {
				temp = GetAt(j);
				if (!t_comm) loop = false;
				if (temp->getProperty(CEProperties::PROP_MASK_IS_TENSOR)) {
					CExpression* unition_res = t->unite_with(IETensor::GetIETensor(temp));
					//[debug] dbg->log(205, "unite_with result", temp, unition_res);
					if (unition_res != NULL) {
						if (unition_res->getProperty(CEProperties::PROP_MASK_IS_SUM))
							need_expand = true;
						SetAtX(j, unition_res);
						RemoveAt(i);
						i--;
						loop = false;
						changed = true;
					}
				}
			}
		}
	}
	if (changed) {
		if (need_expand) {
			res = CExpression::reduce(this, false);
		}
		else {
			res = arrangeTensors();
		}
	}
	else { // if nothing to change
		if (CEAntisymmetric::parse_product_for_zero(this))
			res = CENumber::GetZero();
	}
	return res;
}

CExpression* CEProduct::conjugated(int mode) {
	CEProduct* res = new CEProduct();
	CExpression* temp = NULL;
	long s = size();
	long f = getFirstMatrixPos();
	long l = lastCommutativePos();
	long i;
	for(i=0; i<=l; i++) {
		CExpression* item=GetAt(i);
		temp=CEBasic::Congugate(item,false); if(temp==NULL)CEException::halt("NULL");
		if(temp==item)temp=temp->clone();
		res->addItem(temp);
	}
	for(i=s-1; i>l; i--) {
		CExpression* item=GetAt(i);
		temp=CEBasic::Congugate(item,false); if(temp==NULL)CEException::halt("NULL");
		if(temp==item)temp=temp->clone();
		res->addItem(temp);
	}
	copyAttributesTo(res);
	return res;
}

/*
* Warning: do check up all the permissions
* of this operation before use (commutativities, etc.).
*/
/*
void CEProduct::exchange(long a, long b)
{
	if (a == b) return;
	CExpression* item_a = GetAt(a);
	CExpression* item_b = GetAt(b);
	if ((item_a == NULL) || (item_b == NULL)) {
		log("CEProduct::exchange() : invalid argument suspected.");
		return;
	}
	//item_a = item_a->clone();
	item_b = item_b->clone();
	SetAt(b, item_a);
	SetAt(a, item_b);
	delete item_b;
}
*/

CExpression* CEProduct::numeric()
{
	long s = size();
	if (s<=0) return NULL;
	CExpression* res = GetAt(0)->numeric();
	long i;
	for (i=1; i<s; i++) {
		res = res->multiply(GetAt(i)->numeric());
	}
	copyAttributesTo(res, true);
	return res;
}

/*
*	returns single item or CEProduct
*	when extracting from denominator result will also be a denominator
*	takes no care with the whole product sign
*
*	can be greatly fastered by avoiding spare cloning
*/
CExpression* CEProduct::extract_items(long from, long to)
{
	long s = size();
	if (!((from >= 0) && (to >= from) && (s > to))) return NULL;

	CExpression* res = NULL;
	if (from == to) {
		res = RemoveAtX(from);
		//res = GetAt(from)->clone(); RemoveAt(from);
	}
	else {
		CEProduct* prod_res = new CEProduct();
		long i;
		for (i = from; i<=to; i++) {
			prod_res->addItem(RemoveAtX(from));
			//prod_res->addItem(GetAt(from)->clone()); RemoveAt(from);
		}
		res =  prod_res;
	}
	if (size() == 0) addItem(CENumber::GetUnit());

	if (!getLevel()) res->inverseLevel();
	return res;
}

CExpression* CEProduct::multiply(CExpression *other)
{
	if (!getLevel()) other->inverseLevel(); // TODO: check it for reason!!!
	addItem(other);
	return this;
}

/*
*	Note: does not use the common flags of current product
*	i.e.:
*	(1/{a*b*c})->addItem(1/y)==>1/{a*b*c*(1/y)}
*	({1/a*1/b*1/c})->addItem(1/y)==>{1/a*1/b*1/c*1/y}
*	To be used in internal processing only!
*/
long CEProduct::addItem(CExpression *pNewItem)
{
	return CSequence::addItem(pNewItem);

	/*if (!pNewItem->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
		return CSequence::addItem(pNewItem);
	}

	CEProduct* pr = (CEProduct*)pNewItem;
	long s = pr->size();
	if (s == 0) return size();

	bool sign = pr->getFlag();
	bool lev = pr->getLevel();

	long j;
	for (j=0; j<s; j++) {
		CExpression* item = pr->GetAt(j)->clone();
		if (!lev) item->inverseLevel();
		if ((j == 0) && (!sign)) item->inverseFlag();
		
		addItem(item);
	}
	delete pr;
	return size();*/
}

long CEProduct::addItemInline(CExpression* pNewItem) {

	if (!pNewItem->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
		return CSequence::addItem(pNewItem);
	}

	CEProduct* pr = (CEProduct*)pNewItem;
	long s = pr->size();
	if (s == 0) return size();

	bool sign = pr->getFlag();
	bool lev = pr->getLevel();

	long j;
	for (j=0; j<s; j++) {
		CExpression* item = pr->GetAt(j)->clone();
		if (!lev) item->inverseLevel();
		if ((j == 0) && (!sign)) item->inverseFlag(); //inverse sign at first item
		
		addItemInline(item);
	}
	delete pr;
	return size();
}

CExpression* CEProduct::extract_denominator()
{
	Arrange(AL_INLINE);
	//inline_subproducts();

	CExpression* res = CENumber::GetUnit();
	long i;
	for (i = 0; i<size();) {
		CExpression* item = GetAt(i);
		if (!item->getLevel()) {
			//if (res == NULL) res = item->clone();
			//else res = res->multiply(item->clone());
			res = res->multiply(item->clone());
			RemoveAt(i);
		}
		else i++;
	}

	if (size() == 0) addItem(CENumber::GetUnit());
	return res;
}

/*
*	a(bc)def --> abcdef
*/
void CEProduct::inline_subproducts(bool recursive)
{
	dbg->log(35, "inline_subproducts", this);
	long i;
	for (i=0; i<size();i++) {
		CExpression* item = GetAt(i);
		if (item->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
			CEProduct* p = (CEProduct*)item;
			if (recursive) p->inline_subproducts();

			bool sign = p->getFlag();
			bool lev = p->getLevel();

			long ps = p->size();
			if (ps > 0) {
				if (!sign) inverseFlag();
				long j;
				for (j=(ps-1); j>=0; j--) {
					item = p->GetAt(j);
					if (!lev) {
						item->inverseLevel();
						//if (item->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) {
						//	((CENumber*)item)->inverse();
						//}
						//item = CExpression::arrge(item);
					}

					if (j>0) InsertAt(i+1, item);
					else SetAt(i, item);
				}
				i+=(ps-1);
			}
			else {
				RemoveAt(i);
				i--;
			}
		}
	}
	dbg->log(55, "inline_subproducts res", this);
}

long CEProduct::inline_product(long i, CExpression *var) {
	if (var == NULL) return 0;
	if (!var->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) { InsertAt(i, var); return 1; }
	CEProduct* prod = (CEProduct*)var;
//	if (i >= size()) { addItem(prod->clone()); return; }

	if (i<0) i=0;
	bool last_pos = (i >= size());

	long s = prod->size(); if (s==0) return 0;
	bool sign = prod->getFlag();
	if (!sign) inverseFlag();
	bool lev = prod->getLevel();
	for (long j=0; j<s; j++) {
		CExpression* item = prod->GetAt(j)->clone();
		if (!lev) { item->inverseLevel(); item->Arrange(AL_INLINE); }

		if (last_pos) {
			getMemberList()->add(item);// to avoid extra cloning
		} else {
			getMemberList()->InsertAt(i+j, item);// to avoid extra cloning
		}
	}
	return s;
}

CExpression* CEProduct::Re() {
	//check up if is pure RE or IM.
	bool ret_inline = true;
	long s = size();
	long i;
	for (i=0; i<s; i++) {
		CExpression* item = GetAt(i);
		CExpression* item_re = item->Re();

		if ((item_re == NULL) || (CEBasic::isZero(item_re))) {
			if (item_re != NULL) delete item_re;
			return NULL;
		}
		else if (item_re != item) {
			if (item_re != NULL) delete item_re;
			ret_inline = false;
		}
	}
	if (ret_inline) return this;

	dbg->log(0, "Error: Re() not implemented for complicated object", this);
	return NULL;
}

CExpression* CEProduct::Im() {
	//check up if is pure RE or IM.
	bool ret_inline = true;

	bool im_passed = false;
	long s = size();
	long i;
	for (i=0; i<s; i++) {
		CExpression* item = GetAt(i);
		CExpression* item_im = item->Im();

		if (item_im == item) {
			if (im_passed) ret_inline = false;
			else im_passed = true;
		}
		else if (item_im != NULL) {
			if (!CEBasic::isZero(item_im)) ret_inline = false;
			delete item_im;
		}
	}
	if (!im_passed) return NULL;
	if (ret_inline) return this;

	dbg->log(0, "Error: Im() not implemented for complicated object", this);
	return NULL;
}

CESerializable* CEProduct::restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings) {
	CEProduct* res = new CEProduct();
	bool loop = true;
	for(;loop;) {
		long skip_pos = CESupport::skip_item_in_buffer(data, pos, settings);
		if (skip_pos <= 0) {
			loop = false;
		}
		else {
			CESerializable* item = CESupport::read_buffer(data, pos, settings);
			pos = skip_pos;
			if (item != NULL) {
				res->addItem((CExpression*)item);
			}
			else {
				printf("Error restoring product\n");
				//Error
				loop = false;
				delete res;
				return NULL;
			}
		}
	}
	return res;
}

const char* CEProduct::get_TAG_Name()
{
	return CLASS_TAG_NAME;
}

long CEProduct::store_stream(ofstream *p_ofs, DWORD settings)
{
	long res = 0;
	long s = size();
	long i;
	for (i=0; i<s; i++) {
		CExpression* item = GetAt(i);
		long item_len = CESupport::store_stream(item, p_ofs, settings);
		res += item_len;
		if ((item_len > 40) && (i < (s-1))) {//40 should be defined constant
			*p_ofs << '\n';
			res += 2;
		}
	}
	return res;
}
