////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EProduct.h: interface for the CEProduct class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EPRODUCT_H__9DB7A109_0AA0_11D5_8A94_932004EF1C0E__INCLUDED_)
#define AFX_EPRODUCT_H__9DB7A109_0AA0_11D5_8A94_932004EF1C0E__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CEProduct : public CSequence  
{
public:
	virtual long store_stream(ofstream* p_ofs, DWORD settings);
	virtual const char* get_TAG_Name();
	static const char* CLASS_TAG_NAME;
	static CESerializable* restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings);

	virtual CExpression* Im();
	virtual CExpression* Re();
	long inline_product(long i, CExpression* var); // returns number of items inserted
	void inline_subproducts(bool recursive = true);
	CExpression* extract_denominator();
	virtual unsigned long props_long();
	virtual long addItem(CExpression* pNewItem);
	long addItemInline(CExpression* pNewItem);
	virtual CExpression* multiply(CExpression* other);
	CExpression* extract_items(long from, long to);
	virtual CExpression* numeric();
	static CDebug* dbg;
	//virtual void exchange(long a, long b);
	virtual CExpression* conjugated(int mode = 0);
	virtual bool equals(CExpression* other);
	//virtual bool less_than(CExpression* other);
	virtual long lastCommutativePos();
	virtual long getFirstMatrixPos();
	virtual CExpression* removeFactor();
	virtual CExpression* clone() const;
	virtual CExpression* expand(bool recursive = true);
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;

	bool BasesEquals(CEProduct* other);
	bool BasesEqualsExpr(CExpression* other);
	bool BasesLessOrGreatherThen(CEProduct* other, bool chk_less); // might be replaced by lass_than implementation
	bool BasesLessThenExpr(CExpression* other); // might be replaced by lass_than implementation
	bool BasesGreatherThenExpr(CExpression* other); // might be replaced by lass_than implementation

#ifdef USE_PLAIN_ARRAY_FORSEQ // biuld sequence-expressions on CAPtrArray* basis
	CEProduct(CEPtrArray* pPtrArray);
#else
	CEProduct(CAPtrSeq* pPtrArray);
#endif //USE_PLAIN_ARRAY_FORSEQ

	CEProduct();
	virtual ~CEProduct();

	virtual CExpression* applyRule(CEAbstractRule* rule);
	virtual CExpression* evaluate(int level = 0);
	virtual CExpression* Simplify(int level = 0);
	virtual CExpression* Arrange(int level);
	virtual CEString* toString(int mode = 0);
	virtual void toStream(ostream* os, int mode = 0);
	virtual CExpression* arrangeTensors();

protected:
	void arrangePowers();
	void arrangeSequence();
	void arrangeFactor();
	void arrangeSign();
	bool toCEStringItem(CEString* res, CExpression* pE, int mode); //returns 'true' if brackets were inserted
	virtual void collectProperties(CEProperties* props, unsigned set = CEProperties::PROP_MASK_ALL);

	// counter for warnings log
	static int m_ErrorNumInDinCntr;
};

#endif // !defined(AFX_EPRODUCT_H__9DB7A109_0AA0_11D5_8A94_932004EF1C0E__INCLUDED_)
