////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EProperties.cpp: implementation of the CEProperties class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

const unsigned CEProperties::PROP_MASK_FIRST				= 0x0001;
const unsigned CEProperties::PROP_MASK_IS_SUM				= 0x0002;
const unsigned CEProperties::PROP_MASK_IS_PRODUCT			= 0x0004;
const unsigned CEProperties::PROP_MASK_COMMUTATIVE			= 0x0008;
const unsigned CEProperties::PROP_MASK_SINGLE_ITEM			= 0x0010;
const unsigned CEProperties::PROP_MASK_IS_TENSOR			= 0x0020;
const unsigned CEProperties::PROP_MASK_IS_SPINOR			= 0x0080;
const unsigned CEProperties::PROP_MASK_IS_NUMBER			= 0x0040;
const unsigned CEProperties::PROP_MASK_IS_NUMERIC_VALUE		= 0x0100;
const unsigned CEProperties::PROP_MASK_IS_VIRT_INTEGRAL		= 0x0200;
const unsigned CEProperties::PROP_MASK_IS_FUNCTION			= 0x0400;
const unsigned CEProperties::PROP_MASK_IS_FIELD_OPERATOR	= 0x0800;
const unsigned CEProperties::PROP_MASK_IS_WAVE_FUNCTION		= 0x1000;
const unsigned CEProperties::PROP_MASK_IS_PARTICLE_FUNCTION	= 0x2000;
const unsigned CEProperties::PROP_MASK_SELF_CONJUGATED		= 0x4000;
const unsigned CEProperties::PROP_MASK_ALL					= 0xffff;

CEProperties::CEProperties()  : CEObject(true) {
	m_defined_props = 0;
	m_props = PROP_MASK_FIRST | PROP_MASK_COMMUTATIVE | PROP_MASK_SELF_CONJUGATED | PROP_MASK_SINGLE_ITEM;
}

CEProperties::~CEProperties() { }

const char* CEProperties::CLASS_NAME = "CEProperties";
const char* CEProperties::ClassName() const { return CLASS_NAME; }

bool CEProperties::isSingleItem() {
	return getProperty(PROP_MASK_SINGLE_ITEM);
}

bool CEProperties::isCommutative() {
	return getProperty(PROP_MASK_COMMUTATIVE);
}

bool CEProperties::isTensor() {
	return getProperty(PROP_MASK_IS_TENSOR);
}

bool CEProperties::isSpinor() {
	return getProperty(PROP_MASK_IS_SPINOR);
}

bool CEProperties::getProperty(unsigned prop_mask) {
	if (m_defined_props & prop_mask) {
		return ((m_props & prop_mask) != 0);
	}
	else {
		undefinedPropertyRequest(prop_mask);
		//turtile: this should call an exception!!!
		return false;
	}
}

bool CEProperties::setSingleItem(bool single) {
	return setProperty(PROP_MASK_SINGLE_ITEM, single);
}

bool CEProperties::setCommutative(bool commutative) {
	return setProperty(PROP_MASK_COMMUTATIVE, commutative);
}

bool CEProperties::setTensor(bool is_tensor) {
	return setProperty(PROP_MASK_IS_TENSOR, is_tensor);
}

bool CEProperties::setSpinor(bool is_spinor) {
	return setProperty(PROP_MASK_IS_SPINOR, is_spinor);
}

bool CEProperties::setProperty(unsigned prop_mask, bool value) {
	bool old = ((m_defined_props & prop_mask)?((m_props & prop_mask) != 0):false);
	m_props |= prop_mask;
	if (!value) m_props ^= prop_mask;
	m_defined_props |= prop_mask;
	return old;
}

void CEProperties::undefinedPropertyRequest(unsigned props) {
	log("<Warning><Undefined property requested>");
}

bool CEProperties::isSum()
{
	return getProperty(PROP_MASK_IS_SUM);
}

bool CEProperties::isProduct()
{
	return getProperty(PROP_MASK_IS_PRODUCT);
}

bool CEProperties::isNumber()
{
	return getProperty(PROP_MASK_IS_NUMBER);
}

bool CEProperties::equals(CEProperties *other)
{
	return (data() == other->data());
}

unsigned CEProperties::data()
{
	return m_props;
}

unsigned long CEProperties::set_property_to(unsigned long lprops, unsigned long prop_mask, bool val) {
	unsigned long res=lprops;
	res|=prop_mask; if(!val)res^=prop_mask;
	return res;
}
