////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ERealValue.cpp: implementation of the CERealValue class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <math.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CERealValue::CERealValue(double val) : CEValue()
{
	m_value = val;
}

CERealValue::~CERealValue()
{

}

DWORD CERealValue::get_val_type()
{
	return CEValue::VTYPE_REAL;
}

CExpression* CERealValue::add(CExpression * other)
{
	if (other == NULL) return NULL;
	if (other->getProperty(CEProperties::PROP_MASK_IS_NUMERIC_VALUE)) {
		if (((CEValue*)other)->get_val_type() & CEValue::VTYPE_REAL) {
			CERealValue* ov = (CERealValue*)other;
			normalize();
			m_value += ov->to_double();
			delete ov;
			return this;
		}
		else return other->add(this);
		//	************************	should it works????
		//	************************	it means {delete this; return res;}
	}

	//This should not occur
	dbg->log(0, "(ADD) Unexpected case");
	return CExpression::add(other);
}

CExpression* CERealValue::multiply(CExpression * other)
{
	if (other == NULL) return NULL;
	if (other->getProperty(CEProperties::PROP_MASK_IS_NUMERIC_VALUE)) {
		if (((CEValue*)other)->get_val_type() & CEValue::VTYPE_REAL) {
			CERealValue* ov = (CERealValue*)other;
			normalize();
			m_value *= ov->to_double();
			delete ov;
			return this;
		}
		else return other->multiply(this);
		//	************************	should it works????
		//	************************	it means {delete this; return res;}
	}

	//This should not occur
	dbg->log(1, "(MULTIPLY) Unexpected case");
	return CExpression::multiply(other);
}

CEValue* CERealValue::inverse(bool do_coinverse_level)
{
	if (m_value != 0) {
		m_value = 1/m_value;
		if (do_coinverse_level) inverseLevel();
	}
	else {
		dbg->log(0, "Error: divizion by zero during inversing");
	}
	return this;
}

void CERealValue::apply_sign()
{
	m_value = -m_value;
	inverseFlag();
}

CEString* CERealValue::toString(int mode) {
	char* data = (char*)CMemoryManager::mmalloc(32);
	CEString* res = NULL;
	double val=m_value;
	//if (m_value<0) apply_sign(); // pre-arrage
	bool bMinus=false; // to avoid "--1"
	if(val<0){ val=-val; bMinus=!bMinus; }
	if (!getFlag()) bMinus=!bMinus;

	switch (mode) {
	case STORE_PLAIN_TEXT:
		sprintf(data, "%g", val); res=new CEString(data); if(bMinus)res->insertString("-", 0);
		break;
	case STORE_CPP:
		CEUtil::PrintExpForm(data,val,'e'); res=new CEString(data); if(bMinus)res->insertString("-", 0);
		break;
	case STORE_FORTRAN_SYMBOL:
		CEUtil::PrintExpForm(data,val,'D'); res=new CEString(data); if(bMinus)res->insertString("-", 0);
		break;
	default:
		sprintf(data, "%g", val); res=new CEString(data); if(bMinus){ res->insertString("- \\(",0); res->AddString("\\)"); }
	}

	CMemoryManager::mfree(data);
	return res;
}

CExpression* CERealValue::clone() const
{
	CERealValue* res = new CERealValue(m_value);
	copyAttributesTo(res);
	return res;
}

CERealValue* CERealValue::Sqrt()
{
	//if ((!getFlag()) ^ (m_value < 0)) {
	if (((getFlag()) && (m_value < 0)) || ((m_value > 0) && (!getFlag()))) {
		dbg->log(1, "sqrt(<0)");
		return NULL;
	}
	CERealValue* res = new CERealValue(sqrt(fabs(m_value)));
	if (!getLevel()) res->inverseLevel();
	return res;
}

CERealValue* CERealValue::Sqr()
{
	CERealValue* res = new CERealValue(m_value*m_value);
	if (!getLevel()) res->inverseLevel();
	return res;
}

void CERealValue::Abs()
{
	normalize();
	m_value = fabs(m_value);
}

void CERealValue::normalize()
{
	if (!getFlag()) apply_sign();
	if (!getLevel()) inverse();
}

bool CERealValue::less_than(CExpression * other)
{
	if (other->getProperty(CEProperties::PROP_MASK_IS_NUMERIC_VALUE)) {
		if (((CEValue*)other)->get_val_type() & CEValue::VTYPE_REAL) {
			CERealValue* ov = (CERealValue*)other;
			return (to_double() < ov->to_double());
		}
	}
	return CExpression::less_than(other);
}

CEValue* CERealValue::Sqrtx(bool do_clone)
{
	if (do_clone) return ((CERealValue*)clone())->Sqrtx();
	normalize();
	if (m_value < 0) {
		dbg->log(1, "Warning: sqrtx(<0)");
		Abs();
	}
	m_value = sqrt(m_value);
	return this;
}

CEValue* CERealValue::Absx(bool do_clone)
{
	if (do_clone) return ((CEValue*)clone())->Absx();
	Abs();
	return this;
}

CEValue* CERealValue::multiply(double val)
{
	normalize();
	m_value *= val;
	return this;
}

CEValue* CERealValue::Sqrx(bool do_clone)
{
	if (do_clone) return ((CEValue*)clone())->Sqrx();
	normalize();
	m_value = m_value*m_value;
	return this;;
}

double CERealValue::to_double()
{
	double res = m_value;
	if (!getFlag()) res = -res;
	if (!getLevel()) res = 1/res;
	return res;
}

bool CERealValue::equals(CExpression * other)
{
	if (other->getProperty(CEProperties::PROP_MASK_IS_NUMERIC_VALUE)) {
		if (((CEValue*)other)->get_val_type() & CEValue::VTYPE_REAL) {
			CERealValue* ov = (CERealValue*)other;
			return (to_double() == ov->to_double());
		}
	}

	//	should it be here?
	/*if (other->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) {
		CENumber* n = (CENumber*)other;
		return ((n->getNumIm() == 0) && (n->to_double() == to_double()));
	}*/
	return false;
}
