////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EReplList.cpp: implementation of the CEReplList class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

const char* CEReplList::CLASS_NAME = "CEReplList";

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CEReplList::CEReplList() : CERulesList() {}

// used for rule cloning
CEReplList::CEReplList(const CEPtrArray* _rules_list) : CERulesList(_rules_list) {}

CEReplList::CEReplList(const CExpression* a, const CExpression* b, bool do_exchange) : CERulesList() {
	add(a, b, do_exchange);
}

CEReplList::~CEReplList()
{
}

CERule* CEReplList::GetRuleCopy() const { return new CEReplList(m_list); }
const char* CEReplList::ClassName() const { return CLASS_NAME; }
int CEReplList::size() { return getRulesNum(); }
CEReplasement* CEReplList::get(int i) { return (CEReplasement*)getRule(i); }
void CEReplList::add(CEReplasement *new_repl) { addRule(new_repl); }

void CEReplList::add(const CExpression *a, const CExpression *b, bool do_exchange) {
	add(new CEReplasement(a, b, do_exchange));
}

CExpression* CEReplList::geta(int i) { return get(i)->a(); }
CExpression* CEReplList::getb(int i) { return get(i)->b(); }
bool CEReplList::exchange(int i) { return get(i)->exchange(); }

CEString* CEReplList::toString(int mode) {
	CEString* res  = new CEString("\\({");

	int s = size();
	int i;
	for(i=0; i<s; i++) {
		if (i != 0) res->AddString("; \n");
		res->AddString(geta(i)->toString(mode));
		if (exchange(i)) res->AddString("<->");
		else res->AddString("-->");
		res->AddString(getb(i)->toString(mode));
	}
	res->AddString("}\\)");
	return res;
}

CERule* CEReplList::combine_with(CERule* other) {
	if (other == NULL) return this;
	if (CEUtil::equals(other->ClassName(), CEReplasement::CLASS_NAME)) {
		add((CEReplasement*)other);
		return this;
	}
	return CERulesList::combine_with(other);
}
