////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ERulesList.cpp: implementation of the CERulesList class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

const char* CERulesList::CLASS_NAME = "CERulesList";

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CERulesList::CERulesList() : CERule()
{
	m_list = new CEPtrArray();
}

// used for rule cloning
CERulesList::CERulesList(const CEPtrArray* _rules_list) : CERule() {
	m_list = CEPtrArray::get_copy(_rules_list, true);
}

CERulesList::~CERulesList()
{
	delete m_list;
}

CERule* CERulesList::GetRuleCopy() const { return new CERulesList(m_list); }
int CERulesList::getRulesNum() { return m_list->size(); }
CERule* CERulesList::getRule(int i) { return (CERule*)(m_list->GetAt(i)); }
void CERulesList::addRule(CERule* new_rule) { m_list->add(new_rule); }
int CERulesList::GetRuleKind() { return CERule::RULE_COMPOUND; }
const char* CERulesList::ClassName() const { return CLASS_NAME; }

void CERulesList::ResetTimesApplied() {
	int s = getRulesNum();
	int i;
	for(i=0; i<s; i++) { getRule(i)->ResetTimesApplied(); }
}

DWORD CERulesList::GetTimesApplied() {
	DWORD res = 0;
	int s = getRulesNum();
	int i;
	for(i=0; i<s; i++) { res += getRule(i)->GetTimesApplied(); }
	return res;
}

bool CERulesList::contain_vector(void * obj) {
	if (m_list == obj) return true;
	return m_list->contain_vector(obj);
}

CERule* CERulesList::extractRule(int i) {
	if ((i < 0) && (i >= getRulesNum())) return NULL;
	CERule* res = getRule(i);
	m_list->RemoveAt(i, false);
	return res;
}

void CERulesList::RemoveAll() { m_list->clear_all(); }

CERule* CERulesList::combine_with(CERule* other) {
	if (other == NULL) return this;
	if (other->GetRuleKind() & CERule::RULE_COMPOUND) {
		// do not concat ReplList with RulesList to ReplList.
		if ( (CEUtil::equals(ClassName(), other->ClassName())) ||
			(CEUtil::equals(ClassName(), CLASS_NAME)) ) {
			CERulesList* other_list = (CERulesList*)other;
			int s = other_list->getRulesNum();
			int i;
			for(i=0; i<s; i++) {
				addRule(other_list->extractRule(0));
			}
			delete other_list;
		} else {
			// create default common rule.
			return CERule::combine_with(other);
		}
	} else {
		if (CEUtil::equals(ClassName(), CLASS_NAME)) addRule(other);
		else return CERule::combine_with(other);
	}
	return this;
}

CEString* CERulesList::toString(int mode)
{
	CEString* res  = new CEString("\\({");
	int s = getRulesNum();
	int i;
	for(i=0; i<s; i++) {
		if (i != 0) res->AddString("; \n");
		res->AddString(getRule(i)->toString(mode));
	}
	res->AddString("}\\)");
	return res;
}

CExpression* CERulesList::ApplyTo(CExpression* src, bool del_src) {
	CExpression* res = src;
	int s = getRulesNum();
	bool del_sub_src = del_src;
	int i;
	for(i=0; i<s; i++) {
		res = getRule(i)->ApplyTo(res, del_sub_src);
		if (!del_sub_src) {
			if (res != src) del_sub_src = true;
		}
	}
	return res;
}
