////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ESocketRule.cpp: implementation of the CESocketRule class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

const char* CESocketRule::CLASS_NAME = "CESocketRule";
const int CESocketRule::SR_APPLY_TO_CHANGED_SOURCE	= 0x01;
const int CESocketRule::SR_APPLY_TO_SUBITEMS_FIRST	= 0x02;
const int CESocketRule::USE_EXTERNAL_DATA_ARRAY		= 0x04;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//	NOTE: external function MAY modify, but MAY NOT delete source object.
//////////////////////////////////////////////////////////////////////
CESocketRule::CESocketRule(void* pFunc, CEPtrArray* data_array, int _settings) : CERule() {
	m_settings = _settings;
	if ((m_settings & USE_EXTERNAL_DATA_ARRAY) == 0) {
		m_data_array = CEPtrArray::get_copy(data_array, false);
	} else {
		m_data_array = data_array;
	}
	m_pFunction = NULL;
	if (pFunc != NULL) {
		try{
			//m_pFunction = (CExpression *(__cdecl *)(CExpression*, CEPtrArray*))pFunc;
			m_pFunction = (CExpression *(*)(CExpression*, CEPtrArray*))pFunc;
		} catch(...) {
			m_pFunction = NULL;
			dbg->log("ERROR: illegal CESocketRule initialization (illegal function type).");
			throw(new CEException(CEException::INT_ERROR));
		}
	}
}

CESocketRule::~CESocketRule() {
	m_pFunction = NULL;
	if ((m_settings & USE_EXTERNAL_DATA_ARRAY) == 0) {
		if (m_data_array != NULL) delete m_data_array;
	}
}

CERule* CESocketRule::GetRuleCopy() const { return new CESocketRule((void*)m_pFunction, m_data_array, m_settings); }

CExpression* CESocketRule::ApplyTo(CExpression* src, bool del_src) {
	if (m_pFunction == NULL) return src; // no need to apply NULL function to sub-items

	bool apply_subs_first = ((m_settings & SR_APPLY_TO_SUBITEMS_FIRST) != 0);
	if (apply_subs_first) {
		if (src != NULL) {
			CExpression* sub_res = src->applyRule(this);
			if (sub_res != src) {
				if (del_src) delete src;
				else del_src = true;
				src = sub_res;
			}
		}
	}

	// apply function to source
	CExpression* res = (*m_pFunction)(src, m_data_array);
	bool src_changed = (res != src);
	if ((src_changed) && (del_src) && (src != NULL)) delete src;

	if (!apply_subs_first) {
		// apply to sub-items if function processed inline or if 'SR_APPLY_TO_CHANGED_SOURCE' flag is set
		if ((!src_changed) || (m_settings & SR_APPLY_TO_CHANGED_SOURCE)) {
			if (res != NULL) {
				CExpression* sub_res = res->applyRule(this);
				if (sub_res != res) {
					if ((src_changed) || (del_src)) delete res;
					res = sub_res;
				}
			}
		}
	}
	return res;
}

const char* CESocketRule::ClassName() const { return CLASS_NAME; }

CEString* CESocketRule::toString(int mode) {
	CEString* res  = new CEString("\\({ CESocketRule }\\)");
	return res;
}

bool CESocketRule::contain_vector(void * obj) {
	if (m_data_array != NULL) {
		if (m_data_array == obj) return true;
		if (m_data_array->contain_vector(obj)) return true;
	}
	return false;
}
