////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ESum.cpp: implementation of the CESum class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#define SUM_LENGTH_TO_SCROLL_ARRANGE			3000
#define SUM_LENGTH_TO_SCROLL_ARRANGE_FACTORS	3000
#define SUM_LENGTH_TO_SCROLL_EXPAND				500
#define SUM_LENGTH_TO_SCROLL_TENSOR_ARRANGE		3000
#define SUM_LENGTH_TO_SCROLL_EVALUATE			100

#define SUM_LENGTH_TO_NOCLONE_ARRANGE			50

const char*	CESum::CLASS_TAG_NAME = "sum";

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
class CENamedItemAlt : public CEObject  
{
public:
	virtual bool contain_vector(void * obj);
	char* id();
	CEObject* obj();
	CENamedItemAlt(CEObject* o, char* _id);
	virtual ~CENamedItemAlt();

	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual char* toLine(int mode = 0);

private:
	char* m_id;
	CEObject* m_obj;
};

CENamedItemAlt::CENamedItemAlt(CEObject* o, char* _id) : CEObject(true)
{
	m_obj = o;
	m_id = _id;
}

CENamedItemAlt::~CENamedItemAlt()
{
	CMemoryManager::mfree(m_id);
}

const char* CENamedItemAlt::CLASS_NAME = "CENamedItemAlt";
const char* CENamedItemAlt::ClassName() const { return CLASS_NAME; }

CEObject* CENamedItemAlt::obj()
{
	return m_obj;
}

char* CENamedItemAlt::id()
{
	return m_id;
}

bool CENamedItemAlt::contain_vector(void * obj)
{
	if (m_id == obj) return true;
	/*if ((m_obj == obj) || (m_obj->contain_vector(obj))) return true;*/
	return false;
}

char* CENamedItemAlt::toLine(int mode) {
	char* res = CEUtil::copyString("{");
	char* res_tmp = NULL;
	if (m_id != NULL) {
		res_tmp = CEUtil::concat(res, m_id); CMemoryManager::mfree(res); res = res_tmp;
	} else {
		res_tmp = CEUtil::concat(res, "NULL"); CMemoryManager::mfree(res); res = res_tmp;
	}
	res_tmp = CEUtil::concat(res, ": "); CMemoryManager::mfree(res); res = res_tmp;
	
	char* obj_line = NULL;
	if (m_obj != NULL) obj_line = m_obj->toLine(mode);
	if (obj_line != NULL) {
		res_tmp = CEUtil::concat(res, obj_line); CMemoryManager::mfree(res); res = res_tmp;
		CMemoryManager::mfree(obj_line);
	} else {
		res_tmp = CEUtil::concat(res, "NULL"); CMemoryManager::mfree(res); res = res_tmp;
	}

	res_tmp = CEUtil::concat(res, "}"); CMemoryManager::mfree(res); res = res_tmp;
	return res;
}

/////////////////////////////////////////////////////////////////////////////////////////
///	CESum
/////////////////////////////////////////////////////////////////////////////////////////
CDebug* CESum::dbg = new CDebug("CESum");

#ifdef USE_PLAIN_ARRAY_FORSEQ // biuld sequence-expressions on CAPtrArray* basis
CESum::CESum(CEPtrArray* pPtrArray):CSequence(pPtrArray) {}
#else
CESum::CESum(CAPtrSeq* pPtrArray):CSequence(pPtrArray) {}
#endif //USE_PLAIN_ARRAY_FORSEQ

CESum::CESum():CSequence()
{
}

CESum::~CESum()
{

}

CExpression* CESum::Simplify(int level) { return NULL; }

CExpression* CESum::Arrange(int level)
{
	dbg->log(15, "Arrange", this);
	CExpression* temp = NULL;

	//	*****************************************************	<scroll>
	long initial_size = size();
	CEProgressWatch* pw = NULL;
	//if (initial_size > SUM_LENGTH_TO_SCROLL_ARRANGE) { dbg->logperiod(0, "Sub-items arrangement START"); } // DEBUG
	if (initial_size > SUM_LENGTH_TO_SCROLL_ARRANGE) { pw = CEProcessor::newProgressWatch("Long Sum Sub-items arrangement (%ld items)", initial_size); }
	//	*****************************************************	<\scroll>
	long i;
	for (i=0; i<size();) {
		CExpression* item = GetAt(i);
		temp = CExpression::arrge(item, false);
		//dbg->log(55, "sub-item arranged", temp);
		if (temp != item) {
			if (CEBasic::isZero(temp)) {
				RemoveAt(i);
			} else {
				SetAt(i, temp);
				i++;
			}
			delete temp;
		} else {
			if (CEBasic::isZero(temp)) { RemoveAt(i); }
			else {i++;}
			//i++;
		}
		//	*****************************************************	<scroll>
		if (pw != NULL) pw->step(i+1, size());
		//	*****************************************************	<\scroll>
	}
	dbg->log(45, "sub-items arranged", this);
	//	*****************************************************	<scroll>
	if (pw != NULL) CEProcessor::dismissProgressWatch(pw);
	//if (initial_size > SUM_LENGTH_TO_SCROLL_ARRANGE) { dbg->logperiod(0, "Sub-items arrangement END, %ld"); } // DEBUG
	//	*****************************************************	<\scroll>

	if (size() < SUM_LENGTH_TO_NOCLONE_ARRANGE) {
		arrangeFactors2(); // Arranging sum factors
	} else {
		arrangeFactorsAdv(); // Arranging sum factors
	}

	arrangeSign();
	dbg->log(35, "factors & sign arranged", this);

	if (level == AL_EXTRACT) {
		long s = size();
		if (s == 0) {
			CExpression* res = CENumber::GetZero();
			copyAttributesTo(res, true);
			dbg->log(45, "0 items in sum ==> zero", res);
			return res;
		}
		else if (s == 1) {
			CExpression* res = GetAt(0)->clone();
			copyAttributesTo(res, true);
			dbg->log(45, "1 items in sum", res);
			return res;
		}
	}

	return this;
}

CExpression* CESum::add(CExpression* other) {
	if (!getFlag()) other->inverseFlag();
	addItem(other);
	return this;
}

//void CESum::SetAt(long i, CExpression* _new) { CSequence::SetAt(i, _new); }
//void CESum::SetAtX(long i, CExpression* _new_X) { CSequence::SetAt(i, _new_X); }

CExpression* CESum::evaluate(int level)
{
	//	*****************************************************	<scroll>
	CEProgressWatch* pw = NULL;
	long initial_size = size();
	if (initial_size > SUM_LENGTH_TO_SCROLL_EVALUATE) { pw = CEProcessor::newProgressWatch("Sum Evaluating (%ld items)", initial_size); }
	//	*****************************************************	<\scroll>

	CExpression* _old = NULL;
	CExpression* _new = NULL;
	long i;
	for(i=0; i<size(); i++) // note: size may not be changed during SetAt().
	{
		_old = GetAt(i);
		_new = _old->evaluate();
		if (_old != _new) {
			if (_new != NULL) {
				SetAtX(i, _new);
			} else {
				dbg->log(0, "<Warning><NULL as evaluate()'s result.> failed on item", _old);
			}
		}
		//	*****************************************************	<scroll>
		if (pw != NULL) pw->step(i+1, initial_size);
		//	*****************************************************	<\scroll>
	}
	//	*****************************************************	<scroll>
	if (pw != NULL) CEProcessor::dismissProgressWatch(pw);
	//	*****************************************************	<\scroll>

	return this;
}

CExpression* CESum::applyRule(CEAbstractRule* rule)
{
	CERule* r = (CERule*)rule;

	long i;
	for (i=0; i<size(); i++) {
		CExpression* item = GetAt(i);
		CExpression* item_repl = r->ApplyTo(item, false);
		if ((item_repl != item) && (item_repl != NULL)) {
			SetAtX(i, item_repl);
		}
	}
	return this;
}

CEString* CESum::toString(int mode) {
	CEString* res = NULL;
	long _size = size();
	switch (mode)
	{
	case STORE_TEX:
		if (_size>1)
		{
			bool sign = getFlag();
			if (sign) res = new CEString("");
			else res = new CEString("- (");
			CExpression* pE = GetAt(0);
			if ((pE->getFlag()) && (pE->getProperty(CEProperties::PROP_MASK_IS_SUM))) {
				// +-signed sum should be bracketed here
				// (-signed is bracketed in sum->toString())
				res->AddString("("); res->AddString(pE->toString(mode)); res->AddString(")");
			} else {
				res->AddString(pE->toString(mode));
			}
			long i;
			for(i=1; i<_size; i++)
			{
				pE = GetAt(i);
				bool pE_sign = pE->getFlag();
				if (pE_sign) { res->AddString(" + "); } else { res->AddString(" "); }

				if ((pE_sign) && (pE->getProperty(CEProperties::PROP_MASK_IS_SUM))) {
					// +-signed sum should be bracketed here
					// (-signed is bracketed in sum->toString())
					res->AddString("("); res->AddString(pE->toString(mode)); res->AddString(")");
				} else {
					res->AddString(pE->toString(mode));
				}
			}
			if (sign) res->AddString("");
			else res->AddString(")");
		}
		else
		{
			if (_size == 1)
			{
				CExpression* pE = GetAt(0);
				if (getFlag()) res = pE->toString(mode);
				else {
					if (pE->getFlag()) {
						res = new CEString("- ");
						if (pE->getProperty(CEProperties::PROP_MASK_IS_SUM)) {
							res->AddString("("); res->AddString(pE->toString(mode)); res->AddString(")");
						} else {
							res->AddString(pE->toString(mode));
						}
					}
					else {
						res = new CEString("- (");
						res->AddString(pE->toString(mode));
						res->AddString(")");
					}
				}
			}
			else
			{
				// 0 items
				if (getFlag()) res = new CEString("0");
				else res = new CEString("-0");
			}
		}
		break;
	case STORE_WR_MATHEMATICA:
		if (_size>1)
		{
			bool sign = getFlag();
			if (sign) res = new CEString("\\(");
			else res = new CEString("- \\((");
			CExpression* pE = GetAt(0);
			if ((pE->getFlag()) && (pE->getProperty(CEProperties::PROP_MASK_IS_SUM))) {
				// +-signed sum should be bracketed here
				// (-signed is bracketed in sum->toString())
				res->AddString("\\(("); res->AddString(pE->toString(mode)); res->AddString(")\\)");
			} else {
				res->AddString(pE->toString(mode));
			}
			long i;
			for(i=1; i<_size; i++)
			{
				pE = GetAt(i);
				bool pE_sign = pE->getFlag();
				if (pE_sign) { res->AddString(" + "); } else { res->AddString(" "); }

				if ((pE_sign) && (pE->getProperty(CEProperties::PROP_MASK_IS_SUM))) {
					// +-signed sum should be bracketed here
					// (-signed is bracketed in sum->toString())
					res->AddString("\\(("); res->AddString(pE->toString(mode)); res->AddString(")\\)");
				} else {
					res->AddString(pE->toString(mode));
				}
			}
			if (sign) res->AddString("\\)");
			else res->AddString(")\\)");
		}
		else
		{
			if (_size == 1)
			{
				CExpression* pE = GetAt(0);
				if (getFlag()) res = pE->toString(mode);
				else {
					if (pE->getFlag()) {
						res = new CEString("- ");
						if (pE->getProperty(CEProperties::PROP_MASK_IS_SUM)) {
							res->AddString("\\(("); res->AddString(pE->toString(mode)); res->AddString(")\\)");
						} else {
							res->AddString(pE->toString(mode));
						}
					}
					else {
						res = new CEString("- \\((");
						res->AddString(pE->toString(mode));
						res->AddString(")\\)");
					}
				}
			}
			else
			{
				// 0 items
				if (getFlag()) res = new CEString("0");
				else res = new CEString("-0");
			}
		}
		break;
	default:
		res = new CEString("<Not supported data format.>");
	}
	return res;
}

void CESum::toStream(ostream* os, int mode)
{
}

/*
* The collection of sub-items properties should be added.
*/
void CESum::collectProperties(CEProperties* props, unsigned set) {
	//CExpression::collectProperties(props, set);
	props->setSingleItem(size() <= 1);
	props->setProperty(CEProperties::PROP_MASK_IS_SUM, true);
	long s = size();
	long i;
	for (i=0; i<s; i++) {
		unsigned long lPropsI=GetAt(i)->props_long();
		if ((lPropsI & CEProperties::PROP_MASK_COMMUTATIVE)==0) props->setCommutative(false);
		if ((lPropsI & CEProperties::PROP_MASK_SELF_CONJUGATED)==0) props->setProperty(CEProperties::PROP_MASK_SELF_CONJUGATED, false);
	}
}

unsigned long CESum::props_long() {
	unsigned long res = CExpression::props_long();
	res = CEProperties::set_property_to(res, CEProperties::PROP_MASK_SINGLE_ITEM, (size() <= 1));
	res = CEProperties::set_property_to(res, CEProperties::PROP_MASK_IS_SUM, true);

	//bool bLog=(!getLevel());
	long s = size();
	long i;
	for (i = 0; i<s; i++) {
		unsigned long lPropsI=GetAt(i)->props_long();
		if ((lPropsI & CEProperties::PROP_MASK_COMMUTATIVE)==0) res=CEProperties::set_property_to(res,CEProperties::PROP_MASK_COMMUTATIVE,false);
		//if (bLog) { dbg->log(2,"comm:",this,GetAt(i)); CEException::halt();
		if ((lPropsI & CEProperties::PROP_MASK_SELF_CONJUGATED)==0) res=CEProperties::set_property_to(res,CEProperties::PROP_MASK_SELF_CONJUGATED,false);
	}
	
	return res;
}

const char*	CESum::CLASS_NAME = "CESum";
const char* CESum::ClassName() const { return CLASS_NAME; }

/*
* We do not care about "recursive"'s value.
*/
CExpression* CESum::expand(bool recursive)
{
	dbg->log(15, "expand", this);

	//	First reducing all the sub-items.
	//	*****************************************************	<scroll>
	long initial_size = size();
	CEProgressWatch* pw = NULL;
	if (initial_size > SUM_LENGTH_TO_SCROLL_EXPAND) { pw = CEProcessor::newProgressWatch("Long Sum Expanding (%ld items)", initial_size); }
	//	*****************************************************	<\scroll>
	long i;
	for(i=0; i<size(); i++)
	{
		CExpression* item = GetAt(i);
		//dbg->log(75, "sum-item reducing", item);
		//CExpression* temp = item->expand();

		//CExpression* temp = CExpression::reduce(item, false);
		CExpression* temp2 = item->expand();
		CExpression* temp = arrge(temp2, (temp2 != item));

		//dbg->log(75, "sub-item reduced", temp);
		if ((item != temp) && (temp != NULL)) {
			SetAtX(i, temp);
		}

		//	And expanding sums.
		i += expandElement(i);

		//	*****************************************************	<scroll>
		if (pw != NULL) pw->step(initial_size + i+1 - size(), initial_size);
		//	*****************************************************	<\scroll>
	}
	//	*****************************************************	<scroll>
	if (pw != NULL) CEProcessor::dismissProgressWatch(pw);
	//	*****************************************************	<\scroll>

	//dbg->log(25, "expanded", this);
	return this;
}

/*
* Inlines the sum at 'num'-s place.
* Returns the surplus of items in sum.
*/
long CESum::expandElement(long num)
{
	long res = 0;
	CExpression* item = GetAt(num);

	// do not inline denominator (also sum)
	if (!item->getLevel()) return res;

	if (item->getProperty(CEProperties::PROP_MASK_IS_SUM)) {
		//bool do_log = (((CESum*)item)->size()>100); // DEBUG
		//if (do_log) dbg->logperiod(0, "expandElement"); // DEBUG

		CESum* sum = (CESum*)item;
		long s = sum->size();
		switch (s) {
		case 0:
			//This case should not occur
			break;
		case 1:
			//This case should not occur
			break;
		default:
			bool sign = sum->getFlag();
			CExpression* temp = sum->GetAt(0);
			//if (temp == NULL) { dbg->log(0, "Error, NULL extracted at 0-pos", sum); }
			if (!sign) temp->inverseFlag();
			UnsafeSetAt(num, temp);
			long i;
			for(i=1; i<s; i++) {
				temp = sum->GetAt(i); // all the prevoius are extracted ==> 0 pos
				//if (temp == NULL) { dbg->log(0, "Error, NULL extracted at 0-pos", sum); break; }
				if (!sign) temp->inverseFlag();
				InsertAtX(num+i, temp);
				res++;
			}
			sum->ClearWODelObjs();
			delete sum;
		}
	}

	return res;
}

CExpression* CESum::clone() const
{
	CESum* res = new CESum();
	long _size = size();
	long i;
	for(i=0; i<_size; i++)
	{
		res->addItem(GetAtConst(i)->clone());
	}
	copyAttributesTo(res, true);
	return res;
}

bool CESum::arrangeFactors()
{
	dbg->log(0, "Error: Old-version arrangeFactors() involved.\n");
	dbg->log(25, "arrangeFactors", this);
	bool res = false;
	CExpression* temp = NULL;
	long i;
	for (i=0;i<size(); i++) {
		temp = GetAt(i);
		CExpression* factor = NULL;
		CExpression* base = temp->clone();
		if (base->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
			CEProduct* prd = (CEProduct*)base;
			if (prd->size() > 0) {
				temp = prd->GetAt(0);
				if (temp->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) {
					factor = temp->clone();
					prd->RemoveAt(0);
					//((CEProduct*)base)->RemoveAt(0);//this equals to prd->RemoveAt(0);
				}
			}
		}
		if (factor == NULL) factor = CENumber::GetUnit();
		if (!base->getFlag()) {
			factor->inverseFlag();
			base->setFlag();
		}

		bool changed = false;
		long j;
		for (j=i+1; j<size();) {
			temp = GetAt(j);
			CExpression* o_base = temp->clone();
			CExpression* o_factor = NULL;
			if (o_base->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
				CEProduct* prd = (CEProduct*)o_base;
				if (prd->size() > 0) {
					temp = prd->GetAt(0);
					if (temp->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) {
						o_factor = temp->clone();
						prd->RemoveAt(0);
						//((CEProduct*)o_base)->RemoveAt(0);//this equals to prd->RemoveAt(0);
					}
				}
			}
			if (o_factor == NULL) o_factor = CENumber::GetUnit();
			if (!o_base->getFlag()) {
				o_factor->inverseFlag();
				o_base->setFlag();
			}
			
			if(base->equals(o_base)) {
				factor = factor->add(o_factor);
				RemoveAt(j);
				changed = true;
			} else {
				j++;
				delete o_factor;
			}
			delete o_base;
		}

		//CESupport::Save("CESum::arrangeFactors : 1");
		//CESupport::SaveResults(base);
		//CESupport::SaveResults(factor);
		
		if (changed) {
			dbg->log(75, "arrangeFactors : changed.");
			if (!factor->getFlag()) {
				base->inverseFlag();
				factor->setFlag();
			}
			if (CEBasic::isUnit(factor)) {
				SetAtX(i, base);
				delete factor;
			}
			else if (CEBasic::isZero(factor)) {
				RemoveAt(i);
				i--;
				delete base;
				delete factor;
			}
			else {
				//CESupport::SaveResults(base);
				//CESupport::SaveResults(factor);
				CExpression* subst = CExpression::reduce(factor->multiply(base));
				/*if ((subst_temp != subst) && (subst_temp != NULL)) {
					delete subst;
					subst = subst_temp;
				}*/
				SetAtX(i, subst);
			}
			res = true;
		}
		else {
			delete base;
			delete factor;
		}
	}
	//CESupport::Save("CESum::arrangeFactors finished.");
	//CESupport::SaveResults(this);
	return res;
}

bool CESum::arrangeFactorsAdv()
{
	dbg->log(35, "arrangeFactorsAdv", this);
	bool res = false;

	//	Creating items cashe data.
#ifdef USE_PLAIN_ARRAY_FORSEQ
	CEPtrArray data_arr;// = new CEPtrArray();
#else
	CAPtrSeq data_arr;// = new CAPtrSeq();
#endif //USE_PLAIN_ARRAY_FORSEQ

	long s = size();
	//bool log_time = (s>100); if (log_time) dbg->logperiod(0, "CESum::arrangeFactorsAdv started"); //DEBUG
	//	*****************************************************	<scroll>
	CEProgressWatch* pw = NULL;
	if (s > SUM_LENGTH_TO_SCROLL_ARRANGE_FACTORS) { pw = CEProcessor::newProgressWatch("Long sum arranging (%d items)", s); }
	//	*****************************************************	<\scroll>

	long i;
	/*for (i=0; i<s; i++) {
		CExpression* temp = GetAt(i);
		CExpression* factor = NULL;
		CExpression* base = temp->clone();

		if (base->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
			CEProduct* prd = (CEProduct*)base;
			if (prd->size() > 0) {
				temp = prd->GetAt(0);
				if (temp->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) {
					factor = temp->clone();
					prd->RemoveAt(0);
				}
			}
		}
		else if (base->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) {
			factor = base;
			base = CENumber::GetUnit();
		}

		if (factor == NULL) factor = CENumber::GetUnit();

		if (!base->getFlag()) {
			factor->inverseFlag();
			base->setFlag();
		}

		data_arr.add(new CENamedItemAlt(factor, base->toLine()));
		delete base;
	}*/
	for (i=0; i<s; i++) {
		CExpression* temp = GetAt(i);
		CExpression* factor = NULL;
		bool del_base=false;
		CExpression* base = temp;
		if (base->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
			CEProduct* prd = (CEProduct*)base;
			if (prd->size() > 0) {
				temp = prd->GetAt(0);
				if (temp->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) {
					factor = temp->clone();
					base=base->clone(); del_base=true;
					((CEProduct*)base)->RemoveAt(0);
				}
			}
		} else if (base->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) {
			factor = base->clone();
			base = CENumber::GetUnit(); del_base=true;
		}

		if (factor == NULL) factor = CENumber::GetUnit();
		bool inv_base = false;
		if (!base->getFlag()) {
			factor->inverseFlag();
			base->inverseFlag(); inv_base=true;
		}

		data_arr.add(new CENamedItemAlt(factor, base->toLine()));
		if (del_base) { delete base; }
		else if (inv_base) { base->inverseFlag(); }
		//	*****************************************************	<scroll>
		if (pw != NULL) pw->step(i+1, s);
		//	*****************************************************	<\scroll>
	}
	// Note: we can also store string length for faster equals() check
	//if (log_time) dbg->logperiod(0, "CESum::arrangeFactorsAdv cached"); //DEBUG

	// pre-arrangement check
	//	*****************************************************	<scroll>
	// if (pw != NULL) pw->step(0, 1);
	//	*****************************************************	<\scroll>
	bool pre_arranged=true;
	CENamedItemAlt* prev = (s>0)?((CENamedItemAlt*)data_arr.GetAt(0)):NULL;
	for (i=1; (i<s); i++) {
		CENamedItemAlt* b = (CENamedItemAlt*)data_arr.GetAt(i);
		if (!CEUtil::a_less_b(prev->id(), b->id())) { pre_arranged=false; break; }
		prev=b;
	}
	if (pre_arranged) {
		//	*****************************************************	<scroll>
		if (pw != NULL) CEProcessor::dismissProgressWatch(pw);
		//	*****************************************************	<\scroll>
		// Note: CENamedItemAlt::obj objects should be deleted manually ==> we delete factor at avery loop
		for (i=0;i<s; i++) { CENamedItemAlt* a = (CENamedItemAlt*)data_arr.GetAt(i); delete a->obj(); }
		return false;
	}

	//	*****************************************************	<scroll>
	if (pw != NULL) pw->step(0, 1);
	//	*****************************************************	<\scroll>
	//	Arranging factors
	// Note: CENamedItemAlt::obj objects should be deleted manually ==> we delete factor at avery loop
	for (i=0;i<size(); i++) {
		CENamedItemAlt* a = (CENamedItemAlt*)data_arr.GetAt(i);
		bool changed = false;
		CExpression* factor = (CENumber*)a->obj();
		long j;
		for (j=size()-1; j>i; j--) { // start from the end
			CENamedItemAlt* b = (CENamedItemAlt*)data_arr.GetAt2(j);
			if (CEUtil::equals(a->id(), b->id())) {
				factor = factor->add((CENumber*)b->obj());
				RemoveAt(j);
				data_arr.RemoveAt(j, false);
				delete b;
				changed = true;
			}
			//	*****************************************************	<scroll>
			//if ((j & 0x0f) == 0x0f) { if(pw!=NULL) { long s_tmp=size(); pw->step((2*s_tmp-i-1)*i/2+(s_tmp-j), s_tmp*(s_tmp-1)/2); } }
			//	*****************************************************	<\scroll>
		}

		if (changed) {
			dbg->log(75, "arrangeFactorsAdv : changed.");
			CExpression* base = GetAt(i)->clone();
			if (base->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
				CEProduct* prd = (CEProduct*)base;
				if (prd->size() > 0) {
					if (prd->GetAt(0)->getProperty(CEProperties::PROP_MASK_IS_NUMBER))
						prd->RemoveAt(0);
				}
			}
			else if (base->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) {
				delete base;
				base = CENumber::GetUnit();
			}

			if (!base->getFlag())
				base->setFlag();
			
			if (!factor->getFlag()) {
				base->inverseFlag();
				factor->setFlag();
			}
			//Note.
			//CExpression* subst = CExpression::reduce(factor->multiply(base));
			//Should be here.
			if (CEBasic::isUnit(factor)) {
				SetAtX(i, base);
				delete factor;
			}
			else if (CEBasic::isZero(factor)) {
				RemoveAt(i);
				data_arr.RemoveAt(i, true); // (or "data_arr->RemoveAt(i,false); delete a;")
				i--;
				delete base;
				delete factor;
			}
			else {
				//CExpression* subst = CExpression::reduce(factor->multiply(base));
				// fast arrange: sign & inline factor into possible product
				CExpression* subst = CExpression::times(factor, base);
				SetAtX(i, subst);
			}
			res = true;
		}
		else {
			delete factor; // <== CENamedItemAlt::obj objects should be deleted manually
		}
		//	*****************************************************	<scroll>
		if (pw != NULL) pw->step(i+1, size());
		//	*****************************************************	<\scroll>
	}
	//if (log_time) dbg->logperiod(0, "CESum::arrangeFactorsAdv reduced"); //DEBUG

	dbg->log(55, "alphabetical arrange: changed.");
	//Arranging sequence in alphabetical order.
	s = size();
	//	*****************************************************	<scroll>
	if (pw != NULL) pw->step(0, 1);
	//	*****************************************************	<\scroll>
	for (i=0; i<s; i++) {
		CENamedItemAlt* a = (CENamedItemAlt*)data_arr.GetAt(i);
		bool loop = true;
		long j;
		for (j=i-1; ((j>=0) && (loop)); j--) { // check right order first
			CENamedItemAlt* b = (CENamedItemAlt*)data_arr.GetAt(j);
			if (!CEUtil::a_less_b(a->id(), b->id())) {
				if (i > (j+1)) { move(i, j+1); data_arr.MoveFromTo(i,j+1); }
				loop = false;
			}
		}
		if (loop) { // a is less than all previous
			move(i, 0); data_arr.MoveFromTo(i,0);
		}
		//	*****************************************************	<scroll>
		if (pw != NULL) pw->step(i+1, s);
		//	*****************************************************	<\scroll>
	}

	//	*****************************************************	<scroll>
	if (pw != NULL) CEProcessor::dismissProgressWatch(pw);
	//	*****************************************************	<\scroll>

	//data_arr->clear();
	//delete data_arr;
	//if (log_time) dbg->logperiod(0, "CESum::arrangeFactorsAdv finished"); //DEBUG
	return res;
}

bool CESum::arrangeFactors2() {
	dbg->log(35, "arrangeFactors2", this);
	bool res = false;

	long s = size();
	//	*****************************************************	<scroll>
	CEProgressWatch* pw = NULL;
	if (s > SUM_LENGTH_TO_SCROLL_ARRANGE_FACTORS) { pw = CEProcessor::newProgressWatch("Long sum arranging (%d items)", s); }
	//	*****************************************************	<\scroll>
	long i;

	// pre-arrangement check
	bool pre_arranged=true;
	CExpression* prev = (s>0)?GetAt(0):NULL;
	for (i=1; i<s; i++) {
		CExpression* b = GetAt(i);
		if (!BasesLessThan(prev, b)) { pre_arranged=false; break; }
		prev = b;
	}
	if (pre_arranged) {
		//	*****************************************************	<scroll>
		if (pw != NULL) CEProcessor::dismissProgressWatch(pw);
		//	*****************************************************	<\scroll>
		return false;
	}

	//	Arranging factors
	//	*****************************************************	<scroll>
	// if (pw != NULL) pw->step(0, 1);
	//	*****************************************************	<\scroll>
	for (i=0;i<size(); i++) {
		CExpression* item_i = GetAt(i);
		bool changed = false;
		CExpression* factor = NULL;
		long j;
		for (j=size()-1; j>i; j--) { // start from the end
			CExpression* item_j = GetAt(j);
			if (BasesEquals(item_i,item_j)) {
				if (factor == NULL) { factor = exprGetFactor(item_i); if(changed)CEException::halt("CESum::arrangeFactors2():changed error");}
				factor = factor->add(exprGetFactor(item_j));
				RemoveAt(j);
				changed = true;
			}
		}

		if (changed) {
			dbg->log(75, "arrangeFactors2 : changed.");
			CExpression* base = exprGetBase(item_i);

			if (!factor->getFlag()) { base->inverseFlag(); factor->setFlag(); }

			//Note. CExpression* subst = CExpression::reduce(factor->multiply(base)); Should be here.
			if (CEBasic::isUnit(factor)) {
				SetAtX(i, base); delete factor;
			} else if (CEBasic::isZero(factor)) {
				RemoveAt(i); i--;
				delete base; delete factor;
			} else {
				// fast arrange: sign & inline factor into possible product
				CExpression* subst = CExpression::times(factor, base);
				SetAtX(i, subst);
			}
			res = true;
		}
		// factor is NULL elswhere

		//	*****************************************************	<scroll>
		if (pw != NULL) pw->step(i+1, size());
		//	*****************************************************	<\scroll>
	}

	//Arranging sequence in alphabetical order.
	s = size();
	//	*****************************************************	<scroll>
	if (pw != NULL) pw->step(0, 1);
	//	*****************************************************	<\scroll>
	for (i=0; i<s; i++) {
		CExpression* item_i = GetAt(i);
		//dbg->log(135, "sort item_i:",item_i);
		bool loop = true;
		long j;
		for (j=i-1; ((j>=0) && (loop)); j--) { // check right order first
			CExpression* item_j = GetAt(j);
			//dbg->log(135, "sort item_j:",item_j);
			if (!BasesLessThan(item_i, item_j)) { // previous pos is right
				//dbg->log(135, "sort BasesLessThan:");
				if (i > (j+1)) move(i, j+1);
				loop = false;
			}
		}
		if (loop) { // a is less than all previous
			move(i, 0);
		}
		//	*****************************************************	<scroll>
		if (pw != NULL) pw->step(i+1, s);
		//	*****************************************************	<\scroll>
	}

	//	*****************************************************	<scroll>
	if (pw != NULL) CEProcessor::dismissProgressWatch(pw);
	//	*****************************************************	<\scroll>

	return res;
}

bool CESum::BasesEquals(CExpression* a, CExpression* b) {
	//dbg->log(135, "BasesEquals()", a, b);
	bool a_num = a->getProperty(CEProperties::PROP_MASK_IS_NUMBER);
	bool b_num = b->getProperty(CEProperties::PROP_MASK_IS_NUMBER);
	if ((a_num) && (b_num)) return true;
	if ((a_num) || (b_num)) return false;

	if (a->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
		return ((CEProduct*)a)->BasesEqualsExpr(b); // check for b-product is therein
	} else if (b->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
		return ((CEProduct*)b)->BasesEqualsExpr(a); // check for a-product is therein
	}

	return CEBasic::equalsSignless(a,b);
}

bool CESum::BasesLessThan(CExpression* a, CExpression* b) {
	bool a_num = a->getProperty(CEProperties::PROP_MASK_IS_NUMBER);
	bool b_num = b->getProperty(CEProperties::PROP_MASK_IS_NUMBER);
	if (b_num) return false;
	if (a_num) return true;

	if (a->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
		return ((CEProduct*)a)->BasesLessThenExpr(b); // check for b-product is therein
	} else if (b->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
		return ((CEProduct*)b)->BasesGreatherThenExpr(a); // check for a-product is therein
	}

	bool a_sign = a->getFlag(); if (!a_sign) a->inverseFlag();
	bool b_sign = b->getFlag(); if (!b_sign) b->inverseFlag();
	bool res = a->less_than(b);
	if (!a_sign) a->inverseFlag();
	if (!b_sign) b->inverseFlag();
	return res;
}

CENumber* CESum::exprGetFactor(CExpression* a) { // returns cloned result
	CENumber* res = NULL;
	bool flag_check = true;
	if (a->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
		CEProduct* prd = (CEProduct*)a;
		if (prd->size() > 0) { if (prd->GetAt(0)->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) res = (CENumber*)(prd->GetAt(0)->clone()); }
	} else if (a->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) {
		res = (CENumber*)a->clone();
		flag_check=false; // sign is already involved
	}

	if (res == NULL) res = CENumber::GetUnit();

	if (flag_check) { if (!a->getFlag()) res->inverseFlag(); }
	return res;
}

CExpression* CESum::exprGetBase(CExpression* a) { // returns cloned result
	CExpression* res = a->clone();
	if (res->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
		CEProduct* prd = (CEProduct*)res;
		if (prd->size() > 0) {
			if (prd->GetAt(0)->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) prd->RemoveAt(0);
		}
	} else if (res->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) {
		delete res;
		res = CENumber::GetUnit();
	}

	if (!res->getFlag()) res->setFlag();
	return res;
}

CExpression* CESum::conjugated(int mode) {
	CESum* res = new CESum();
	CExpression* temp = NULL;
	long _size = size();
	long i;
	for(i=0; i<_size; i++)
	{
		CExpression* item=GetAt(i);
		temp=CEBasic::Congugate(item,false); if(temp==NULL)CEException::halt("NULL");
		if(temp==item)temp=temp->clone();
		if (temp != NULL) res->addItem(temp);
	}
	copyAttributesTo(res);
	return res;
}

/*
*	Note: inlines sum-results. TODO: check if inlining is valid/good here.
*/
CExpression* CESum::arrangeTensors() {
	long s = size();
	//if (s>100) dbg->logperiod(0, "arrangeTensors"); // DEBUG

	//	*****************************************************	<scroll>
	CEProgressWatch* pw = NULL;
	if (s > SUM_LENGTH_TO_SCROLL_TENSOR_ARRANGE) {
		pw = CEProcessor::newProgressWatch("Long sum tensor arrange (%d items)", s);
	}
	//	*****************************************************	<\scroll>

	//CExpression* res = this;
	long i;
	for(i=0; i<size(); i++)
	{
		CExpression* item = GetAt(i);
		CExpression* item_res = item->arrangeTensors();
		if ((item_res != item) && (item_res != NULL)) {
			SetAtX(i, item_res); // TODO: CHECK THIS LINE!!!!!
			i += expandElement(i); // inlines sum.
			// or add inline into further arrange
			// (no! inlining is the expanding).
		}

		//	*****************************************************	<scroll>
		if (pw != NULL) pw->step(i+1, size());
		//	*****************************************************	<\scroll>
	}

	//	*****************************************************	<scroll>
	if (pw != NULL) CEProcessor::dismissProgressWatch(pw);
	//	*****************************************************	<\scroll>

	//if (s>100) dbg->logperiod(0, "arrangeTensors END"); // DEBUG
	return this;
}

CExpression* CESum::numeric()
{
	//CEProcessor::find_numeric(this);
	long s = size();
	if (s<=0) return NULL;
	CExpression* res = GetAt(0)->numeric();
	long i;
	for (i=1; i<s; i++) {
		res = res->add(GetAt(i)->numeric());
	}
	copyAttributesTo(res, true);
	return res;
}

CESum* CESum::get_n_items(long n)
{
	CESum* res = new CESum();
	long _size = size();
	long i;
	for(i=0; ((i<_size) && (i<n)); i++)
	{
		res->addItem(GetAt(i)->clone());
	}
	copyAttributesTo(res);
	return res;
}

bool CESum::equals(CExpression *other)
{
	if (!other->getProperty(CEProperties::PROP_MASK_IS_SUM)) return false;

	CESum* o = (CESum*)other;
	long s = size();

	if (s != o->size()) return false;
	if (!attributes_equals_to(o)) return false;

	long i;
	for (i=0; i<s; i++) {
		if (!GetAt(i)->equals(o->GetAt(i))) return false;
	}
	return true;
}

bool CESum::arrangeSign() {
	long s=size(); //'s' may not be <0

	//truncate sign for empty sum
	if(s==0){ if(!getFlag()){ inverseFlag(); return true; } return false; }

	//truncate sign in ZERO expression: -{0}=>{0}.
	if(s==1){
		//note: 'GetAt(0)' should be already arranged, so that no '-0' may occur here
		if ((CEBasic::isZero(GetAt(0))) && (!getFlag())) { setFlag(); return true; }
	}

	//Note: sum is not empty here
	bool do_inverse_sign=false; long i;

	//1. First item in sum should be +-signed (to uniform 1/(a-b) and 1/(-a+b) items)
	if(!GetAt(0)->getFlag()){ do_inverse_sign=true; }

	//2. (useless after modern #1) If all the sub-items have "-" ==> move it to all the sum.
	//Old (case #2, useless): do_inverse_sign=true; for(i=0;i<s;i++){ if(GetAt(i)->getFlag()){do_inverse_sign=false;break;} }

	//3. [Very old], now in conflict to #1-2
	//If the sum has sign "-" and any of the sub-items also have "-" ==> apply the whole "-" to all the sub-items.
	//Very old: if(!getFlag()){ do_inverse_sign=false; for(i=0;i<s;i++){ if (!GetAt(i)->getFlag()){do_inverse_sign=true;break;} } }

	//Inverse sign
	if(do_inverse_sign){ for(i=0; i<s; i++){ GetAt(i)->inverseFlag(); } inverseFlag(); return true; }
	return false;
}

CExpression* CESum::Re()
{
	CESum* res = this;
	long i;
	for (i=0; i<res->size(); i++) {
		CExpression* item = res->GetAt(i);
		CExpression* item_re = item->Re();
		if (item_re != item) {
			if (res == this) res = (CESum*)clone();
			if (item_re != NULL) {
				res->SetAtX(i, item_re);
			}
			else {
				res->RemoveAt(i);
				i--;
			}
		}
	}
	return res;
}

CExpression* CESum::Im()
{
	CESum* res = this;
	long i;
	for (i=0; i<res->size(); i++) {
		CExpression* item = res->GetAt(i);
		CExpression* item_im = item->Im();
		if (item_im != item) {
			if (res == this) res = (CESum*)clone();
			if (item_im != NULL) {
				res->SetAtX(i, item_im);
			}
			else {
				res->RemoveAt(i);
				i--;
			}
		}
	}
	return res;
}

CESerializable* CESum::restore_from_buffer(const char *data, long pos, CEMap *params, DWORD settings)
{
	CESum* res = new CESum();
	bool loop = true;
	for(;loop;) {
		long skip_pos = CESupport::skip_item_in_buffer(data, pos, settings);
		if (skip_pos <= 0) {
			loop = false;
		}
		else {
			CESerializable* item = CESupport::read_buffer(data, pos, settings);
			pos = skip_pos;
			if (item != NULL) {
				res->addItem((CExpression*)item);
			}
			else {
				//Error
				loop = false;
				delete res;
				return NULL;
			}
		}
	}
	return res;
}

const char* CESum::get_TAG_Name()
{
	return CLASS_TAG_NAME;
}

long CESum::store_stream(ofstream *p_ofs, DWORD settings)
{
	long res = 0;
	long s = size();
	long i;
	for (i=0; i<s; i++) {
		CExpression* item = GetAt(i);
		long item_len = CESupport::store_stream(item, p_ofs, settings);
		res += item_len;
		if ((item_len > 40) && (i < (s-1))) {//40 should be defined constant
			res += CESupport::storeLineBreak(p_ofs);
		}
	}
	return res;
}
