////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ESymbol.h: interface for the CESymbol class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ESYMBOL_H__E2DE8DC1_0BEA_11D5_8778_0010A7027CA2__INCLUDED_)
#define AFX_ESYMBOL_H__E2DE8DC1_0BEA_11D5_8778_0010A7027CA2__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CENull : public CExpression  
{
public:
	CENull();
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	static bool IsNull(CExpression* src); // Note: returns FALSE for NULL!
	virtual CExpression* clone() const;
	virtual bool equals(CExpression *other);
	virtual CEString* toString(int mode = 0);

	static const char* CLASS_TAG_NAME;
	virtual long store_stream(ofstream* p_ofs, DWORD settings);
	virtual const char* get_TAG_Name();
	static CESerializable* restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings);
};

class CEInfinity : public CExpression  
{
public:
	CEInfinity();
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual CExpression* clone() const;
	virtual bool equals(CExpression *other);
	virtual CEString* toString(int mode = 0);

	static const char* CLASS_TAG_NAME;
	virtual long store_stream(ofstream* p_ofs, DWORD settings);
	virtual const char* get_TAG_Name();
	static CESerializable* restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings);
};

//##ModelId=3E3FF2D10119
class CESymbol : public CExpression  
{
private:
	//##ModelId=3E3FF2D1012A
	static long unique_num;
public:
	static const char* CLASS_NAME;
	static const char* CLASS_TAG_NAME;
	virtual void* getStorageFlags();
	virtual long store_stream(ofstream* p_ofs, DWORD settings);
	virtual const char* get_TAG_Name();
	static CESerializable* restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings);

	static CESymbol* getUnique();
	static CESymbol* getUnique(bool commutative);
	static bool isSymbolUnique(CESymbol* sym);

	// char bank definitions
	static bool m_UseCharMap;
	static bool m_NoCharMapForUniques;
	bool isCharFromMap();
private:
	bool m_CharFromMap;
	CESymbol(bool is_char_from_bank, const char* name, unsigned len, bool _is_commutative, bool _bSelfConjugated);

public:
	CESymbol(const char* name);
	CESymbol(const char* name, bool _is_commutative, bool _bSelfConjugated=true);
	virtual ~CESymbol();

	const char* get_name();
	long get_name_length();
	bool isSymbolCommutative();
	bool isSymbolSelfConjugated();

	virtual void collectProperties(CEProperties* props, unsigned set = CEProperties::PROP_MASK_ALL);
	virtual unsigned long props_long();
	virtual bool less_than(CExpression* other);
	virtual CExpression* numeric();
	virtual bool contain_vector(void * obj);
	virtual CExpression* conjugated(int mode = 0);
	virtual CExpression* clone() const;
	virtual bool equals(CExpression *other);
	virtual const char* ClassName() const;
	virtual CExpression* applyRule(CEAbstractRule* rule);
	virtual CExpression* evaluate(int level = 0);
	virtual CExpression* Simplify(int level = 0);
	virtual CExpression* Arrange(int level);
	virtual CEString* toString(int mode = 0);
	virtual void toStream(ostream* os, int mode = 0);

protected:
	char* m_name;
	unsigned m_len;
	
	bool m_is_commutative;
	bool m_bSelfConjugated;
};

#endif // !defined(AFX_ESYMBOL_H__E2DE8DC1_0BEA_11D5_8778_0010A7027CA2__INCLUDED_)
