////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// Expression.cpp: implementation of the CExpression class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

char* CExpression::m_LT_temp_1				= NULL;	// for less_than function
char* CExpression::m_LT_temp_2				= NULL;	// for less_than function
const DWORD CExpression::m_LT_temp_LENTH	= 1000; // sizeof m_LT_temp_1

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CExpression::CExpression() : CESerializable()
{
	//props = NULL;
	clear_all_attributes();
}

CExpression::~CExpression()
{

}

CExpression* CExpression::Simplify(int level)
{
	return NULL;
}

CExpression* CExpression::Arrange(int level)
{
	return this;
}

CEProperties* CExpression::getProperties(unsigned set)
{
	CEProperties* p = new CEProperties();
	//Default values;
	p->setSingleItem();
	p->setCommutative();
	//Call overloaded redefinitions
	collectProperties(p, set);
	return p;
}

void CExpression::collectProperties(CEProperties* props, unsigned set) {
	CEBasic::dbg->log(2, "Wanrnig: old PROPERTIES implementation is used", this);
	props->setProperty(CEProperties::PROP_MASK_SINGLE_ITEM, true);
	props->setProperty(CEProperties::PROP_MASK_COMMUTATIVE, true);
	props->setProperty(CEProperties::PROP_MASK_SELF_CONJUGATED, true);
}

/*
*	Question: what is better?
*	Overwriting this function in any child ==> slow work,
*	but some checking for all the items are processed by rule.
*/
CExpression* CExpression::applyRule(CEAbstractRule* rule)
{
	//printf("<Warning><applyRule() does not implemented at object:>");
	//printf(ClassName());
	return this;
}

CExpression* CExpression::evaluate(int level)
{
	return this;
}

/*
* Some expressions have no defined string repesentation.
*/
CEString* CExpression::toString(int mode)
{
	CEString* res = new CEString("<Abstract expression>");
	return res;
}

void CExpression::toStream(ostream* os, int mode)
{
	return;
}

unsigned CExpression::getFlags() const { return flags; }
bool CExpression::getFlag(unsigned flag) const { return ((flags & flag) != 0); }
bool CExpression::setFlag(unsigned flag) { return putFlag(true, flag); }

bool CExpression::putFlag(bool val, unsigned flag)
{
	//printf("<put: %d, flag: %d (falgs = %d)", val, flag, flags);
	bool res = getFlag(flag);
	//if (val) flags |= flag;
	//else flags &= (!flag);// WRONG: "!" is logical operation (not a bitwise)
	flags = flags | flag;
	if (!val) flags = flags ^ flag;
	//printf("==>(%d)>", flags);
	return res;
}

bool CExpression::unsetFlag(unsigned flag)
{
	return putFlag(false, flag);
}

bool CExpression::inverseFlag(unsigned flag) {
	//return putFlag(!getFlag(flag), flag);
	bool old = getFlag(flag);
	flags = flags ^ flag;
	return old;
}

bool CExpression::setLevel(bool level) { return putFlag(level, LEVEL_FLAG); }
bool CExpression::getLevel() const { return getFlag(LEVEL_FLAG); }
bool CExpression::inverseLevel() { return inverseFlag(LEVEL_FLAG); }

/*
*	IsSimpleItem: Used to define if brackets are needed
*	Question exactly: "Are visible brackets needed to insert this item (UNSIGNED) into product?"
*	Note: same function of props->(CEProperties::PROP_MASK_SINGLE_ITEM). TODO: iniform it.
*/
bool CExpression::IsSimpleItem(CExpression* pE) {
	//Old: return ((pE->getFlag()) && (!pE->getProperty(CEProperties::PROP_MASK_IS_SUM)));

	if(!pE->getFlag()) return false;
	unsigned long props = pE->props_long();
	if((props & (CEProperties::PROP_MASK_IS_SUM | CEProperties::PROP_MASK_IS_PRODUCT)) != 0) return false;
	if((props & CEProperties::PROP_MASK_SINGLE_ITEM) == 0) return false; //TODO: Is it the only required check?
	//Note: CENumber-specific checks are moved into CENumber's 'PROP_MASK_SINGLE_ITEM' property
	return true;
}
/*
*	StoreInBrackets: Adds '('..')' for complicated expressions
*	Used by toString() implementations.
*	Uses CExpression::IsSimpleItem() function.
*	See more on brackets politics in InternalNotes[1.1].
*/
void CExpression::StoreInBrackets(CEString* str, CExpression* pE, int mode) {
	if(IsSimpleItem(pE)){
		str->AddString(pE->toString(mode));
	}else{
		switch(mode) {
		case STORE_WR_MATHEMATICA:
			str->AddString("\\((");str->AddString(pE->toString(mode));str->AddString(")\\)"); break;
		default:
			str->AddString("(");str->AddString(pE->toString(mode));str->AddString(")");
		}
	}
}

bool CExpression::equals(CExpression* other) {
	//return (other == this);
	if (other == NULL) return false;

	if (CEProcessor::m_ExamEquals) {
		CEData* data = CEProcessor::getDataStorage(); const char* cn = ClassName();
		long x=0; CEObject* x_obj = data->get(cn); if(x_obj != NULL) x=((CENumber*)x_obj)->getLong();
		if (x==LONG_MAX) { printf("********************* equals() counter overflow for %s ******", cn); x=0; }
		data->put((char*)cn,CENumber::GetNumber(x+1));
	}

	if (other == this) return true;
	bool res = false;
	if ( (CEUtil::equals(ClassName(), other->ClassName())) && (attributes_equals_to(other)) ) {
		CEString* s1 = toString();
		CEString* s2 = other->toString();
		res = s1->equals(s2);
		delete s1;
		delete s2;
	}
	return res;
}

bool CExpression::obj_equals(CEObject* other) {
	if (other == NULL) return false;
	if (other == this) return true;
	if (other->GetObjectKind() & CEObject::KIND_EXPRESSION) { return equals((CExpression*)other); }
	return false;
}

void* CExpression::GetInterface(long interface_ID) { return NULL; }

CExpression* CExpression::expand(bool recursive)
{
	return this;
}
//CExpression* CExpression::clone() const { printf("<Warning><Non-valid cloning performed.>"); return this; }

CEObject* CExpression::copy() const { return clone(); }

/*
* Presumed to be used for commutative items.
* Otherwise we should possible use a different product class.
*/
CExpression* CExpression::multiply(CExpression *other)
{
	if (other == NULL) return this;
	CEProduct* prod = new CEProduct();
	prod->addItem(this);
	prod->addItem(other);
	return prod;
}

CExpression* CExpression::add(CExpression *other)
{
	if (other == NULL) return this;
	//CExpression* res = NULL;
	CESum* sum = new CESum();
	sum->addItem(this);
	sum->addItem(other);
	return sum;//sum->expand();
}

void CExpression::copyAttributesTo(CExpression *_other, bool do_xor) const
{
	if (do_xor) {
		bool s = getFlag();
		if (!_other->getFlag()) s = !s;
		bool l = getLevel();
		if (!_other->getLevel()) l = !l;
		_other->setAllAttributes(flags);
		_other->putFlag(s);
		_other->setLevel(l);
	}
	else _other->setAllAttributes(flags);
}

void CExpression::setAllAttributes(unsigned int _flags)
{
	flags = _flags;
}

CExpression* CExpression::conjugated(int mode) {
	CExpression* res=NULL;
	//res=clone();
	if (getProperty(CEProperties::PROP_MASK_SELF_CONJUGATED)) {
		res=clone();
	} else {
		res=new CFuncConjugated(this);
	}
	return res;
}

CExpression* CExpression::arrangeTensors()
{
	return this;
}

bool CExpression::less_than(CExpression *other)
{
	if (other == NULL) { CEBasic::dbg->log(0, "<Warning><CExpression::less_than() : Invaid data.>"); return false; }

	if (CEProcessor::m_ExamLessThen) {
		CEData* data = CEProcessor::getDataStorage(); const char* cn = ClassName();
		long x=0; CEObject* x_obj = data->get(cn); if(x_obj != NULL) x=((CENumber*)x_obj)->getLong();
		if (x==LONG_MAX) { printf("********************* less_than() counter overflow for %s ******", cn); x=0; }
		data->put((char*)cn,CENumber::GetNumber(x+1));
	}

	// the following objects are 'righter' than abstract object
	// uncomment after the proper scheme of less_than will be implemented
	// if (other->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) return true;

	///
	bool res;
	if(ClassName() == other->ClassName()){
		char* a=toLine(); char* b=other->toLine();
		res=CEUtil::a_less_b(a,b);
		CMemoryManager::mfree(a); CMemoryManager::mfree(b);
	}else{
		res=(CEUtil::compare_ab(ClassName(),other->ClassName()) > 0);
	}
	return res;

	// idia: print objects to previously allocated buffers, add len=toString(buffer, pos, max) function to every object
	// problem: objects greather than MAX_LEN will be incomparable
	//if (m_LT_temp_1 == NULL) m_LT_temp_1 = CMemoryManager::mmalloc(m_LT_temp_LENTH);
	//if (m_LT_temp_2 == NULL) m_LT_temp_2 = CMemoryManager::mmalloc(m_LT_temp_LENTH);
}

long CExpression::env()
{
	return CEProcessor::getEnvironment()->get_env();
}

bool CExpression::getProperty(unsigned int prop_mask)
{
	/*CEProperties* props = getProperties();
	bool res = props->getProperty(prop_mask);
	delete props;
	return res;*/

	unsigned long props = props_long();
	return ((props & prop_mask) != 0);
}

CExpression* CExpression::reduce(CExpression *old, bool do_delete_old)
{
	CExpression* res = NULL;
	//CEBasic::dbg->log(95,"CExpression::reduce",old);
	//int dbg_lev=CEBasic::dbg->set_level(5);
	//int dbg_lev=CEBasic::dbg->get_level(); if (dbg_lev>95) { CEProduct::dbg->set_level(dbg_lev); CESum::dbg->set_level(dbg_lev); CEBasic::dbg->set_level(5); }

#if defined(ALTA_DEBUG)
	try { // e.g. for 0/0 division
#endif //ALTA_DEBUG

	// reduction operations begin
	res = old->expand();
	if ((res != old) && (do_delete_old)) { delete old; }

	//CEBasic::dbg->set_level(dbg_lev); CEBasic::dbg->log(95,"CExpression::reduce expanded",res); dbg_lev=CEBasic::dbg->set_level(5);
	//if (dbg_lev>95) { CEProduct::dbg->set_level(5); CESum::dbg->set_level(5); CEBasic::dbg->set_level(dbg_lev); CEBasic::dbg->log(95,"CExpression::reduce expanded",res); CEBasic::dbg->set_level(5); }

	CExpression* res_expand = res; // remember pointer
	res = res_expand->arrangeTensors();
	if (res != res_expand) {
		if ((res_expand != old) || (do_delete_old)) delete res_expand;
	}

	//CEBasic::dbg->set_level(dbg_lev); CEBasic::dbg->log(95,"CExpression::reduce tensors arranged",res); dbg_lev=CEBasic::dbg->set_level(5);
	
	bool arr_delete = (do_delete_old || (res != old));
	res = arrge(res, arr_delete);
	// reduction operations end

#if defined(ALTA_DEBUG)
	} catch(CEException* exc) { // catch 0/0 division throw
		CEBasic::dbg->log(0, "CExpression::reduce() <HALT>, res:", res);
		if (!do_delete_old) CEBasic::dbg->log(0, "CExpression::reduce() source", old);
		throw exc;
	}
#endif //ALTA_DEBUG
	//CEBasic::dbg->set_level(dbg_lev); CEBasic::dbg->log(95,"CExpression::reduce res",res);
	return res;
}

CExpression* CExpression::arrge(CExpression* old, bool do_delete_old)
{
	CExpression* res = NULL;

#if defined(ALTA_DEBUG)
	try { // e.g. for 0/0 division
#endif //ALTA_DEBUG
		
		res = old->Arrange(AL_EXTRACT);

#if defined(ALTA_DEBUG)
	} catch(CEException* exc) { // catch 0/0 division throw
		CEBasic::dbg->log(0, "CExpression::arrge() <HALT>, source:", old);
		throw exc;
	}
#endif //ALTA_DEBUG

	if ((res != old) && (do_delete_old)) delete old;
	return res;
}

bool CExpression::contain_vector(void * obj)
{
	return false;
}

CExpression* CExpression::plus(CExpression * other, bool do_clone)
{
	if (other == NULL) return ((do_clone)?clone():this);
	CExpression* a;
	CExpression* b;
	if (do_clone) {
		a = clone();
		b = other->clone();
	}
	else {
		a = this;
		b = other;
	}
	return a->add(b);
}

CExpression* CExpression::minus(CExpression * other, bool do_clone)
{
	if (other == NULL) return ((do_clone)?clone():this);
	CExpression* a;
	CExpression* b;
	if (do_clone) {
		a = clone(); b = other->clone();
	}
	else {
		a = this; b = other;
	}
	b->inverseFlag();
	return a->add(b);
}

CExpression* CExpression::times(CExpression * a, CExpression * b)
{
	if ((a == NULL) || (b == NULL)) return NULL;

	bool sign = (a->getFlag() == b->getFlag());
	a->setFlag();
	b->setFlag();

	CExpression* res = NULL;
	if (b->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
		if ((a->getLevel()) && (b->getLevel())) {
			((CEProduct*)b)->InsertAtX(0, a);
			res = b;
		}
	}

	if (res == NULL) res = a->multiply(b);

	if (!sign) res->unsetFlag();
	return res;
}

CExpression* CExpression::numeric()
{
	return NULL;
}

void CExpression::clear_all_attributes()
{
	setAllAttributes(SIGN_FLAG | LEVEL_FLAG); // "+" sign and "+" level are defaults
}

CExpression* CExpression::divide(CExpression * other, bool do_clone)
{
	if (other == NULL) return ((do_clone)?clone():this);
	CExpression* a;
	CExpression* b;
	if (do_clone) {
		a = clone();
		b = other->clone();
	}
	else {
		a = this;
		b = other;
	}
	b->inverseLevel();
	return a->multiply(b);
}

unsigned long CExpression::props_long() {
	//Default values
	unsigned long res = CEProperties::PROP_MASK_SINGLE_ITEM | CEProperties::PROP_MASK_COMMUTATIVE | CEProperties::PROP_MASK_SELF_CONJUGATED;
	//Same: unsigned long res = CEProperties::set_property_to(0, CEProperties::PROP_MASK_SINGLE_ITEM, true);
	//Same: res = CEProperties::set_property_to(res, CEProperties::PROP_MASK_COMMUTATIVE, true);
	//Same: res = CEProperties::set_property_to(res, CEProperties::PROP_MASK_SELF_CONJUGATED, true);
	return res;
}

bool CExpression::attributes_equals_to(CExpression *other)
{
	return ((other->getFlag() == getFlag()) &&
		(other->getLevel() == getLevel()));
}

CExpression* CExpression::sqra()
{
	return new CEPower(this, (long)2);
}

/*
*	A =?= -B
*/
bool CExpression::is_opposite_to(CExpression *other)
{
	inverseFlag();
	bool res = equals(other);
	inverseFlag();
	return res;
}

CExpression* CExpression::Re()
{
	return this;
}

CExpression* CExpression::Im()
{
	return NULL;
}

bool CExpression::equal_or_opposite(CExpression *other)
{
	if (other == NULL) return false;

	if (equals(other)) return true;

	bool res = false;
	other->inverseFlag(); //	            !!!! May fail in multi-thread calculation !
	if (equals(other)) res = true;
	other->inverseFlag();
	return res;
}

int CExpression::GetObjectKind() { return CEObject::KIND_EXPRESSION; }
const char* CExpression::ClassName() const { return "Unknown EXPRESSION"; }

/*
*	Flags used: "f"(here), "c","j"(CESymbol)
*/
void* CExpression::getStorageFlags() { // returns CEMap* object or NULL
	void* p_sup = CESerializable::getStorageFlags();

	CEMap* res = NULL;
	if (p_sup != NULL) res = (CEMap*)p_sup;

	bool Nsign = !getFlag();
	bool Nlev = !getLevel();

	if (Nsign || Nlev) {
		if (res == NULL) res = CEMap::createNewMap();

		char* f_val = NULL;
		if (Nsign) {
			if (Nlev) {
				f_val = "sl";
			} else {
				f_val = "s";
			}
		} else if (Nlev) {
			f_val = "l";
		}
		
		if (f_val == NULL) { CEException::halt("CExpression storage INTERNAL ERROR"); }
		res->put_ptr("f", f_val);
	}

	return res;
}
