////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// Expression.h: interface for the CExpression class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EXPRESSION_H__9DB7A102_0AA0_11D5_8A94_932004EF1C0E__INCLUDED_)
#define AFX_EXPRESSION_H__9DB7A102_0AA0_11D5_8A94_932004EF1C0E__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define SIGN_FLAG				0x01
#define LEVEL_FLAG				0x02

#define NUMERATOR				true
#define DENOMINATOR				false

#define AL_EXTRACT				0
#define AL_INLINE				1

//##ModelId=3E3FF2DF03C8
class CExpression : public CESerializable  
{
private:
	//CEProperties* props;
	//##ModelId=3E3FF2DF03CC
	unsigned flags;

public:
	// idia for less_than(): print objects to previously allocated buffers, add len=toString(buffer, pos, max) function to every object
	// problem: objects greather than MAX_LEN will be incomparable
	static char* m_LT_temp_1; // for less_than function, deleted in CEProcessor::finalize()
	static char* m_LT_temp_2; // for less_than function, deleted in CEProcessor::finalize()
	static const DWORD m_LT_temp_LENTH; // sizeof m_LT_temp_1

public:
	virtual void* getStorageFlags(); // returns CEMap* object or NULL

public:
	virtual const char* ClassName() const;
	virtual int GetObjectKind();
	bool equal_or_opposite(CExpression* other);
	virtual CExpression* Im();
	virtual CExpression* Re();
	//##ModelId=3E3FF2E0005E
	bool is_opposite_to(CExpression* other);
	//##ModelId=3E3FF2E0005D
	CExpression* sqra();
	//##ModelId=3E3FF2E00059
	bool attributes_equals_to(CExpression* other);
	//##ModelId=3E3FF2E00058
	virtual unsigned long props_long();
	//##ModelId=3E3FF2E00055
	CExpression* divide(CExpression* other, bool do_clone = false);
	//##ModelId=3E3FF2E00054
	virtual CExpression* numeric();
	//##ModelId=3E3FF2E00051
	static CExpression* times(CExpression* a, CExpression* b);
	//##ModelId=3E3FF2E0004E
	CExpression* plus(CExpression* other, bool do_clone = false);
	//##ModelId=3E3FF2E00048
	CExpression* minus(CExpression* other, bool do_clone = false);
	//##ModelId=3E3FF2E00046
	virtual bool contain_vector(void* obj);
	//##ModelId=3E3FF2E00043
	static CExpression* reduce(CExpression* old, bool do_delete_old = true);
	//##ModelId=3E3FF2E00040
	static CExpression* arrge(CExpression* old, bool do_delete_old = true);
	//##ModelId=3E3FF2E0003E
	bool getProperty(unsigned prop_mask);
	//##ModelId=3E3FF2E00037
	static long env();
	virtual CExpression* arrangeTensors();

	/*
	*	Please, do not call conjugated() directly!
	*	It will be changed soon.
	*				Upd[Feb.2011] --> How do you want to change it???
	*	Use CEBasic::Congugate() instead.
	*/
	virtual CExpression* conjugated(int mode = 0);
	//##ModelId=3E3FF2E00030
	virtual void setAllAttributes(unsigned _flags);
	//##ModelId=3E3FF2E0002E
	virtual CExpression* add(CExpression* other);
	//##ModelId=3E3FF2E00028
	virtual CExpression* multiply(CExpression* other);

	//##ModelId=3E3FF2E00027
	virtual CEObject* copy() const;
	//##ModelId=3E3FF2E00026
	virtual CExpression* clone() const = 0;

	//##ModelId=3E3FF2E00024
	virtual CExpression* expand(bool recursive = true);
	
	//##ModelId=3E3FF2E00022
	virtual CExpression* applyRule(CEAbstractRule* rule);
	//##ModelId=3E3FF2E00020
	virtual CExpression* evaluate(int level = 0);
	//##ModelId=3E3FF2E0001B
	CEProperties* getProperties(unsigned set = CEProperties::PROP_MASK_ALL);
	//##ModelId=3E3FF2E0001A
	unsigned getFlags() const;
	//##ModelId=3E3FF2E00018
	bool getFlag(unsigned flag = SIGN_FLAG) const;
	//##ModelId=3E3FF2E00016
	bool setFlag(unsigned flag = SIGN_FLAG);
	//##ModelId=3E3FF2E00013
	bool putFlag(bool val, unsigned flag = SIGN_FLAG);
	//##ModelId=3E3FF2E00011
	bool unsetFlag(unsigned flag = SIGN_FLAG);
	//##ModelId=3E3FF2E0000F
	bool inverseFlag(unsigned flag = SIGN_FLAG);

	//##ModelId=3E3FF2E0000A
	bool setLevel(bool level);
	//##ModelId=3E3FF2E00009
	bool getLevel() const;
	//##ModelId=3E3FF2E00008
	bool inverseLevel();

	//##ModelId=3E3FF2E00006
	virtual CExpression* Simplify(int level = 0);
	//##ModelId=3E3FF2E00004
	virtual CExpression* Arrange(int level);
	//##ModelId=3E3FF2E00002
	virtual CEString* toString(int mode = 0);
	//##ModelId=3E3FF2DF03E3
	virtual void toStream(ostream* os, int mode = 0);

	//##ModelId=3E3FF2DF03E1
	virtual bool equals(CExpression* other);
	virtual bool less_than(CExpression* other); // used for sorting objects
	virtual bool obj_equals(CEObject* other);

	virtual void* GetInterface(long interface_ID);

	//##ModelId=3E3FF2DF03E0
	CExpression();
	//##ModelId=3E3FF2DF03DF
	virtual ~CExpression();

	//##ModelId=3E3FF2DF03DE
	void clear_all_attributes();
	//##ModelId=3E3FF2DF03DB
	void copyAttributesTo(CExpression* _other, bool do_xor = false) const;
protected:
	/*
	*	IsSimpleItem: Used to define if brackets are needed
	*	Note: same function of props->SINGLE_ITEM. TODO: iniform it.
	*/
	static bool IsSimpleItem(CExpression* pE);
	/*
	*	StoreInBrackets: Adds '('..')' for complicated expressions
	*	Used by toString() implementations.
	*	Uses CExpression::IsSimpleItem() function
	*	See more on brackets politics in InternalNotes[1.1].
	*/
	static void StoreInBrackets(CEString* str, CExpression* pE, int mode);
protected:
	virtual void collectProperties(CEProperties* props, unsigned set = CEProperties::PROP_MASK_ALL);
};

#endif // !defined(AFX_EXPRESSION_H__9DB7A102_0AA0_11D5_8A94_932004EF1C0E__INCLUDED_)
