////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// RCollectSymbols.cpp: implementation of the CRCollectSymbols class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

const int CRCollectSymbols::COLLECT_UNIQUES				= 0x01;
const int CRCollectSymbols::COLLECT_TENSOR_INDICES_ONLY	= 0x02;
const int CRCollectSymbols::COLLECT_DENOMINATORS		= 0x04;
const int CRCollectSymbols::COUNT_OBJECTS				= 0x08;

const char* CRCollectSymbols::CLASS_NAME = "CRCollectSymbols";

//////////////////////////////////////////////////////////////////////
// Static entry
//////////////////////////////////////////////////////////////////////
long CRCollectSymbols::CountObjects(CExpression* src) { // source is unaffected
	if(src==NULL)return 0;
	CRCollectSymbols* r = new CRCollectSymbols(CRCollectSymbols::COUNT_OBJECTS);
	r->ApplyTo(src,false);
	long res = r->m_Counter;
	delete r;
	return res;
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CRCollectSymbols::CRCollectSymbols(int _settings, const CEPtrArray* _ignore_list, const char* _ClassName) : CERule()
{
	m_ignore_list = CEPtrArray::get_copy(_ignore_list, false);
	m_result_list = new CEPtrArray();
	m_Counter=0;
	//m_sClassName=NULL; if(_ClassName!=NULL)m_sClassName=CEUtil::copyString(_ClassName);
	m_sClassName=_ClassName;

	m_settings = _settings;
}

// used for rule cloning
CRCollectSymbols::CRCollectSymbols(int _settings, const CEPtrArray* result_list, const CEPtrArray* _ignore_list, long _Counter, const char* _ClassName)
: CERule()
{
	m_ignore_list = CEPtrArray::get_copy(_ignore_list, false);
	m_result_list = CEPtrArray::get_copy(result_list, true);
	m_Counter=_Counter;
	m_settings = _settings;
	//m_sClassName=NULL; if(_ClassName!=NULL)m_sClassName=CEUtil::copyString(_ClassName);
	m_sClassName=_ClassName;
}

CRCollectSymbols::~CRCollectSymbols() {
	if (m_ignore_list != NULL) delete m_ignore_list;
	if (m_result_list != NULL) delete m_result_list;
	//if (m_sClassName != NULL) CMemoryManager::mfree(m_sClassName);
}

CERule* CRCollectSymbols::GetRuleCopy() const {
	return new CRCollectSymbols(m_settings, m_result_list, m_ignore_list, m_Counter, m_sClassName);
}

CEPtrArray* CRCollectSymbols::getCollectedSymbols(bool extract_res) {
	CEPtrArray* res = m_result_list;
	if (extract_res) m_result_list=NULL;
	return res;
}

const char* CRCollectSymbols::ClassName() const { return CLASS_NAME; }
CEString* CRCollectSymbols::toString(int mode) {
	CEString* res  = new CEString("\\({ ");
	res->add_int((m_ignore_list != NULL)?(m_ignore_list->size()):0);
	res->AddString(" ignores, ");
	res->add_int((m_result_list!=NULL)?(m_result_list->size()):0);
	res->AddString(" symbols found");
	res->AddString(" }\\)");
	return res;
}

bool CRCollectSymbols::contain_vector(void * obj) {
	if ((m_ignore_list == obj) || (m_result_list == obj)) return true;
	if (m_ignore_list != NULL) { if (m_ignore_list->contain_vector(obj)) return true; }
	if (m_result_list != NULL) { if (m_result_list->contain_vector(obj)) return true; }
	return false;
}

CExpression* CRCollectSymbols::ApplyTo(CExpression* src, bool del_src) {
	bool apply_to_subitems = false;
	bool _Applied=false; bool src_fits=false;

	if ((m_settings & COLLECT_DENOMINATORS) == COLLECT_DENOMINATORS) {
		if (!src->getLevel()) { src_fits=CheckSymbol(src); _Applied=true; } apply_to_subitems = true;
	}

	if ((m_settings & COLLECT_UNIQUES) == COLLECT_UNIQUES) {
		apply_to_subitems = true;
		if ((m_settings & COLLECT_TENSOR_INDICES_ONLY) == COLLECT_TENSOR_INDICES_ONLY) {
			if (src->getProperty(CEProperties::PROP_MASK_IS_TENSOR)) {
				IETensor* t = IETensor::GetIETensor(src);
				int r = t->rank();
				for (int i=1; i<=r; i++) { CheckSymbol(t->index(i)); }
				apply_to_subitems = false;
			}
		} else {
			if (!_Applied) { src_fits=CheckSymbol(src); _Applied=true; }
			if (src_fits) apply_to_subitems = false;
		}
	}

	if ((m_settings & COUNT_OBJECTS) == COUNT_OBJECTS) { m_Counter++; apply_to_subitems = true; }

	if ((m_sClassName != NULL) && (!_Applied)) {
		src_fits=CheckSymbol(src); _Applied=true; apply_to_subitems=true;
	}

	if (apply_to_subitems) {
		// Applying to sub-items.
		CExpression* sub_res = src->applyRule(this);
		if (sub_res != src) { dbg->log(0, "CRCollectSymbols: internal error."); throw(new CEException(CEException::INT_ERROR)); }
	}
	return src;
}

/*
*	returns true if the expression fits the rule
*/
bool CRCollectSymbols::CheckSymbol(CExpression* sym) {
	if (m_settings & COLLECT_UNIQUES) {
		if(sym->ClassName()!=CESymbol::CLASS_NAME) return false;
		if(!CESymbol::isSymbolUnique((CESymbol*)sym)) return false;
	}

	if (m_sClassName!=NULL) {
		if(sym->ClassName() != m_sClassName) return false;
	}

	// truncate flags
	bool to_be_deleted = false;
	if (m_settings & COLLECT_DENOMINATORS) { // truncate sign only
		if (!sym->getFlag()) {
			sym = sym->clone();
			if (!sym->getFlag()) sym->setFlag();
			to_be_deleted = true;
		}
	} else {
		if ((!sym->getFlag()) || (!sym->getLevel())) {
			sym = sym->clone();
			if (!sym->getFlag()) sym->setFlag();
			if (!sym->getLevel()) sym->setLevel(true);
			to_be_deleted = true;
		}
	}

	long i; long s;
	// search for it in ignores list
	if (m_ignore_list != NULL) {
		s = m_ignore_list->size();
		for(i=0; i<s; i++) {
			CExpression* item = (CExpression*)m_ignore_list->GetAt(i);
			if (item->equals(sym)) { if (to_be_deleted) delete sym; return false; }
		}
	}
	// search for "sym" in results list
	if (m_result_list==NULL) m_result_list=new CEPtrArray();
	s = m_result_list->size();
	for(i=0; i<s; i++) {
		CExpression* item = (CExpression*)m_result_list->GetAt(i);
		if (item->equals(sym)) { if (to_be_deleted) delete sym; return false; }
	}
	// add to result list
	if (!to_be_deleted) sym = sym->clone();
	m_result_list->add(sym);
	return true;
}
