////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// RContainCheck.cpp: implementation of the CRContainCheck class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

const char* CRContainCheck::CLASS_NAME = "CRContainCheck";
//////////////////////////////////////////////////////////////////////
// Static Entries
//////////////////////////////////////////////////////////////////////
bool CRContainCheck::search(CExpression* source, CExpression* target) {
	if ((source == NULL) && (target == NULL)) return true;
	if ((source == NULL) || (target == NULL)) return false;

	CRContainCheck* r = new CRContainCheck(target);
	r->ApplyTo(source, false); // does not change source no matter delete flag is set.
	bool res = r->getSearchResult();
	delete r;
	return res;
}

bool CRContainCheck::search(CExpression* source, unsigned long target_props) {
	if ((source == NULL) || (target_props == 0)) return false;
	
	CRContainCheck* r = new CRContainCheck(target_props);
	r->ApplyTo(source, false); // does not change source no matter delete flag is set.
	bool res = r->getSearchResult();
	delete r;
	return res;
}

bool CRContainCheck::search(CExpression* source, const char* target_ClassName) {
	if ((source == NULL) && (target_ClassName == NULL)) return true;
	if ((source == NULL) || (target_ClassName == NULL)) return false;

	CRContainCheck* r = new CRContainCheck(target_ClassName);
	r->ApplyTo(source, false); // does not change source no matter delete flag is set.
	bool res = r->getSearchResult();
	delete r;
	return res;
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRContainCheck::CRContainCheck(const CExpression* target) : CERule() {
	m_target_expr = (target != NULL)?(target->clone()):NULL;
	m_target_props = 0;
	m_search_res = false;
	m_target_ClassName = NULL;
}

CRContainCheck::CRContainCheck(unsigned long target_props) : CERule() {
	m_target_expr = NULL;
	m_target_props = target_props;
	m_search_res = false;
	m_target_ClassName = NULL;
}

CRContainCheck::CRContainCheck(const char* target_ClassName) : CERule() {
	m_target_expr = NULL;
	m_target_props = 0;
	m_search_res = false;
	m_target_ClassName = target_ClassName;
}

// used for rule cloning
CRContainCheck::CRContainCheck(const CExpression* target, unsigned long target_props, const char* target_ClassName, bool _search_res) : CERule() {
	m_target_expr = (target != NULL)?(target->clone()):NULL;
	m_target_props = target_props;
	m_search_res = _search_res;
	m_target_ClassName = target_ClassName;
}

CRContainCheck::~CRContainCheck() {
	if (m_target_expr != NULL) delete m_target_expr;
}

CERule* CRContainCheck::GetRuleCopy() const { return new CRContainCheck(m_target_expr, m_target_props, m_target_ClassName, m_search_res); }

/*
*	Compare various criteria with "AND" result.
*/
CExpression* CRContainCheck::ApplyTo(CExpression* src, bool del_src) {
	if (m_search_res) return src; // stop the search;

	if (src == NULL) {
		// search NULL in NULL results "true", otherwise "false"
		if ((m_target_props == 0) && (m_target_ClassName==NULL) && (m_target_expr==NULL)) m_search_res=true;
		return src;
	}

	bool cmp_res = false;
	bool need_cmp_expr = true;

	// compare class-name
	if ((m_target_ClassName!=NULL) && (need_cmp_expr)) {
		if (src->ClassName() == m_target_ClassName) { // fast check only!
			cmp_res=true;
		} else {
			need_cmp_expr=false;
		}
	}

	// compare props
	if ((m_target_props != 0) && (need_cmp_expr)) {
		unsigned long src_props = src->props_long();
		if ((src_props & m_target_props) == m_target_props) {
			cmp_res=true;
		} else {
			need_cmp_expr=false;
		}
	}

	// quick test
	if ((m_target_expr != NULL) && (need_cmp_expr)) {
		if (src == m_target_expr) {
			cmp_res=true; need_cmp_expr=false;
		}
	}

	// compare expressions
	if ((m_target_expr != NULL) && (need_cmp_expr)) { // compare sources
		if (src == m_target_expr) cmp_res=true;
		cmp_res = false; // forget result
		CExpression* test_val = src;
		bool del_test = false;
		// Note:
		//	1/s contains s, but s does not contain 1/s;
		//	(-s) contains s, but s does not contain (-s).
		if ((m_target_expr->getFlag()) && (!test_val->getFlag())) {
			if (!del_test) { test_val = test_val->clone(); del_test = true; }
			test_val->inverseFlag();
		}

		if ((m_target_expr->getLevel()) && (!test_val->getLevel())) {
			if (!del_test) { test_val = test_val->clone(); del_test = true; }
			test_val->inverseLevel();
		}

		if (test_val->equals(m_target_expr)) cmp_res = true;
		if (del_test) delete test_val;
	}

	if (cmp_res) m_search_res = true;
	
	if (!m_search_res) {
		// search in sub-items
		CExpression* sub_res = src->applyRule(this);
		if (sub_res != src) { dbg->log(0, "CRContainCheck: internal error."); throw(new CEException(CEException::INT_ERROR)); }
	}
	return src;
}

bool CRContainCheck::getSearchResult() { return m_search_res; }
const char* CRContainCheck::ClassName() const { return CLASS_NAME; }
CEString* CRContainCheck::toString(int mode) {
	CEString* res  = new CEString("\\({CRContainCheck: ");
	if (m_target_expr != NULL) res->AddString(m_target_expr->toString(mode));
	else res->AddString("NULL");
	if (m_target_props != 0) { res->AddString("{"); res->add_int(m_target_props); res->AddString("}"); }
	res->AddString("; found =");
	res->add_int(m_search_res);
	res->AddString(" }\\)");
	return res;
}

bool CRContainCheck::contain_vector(void * obj) {
	if (m_target_expr != NULL) {
		if (m_target_expr == obj) return true;
		if (m_target_expr->contain_vector(obj)) return true;
	}
	return false;
}
