////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// RReplaceDenominators.cpp: implementation of the CRReplaceDenominators class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

const int CRReplaceDenominators::REPL_DENS_DEFAULT	= 0x01;
const char* CRReplaceDenominators::CLASS_NAME = "CRReplaceDenominators";
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRReplaceDenominators::CRReplaceDenominators(int _settings, int _dens_flags) : CERule()
{
	m_settings = _settings;
	m_replaced_values = new CEPtrArray();
	m_dens_flags = _dens_flags;
}

// used for rule cloning or to continue the previous procedure
CRReplaceDenominators::CRReplaceDenominators(int _settings, const CEPtrArray* _replaced_values, int _dens_flags)
: CERule()
{
	m_settings = _settings;
	m_replaced_values = CEPtrArray::get_copy(_replaced_values, true);
	m_dens_flags = _dens_flags;
}

CRReplaceDenominators::~CRReplaceDenominators()
{
	delete m_replaced_values;
}

CERule* CRReplaceDenominators::GetRuleCopy() const {
	return new CRReplaceDenominators(m_settings, m_replaced_values, m_dens_flags);
}

void CRReplaceDenominators::SetDensFlags(int _dens_flags) { m_dens_flags = _dens_flags; }
CEPtrArray* CRReplaceDenominators::getReplacedValues() { return m_replaced_values; }
const char* CRReplaceDenominators::ClassName() const { return CLASS_NAME; }
CEString* CRReplaceDenominators::toString(int mode) {
	CEString* res  = new CEString("\\({ total ");
	res->add_int(m_replaced_values->size());
	res->AddString(" items replaced }\\)");
	return res;
}

bool CRReplaceDenominators::contain_vector(void * obj) {
	if (m_replaced_values != NULL) {
		if (m_replaced_values == obj) return true;
		if (m_replaced_values->contain_vector(obj)) return true;
	}
	return false;
}

CExpression* CRReplaceDenominators::ApplyTo(CExpression* src, bool del_src) {

	if (!src->getLevel()) {
		// truncate sign at denominator, do not affect source
		bool src_sign = src->getFlag();
		if (!src_sign) {
			if (!del_src) { src = src->clone(); del_src = true; }
			src->inverseFlag();
		}
		
		CExpression* d_subst = GetSubstSym(src);
		if (d_subst != NULL) {
			d_subst = d_subst->clone();
		} else {
			d_subst = InvolveNewSubstSym(src);
		}

		if (del_src) delete src;
		if (!src_sign) d_subst->inverseFlag();
		return d_subst;
	}

	CExpression* res = src->applyRule(this);
	if ((res != src) && (del_src)) delete src;
	return res;
}

/*
*	Searches denominator in previous data, NULL if not found.
*	Value is supposed to be unsigned denominator.
*/
CExpression* CRReplaceDenominators::GetSubstSym(CExpression* den_val) {

	int s = m_replaced_values->size();
	int i;
	for(i=0; i<s; i++) {
		CExprContainer* cart = (CExprContainer*)m_replaced_values->GetAt(i);
		CExpression* cart_val = cart->get_expr();
		if (cart_val == NULL) { dbg->log(0, "Warning: NULL expression in denominators cache", m_replaced_values); continue; }

		if ((cart_val->equals(den_val)) && (cart->get_info() == m_dens_flags)) {
			return cart->get_info_expr();
		}
	}

	return NULL;
}

/*
*	Creates new symbol.
*	Value is supposed to be unsigned denominator.
*/
CExpression* CRReplaceDenominators::InvolveNewSubstSym(CExpression* den_val) {
	
	CExpression* d_subst = CESymbol::getUnique();
	m_replaced_values->add(new CExprContainer(den_val, m_dens_flags, 0, d_subst));
	return d_subst;
}
