////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// Const.cpp: implementation of the CConst class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

const char* CConst::DN_NUM_VALUES		= "values";
const char* CConst::DN_VECTORS			= "vectors";
const char* CConst::DN_GREEK_LETTERS	= "";
const char* CConst::DNV_SQUARE			= "square";
const char* CConst::DNV_MODULE			= "module";
const char* CConst::DNV_SYMBOL			= "symbol";

const char CConst::DN_DOT	= '.';

const char* CConst::DV_ZERO				= "zero";
const char* CConst::DV_UNIT				= "unit";
const char* CConst::DV_TWO				= "two";
const char* CConst::DV_IMAGINARY		= "imaginary";
const char* CConst::DV_HALF				= "half";
const char* CConst::DV_GAMMA5			= "gamma5";
const char* CConst::DV_GAMMA_P			= "gamma_P";
const char* CConst::DV_GAMMA_M			= "gamma_M";

const char* CConst::GL_ALPHA			= "alpha";
const char* CConst::GL_BETA				= "beta";
const char* CConst::GL_GAMMA			= "gamma";
const char* CConst::GL_DELTA			= "delta";

const char* CConst::GL_CHI				= "chi";
const char* CConst::GL_ETA				= "eta";
const char* CConst::GL_LAMBDA			= "lambda";
const char* CConst::GL_OMEGA			= "omega";
const char* CConst::GL_PHI				= "phi";
const char* CConst::GL_PSI				= "psi";
const char* CConst::GL_PI				= "pi";

const char* CConst::DS_METRIC_BASE				= "metric_base";
const char* CConst::DS_ANTISYMMETRIC_BASE	= "antisymmetric_base";
const char* CConst::DS_SPACE							= "space";
const char* CConst::DS_GAMMA_SPECIAL_BASE	= "gamma_special_base";

#if defined(ALTA_DEBUG)
const char* CConst::EXPR_CORE_TITLE			= "ALHEP (debug)"; // "ALTA_HEP";
const char* CConst::EXPR_CORE_TITLE_SHORT	= "ALHEP (debug)"; // "ALTA_HEP";
#else
const char* CConst::EXPR_CORE_TITLE			= "ALHEP"; // "ALTA_HEP";
const char* CConst::EXPR_CORE_TITLE_SHORT	= "ALHEP"; // "ALTA_HEP";
#endif //ALTA_DEBUG

const char* CConst::EXPR_CORE_BUILD_DATE	= __DATE__;

const char* CConst::EXPR_LOGO_TEXT1		= "*******************************************************************************";
const char* CConst::EXPR_LOGO_TEXT2		= "***                                 ALHEP                                   ***";
const char* CConst::EXPR_LOGO_TEXT_EMPTY= "***                                                                         ***";
const char* CConst::EXPR_LOGO_TEXT3		= "***             Project website: http://cern.ch/~makarenko/alhep            ***";
const char* CConst::EXPR_LOGO_TEXT3b	= "***                 Mirror page: http://www.hep.by/alhep                    ***";
const char* CConst::EXPR_LOGO_TEXT4		= "***  Please send suggestions & bug reports to: Vladimir.Makarenko@cern.ch   ***";
const char* CConst::EXPR_LOGO_TEXT5		= "*******************************************************************************";

const unsigned CConst::EXPR_VERSION_ID		= 1;
const unsigned CConst::EXPR_VERSION_MAJOR	= 1;
const unsigned CConst::EXPR_VERSION_MINOR	= 4;
const unsigned CConst::EXPR_VERSION_BUILD	= 1; // unused
//Note: use 'CESupport::CreateVersionString()' to obtain version string

const char* CConst::EXPR_AUTHOR_CONTACT		= "makarenko@hep.by; cc: Vladimir.Makarenko@cern.ch";

const long CConst::dwIETensor	= 1;

CConst::CConst() { }
CConst::~CConst() { }

const char* CConst::GetValidSymMSubst(const char* s, int l) {
	if (s==NULL) return NULL;
	if (l<0) l = CEUtil::length(s);
	if (l==0) return NULL;
	if (*(s)=='\\') {
		//if (*(s+1)=='\\') { return GetValidSymMSubst(s+1); }
		return GetValidSymMSubst(s+1,l-1);
	}
	if ((*(s)=='[') && (*(s+l-1)==']')) {
		return GetValidSymMSubst(s+1,l-2);
	}

	const char* res=NULL;
	if ((CEUtil::compareSubStrings(s,0,l-1,"gamma")) || (CEUtil::compareSubStrings(s,0,l-1,"Gamma"))) { res="\\[Gamma]"; }
	else if ((CEUtil::compareSubStrings(s,0,l-1,"nu")) || (CEUtil::compareSubStrings(s,0,l-1,"Nu"))) { res="\\[Nu]"; }
	else if ((CEUtil::compareSubStrings(s,0,l-1,"mu")) || (CEUtil::compareSubStrings(s,0,l-1,"Mu"))) { res="\\[Mu]"; }
	else if ((CEUtil::compareSubStrings(s,0,l-1,"alpha")) || (CEUtil::compareSubStrings(s,0,l-1,"Alpha"))) { res="\\[Alpha]"; }
	return res;
}

const char* CConst::GetValidSymTexSubst(const char* s, int l) {
	if (s==NULL) return NULL;
	if (l<0) l = CEUtil::length(s);
	if (l==0) return NULL;
	if (*(s)=='\\') {
		//if (*(s+1)=='\\') { return GetValidSymMSubst(s+1); }
		return GetValidSymTexSubst(s+1,l-1);
	}
	if ((*(s)=='[') && (*(s+l-1)==']')) {
		return GetValidSymTexSubst(s+1,l-2);
	}

	const char* res=NULL;
	if ((CEUtil::compareSubStrings(s,0,l-1,"gamma")) || (CEUtil::compareSubStrings(s,0,l-1,"Gamma"))) { res="\\gamma"; }
	else if ((CEUtil::compareSubStrings(s,0,l-1,"nu")) || (CEUtil::compareSubStrings(s,0,l-1,"Nu"))) { res="\\nu"; }
	else if ((CEUtil::compareSubStrings(s,0,l-1,"mu")) || (CEUtil::compareSubStrings(s,0,l-1,"Mu"))) { res="\\mu"; }
	else if ((CEUtil::compareSubStrings(s,0,l-1,"alpha")) || (CEUtil::compareSubStrings(s,0,l-1,"Alpha"))) { res="\\alpha"; }
	return res;
}
