////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EData.cpp: implementation of the CEData class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

CDebug* CEData::dbg = new CDebug("CEData");

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CEData::CEData() : CEMap()
{
	m_data = CEDataNode::createDataNode();
}

CEData::~CEData()
{
	delete m_data;
}

long CEData::size() { return m_data->size(); }

void CEData::put(const char* id, CEObject* o) {
	//log(">put:");
	//log(id);
	//log(o);
	m_data->put(id, o);
}

CEObject* CEData::get(const char* id) {
	return m_data->get(id);
}

CExpression* CEData::gete(const char *id)
{
	return (CExpression*)get(id);
}

void CEData::pute(const char *id, CExpression *expr)
{
	put(id, expr);
}

char* CEData::toLine(int mode) { return m_data->toLine(mode); }
CEString* CEData::toString(int mode) { return new CEString(toLine(mode), true); }

CExpression* CEData::getv(const char *id, const char* param)
{
	char* _id = CEUtil::composeId(id, param);
	CExpression* res = gete(_id);
	CMemoryManager::mfree(_id);
	return res;
}

CExpression* CEData::getvsqr(const char *id)
{
	return getv(id, CConst::DNV_SQUARE);
}

CExpression* CEData::getvmod(const char *id)
{
	return getv(id, CConst::DNV_MODULE);
}

void CEData::putv(const char *id, const char *param, CExpression *expr)
{
	char* _id = CEUtil::composeId(id, param);
	pute(_id, expr);
	//CExpression* res = pute(_id, expr);
	CMemoryManager::mfree(_id);
	//return res;
}

bool CEData::contain_vector(void * obj)
{
	if ((m_data == obj) || (m_data->contain_vector(obj)))
		return true;
	return false;
}

CExpression* CEData::getvalue(CExpression * item)
{
	CEString* id_str = item->toString();
	CExpression* res = getv(CConst::DN_NUM_VALUES,
		id_str->getData());
	delete id_str;
	return res;
}

void CEData::putvalue(CExpression * item, CExpression * val, bool delete_item)
{
	pute_to_node(CConst::DN_NUM_VALUES, item, val, delete_item);
}

void CEData::store_node(const char *node_id)
{
	if (node_id != NULL) return;

	dbg->log(2, "Data storage contents ********************");
	dbg->log(2, this);
	/*m_data->reset_iterator();
	if (m_data->has_next()) {
		CEObject* item = m_data->next();
		dbg->log(10, item);
	}*/
	dbg->log(2, "Data storage contents ********** finished");
}

bool CEData::has_next()
{
	return m_data->has_next();
}

void CEData::reset_iterator()
{
	m_data->reset_iterator();
}

CEObject* CEData::next()
{
	return m_data->next();
}

void CEData::pute_to_node(const char *node, CExpression *item, CExpression *val, bool delete_item)
{
	CEString* id_str = item->toString();
	putv(node, id_str->getData(), val);
	delete id_str;
	if (delete_item) delete item;
}

CEMap* CEData::get_node(const char* node)
{
	if (node == NULL) return m_data;
	
	CEObject* o = m_data->get(node);
	//To do the safe type casting.
	CEMap* res = NULL;
	if (o != NULL) {
		try {res = (CEMap*)o;}
		catch(...) {printf("Error : map object expected.\n");}
	}
	return res;
}

/*
*	Not implemented
*/
void CEData::put_int(char* id, long _val, bool do_copy_id) {
	if (!do_copy_id) CMemoryManager::mfree(id);
}

/*
*	Not implemented
*/
long CEData::get_int(const char* id) {
	return -1;
}

void CEData::put_ptr(const char* id, void* _p) {}
void* CEData::get_ptr(const char* id) {return NULL;}

const char* CEData::CLASS_NAME = "CEData";
const char* CEData::ClassName() const { return CLASS_NAME; }
