////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EDataNodeImpl.cpp: implementation of the CEDataNodeImpl class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CEDataNodeImpl::CEDataNodeImpl() : CEDataNode()
{
	m_data = CEMap::createNewMap();
}

CEDataNodeImpl::~CEDataNodeImpl()
{
	delete m_data;
}

/*
* May be unified with put() using getFinalNode()
*/
CEObject* CEDataNodeImpl::get(const char *id)
{
	//log(">CEDataNodeImpl::get");
	//log(id);
	//printf("<");printf(id);printf(">");
	//log("m_data = ");
	//log(m_data);
	int proxy_pos = CEUtil::findSymbol(id, '.');
	if (proxy_pos<0) return m_data->get(id);
	
	CEObject* res = NULL;
	char* new_id = CEUtil::copyStringFrom(id, proxy_pos+1);
	if (proxy_pos == 0) {
		CEData::dbg->log(0, "<CEDataNodeImpl Warning><Invalid ID: Starting dot skipped>");
		CEObject* res = get(new_id);
	}
	else {
		char* node_id = CEUtil::copyStringTo(id, proxy_pos-1);
		//log(node_id);
		CEObject* o = m_data->get(node_id);
		//log("Node found = ");
		//log(o);
		CMemoryManager::mfree(node_id);
		//To do the safe type casting.
		if (o != NULL) {
			CEDataNode* sub_node = (CEDataNode*)o;
			res = sub_node->get(new_id);
			//log("From subnode:");
			//log(res);
		}
	}
	CMemoryManager::mfree(new_id);
	return res;
}

/*
* May be unified with get() using getFinalNode()
*/
void CEDataNodeImpl::put(const char *id, CEObject *o)  // object is deleted
{
	int proxy_pos = CEUtil::findSymbol(id, '.');
	if (proxy_pos<0) {
		m_data->put(id, o);
		return;
	}
	char* new_id = CEUtil::copyStringFrom(id, proxy_pos+1);
	if (proxy_pos == 0) {
		log("<Warning><Invalid ID: Starting dot skipped>");
		put(new_id, o);
	}
	else {
		char* node_id = CEUtil::copyStringTo(id, proxy_pos-1);
		CEObject* _sub = m_data->get(node_id);
		CEDataNode* sub_node = NULL;
		if (_sub == NULL) {
			sub_node = CEDataNode::createDataNode();
			m_data->put(node_id, sub_node);
			//log("<CEDataNodeImpl:><New node created.>");
		}
		else {
			sub_node = (CEDataNode*)_sub;
		}
		CMemoryManager::mfree(node_id);
		sub_node->put(new_id, o);
	}
	CMemoryManager::mfree(new_id);
}

/*
*	Not implemented
*/
void CEDataNodeImpl::put_int(char* id, long _val, bool do_copy_id) {
	CEData::dbg->log(0, "ERROR: CEDataNodeImpl::put_int() unimplemented call");
	if (!do_copy_id) CMemoryManager::mfree(id);
}

/*
*	Not implemented
*/
long CEDataNodeImpl::get_int(const char* id) {
	CEData::dbg->log(0, "ERROR: CEDataNodeImpl::get_int() unimplemented call");
	return -1;
}

void CEDataNodeImpl::put_ptr(const char* id, void* _p) { CEData::dbg->log(0, "ERROR: CEDataNodeImpl::put_ptr() unimplemented call"); }
void* CEDataNodeImpl::get_ptr(const char* id) { CEData::dbg->log(0, "ERROR: CEDataNodeImpl::get_ptr() unimplemented call"); return NULL;}

long CEDataNodeImpl::size() { return m_data->size(); }

/*
* Not used.
* Presumed to unify get & put mechanisms.
* But it requires additional searching for the last dot in id.
*/
CEDataNode* CEDataNodeImpl::getFinalNode(const char *id)
{
	int proxy_pos = CEUtil::findSymbol(id, '.');
	if (proxy_pos<0) return this;
	
	if (proxy_pos == 0) {
		log("<Warning><Invalid ID: Starting dot skipped>");
		return this;
	}
	
	CEDataNode* res = NULL;
	char* new_id = CEUtil::copyStringFrom(id, proxy_pos+1);
	char* node_id = CEUtil::copyStringTo(id, proxy_pos-1);
	CEObject* o = m_data->get(node_id);
	CMemoryManager::mfree(node_id);
	//To do the safe type casting.
	res = ((CEDataNode*)o)->getFinalNode(new_id);
	CMemoryManager::mfree(new_id);
	return res;
}

char* CEDataNodeImpl::toLine(int mode) { return m_data->toLine(mode); }
CEString* CEDataNodeImpl::toString(int mode) { return new CEString(toLine(mode), true); }

bool CEDataNodeImpl::contain_vector(void * obj)
{
	if ((m_data == obj) || (m_data->contain_vector(obj)))
		return true;
	return false;
}

CEObject* CEDataNodeImpl::next()
{
	return m_data->next();
}

void CEDataNodeImpl::reset_iterator()
{
	m_data->reset_iterator();
}

bool CEDataNodeImpl::has_next()
{
	return m_data->has_next();
}
