////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// FLogN.cpp: implementation of the CFLogN class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

CDebug* CFLogN::dbg = new CDebug("CFLogN");
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFLogN::CFLogN(const CExpression* _value) : CEAbstractFunction("Log", _value) {
}

CFLogN::~CFLogN() { }

const char* CFLogN::CLASS_NAME = "CFLogN";
const char* CFLogN::ClassName() const { return CLASS_NAME; }
bool CFLogN::contain_vector(void * obj) {
	if (CEAbstractFunction::contain_vector(obj)) return true;
	return false; 
}

CExpression* CFLogN::clone() const {
	CExpression* res = new CFLogN(dataConst());
	copyAttributesTo(res);
	return res;
}

/*
*	Note: No param->0 replacements are allowed.
*/
CExpression* CFLogN::applyRule(CEAbstractRule* rule) {
	//return CEAbstractFunction::applyRule(rule);
	CERule* r = (CERule*)rule;

	int s = params_num(); if (s != 1) { dbg->log(0, s, "<Warning> CFLogN: %ld parameters."); CEException::halt(); }
	CExpression* par = get_param(0); if (par == NULL) CEException::halt(dbg, "Log: NULL parameter");
	
	CExpression* it_new = r->ApplyTo(par->clone(), true);
	if ((it_new != NULL) && (!par->equals(it_new))) {
		// check for 0
		bool is_zero = CEBasic::isZero(it_new);
		if (!is_zero) {
			CExpression* it_new_reduced = CExpression::reduce(it_new->clone());
			is_zero = CEBasic::isZero(it_new_reduced);
			delete it_new_reduced;
		}
		if (!is_zero) { set_param(0, it_new); }
	}
	if (it_new != NULL) delete it_new;

	return this;
}

CExpression* CFLogN::Arrange(int level) {
	CEAbstractFunction::Arrange(AL_INLINE);
	CExpression* res = this;

	if (level == AL_EXTRACT) {
		CExpression* par = get_param(0);
		if (CEBasic::isUnit(par)) {
			res = CENumber::GetZero(); copyAttributesTo(res, true);
		}
	}

	return res;
}

CExpression* CFLogN::evaluate(int level) { return this; }

const char* CFLogN::CLASS_TAG_NAME = "ln";
const char* CFLogN::get_TAG_Name() { return CLASS_TAG_NAME; }

long CFLogN::store_stream(ofstream* p_ofs, DWORD settings) {
	long res = CESupport::store_stream(data(), p_ofs, settings);
	return res;
}

CESerializable* CFLogN::restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings) {
	long skip_pos = CESupport::skip_item_in_buffer(data, pos, settings);
	if (skip_pos <= 0) { dbg->log(0, "Error restoring CFLogN : Can't load parameter expression."); return NULL; }
	CESerializable* par_obj = CESupport::read_buffer(data, pos, settings);
	if ((par_obj == NULL) || (!CEBasic::isExpression(par_obj))) {
		dbg->log(0, "Error loading parameter expression for CFLogN.");
		if (par_obj != NULL) delete par_obj;
		return NULL;
	}
	CExpression* par = (CExpression*)par_obj;

	CFLogN* res = new CFLogN(par);
	delete par;
	return res;
}

CEString* CFLogN::getTitleAdv(unsigned conventions, int mode) {
	CEString* res = NULL;
	switch (mode) {
	case STORE_CPP:
		res = new CEString("log"); break;
	case STORE_FORTRAN_SYMBOL:
		res = new CEString("DLOG"); break;
	}

	if (res == NULL) res = CEAbstractFunction::getTitleAdv(conventions, mode);
	return res;
}
