////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// MiscExpressions.cpp
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// CFuncIf
//////////////////////////////////////////////////////////////////////
CFuncIf::CFuncIf(CExpression* condition, CExpression* valTrue, CExpression* valFalse) : CEAbstractFunction("If") {
	add_param(condition);
	add_param(valTrue);
	add_param(valFalse);
}

CFuncIf::CFuncIf(const CEPtrArray* params) : CEAbstractFunction("If",params) {}

CFuncIf::~CFuncIf() {}

CEString* CFuncIf::getTitleAdv(unsigned conventions, int mode) {
	CEString* res = NULL;
	switch (mode) {
	case STORE_CPP:
	case STORE_FORTRAN_SYMBOL: res = new CEString("ALHEP_IF"); break;
	}
	if (res == NULL) res = CEAbstractFunction::getTitleAdv(conventions, mode);
	return res;
}

const char* CFuncIf::CLASS_NAME = "CFuncIf";
const char* CFuncIf::ClassName() const { return CLASS_NAME; }
bool CFuncIf::contain_vector(void * obj) { return (CEAbstractFunction::contain_vector(obj)); }
CExpression* CFuncIf::clone() const { CExpression* res = new CFuncIf(GetParamsConst()); copyAttributesTo(res); return res; }
CExpression* CFuncIf::evaluate(int level) { return Arrange(AL_EXTRACT); /*return this;*/ }

CExpression* CFuncIf::Arrange(int level) {
	CEAbstractFunction::Arrange(AL_INLINE);

	if (level == AL_EXTRACT) {
		CExpression* condition = get_param(0);
		CExpression* res=NULL;
		if (condition!=NULL) {
			if (condition->ClassName() == CENumber::CLASS_NAME) {
				CENumber* nCondition=(CENumber*)condition;
				if (CEBasic::isZero(nCondition)) { // zero condition
					res=get_param(2);
				} else if (nCondition->getNumIm()==0) { // Non-zero, but real
					res=get_param(1);
				}
			}
		}
		if (res != NULL) {
			res=res->clone();
			// add parameters 4..N
			long parN=params_num();
			if (parN>3) {
				if (res->ClassName()==CESymbol::CLASS_NAME) {
					const char* s_res=((CESymbol*)res)->get_name(); if(s_res==NULL)s_res="CFuncIfError";
					CEFunction* f_res=new CEFunction(s_res);
					for(long i=3; i<parN; i++) { f_res->add_param(get_param(i)); }
					res->copyAttributesTo(f_res, true);
					delete res;
					res=f_res;
				} else {
					CEBasic::dbg->log(0,"WARNING: Can't process If[] with more than 3 parameters",this);
					CEBasic::dbg->log(0,"Target parameter (1/2) must be symbol here, to create Sym[par4,par5...] function",res);
					// create non-arrangable result (to avoid this message at every Arrange())
					CEFunction* f_res=new CEFunction("UnknownIfResult");
					f_res->add_param(res); delete res;
					for(long i=3; i<parN; i++) { f_res->add_param(get_param(i)); }
					res=f_res;
				}
			}
			if (res!=NULL) {
				copyAttributesTo(res, true);
				return res;
			}
		}
	}
	return this;
}

CExpression* CFuncIf::applyRule(CEAbstractRule* rule) { return CEAbstractFunction::applyRule(rule); }

const char* CFuncIf::CLASS_TAG_NAME = "bIf";
const char* CFuncIf::get_TAG_Name() { return CLASS_TAG_NAME; }
long CFuncIf::store_stream(ofstream* p_ofs, DWORD settings) {
	long res = 0;
	int s = params_num();
	res += CESupport::storeXMLLong(s,p_ofs,settings);
	int i;
	for (i=0; i<s; i++) {
		CExpression* par = get_param(i);
		if (par!=NULL) {
			res+=CESupport::store_stream(par, p_ofs, settings);
		} else {
			CENull nl; res+=CESupport::store_stream(&nl, p_ofs, settings);
		}
	}
	return res;
}

CESerializable* CFuncIf::restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings) {
	long lParsN=-1;
	if (!CESupport::readXMLLong(data,&pos,&lParsN,settings,CLASS_NAME)) { return NULL; }

	// load parameters
	CEPtrArray params_list;
	long i=0;
	for(;i<lParsN;i++) {
		long skip_pos = CESupport::skip_item_in_buffer(data, pos, settings);
		if (skip_pos > 0) {
			CESerializable* par = CESupport::read_buffer(data, pos, settings);
			pos = skip_pos;

			if ((par!=NULL)&&(CEBasic::isExpression(par))) {
				params_list.add(par);
			} else {
				CEBasic::dbg->log(0,i+1,"Error restoring CFuncIf: invalid parameter #%ld.",par); if(par!=NULL)delete par; return NULL;
			}
		} else {
			CEBasic::dbg->log(0,i+1,"Error restoring CFuncIf: can't read parameter #%ld."); return NULL;
		}
	}

	CESupport::checkAllRead(data, &pos, settings, CLASS_NAME);

	lParsN = params_list.size(); // reinit (unused now)
	if (lParsN<3) {
		CEBasic::dbg->log(0,lParsN,"Error restoring CFuncIf: At least 3 parameters are required. Read %ld parameters."); return NULL;
	}

	CFuncIf* res = new CFuncIf(&params_list);
	return res;
}

//////////////////////////////////////////////////////////////////////
// CEBoolean
//////////////////////////////////////////////////////////////////////
CEBoolean::CEBoolean() : CExpression() {}
const int CEBoolean::BOOL_AND	= 1;
const int CEBoolean::BOOL_OR	= 2;

//////////////////////////////////////////////////////////////////////
// CEBooleanEquZero
//////////////////////////////////////////////////////////////////////
CEBooleanEquZero::CEBooleanEquZero(CExpression* _Value) : CEBoolean() {
	m_Value=((_Value!=NULL)?_Value->clone():new CENull());
}

CEBooleanEquZero::~CEBooleanEquZero() { if (m_Value!=NULL) delete m_Value; }
CExpression* CEBooleanEquZero::getValue() { return m_Value; }
const char* CEBooleanEquZero::CLASS_NAME = "CEBooleanEquZero";
const char* CEBooleanEquZero::ClassName() const { return CLASS_NAME; }
CExpression* CEBooleanEquZero::clone() const { CEBooleanEquZero* res = new CEBooleanEquZero(m_Value); copyAttributesTo(res); return res; }
bool CEBooleanEquZero::equals(CExpression *other) {
	if (other==NULL) return false;
	if (other->ClassName()!=CEBooleanEquZero::CLASS_NAME) return false;
	if (!attributes_equals_to(other)) return false;
	CExpression* oValue=((CEBooleanEquZero*)other)->getValue();
	if ((m_Value != NULL) || (oValue != NULL)) {
		if ((m_Value == NULL) || (oValue == NULL)) return false;
		if (!m_Value->equals(oValue)) return false;
	}
	return true;
}
CEString* CEBooleanEquZero::toString(int mode) {
	CEString* res=new CEString("(");
	if (m_Value!=NULL) { res->AddString(m_Value->toString(mode)); } else { res->AddString("NULL"); }
	res->AddString("==0)");
	if(!getFlag())res->insertString("- ",0);
	return res;
}
bool CEBooleanEquZero::contain_vector(void * obj) {
	if (obj==NULL) return false;
	if (m_Value!=NULL) { if((m_Value==obj) || (m_Value->contain_vector(obj))) return true; }
	return false;
}

CExpression* CEBooleanEquZero::Arrange(int level) {
	if (m_Value!=NULL) {
		CExpression* _Value=m_Value->Arrange(AL_EXTRACT);
		if ((_Value!=NULL) && (_Value!=m_Value)) { delete m_Value; m_Value=_Value; }

		if (level==AL_EXTRACT) {
			CExpression* res=NULL;
			if (m_Value->ClassName() == CENumber::CLASS_NAME) {
				if (!m_Value->getLevel()) ((CENumber*)m_Value)->to_num_level();
				if (CEBasic::isZero((CENumber*)m_Value)) { res=CENumber::GetUnit(); } else { res=CENumber::GetZero(); }
			}
			if (res != NULL) {
				copyAttributesTo(res, true); return res;
			}
		}
	}
	return this;
}
CExpression* CEBooleanEquZero::applyRule(CEAbstractRule* rule) {
	CERule* r = (CERule*)rule;
	if (m_Value!=NULL) {
		CExpression* item_repl = r->ApplyTo(m_Value, false);
		if ((item_repl != m_Value) && (item_repl != NULL)) { delete m_Value; m_Value=item_repl; }
	}
	return this;
}

const char* CEBooleanEquZero::CLASS_TAG_NAME = "bool_equ0";
const char* CEBooleanEquZero::get_TAG_Name() { return CLASS_TAG_NAME; }
long CEBooleanEquZero::store_stream(ofstream* p_ofs, DWORD settings) {
	long res = 0;
	if (m_Value!=NULL) {
		res+=CESupport::store_stream(m_Value, p_ofs, settings);
	} else {
		CENull nl; res+=CESupport::store_stream(&nl, p_ofs, settings);
	}
	return res;
}

CESerializable* CEBooleanEquZero::restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings) {
	long skip_pos = CESupport::skip_item_in_buffer(data, pos, settings);
	if (skip_pos <= 0) { CEBasic::dbg->log(0, "Error restoring CEBooleanEquZero : Can't load parameter."); return NULL; }
	CESerializable* oPar0 = CESupport::read_buffer(data, pos, settings);
	if ((oPar0==NULL)||(!CEBasic::isExpression(oPar0))) { CEBasic::dbg->log(0, "Error restoring CEBooleanEquZero: invalid parameter."); if(oPar0!=NULL)delete oPar0; return NULL; }
	pos = skip_pos;
	CESupport::checkAllRead(data, &pos, settings, CLASS_NAME);

	CEBooleanEquZero* res = new CEBooleanEquZero((CExpression*)oPar0);
	delete oPar0;
	return res;
}

//////////////////////////////////////////////////////////////////////
// CEBooleanAndOr
//////////////////////////////////////////////////////////////////////
CEBooleanAndOr::CEBooleanAndOr(int iOperation) : CEBoolean() {
	m_iOperation=iOperation;
}

CEBooleanAndOr::CEBooleanAndOr(int iOperation, const CSequence* _Values) : CEBoolean() {
	m_Values.copy_from(_Values);
	m_iOperation=iOperation;
}

CEBooleanAndOr::~CEBooleanAndOr() {}
void CEBooleanAndOr::addOperand(CExpression* _Value) { if(_Value!=NULL)m_Values.addItem(_Value->clone()); }
int CEBooleanAndOr::getOperationFlag() { return m_iOperation; }
CSequence* CEBooleanAndOr::getOperands() { return &m_Values; }
const char* CEBooleanAndOr::CLASS_NAME = "CEBooleanAndOr";
const char* CEBooleanAndOr::ClassName() const { return CLASS_NAME; }
CExpression* CEBooleanAndOr::clone() const { CEBooleanAndOr* res = new CEBooleanAndOr(m_iOperation,&m_Values); copyAttributesTo(res); return res; }

bool CEBooleanAndOr::equals(CExpression *other) {
	if (other==NULL) return false;
	if (other->ClassName()!=CEBooleanAndOr::CLASS_NAME) return false;
	if (!attributes_equals_to(other)) return false;
	if (m_iOperation != ((CEBooleanAndOr*)other)->getOperationFlag()) return false;
	if (!m_Values.equals(((CEBooleanAndOr*)other)->getOperands())) return false;
	return true;
}

CEString* CEBooleanAndOr::toString(int mode) {
	CEString* res=new CEString("(");
	long s=m_Values.size();
	for(long i=0;i<s;i++) {
		CExpression* item=m_Values.GetAt(i);
		if (i>0) {
			if (m_iOperation==CEBoolean::BOOL_AND) { res->AddString("&&"); }
			else if (m_iOperation==CEBoolean::BOOL_OR) { res->AddString("||"); }
			else { res->AddString("??"); }
		}
		if (item!=NULL) { res->AddString(item->toString(mode)); } else { res->AddString("NULL"); }
	}
	res->AddString(")");
	if(!getFlag())res->insertString("- ",0);
	return res;
}

bool CEBooleanAndOr::contain_vector(void * obj) {
	if (obj==NULL) return false;
	if ((&m_Values==obj) || (m_Values.contain_vector(obj))) return true;
	return false;
}

CExpression* CEBooleanAndOr::Arrange(int level) {
	m_Values.Arrange(AL_INLINE);
	
	if (level==AL_EXTRACT) {
		bool bTrueSpotted=false;
		bool bFalseSpotted=false;
		bool bUnknownSpotted=false;
		long s=m_Values.size();
		for(long i=0; i<s; i++) {
			CExpression* item=(CExpression*)m_Values.GetAt(i); if(item==NULL) { CEException::halt(NULL,__FILE__,__LINE__); bUnknownSpotted=true; continue; }
			if (item->ClassName() == CENumber::CLASS_NAME) {
				if (!item->getLevel()) ((CENumber*)item)->to_num_level();
				if (CEBasic::isZero((CENumber*)item)) { // item: FALSE
					bFalseSpotted=true;
					if (m_iOperation == CEBoolean::BOOL_AND) break;
				} else if (((CENumber*)item)->getNumIm()==0) { // item: TRUE
					bTrueSpotted=true; 
					if (m_iOperation == CEBoolean::BOOL_OR) break;
				} else {
					bUnknownSpotted=true;
				}
			} else {
				bUnknownSpotted=true;
			}
		}

		CExpression* res=NULL;
		if (m_iOperation == CEBoolean::BOOL_AND) {
			if (bFalseSpotted) { // FALSE
				res=CENumber::GetZero();
			} else if (!bUnknownSpotted) { // TRUE
				res=CENumber::GetUnit();
			} // else UNKNOWN
		} else if (m_iOperation == CEBoolean::BOOL_OR) {
			if (bTrueSpotted) { // TRUE
				res=CENumber::GetUnit();
			} else if (!bUnknownSpotted) { // FALSE
				res=CENumber::GetZero();
			} // else UNKNOWN
		}
		if (res != NULL) {
			copyAttributesTo(res, true); return res;
		}
	}
	return this;

}

CExpression* CEBooleanAndOr::applyRule(CEAbstractRule* rule) {
	CExpression* tmp=m_Values.applyRule(rule); if(tmp!=&m_Values)CEException::halt("CEBooleanAndOr::applyRule");
	return this;
}

const char* CEBooleanAndOr::CLASS_TAG_NAME = "bool_seq";
const char* CEBooleanAndOr::get_TAG_Name() { return CLASS_TAG_NAME; }

long CEBooleanAndOr::store_stream(ofstream* p_ofs, DWORD settings) {
	long res = 0;
	res += CESupport::storeXMLLong((long)m_iOperation,p_ofs,settings);
	res += CESupport::store_stream(&m_Values, p_ofs, settings);
	return res;
}

CESerializable* CEBooleanAndOr::restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings) {
	long iOperation=-1;
	if (!CESupport::readXMLLong(data,&pos,&iOperation,settings,CLASS_NAME)) return NULL;
	
	long skip_pos = CESupport::skip_item_in_buffer(data, pos, settings);
	if (skip_pos <= 0) { CEBasic::dbg->log(0, "Error restoring CEBooleanAndOr : Can't load parameters."); return NULL; }
	CESerializable* oPar0 = CESupport::read_buffer(data, pos, settings);
	if ((oPar0==NULL)||(!CEBasic::isExpression(oPar0))) { CEBasic::dbg->log(0, "Error restoring CEBooleanAndOr: invalid parameters."); if(oPar0!=NULL)delete oPar0; return NULL; }
	CExpression* ePar0=(CExpression*)oPar0;
	if (ePar0->ClassName()!=CSequence::CLASS_NAME) { CEBasic::dbg->log(0, "Error restoring CEBooleanAndOr: invalid parameters (sequence expected)."); delete ePar0; return NULL; }
	pos = skip_pos;
	
	CESupport::checkAllRead(data, &pos, settings, CLASS_NAME);

	CEBooleanAndOr* res = new CEBooleanAndOr((int)iOperation,(CSequence*)ePar0);
	delete ePar0;
	return res;
}

//////////////////////////////////////////////////////////////////////
// CFuncConjugated
//////////////////////////////////////////////////////////////////////
CFuncConjugated::CFuncConjugated(CExpression* src) : CEAbstractFunction("Conjugate",src) {}
CFuncConjugated::CFuncConjugated(const CEPtrArray* params) : CEAbstractFunction("Conjugate",params) {}
CFuncConjugated::~CFuncConjugated() {}

CEString* CFuncConjugated::getTitleAdv(unsigned conventions, int mode) { //DCONJG
	CEString* res = NULL;
	switch (mode) {
	case STORE_CPP: res = new CEString("Conjg"); break;
	case STORE_FORTRAN_SYMBOL: res = new CEString("DCONJG"); break;
	}
	if (res == NULL) res = CEAbstractFunction::getTitleAdv(conventions, mode);
	return res;
}

const char* CFuncConjugated::CLASS_NAME = "CFuncConjugated";
const char* CFuncConjugated::ClassName() const { return CLASS_NAME; }
bool CFuncConjugated::contain_vector(void * obj) { return (CEAbstractFunction::contain_vector(obj)); }
CExpression* CFuncConjugated::clone() const { CExpression* res = new CFuncConjugated(GetParamsConst()); copyAttributesTo(res); return res; }
CExpression* CFuncConjugated::evaluate(int level) { return this; }

CExpression* CFuncConjugated::Arrange(int level) {
	return CEAbstractFunction::Arrange(level); //AL_INLINE
}

CExpression* CFuncConjugated::applyRule(CEAbstractRule* rule) { return CEAbstractFunction::applyRule(rule); }

CExpression* CFuncConjugated::conjugated(int mode) {
	CExpression* res=data(); if(res==NULL)CEException::halt("CFuncConjugated: NULL source");
	res=res->clone();
	copyAttributesTo(res,true);
	return res;
}

unsigned long CFuncConjugated::props_long() {
	unsigned long res=CEAbstractFunction::props_long();
	res=CEProperties::set_property_to(res,CEProperties::PROP_MASK_SELF_CONJUGATED,false);
	return res;
}

void CFuncConjugated::collectProperties(CEProperties* props, unsigned set) {
	CEAbstractFunction::collectProperties(props, set);
	props->setProperty(CEProperties::PROP_MASK_SELF_CONJUGATED, false);
}

const char* CFuncConjugated::CLASS_TAG_NAME = "FConjg";
const char* CFuncConjugated::get_TAG_Name() { return CLASS_TAG_NAME; }
long CFuncConjugated::store_stream(ofstream* p_ofs, DWORD settings) {
	long res = 0;
	CExpression* par = data();
	if (par!=NULL) { res+=CESupport::store_stream(par, p_ofs, settings); }
	else { CENull nl; res+=CESupport::store_stream(&nl, p_ofs, settings); }
	return res;
}

CESerializable* CFuncConjugated::restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings) {
	CESerializable* par = NULL;
	long skip_pos = CESupport::skip_item_in_buffer(data, pos, settings);
	if (skip_pos > 0) { par=CESupport::read_buffer(data, pos, settings); pos=skip_pos; }
	if ((par==NULL)||(!CEBasic::isExpression(par))) {
		CEBasic::dbg->log(0,"Error restoring CFuncConjugated: invalid parameter.",par); if(par!=NULL)delete par; return NULL;
	}
	if (!CESupport::checkAllRead(data, &pos, settings, CLASS_NAME)) {
		CEBasic::dbg->log(0,"Error restoring CFuncConjugated: invalid parameters."); if(par!=NULL)delete par; return NULL;
	}

	CFuncConjugated* res = new CFuncConjugated((CExpression*)par);
	delete par;
	return res;
}
