////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EPower.h: interface for the CEPower class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EPOWER_H__CFC69D00_9CA3_11D5_90CB_00403393DD18__INCLUDED_)
#define AFX_EPOWER_H__CFC69D00_9CA3_11D5_90CB_00403393DD18__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//##ModelId=3E3FF2D9009C
class CEPower : public CExpression  
{
private:
	CExpression* m_basis;
	CExpression* m_power;

private:
	void arrange_basis_sign();
	//void arrange_basis_level();
	//Note: use 'as_product' for external use!
	CExpression* to_product(CExpression* _base, CExpression* _power, bool allowLevelInverse=false);

	CExpression* ExpandABCpN(); //Expand (a*b*c)^N, NULL if unavailable
	bool ExpandApBpC(); //Expands (a^b)^c => a^(b*c) inline, 'false' if unavailable

public:
	long getLongPower();
	bool isPowerInteger();

	virtual long store_stream(ofstream* p_ofs, DWORD settings);
	virtual const char* get_TAG_Name();
	static CESerializable* restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings);

	CExpression* as_product();

	static const char* CLASS_NAME;
	static const char* CLASS_TAG_NAME;

	static CExpression* square(CExpression* source); // deletes source
	static CExpression* sqrt(CExpression* source); // deletes source

	virtual bool equals(CExpression* other);
	virtual bool less_than(CExpression* other);
	virtual CExpression* numeric();
	virtual bool contain_vector(void * obj);

	static CDebug* dbg;

	virtual CExpression* expand(bool recursive = true);

	CExpression* power() { return m_power; }
	CExpression* basis() { return m_basis; }

	virtual CExpression* clone() const;
	virtual const char* ClassName() const;

	CEPower(CExpression* basis, CExpression* power);
	CEPower(CExpression* basis, long power);
	CEPower(const char* basis, long power);
	virtual ~CEPower();

	void setBasis(CExpression* basis, bool do_clone = true);
	void setPower(CExpression* power, bool do_clone = true);
	void setPower(long power);

	virtual CExpression* applyRule(CEAbstractRule* rule);
	virtual CExpression* evaluate(int level = 0);
	virtual CExpression* Simplify(int level = 0);
	virtual CExpression* Arrange(int level);
	virtual CEString* toString(int mode = 0);
	virtual void toStream(ostream* os, int mode = 0);

public:
	virtual unsigned long props_long();
protected:
	virtual void collectProperties(CEProperties* props, unsigned set = CEProperties::PROP_MASK_ALL);
};

#endif // !defined(AFX_EPOWER_H__CFC69D00_9CA3_11D5_90CB_00403393DD18__INCLUDED_)
