////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// MiscExpressions.h
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_MISCEXPRESSIONS_H__0DF7541E_F097_42C7_8289_581C93CD688B__INCLUDED_)
#define AFX_MISCEXPRESSIONS_H__0DF7541E_F097_42C7_8289_581C93CD688B__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/*
*	CFuncIf
*/
class CFuncIf : public CEAbstractFunction  
{
	CFuncIf(const CEPtrArray* params);
public:
	CFuncIf(CExpression* condition, CExpression* valTrue, CExpression* valFalse);
	virtual ~CFuncIf();

	virtual CEString* getTitleAdv(unsigned conventions, int mode);

	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual bool contain_vector(void * obj);
	virtual CExpression* clone() const;
	virtual CExpression* evaluate(int level = 0);
	virtual CExpression* Arrange(int level);
	virtual CExpression* applyRule(CEAbstractRule* rule);

	static const char* CLASS_TAG_NAME;
	virtual const char* get_TAG_Name();
	virtual long store_stream(ofstream* p_ofs, DWORD settings);
	static CESerializable* restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings);
};

/*
*	boolean expressions turns 1 or 0 at Arrange(AL_EXTRACT)
*/
class CEBoolean : public CExpression {
public:
	CEBoolean();
	static const int BOOL_AND;
	static const int BOOL_OR;
};

class CEBooleanEquZero : public CEBoolean {
	CExpression* m_Value;
public:
	CEBooleanEquZero(CExpression* _Value);
	virtual ~CEBooleanEquZero();
	CExpression* getValue();
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual CExpression* clone() const;
	virtual bool equals(CExpression *other);
	virtual CEString* toString(int mode = 0);
	virtual bool contain_vector(void * obj);
	virtual CExpression* Arrange(int level);
	virtual CExpression* applyRule(CEAbstractRule* rule);
	static const char* CLASS_TAG_NAME;
	virtual const char* get_TAG_Name();
	virtual long store_stream(ofstream* p_ofs, DWORD settings);
	static CESerializable* restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings);
};

class CEBooleanAndOr : public CEBoolean {
	CSequence m_Values;
	int m_iOperation;
public:
	CEBooleanAndOr(int iOperation);
	CEBooleanAndOr(int iOperation, const CSequence* _Values);
	virtual ~CEBooleanAndOr();
	void addOperand(CExpression* _Value);
	int getOperationFlag();
	CSequence* getOperands();
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual CExpression* clone() const;
	virtual bool equals(CExpression *other);
	virtual CEString* toString(int mode = 0);
	virtual bool contain_vector(void * obj);
	virtual CExpression* Arrange(int level);
	virtual CExpression* applyRule(CEAbstractRule* rule);
	static const char* CLASS_TAG_NAME;
	virtual const char* get_TAG_Name();
	virtual long store_stream(ofstream* p_ofs, DWORD settings);
	static CESerializable* restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings);
};

class CFuncConjugated : public CEAbstractFunction
{
	CFuncConjugated(const CEPtrArray* params);
public:
	CFuncConjugated(CExpression* src);
	virtual ~CFuncConjugated();
	virtual CEString* getTitleAdv(unsigned conventions, int mode);
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual bool contain_vector(void * obj);
	virtual CExpression* clone() const;
	virtual CExpression* evaluate(int level = 0);
	virtual CExpression* Arrange(int level);
	virtual CExpression* applyRule(CEAbstractRule* rule);
	virtual CExpression* conjugated(int mode = 0);

public:
	static const char* CLASS_TAG_NAME;
	virtual const char* get_TAG_Name();
	virtual long store_stream(ofstream* p_ofs, DWORD settings);
	static CESerializable* restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings);

public:
	virtual unsigned long props_long();
protected:
	virtual void collectProperties(CEProperties* props, unsigned set = CEProperties::PROP_MASK_ALL);
};

#endif // !defined(AFX_MISCEXPRESSIONS_H__0DF7541E_F097_42C7_8289_581C93CD688B__INCLUDED_)
