////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EFuncBondarev.cpp: implementation of the CEFuncBondarev class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

CDebug* CEFuncBondarev::dbg = new CDebug("CEFuncBondarev");
bool CEFuncBondarev::m_DoExpandF3 = false; // Expand F3->gamma-F1
CExpression* CEFuncBondarev::m_FuncBondarevSym = NULL;
CExpression* CEFuncBondarev::GetFuncBondarevSym() {
	if (m_FuncBondarevSym==NULL) { m_FuncBondarevSym = new CESymbol("F"); CEProcessor::AddToDeleteList(m_FuncBondarevSym); }
	return m_FuncBondarevSym;
}

CESymbol* CEFuncBondarev::m_BondarevBaseQP = NULL; // "q+"
CESymbol* CEFuncBondarev::m_BondarevBaseQM = NULL; // "q-"
CESymbol* CEFuncBondarev::m_BondarevBaseEP = NULL; // "e+"
CESymbol* CEFuncBondarev::m_BondarevBaseEM = NULL; // "e-"
CESymbol* CEFuncBondarev::GetBondarevBaseSym(int i) { //{i: 0(q+),1(q-),2(e+),3(e-)
	CESymbol* res = NULL;
	switch(i) {
	case 0: res =m_BondarevBaseQP; break;
	case 1: res =m_BondarevBaseQM; break;
	case 2: res =m_BondarevBaseEP; break;
	case 3: res =m_BondarevBaseEM; break;
	}
	//if (do_init) { if(res==NULL)SetBondarevBaseSymsDefaults(); res=GetBondarevBaseSym(i);}
	return res;
}

int CEFuncBondarev::FindBondarevBaseSyms(CExpression* sym) { // -1 if failed
	if (sym==NULL) return -1;
	int res=-1;
	if (sym->equals(m_BondarevBaseQP)) { res=0; }
	else if (sym->equals(m_BondarevBaseQM)) { res=1; }
	else if (sym->equals(m_BondarevBaseEP)) { res=2; }
	else if (sym->equals(m_BondarevBaseEM)) { res=3; }
	return res;
}

void CEFuncBondarev::SetBondarevBaseSyms(CESymbol* qP,CESymbol* qM,CESymbol* eP,CESymbol* eM) { //{q+,q-,e+,e-}
	if ((qP==NULL) || (qM==NULL) || (eP==NULL) || (eM==NULL)) {
		dbg->log(0, "ERROR: SetBondarevBaseSyms(NULLs) call. Previous(default) symbols are kept"); SetBondarevBaseSymsDefaults(false); return;
	}
	if ((CLHEPBasics::isSymbolBooked(qP)) || (CLHEPBasics::isSymbolBooked(qM)) || (CLHEPBasics::isSymbolBooked(eP)) || (CLHEPBasics::isSymbolBooked(eM))) {
		dbg->log(0, "ERROR. SetBondarevBaseSyms() symbols are already used. Previous(default) symbols are kept"); SetBondarevBaseSymsDefaults(false); return;
	}
	
	// Note: old m_BondarevBaseXX objects are added to "to-delete-list", do not delete old objects here
	m_BondarevBaseQP=(CESymbol*)qP->clone(); CEProcessor::AddToDeleteList(m_BondarevBaseQP);
	m_BondarevBaseQM=(CESymbol*)qM->clone(); CEProcessor::AddToDeleteList(m_BondarevBaseQM);
	m_BondarevBaseEP=(CESymbol*)eP->clone(); CEProcessor::AddToDeleteList(m_BondarevBaseEP);
	m_BondarevBaseEM=(CESymbol*)eM->clone(); CEProcessor::AddToDeleteList(m_BondarevBaseEM);
}

void CEFuncBondarev::SetBondarevBaseSymsDefaults(bool _reset) {
	// Note: old m_BondarevBaseXX objects are added to "to-delete-list", do not delete old objects here
	if ((m_BondarevBaseQP==NULL)||(_reset)) {
		m_BondarevBaseQP=NULL;
		CESymbol* tmp=new CESymbol("BqP");
		if (CLHEPBasics::isSymbolBooked(tmp)) {
			delete tmp; tmp = CESymbol::getUnique(); dbg->log(0, "ERROR: Can't book symbol for BONDAREV FUNCTIONS BASIS, unique is used QP=",tmp);
		}
		m_BondarevBaseQP=tmp;
		CEProcessor::AddToDeleteList(m_BondarevBaseQP);
	}
	if ((m_BondarevBaseQM==NULL)||(_reset)) {
		m_BondarevBaseQM=NULL;
		CESymbol* tmp=new CESymbol("BqM");
		if (CLHEPBasics::isSymbolBooked(tmp)) {
			delete tmp; tmp = CESymbol::getUnique(); dbg->log(0, "ERROR: Can't book symbol for BONDAREV FUNCTIONS BASIS, unique is used QM=",tmp);
		}
		m_BondarevBaseQM=tmp;
		CEProcessor::AddToDeleteList(m_BondarevBaseQM);
	}
	if ((m_BondarevBaseEP==NULL)||(_reset)) {
		m_BondarevBaseEP=NULL;
		CESymbol* tmp=new CESymbol("BeP");
		if (CLHEPBasics::isSymbolBooked(tmp)) {
			delete tmp; tmp = CESymbol::getUnique(); dbg->log(0, "ERROR: Can't book symbol for BONDAREV FUNCTIONS BASIS, unique is used EP=",tmp);
		}
		m_BondarevBaseEP=tmp;
		CEProcessor::AddToDeleteList(m_BondarevBaseEP);
	}
	if ((m_BondarevBaseEM==NULL)||(_reset)) {
		m_BondarevBaseEM=NULL;
		CESymbol* tmp=new CESymbol("BeM");
		if (CLHEPBasics::isSymbolBooked(tmp)) {
			delete tmp; tmp = CESymbol::getUnique(); dbg->log(0, "ERROR: Can't book symbol for BONDAREV FUNCTIONS BASIS, unique is used EM=",tmp);
		}
		m_BondarevBaseEM=tmp;
		CEProcessor::AddToDeleteList(m_BondarevBaseEM);
	}
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CEFuncBondarev::CEFuncBondarev(int _No, const CExpression* couple1, const CExpression* couple2, const CExpression* index1, bool cov1, const CExpression* index2, bool cov2)
: CETensor(GetFuncBondarevSym())
{
	m_No = _No;
	m_couple1 = (couple1!=NULL)?(couple1->clone()):NULL;
	m_couple2 = (couple2!=NULL)?(couple2->clone()):NULL;
	if (index1 != NULL) { m_indices->add(CTIndex::CreateVectorIndex(index1, cov1)); }
	if (index2 != NULL) { m_indices->add(CTIndex::CreateVectorIndex(index2, cov2)); }
}

CEFuncBondarev::~CEFuncBondarev() {
	if (m_couple1 != NULL) delete m_couple1;
	if (m_couple2 != NULL) delete m_couple2;
}

CExpression* CEFuncBondarev::GetCouple1() { return m_couple1; }
CExpression* CEFuncBondarev::GetCouple2() { return m_couple2; }
int CEFuncBondarev::GetNo() { return m_No; }
CExpression* CEFuncBondarev::GetIndexLogical(int i) {
	CExpression* res = NULL;
	switch(i) {
	case 1:
		if (m_couple1 == NULL) res = index(1);
		break;
	case 2:
		if (m_couple2 == NULL) res = index(rank());
		break;
	}
	return res;
}
bool CEFuncBondarev::GetCovLogical(int i) {
	bool res = true;
	switch(i) {
	case 1:
		if (m_couple1 == NULL) res = covariance(1);
		break;
	case 2:
		if (m_couple2 == NULL) res = covariance(rank());
		break;
	}
	return res;
}
bool CEFuncBondarev::check_struture() {
	bool res = true;
	int r = rank();
	switch(r) {
	case 0: if ((m_couple1==NULL)||(m_couple2==NULL)) res=false; break;
	case 1:
		if (m_couple1==NULL) {
			if (m_couple2==NULL) res=false;
		} else {
			if (m_couple2!=NULL) res=false;
		}
		break;
	case 2: if ((m_couple1!=NULL)||(m_couple2!=NULL)) res=false; break;
	default:
		res=false;
	}
	if ((m_No<1) || (m_No>8)) res = false;
	if (!res) dbg->log(0, "CEFuncBondarev internal check failed: invalid structure", this);
	return res;
}

const char* CEFuncBondarev::CLASS_NAME = "CEFuncBondarev";
const char* CEFuncBondarev::ClassName() const { return CLASS_NAME; }

CExpression* CEFuncBondarev::clone() const {
	const CExpression* ind1 = NULL; bool cov1=true;
	const CExpression* ind2 = NULL; bool cov2=true;
	int r = rank();
	if (r>0) { ind1 = GetIndex(1); cov1=covariance(1); }
	if (r>1) { ind2 = GetIndex(2); cov2=covariance(2); }
	CEFuncBondarev* res = new CEFuncBondarev(m_No, m_couple1, m_couple2, ind1, cov1, ind2, cov2);
	copyAttributesTo(res);
	return res;
}

CEString* CEFuncBondarev::toString(int mode) {
	CEString* res = NULL;
	int r;int i;
	switch (mode)
	{
	case STORE_CPP:
	case STORE_FORTRAN_SYMBOL:
		res = new CEString((getFlag()?"":"-"));
		res->AddString("FB"); res->add_int(m_No);
		if (m_couple1 != NULL) { res->AddString(m_couple1->toString(mode)); } else { res->AddString("_"); }
		if (m_couple2 != NULL) { res->AddString(m_couple2->toString(mode)); } else { res->AddString("_"); }
		r = rank();
		for(i=1; i<=r; i++) { res->AddString(index(i)->toString(mode)); int i_cov = (covariance(i)?1:0); res->add_int(i_cov); }
		break;
	case STORE_PLAIN_TEXT:
	case STORE_WR_MATHEMATICA:
		if (getFlag()) res = new CEString("\\(");
		else res = new CEString("- \\(");
		r = rank(); if (r>0) res->AddString("\\(");
		res->AddString(GetFuncBondarevSym()->toString(mode));
		res->add_int(m_No);
		if ((m_couple1 != NULL) || (m_couple2 != NULL)) {
			res->AddString("\\+\\(");
			if (m_couple1 != NULL) { res->AddString(m_couple1->toString(mode)); }
			else { res->AddString("*"); }
			if (m_couple2 != NULL) { res->AddString(m_couple2->toString(mode)); }
			else { res->AddString(""); }
			res->AddString("\\)");
		}
		if (r > 0) {
			res->AddString("\\)");
			bool last_covariance;
			for(i=0; i<r; i++) {
				if (i==0) {
					last_covariance = covariance(1); res->AddString((last_covariance)?"\\_":"\\&");
					res->AddString("\\("); res->AddString(index(1)->toString(mode)); res->AddString("\\)");
				} else {
					bool cov = covariance(i+1);
					if (cov == last_covariance) {
						long ins_pos = res->length()-2;//insert before bracket closed
						res->insertString(index(i+1)->toString(mode), ins_pos); res->insertString("\\ ", ins_pos);
					} else {
						res->insertString("\\(", 0); res->AddString("\\)"); res->AddString((cov)?"\\_":"\\&");
						res->AddString("\\("); res->AddString(index(i+1)->toString(mode)); res->AddString("\\)");
						last_covariance = cov;
					}
				}
			}
		}
		res->AddString("\\)");
		break;
	default:
		res = new CEString("<Not supported data format.>");
	}
	return res;
}

const char* CEFuncBondarev::get_TAG_Name() { return CLASS_TAG_NAME; }
const char* CEFuncBondarev::CLASS_TAG_NAME = "fBond";
long CEFuncBondarev::store_stream(ofstream* p_ofs, DWORD settings) {
	long res = 0;
	CENumber* _zero = CENumber::GetZero(); //TODO: avoid spare ZERO create/delete
	CENumber _no(m_No);
	res += CESupport::store_stream(&_no, p_ofs, settings);
	if (m_couple1 != NULL) { res += CESupport::store_stream(m_couple1, p_ofs, settings); }
	else { res += CESupport::store_stream(_zero, p_ofs, settings); }
	if (m_couple2 != NULL) { res += CESupport::store_stream(m_couple2, p_ofs, settings); }
	else { res += CESupport::store_stream(_zero, p_ofs, settings); }
	int r = rank();
	if (r>0) {
		CENumber _no2(r);
		res += CESupport::store_stream(&_no2, p_ofs, settings);
		for (int i=1; i<=r; i++) {
			int i_cov = (covariance(i))?1:0; CENumber _nCov(i_cov);
			res += CESupport::store_stream(index(i), p_ofs, settings);
			res += CESupport::store_stream(&_nCov, p_ofs, settings);
		}
	}
	return res;
}
CESerializable* CEFuncBondarev::restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings) {
	long _no = 0; if (!CESupport::readXMLLong(data, &pos, &_no, settings, CLASS_NAME)) return NULL;
	CExpression* c1 = CESupport::readXMLExpr(data, &pos, settings, CLASS_NAME); if(c1==NULL) return NULL;
	CExpression* c2 = CESupport::readXMLExpr(data, &pos, settings, CLASS_NAME); if(c2==NULL) { delete c1; return NULL; }

	long _r = 0;
	CEPtrArray* _inds=NULL;
	if (CESupport::readXMLLong(data, &pos, &_r, settings, CLASS_NAME, true)) {
		_inds=new CEPtrArray();
		for(long i=0; i<_r; i++) {
			CExpression* _ind=CESupport::readXMLExpr(data,&pos,settings,CLASS_NAME);if(_ind==NULL){delete c1;delete c2;delete _inds;return NULL;}
			long _cov=0; if (!CESupport::readXMLLong(data,&pos,&_cov,settings,CLASS_NAME)){delete c1;delete c2;delete _inds;return NULL;}
			_inds->add(CTIndex::CreateVectorIndex(_ind,(_cov>0))); delete _ind;
		}
	}

	int _sz=_r+2;
	if (CEBasic::isZero(c1)) { delete c1; c1=NULL; _sz--; }
	if (CEBasic::isZero(c2)) { delete c2; c2=NULL; _sz--; }
	if (_sz != 2) { dbg->log(0, "Error restoring CEFuncBondarev, invalid number of arguments", c1,c2,_inds); }
	
	CExpression* ind1 = NULL; bool cov1=true; CExpression* ind2 = NULL; bool cov2=true;
	if (_r>0) { CTIndex* _ti=(CTIndex*)_inds->GetAt(0); ind1=_ti->index(); cov1=_ti->covariance(); }
	if (_r>1) { CTIndex* _ti=(CTIndex*)_inds->GetAt(1); ind2=_ti->index(); cov2=_ti->covariance(); }
	CEFuncBondarev* res = new CEFuncBondarev(_no, c1, c2, ind1, cov1, ind2, cov2);

	delete _inds;
	if (c1 != NULL) delete c1;
	if (c2 != NULL) delete c2;
	return res;
}


bool CEFuncBondarev::contain_vector(void * obj) {
	if ((m_couple1 == obj) || (m_couple2 == obj)) return true;
	if (m_couple1 != NULL) { if (m_couple1->contain_vector(obj)) return true; }
	if (m_couple2 != NULL) { if (m_couple2->contain_vector(obj)) return true; }
	return CETensor::contain_vector(obj);
}

CExpression* CEFuncBondarev::applyRule(CEAbstractRule *rule) {
	CERule* r = (CERule*)rule;
	if (m_couple1 != NULL) m_couple1 = r->ApplyTo(m_couple1);
	if (m_couple2 != NULL) m_couple2 = r->ApplyTo(m_couple2);
	return CETensor::applyRule(rule);
}

bool CEFuncBondarev::equals(CExpression* other) {
	if (other->ClassName() != CLASS_NAME) return false; // fast check
	if (m_No != ((CEFuncBondarev*)other)->GetNo()) return false;

	if (!CETensor::equals(other)) return false; // attributes, indices

	CExpression* o_cpl = ((CEFuncBondarev*)other)->GetCouple1();
	if (m_couple1 != NULL) {
		if (o_cpl == NULL) return false;
		if (!m_couple1->equals(o_cpl)) return false;
	} else {
		if (o_cpl != NULL) return false;
	}

	o_cpl = ((CEFuncBondarev*)other)->GetCouple2();
	if (m_couple2 != NULL) {
		if (o_cpl == NULL) return false;
		if (!m_couple2->equals(o_cpl)) return false;
	} else {
		if (o_cpl != NULL) return false;
	}
	return true;
}

bool CEFuncBondarev::less_than(CExpression* other) {
	if (other == NULL) return false;

	// compare to CESymbol, CEPower, CESProduct, CEVector, CEMetric, CEAntisymmetric, CEAntiSymmProd, CEFuncBondarev
	// a < a^n < a.b < a_nu < a_mu_nu < a_abcd < F[ab] < F_i[a] < F_ij
	const char* o_cn = other->ClassName();
	if (o_cn == CEPower::CLASS_NAME) {
		if (!CEUtil::LEQU(getFlag(), other->getFlag())) return other->getFlag(); // compare sign: "-"sym is < than "sym"
		return false;
	}

	if (o_cn == CESymbol::CLASS_NAME) {
		if (!CEUtil::LEQU(getFlag(), other->getFlag())) return other->getFlag(); // compare sign: "-"sym is < than "sym"
		return false;
	}

	if (o_cn == CESProduct::CLASS_NAME) {
		if (!CEUtil::LEQU(getFlag(), other->getFlag())) return other->getFlag(); // compare sign: "-"sym is < than "sym"
		return false;
	}

	if ((o_cn==CEVector::CLASS_NAME)||(o_cn==CEMetric::CLASS_NAME)||(o_cn==CEAntisymmetric::CLASS_NAME)) {
		if (!CEUtil::LEQU(getFlag(), other->getFlag())) return other->getFlag(); // compare sign: "-"sym is < than "sym"
		return false;
	}

	if (o_cn==CEAntiSymmProd::CLASS_NAME) {
		if (!CEUtil::LEQU(getFlag(), other->getFlag())) return other->getFlag(); // compare sign: "-"sym is < than "sym"
		return false;
	}

	if (o_cn==CEFuncBondarev::CLASS_NAME) {
		if (!CEUtil::LEQU(getFlag(), other->getFlag())) return other->getFlag(); // compare sign: "-"sym is < than "sym"
		CEFuncBondarev* o = (CEFuncBondarev*)other;
		int o_No=o->GetNo();
		if (m_No != o_No) return (m_No < o_No);

		CExpression* cX = o->GetCouple1();
		if (m_couple1 != NULL) {
			if (cX == NULL) return false;
			if (m_couple1->less_than(cX)) return true;
			if (!m_couple1->equals(cX)) return false;
		} else {
			if (cX != NULL) return true;
		}

		cX= o->GetCouple2();
		if (m_couple2 != NULL) {
			if (cX == NULL) return false;
			if (m_couple2->less_than(cX)) return true;
			if (!m_couple2->equals(cX)) return false;
		} else {
			if (cX != NULL) return true;
		}

		int s = rank(); int s2 = o->rank();
		for(int i=1; ((i<=s) && (i<=s2)); i++) {
			if (!CEUtil::LEQU(covariance(i), o->covariance(i))) return covariance(i); // cov is < than !cov
			CExpression* ind1 = index(i); CExpression* ind2 = o->index(i);
			if (ind1->less_than(ind2)) return true;
			if (!ind1->equals(ind2)) return false;
		}
		if (s != s2) return (s<s2);
		return false;
	}

	return CExpression::less_than(other);
}

CExpression* CEFuncBondarev::expand(bool recursive) {
	if (m_couple1 != NULL) {
		m_couple1=CExpression::reduce(m_couple1);
		if (CEBasic::isZero(m_couple1)) {
			CExpression* res = CENumber::GetZero(); copyAttributesTo(res, true); return res;
		}
		if (m_couple1->getProperty(CEProperties::PROP_MASK_IS_SUM)) {
			CExpression* res=NULL;
			CESum* cpl_sum = (CESum*)m_couple1;
			long s = cpl_sum->size();
			if (s==0) { res = CENumber::GetZero(); copyAttributesTo(res, true); return res; }
			if (!cpl_sum->getLevel()) { CEException::halt("-level at CEFuncBondarev coupling"); }
			CESum* res_sum = new CESum();
			for(long i=0; i<s; i++) { m_couple1 = cpl_sum->GetAt(i); res_sum->addItem(clone()); }
			m_couple1=cpl_sum; // turn object back

			// further reduce (by second coupling)
			res = CExpression::reduce(res_sum);
			copyAttributesTo(res, true);
			return res;
		}
	}

	if (m_couple2 != NULL) {
		m_couple2=CExpression::reduce(m_couple2);
		if (CEBasic::isZero(m_couple2)) {
			CExpression* res = CENumber::GetZero(); copyAttributesTo(res, true); return res;
		}

		if (m_couple2->getProperty(CEProperties::PROP_MASK_IS_SUM)) {
			CExpression* res=NULL;
			CESum* cpl_sum = (CESum*)m_couple2;
			long s = cpl_sum->size();
			if (s==0) { res = CENumber::GetZero(); copyAttributesTo(res, true); return res; }
			if (!cpl_sum->getLevel()) { CEException::halt("-level at CEFuncBondarev coupling"); }
			CESum* res_sum = new CESum();
			for(long i=0; i<s; i++) { m_couple2 = cpl_sum->GetAt(i); res_sum->addItem(clone()); }
			m_couple2=cpl_sum; // turn object back

			// no further reduce required (m_couple1 is aready expanded)
			res = res_sum;
			copyAttributesTo(res, true);
			return res;
		}
	}

	if (m_DoExpandF3) {
		if ((m_No == 3)||(m_No == 7)) {
			int d = m_No>4;
			// F3[a,b]=2 a.b - F1[a,b]
			CExpression* ind1 = GetIndexLogical(1); CExpression* ind2 = GetIndexLogical(2);
			bool cov1 = GetCovLogical(1); bool cov2 = GetCovLogical(2);
			CExpression* res=NULL;
			if (ind1!=NULL) {
				if (ind2!=NULL) {
					res = new CEMetric(ind1,ind2,cov1,cov2);
				} else {
					res = new CEVector(m_couple2,ind1,cov1);
				}
			} else {
				if (ind2!=NULL) {
					res = new CEVector(m_couple1,ind2,cov2);
				} else {
					res = new CESProduct(m_couple1,m_couple2);
				}
			}
			res = res->multiply(CENumber::GetTwo());
			res = res->minus(new CEFuncBondarev(1+d,m_couple1,m_couple2,GetIndexLogical(1),GetCovLogical(1),GetIndexLogical(2),GetCovLogical(2)));
			copyAttributesTo(res, true);
			return res;
		}
	}
	return this;
}

CExpression* CEFuncBondarev::Arrange(int level) {
	CExpression* ind1 = GetIndexLogical(1); CExpression* ind2 = GetIndexLogical(2);
	if (ind1 != NULL) {
		if (ind2 != NULL) {
			// compare indices
			if (ind2->less_than(ind1)) {
				exchange_indices(1, 2);
				int No_old = m_No;
				switch(No_old) {
				case 1: m_No=3; break;
				case 2: inverseFlag(); break;
				case 3: m_No=1; break;
				case 4: inverseFlag(); break;
				case 5: m_No=7; break;
				case 6: inverseFlag(); break;
				case 7: m_No=5; break;
				case 8: inverseFlag(); break;
				}
			}
		} else {
			// nothing to arrange
		}
	} else {
		if (ind2 != NULL) {
			// exchange logical indices
			if(m_couple2!=NULL) { dbg->log(0,"CEFuncBondarev::Arrange() ERROR: invalid structure", this); CEException::halt(); } //TODO: remove after debug
			m_couple2=m_couple1; m_couple1=NULL;
			int No_old = m_No;
			switch(No_old) {
			case 1: m_No=3; break;
			case 2: inverseFlag(); break;
			case 3: m_No=1; break;
			case 4: inverseFlag(); break;
			case 5: m_No=7; break;
			case 6: inverseFlag(); break;
			case 7: m_No=5; break;
			case 8: inverseFlag(); break;
			}
		} else {
			// compare couplings
			if((m_couple1==NULL)||(m_couple2==NULL)) { dbg->log(0,"CEFuncBondarev::Arrange() ERROR: invalid structure", this); CEException::halt(); } //TODO: remove after debug
			if (m_couple2->less_than(m_couple1)) {
				CExpression* temp=m_couple2;
				m_couple2=m_couple1; m_couple1=temp;
				int No_old = m_No;
				switch(No_old) {
				case 1: m_No=3; break;
				case 2: inverseFlag(); break;
				case 3: m_No=1; break;
				case 4: inverseFlag(); break;
				case 5: m_No=7; break;
				case 6: inverseFlag(); break;
				case 7: m_No=5; break;
				case 8: inverseFlag(); break;
				}
			}
		}
	}
	return this;
}

CExpression* CEFuncBondarev::unite_with(IETensor* other, bool other_is_at_right, bool bCalledByOther) {
	if (other == NULL) return NULL;

	int r = rank();
	if (r == 0) return NULL;

	if (!check_struture()) { CEException::halt(); } //TODO: remove this check after debug

	CExpression* o_e = other->ThisExpr();
	const char* o_CN = o_e->ClassName();
	int o_rank = other->rank();
	if (o_rank == 0) return NULL;
	
	// couple to Metric, (Antisymetric??), Vector & CEFuncBondarev
	if ((o_CN == CEMetric::CLASS_NAME) && (!bCalledByOther)) {
		return other->unite_with(this, !other_is_at_right, true);
	}

	if (o_CN == CEVector::CLASS_NAME) {
		CExpression* o_index = other->index(1); bool o_cov = other->covariance(1);
		if (index(1)->equals(o_index)) {
			if (covariance(1) == o_cov) dbg->log(0, "<Warning><covariance() == other->covariance(1)>", this, other->ThisExpr());
			CExpression* res = NULL;
			if (m_couple1 == NULL) {
				if ((r > 1) && (m_couple2 != NULL)) dbg->log(0, "ERRRO: CEFuncBondarev invalid structure", this);
				res = new CEFuncBondarev(m_No, other->base(), m_couple2, NULL, true, index(2), covariance(2));
			} else {
				if ((r > 1) || (m_couple2 != NULL)) dbg->log(0, "ERRRO: CEFuncBondarev invalid structure", this);
				res = new CEFuncBondarev(m_No, m_couple1, other->base(), NULL, true, NULL, true);
			}
			copyAttributesTo(res,true);
			return res;
		}
		if (r>1) {
			if (index(2)->equals(o_index)) {
				if (covariance(2) == o_cov) dbg->log(0, "<Warning><covariance() == other->covariance(1)>", this, other->ThisExpr());
				CExpression* res = NULL;
				res = new CEFuncBondarev(m_No, NULL, other->base(), index(1), covariance(1), NULL, true);
				copyAttributesTo(res,true);
				return res;
			}
		}
		return NULL;
	}

	if (o_CN == CEFuncBondarev::CLASS_NAME) { // o_rank>0
		CEFuncBondarev* o_func = (CEFuncBondarev*)o_e;
		if (!o_func->check_struture()) CEException::halt(); //TODO: remove this check after debug
		int o_No = o_func->GetNo();
		int d=0;int o_d=0;
		int _No=m_No; if (_No>4) { d=4; _No-=4; }
		if (o_No>4) { o_d=4; o_No-=4; }

		CExpression* res = NULL;
		CExpression* ind1 = GetIndexLogical(1); bool cov1 = GetCovLogical(1);
		CExpression* o_ind1 = o_func->GetIndexLogical(1); bool o_cov1 = o_func->GetCovLogical(1);
		if ((d==o_d) && (ind1 != NULL) && (o_ind1 != NULL)) {
			if (ind1->equals(o_ind1)) {
				if (cov1 == o_cov1) dbg->log(0, "<Warning><covariance() == other->covariance()>", this, other->ThisExpr());
				switch(m_No) {
				case 1:
					if(o_No==3) { res=new CEFuncBondarev(3+d,m_couple2,o_func->GetCouple2(),GetIndexLogical(2),GetCovLogical(2),o_func->GetIndexLogical(2),o_func->GetCovLogical(2)); }
					if(o_No==4) { res=new CEFuncBondarev(4+d,m_couple2,o_func->GetCouple2(),GetIndexLogical(2),GetCovLogical(2),o_func->GetIndexLogical(2),o_func->GetCovLogical(2)); }
					break;
				case 2:
					if(o_No==3) { res=new CEFuncBondarev(2+d,o_func->GetCouple2(),m_couple2,o_func->GetIndexLogical(2),o_func->GetCovLogical(2),GetIndexLogical(2),GetCovLogical(2)); }
					if(o_No==4) { res=new CEFuncBondarev(1+d,m_couple2,o_func->GetCouple2(),GetIndexLogical(2),GetCovLogical(2),o_func->GetIndexLogical(2),o_func->GetCovLogical(2)); res->inverseFlag(); }
					break;
				case 3:
					if(o_No==1) { res=new CEFuncBondarev(1+d,m_couple2,o_func->GetCouple2(),GetIndexLogical(2),GetCovLogical(2),o_func->GetIndexLogical(2),o_func->GetCovLogical(2)); }
					if(o_No==2) { res=new CEFuncBondarev(2+d,m_couple2,o_func->GetCouple2(),GetIndexLogical(2),GetCovLogical(2),o_func->GetIndexLogical(2),o_func->GetCovLogical(2)); }
					break;
				case 4:
					if(o_No==1) { res=new CEFuncBondarev(4+d,m_couple2,o_func->GetCouple2(),GetIndexLogical(2),GetCovLogical(2),o_func->GetIndexLogical(2),o_func->GetCovLogical(2)); res->inverseFlag(); }
					if(o_No==2) { res=new CEFuncBondarev(3+d,m_couple2,o_func->GetCouple2(),GetIndexLogical(2),GetCovLogical(2),o_func->GetIndexLogical(2),o_func->GetCovLogical(2)); res->inverseFlag(); }
					break;
				}
				if (res != NULL) { //*2 all
					res = res->multiply(CENumber::GetTwo());
				} else {
					res = CENumber::GetZero();
				}
				copyAttributesTo(res, true);
				return res;
			}
		}
		CExpression* o_ind2 = o_func->GetIndexLogical(2); bool o_cov2 = o_func->GetCovLogical(2);
		if ((d==o_d) && (ind1 != NULL) && (o_ind2 != NULL)) {
			if (ind1->equals(o_ind2)) {
				if (cov1 == o_cov2) dbg->log(0, "<Warning><covariance() == other->covariance()>", this, other->ThisExpr());
				switch(m_No) {
				case 1:
					if(o_No==1){res=new CEFuncBondarev(3+d,m_couple2,o_func->GetCouple1(),GetIndexLogical(2),GetCovLogical(2),o_ind1,o_cov1);}
					if(o_No==4){res=new CEFuncBondarev(4+d,m_couple2,o_func->GetCouple1(),GetIndexLogical(2),GetCovLogical(2),o_ind1,o_cov1);res->inverseFlag();}
					break;
				case 2:
					if(o_No==4){res=new CEFuncBondarev(1+d,m_couple2,o_func->GetCouple1(),GetIndexLogical(2),GetCovLogical(2),o_ind1,o_cov1);}
					if(o_No==1){res=new CEFuncBondarev(2+d,m_couple2,o_func->GetCouple1(),GetIndexLogical(2),GetCovLogical(2),o_ind1,o_cov1);res->inverseFlag();}
					break;
				case 3:
					if(o_No==3){res=new CEFuncBondarev(1+d,m_couple2,o_func->GetCouple1(),GetIndexLogical(2),GetCovLogical(2),o_ind1,o_cov1);}
					if(o_No==2){res=new CEFuncBondarev(2+d,m_couple2,o_func->GetCouple1(),GetIndexLogical(2),GetCovLogical(2),o_ind1,o_cov1);res->inverseFlag();}
					break;
				case 4:
					if(o_No==2){res=new CEFuncBondarev(3+d,m_couple2,o_func->GetCouple1(),GetIndexLogical(2),GetCovLogical(2),o_ind1,o_cov1);}
					if(o_No==3){res=new CEFuncBondarev(4+d,m_couple2,o_func->GetCouple1(),GetIndexLogical(2),GetCovLogical(2),o_ind1,o_cov1);res->inverseFlag();}
					break;
				}
				if (res != NULL) { //*2 all
					res = res->multiply(CENumber::GetTwo());
				} else {
					res = CENumber::GetZero();
				}
				copyAttributesTo(res, true);
				return res;
			}
		}
		CExpression* ind2 = GetIndexLogical(2); bool cov2 = GetCovLogical(2);
		if ((d==o_d) && (ind2 != NULL) && (o_ind1 != NULL)) {
			if (ind2->equals(o_ind1)) {
				if (cov2 == o_cov1) dbg->log(0, "<Warning><covariance() == other->covariance()>", this, other->ThisExpr());
				switch(m_No) {
				case 1:
					if(o_No==1){res=new CEFuncBondarev(1+d,m_couple1,o_func->GetCouple2(),ind1,cov1,o_ind2,o_cov2);}
					if(o_No==2){res=new CEFuncBondarev(2+d,m_couple1,o_func->GetCouple2(),ind1,cov1,o_ind2,o_cov2);}
					break;
				case 2:
					if(o_No==3){res=new CEFuncBondarev(2+d,m_couple1,o_func->GetCouple2(),ind1,cov1,o_ind2,o_cov2);}
					if(o_No==4){res=new CEFuncBondarev(1+d,m_couple1,o_func->GetCouple2(),ind1,cov1,o_ind2,o_cov2);}
					break;
				case 3:
					if(o_No==3){res=new CEFuncBondarev(3+d,m_couple1,o_func->GetCouple2(),ind1,cov1,o_ind2,o_cov2);}
					if(o_No==4){res=new CEFuncBondarev(4+d,m_couple1,o_func->GetCouple2(),ind1,cov1,o_ind2,o_cov2);}
					break;
				case 4:
					if(o_No==1){res=new CEFuncBondarev(4+d,m_couple1,o_func->GetCouple2(),ind1,cov1,o_ind2,o_cov2);}
					if(o_No==2){res=new CEFuncBondarev(3+d,m_couple1,o_func->GetCouple2(),ind1,cov1,o_ind2,o_cov2);}
					break;
				}
				if (res != NULL) { //*2 all
					res = res->multiply(CENumber::GetTwo());
				} else {
					res = CENumber::GetZero();
				}
				copyAttributesTo(res, true);
				return res;
			}
		}
		if ((d==o_d) && (ind2 != NULL) && (o_ind2 != NULL)) {
			if (ind2->equals(o_ind2)) {
				if (cov2 == o_cov1) dbg->log(0, "<Warning><covariance() == other->covariance()>", this, other->ThisExpr());
				switch(m_No) {
				case 1:
					if(o_No==3){res=new CEFuncBondarev(1+d,m_couple1,o_func->GetCouple1(),ind1,cov1,o_ind1,o_cov1);}
					if(o_No==2){res=new CEFuncBondarev(2+d,m_couple1,o_func->GetCouple1(),ind1,cov1,o_ind1,o_cov1);res->inverseFlag();}
					break;
				case 2:
					if(o_No==1){res=new CEFuncBondarev(2+d,m_couple1,o_func->GetCouple1(),ind1,cov1,o_ind1,o_cov1);}
					if(o_No==4){res=new CEFuncBondarev(1+d,m_couple1,o_func->GetCouple1(),ind1,cov1,o_ind1,o_cov1);res->inverseFlag();}
					break;
				case 3:
					if(o_No==1){res=new CEFuncBondarev(3+d,m_couple1,o_func->GetCouple1(),ind1,cov1,o_ind1,o_cov1);}
					if(o_No==4){res=new CEFuncBondarev(4+d,m_couple1,o_func->GetCouple1(),ind1,cov1,o_ind1,o_cov1);res->inverseFlag();}
					break;
				case 4:
					if(o_No==3){res=new CEFuncBondarev(4+d,m_couple1,o_func->GetCouple1(),ind1,cov1,o_ind1,o_cov1);}
					if(o_No==2){res=new CEFuncBondarev(3+d,m_couple1,o_func->GetCouple1(),ind1,cov1,o_ind1,o_cov1);res->inverseFlag();}
					break;
				}
				if (res != NULL) { //*2 all
					res = res->multiply(CENumber::GetTwo());
				} else {
					res = CENumber::GetZero();
				}
				copyAttributesTo(res, true);
				return res;
			}
		}
	}
	return NULL;
}
