////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EGamma.cpp: implementation of the CEGamma class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

CExpression* CEGamma::m_SymGamma = NULL;
const CExpression* CEGamma::GetBaseSymGamma() {
	if (m_SymGamma==NULL) {
		m_SymGamma = CEProcessor::getDataStorage()->gete(CConst::GL_GAMMA);
		if (m_SymGamma==NULL) { m_SymGamma=new CESymbol("GAMMAERR"); CEProcessor::AddToDeleteList(m_SymGamma); }
	}
	return m_SymGamma;
}
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

const char* CEGamma::CLASS_NAME	= "CEGamma";

CEGamma::CEGamma(const CExpression* index, bool covariance) :CEVector(CEGamma::GetBaseSymGamma(), index, covariance)
{
}

CEGamma::~CEGamma()
{

}

void CEGamma::collectProperties(CEProperties* props, unsigned set)
{
	CETensor::collectProperties(props, set);
	props->setCommutative(false);
}

unsigned long CEGamma::props_long()
{
	unsigned long res = CETensor::props_long();
	res = CEProperties::set_property_to(res, CEProperties::PROP_MASK_COMMUTATIVE, false);
	return res;
}

const char* CEGamma::ClassName() const
{
	return CEGamma::CLASS_NAME;
}

CExpression* CEGamma::clone() const
{
	CExpression* res = new CEGamma(GetIndex(1), covariance());
	copyAttributesTo(res);
	return res;
}

CExpression* CEGamma::unite_with(IETensor *other, bool other_is_at_right, bool bCalledByOther)
{
	if (other == NULL) { dbg->log(0, "<Warning><unite_with : NULL parameter.>"); return NULL; }
	if (other->rank() == 1) {
		if(!CTIndex::CheckCoupled(GetTIndex(1), other->GetTIndex(1)))return NULL;
		CExpression* res = NULL;
		CExpression* o_e = other->ThisExpr();
		const char* o_CN = o_e->ClassName();
		if (o_CN == CEVector::CLASS_NAME) {
			res = new CESGammaProduct(other->base());
		} else if (o_CN == CEGamma::CLASS_NAME) {
			res = new CEMetric(index(), other->index(1), covariance(), other->covariance(1));
		}
		if (res != NULL) {
			if (!getFlag()) res->inverseFlag();
			if (!o_e->getFlag()) res->inverseFlag();
			return res;
		}
	} else if(!bCalledByOther) {
		return other->unite_with(this, !other_is_at_right, true);
	}
	return NULL;
}

CExpression* CEGamma::exchange_subst(IETensor* other, bool other_is_at_right, bool bCalledByOther) 
{
	//Note: CETensor::exchange_subst() must be called LAST!
	CExpression* eOther = other->ThisExpr();

	//1. Commutation to Spinor
	if (eOther->getProperty(CEProperties::PROP_MASK_IS_SPINOR)) {
		return NULL;
	}

	/*
	*	Old: try unition
	*	if(!bCalledByOther){ CExpression* res=unite_with(other,other_is_at_right); if(res!=NULL)return res; }
	*/

	//2. Commutation to CEGamma & CESGammaProduct
	const char* cnOther=eOther->ClassName();
	if ((cnOther==ClassName()) || //==CEGamma::CLASS_NAME
		(cnOther==CESGammaProduct::CLASS_NAME)) {
		CExpression* res = NULL;
		if(cnOther==CESGammaProduct::CLASS_NAME){
			res = new CEVector(((CESGammaProduct*)eOther)->getBaseExpression(),index(), covariance());
		}else{ //==CEGamma::CLASS_NAME
			res = new CEMetric(index(),((CEGamma*)eOther)->index(),covariance(),((CEGamma*)eOther)->covariance());
		}
		if(!getFlag())res->inverseFlag(); if(!eOther->getFlag())res->inverseFlag();
		res=res->multiply(CENumber::GetTwo());
		res=res->minus(other_is_at_right?(eOther->clone()->multiply(clone())):(clone()->multiply(eOther->clone())));
		return res;
	}

	return CETensor::exchange_subst(other,other_is_at_right,bCalledByOther);
}
