////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EMatrix.cpp: implementation of the CEMatrix class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

const char* CEMatrix::CLASS_NAME	= "CEMatrix";

CEMatrix::CEMatrix(const CExpression* base,
		const CExpression* index1, const CExpression* index2,
		bool covariance1, bool covariance2, int _symm):CETensor(base)
{
	m_indices->add(CTIndex::CreateVectorIndex(index1, covariance1));
	m_indices->add(CTIndex::CreateVectorIndex(index2, covariance2));
	m_Symmetry=_symm;
}

CEMatrix::CEMatrix(const CExpression* base,
				   const CTIndex* index1, const CTIndex* index2,
				   int _symm) :CETensor(base)
{
	if((index1==NULL) || (index2==NULL))CEException::halt("CEMatrix::CEMatrix(NULL index)");
	m_indices->add(index1->clone());
	m_indices->add(index2->clone());
	m_Symmetry=_symm;
}

CEMatrix::~CEMatrix()
{
}

CExpression* CEMatrix::clone() const
{
	CExpression* res = new CEMatrix(GetBase(), GetIndex(1), GetIndex(2), covariance(1), covariance(2), m_Symmetry);
	copyAttributesTo(res);
	return res;
}

CEString* CEMatrix::toString(int mode)
{
	CEString* res = NULL;
	switch (mode)
	{
	case STORE_WR_MATHEMATICA:
		//////////////////
		if (getFlag()) res = new CEString("\\(");
		else res = new CEString("- \\(");
		if (covariance(1) == covariance(2)) {
			res->AddString("\\(");
			res->AddString(m_base->toString(mode));
			res->AddString("\\)");

			res->AddString((covariance(1))?"\\_":"\\&");
			res->AddString("\\(");
			res->AddString(index(1)->toString(mode));
			res->AddString("\\ ");
			res->AddString(index(2)->toString(mode));
			res->AddString("\\)");
		} else {
			res->AddString("\\(");
			res->AddString("\\(");
			res->AddString(m_base->toString(mode));
			res->AddString("\\)");

			res->AddString((covariance(1))?"\\_":"\\&");
			res->AddString(index(1)->toString(mode));
			res->AddString("\\)");
			res->AddString((covariance(2))?"\\_":"\\&");
			res->AddString(index(2)->toString(mode));
		}
		res->AddString("\\)");
		break;
	default:
		res = new CEString("<Not supported data format.>");
	}
	return res;
}

CExpression* CEMatrix::Arrange(int level) {
	if (m_Symmetry == 1) { // for symmetric ones
		if (index(2)->less_than(index(1))) exchange_indices(1, 2);
	}
	return this;
}

CExpression* CEMatrix::unite_with(IETensor *other, bool other_is_at_right, bool bCalledByOther)
{
	return NULL;
}

const char* CEMatrix::ClassName() const
{
	return CLASS_NAME;
}
