////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EMetric.cpp: implementation of the CEMetric class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

CExpression* CEMetric::m_MetricBase	= NULL;
const CExpression* CEMetric::GetMetricBase() {
	if (m_MetricBase==NULL) {
		m_MetricBase = CEProcessor::getDataStorage()->gete(CConst::DS_METRIC_BASE);
		// Note: m_MetricBase will be deleted with DataStorage
		if (m_MetricBase==NULL) { m_MetricBase = new CESymbol("g"); CEProcessor::AddToDeleteList(m_MetricBase); }
	}
	return m_MetricBase;
}
CExpression* CEMetric::m_refDeltaSym=NULL; // to cache dataStorageCalls;
CExpression* CEMetric::GetDeltaSymRef() {
	if (m_refDeltaSym==NULL) {
		m_refDeltaSym = CEProcessor::getDataStorage()->gete(CConst::GL_DELTA);
		if (m_refDeltaSym==NULL) { m_refDeltaSym= new CESymbol("delta"); CEProcessor::AddToDeleteList(m_refDeltaSym); }
	}
	return m_refDeltaSym;
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
const char* CEMetric::CLASS_NAME	= "CEMetric";

CEMetric::CEMetric(const CExpression* index1, const CExpression* index2,
				   bool covariance1, bool covariance2)
				   :CEMatrix(GetMetricBase(), index1, index2, covariance1, covariance2)
{
}

CEMetric::CEMetric(const CTIndex* index1, const CTIndex* index2)
:CEMatrix(GetMetricBase(), index1, index2)
{
}

CEMetric::~CEMetric()
{
}

CExpression* CEMetric::clone() const
{
	CExpression* res = new CEMetric(GetTIndexConst(1),GetTIndexConst(2));
	copyAttributesTo(res);
	return res;
}

CEString* CEMetric::toString(int mode)
{
	CEString* res = NULL;
	switch (mode)
	{
	case STORE_WR_MATHEMATICA: {
		//////////////////
		const CTIndex* ti1=GetTIndexConst(1); const CTIndex* ti2=GetTIndexConst(2);
		if (getFlag()) res = new CEString("\\(");
		else res = new CEString("- \\(");
		if (ti1->covariance() == ti2->covariance()) {
			res->AddString("\\(");
			if(ti1->IsCovarianceUsed()) {
				res->AddString(m_base->toString(mode));
			}else{
				res->AddString(CEMetric::GetDeltaSymRef()->toString(mode));	// \delta_{ij}
			}
			res->AddString("\\)");

			res->AddString(ti1->covariance()?"\\_":"\\&");
			res->AddString("\\(");
			res->AddString(index(1)->toString(mode));
			res->AddString("\\ ");
			res->AddString(index(2)->toString(mode));
			res->AddString("\\)");
		} else {
			res->AddString("\\(");
			if (m_base->getFlag()) res->AddString("\\(");
			else res->AddString("\\(- ");
			res->AddString(CEMetric::GetDeltaSymRef()->toString(mode));
			res->AddString("\\)");

			res->AddString(ti1->covariance()?"\\_":"\\&");
			res->AddString(index(1)->toString(mode));
			res->AddString("\\)");
			res->AddString(ti2->covariance()?"\\_":"\\&");
			res->AddString(index(2)->toString(mode));
		}
		res->AddString("\\)");
							   } break;
	default:
		res = new CEString("<Not supported data format.>");
	}
	return res;
}

CExpression* CEMetric::unite_with(IETensor *other, bool other_is_at_right, bool bCalledByOther) {
	if (other == NULL) { dbg->log(0, "<Warning><CEVector::unite_with : NULL parameter.>"); return NULL; }

	int other_rank = other->rank();
	CExpression* o_e = other->ThisExpr();
	for(int i=0; i<other_rank; i++) {
		CTIndex* tindex = other->GetTIndex(i+1);
		if(CTIndex::CheckCoupled(tindex,GetTIndex(1))) {
			CExpression* res = o_e->clone();
			IETensor* tens_res = IETensor::GetIETensor(res);
			tens_res->changeIndex(i+1, index(2), covariance(2));
			if (!getFlag()) res->inverseFlag();

			res = CExpression::arrge(res);
			//checking for e_{a}^{a}, -- it can be coupled to another "e" before arranging,
			//that may lead to avoidable delay
			return res;
		}
		else if (CTIndex::CheckCoupled(tindex,GetTIndex(2))) {
			CExpression* res = o_e->clone();
			IETensor* tens_res = IETensor::GetIETensor(res);
			tens_res->changeIndex(i+1, index(1), covariance(1));
			if (!getFlag()) res->inverseFlag();

			res = CExpression::arrge(res);
			//checking for e_{a}^{a}, -- it can be coupled to another "e" before arranging,
			//that may lead to avoidable delay
			return res;
		}
	}
	//no calls to other->unite_with() - no other couplings possible
	return NULL;
}

const char* CEMetric::ClassName() const { return CLASS_NAME; }

CExpression* CEMetric::Arrange(int level)
{
	if (index(2)->less_than(index(1))) {
		exchange_indices(1, 2);
	}

	if (level == AL_EXTRACT) {
		if(CTIndex::CheckCoupled(GetTIndex(1),GetTIndex(2))) {
			//Note: here must be the ONLY place to invlove the dimension of tensor!
			CExpression* res = GetNDimExpr(1);
			copyAttributesTo(res, true);
			return res;
		}
	}
	return this;
}

CExpression* CEMetric::expand(bool recursive)
{
	//return Arrange(AL_EXTRACT);
	return this;
}

bool CEMetric::less_than(CExpression *other) {
	if (other == NULL) return false;

	// compare to CESymbol, CEPower, CESProduct, CEVector, CEMetric, CEAntisymmetric, CEAntiSymmProd, CEFuncBondarev
	// a < a^n < a.b < a_nu < a_mu_nu < a_abcd
	const char* o_cn = other->ClassName();
	if (o_cn == CEPower::CLASS_NAME) {
		if (!CEUtil::LEQU(getFlag(), other->getFlag())) return other->getFlag(); // compare sign: "-"sym is < than "sym"
		if (base()->less_than(((CEPower*)other)->basis())) return true;
		return false;
	}

	if (o_cn == CESymbol::CLASS_NAME) {
		if (!CEUtil::LEQU(getFlag(), other->getFlag())) return other->getFlag(); // compare sign: "-"sym is < than "sym"
		if (base()->less_than(other)) return true;
		return false;
	}

	if (o_cn == CESProduct::CLASS_NAME) {
		if (!CEUtil::LEQU(getFlag(), other->getFlag())) return other->getFlag(); // compare sign: "-"sym is < than "sym"
		if (base()->less_than(((CESProduct*)other)->get_a())) return true;
		return false;
	}

	if ((o_cn==CEVector::CLASS_NAME)||(o_cn==CEMetric::CLASS_NAME)||(o_cn==CEAntisymmetric::CLASS_NAME)) {
		if (!CEUtil::LEQU(getFlag(), other->getFlag())) return other->getFlag(); // compare sign: "-"sym is < than "sym"
		return TLessThen(IETensor::GetIETensor(other), o_cn);
	}

	if (o_cn==CEAntiSymmProd::CLASS_NAME) {
		if (!CEUtil::LEQU(getFlag(), other->getFlag())) return other->getFlag(); // compare sign: "-"sym is < than "sym"
		return true;
	}

	if (o_cn==CEFuncBondarev::CLASS_NAME) {
		if (!CEUtil::LEQU(getFlag(), other->getFlag())) return other->getFlag(); // compare sign: "-"sym is < than "sym"
		return true;
	}

	return CExpression::less_than(other);
}
