////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ERank4Tensor.cpp: implementation of the CERank4Tensor class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CERank4Tensor::CERank4Tensor(const CExpression* _base,
		const CExpression* index1, bool covariance1,
		const CExpression* index2, bool covariance2,
		const CExpression* index3, bool covariance3,
		const CExpression* index4, bool covariance4, int symm12, int symm34) : CETensor(_base)
{
	m_indices->add(CTIndex::CreateVectorIndex(index1, covariance1));
	m_indices->add(CTIndex::CreateVectorIndex(index2, covariance2));
	m_indices->add(CTIndex::CreateVectorIndex(index3, covariance3));
	m_indices->add(CTIndex::CreateVectorIndex(index4, covariance4));
	m_Symmetry12 = symm12;
	m_Symmetry34 = symm34;
}

CERank4Tensor::~CERank4Tensor() {}

const char* CERank4Tensor::CLASS_NAME = "CERank4Tensor";
const char* CERank4Tensor::ClassName() const { return CLASS_NAME; }


CExpression* CERank4Tensor::clone() const {
	CExpression* res = new CERank4Tensor(GetBase(),
		GetIndex(1), covariance(1),
		GetIndex(2), covariance(2),
		GetIndex(3), covariance(3),
		GetIndex(4), covariance(4),
		m_Symmetry12, m_Symmetry34);
	copyAttributesTo(res);
	return res;
}

CExpression* CERank4Tensor::Arrange(int level) {
	if (m_Symmetry12 == 1) { // for symmetric ones
		if (index(2)->less_than(index(1))) exchange_indices(1, 2);
	}
	if (m_Symmetry34 == 1) { // for symmetric ones
		if (index(4)->less_than(index(3))) exchange_indices(3, 4);
	}
	return this;
}
