////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ESGammaProduct.cpp: implementation of the CESGammaProduct class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

const char* CESGammaProduct::CLASS_NAME	= "CESGammaProduct";

CESGammaProduct::CESGammaProduct(const CExpression* v) :CESProduct(CEGamma::GetBaseSymGamma(), v) {
}

CESGammaProduct::~CESGammaProduct()
{
}

void CESGammaProduct::collectProperties(CEProperties* props, unsigned set) {
	CESProduct::collectProperties(props, set);
	props->setCommutative(false);
}

unsigned long CESGammaProduct::props_long() {
	unsigned long res = CESProduct::props_long();
	res = CEProperties::set_property_to(res, CEProperties::PROP_MASK_COMMUTATIVE, false);
	return res;
}

CEString* CESGammaProduct::toString(int mode)
{
	CEString* res = NULL;
	switch (mode)
	{
	case STORE_WR_MATHEMATICA:
		/*if (m_b->getFlag()) res = m_b->toString(mode);
		else {
			res = new CEString("\\(- ");
			res->AddString(m_b->toString(mode));
			res->AddString("\\)");
		}
		res->AddString("\\&^");*/
		res = new CEString((getFlag()?"\\(":"- \\("));
		//if (!m_b->getFlag()) res->AddString("- ");
		res->AddString(m_b->toString(mode));
		res->AddString("\\)\\&^");
		break;
	default:
		res = new CEString("<Not supported data format.>");
	}
	return res;
}

const char* CESGammaProduct::ClassName() const
{
	return CESGammaProduct::CLASS_NAME;
}

CExpression* CESGammaProduct::getBaseExpression()
{
	return m_b;
}

CExpression* CESGammaProduct::expand(bool recursive)
{
	if (recursive) {m_b = CExpression::reduce(m_b);}
	//Arrange(AL_INLINE);

	// arrange sign
	if (!m_b->getFlag()) { m_b->inverseFlag(); inverseFlag(); }

	if (m_b->getProperty(CEProperties::PROP_MASK_IS_SUM)) {
		CESum* base = (CESum*)m_b;
		if (!base->getFlag()) CEException::halt("CESGammaProduct::expand() unexpected sign");
		CESum* res = new CESum();
		CESGammaProduct* temp = NULL;
		long size = base->size();
		long i;
		for(i=0; i<size; i++) {
			temp = new CESGammaProduct(base->GetAt(i));
			if (recursive) res->addItem(CExpression::reduce(temp));
			else res->addItem(temp);
		}
		copyAttributesTo(res, true);
		return res;
	}
	return this;
}

CExpression* CESGammaProduct::clone() const
{
	CExpression* res = new CESGammaProduct(m_b);
	copyAttributesTo(res);
	return res;
}

bool CESGammaProduct::equals(CExpression *other)
{
	if (other == NULL) return false;
	bool res = false;
	if ( (CEUtil::equals(ClassName(), other->ClassName())) && (attributes_equals_to(other)) ) {
		CEString* a = toString();
		CEString* b = other->toString();
		res = a->equals(b);
		delete a;
		delete b;
	}
	return res;
}

CExpression* CESGammaProduct::exchange_subst(IETensor* other, bool other_is_at_right, bool bCalledByOther)
{
	//Note: CETensor::exchange_subst() must be called LAST!

	//1. Commutation to CESGammaProduct
	CExpression* eOther = other->ThisExpr();
	if(eOther->ClassName()==ClassName()) { //==CESGammaProduct::CLASS_NAME
		CExpression* res = new CESProduct(getBaseExpression(),((CESGammaProduct*)eOther)->getBaseExpression());
		res = res->multiply(CENumber::GetTwo());
		if(!getFlag())res->inverseFlag(); if(!eOther->getFlag())res->inverseFlag();
		res = res->minus(other_is_at_right?(eOther->clone()->multiply(clone())):(clone()->multiply(eOther->clone())));
		return res;
	}

	return CETensor::exchange_subst(other,other_is_at_right,bCalledByOther);
}

CExpression* CESGammaProduct::Arrange(int level)
{
	m_b = CExpression::arrge(m_b);

	if (!m_b->getFlag()) { m_b->inverseFlag(); inverseFlag(); }
	return this;
}

CExpression* CESGammaProduct::unite_with(IETensor * other, bool other_is_at_right, bool bCalledByOther)
{
	if (other == NULL) { dbg->log(0, "<Warning><CESGammaProduct::unite_with : NULL parameter.>"); return NULL; }

	CExpression* o_e = other->ThisExpr();
	if ((o_e->getProperty(CEProperties::PROP_MASK_IS_SPINOR)) && (!bCalledByOther)) {
		return other->unite_with(this, !other_is_at_right, true);
	}

	if (CEUtil::equals(o_e->ClassName(), CESGammaProduct::CLASS_NAME)) {
		CESGammaProduct* o = (CESGammaProduct*)o_e;
		
		if (!getBaseExpression()->equals(o->getBaseExpression())) return NULL;

		CExpression* res = CEVector::square(getBaseExpression());
		if (getFlag() != o_e->getFlag()) res->inverseFlag();
		return res;
	}
	return NULL;
}
