////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EFuncBondarev.h: interface for the CEFuncBondarev class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EFUNCBONDAREV_H__6DC00FD5_43EA_4725_9297_C0923E4C7428__INCLUDED_)
#define AFX_EFUNCBONDAREV_H__6DC00FD5_43EA_4725_9297_C0923E4C7428__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CEFuncBondarev : public CETensor  
{
	CExpression* m_couple1;
	CExpression* m_couple2;
	int m_No;
public:
	static CDebug* dbg;

	// settings
	static bool m_DoExpandF3; // Expand F3->gamma-F1

	static CExpression* m_FuncBondarevSym; // "F"
	static CExpression* GetFuncBondarevSym();

	static CESymbol* m_BondarevBaseQP; // "q+"
	static CESymbol* m_BondarevBaseQM; // "q-"
	static CESymbol* m_BondarevBaseEP; // "e+"
	static CESymbol* m_BondarevBaseEM; // "e-"
	static CESymbol* GetBondarevBaseSym(int i); //{i: 0(q+),1(q-),2(e+),3(e-)}
	static int FindBondarevBaseSyms(CExpression* sym); // -1 if failed
	static void SetBondarevBaseSyms(CESymbol* qP,CESymbol* qM,CESymbol* eP,CESymbol* eM); //{q+,q-,e+,e-}
	static void SetBondarevBaseSymsDefaults(bool _reset=false);

	CEFuncBondarev(int _No, const CExpression* couple1, const CExpression* couple2, const CExpression* index1, bool cov1, const CExpression* index2, bool cov2);
	virtual ~CEFuncBondarev();

	CExpression* GetCouple1();
	CExpression* GetCouple2();
	int GetNo();
	CExpression* GetIndexLogical(int i); // Index for LOGICAL pos I, index(i) or NULL, no errors
	bool GetCovLogical(int i); // coariance for LOGICAL pos I or ANY, no errors
	bool check_struture();

	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual CExpression* clone() const;
	virtual CEString* toString(int mode = 0);
	virtual bool contain_vector(void * obj);
	virtual CExpression* applyRule(CEAbstractRule* rule);
	virtual bool equals(CExpression* other);
	virtual bool less_than(CExpression* other);

	virtual CExpression* expand(bool recursive = true);
	virtual CExpression* Arrange(int level);
	virtual CExpression* unite_with(IETensor* other, bool other_is_at_right=true, bool bCalledByOther=false);

	virtual long store_stream(ofstream* p_ofs, DWORD settings);
	virtual const char* get_TAG_Name();
	static const char* CLASS_TAG_NAME;
	static CESerializable* restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings);
};

#endif // !defined(AFX_EFUNCBONDAREV_H__6DC00FD5_43EA_4725_9297_C0923E4C7428__INCLUDED_)
