////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ETensor.h: interface for the CETensor class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ETENSOR_H__BF8FC201_C3EE_11D5_90CB_00403393DD18__INCLUDED_)
#define AFX_ETENSOR_H__BF8FC201_C3EE_11D5_90CB_00403393DD18__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CETensor : public IETensor, public CExpression
{
protected:
	CExpression* m_base;
	CEPtrArray* m_indices;

public:
	bool contain_index(CExpression* ind);
	static CDebug* dbg;
	virtual bool contain_vector(void * obj);
	virtual CExpression* applyRule(CEAbstractRule* rule);
	virtual CEString* toString(int mode = 0);

	CETensor(const CExpression* base);
	virtual ~CETensor();

	// IETensor interface <BEGIN>
	virtual CExpression* ThisExpr(); // returns this as (CExpression*)
	virtual CExpression* base();
	virtual const CExpression* GetBase() const;
	virtual bool setBase(CExpression* _new);
	virtual int rank() const;
	virtual CExpression* index(int i); // 1..rank
	virtual const CExpression* CETensor::GetIndex(int i) const; // 1..rank
	virtual CTIndex* GetTIndex(int i); // 1..rank
	virtual const CTIndex* GetTIndexConst(int i) const; // 1..rank
	virtual bool covariance(int i) const;
	virtual bool changeIndex(int i, CExpression* index, bool covariance);
	virtual bool liftIndex(int i, bool to_covariance);
	virtual bool setIndex(int i, CExpression* index);
	virtual CExpression* exchange_subst(IETensor* other, bool other_is_at_right, bool bCalledByOther=false);
	virtual CExpression* unite_with(IETensor* other, bool other_is_at_right=true, bool bCalledByOther=false);
	// IETensor interface <END>
	virtual void* GetInterface(long interface_ID);

	virtual bool equals(CExpression* other);
	virtual bool less_than(CExpression* other);			//asdfg: newly inserted --> test if used or not!

	// changes the UNIQUE-SYMBOL tensors indices in the expression
	// (except those declared in protected_syms array) to another uniques
	// does not affects protected_syms list
	static CExpression* ChangeVolatileIndices(CExpression* src, CEPtrArray* protected_syms = NULL);
protected:
	void exchange_indices(int i, int j);
	virtual void collectProperties(CEProperties* props, unsigned set = CEProperties::PROP_MASK_ALL);
public:
	virtual unsigned long props_long();

protected:
	bool TLessThen(IETensor* o, const char* o_cn);

	// dimension settings
protected:
	static bool m_IsNDimensionSpace;
private:
	static CExpression* m_refSymDimensionDelta; // to cache getDimensionDelta() calls
public:
	static CExpression* getDimensionDelta(); // returns not-copied expr or NULL for 4-dim space
	static void setNDimensionSpace(bool set_N); // true for N-dim, false for 4-space
	CExpression* GetNDimExpr(int i); // i:1..rank, returns copied expr, never NULL
	static bool isNDimensionSpace() { return m_IsNDimensionSpace; } // true for N-dim, false for 4-space
};

#endif // !defined(AFX_ETENSOR_H__BF8FC201_C3EE_11D5_90CB_00403393DD18__INCLUDED_)
