////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// TIndex.h: interface for the CTIndex class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TINDEX_H__1A024FDF_44BC_4BE4_9B89_92FA8A7CFA93__INCLUDED_)
#define AFX_TINDEX_H__1A024FDF_44BC_4BE4_9B89_92FA8A7CFA93__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define CETensor_NDIM_4orN		-1		//means 4- or N-dim depending on the 'isNDimensionSpace()'

class CTIndex : public CEObject
{
private:
	CExpression* m_index;
	bool m_covariance;
	bool m_bUseCovariance;
	bool m_bMinkowskySpace;
	int m_nDim;			//'CETensor_NDIM_4orN' means 4- or N-dim depending on the 'CETensor::isNDimensionSpace()'
public:
	CTIndex(int nDim, bool bMinkowskySpace, const CExpression* index, bool covariance, bool bUseCovariance);
	virtual ~CTIndex();
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual bool contain_vector(void * obj);
	virtual char* toLine(int mode = 0);
	CTIndex* clone() const;

	//shorter constructors
public:
	static inline CTIndex* CreateVectorIndex(const CExpression* index, bool covariance) { return new CTIndex(CETensor_NDIM_4orN, true, index, covariance, true); }
	static inline CTIndex* CTIndex::CreateMatrixIndex(const CExpression* index, int nDim) { return new CTIndex(nDim, false, index, true, false); }

	//index data access
public:
	CExpression* index() { return m_index; }
	const CExpression* GetIndex() const { return m_index; }
	bool covariance() const { return m_covariance; }
	bool IsCovarianceUsed() const { return m_bUseCovariance; }
	bool IsMinkowskySpace() const { return m_bMinkowskySpace; }
	bool equalsIndex(CTIndex* other);
	bool changeIndex(CExpression* index, bool covariance);
	bool liftIndex(bool to_covariance);
	bool setIndex(CExpression* index);

	//dimension data
public:
	int GetNDimID() const { return m_nDim; }
	CExpression* GetNDimExpr();									// returns copied expr

	//compartibility check
public:
	static bool CheckCoupled(CTIndex* index1, CTIndex* index2); //[main check] if 'index1' is coupled to 'index2'
};

#endif // !defined(AFX_TINDEX_H__1A024FDF_44BC_4BE4_9B89_92FA8A7CFA93__INCLUDED_)
