////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EBasic.h: interface for the CEBasic class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EBASIC_H__FAD666A3_B699_45E8_A063_DA6C682C70DC__INCLUDED_)
#define AFX_EBASIC_H__FAD666A3_B699_45E8_A063_DA6C682C70DC__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/*
*	CEObjRef: reference to external object
*/
class CEObjRef : public CEObject {
	CEObject* m_ref;
public:
	CEObjRef(CEObject* ref);
	CEObject* GetRef() { return m_ref; }

	virtual bool contain_vector(void * obj);
	virtual CEObject* copy() const;
};

class CEBasic : public CEObject  
{
	CEBasic();
public:
	static CDebug* dbg;
	virtual ~CEBasic();

	// library functions
	// DeleteObj: used to short 'delete x' statements before 'return'
	static void DeleteObj(CEObject* o1) { DeleteObj(1,o1); }
	static void DeleteObj(CEObject* o1, CEObject* o2)  { DeleteObj(2,o1,o2); }
	static void DeleteObj(CEObject* o1, CEObject* o2, CEObject* o3)  { DeleteObj(3,o1,o2,o3); }
	static void DeleteObj(CEObject* o1, CEObject* o2, CEObject* o3, CEObject* o4)  { DeleteObj(4,o1,o2,o3,o4); }
	static void DeleteObj(CEObject* o1, CEObject* o2, CEObject* o3, CEObject* o4, CEObject* o5)  { DeleteObj(5,o1,o2,o3,o4,o5); }
	static void DeleteObj(CEObject* o1, CEObject* o2, CEObject* o3, CEObject* o4, CEObject* o5, CEObject* o6)  { DeleteObj(6,o1,o2,o3,o4,o5,o6); }
	static void DeleteObj(CEObject* o1, CEObject* o2, CEObject* o3, CEObject* o4, CEObject* o5, CEObject* o6, CEObject* o7)  { DeleteObj(7,o1,o2,o3,o4,o5,o6,o7); }
	static void DeleteObj(int n, CEObject* obj1, ...);	//'...' is CEObject* obj2, obj3, obj4,...

	static CExpression* Congugate(CExpression* src, bool del_src); // destroys the source
	///static CExpression* Congugate(CExpression* src_const); // unaffects the source
	static CESum* convertToSum(CExpression* src); // destroys the source
	static CEProduct* convertToProduct(CExpression* src); // destroys the source
	static CExpression* inverseLevelAdv(CExpression* src); // x->Prod{x^{-1}}, destroys the source

	static bool isZero(CExpression* src);
	static bool isZeroProd(CExpression* src); // parces products
	static bool isZeroSeq(CExpression* src); // parces sequences
	static bool isUnit(CExpression* src);
	static bool isExpression(CEObject* src);
	static bool tendsZero(CExpression* src); // reduces & checks zero

	static bool equalsSignless(CExpression* a, CExpression* b);
	static bool equalsToSymbol(CExpression* a, const char* s);

	// Complexity of expressions, 'size of expression' or 'number of sub-items'
	static long CalcComplexity(CExpression* src); // uses 'CRCollectSymbols::CountObjects(src);'

	// Math expressions
	static CExpression* GetSqrt2();
	static CExpression* m_refPi; // cache for GetPiSym()
	static CExpression* GetPiSym(); // const result, never NULL

	static void appendLineBreak(CEString* str, int mode);
	static char* GetSymbolLine(int len, char sym); //TODO: move to CEUtil
	static CEString* GetSymbolString(int len, char sym); //TODO: move to CEString?
	
	static CEString* ObjToString(CESerializable* obj, int mode); // "NULL" for NULL object

	static CExpression* m_TempIndex;
	static CExpression* InverseTensorIndex(CExpression* src_const, CExpression* ind, bool to_cov); // returns NULL if fails

	//static bool LeaveSeries(CEProduct* src, CExpression* var, int order_to);
};

#endif // !defined(AFX_EBASIC_H__FAD666A3_B699_45E8_A063_DA6C682C70DC__INCLUDED_)
