////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EProcessEnvironment.cpp: implementation of the CEProcessEnvironment class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

#define DEFAULT_ENVIRONMENT	ENV_EXPAND_ALL

const long CEProcessEnvironment::ENV_EXPAND_ALL = 0x0001;

CEProcessEnvironment::CEProcessEnvironment(CEFunctionProvider* fp) : CEObject(false)
{
	m_fp = fp;
	m_env = DEFAULT_ENVIRONMENT;

	m_mass_order = -1;
	m_photon_helisity_notation = CPPhoton::PH_VECTOR_NOTATION;

	m_mass_order_info = new CEPtrArray();
	m_CALC_MASS_EXACT = false;
}

CEProcessEnvironment::~CEProcessEnvironment()
{
	delete m_mass_order_info;
}

const char* CEProcessEnvironment::CLASS_NAME = "CEProcessEnvironment";
const char* CEProcessEnvironment::ClassName() const { return CLASS_NAME; }
bool CEProcessEnvironment::contain_vector(void * obj) {
	if (m_mass_order_info == obj) return true;
	if (m_mass_order_info != NULL) { if (m_mass_order_info->contain_vector(obj)) return true; }
	return false;
}

CEFunctionProvider* CEProcessEnvironment::getFunctionProvider()
{
	return m_fp;
}

long CEProcessEnvironment::get_env()
{
	return m_env;
}

long CEProcessEnvironment::set_env(long _env)
{
	long old = m_env;
	m_env = _env;
	return old;
}

void CEProcessEnvironment::SetMassCalcOrder(int part_kind, int mass_order) {
	if (part_kind < 0) part_kind = -part_kind;

	CDiagramPhysics* phys = CDiagramPhysics::get_curr_physics();
	if (phys != NULL) {
		if (!phys->validate_part_kind(part_kind)) {
			// Note: pk could be valid after further physics re-initialization
			CEProcessor::dbg->log(0,part_kind,"WARNING. SetMassCalcOrder(illegal pk: %x). The particle is probably switched off.");
		}
	}

	long s = m_mass_order_info->size();
	long i;
	for(i=0; i<s; i++) {
		CExprContainer* cart = (CExprContainer*)m_mass_order_info->GetAt(i);
		if (cart->get_info() == part_kind) {
			if (mass_order < 0) { m_mass_order_info->RemoveAt(i, true); return; }

			if (cart->get_info(1) == mass_order) return; // nothing to change

			cart->set_info(1, mass_order);
			return;
		}
	}

	// Note: CExprContainer can also contain mass symbol
	CExprContainer* cart_new = new CExprContainer(NULL, part_kind, mass_order, (CExpression*)NULL);
	m_mass_order_info->add(cart_new);
}

int CEProcessEnvironment::GetMassCalcOrder(int part_kind) {
	if (part_kind < 0) part_kind = -part_kind;
	long s = m_mass_order_info->size();
	for(long i=0; i<s; i++) {
		CExprContainer* cart = (CExprContainer*)m_mass_order_info->GetAt(i);
		if (cart->get_info() == part_kind) return cart->get_info(1);
	}
	return -1;
}

void CEProcessEnvironment::ClearMassCalcOrders() { // cancel all the SetMassCalcOrder() calls
	m_mass_order_info->clear_all();
}

CEPtrArray* CEProcessEnvironment::GetMassOrderInfo() {
	if (m_CALC_MASS_EXACT) return NULL;
	return m_mass_order_info;
}
void CEProcessEnvironment::SetMassExactCalc() { m_CALC_MASS_EXACT = true; }
void CEProcessEnvironment::RestoreMassCalcOrders() { m_CALC_MASS_EXACT = false; } // cancels SetMassExactCalc() flag only!
