////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// APtrSeq.cpp: implementation of the CAPtrSeq class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

CAPtrSeqCart::CAPtrSeqCart(CEObject* _Obj)  : CEObject(true) {
	m_Obj=_Obj; m_Next=NULL;
}

CAPtrSeqCart::CAPtrSeqCart(CEObject* _Obj, CAPtrSeqCart* _Next)  : CEObject(true) {
	m_Obj=_Obj; m_Next=_Next;
}

CAPtrSeqCart::~CAPtrSeqCart() {
	if (m_Obj != NULL) delete m_Obj;
}

const char* CAPtrSeqCart::CLASS_NAME = "CAPtrSeqCart";
const char* CAPtrSeqCart::ClassName() const { return CLASS_NAME; }
CEObject* CAPtrSeqCart::copy() const { return new CAPtrSeqCart((m_Obj!=NULL)?(m_Obj->copy()):NULL); }
bool CAPtrSeqCart::contain_vector(void * obj) {
	if (m_Obj == obj) return true;
	if (m_Obj != NULL) {if (m_Obj->contain_vector(obj)) return true;}
	return false;
}

//////////////////////////////////////////////////////////////////////
CAPtrSeq::CASeqIterationCart::CASeqIterationCart() : CEObject(true) { m_Cart=NULL; m_Pos=-1; }
void CAPtrSeq::CASeqIterationCart::Clear() { m_Pos=-1; m_Cart=NULL; }

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CAPtrSeq::CAPtrSeq(bool allow_NULLs) : CEObject(true) {
	m_CartFirst=NULL; m_size=0; m_CartLast=NULL;
	m_allow_NULLs=allow_NULLs;
}

CAPtrSeq::CAPtrSeq(CAPtrSeqCart* _cart_first, bool allow_NULLs) : CEObject(true) {
	// copy
	CAPtrSeqCart* o_cart = _cart_first;
	m_CartFirst = NULL; m_size=0;
	m_allow_NULLs = allow_NULLs;
	m_CartLast = NULL;

	while(o_cart != NULL) {
		CEObject* obj = o_cart->m_Obj;
		if ((obj != NULL) || (m_allow_NULLs)) {
			CAPtrSeqCart* curr = new CAPtrSeqCart((obj!=NULL)?(obj->copy()):NULL);
			if(m_CartLast!=NULL){ m_CartLast->m_Next=curr; }else{ m_CartFirst=curr; }
			m_size++;
			m_CartLast=curr;
		}
		o_cart=o_cart->m_Next;
	}
}

CAPtrSeq::~CAPtrSeq() {
	CAPtrSeqCart* curr=m_CartFirst; while(curr!=NULL){ CAPtrSeqCart* c_next = curr->m_Next; delete curr; curr=c_next; }
}

const char* CAPtrSeq::CLASS_NAME = "CAPtrSeq";
const char* CAPtrSeq::ClassName() const { return CLASS_NAME; }

void CAPtrSeq::clearall_without_delete_objects() {
	CAPtrSeqCart* c=m_CartFirst; while(c!=NULL){ CAPtrSeqCart* c_next=c->m_Next; c->m_Obj=NULL; delete c; c=c_next; }
	m_CartFirst=NULL; m_size=0; m_CartLast=NULL;
	m_itInfo[0].Clear(); m_itInfo[1].Clear();
}

void CAPtrSeq::clear_all() {
	CAPtrSeqCart* c=m_CartFirst; while(c!=NULL){ CAPtrSeqCart* c_next=c->m_Next; delete c; c=c_next; }
	m_CartFirst=NULL; m_size=0; m_CartLast=NULL;
	m_itInfo[0].Clear(); m_itInfo[1].Clear();
}

CAPtrSeq* CAPtrSeq::get_copy(CAPtrSeq* other, bool do_create) {
	CAPtrSeq* res = NULL;
	if (other != NULL) { res = other->clone(); }
	else { if (do_create) res = new CAPtrSeq(false); }
	return res;
}

long CAPtrSeq::size() { return m_size; }

CEObject* CAPtrSeq::GetAt(long i) {
	//[Old: check m_itInfo usage!]ToICart(i);
	//[Old: check m_itInfo usage!]if ((i<0) || (m_CartFirst == NULL)) return NULL;
	//[Old: check m_itInfo usage!]CEBasic::dbg->log(0, i, "CAPtrSeq::GetAt %ld", this);
	//[Old: check m_itInfo usage!]m_itInfo[1].Clear(); // destroy J-mark

	// fast cache
	//[Old: check m_itInfo usage!]if (i==m_itInfo[0].m_Pos) { return m_itInfo[0].m_Cart; }
	//[Old: check m_itInfo usage!]if (i==m_itInfo[1].m_Pos) { return m_itInfo[1].m_Cart; }

	// find nearest I or J
	/*[Old: check m_itInfo usage!]bool use_I = true;
	if (i<m_itInfo[0].m_Pos) {
		use_I=false;
		if (m_itInfo[1].m_Cart==NULL) { m_itInfo[1].m_Pos=0; m_itInfo[1].m_Cart=m_CartFirst; }
	} else {
		if(m_itInfo[0].m_Pos<0) {
			if (m_itInfo[1].m_Cart==NULL) {
				m_itInfo[0].m_Pos=0; m_itInfo[0].m_Cart=m_CartFirst;
			} else {
				use_I=false;
			}
		} else {
			if ((m_itInfo[1].m_Pos>m_itInfo[0].m_Pos) && (i>=m_itInfo[1].m_Pos) && (m_itInfo[1].m_Cart!=NULL)) { use_I=false; }
			//if ((m_itInfo[1].m_Pos>m_itInfo[0].m_Pos) && (i>=m_itInfo[1].m_Pos)) { use_I=false; }
		}
	}*/
	/*[Old: check m_itInfo usage!]long j; CAPtrSeqCart* c;
	if ((m_itInfo[0].m_Pos>i) || (m_itInfo[0].m_Pos<0)) {
		if ((m_itInfo[1].m_Pos>=0) && (m_itInfo[1].m_Pos<=i)) { j=m_itInfo[1].m_Pos; c=m_itInfo[1].m_Cart; }
		else { j=0; c=m_CartFirst; }
	} else {
		if ((m_itInfo[1].m_Pos<=i) && (m_itInfo[1].m_Pos>m_itInfo[0].m_Pos)) { j=m_itInfo[1].m_Pos; c=m_itInfo[1].m_Cart; }
		else { j=m_itInfo[0].m_Pos; c=m_itInfo[0].m_Cart; }
	}

	while(j<i) {
		if (c == NULL) { CEBasic::dbg->log(0, j, "CAPtrSeq internal error: unexpeced NULL (pos %ld)"); break; }
		//m_itInfo[1].m_Cart=c; m_itInfo[1].m_Pos=j;
		c = c->m_Next; j++;
	}

	if ((j != m_itInfo[1].m_Pos) && (j != m_itInfo[0].m_Pos)) {
		if (m_itInfo[0].m_Pos < m_itInfo[1].m_Pos) {
			if(j>m_itInfo[1].m_Pos) {
				m_itInfo[1].m_Pos=j; m_itInfo[1].m_Cart=c;
			} else {
				m_itInfo[0].m_Pos=j; m_itInfo[0].m_Cart=c;
			}
		} else {
			if(j>m_itInfo[0].m_Pos) {
				m_itInfo[0].m_Pos=j; m_itInfo[0].m_Cart=c;
			} else {
				m_itInfo[1].m_Pos=j; m_itInfo[1].m_Cart=c;
			}
		}
	}
	return (c == NULL)?NULL:(c->m_Obj);*/

	long j; CAPtrSeqCart* c; long i1Pos=m_itInfo[0].m_Pos;
	if ((i1Pos<0) || (i1Pos>i)) {
		if(m_CartFirst==NULL)return NULL;
		long i2Pos=m_itInfo[1].m_Pos; if((i2Pos>=0)&&(i2Pos<=i)){ j=i2Pos; c=m_itInfo[1].m_Cart; }else{ j=0; c=m_CartFirst; }
	}else{
		j=i1Pos; c=m_itInfo[0].m_Cart;
	}
	while(j<i) { if(c==NULL)break; c=c->m_Next; j++; }

	if (c==NULL) return NULL;
	if (j != i1Pos) { m_itInfo[0].m_Pos=j; m_itInfo[0].m_Cart=c; }
	return c->m_Obj;
	//CEBasic::dbg->log(0, use_I, "CAPtrSeq::GetAt use_I=%ld");

	// start search from use_I?I:J
	/*[Old: check m_itInfo usage!] long j; CAPtrSeqCart* c;
	if (use_I) {
		j=m_itInfo[0].m_Pos; c=m_itInfo[0].m_Cart;
	} else {
		j=m_itInfo[1].m_Pos; c=m_itInfo[1].m_Cart;
	}
	//CEBasic::dbg->log(0, j, "CAPtrSeq::GetAt j=%ld");

	while(j<i) {
		if (c == NULL) { CEBasic::dbg->log(0, j, "CAPtrSeq internal error: unexpeced NULL (pos %ld)"); break; }
		if (j>i) { CEBasic::dbg->log(0, j, i, "CAPtrSeq internal error: (j>i) unexpeced NULL (%ld > %ld)"); break; }
		c = c->m_Next; j++;
	}

	if (c!=NULL) {
		if (use_I) { m_itInfo[0].m_Pos=j; m_itInfo[0].m_Cart=c; }
		else { m_itInfo[1].m_Pos=j; m_itInfo[1].m_Cart=c; }
	}

	return (c == NULL)?NULL:(c->m_Obj);*/

	/*[Old: check m_itInfo usage!]if ((i<m_itInfo[0].m_Pos) || (m_itInfo[0].m_Pos<0)) {
		m_itInfo[0].m_Pos=0; m_itInfo[0].m_Cart=m_CartFirst;
	}
	//long j; CAPtrSeqCart* c;
	//if ((m_itInfo[0].m_Pos > 0) && (i >= m_itInfo[0].m_Pos)) { j=m_itInfo[0].m_Pos; c=m_itInfo[0].m_Cart; } else { j=0; c=m_CartFirst; }

	while(m_itInfo[0].m_Pos<i) {
		if (m_itInfo[0].m_Cart == NULL) {
			CEBasic::dbg->log(0, m_itInfo[0].m_Pos, "CAPtrSeq internal error: unexpeced NULL (pos %ld)", this);
			m_itInfo[0].m_Pos=-1;
			return NULL;
		}
		m_itInfo[0].m_Cart = m_itInfo[0].m_Cart->m_Next;
		m_itInfo[0].m_Pos++;
	}

	return (m_itInfo[0].m_Cart == NULL)?NULL:(m_itInfo[0].m_Cart->m_Obj);*/
}

CEObject* CAPtrSeq::GetAt2(long i) { //uses J-iterator
	long j; CAPtrSeqCart* c; long i2Pos=m_itInfo[1].m_Pos;
	if ((i2Pos<0) || (i2Pos>i)) {
		if(m_CartFirst==NULL)return NULL;
		long i1Pos=m_itInfo[0].m_Pos; if((i1Pos>=0)&&(i1Pos<=i)){ j=i1Pos; c=m_itInfo[0].m_Cart; }else{ j=0; c=m_CartFirst; }
	} else {
		j=i2Pos; c=m_itInfo[1].m_Cart;
	}
	while(j<i) { if(c==NULL)break; c=c->m_Next; j++; }
	if (c==NULL) return NULL;
	if (j != i2Pos) { m_itInfo[1].m_Pos=j; m_itInfo[1].m_Cart=c; }
	return c->m_Obj;
}

const CEObject* CAPtrSeq::GetAtConst(long i) const {
	//code copied from 'GetAt(i)'

	long j; CAPtrSeqCart* c; long i1Pos=m_itInfo[0].m_Pos;
	if ((i1Pos<0) || (i1Pos>i)) {
		if(m_CartFirst==NULL)return NULL;
		long i2Pos=m_itInfo[1].m_Pos; if((i2Pos>=0)&&(i2Pos<=i)){ j=i2Pos; c=m_itInfo[1].m_Cart; }else{ j=0; c=m_CartFirst; }
	}else{
		j=i1Pos; c=m_itInfo[0].m_Cart;
	}
	while(j<i) { if(c==NULL)break; c=c->m_Next; j++; }

	if (c==NULL) return NULL;
	if (j != i1Pos) {
		CASeqIterationCart* inf = (CASeqIterationCart*)(&m_itInfo[0]); //cast 'const ptr'->'ptr'
		inf->m_Pos=j; inf->m_Cart=c;
	}
	return c->m_Obj;
}

void CAPtrSeq::ToICart(long i) {
	if ((i<0) || (i>=m_size)) { CEBasic::dbg->log(0, i, "CAPtrSeq:ToICart(wrong pos: %ld)", this); return; }

	long i1Pos=m_itInfo[0].m_Pos; CAPtrSeqCart* c;
	if((i1Pos<0)||(i1Pos>i)){ i1Pos=0; c=m_CartFirst; }else{ c=m_itInfo[0].m_Cart; }

	for(; i1Pos<i;) {
		if(c==NULL){ CEBasic::dbg->log(0, i1Pos, "CAPtrSeq internal error: unexpeced NULL (pos %ld)", this); i1Pos=-1; break; }
		c=c->m_Next; i1Pos++;
	}

	m_itInfo[0].m_Pos=i1Pos; m_itInfo[0].m_Cart=c;
}

// do not clone obj
void CAPtrSeq::add(CEObject* obj) {
	if ((obj == NULL) && (!m_allow_NULLs)) return;

	CAPtrSeqCart* c = new CAPtrSeqCart(obj);
	if (m_CartLast != NULL) {
		m_CartLast->m_Next = c;
	} else {
		if (m_CartFirst != NULL) { CEBasic::dbg->log(0, "CAPtrSeq Error: NULL last, not-NULL first", this); }
		m_CartFirst = c;
	}
	m_CartLast = c;
	m_size++;
}

// clone obj
void CAPtrSeq::SetAt(long i, CEObject *obj, bool delete_old) {
	if ((obj == NULL) && (!m_allow_NULLs)) return;
	ToICart(i);
	if (m_itInfo[0].m_Cart != NULL) {
		//obj = (obj!=NULL)?(obj->copy()):NULL;
		if (delete_old) { CEObject* obj_old = m_itInfo[0].m_Cart->m_Obj; if (obj_old != NULL) delete obj_old; }
		m_itInfo[0].m_Cart->m_Obj = obj;
	}
}

void CAPtrSeq::InsertAt(long i, CEObject* obj) {
	if ((i<0) || (i>m_size)) { CEBasic::dbg->log(0, i, "CAPtrSeq: InsertAt wrong pos: %ld", this); return; }
	if (i == m_size) { add(obj); return; }
	if ((obj == NULL) && (!m_allow_NULLs)) return;
	//CAPtrSeqCart* c_new = new CAPtrSeqCart((obj!=NULL)?(obj->copy()):NULL);
	CAPtrSeqCart* c_new = new CAPtrSeqCart(obj);

	if (m_itInfo[1].m_Pos>=i) { m_itInfo[1].Clear(); } // destroy J-mark

	if (i>0) { // search for (i-1) cart
		ToICart(i-1); // m_itInfo[0].m_Pos is set to i-1
		if (m_itInfo[0].m_Cart != NULL) {
			c_new->m_Next=m_itInfo[0].m_Cart->m_Next; m_itInfo[0].m_Cart->m_Next=c_new; m_size++;
		} else {
			CEBasic::dbg->log(0, "CAPtrSeq: InsertAt ERROR", this); delete c_new;
		}
	} else { // i==0
		c_new->m_Next=m_CartFirst; m_CartFirst=c_new; m_size++; m_itInfo[0].Clear();
	}
}

void CAPtrSeq::RemoveAt(long i, bool delete_old) {
	if ((i<0) || (i>=m_size)) { CEBasic::dbg->log(0, i, "CAPtrSeq: RemoveAt wrong pos: %ld", this); return; }
	CAPtrSeqCart* c_del = ExtractCartAt(i);
	if(c_del!=NULL){ if(!delete_old){ c_del->m_Obj=NULL; } delete c_del; }
}

CEObject* CAPtrSeq::ExtractAt(long i) {
	if((i<0)||(i>=m_size)){ CEBasic::dbg->log(0, i, "CAPtrSeq: ExtractAt wrong pos: %ld", this); return NULL; }
	CAPtrSeqCart* c_del=ExtractCartAt(i);
	CEObject* res=NULL; if(c_del!=NULL){ res=c_del->m_Obj; delete c_del; } return res;
}

// position should be valid here ==> no check
CAPtrSeqCart* CAPtrSeq::ExtractCartAt(long i) {
	if(m_itInfo[1].m_Pos>=i){ m_itInfo[1].Clear(); } // destroy J-mark

	// extract item
	CAPtrSeqCart* c=NULL;
	if(i==0){
		c=m_CartFirst; m_CartFirst=c->m_Next;
		if(c==m_CartLast)m_CartLast=NULL;
		m_size--;
		if(m_itInfo[0].m_Pos>0){ m_itInfo[0].m_Pos--; }else{ m_itInfo[0].Clear(); }
	} else {
		ToICart(i-1); // m_itInfo[0].m_Pos is set to i-1
		if(m_itInfo[0].m_Cart==NULL){ CEBasic::dbg->log(0, "CAPtrSeq: ExtractCartAt internal error", this); return NULL; }
		c=m_itInfo[0].m_Cart->m_Next;
		if(c==NULL){ CEBasic::dbg->log(0, "CAPtrSeq: ExtractCartAt internal error #2", this); return NULL; }
		m_itInfo[0].m_Cart->m_Next=c->m_Next;
		if(c==m_CartLast){
			m_CartLast=m_itInfo[0].m_Cart; //Must already be: m_itInfo[0].m_Cart->m_Next=NULL;
		}
		m_size--;
	}
	c->m_Next=NULL;
	return c;
}

void CAPtrSeq::MoveFromTo(long from, long to) {
	if((from<0) || (from>=m_size) || (to<0) || (to>=m_size)){ CEBasic::dbg->log(0, from, to, m_size, "CAPtrSeq: MoveFromTo wrong pos(%ld, %ld), size: %ld", this); return; }
	if(to==from)return;
	m_itInfo[1].Clear(); // destroy J-mark

	//Use cache more optimal
	CAPtrSeqCart* c_to_prev=NULL; if((to<from) && (to>0)) { ToICart(to-1); c_to_prev=m_itInfo[0].m_Cart; }

	CAPtrSeqCart* c=ExtractCartAt(from); if(c==NULL){ CEException::halt("CAPtrSeq: MoveFromTo internal error"); return; }
	//Note: m_size is now m_size-1

	//Insert cart at
	if(to>0) {
		if(c_to_prev!=NULL){
			m_itInfo[0].m_Pos=to-1; m_itInfo[0].m_Cart=c_to_prev; // for faster work
		}else{
			ToICart(to-1); // m_itInfo[0].m_Pos is set to i-1
		}
		if(m_itInfo[0].m_Cart==NULL){ CEException::halt("CAPtrSeq: MoveFromTo internal error"); return; }
		c->m_Next=m_itInfo[0].m_Cart->m_Next; m_itInfo[0].m_Cart->m_Next=c;
		if(m_itInfo[0].m_Cart==m_CartLast)m_CartLast=c;
	} else { //to==0
		c->m_Next=m_CartFirst; m_CartFirst=c;
		if(m_CartLast==NULL)m_CartLast=m_CartFirst;
		m_itInfo[0].m_Pos=0; m_itInfo[0].m_Cart=m_CartFirst;
	}
	//Increase 'm_size' back
	m_size++;
}

CEObject* CAPtrSeq::copy() const { return clone(); }

CAPtrSeq* CAPtrSeq::clone() const {
	CAPtrSeq* res = new CAPtrSeq(m_CartFirst, m_allow_NULLs);
	return res;
}

void CAPtrSeq::add_sequence(CAPtrSeq* other) {
	if (other == NULL) return;
	other->reset_iterator();
	while(other->has_next()) { CEObject* o = other->next(); add((o!=NULL)?(o->copy()):NULL); }
}

// iterator operations
void CAPtrSeq::reset_iterator() {
	//'m_itInfo[0].m_Pos' means next number
	m_itInfo[0].m_Pos=0; m_itInfo[0].m_Cart=m_CartFirst;
	if (m_itInfo[0].m_Cart==NULL){ m_itInfo[0].m_Pos=-1; }
}

CEObject* CAPtrSeq::next() {
	if(m_itInfo[0].m_Cart==NULL)return NULL;
	CEObject* res=m_itInfo[0].m_Cart->m_Obj;
	m_itInfo[0].m_Pos++; m_itInfo[0].m_Cart=m_itInfo[0].m_Cart->m_Next;
	if(m_itInfo[0].m_Cart==NULL){ m_itInfo[0].m_Pos=-1; }
	return res;
}

bool CAPtrSeq::has_next(){ return (m_itInfo[0].m_Cart!=NULL); }

bool CAPtrSeq::obj_equals(CEObject* other) {
	if (other == NULL) return false;
	if (other == this) return true;
	if (ClassName() != CAPtrSeq::CLASS_NAME) return false;
	CAPtrSeq* o_seq=(CAPtrSeq*)other; long s=size(); if(o_seq->size()!=s)return false;

	reset_iterator(); o_seq->reset_iterator();
	while(has_next()){ // compare sequences by items
		if(!o_seq->has_next())return false;
		CEObject* obj_1=next(); CEObject* obj_2=o_seq->next();
		if (obj_1!=obj_2) { //(NULL==NULL) check
			if((obj_1==NULL)||(obj_2==NULL)) return false;
			if(!obj_1->obj_equals(obj_2)) return false;
		}
	}
	if(o_seq->has_next())return false;
	return true;
}

char* CAPtrSeq::toLine(int mode) {
	char* res=NULL; long s=m_size; res=CEUtil::copyString("{"); CAPtrSeqCart* c=m_CartFirst;
	while(c != NULL) {
		CEObject* p=c->m_Obj; c=c->m_Next;
		if (p!=NULL) {
			char* str=p->toLine(mode);
			char* res_new=CEUtil::concat(res, str);
			CMemoryManager::mfree(str); CMemoryManager::mfree(res);
			res=res_new;
		}
		else { char* res_new=CEUtil::concat(res, "NULL"); CMemoryManager::mfree(res); res=res_new; }

		if(c!=NULL){ char* res_new=CEUtil::concat(res, "; "); CMemoryManager::mfree(res); res=res_new; }
	}
	char* res_new = CEUtil::concat(res, "}"); CMemoryManager::mfree(res); res=res_new;
	return res;
}

bool CAPtrSeq::contain_vector(void * obj) {
	CAPtrSeqCart* c=m_CartFirst; while(c != NULL){ if(c==obj)return true; c=c->m_Next; } //direct search
	c=m_CartFirst; while(c!=NULL){ if(c->contain_vector(obj))return true; c=c->m_Next; } //scan sub-items
	return false;
}

void CAPtrSeq::SelfCheck() {
	CDebug* dbg = CEBasic::dbg;
	dbg->log(1, m_size, m_itInfo[0].m_Pos, "CAPtrSeq::SelfCheck(size: %ld, i: %ld)");

	// log whole sequence
	//dbg->log(1, "CAPtrSeq", this);

	// size/last
	ToICart(m_size-1);
	if (m_itInfo[0].m_Cart == NULL) dbg->log(1, "NULL GetICart(m_size-1)");
	dbg->log(1, (m_itInfo[0].m_Cart == m_CartLast), "GetICart(m_size-1) == m_CartLast: %ld");

	// NULL carts
	long i;
	for(i=0; i<m_size; i++) {
		ToICart(i);
		if (m_itInfo[0].m_Cart == NULL) dbg->log(1, i, "GetICart(%ld) == NULL");
	}

	reset_iterator();
	i=0;
	while(has_next()) { next(); i++; }
	dbg->log(1, i, m_size, "has_next()...next() iterations number: %ld (size: %ld)");
}
