////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// Debug.cpp: implementation of the CDebug class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//#include <time.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

#define DEBUG_INI_FILE      "debug.ini"
#define DEBUG_OUTPUT_FILE   "debug.log"
#define DEBUG_NB_FILE       "debug.nb"
#define DEBUG_TEX_FILE      "debug.tex"

#define DEFAULT_LEVEL           10
#define DEFAULT_EXPRESSION_GAP  2

#define MAX_CN_LENGTH           100

const int CDebug::LOG_DEFAULT		= 0x00;
const int CDebug::LOG_NO_OBJECT		= 0x01;
const int CDebug::LOG_TEX			= 0x02;
bool CDebug::TEX_LOG_STARTED = false;

CEPtrArray* CDebug::data = NULL;
CEPtrArray* CDebug::m_debug_objects = NULL;
char* CDebug::m_sFullPathDebugNB = NULL;		//NULL if until path is changed
char* CDebug::m_sFullPathDebugTeX = NULL;		//NULL if until path is changed
//bool CDebug::updated = false;
bool CDebug::m_busy = false;
DWORD CDebug::m_debugFN_counter = 0;


//##ModelId=3E3FF2CD0399
class CDataFormat : public CEObject
{
	//##ModelId=3E3FF2CD039C
    long m_value;
	//##ModelId=3E3FF2CD039B
    char* m_id;

public:
	//##ModelId=3E3FF2CD03A2
	CDataFormat(const char* id, long value);
	//##ModelId=3E3FF2CD03A1
	~CDataFormat();
	//##ModelId=3E3FF2CD03A0
	long value();
	//##ModelId=3E3FF2CD039F
	const char* getId();
	//##ModelId=3E3FF2CD039D
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual bool contain_vector(void * obj);
};

CDataFormat::CDataFormat(const char* id, long value) : CEObject(false)
{
    m_value = value;
    m_id = CEUtil::copyString(id);
}

CDataFormat::~CDataFormat()
{
    CMemoryManager::mfree(m_id);
}

long CDataFormat::value()
{
    return m_value;
}

const char* CDataFormat::getId()
{
    return m_id;
}

const char* CDataFormat::CLASS_NAME = "CDataFormat";
const char* CDataFormat::ClassName() const { return CLASS_NAME; }
bool CDataFormat::contain_vector(void * obj)
{
	return (m_id == obj);
}

CDebug::CDebug(const char* class_name)
{
	m_cn = CEUtil::copyString(class_name);
    if (data == NULL) init();
    m_level = get(class_name);
	//updated = false;
	m_num = 0;
	do_watch_period = false;
	//period = -1;
	m_time_mark = NULL;
	//period = 0;
	//reset_period()
	m_debug_exprs = NULL;

	if (m_debug_objects == NULL) m_debug_objects = new CEPtrArray();
	m_debug_objects->add(this);
}

CDebug::~CDebug()
{
	if (m_debug_exprs != NULL) delete m_debug_exprs;
	if (m_time_mark != NULL) delete m_time_mark;
}

const char* CDebug::GetDebugFileName() { return DEBUG_NB_FILE; }

void CDebug::init() {
	ResetAbsTime();

	TEX_LOG_STARTED = false;
	m_debugFN_counter = 0;
    data = new CEPtrArray();
    //Creating log file.
    ofstream ofs(DEBUG_NB_FILE);
	if (ofs) {
		ofs << getPrefix();
		ofs << "Cell[\"ALHEP debug file. Use debug.ini settings to adjust its content.\", \"Text\"]";
	}
    ofs.close();
	m_busy = false;

    // Loading ini file
	ifstream ifs((const char*)DEBUG_INI_FILE, ios::in);
	if (!ifs) return;
    char* prev = NULL;
	bool log_first=true;
    for(char* item = get_next(&ifs); (item!=NULL); item = get_next(&ifs)) {
		long value = atol(item);
        if (value > 0) {
			if ((prev != NULL) && (CEUtil::IsSymbol(*prev))) {
				put(prev, value);
				if (value > 5) { if(log_first){printf("Debug levels loaded:\n");log_first=false;} printf("<%s> : <%d>.\n",prev,value); }
            }
            CMemoryManager::mfree(item); item=NULL;
        }
		CMemoryManager::mfree(prev); prev=item;
    }
    if (prev != NULL) CMemoryManager::mfree(prev);
    ifs.close();
}

//
//	Results: OPERAND or NUMBER only
//	Comments: "//..." , ";...", /*...*/
//
char* CDebug::get_next(ifstream* pifs) {
    char* item = (char*)CMemoryManager::mmalloc(MAX_CN_LENGTH);
    long pos = 0;
	bool skippingEOL=false;
	bool skippingEOComm=false;
    bool _OperandComposed = false;
    for(;!_OperandComposed;) {
		int c=pifs->peek();

		if (skippingEOL) {
			pifs->get(); // skip peeked
			if (c == EOF) break;
			if ((c=='\n') || (c==0x0a)) skippingEOL=false;
			continue;
		}
		if (skippingEOComm) {
			pifs->get(); // skip peeked
			if (c == EOF) break;
			if (c=='*') {
				c=pifs->get(); if (c == EOF) break;
				if (c == '/') skippingEOComm=false;
			}
			continue;
		}

		if (CEUtil::IsSymbol(c) || CEUtil::IsNumber(c)) {
			pifs->get(); // skip peeked
            *(item + pos)=c; pos++;
            if (pos >= MAX_CN_LENGTH) {
				*(item + MAX_CN_LENGTH - 1) = '\0';
                printf("CDebug::init() <Error><Parsed Item Too Long: %s, file: %s>.", item, DEBUG_INI_FILE);
                pos=1;
            }
			continue;
        }
		
		if (pos > 0) {
			// do not skip peeked here
			*(item + pos) = '\0'; _OperandComposed=true;
			continue;
		}

		if (c == EOF) break;

		// pos==0 here
		pifs->get(); // skip peeked
		if (c == ';') {
			skippingEOL=true;
		} else if (c == '/') {
			c=pifs->peek();
			if (c == '/') { pifs->get(); skippingEOL=true; }
			else if (c == '*') { pifs->get(); skippingEOComm=true; }
		}
    }
	if ((!_OperandComposed) || (pos==0)) { CMemoryManager::mfree(item); item=NULL; }
    return item;
}

void CDebug::put(const char* id, long value) {
    long size = data->size();
	long i;
    for(i=0; i<size; i++)
    {
        CDataFormat* item = (CDataFormat*)data->GetAt(i);
        if (CEUtil::equals(item->getId(), id)) {
            long old = item->value();
			if (old != value) {
				data->SetAt(i, new CDataFormat(id, value));
				//updated = true;
			}
            return;
        }
    }
    data->add(new CDataFormat(id, value));
	//updated = true;
}

long CDebug::get(const char* id) {
    long size = data->size();
	long i;
    for(i=0; i<size; i++)
    {
        CDataFormat* item = (CDataFormat*)data->GetAt(i);
        if (CEUtil::equals(item->getId(), id)) return item->value();
    }
    return 0;
}

const char* CDebug::getPrefix() { return CESupport::getNBPrefix(); }
const char* CDebug::getPostfix() { return CESupport::getNBPostfix(); }

void CDebug::log(int level, const char *message, int mode)
{
	log(level, message, NULL, mode | LOG_NO_OBJECT);
}

void CDebug::log(const char *message, int mode)
{
	log(DEFAULT_LEVEL, message, mode);
}

void CDebug::log(int level, CEObject *obj, int mode)
{
	log(level, NULL, obj, mode);
}

void CDebug::log(CEObject *obj, int mode)
{
	log(DEFAULT_LEVEL, obj, mode);
}

void CDebug::log(int level, const char* message, CEObject* obj_1, CEObject* obj_2, int mode) {
	log(level, message, obj_1, mode);
	log(level, obj_2, mode);
}

void CDebug::log(int level, const char* message, CEObject* obj_1, CEObject* obj_2, CEObject* obj_3, int mode) {
	log(level, message, obj_1, mode);
	log(level, obj_2, mode);
	log(level, obj_3, mode);
}

void CDebug::finalize()
{
	ofstream ofs(DEBUG_NB_FILE, ios::out|ios::app);
	if (ofs) {
		ofs << getPostfix();
	}
    ofs.close();
	
	// store the debug sequences from all the CDebug objects
	if (m_debug_objects != NULL) {
		long cdeb_sz = m_debug_objects->size();
		long i;
		for(i=0; i<cdeb_sz; i++)
	    {
			CDebug* dbg_item = (CDebug*)m_debug_objects->GetAt(i);
			dbg_item->store_debug_list();
	    }
	}

	// delete data format structures
	delete data;

	// finalize all the CDebug objects
	delete m_debug_objects;

	if (m_start_time != NULL) { delete m_start_time; m_start_time = NULL; }

	if(m_sFullPathDebugNB != NULL){ char* tmp=m_sFullPathDebugNB; m_sFullPathDebugNB=NULL; CMemoryManager::mfree(tmp); }
	if(m_sFullPathDebugTeX != NULL){ char* tmp=m_sFullPathDebugTeX; m_sFullPathDebugTeX=NULL; CMemoryManager::mfree(tmp); }
	///
}

CDebug* CDebug::GetDebugObject(const char* id) {
	if (id == NULL) return NULL;
	if (m_debug_objects == NULL) return NULL;
	long s = m_debug_objects->size();
	long i;
	for(i=0; i<s; i++) {
		CDebug* dbg_obj = (CDebug*)m_debug_objects->GetAt(i);
		if (CEUtil::equals(dbg_obj->GetDebugClassID(), id)) { return dbg_obj; }
	}
	return NULL;
}

void CDebug::log(int level, const char *message, CEObject *obj, int mode) {
	if (level > m_level) return;
	if (m_busy) { printf("busy(%ld) {missing message: %s }\n", level, message); return; }

	//DEBUG_TEX_FILE

	// print error/warning notification to console
	if (level <= 1) {
		if (message != NULL) printf("<%s>: %s\n",m_cn,message);
		// Note: NULL messages appear between two objects attached to single message.
		//else printf("<%s>: NULL message error\n",m_cn);
	}

	m_busy = true;

	//TEX_LOG_STARTED
	if (mode & LOG_TEX) {
		ofstream ofs;
		if(CVMFileUtil::IsInitialDirChanged()){
			if(m_sFullPathDebugTeX==NULL){
				m_sFullPathDebugTeX = (char*)CMemoryManager::mmalloc(500);
				if(m_sFullPathDebugTeX!=NULL){
					const char* sPathInit = CVMFileUtil::GetInitialPath();
					if(sPathInit!=NULL){ sprintf(m_sFullPathDebugTeX,"%s\\%s",sPathInit,DEBUG_TEX_FILE); }else{ sprintf(m_sFullPathDebugTeX,"Error"); }
				}
			}
			if(m_sFullPathDebugTeX!=NULL){
				ofs.open(m_sFullPathDebugTeX, (TEX_LOG_STARTED)?(ios::out|ios::app):(ios::out|ios::trunc));
			}
		}else{
			ofs.open(DEBUG_TEX_FILE, (TEX_LOG_STARTED)?(ios::out|ios::app):(ios::out|ios::trunc));
		}
		if (!TEX_LOG_STARTED) TEX_LOG_STARTED = true;
		if (ofs) {
			ofs << "\n" << m_num <<". "; ofs << "<" << m_cn <<">";
			if (message != NULL) ofs << message;
			if (do_watch_period) {
				CETimeMark* per_new = update_period();
				char* per_str = per_new->toLine(STORE_TEX);
				ofs << " <" << per_str <<">";
				CMemoryManager::mfree(per_str); delete per_new;
			}
			if (obj != NULL) { ofs << " (" << obj->ClassName() << ") :"; }
			
			bool do_print_expr = ((DEFAULT_EXPRESSION_GAP + level) <= m_level);
			if (level <= 1) do_print_expr = true;
			if ((mode & LOG_NO_OBJECT) == 0) {
				if ((obj == NULL) && (do_print_expr)) {ofs << " <NULL>";}
			}
			ofs << "\n";
			
			if ((mode & LOG_NO_OBJECT) == 0) {
				if ((obj != NULL) && (do_print_expr)) {
					char* line = obj->toLine(STORE_TEX);
					if (line != NULL) { ofs << line << "\n"; }
					else { ofs << "NULL\n"; }
					CMemoryManager::mfree(line);
				}
			}
			ofs.close();
		} else printf("<Error creating CDebug output TeX file.>");
	} else {
		//save message to DEBUG_NB_FILE.
		ofstream ofs;
		if(CVMFileUtil::IsInitialDirChanged()){
			if(m_sFullPathDebugNB==NULL){
				m_sFullPathDebugNB = (char*)CMemoryManager::mmalloc(500);
				if(m_sFullPathDebugNB!=NULL){
					const char* sPathInit = CVMFileUtil::GetInitialPath();
					if(sPathInit!=NULL){ sprintf(m_sFullPathDebugNB,"%s\\%s",sPathInit,DEBUG_NB_FILE); }else{ sprintf(m_sFullPathDebugNB,"Error"); }
				}
			}
			if(m_sFullPathDebugNB!=NULL){
				ofs.open(m_sFullPathDebugNB, ios::out|ios::app);
			}
		}else{
			ofs.open(DEBUG_NB_FILE, ios::out|ios::app);
		}
		if (ofs) {
			ofs << ",\n\nCell[\"" << m_num << ". " << "<" << m_cn <<">";

			if (message != NULL) {
				bool bHasComma = (CEUtil::findSymbol(message,'\"')>=0);
				if (bHasComma) {
					long _MessL=CEUtil::length(message);
					for(long _MessI=0; _MessI<_MessL; _MessI++) {
						char c= *(message+_MessI); if(c=='\0')break;
						if (c=='\"') ofs << '\\';
						ofs << c;
					}
				} else { // faster
					ofs << " : " << message;
				}
			}

#if defined(ALTA_DEBUG)
			// watch objects counter
			ofs << " <oc: " << CMemoryManager::GetObjCounter() <<">"; 
#else
			// watch periods
			if (do_watch_period) {
				CETimeMark* per_new = update_period();
				char* per_str = per_new->toLine(STORE_WR_MATHEMATICA);
				ofs << " <" << per_str <<">";
				CMemoryManager::mfree(per_str); delete per_new;
			}
#endif //ALTA_DEBUG

			if (obj != NULL) {
				ofs << " (" << obj->ClassName();
				if (CEBasic::isExpression(obj)) {
					CExpression* expr_obj = (CExpression*)obj;
					if (!expr_obj->getFlag()) ofs << " s-";
					if (!expr_obj->getLevel()) ofs << " l-";
				}
				ofs << ") :";
			}
			
			bool do_print_expr = ((DEFAULT_EXPRESSION_GAP + level) <= m_level);
			if (level <= 1) do_print_expr = true;
			if (mode & LOG_NO_OBJECT) do_print_expr = false;
			
			if ((obj == NULL) && (do_print_expr)) {ofs << " <NULL>";}
			ofs << "\", \"Text\"]";
			if ((obj != NULL) && (do_print_expr)) {
				char* line = obj->toLine(STORE_WR_MATHEMATICA);
				if (line != NULL) { ofs << ",\nCell[BoxData[\\(" << line << "\\)], \"Output\"]"; }
				else { ofs << "{NULL string}"; }
				CMemoryManager::mfree(line);
			}
			ofs.close();
		}
		else printf("<Error creating CDebug output NB file.>");
	}
	m_num++;
	m_busy = false;
}

int CDebug::set_level(int level)
{
	int old = m_level;
	m_level = level;
	return old;
}

int CDebug::get_level() { return m_level; }
const char* CDebug::GetDebugClassID() { return m_cn; }

const char* CDebug::CLASS_NAME = "CDebug";
const char* CDebug::ClassName() const {return CLASS_NAME; }
bool CDebug::contain_vector(void * obj)
{
	long size = data->size();
	long i;
	for(i=0; i<size; i++)
	{
		CDataFormat* item = (CDataFormat*)data->GetAt(i);
		if ((item == obj) || (item->contain_vector(obj)))
			return true;
	}
	return false;
}

long CDebug::GetMessagesCounter() { return m_num; }
CETimeMark* CDebug::m_start_time = NULL;
CETimeMark* CDebug::GetAbsTime() {
	CETimeMark* t_curr = NULL;
	if (m_start_time == NULL) {
		ResetAbsTime();
		t_curr = CETimeMark::GetZeroTimeMark();
	} else {
		t_curr = CETimeMark::GetCurrTimeMark();
		t_curr->substrTimeMark(m_start_time);
	}
	return t_curr;
}

char* CDebug::GetAbsTimeString() {
	CETimeMark* tm = GetAbsTime();
	char* res = tm->toLine();
	delete tm;
	return res;
}

void CDebug::ResetAbsTime() {
	if (m_start_time != NULL) delete m_start_time;
	m_start_time = CETimeMark::GetCurrTimeMark();
}

void CDebug::ClearAllTimers() { // deletes all the timer marks
	if (m_start_time != NULL) { delete m_start_time; m_start_time = NULL; }

	if (m_debug_objects != NULL) {
		long s = m_debug_objects->size();
		long i;
		for(i=0; i<s; i++) {
			CDebug* dbg_obj = (CDebug*)m_debug_objects->GetAt(i);
			dbg_obj->clear_periods_data();
		}
	}
}

//void CDebug::ClearAbsTime() { if (m_start_time != NULL) { delete m_start_time; m_start_time = NULL; } }
void CDebug::clear_periods_data() { if (m_time_mark != NULL) { delete m_time_mark; m_time_mark = NULL; } }

void CDebug::reset_periods(int level) {
	if (level > m_level) return;

	if (m_time_mark != NULL) delete m_time_mark;
	m_time_mark = CETimeMark::GetCurrTimeMark();
}

CETimeMark* CDebug::update_period()
{
	CETimeMark* t_res = NULL;
	CETimeMark* curr_t = CETimeMark::GetCurrTimeMark();
	if (m_time_mark != NULL) {
		t_res = (CETimeMark*)curr_t->copy();
		t_res->substrTimeMark(m_time_mark);
		delete m_time_mark;
	} else {
		t_res = CETimeMark::GetZeroTimeMark();
	}
	m_time_mark = curr_t;
	return t_res;

	//long per_new = (long)(clock()*1000/CLOCKS_PER_SEC);
	//long res = per_new-period;
	//period = per_new;
	//return res;
}

void CDebug::watch_periods(bool do_watch)
{
	if (do_watch_period != do_watch) {
		do_watch_period = do_watch;
		if (do_watch) reset_periods();
		else { // clear time data
			if (m_time_mark != NULL) { delete m_time_mark; m_time_mark = NULL; }
		}
	}
}

void CDebug::log(int level, long val, const char * message, int mode) {
	log(level, val, message, NULL, mode | LOG_NO_OBJECT);
}

void CDebug::log(int level, long val_1, long val_2, const char* message, int mode) {
	log(level, val_1, val_2, message, NULL, mode | LOG_NO_OBJECT);
}

void CDebug::log(int level, long val_1, long val_2, long val_3, const char* message, int mode) {
	log(level, val_1, val_2, val_3, message, NULL, mode | LOG_NO_OBJECT);
}

void CDebug::log(int level, const char* message, const char* message2) {
	log(level, message, message2, NULL, LOG_NO_OBJECT);
}

void CDebug::log(int level, const char* message, const char* message2, CEObject* obj, int mode) {
	if (level > m_level) return;

	char* s = CEUtil::concat(message, message2);
	log(level, s, obj, mode);
	CMemoryManager::mfree(s);
}

void CDebug::logIns(int level, const char* message, const char* messageToInsert) {
	if (level > m_level) return;
	if (messageToInsert!=NULL) {
		char* s = (char*)CMemoryManager::mmalloc(CEUtil::length(message)+CEUtil::length(messageToInsert)+4);
		sprintf(s, message, messageToInsert);
		log(level, s, NULL, LOG_NO_OBJECT);
		CMemoryManager::mfree(s);
	} else {
		log(level, message, NULL, LOG_NO_OBJECT);
	}
}

void CDebug::log(int level, long val, const char* message, CEObject* obj, int mode) {
	if (level > m_level) return;
	if (message != NULL) {
		char* s = (char*)CMemoryManager::mmalloc(CEUtil::length(message)+10);
		sprintf(s, message, val);
		log(level, s, obj, mode);
		CMemoryManager::mfree(s);
	} else {
		printf("<Error> log(%ld): unexpected NULL message.\n", val);
		log(level, message, obj, mode);
	}
}

void CDebug::log(int level, long val_1, long val_2, const char* message, CEObject* obj, int mode) {
	if (level > m_level) return;
	if (message != NULL) {
		char* s = (char*)CMemoryManager::mmalloc(CEUtil::length(message)+20);
		sprintf(s, message, val_1, val_2);
		log(level, s, obj, mode);
		CMemoryManager::mfree(s);
	} else {
		printf("<Error> log(%ld, %ld): unexpected NULL message.\n", val_1, val_2);
		log(level, message, obj, mode);
	}
}

void CDebug::log(int level, long val_1, long val_2, long val_3, const char* message, CEObject* obj, int mode) {
	if (level > m_level) return;
	if (message != NULL) {
		char* s = (char*)CMemoryManager::mmalloc(CEUtil::length(message)+30);
		sprintf(s, message, val_1, val_2, val_3);
		log(level, s, obj, mode);
		CMemoryManager::mfree(s);
	} else {
		printf("<Debug Error> log(%ld, %ld, %ld): unexpected NULL message.\n", val_1, val_2, val_3);
		log(level, message, obj, mode);
	}
}

void CDebug::logperiod(int level, const char *message)
{
	if (level > m_level) return;
	if (m_time_mark == NULL) {
		reset_periods();
		if (message != NULL) {
			char* s = CEUtil::concat(message, " {Timer started}");
			log(level, s);
			CMemoryManager::mfree(s);
		} else log(level, "{Timer started}");
	}
	else {
		CETimeMark* t_mark = update_period();
		char* t_s = t_mark->toLine();
		if (message != NULL) {
			char* s = CEUtil::concat(message, t_s, ' ');
			log(level, s);
			CMemoryManager::mfree(s);
		} else log(level, t_s);
		CMemoryManager::mfree(t_s); delete t_mark;
	}
}

void CDebug::add_debug_expr(CEObject *expr, bool store_immediately)
{
	if (expr == NULL) {
		log(0, "<Error> NULL expression addition to debug list");
		return;
	}

	if (store_immediately) {
		store_debug_expression(0, NULL, expr);
		return;
	}

	CExpression* e = NULL;
	try {e = (CExpression*)expr;}
	catch(...) {
		log(0, "<Error> Non-expression object in debug watch");
		return;
	}

	e = e->clone();
	if (m_debug_exprs == NULL) m_debug_exprs = new CEPtrArray();
	m_debug_exprs->add(e);
}

void CDebug::store_debug_list()
{
	if (m_debug_exprs == NULL) return;
	long ds = m_debug_exprs->size();
	if (ds == 0) return;

	long i;
	for (i=0; i<ds; i++) {
		CExpression* item = (CExpression*)m_debug_exprs->GetAt(i);
		store_debug_expression(0, NULL, item);
	}
}

char* CDebug::get_unique_debugFN()
{
	m_debugFN_counter++;

	char* res = (char*)CMemoryManager::mmalloc(32);
	sprintf(res, "dbg_%ld.xml", m_debugFN_counter);
	return res;
}

void CDebug::store_debug_expression(int level, const char* message, CEObject* expr)
{
	if ((level > m_level) || (m_busy)) return;
	m_busy = true;

	CExpression* e = NULL;
	try {e = (CExpression*)expr;}
	catch(...) {
		log(0, "<Error> Non-expression debug object serialization");
		m_busy = false;
		return;
	}

	char* fn = get_unique_debugFN();
	CESupport::store(e, fn, CESupport::SERIAL_EXPRESSION);

	long l = 100;
	if (message != NULL) {
		l += CEUtil::length(message);
		if ((l<100) || (l > 200)) {l = 200; message = NULL;}
	}
	char* msg = (char*)CMemoryManager::mmalloc(l);
	if (message != NULL) sprintf(msg, "<%s> %s (saved into %s)", m_cn, message, fn);
	else sprintf(msg, "<%s> debug expression saved into %s", m_cn, fn);
	CMemoryManager::mfree(fn);
	m_busy = false;
	log(level, msg, e);
	CMemoryManager::mfree(msg);
}

bool CDebug::is_log_keeped(int level)
{
	if ((level <= m_level) && (!m_busy)) return true;
	return false;
}
