////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ECache.cpp: implementation of the CECache class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CECache::CECache(bool keep_NULLs) : CESerializable() {
#ifdef BUILD_CECACHE_AS_ARRAY
	m_cache_data = new CEPtrArray();
#else
	m_cache_data = new CAPtrSeq();
#endif //BUILD_CECACHE_AS_ARRAY
	m_keep_NULLs=keep_NULLs;
}

#ifdef BUILD_CECACHE_AS_ARRAY
CECache::CECache(CEPtrArray* _cache_data,bool keep_NULLs) : CESerializable() {
	m_cache_data = CEPtrArray::get_copy(_cache_data, true); m_keep_NULLs=keep_NULLs;
}
#else
CECache::CECache(CAPtrSeq* _cache_data, bool keep_NULLs) : CESerializable() {
	m_cache_data = CAPtrSeq::get_copy(_cache_data, true); m_keep_NULLs=keep_NULLs;
}
#endif //BUILD_CECACHE_AS_ARRAY

CECache::~CECache() {
	delete m_cache_data;
}

CEString* CECache::toString(int mode) {
	CEString* res = new CEString("{");
	long s = m_cache_data->size();
	long i;
	for (i=0; i<s; i++) {
		CExprContainer* cart = (CExprContainer*)m_cache_data->GetAt(i);
		CExpression* src = cart->get_expr();
		CExpression* val = cart->get_info_expr();
		if (src != NULL ) { res->AddString(src->toString(mode)); }
		else { res->AddString("NULL"); }
		res->AddString("-->>");
		if (val != NULL ) { res->AddString(val->toString(mode)); }
		else { res->AddString("NULL"); }
		if (i < (s-1)) CEBasic::appendLineBreak(res ,mode);
	}
	res->AddString("}");
	return res;
}

const char* CECache::CLASS_NAME = "CECache";
const char* CECache::ClassName() const { return CLASS_NAME; }
CECache* CECache::clone() const { return new CECache(m_cache_data,m_keep_NULLs); }
CEObject* CECache::copy() const { return clone(); }
bool CECache::contain_vector(void * obj) {
	if (CESerializable::contain_vector(obj)) return true;
	if (m_cache_data == obj) return true;
	if (m_cache_data == NULL) { if (m_cache_data->contain_vector(obj)) return true; }
	return false;
}

void CECache::put_value(CExpression* src, CExpression* val) {
	if (src == NULL) return;
	if ((val != NULL) || (m_keep_NULLs)) { // add
		// search cached
		CExprContainer* cart = get_val_container(src);
		if (cart != NULL) {
			cart->set_info_expr(val);
		} else {
			cart = new CExprContainer(src, 0, 0, val); m_cache_data->add(cart);
		}
	} else { // remove from cache
		remove_val(src);
	}
}

CExpression* CECache::get_val(CExpression* src) {
	CExprContainer* cart = get_val_container(src);
	CExpression* res = NULL;
	if (cart != NULL) { res = cart->get_info_expr(); }
	return res;
}

CExprContainer* CECache::get_val_container(CExpression* src) {
	if (src == NULL) return NULL;
	long s = m_cache_data->size();
	long i;
	for (i=0; i<s; i++) {
		CExprContainer* cart = (CExprContainer*)m_cache_data->GetAt(i);
		if (src->equals(cart->get_expr())) return cart;
	}
	return NULL;
}

void CECache::remove_val(CExpression* src) {
	if (src == NULL) return;
	long s = m_cache_data->size();
	long i;
	for (i=0; i<s; i++) {
		CExprContainer* cart = (CExprContainer*)m_cache_data->GetAt(i);
		if (src->equals(cart->get_expr())) { m_cache_data->RemoveAt(i, true); break; }
	}
}

void CECache::clear_all() { m_cache_data->clear_all(); }
