////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ECharMap.cpp: implementation of the CECharMap class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// CECharMapCart
//////////////////////////////////////////////////////////////////////
CECharMapCart::CECharMapCart(const char* _value) : CESerializable() {
	m_pLeft = NULL;
	m_pRight = NULL;
	m_value = (_value != NULL)?(CEUtil::copyString(_value)):NULL;
	if (m_value == NULL) CEException::halt("new CECharMapCart(NULL) creation");
	m_mark = -1;
}

// is cloning usefull for CECharMapCart?
CECharMapCart::CECharMapCart(CECharMapCart* _pLeft, CECharMapCart* _pRight, const char* _value) : CESerializable() {
	m_pLeft = (_pLeft != NULL)?((CECharMapCart*)_pLeft->copy()):NULL;
	m_pRight = (_pRight != NULL)?((CECharMapCart*)_pRight->copy()):NULL;
	m_value = (_value != NULL)?(CEUtil::copyString(_value)):NULL;
	m_mark = -1;
}

CECharMapCart::~CECharMapCart() {
	if (m_pLeft != NULL) delete m_pLeft;
	if (m_pRight != NULL) delete m_pRight;
	if (m_value != NULL) CMemoryManager::mfree(m_value);
}

const char* CECharMapCart::GetValue() { return m_value; }
CECharMapCart* CECharMapCart::GetLeft() { return m_pLeft; }
CECharMapCart* CECharMapCart::GetRight() { return m_pRight; }
void CECharMapCart::SetLeft(CECharMapCart* _cart) { m_pLeft = _cart; }
void CECharMapCart::SetRight(CECharMapCart* _cart) { m_pRight = _cart; }
bool CECharMapCart::is_ident(const char* val) { return (m_value == val); }
bool CECharMapCart::is_equal(const char* val) { return CEUtil::equals(m_value, val); }

// 1 means 'val is greather' (order this-val is rising)
int CECharMapCart::compare_value(const char* val) {
	if (val == NULL) CEException::halt("CECharMapCart::compare_value(NULL)");
	if (m_value == NULL) CEException::halt("CECharMapCart::compare_value(): NULL source");
	int res = CEUtil::compare_ab(m_value, val);
	return res;
}

CEString* CECharMapCart::toString(int mode) {
	CEString* res = storeTree(0, mode);
	return res;
}

CEString* CECharMapCart::storeTree(int depth, int mode) {
	CEString* res = new CEString("");
	if (m_pLeft != NULL) {
		res->AddString(m_pLeft->storeTree(depth+1, mode));
		//CEBasic::appendLineBreak(res, mode);
	}
	if (depth > 0) {
		char* dot_str = CEBasic::GetSymbolLine(depth, '_');
		res->AddString(dot_str);
		CMemoryManager::mfree(dot_str);
	}
	res->AddString(" ");
	res->AddString((m_value != NULL)?m_value:"NULL");
	res->AddString(" ("); res->add_int(m_mark); res->AddString(")");
	CEBasic::appendLineBreak(res, mode);
	if (m_pRight != NULL) {
		res->AddString(m_pRight->storeTree(depth+1, mode));
		//CEBasic::appendLineBreak(res, mode);
	}
	return res;
}

const char* CECharMapCart::CLASS_NAME = "CECharMapCart";
const char* CECharMapCart::ClassName() const { return CLASS_NAME; }
CEObject* CECharMapCart::copy() const { return new CECharMapCart(m_pLeft, m_pRight, m_value); }
bool CECharMapCart::contain_vector(void * obj) {
	if (obj == NULL) return false;
	if ((m_value == obj) || (m_pLeft == obj) || (m_pRight == obj)) return true;
	if ((m_pLeft != NULL) && (m_pLeft->contain_vector(obj))) return true;
	if ((m_pRight != NULL) && (m_pRight->contain_vector(obj))) return true;
	return false;
}

//////////////////////////////////////////////////////////////////////
// CECharMap library
//////////////////////////////////////////////////////////////////////
CDebug* CECharMap::dbg = new CDebug("CECharMap");
CECharMap::CECharMap() : CESerializable() {
	m_root_value = NULL;
	m_norm_cntr = 0;
}

// clones root tree
CECharMap::CECharMap(CECharMapCart* _root, long _norm_cntr) : CESerializable() {
	if (_root != NULL) {
		m_root_value = (CECharMapCart*)_root->copy();
	} else {
		m_root_value = NULL;
	}
	m_norm_cntr = _norm_cntr;
}

CECharMap::~CECharMap() {
	if (m_root_value != NULL) delete m_root_value;
}

/*
*	Note: root cloning clones also the char-string values.
*	TODO: check if CECharMap::copy() can be useful.
*/
CEObject* CECharMap::copy() const { return new CECharMap(m_root_value, m_norm_cntr); }

bool CECharMap::is_exist(const char* val) {
	CECharMapCart* c = get_value_cart(val, false);
	return (c != NULL);
}

const char* CECharMap::getValueOrigin(const char* val, bool do_create) {
	CECharMapCart* c = get_value_cart(val, do_create);
	if (c != NULL) return c->GetValue();
	if (do_create) CEException::halt("CECharMap::getValueOrigin(): can't create data cart");
	return NULL;
}

void CECharMap::put_value(const char* val) {
	CECharMapCart* c = get_value_cart(val, true);
	// nothing more to do
}

CEString* CECharMap::toString(int mode) {
	CEString* res = new CEString("<CECharMap>");
	if (m_root_value != NULL) {
		CEBasic::appendLineBreak(res, mode);
		res->AddString(m_root_value->toString(mode));
	} else {
		res->AddString("<NULL root>");
	}
	return res;
}

const char* CECharMap::CLASS_NAME = "CECharMap";
const char* CECharMap::ClassName() const { return CLASS_NAME; }

bool CECharMap::contain_vector(void * obj) {
	if (obj == NULL) return false;
	if (m_root_value == obj) return true;
	if ((m_root_value != NULL) && (m_root_value->contain_vector(obj))) return true;
	return false;
}

/*
*	TODO: add automatic tree correction here
*	(keep CECharMapCart* best_root for value)
*	NO. ^^^ is too slow. This function should be fast, do post-creation noramlizing.
*/
CECharMapCart* CECharMap::get_value_cart(const char* val, bool do_create) {
	if (val == NULL) CEException::halt("CECharMap::get_value_cart(NULL)");

	if (m_root_value == NULL) {
		if (do_create) { m_root_value = new CECharMapCart(val); return m_root_value; }
		m_norm_cntr = 1;
		return NULL;
	}

	CECharMapCart* c = m_root_value;
	for(;c!=NULL;) {
		int cmp = c->compare_value(val);
		if (cmp == 0) return c;

		CECharMapCart* next = (cmp>0)?c->GetRight():c->GetLeft();
		if ((next == NULL) && (do_create)) {
			next = new CECharMapCart(val);
			if (cmp>0) {c->SetRight(next);} else {c->SetLeft(next);}
			
			// Note: normalization-required check needs the same time
			// as normalization procedure itself
			// normalize after avery 8(?) cart creation
			m_norm_cntr++;
			if (m_norm_cntr > 17) {
				NormalizeMap(m_root_value, NULL, false, false);
				m_norm_cntr = 0;
			}

			return next;
		}
		c = next;
	}
	return NULL; // unused branch
}

/*
*	NULL root means m_root_value is normalizing
*	TODO: add also fast normalization: |n2-n1|>1+last_lev_len condition instead of |n2-n1|>1
*/
void CECharMap::NormalizeMap(CECharMapCart* cart, CECharMapCart* root, bool as_left, bool size_is_marked) {
	dbg->log(45, "NormalizeMap");
	if (cart == NULL) return;
	long s = (size_is_marked)?(cart->m_mark):(CalcMapSize(cart, true));
	bool cart_changed = false;
	// TODO: continue implementation
	//long last_lev_len = ~sqrt{s};

	for(;;) {
		dbg->log(75, "Tree normalization", cart);
		if (CheckSizeMarked(cart) < 0) { CEException::halt(dbg, "Error: wrong size marks after the normalization"); }

		CECharMapCart* c1 = cart->GetLeft();
		CECharMapCart* c2 = cart->GetRight();
		bool left_exist = (c1 != NULL);
		bool right_exist = (c2 != NULL);
		long n1 = (left_exist)?(c1->m_mark):0;
		long n2 = (right_exist)?(c2->m_mark):0;

		long nd = n2-n1;
		if (nd > 1) { // get right (n2>n1)
			if (!right_exist) CEException::halt("CECharMap::NormalizeMap: Internal Error (!right_exist)");
			CECharMapCart* c21 = c2->GetLeft();
			long n21 = (c21 != NULL)?(c21->m_mark):0;
			long n22 = n2-1-n21;
			if (n1+1+n21 <= n22) { // 1.
				c2->m_mark = s; cart->m_mark = n1+1+n21;
				cart->SetRight(c21); c2->SetLeft(cart);
				cart = c2;
			} else { // 2.
				if (c21 == NULL) CEException::halt("CECharMap::NormalizeMap: Internal Error (c21 == NULL)");
				CECharMapCart* c211 = c21->GetLeft(); long n211 = (c211 != NULL)?(c211->m_mark):0;
				CECharMapCart* c212 = c21->GetRight(); long n212 = (c212 != NULL)?(c212->m_mark):0;
				c21->m_mark = s; cart->m_mark = n1+1+n211; c2->m_mark = n22+1+n212;
				c21->SetLeft(cart); c21->SetRight(c2);
				cart->SetRight(c211); c2->SetLeft(c212);
				cart = c21;
			}
			cart_changed = true;
			continue;
		} else if (nd < -1) { // get left (n1>n2)
			if (!left_exist) CEException::halt("CECharMap::NormalizeMap: Internal Error (!left_exist)");
			CECharMapCart* c12 = c1->GetRight();
			long n12 = (c12 != NULL)?(c12->m_mark):0;
			long n11 = n1-1-n12;
			if (n2+1+n12 <= n11) { // 1.
				c1->m_mark = s; cart->m_mark = n2+1+n12;
				cart->SetLeft(c12); c1->SetRight(cart);
				cart = c1;
			} else { // 2.
				if (c12 == NULL) CEException::halt("CECharMap::NormalizeMap: Internal Error (c12 == NULL)");
				CECharMapCart* c122 = c12->GetRight(); long n122 = (c122 != NULL)?(c122->m_mark):0;
				CECharMapCart* c121 = c12->GetLeft(); long n121 = (c121 != NULL)?(c121->m_mark):0;
				c12->m_mark = s; cart->m_mark = n2+1+n122; c1->m_mark = n11+1+n121;
				c12->SetRight(cart); c12->SetLeft(c1);
				cart->SetLeft(c122); c1->SetRight(c121);
				cart = c12;
			}
			cart_changed = true;
			continue;
		} else {
			// normalized
			break;
		}
	}

	// normalize branches
	if (cart->GetLeft() != NULL) NormalizeMap(cart->GetLeft(), cart, true, true);
	if (cart->GetRight() != NULL) NormalizeMap(cart->GetRight(), cart, false, true);

	// attach to root
	if (cart_changed) {
		if (root == NULL) {
			m_root_value = cart;
		} else {
			if (as_left) { root->SetLeft(cart); } else { root->SetRight(cart); }
		}
	}
	
	// internal check. TODO: move it to ALTA_DEBUG or comment after debug is finished
	if (CheckSizeMarked(cart) < 0) {
		dbg->log(0, "Error: wrong size marks after the normalization", cart);
		CEException::halt();
	}
	dbg->log(45, "NormalizeMap: OK", cart);
}

long CECharMap::CalcMaxMapRepth(CECharMapCart* root, bool mark_items) {
	if (root == NULL) return 0;
	long d1 = CalcMaxMapRepth(root->GetLeft(), mark_items);
	long d2 = CalcMaxMapRepth(root->GetRight(), mark_items);
	long res = (d1>d2)?d1:d2;
	res++;
	if (mark_items) root->m_mark = res;
	return res;
}

long CECharMap::CalcMinMapRepth(CECharMapCart* root, bool mark_items) {
	if (root == NULL) return 0;
	long d1 = CalcMinMapRepth(root->GetLeft(), mark_items);
	long res;
	if (d1 == 0) { res = d1; }
	else { long d2 = CalcMinMapRepth(root->GetRight(), mark_items); res = (d1<d2)?d1:d2; }
	res++;
	if (mark_items) root->m_mark = res;
	return res;
}

long CECharMap::CalcMapSize(CECharMapCart* root, bool mark_items) {
	if (root == NULL) return 0;
	long res = CalcMapSize(root->GetLeft(), mark_items);
	res += CalcMapSize(root->GetRight(), mark_items);
	res++;
	if (mark_items) root->m_mark = res;
	return res;
}

// declares error (-1) for wrong marks
long CECharMap::CheckSizeMarked(CECharMapCart* root) {
	if (root == NULL) return 0;
	long s1 = CheckSizeMarked(root->GetLeft());
	if (s1<0) return -1;
	long s2 = CheckSizeMarked(root->GetRight());
	if (s2<0) return -1;
	long sz = s1+s2+1;
	if (root->m_mark != sz) return -1;
	return sz;
}
