////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EDoubleArray.cpp: implementation of the CEDoubleArray class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CEDoubleArray::CEDoubleArray() {
	m_size = 0; m_size_allocated=0;
	m_data = NULL;
}

CEDoubleArray::CEDoubleArray(int sz) {
	m_data = NULL;
	m_size = 0; m_size_allocated=0;
	if (sz<0) sz=1;
	ToSize(sz);
}

CEDoubleArray::~CEDoubleArray() {
	if (m_data != NULL) { CMemoryManager::mfree(m_data); m_data=NULL; m_size_allocated=0; }
	m_size=0;
}

CEDoubleArray* CEDoubleArray::copy() const {
	CEDoubleArray* res = new CEDoubleArray();
	res->ToSize(m_size);
	for (int i=0; i<m_size; i++) { res->SetAt(i, GetAt(i)); }
	return res;
}

int CEDoubleArray::GetSize() const { return m_size; }

double CEDoubleArray::GetAt(int i) const {
	if ((i<0) || (i>=m_size) || (m_data == NULL)) return -1;
	return m_data[i];
}

void CEDoubleArray::SetAt(int i, double val) {
	if (i<0) return; // insert?

	if ((i>=m_size) || (m_data == NULL)) ToSize(i+1);
	m_data[i] = val;
}

void CEDoubleArray::Add(double val) {
	long pos = m_size;
	ToSize(pos+1);
	m_data[pos] = val;
	//CWSUtils::PopupMessage("Add(%ls),%ls",val,GetAt())
}

void CEDoubleArray::PreAllocateTo(int sz) {
	if(sz<=0) return;
	if (m_data == NULL) { m_data = (double*)CMemoryManager::mmalloc(sz*sizeof(double)); m_size_allocated=sz; return; }
	if (sz > m_size_allocated) { m_data = (double*)CMemoryManager::mrealloc(m_data, sz*sizeof(double)); m_size_allocated=sz; }
}

void CEDoubleArray::ToSize(int sz) {
	if(sz<=0) return;
	if (m_data == NULL) {
		m_data = (double*)CMemoryManager::mmalloc(sz*sizeof(double));
		m_size_allocated=sz;
		m_size=sz;
		return;
	}

	if (sz <= m_size) return;
	
	if (sz > m_size_allocated) {
		m_data = (double*)CMemoryManager::mrealloc(m_data, sz*sizeof(double));
		m_size_allocated=sz;
	}
	for (int i=m_size; i<sz; i++) { m_data[i] = 0; }
	m_size = sz;
}

void CEDoubleArray::RemoveAll() {
	bool kill_alloc=(m_size_allocated>10000);
	if (kill_alloc) {
		if (m_data != NULL) { CMemoryManager::mfree(m_data); m_data=NULL; m_size_allocated=0; }
	}
	m_size = 0;
}

void CEDoubleArray::RemoveAt(int i) {
	if ((i>=m_size) || (i<0)) return;
	for(long j=i+1; j<m_size; j++) { m_data[j-1]=m_data[j]; } //slow
	m_size--;
}

void CEDoubleArray::FillAll(double val) {
	for (int i=0; i<m_size; i++) { m_data[i] = val; }
}

bool CEDoubleArray::AreAllGTZero() { // are all > 0
	for (int i=0; i<m_size; i++) { if (m_data[i] <= 0) return false; }
	return true;
}

bool CEDoubleArray::AreAllGEZero() { // are all >= 0
	for (int i=0; i<m_size; i++) { if (m_data[i] < 0) return false; }
	return true;
}

long CEDoubleArray::SprintfTo(char* s, long len_max) {
	CEString res("<");
	for (int i=0; i<m_size; i++) {
		if (i>0) res.AddString(",");
		res.add(m_data[i]);
	}
	res.AddString(">");
	if ((len_max>=0) && (res.length()>len_max)) res.CutToSize(len_max);
	long l = sprintf(s,"%s",res.getData());
	return l;
}
