////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ENamedItem.cpp: implementation of the CENamedItem class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

unsigned CENamedItem::NI_OBJECT		= 0x01;
unsigned CENamedItem::NI_INT		= 0x02;
unsigned CENamedItem::NI_VOID_PTR	= 0x04;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CENamedItem::CENamedItem(CEObject* o, const char* id) : CEObject(true)
{
	m_obj = o;
	m_int_val = CEMap::NO_INTEGER_FOUND;
	m_ptr = NULL;
	m_id = CEUtil::copyString(id);
	m_item_type = CENamedItem::NI_OBJECT;
}

CENamedItem::CENamedItem(CEObject* o, bool do_copy_id, char* id) : CEObject(true)
{
	m_obj = o;
	m_int_val = CEMap::NO_INTEGER_FOUND;
	m_ptr = NULL;
	if (do_copy_id) { m_id=CEUtil::copyString(id); } else { m_id=id; }
	m_item_type = CENamedItem::NI_OBJECT;
}

CENamedItem::CENamedItem(long _val, char* id, bool do_copy_id) : CEObject(true)
{
	m_int_val = _val;
	m_obj = NULL;
	m_ptr = NULL;
	if (do_copy_id) { m_id=CEUtil::copyString(id); } else { m_id=id; }
	m_item_type = CENamedItem::NI_INT;
}

CENamedItem::CENamedItem(void* _p, const char* id) : CEObject(true)
{
	m_ptr = _p;
	m_obj = NULL;
	m_int_val = CEMap::NO_INTEGER_FOUND;
	m_id = CEUtil::copyString(id);
	m_item_type = CENamedItem::NI_VOID_PTR;
}

CENamedItem::~CENamedItem()
{
	CMemoryManager::mfree(m_id);
	if (((m_item_type & CENamedItem::NI_OBJECT)!=0) && (m_obj!=NULL)) delete m_obj;
}

const char* CENamedItem::getId() { return m_id; }
void CENamedItem::setId(const char* id_new) {
	if (m_id != id_new) {
		if (m_id!=NULL) delete m_id;
		m_id=((id_new!=NULL)?CEUtil::copyString(id_new):NULL);
	}
}

/*CEString* CENamedItem::toString(int mode)
{
	CEString* res = new CEString("{");
	res->AddString(m_id);
	res->AddString(":");

	bool val_added = false;
	if (m_item_type & NI_INT) {
		res->add_int(get_int());
		val_added = true;
	}
	if (m_item_type & NI_OBJECT) {
		if (val_added) res->AddString(",");
		res->AddString("CEObject*");
		val_added = true;
	}
	if (m_item_type & NI_VOID_PTR) {
		if (val_added) res->AddString(",");
		res->AddString("void*");
		val_added = true;
	}

	res->AddString("}");
	return res;
}*/

unsigned CENamedItem::get_item_type()
{
	return m_item_type;
}

// ************************************** CEObject operations
CEObject* CENamedItem::getObject()
{
	return m_obj;
}

void CENamedItem::set_object(CEObject *o)
{
	if ((m_item_type & CENamedItem::NI_OBJECT) == 0) m_item_type |= CENamedItem::NI_OBJECT;
	if (m_obj != o) {
		if (m_obj != NULL) delete m_obj;
		m_obj = o;
	}
}

// ************************************** integer operations
long CENamedItem::get_int()
{
	return m_int_val;
}

long CENamedItem::set_int(long _val)
{
	if ((m_item_type & CENamedItem::NI_INT) == 0) m_item_type |= CENamedItem::NI_INT;
	long old = m_int_val;
	m_int_val = _val;
	return old;
}

// ************************************** void pointer operations
void* CENamedItem::set_ptr(void* _p) {
	if ((m_item_type & CENamedItem::NI_VOID_PTR) == 0) m_item_type |= CENamedItem::NI_VOID_PTR;
	void* old = m_ptr;
	m_ptr = _p;
	return old;
}

void* CENamedItem::get_ptr() {
	return m_ptr;
}

const char* CENamedItem::CLASS_NAME = "CENamedItem";
const char* CENamedItem::ClassName() const {return CLASS_NAME; }

char* CENamedItem::toLine(int mode) {
	char* res = CEUtil::copyString("{");
	char* res_tmp = NULL;
	if (m_id != NULL) {
		res_tmp = CEUtil::concat(res, m_id); CMemoryManager::mfree(res); res = res_tmp;
	} else {
		res_tmp = CEUtil::concat(res, "NULL"); CMemoryManager::mfree(res); res = res_tmp;
	}
	res_tmp = CEUtil::concat(res, ": "); CMemoryManager::mfree(res); res = res_tmp;
	bool data_logged = false;

	if (m_item_type & CENamedItem::NI_OBJECT) {
		char* obj_line = NULL;
		if (m_obj != NULL) obj_line = m_obj->toLine(mode);
		if (obj_line != NULL) {
			res_tmp = CEUtil::concat(res, obj_line); CMemoryManager::mfree(res); res = res_tmp;
			CMemoryManager::mfree(obj_line);
		} else {
			res_tmp = CEUtil::concat(res, "NULL"); CMemoryManager::mfree(res); res = res_tmp;
		}
		data_logged = true;
	}

	if (m_item_type & CENamedItem::NI_INT) {
		res_tmp = (char*)CMemoryManager::mmalloc(CEUtil::length(res)+14);
		if (data_logged) sprintf(res_tmp, "%s, %ld", res, m_int_val);
		else sprintf(res_tmp, "%s%ld", res, m_int_val);
		CMemoryManager::mfree(res); res = res_tmp;
		data_logged = true;
	}

	if (m_item_type & CENamedItem::NI_VOID_PTR) {
		res_tmp = (char*)CMemoryManager::mmalloc(CEUtil::length(res)+7);
		if (data_logged) sprintf(res_tmp, "%s, *%ld", res, m_ptr);
		else sprintf(res_tmp, "%s*%ld", res, m_ptr);
		CMemoryManager::mfree(res); res = res_tmp;
		data_logged = true;
	}
	
	res_tmp = CEUtil::concat(res, "}"); CMemoryManager::mfree(res); res = res_tmp;
	return res;
}

//	*********************************** rare operations
bool CENamedItem::contain_vector(void * obj)
{
	if (m_id == obj) return true;
	if (m_item_type & CENamedItem::NI_OBJECT) {
		if ((m_obj == obj) || (m_obj->contain_vector(obj)))
			return true;
	}
	return false;
}
