////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ENumberArray.cpp: implementation of the CENumberArray class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CENumberArray::CENumberArray(int _size, int* _data_buf) : CESerializable() {
	m_size = _size;
	m_numbers = (int*)CMemoryManager::mmalloc(m_size*sizeof(int));
	long i;
	for(i=0; i<m_size; i++) { m_numbers[i] = _data_buf[i]; }
}

CENumberArray::CENumberArray(int _size) : CESerializable() {
	m_size = _size;
	m_numbers = (int*)CMemoryManager::mmalloc(m_size*sizeof(int));
	long i;
	for(i=0; i<m_size; i++) { m_numbers[i] = 0; }
}

CENumberArray::CENumberArray(int _val_1, int _val_2) : CESerializable() {
	m_size = 2;
	m_numbers = (int*)CMemoryManager::mmalloc(m_size*sizeof(int));
	m_numbers[0] = _val_1;
	m_numbers[1] = _val_2;
}

CENumberArray::CENumberArray(int _val_1, int _val_2, int _val_3) : CESerializable() {
	m_size = 3;
	m_numbers = (int*)CMemoryManager::mmalloc(m_size*sizeof(int));
	m_numbers[0] = _val_1;
	m_numbers[1] = _val_2;
	m_numbers[2] = _val_3;
}

CENumberArray::~CENumberArray() {
	CMemoryManager::mfree(m_numbers);
}

const char* CENumberArray::CLASS_NAME = "CENumberArray";
const char* CENumberArray::ClassName() const { return CLASS_NAME; }

CEString* CENumberArray::toString(int mode) {
	CEString* res = new CEString("[");
	long i;
	for(i=0; i<m_size; i++) {
		res->add_int(m_numbers[i]);
		if (i < (m_size-1)) res->AddString(", ");
	}
	res->AddString("]");
	return res;
}

CEObject* CENumberArray::copy() const { return clone(); }
CENumberArray* CENumberArray::clone() const { return new CENumberArray(m_size, m_numbers); }

bool CENumberArray::contain_vector(void * obj) {
	if (m_numbers == obj) return true;
	return false;
}

bool CENumberArray::obj_equals(CEObject* other) {
	if (other == NULL) return false;
	if (other == this) return true;
	if (!CEUtil::equals(ClassName(), other->ClassName())) return false;
	CENumberArray* o = (CENumberArray*)other;

	// compare containers
	if (o->get_size() != m_size) return false;
	long i;
	for(i=0; i<m_size; i++) { if (get_value(i) != o->get_value(i)) return false; }
	return true;
}

void CENumberArray::set_value(int i, int val) {
	if ((i < 0) || (i>=m_size)) CEException::halt("CENumberArray::set_value() invalid call");
	m_numbers[i] = val;
}
int CENumberArray::get_value(int i) {
	if ((i < 0) || (i>=m_size)) CEException::halt("CENumberArray::get_value() invalid call");
	return m_numbers[i];
}
int CENumberArray::get_size() { return m_size; }

void CENumberArray::realloc_size(int _size) {
	int old_size = m_size;
	m_size = _size;
	m_numbers = (int*)CMemoryManager::mrealloc(m_numbers, m_size*sizeof(int));
	// TODO: check if values are copying in reallocation
	// fill zero for new numbers
	long i;
	for(i=old_size; i<m_size; i++) { m_numbers[i] = 0; }
}

void CENumberArray::move_value(int from, int to) {
	if ((from < 0) || (from>=m_size) || (to < 0) || (to>=m_size)) CEException::halt("CENumberArray::move_value() invalid call");
	if (from == to) return;
	int val = m_numbers[from];

	long i;
	if (to > from) {
		for(i=from; i<to; i++) { m_numbers[i] = m_numbers[i+1]; }
	} else {
		for(i=from; i>to; i--) { m_numbers[i] = m_numbers[i-1]; }
	}

	m_numbers[to] = val;
}
