////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EProgressWatch.cpp: implementation of the CEProgressWatch class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CEProgressWatch::CEProgressWatch(const char* _description) : CEObject(true)
{
	m_description = CEUtil::copyString((_description != NULL)?_description:"Unknown Process");
	m_val_max = 64;
	m_val_curr = 0;

	printf("Progress: %s\n", m_description);
	printf("0%%");
	int i;
	for(i=0; i<(m_val_max-6); i++) { printf("."); } printf("100%%\n");
}

CEProgressWatch::~CEProgressWatch() {
	//printf("\n%s completed.\n", m_description);
	printf(".Done\n");
	CMemoryManager::mfree(m_description);
}

const char* CEProgressWatch::CLASS_NAME = "CEProgressWatch";
const char* CEProgressWatch::ClassName() const { return CLASS_NAME; }
bool CEProgressWatch::contain_vector(void * obj) {
	if (m_description == obj) return true;
	return false;
}

void CEProgressWatch::step(int val, int val_max) {
	if ((val < 0) || (val_max < 0)) return; // invalid step
	if (val > val_max) val = val_max;

	int val_new;
	if (val_max == 0) {
		val_new = m_val_max;
	} else {
		if (val<val_max) {
			val_new=CLMath::Round(((double)(m_val_max*val)+m_val_max/2)/val_max);
			//val_new = CLMath::Round(((double)(m_val_max*val))/val_max);
			if (val_new>=m_val_max) val_new=m_val_max;
		} else { // val>=val_max
			val_new=m_val_max;
		}
	}
	if (val_new == m_val_curr) return;

	// redraw progress bar
	int i;
	if (val_new > m_val_curr) {
		for(i=m_val_curr; i<val_new; i++) { putchar('#'); /*printf("#");*/ cout.flush(); }
		m_val_curr = val_new;
	} else {

#ifdef ALHEP_WIN
		for(i=m_val_curr; i>val_new; i--) {
			putchar('\b');
			putchar(' ');
			putchar('\b');
			//printf("\b");
		}
		m_val_curr = val_new;
#else //Linux, Unix
		// no back-symbol are allowed
		if ((val==0) && (val_new==1)) { // new line
			for(i=m_val_curr; i<m_val_max; i++) { putchar('#'); cout.flush(); }
			printf("*\n"); cout.flush();
			m_val_curr = 0;
		}
#endif //ALHEP_WIN
	}
}
