////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EPtrArray.h: interface for the CEPtrArray class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EPTRARRAY_H__9DB7A10F_0AA0_11D5_8A94_932004EF1C0E__INCLUDED_)
#define AFX_EPTRARRAY_H__9DB7A10F_0AA0_11D5_8A94_932004EF1C0E__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//##ModelId=3E3FF2D50196
class CEPtrArray : public CEObject  
{
	CEObject** m_data;
	long m_size;
	long m_allocated_size;
	long m_alloc_increment;

	bool m_allow_NULLs;
	long m_iterator;

	CEPtrArray(long _size, long _allocate_size, long _alloc_increment,
		long _iterator, CEObject** _buf, bool allow_NULLs);

public:
	CEPtrArray(bool allow_NULLs=false, long sz_incr=-1);
	virtual ~CEPtrArray();
	void SetAllowNULLs(bool allow_NULLs);

public:
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual char* toLine(int mode = 0);
	virtual CEObject* copy() const;
	virtual bool obj_equals(CEObject* other);
	virtual bool contain_vector(void * obj);

	// basic operations
	long size() const;
	CEObject* GetAt(long i);
	CEObject* GetAt2(long i);
	//CEObject* GetCopyAt(long i) const;
	const CEObject* GetAtConst(long i) const;
	void SetAt(long i, CEObject *obj, bool delete_old = false); // no-clone
	void InsertAt(long i, CEObject* obj); // no-clone
	void MoveFromTo(long from, long to);
	void RemoveAt(long i, bool delete_old);
	CEObject* ExtractAt(long i);
	void add(CEObject* obj);
	CEObject* get(long i);
	void add_array(CEPtrArray* other); // deletes other

	// iterator operation
	void reset_iterator();
	CEObject* first();
	CEObject* next();
	CEObject* last();
	bool has_next();

	CEPtrArray* clone() const;
	void clearall_without_delete_objects();
	static CEPtrArray* get_copy(const CEPtrArray* other, bool do_create=false);
	void clear_all();

private:
	void incAllocatedSize();
	void decAllocatedSize();
	bool checkDecSizeProfit();
};

#endif // !defined(AFX_EPTRARRAY_H__9DB7A10F_0AA0_11D5_8A94_932004EF1C0E__INCLUDED_)
