////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ESimpleMap.cpp: implementation of the CESimpleMap class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDebug* CESimpleMap::dbg = new CDebug("CESimpleMap");

CESimpleMap::CESimpleMap()
{
	m_pitems = new CEPtrArray();
}

CESimpleMap::~CESimpleMap()
{
	delete m_pitems;
}

const char* CESimpleMap::CLASS_NAME = "CESimpleMap";
const char* CESimpleMap::ClassName() const { return CLASS_NAME; }

CEObject* CESimpleMap::get(const char* id) {

	CENamedItem* item = get_NI(id);
	if (item != NULL) return item->getObject();
	return NULL;
}

long CESimpleMap::get_int(const char* id) {
	CENamedItem* item = get_NI(id);
	if (item != NULL) return item->get_int();
	return NO_INTEGER_FOUND;
}

void* CESimpleMap::get_ptr(const char* id) {
	CENamedItem* item = get_NI(id);
	if (item != NULL) return item->get_ptr();
	return NULL;
}

void CESimpleMap::put(const char* id, CEObject* o) { // object is deleted
	CENamedItem* item = get_NI(id);
	if (item != NULL) item->set_object(o);
	else m_pitems->add(new CENamedItem(o, id));
}

void CESimpleMap::put_int(char* id, long _val, bool do_copy_id) {
	CENamedItem* item = get_NI(id);
	if (item != NULL) item->set_int(_val);
	else m_pitems->add(new CENamedItem(_val, id, do_copy_id));
}

void CESimpleMap::put_ptr(const char* id, void* _p) {
	CENamedItem* item = get_NI(id);
	if (item != NULL) item->set_ptr(_p);
	else m_pitems->add(new CENamedItem(_p, id));
}

long CESimpleMap::size() { return m_pitems->size(); }

char* CESimpleMap::toLine(int mode)
{
	CEString* temp = new CEString("\\(<Map>\\)\\[IndentingNewLine]\n");
	long i;
	for(i=0; i<size(); i++)
	{
		temp->AddString("\\(");
		CENamedItem* item = (CENamedItem*)m_pitems->GetAt(i);
		if (item != NULL) {
			temp->AddString(item->getId());
			temp->AddString("-->");
			CEObject* itm_obj = item->getObject();
			if (itm_obj != NULL) {
				char* t2 = item->getObject()->toLine(mode);
				temp->AddString(t2);
				CMemoryManager::mfree(t2);
			} else {
				temp->AddString("NULL");
			}
		} else {
			temp->AddString("???");
		}
		temp->AddString("\\);\\[IndentingNewLine]\n");
	}
	temp->AddString("\\(</Map>\\)");
	char* res = CEUtil::copyString(temp->getData());
	delete temp;
	return res;
}

CEString* CESimpleMap::toString(int mode) { return new CEString(toLine(mode), true); }

bool CESimpleMap::contain_vector(void * obj)
{
	long _s = size();
	long i;
	for(i=0; i<_s; i++)
	{
		CENamedItem* item = (CENamedItem*)m_pitems->GetAt(i);
		if ((item == obj) || (item->contain_vector(obj)))
			return true;
	}
	return false;
}

void CESimpleMap::reset_iterator()
{
	m_pitems->reset_iterator();
}

bool CESimpleMap::has_next()
{
	return m_pitems->has_next();
}

CEObject* CESimpleMap::next()
{
	return m_pitems->next();
}

CENamedItem* CESimpleMap::get_NI(const char *id)
{
	long s = size();
	long i;
	for(i=0; i<s; i++) {
		CENamedItem* item = (CENamedItem*)m_pitems->GetAt(i);
		if (CEUtil::equals(item->getId(), id)) return item;
	}

	// no item found
	return NULL;
}
