////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EString.cpp: implementation of the CEString class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CEString::CEString(const char* data) : CEObject(true) {
	if(data==NULL)data="";
	m_len = CEUtil::length(data);
	m_data=NULL; m_len_alloc=0; // to initialized before first AllocBuf() call
	AllocBuf();
	CEUtil::CopyStringAdv(m_data,data,m_len+1);
}

CEString::CEString() {
	m_len = 0;
	m_data=NULL; m_len_alloc=0; // to initialized before first AllocBuf() call
	AllocBuf();
	*(m_data)='\0';
}

CEString::CEString(char* data, bool delete_deta) {
	if (data == NULL) { data="NULL"; delete_deta=false; }
	m_len = CEUtil::length(data);
	if (delete_deta) {
		m_data = data;
		m_len_alloc = m_len;
	} else {
		m_data=NULL; m_len_alloc=0; // to initialized before first AllocBuf() call
		AllocBuf();
		CEUtil::CopyStringAdv(m_data,data,m_len+1);
	}
}

CEString::CEString(const CEString& s) {
	m_len=s.m_len; m_len_alloc=s.m_len_alloc; // absolute copy here
	m_data = (char*)CMemoryManager::mmalloc(m_len_alloc+1);
	CEUtil::CopyStringAdv(m_data,s.m_data,m_len+1);
}

//[private]. Used for cloning only
CEString::CEString(const char* data, long len, long len_alloc) : CEObject(true) {
	m_len=len; m_len_alloc=len_alloc;
	m_data = (char*)CMemoryManager::mmalloc(m_len_alloc+1);
	CEUtil::CopyStringAdv(m_data,data,m_len+1);
}

CEString::CEString(CEString* other) {
	m_len = other->length();
	m_data=NULL; m_len_alloc=0; // to initialized before first AllocBuf() call
	AllocBuf();
	CEUtil::CopyStringAdv(m_data,other->getData(),m_len+1);
}

CEString::~CEString() {
	CMemoryManager::mfree(m_data);
}

void CEString::Reset() { // TODO: optimize this by avoiding delete/malloc operations
	if((m_len==0) && (m_len_alloc<20)){
		//do nothing
	}else{
		CMemoryManager::mfree(m_data);
		m_len = 0;
		m_data=NULL; m_len_alloc=0; // to initialized before first AllocBuf() call
		AllocBuf();
		*(m_data)='\0';
	}
}

void CEString::AllocBuf() { // to fit m_len, (m_len>m_len_alloc) is presumed
	long del=((m_len>m_len_alloc)?(m_len-m_len_alloc):0);
	//calculate required 'm_len_alloc':
	m_len_alloc=m_len;
	if (m_len<8) { m_len_alloc=16; }
	//else if (m_len<1024) { m_len_alloc=m_len_alloc*2; }
	else if (del>1024) { m_len_alloc+=del; }
	else if (del>8) { m_len_alloc+=del*2; }
	else { m_len_alloc+=16; }
	if (m_data==NULL) {
		m_data = (char*)CMemoryManager::mmalloc(m_len_alloc+1);
	} else {
		m_data = CMemoryManager::mReallocChar(m_data, m_len_alloc+1);
	}
}

void CEString::CutToSize(long s) {
	if ((s<0)||(s>=m_len)) return;
	m_len=s;
	*(m_data+m_len)='\0';
}

//char* CEString::getData() { return m_data; }
//long CEString::length() const { return m_len; }
char CEString::GetChar(int i) { return (((i>=0)&&(i<m_len))?m_data[i]:0); }
void CEString::SetChar(int i, char c) { if((i>=0)&&(i<m_len))m_data[i]=c; }

/*
*	ExtractBuffer(): NOT a fast function!
*	it remains string empty, BUT pre-allocate it to 16 symbols
*	So "s.ExtractBuffer()" is NOT faster than "CEUtils::copyString(s.getData())"
*	TODO: One should allow (m_data==NULL) to make it faster. Or declare "NO more operations after ExtractBuffer()"
*/
/*
char* CEString::ExtractBuffer() { //NOT a fast function
	char* res=m_data;
	m_len=0; m_data=NULL; m_len_alloc=0; // to initialized before first AllocBuf() call
	AllocBuf(); *(m_data)='\0';
	return res;
}
*/

void CEString::AddString(CEString* other, bool do_del_other) {
	if (other != NULL) { AddString(other->getData()); if (do_del_other) delete other; }
}

void CEString::AddString(const char* other) {
	if (other == NULL) other = "NULL";

	if (other != NULL) {
		long i;
		long other_len=0; for (i=0; *(other+i)!='\0'; i++) {other_len++;}
		long old_len=m_len;
		m_len+=other_len;
		if (m_len>m_len_alloc) AllocBuf();
		CEUtil::CopyStringAdv(m_data+old_len,other,other_len+1);
	}
}

/*
* The first case does not delete temporary string.
*/
char* CEString::toLine(int mode) {
	return CEUtil::concat("<CEString>:", getData());
}

bool CEString::equals(CEString *other) {
	if (other == NULL) return false;
	if (other == this) return true;
	bool res = false;
	res = CEUtil::equals(getData(), other->getData());
	//printf("res: %ld", res);
	return res;
}

bool CEString::obj_equals(CEObject* other) {
	if (other == NULL) return false;
	if (other == this) return true;
	if (other->GetObjectKind() & CEObject::KIND_STRING) { return equals((CEString*)other); }
	return false;
}

void CEString::insertString(const char *other, int pos) {
	if (pos >= m_len) { AddString(other); return; }
	if (pos<0) pos=0;
	if (other != NULL) {
		long other_len = CEUtil::length(other);
		long old_len = m_len;
		m_len+=other_len;
		if (m_len>m_len_alloc) AllocBuf();

		if (old_len-pos<=10) {
			for(long i=old_len; i>=pos; i--) { *(m_data + other_len + i) = *(m_data + i); }
		} else {
			CMemoryManager::mmemmove(m_data+pos+other_len, m_data+pos, (old_len-pos+1)*sizeof(char));
		}

		CEUtil::CopyStringAdv(m_data+pos,other,other_len);
	}
}

void CEString::insertString(CEString *other, int pos, bool delete_other) {
	if (other != NULL) { insertString(other->getData(), pos); if (delete_other) delete other; }
}

void CEString::add(double val, bool bNotExp) {
	char* data = (char*)CMemoryManager::mmalloc(20);
	if(bNotExp) { //no exponent
		sprintf(data, "%f", val);
	}else{ // use exponent if required
		sprintf(data, "%.2g", val);
	}
	AddString(data);
	CMemoryManager::mfree(data);
}

void CEString::add_int(long num, bool hex) {
	char* data = (char*)CMemoryManager::mmalloc(20);
	if (hex) {
		if (num>=0) sprintf(data, "0x%lx", num); else sprintf(data, "-0x%lx", -num);
	} else {
		sprintf(data, "%ld", num);
	}
	AddString(data);
	CMemoryManager::mfree(data);
}

void CEString::addChar(char sym) {
	m_len++;
	if (m_len>m_len_alloc) AllocBuf();
	*(m_data + m_len-1) = sym;
	*(m_data + m_len) = '\0';
}

CEString* CEString::clone() const { return new CEString(m_data,m_len,m_len_alloc); }

int CEString::GetObjectKind() {return CEObject::KIND_STRING;}
const char* CEString::CLASS_NAME = "CEString";
const char* CEString::ClassName() const { return CLASS_NAME; }
bool CEString::contain_vector(void * obj) {
	if (m_data == obj) return true;
	return false;
}
