////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EString.h: interface for the CEString class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ESTRING_H__9DB7A10E_0AA0_11D5_8A94_932004EF1C0E__INCLUDED_)
#define AFX_ESTRING_H__9DB7A10E_0AA0_11D5_8A94_932004EF1C0E__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//##ModelId=3E3FF2DB00CB
class CEString : public CEObject 
{
	CEString(const char* data, long len, long len_alloc);
public:
	virtual int GetObjectKind();
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual bool contain_vector(void * obj);

	virtual CEString* clone() const;
	void add_int(long num, bool hex=false);
	void add(double val, bool bNotExp=false);
	void insertString(CEString* other, int pos = 0, bool delete_other = true);
	void insertString(const char* other, int pos = 0);
	bool equals(CEString* other);

	virtual bool obj_equals(CEObject* other);
	virtual char* toLine(int mode);
	CEString(const char* data);
	CEString();
	CEString(char* data, bool delete_deta);
	CEString(const CEString& s);
	CEString(CEString* other);
	virtual ~CEString();

	char* getData() { return m_data; } //TODO: avoid this unsafe function!
	const char* GetBuffer() const { return m_data; }
	long length() const { return m_len; }
	/*
	*	ExtractBuffer(): NOT a fast function!
	*	it remains string empty, BUT pre-allocate it to 16 symbols
	*	So "s.ExtractBuffer()" is NOT faster than "CEUtils::copyString(s.getData())"
	*	TODO: One should allow (m_data==NULL) to make it faster. Or declare "NO more operations after ExtractBuffer()"
	*/
	//char* ExtractBuffer();

	void AddString(CEString* other, bool do_del_other = true);
	void AddString(const char* other);
	void addChar(char sym);
	void Reset();
	void CutToSize(long s);

	CEString& operator<<(CEString* str) { AddString(str, true); return *this; } //destroys 'str;
	CEString& operator<<(const char* s) { AddString(s); return *this; }
	CEString& operator<<(const long& i) { add_int(i); return *this; }
	//CEString& operatorv<<(const double& f) { add(f); return *this; } //conflicts with <<(long) for <<(unsigned long)
	//CEString& operator<<(const char& c) { addChar(c); return *this; } //Seems UNSAFE ("<<60"-->'<<(int)60' or <<'A')

	char GetChar(int i);
	void SetChar(int i, char c);
protected:
	//##ModelId=3E3FF2DB00CE
	char* m_data; // should be allocated to (m_len+1) size
	//##ModelId=3E3FF2DB00CD
	long m_len; // number of characters, not including '\0'
	long m_len_alloc; // number of characters allocated, not including '\0'
	void AllocBuf(); // to fit m_len, (m_len>m_len_alloc) is presumed
};

#endif // !defined(AFX_ESTRING_H__9DB7A10E_0AA0_11D5_8A94_932004EF1C0E__INCLUDED_)
