////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ETimeMark.cpp: implementation of the CETimeMark class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <time.h>

#define MAX_VALS_LOGGED	2
CETimeMark* CETimeMark::m_time_start = NULL;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CETimeMark::CETimeMark(int _hours, int _minutes, int _seconds, long _millis) : CEObject(true) {
	m_hours = _hours; m_minutes = _minutes; m_seconds = _seconds; m_millis = _millis;
}

CETimeMark::~CETimeMark() {}
const char* CETimeMark::CLASS_NAME = "CETimeMark";
const char* CETimeMark::ClassName() const { return CLASS_NAME; }

char* CETimeMark::toLine(int mode) {
	CEString* s = new CEString("{Timer: ");
	int vals_logged = 0;
	if ((vals_logged < MAX_VALS_LOGGED) && ((m_hours != 0) || (vals_logged>0))) {
		if (vals_logged>0) s->AddString(", "); s->add_int(m_hours); s->AddString("h"); vals_logged++; }
	if ((vals_logged < MAX_VALS_LOGGED) && ((m_minutes != 0) || (vals_logged>0))) {
		if (vals_logged>0) s->AddString(", "); s->add_int(m_minutes); s->AddString("min"); vals_logged++; }
	if ((vals_logged < MAX_VALS_LOGGED) && ((m_seconds != 0) || (vals_logged>0))) {
		if (vals_logged>0) s->AddString(", "); s->add_int(m_seconds); s->AddString("sec"); vals_logged++; }
	if ((vals_logged < MAX_VALS_LOGGED) && ((m_millis != 0) || (vals_logged>0))) {
		if (vals_logged>0) s->AddString(", "); s->add_int(m_millis); s->AddString("ms"); vals_logged++; }
	if (vals_logged == 0) s->AddString("0sec");
	s->AddString("}");
	char* res = CEUtil::copyString(s->getData());
	delete s;
	return res;
}

CEObject* CETimeMark::copy() const { return new CETimeMark(m_hours, m_minutes, m_seconds, m_millis); }

int CETimeMark::get_hours() { return m_hours; }
int CETimeMark::get_minutes() { return m_minutes; }
int CETimeMark::get_seconds() { return m_seconds; }
long CETimeMark::get_millis() { return m_millis; }

// does not affects other
void CETimeMark::substrTimeMark(CETimeMark* other) {
	if (other == NULL) return;

	// millis are processed separately
	m_millis -= other->get_millis();
	if (m_millis < 0) { m_millis += 1000; m_seconds--; }

	m_seconds -= other->get_seconds();
	if (m_seconds < 0) { m_seconds += 60; m_minutes--; }

	m_minutes -= other->get_minutes();
	if (m_minutes < 0) { m_minutes += 60; m_hours--; }

	m_hours -= other->get_hours();
	if (m_hours < 0) { m_hours += 24; }
}

// does not affects other
void CETimeMark::addTimeMark(CETimeMark* other) {
	if (other == NULL) return;

	m_millis += other->get_millis();
	if (m_millis >= 1000) { m_millis -= 1000; m_seconds++; }

	m_seconds += other->get_seconds();
	if (m_seconds >= 60) { m_seconds -= 60; m_minutes++; }

	m_minutes += other->get_minutes();
	if (m_minutes >= 60) { m_minutes -= 60; m_hours++; }

	m_hours += other->get_hours();
	if (m_hours > 24) {}
}

void CETimeMark::addTimeValue(int h, int m, int s, int mill) {
	m_millis += mill; if (m_millis >= 1000) { m_millis -= 1000; m_seconds++; }
	m_seconds += s; if (m_seconds >= 60) { m_seconds -= 60; m_minutes++; }
	m_minutes += m; if (m_minutes >= 60) { m_minutes -= 60; m_hours++; }
	m_hours += h; if (m_hours > 24) {}
}

bool CETimeMark::greatherThan(CETimeMark* other) {
	if (other == NULL) return false;

	if (m_hours != other->get_hours()) return (m_hours > other->get_hours());
	if (m_minutes != other->get_minutes()) return (m_minutes > other->get_minutes());
	if (m_seconds != other->get_seconds()) return (m_seconds > other->get_seconds());
	if (m_millis != other->get_millis()) return (m_millis > other->get_millis());
	return false;
}

bool CETimeMark::lessThan(CETimeMark* other) {
	if (other == NULL) return false;

	if (m_hours != other->get_hours()) return (m_hours < other->get_hours());
	if (m_minutes != other->get_minutes()) return (m_minutes < other->get_minutes());
	if (m_seconds != other->get_seconds()) return (m_seconds < other->get_seconds());
	if (m_millis != other->get_millis()) return (m_millis < other->get_millis());
	return false;
}

bool CETimeMark::lessThanZero() {
	return (m_hours >= 23);
}

void CETimeMark::copyFrom(CETimeMark* other) {
	if (other != NULL) {
		m_hours = other->get_hours();
		m_minutes = other->get_minutes();
		m_seconds = other->get_seconds();
		m_millis = other->get_millis();
	} else {
		m_hours = 0; m_minutes = 0; m_seconds = 0; m_millis = 0;
	}
}

void CETimeMark::divide(unsigned d) {
	//unsigned del;
	int del;

	del = (m_hours%(int)d);
	m_hours = (m_hours-del)/d;
	m_minutes += del*60;

	del = (m_minutes%(int)d);
	m_minutes = (m_minutes-del)/d;
	m_seconds += del*60;

	del = (m_seconds%(int)d);
	m_seconds = (m_seconds-del)/d;
	m_millis += del*1000;

	m_millis = (m_millis)/d;
}

DWORD CETimeMark::to_millis() {
	DWORD res = m_hours;
	res = (res*60 + m_minutes);
	res = (res*60 + m_seconds);
	res = (res*1000 + m_millis);
	return res;
}

void CETimeMark::truncate(bool ahead) {
	int steps_num = 7;
	int _steps[7] = {0,1,2,5,10,20,30};
	int i = 0;

	// millis
	if (m_millis != 0) { m_millis = 0; if (ahead) addTimeValue(0, 0, 1, 0); }

	// sec
	if ((m_hours == 0) && (m_minutes == 0)) {
		for (i=0; i<steps_num; i++) {
			if (m_seconds == _steps[i]) return;
			if (m_seconds < _steps[i]) {
								if (ahead) { m_seconds = _steps[i]; } else { m_seconds = _steps[(i>0)?(i-1):0]; }
				return;
			}
		}
		if (!ahead) { m_seconds = _steps[steps_num-1]; return; }
	}
	if (m_seconds != 0) { m_seconds = 0; if (ahead) addTimeValue(0, 1, 0, 0); }

	// min
	if (m_hours == 0) {
		for (i=0; i<steps_num; i++) {
			if (m_minutes == _steps[i]) return;
			if (m_minutes < _steps[i]) {
				if (ahead) { m_minutes = _steps[i]; } else { m_minutes = _steps[(i>0)?(i-1):0]; }
				return;
			}
		}
		if (!ahead) { m_minutes = _steps[steps_num-1]; return; }
	}
	if (m_minutes != 0) { m_minutes= 0; if (ahead) addTimeValue(1, 0, 0, 0); }

	// h
	for (i=0; i<steps_num; i++) {
		if (m_hours == _steps[i]) return;
		if (m_hours < _steps[i]) {
			if (ahead) { m_hours = _steps[i]; } else { m_hours = _steps[(i>0)?(i-1):0]; }
			return;
		}
	}
	if (!ahead) { m_hours = _steps[steps_num-1]; return; }
}

// by 1/2/5/10/20/30 sec, 1/2/5/10/20/30 min, 1/2/5/10/20h
// mod should be truncated
void CETimeMark::truncate(CETimeMark* mod, bool ahead) {
	if (mod == NULL) { truncate(ahead); return; }

	int k = 0;

	// millis
	if (m_millis != 0) { m_millis = 0; if (ahead) addTimeValue(0, 0, 1, 0); }

	int mod_h = mod->get_hours(); int mod_m = mod->get_minutes(); int mod_s = mod->get_seconds();
	// sec
	if ((mod_h == 0) && (mod_m == 0)) {
		k = m_seconds/mod_s;
		int sec_new = k*mod_s;
		if ((ahead) && (sec_new<m_seconds)) sec_new += mod_s;
		m_seconds = sec_new;
		return;
	}
	if (m_seconds != 0) { m_seconds = 0; if (ahead) addTimeValue(0, 1, 0, 0); }

	// min
	if (mod_h == 0) {
		k = m_minutes/mod_m;
		int min_new = k*mod_m;
		if ((ahead) && (min_new<m_minutes)) min_new += mod_m;
		m_minutes = min_new;
		return;
	}
	if (m_minutes != 0) { m_minutes= 0; if (ahead) addTimeValue(1, 0, 0, 0); }

	// h
	k = m_hours/mod_h;
	int h_new = k*mod_h;
	if ((ahead) && (h_new<m_hours)) h_new += mod_h;
	m_hours = h_new;
}

CETimeMark* CETimeMark::GetCurrTimeAbs() {
	CETimeMark* res = NULL;
	tm curr_time;

#ifdef ALHEP_WIN
	unsigned ms = ::_getsystime(&curr_time);
//	unsigned ms=0;
//	time_t t_curr;
//	time(&t_curr);
//	curr_time = *localtime(&t_curr);
#else // Linux, Unix
	unsigned ms=0;
	time_t t_curr;
	time(&t_curr);
	curr_time = *localtime(&t_curr);
#endif //ALHEP_WIN

	int h = curr_time.tm_hour;
	int m = curr_time.tm_min;
	int s = curr_time.tm_sec;

	res = new CETimeMark(h, m, s, ms);

	//long l_ms = (long)(clock()*1000/CLOCKS_PER_SEC);
	//int ms = (int)(l_ms%1000);

	return res;
}

CETimeMark* CETimeMark::GetStartTimeAbs() { return m_time_start; }

void CETimeMark::SetStartTimeCurr() {
	if (m_time_start != NULL) delete m_time_start;
	m_time_start = GetCurrTimeAbs();
}

CETimeMark* CETimeMark::GetCurrTimeMark() {
	CETimeMark* res = GetCurrTimeAbs();
	if (m_time_start != NULL) res->substrTimeMark(m_time_start);
	return res;
}

CETimeMark* CETimeMark::GetZeroTimeMark() { return new CETimeMark(0, 0, 0, 0); }
