////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EUtil.cpp: implementation of the CEUtil class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

// Microsoft specific [?]
#include <direct.h>		// for file operations
#include <sys/stat.h>	// for file operations (used in IsFileExist()), Nt: the header file may be different on linux/unix/etc.
//[linux/unix]#include <unistd.h>	// for file operations ('access()' used in IsFileExist())
#include  <io.h>		// for file operations (e.g. 'access()' used in IsFileExist())

CDebug* CEUtil::dbg = new CDebug("CEUtil");

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CEUtil::CEUtil() : CEObject(true) {}
CEUtil::~CEUtil() {}

int CEUtil::m_WarningLongStringCnt=0;
void CEUtil::WarningLongString() {
	if (m_WarningLongStringCnt>0) dbg->log(0,"<Warning: too long string>");
	m_WarningLongStringCnt++;
}
int CEUtil::m_ErrorLongStringCnt=0;
void CEUtil::ErrorLongString() {
	if (m_ErrorLongStringCnt>0) { dbg->log(0,"<Error: too long string>"); CEException::halt("Max string length exceeded"); }
	m_ErrorLongStringCnt++;
}

char* CEUtil::copyString(const char* other)
{
	if (other == NULL) return NULL;
	return copySubString(other, length(other), 0);
}

unsigned CEUtil::UTL_COPY_NUMBERS			= 0x01;
unsigned CEUtil::UTL_COPY_SYMBOLS			= 0x02;
unsigned CEUtil::UTL_COPY_NO_FIRST_NUMBER	= 0x04;
unsigned CEUtil::UTL_COPY_SKIP_BSLASH_CMD	= 0x08; // {p\_1->p1} but {p\gamma_y->pgamma_y}
char* CEUtil::copyStringSyms(const char* src, unsigned sym_flags, const char* sym_list) {
	int len = CEUtil::length(src);
	char* res = (char*)CMemoryManager::mmalloc(len+1);

	int i_src = 0; int i_res = 0; bool bSkipNext=false; bool bSkipNextIfNotSym=false;
	for(i_src=0; i_src<len; i_src++) {
		char c = *(src+i_src);
		if(bSkipNext){ bSkipNext=false; continue; }
		bool is_valid_sym = false;

		//Symbols
		if (sym_flags & UTL_COPY_SYMBOLS) {
			if(((c>='a') && (c<='z')) || ((c>='A') && (c<='Z')))is_valid_sym=true;
		}
		//Numbers
		if ((!is_valid_sym) && (sym_flags & UTL_COPY_NUMBERS)) {
			if ((c>='0') && (c<='9')) { is_valid_sym=((i_res>0) || ((sym_flags&UTL_COPY_NO_FIRST_NUMBER)==0)); }
		}
		if(bSkipNextIfNotSym){ bSkipNextIfNotSym=false; if(!is_valid_sym)continue; }

		if ((!is_valid_sym) && (sym_flags & CEUtil::UTL_COPY_SKIP_BSLASH_CMD)){ if(c=='\\'){ bSkipNextIfNotSym=true; continue; } }

		//Additional symbols
		if ((!is_valid_sym) && (sym_list != NULL)) {
			if(findSymbol(sym_list, c) >= 0)is_valid_sym = true;
		}
		if(is_valid_sym){ *(res+i_res)=c; i_res++; }
	}
	*(res+i_res) = '\0';
	return res;
}

/*
*	Returns position of end_pos symbol;
*	Note: Memory size is length+1.
*/
long CEUtil::length(const char *s, char end_pos)
{
	if (s == NULL) { dbg->log(0, "<Error> CEUtil::length(NULL)"); CEException::halt(); }
	unsigned long l;
	for(l = 0; l<MAX_STRING_LENGTH; l++) {
		if (l > WARNING_STRING_LENGTH) WarningLongString();
		//if ((c == end_pos) || (c == '\0')) return l;
		if (*s == end_pos) return l;
		s++;
	}
	ErrorLongString();
	return 0;
}

bool CEUtil::equals(const char *a, const char *b)
{
	if (a == b) return true;
	if ((a == NULL) || (b == NULL)) { dbg->log(0, "<Error> CEUtil::equals(NULL parameter)"); CEException::halt(); }
	for(long i = 0; i<MAX_STRING_LENGTH; i++) {
		if (i > WARNING_STRING_LENGTH) WarningLongString();
		if (*a != *b) return false;
		if (*a == '\0') return true;
		a++; b++;
	}
	ErrorLongString();
	return false;
}

long CEUtil::findSymbol(const char *s, char c) { return findSymbol(s,c,0,false); }
long CEUtil::findSymbol(const char *s, char c, long start_pos, bool check_length) {
	if (s == NULL) { dbg->log(0, "<Error> CEUtil::findSymbol(NULL string)"); CEException::halt(); }
	//long len = length(s);
	long i;
	for(i = start_pos; i<MAX_STRING_LENGTH; i++) {
		if (check_length) {
			if (i > WARNING_STRING_LENGTH) WarningLongString();
			if (i > MAX_STRING_LENGTH) ErrorLongString();
		}
		char sym_curr = *(s + i);
		if (sym_curr == c) return i;
		if (sym_curr == '\0') return -1;
	}
	return -1;
}

long CEUtil::findSymbolFromTo(const char *s, char c, long start_pos, long end_pos) {
	if (s == NULL) { dbg->log(0, "<Error> CEUtil::findSymbol(NULL string)"); CEException::halt(); }
	long i;
	for(i=start_pos; i<=end_pos; i++) {
		char sym_curr = *(s + i);
		if (sym_curr == c) return i;
		if (sym_curr == '\0') return -1;
	}
	return -1;
}

char* CEUtil::copySubString(const char *s, long len, long start_pos) {
	if (s == NULL) { dbg->log(0, "<Error> CEUtil::copySubString(NULL string)"); CEException::halt(); }
	if ((len < 0) || (start_pos < 0)) { dbg->log(0, "<Error> CEUtil::copySubString(invalid parameters)"); CEException::halt(); }

	char* res = (char*)CMemoryManager::mmalloc(len + 1);
	CEUtil::CopyStringAdv(res,s+start_pos,len);
	*(res + len) = '\0';
	return res;
}

// does not appends '\0'
// len == -1 means copy all smbols from the src string
// WARNING: trg string should be allocated to keep whole source string
void CEUtil::copySymsTo(char *trg, const char *src, long len, long max_len) {
	if ((trg == NULL) || (src == NULL)) return;
	if (len<0) { len=length(src); }
	if(len>=max_len) { ErrorLongString(); return; }
	CEUtil::CopyStringAdv(trg,src,len);
}

char* CEUtil::removeSubString(const char *s, long start_pos, long len) {
	if (s == NULL) { dbg->log(0, "<Error> CEUtil::removeSubString(NULL string)"); CEException::halt(); }

	//do_copy
	long len_old = length(s);
	if ((start_pos < 0) || (len < 0) || ((start_pos+len) > len_old)) {
		dbg->log(0, "Error: CEUtil::removeSubString(invalid parmeters)"); return NULL;
	}

	if (len == 0) {
		return copyString(s);
	}

	char* res = (char*)CMemoryManager::mmalloc(len_old - len + 1);
	CEUtil::CopyStringAdv(res,s,start_pos);
	if (len_old>start_pos+len) {
		CEUtil::CopyStringAdv(res+start_pos,s+start_pos+len,len_old-start_pos-len+1); // also copy '\0'
	} else {
		*(res+len_old-len)='\0';
	}
	//long i; for(i = 0; i<start_pos; i++) {*(res + i) = *(s + i);}
	// for(i = start_pos+len; i<len_old; i++) {*(res - len + i) = *(s + i);}
	//*(res + len_old - len) = '\0';
	return res;
}

char* CEUtil::copyStringTo(const char *s, long end_pos) { return copySubString(s, end_pos+1, 0); }
char* CEUtil::copyStringFrom(const char *s, long start_pos) { return copySubString(s, length(s)-start_pos, start_pos); }
char* CEUtil::copyStringFromTo(const char *s, long start_pos, long end_pos) { return copySubString(s, end_pos-start_pos+1, start_pos); }

char* CEUtil::concat(const char *a, const char *b, char insert_sym) {
	if ((a == NULL) || (b == NULL)) { dbg->log(0, "CEUtil::concat(NULL parameter)"); CEException::halt(); }
	long lena = length(a);
	long lenb = length(b);
	if ((lena+lenb) > WARNING_STRING_LENGTH) WarningLongString();
	if ((lena+lenb) > MAX_STRING_LENGTH) ErrorLongString();

	char* res = (char*)CMemoryManager::mmalloc(lena + lenb + ((insert_sym == '\0')?1:2));
	CEUtil::CopyStringAdv(res,a,lena);
	// for(i=0; i<lena; i++) { *(res + i) = *(a + i); }
	long i=lena;
	if (insert_sym != '\0') { *(res+i)=insert_sym; i++; }
	CEUtil::CopyStringAdv(res+i,b,lenb+1); // also copy '\0'
	//long j; for(j=0; j<=lenb; j++) { *(res + i) = *(b + j); i++; }
	return res;
}

char* CEUtil::composeId(const char *a, const char *b) { return concat(a, b, CConst::DN_DOT); }

/*
*	Note: impementation is the same to
*	int cpm_val = compare_ab(a, b);
*	return (cpm_val>0);
*	but we avoid extra far-function call for faster work
*/
bool CEUtil::a_less_b(const char *a, const char *b) {
	if ((a == NULL) || (b == NULL)) {
		dbg->log(0, "<Error> CEUtil::a_less_b(NULL parameter)");
		/*
		const char* a_log=a; if (a_log==NULL) a_log="NULL";
		const char* b_log=b; if (b_log==NULL) b_log="NULL";
		CEString tmp_log("call is: CEUtil::a_less_b("); tmp_log.AddString(a_log); tmp_log.AddString(","); tmp_log.AddString(b_log); tmp_log.AddString(")");
		dbg->log(0, tmp_log.getData());
		*/
		CEException::halt();
	}

	long i;
	for(i=0; i<MAX_STRING_LENGTH; i++) {
		if (i > WARNING_STRING_LENGTH) WarningLongString();

		if (*a == *b) {
			if (*a == '\0') return false;
		}
		else return (*a < *b);

		a++; b++;
	}
	ErrorLongString();
	return false;
}

/*
*	fast function for internal use (no extra check are performed)
*	string len check should probably be also extracted
*	1 means a<b, -1 means a>b
*/
int CEUtil::compare_ab(const char* a, const char* b) {
	if (a == b) return 0;
	long i;
	for(i=0; i<MAX_STRING_LENGTH;) {
		if (i > WARNING_STRING_LENGTH) WarningLongString();
		int ca = *a;
		int cb = *b;
		if (ca != cb) {
			if (ca == '\0') return 1;
			if (cb == '\0') return -1;
			return (ca<cb)?1:(-1);
		}
		if (ca == '\0') return 0;
		a++; // a+=sizeof(char);
		b++; // b+=sizeof(char);
	}
	ErrorLongString();
	return 0;
}

bool CEUtil::compareSubStrings(const char *s1, long start1, long end1, const char *s2, long start2, long end2)
{
	if ((s1 == NULL) || (s2 == NULL)) { dbg->log(0, "<Error> CEUtil::compareSubStrings(NULL parameter)"); CEException::halt(); }
	if (end2<0) end2=length(s2)-1;

	if ((end2-start2) != (end1-start1)) return false;

	char c1;
	char c2;
	long i1=start1;
	long i2=start2;
	for(;i1<=end1;) {
		c1 = *(s1 + i1);
		c2 = *(s2 + i2);
		if (c1 != c2) return false;
		if (c1 == '\0') return true;// both the strings are completed before or at the end position
		i1++;i2++;
	}
	return true;
}

bool CEUtil::LXOR(bool a, bool b)
{
	if (a && b) return false;
	if ((!a) && (!b)) return false;
	return true;
}

bool CEUtil::LEQU(bool a, bool b)
{
	return ((a && b) || ((!a) && (!b)));
	//return ((!a) == (!b));
}

bool CEUtil::LCPY(bool a) { // copies int to "true" or "false"
	return (a)?true:false;
}

// string should be allocated, and NOT terminated before 'f_pos'
int CEUtil::addACBNumber(char* str, long n, int f_pos) {
	if (str == NULL) return -1;
	if (n<0)n=0; //error

	long val32;
	int str_len;
	if (n >= 0x20) {
		val32= (n & 0x1f); //%32
		str_len= CEUtil::addACBNumber(str, n>>5, f_pos);
	} else {
		val32= n;
		str_len= f_pos;
	}
	
	if(val32<10){ *(str+str_len) = '0'+(char)val32; }	//0..9
	else{ *(str+str_len) = 'a'+(char)(val32-10); }		// 10..31
	str_len++;
	*(str+str_len)='\0';
	return str_len;
}

bool CEUtil::IsSymbol(char c) { return (((c>='a') && (c<='z')) || ((c>='A') && (c<='Z'))); }
bool CEUtil::IsNumber(char c) { return ((c>='0') && (c<='9')); }
bool CEUtil::IsAllowedFileName(const char* str) {
	if(str == NULL) return false;
	long l=CEUtil::length(str); if((l<=0)||(l>250))return false;
	bool bRes=true;
	for(long i=0; i<l; i++) {
		char c=*(str+i);
		if (((c>='a')&&(c<='z')) ||
			((c>='A')&&(c<='Z')) ||
			((c>='0')&&(c<='9')) ||
			(c=='_') || (c=='-')
			) continue;
		bRes=false; break;
	}
	return bRes;
}

void CEUtil::CopyStringAdv(char *trg, const char *s, long nSymsToCpy) { // nSymsToCpy should be length(s)+1 or -1
	if ((s == NULL)||(trg == NULL)) return;
	if (nSymsToCpy>10) {
		CMemoryManager::mmemmove(trg, s, (nSymsToCpy)*sizeof(char)); // copy nSymsToCpy symbols
	} else if (nSymsToCpy>=0) {
		for(long i=0; i<nSymsToCpy; i++) { *(trg+i)=*(s+i); } // copy nSymsToCpy symbols
	} else {
		for(long i=0;; i++) { char c=*(s+i); *(trg+i)=c; if(c=='\0')break; }
	}
}

char* CEUtil::addint(const char* a, int i) {
	int l = length(a);
	char* res = (char*)CMemoryManager::mmalloc(l+30);
	sprintf(res, "%s.%ld", a, i);
	return res;
}

/*
int CEUtil::Round(double val) {
	int res = (int)val; double gap = val-res;
	if (gap > 0.5) { res++; } else if (gap < (-0.5)) { res--; }
	return res;
}

int CEUtil::Trunc(double val) {
	int res = (int)val; double gap = val-res;
	if (gap < 0) { res--; }
	return res;
}
*/

int CEUtil::GetDigitsNum(long val) {
	if (val < 0) return GetDigitsNum(-val);
	if (val == 0) return 0;
	int res = 1; for(;val>=10;) { val = val/10; res++; }
	return res;
}
//, bool bForceExp
void CEUtil::PrintExpFormSimple(char* s, double val, char cExp) { // to D0 always
	bool bMinus=false; if(val<0){ val=-val; bMinus=true; }
	sprintf(s, ((bMinus)?"-%.15g%c0":"%.15g%c0"), val, cExp);
	//if (bForceExp) { sprintf(s, ((bMinus)?"-%.15g%c0":"%.15g%c0"), val, cExp); } else { sprintf(s, ((bMinus)?"-%.15g":"%.15g"), val, cExp); }
}

/*
*	Note1: unexpected problems spotted with inline 'IsInf' and -O2 on linux.
*		==> make'em NOT inline
*	Note2: 'unsigned long long' type is absent in GCC ==> the following implementation works at Win32 only:
*		static double GetNaN() { unsigned long long raw=0x7ff8000000000000; return *(double*)&raw; }
*		static double GetInf() { unsigned long long raw=0x7ff0000000000000; return *(double*)&raw; }
*		static bool IsInf(double val) { return (((*(unsigned long long*)&val) & 0x7ff8000000000000) == 0x7ff8000000000000); }
*/
bool CEUtil::IsNaN(double val) { return (val!=val); }
double CEUtil::GetNaN() { double res=0; ((unsigned long*)&res)[1]=0x7ff80000; return res; }
bool CEUtil::IsInf(double val) { return ( (((unsigned long*)&val)[1] & 0x7ff80000) == 0x7ff00000 ); }
double CEUtil::GetInf() { double res=0; ((unsigned long*)&res)[1]=0x7ff00000; return res; }

/*
*	Note: this function may be fastered by improving of CLMath::frexp10()
*	s is already allocated to at least of 32
*/
void CEUtil::PrintExpForm(char* s, double val, char cExp) {//, bool bForceExp
	double valAbs=((val>=0)?val:-val);
	if (((valAbs<10000) && (valAbs>=0.1)) || (valAbs==0.)) { CEUtil::PrintExpFormSimple(s, val, cExp); return; }//, bForceExp

	int d=0;
	double m=CLMath::frexp10(val,&d); // works OK with negative value
	if (m<10)/*No Error*/ {
		//sprintf(s, "%.18gD%d", m, d);
		// Nt: leave first 10 digits only <== CLMath::frexp10() makes error in last digits
		sprintf(s, "%.10g%c%d", m, cExp, d);
		//sprintf(s, ((bMinus)?"-%.15gD%ld":"%.15gD%d"), val, d);
	} else { // error in CLMath::frexp10()
		dbg->log(0,"Internal warning in CEUtil::PrintAsFortran(). frexp10() failed.");
		// Ignore frexp10 result
		long d=0; bool bMinus=false; if(val<0){ val=-val; bMinus=true; }
		for(;val>=10;) { val = val/10; d++; }
		if(val>0){ // the same to if(val!=0)
			for(;val<0.1;) { val = val*10; d--; }
		}
		if ((val<1) && (d!=0)) { val = val*10; d--; }
		sprintf(s, ((bMinus)?"-%.15g%c%d":"%.15g%c%d"), val, cExp, d);
	}
}

// Microsoft specific
/*
char* CEUtil::GetCurrentPath() { // result is to be deleted
	char* res = (char*)CMemoryManager::mmalloc(_MAX_PATH);
	//_getdrive()
	if (_getcwd(res, _MAX_PATH)==NULL) { sprintf(res,"[Path unknown]/"); }
	return res;
}
*/

bool CEUtil::atolSafe(const char* s, long* res) {
	long val=atol(s);
	bool atol_OK=true;
	// check up conversion
	if (val==0) { // is string " +/-000"
		int op_num_len=CEUtil::length(s);
		if(op_num_len>0){
			bool zeroFound=false; int iActSym=0;
			for(int op_num_cnt=0; op_num_cnt<op_num_len; op_num_cnt++) {
				char c=*(s+op_num_cnt);
				if(c==' '){ if(iActSym==0){ continue; }else{ atol_OK=false; break; } }
				iActSym++;
				if((c=='+')||(c=='-')){ if(iActSym==1){ continue; }else{ atol_OK=false; break; } }
				if(c=='0'){ zeroFound=true; }else{ atol_OK=false; break; }
			}
			if(!zeroFound){ atol_OK=false; }
		}else{ atol_OK=false; }
	}
	if (atol_OK) { *res=val; }
	return atol_OK;
}
bool CEUtil::atofSafe(const char* s, double* res) {
	double val=(double)atof(s);
	bool res_OK=true;
	// check up conversion
	if(val==0.){ // is string "0" or " +/-00.00"
		int op_num_len=CEUtil::length(s); if(op_num_len<=0)res_OK=false;
		bool zeroFound=false; int iActSym=0; int iDots=0;
		for(int op_num_cnt=0; op_num_cnt<op_num_len; op_num_cnt++) {
			char c = *(s+op_num_cnt);
			if(c==' '){ if(iActSym==0){ continue; }else{ res_OK=false; break; } }
			iActSym++;
			if((c=='+')||(c=='-')){ if(iActSym==1){ continue; }else{ res_OK=false; break; } }
			if(c=='.'){ iDots++; if(iDots>1){ res_OK=false; break; } continue; }
			if(c=='0'){ zeroFound=true; }else{ res_OK=false; break; }
		}
		if (!zeroFound) { res_OK=false; }
	}
	if (res_OK) { *res = val; }
	return res_OK;
}
/*
// Possible extension to convert "inf" & "NaN"
double atofSafe(const char *str) {
    double dVal;
#if defined(RDMS_LINUX)
    dVal = atof(str);
#else //RDMS_WINDOWS
    char sign = 0;
    char *cp = str;
    if (*cp == '+' || *cp == '-') sign = *cp++;

    if (stricmp(cp, "inf") == 0) {
		if (!sign || sign == '+') dVal = //assign +Inf
		else dVal = //assign -Inf
    } else if (stricmp(cp, "nan") == 0) dVal = //assign NaN
    else dVal = atof(cp); //valid number
#endif
    return dVal;
}
*/

long CEUtil::findSubString(const char *s, const char *sub) {
	if ((s==NULL)||(sub==NULL)) return -1;
	long sub_i=0;
	for(long i=0; i<MAX_STRING_LENGTH; i++) {
		char c = *(s+i);
		char sub_c = *(sub+sub_i);
		if (sub_c == '\0') return (i-sub_i);
		if (c == '\0') return -1;
		if (sub_c == c) { sub_i++; } else { sub_i=0; }
	}
	return -1;
}
