////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ExprContainer.cpp: implementation of the CExprContainer class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////	CEObjIdCart
CEObjIdCart::CEObjIdCart(CEObject* _obj, int id) : CEObject(true) { m_obj=_obj; m_id=id; }
CEObjIdCart::~CEObjIdCart() { if(m_obj!=NULL){ delete m_obj; m_obj=NULL; } }
CEObjIdCart* CEObjIdCart::clone() const { return new CEObjIdCart(((m_obj!=NULL)?m_obj->copy():NULL),m_id); }
const char* CEObjIdCart::CLASS_NAME = "CEObjIdCart";
const char* CEObjIdCart::ClassName() const { return CLASS_NAME; }
char* CEObjIdCart::toLine(int mode) {
	CEString s("[Obj: ");
	if(m_obj!=NULL){ char* tmp=m_obj->toLine(mode); s<<tmp<<", id:"; CMemoryManager::mfree(tmp); }else{ s<<"NULL, id:"; }
	s<<m_id<<"]";
	return CEUtil::copyString(s.getData());
}
CEObject* CEObjIdCart::copy() const { return clone(); }
bool CEObjIdCart::obj_equals(CEObject* other) {
	if(other==NULL)return false;
	if(other->ClassName()!=CEObjIdCart::CLASS_NAME)return false;
	if(m_id != ((CEObjIdCart*)other)->GetId())return false;
	if(m_obj != NULL){ if(!m_obj->obj_equals(((CEObjIdCart*)other)->GetObj()))return false; }
	else{ if(((CEObjIdCart*)other)->GetObj()!=NULL)return false; }
	return true;
}
bool CEObjIdCart::contain_vector(void * obj) {
	if(obj==NULL)return false;
	if(m_obj!=NULL){ if((m_obj==obj)||(m_obj->contain_vector(obj)))return true; }
	return false;
}

//////////////////////////////////////////////////////////////////////	CExprContainer
const char* CExprContainer::CLASS_NAME = "CExprContainer";
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

/*CExprContainer::CExprContainer(CExpression* _source, int info_1, int info_2)
: CESerializable()
{
	m_source = (_source==NULL)?NULL:(_source->clone());// clone the source
	m_info_num = 2;
	m_info = (int*)CMemoryManager::mmalloc(m_info_num*sizeof(int));
	m_info[0] = info_1;
	m_info[1] = info_2;
	m_info_expr = NULL;
}*/

CExprContainer::CExprContainer(CExpression* _source, int info)
: CESerializable() {
	m_source = (_source==NULL)?NULL:(_source->clone());// clone the source
	m_info_num = 1;
	m_info = (int*)CMemoryManager::mmalloc(m_info_num*sizeof(int));
	m_info[0] = info;
	m_info_expr = NULL;
}

CExprContainer::CExprContainer(CExpression* _source, int info_1, int info_2, CExpression* expr_info)
: CESerializable()
{
	m_source = (_source==NULL)?NULL:(_source->clone());// clone the source
	m_info_num = 2;
	m_info = (int*)CMemoryManager::mmalloc(m_info_num*sizeof(int));
	m_info[0] = info_1;
	m_info[1] = info_2;
	m_info_expr = (expr_info==NULL)?NULL:(expr_info->clone()); // clone expr_info
}

CExprContainer::CExprContainer(CExpression* _source, int info_1, int info_2, int info_3, CExpression* expr_info)
: CESerializable()
{
	m_source = (_source==NULL)?NULL:(_source->clone());// clone the source
	m_info_num = 3;
	m_info = (int*)CMemoryManager::mmalloc(m_info_num*sizeof(int));
	m_info[0] = info_1;
	m_info[1] = info_2;
	m_info[2] = info_3;
	m_info_expr = (expr_info==NULL)?NULL:(expr_info->clone()); // clone expr_info
}

CExprContainer::CExprContainer(CExpression* _source, CExpression* expr_info) : CESerializable() {
	m_source = (_source==NULL)?NULL:(_source->clone());// clone the source
	m_info_expr = (expr_info==NULL)?NULL:(expr_info->clone()); // clone expr_info
	m_info_num = 0; m_info = NULL;
}

CExprContainer::CExprContainer(CExpression* _source, int _info_num, int* _info_buf, CExpression* _expr_info) : CESerializable() {
	m_source = (_source==NULL)?NULL:(_source->clone());// clone the source
	m_info_num = _info_num;
	m_info = (int*)CMemoryManager::mmalloc(_info_num*sizeof(int));
	int i;
	for(i=0; i<_info_num; i++) { m_info[i] = _info_buf[i]; }
	m_info_expr = (_expr_info==NULL)?NULL:(_expr_info->clone()); // clone expr_info
}

CExprContainer::~CExprContainer()
{
	if (m_source != NULL) delete m_source;
	if (m_info != NULL) { CMemoryManager::mfree(m_info); }
	if (m_info_expr != NULL) delete m_info_expr;
}

CExpression* CExprContainer::get_expr() { return m_source; }

void CExprContainer::set_expr(CExpression* val) {
	if (m_source != val) {
		if (m_source != NULL) delete m_source;
		if (val != NULL) m_source = val->clone();
		else m_source = NULL;
	}
}

int CExprContainer::get_info_num() { return m_info_num; }
int CExprContainer::get_info(int i)
{
	if (i < 0) i = 0;
	if (i >= m_info_num) return 0;
	//return *(m_info + (i-1)*sizeof(int));
	return m_info[i];
}

void CExprContainer::set_info(int i, int val) {
	if (i >= m_info_num) { // realloc info data
		int old_info_num = m_info_num;
		m_info_num = i+1;
		m_info = (int*)CMemoryManager::mrealloc(m_info, m_info_num*sizeof(int));
		int j;
		for(j=old_info_num; j<m_info_num; j++) { m_info[j] = 0; }
	}
	m_info[i] = val;
}

CExpression* CExprContainer::get_info_expr() {
	return m_info_expr;
}

void CExprContainer::set_info_expr(CExpression* info_val) {
	if (m_info_expr != info_val) {
		if (m_info_expr != NULL) delete m_info_expr;
		if (info_val != NULL) m_info_expr = info_val->clone();
		else m_info_expr = NULL;
	}
}

const char* CExprContainer::ClassName() const { return CLASS_NAME; }
CEString* CExprContainer::toString(int mode) {
	CEString* res = new CEString("[");
	if (m_source != NULL) res->AddString(m_source->toString(mode));
	else res->AddString("NULL");
	res->AddString(", ");
	int i;
	for(i=0; i<m_info_num; i++) {
		res->add_int(get_info(i));
		if (i < (m_info_num-1)) res->AddString(", ");
	}

	if (m_info_expr != NULL) {
		res->AddString(", ");
		res->AddString(m_info_expr->toString(mode));
	}

	res->AddString("]");
	return res;
}

CEObject* CExprContainer::copy() const { return clone(); }
CExprContainer* CExprContainer::clone() const {
	return new CExprContainer(m_source, m_info_num, m_info, m_info_expr);
}

bool CExprContainer::contain_vector(void * obj) {
	if ((m_source == obj) || (m_info_expr == obj) || (m_info == obj)) return true;
	if (m_source != NULL) { if (m_source->contain_vector(obj)) return true; }
	if (m_info_expr != NULL) { if (m_info_expr->contain_vector(obj)) return true; }
	return false;
}

bool CExprContainer::obj_equals(CEObject* other) {
	if (other == NULL) return false;
	if (other == this) return true;
	if (!CEUtil::equals(ClassName(), other->ClassName())) return false;
	CExprContainer* o = (CExprContainer*)other;

	// compare containers
	int s = get_info_num();
	if (o->get_info_num() != s) return false;
	int i;
	for(i=0; i<s; i++) { if (get_info(i) != o->get_info(i)) return false; }

	CExpression* a = get_expr();
	CExpression* b = o->get_expr();
	if (a != b) {
		if (a == NULL) return false; // b != NULL
		if (!a->equals(b)) return false;
	}

	a = get_info_expr();
	b = o->get_info_expr();
	if (a != b) {
		if (a == NULL) return false; // b != NULL
		if (!a->equals(b)) return false;
	}
	return true;
}
