////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// IntArray.cpp: implementation of the CIntArray class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CIntArray::CIntArray() : CEObject(true) {
	m_size = 0; m_size_allocated=0;
	m_data = NULL;
}

CIntArray::CIntArray(long sz) {
	m_data = NULL;
	m_size = 0; m_size_allocated=0;
	//if (sz<0) sz=1; ToSize(sz);
	PreAllocateTo(sz);
}

CIntArray::CIntArray(const CIntArray& arr) {
	m_size = 0; m_size_allocated=0; m_data = NULL;

	//printf("CIntArray copy: %s\n", ((CIntArray*)&arr)->toLine());
	ToSize(arr.GetSize());
	//printf("CIntArray copy size %ld (x: %ld)\n",m_size,arr.GetSize());
	for (long i=0; i<m_size; i++) {
		//printf("CIntArray copy %ld\n",i);
		SetAt(i, arr.GetAt(i));
	}
	//printf("CIntArray copy done\n");
}

CIntArray::~CIntArray() {
	if (m_data != NULL) { CMemoryManager::mfree(m_data); m_data=NULL; m_size_allocated=0; }
	m_size=0;
}

CEObject* CIntArray::copy() const {
	return new CIntArray(*this);
}

const char* CIntArray::CLASS_NAME = "CIntArray";
const char* CIntArray::ClassName() const { return CLASS_NAME; }

char* CIntArray::toLine(int mode) {
	CEString res("{int:");
	for (long i=0; i<m_size; i++) {
		if(i>0)res.AddString(",");
		res.add_int(GetAt(i));
	}
	res.AddString("}");
	char* resS=CEUtil::copyString(res.getData());
	return resS;
}

long CIntArray::GetSize() const { return m_size; }

long CIntArray::GetAt(long i) const {
	if ((i<0) || (i>=m_size) || (m_data == NULL)) return -1;
	return m_data[i];
}

void CIntArray::SetAt(long i, long val) {
	if (i<0) return; // insert?

	if ((i>=m_size) || (m_data == NULL)) ToSize(i+1);
	m_data[i] = val;
}

void CIntArray::AddValTo(long val, long posFrom, long posTo) { //add value fo cell's content, does not increase array size
	if (posFrom<0) posFrom=0;
	if (posTo<0) posTo=m_size-1;
	if (posTo>=m_size) posTo=m_size-1;
	for(long i=posFrom;i<=posTo;i++) { m_data[i] += val; }
}

void CIntArray::Add(long val) {
	long pos = m_size;
	ToSize(pos+1);
	m_data[pos] = val;
	//CWSUtils::PopupMessage("Add(%ls),%ls",val,GetAt())
}

void CIntArray::PreAllocateTo(long sz) {
	if(sz>0) {
		if (m_data == NULL) {
			m_data = (long*)CMemoryManager::mmalloc(sz*sizeof(long));
			m_size_allocated=sz;
		} else {
			if (sz > m_size_allocated) {
				m_data = (long*)CMemoryManager::mrealloc(m_data, sz*sizeof(long));
				m_size_allocated=sz;
			}
		}
	}
}

void CIntArray::ToSize(long sz) {
	if(sz<=0) return;
	if (m_data == NULL) {
		m_data = (long*)CMemoryManager::mmalloc(sz*sizeof(long));
		m_size_allocated=sz;
		m_size=sz;
		return;
	}

	if (sz <= m_size) return;
	
	if (sz > m_size_allocated) {
		m_data = (long*)CMemoryManager::mrealloc(m_data, sz*sizeof(long));
		m_size_allocated=sz;
	}
	for (long i=m_size; i<sz; i++) { m_data[i] = 0; }
	m_size = sz;
}

void CIntArray::RemoveAll() {
	bool kill_alloc=(m_size_allocated>10000);
	if (kill_alloc) {
		if (m_data != NULL) { CMemoryManager::mfree(m_data); m_data=NULL; m_size_allocated=0; }
	}
	m_size = 0;
}

void CIntArray::RemoveAt(long i) {
	if ((i>=m_size) || (i<0)) return;
	for(long j=i+1; j<m_size; j++) { m_data[j-1]=m_data[j]; } //slow
	m_size--;
}

void CIntArray::AddNItems(long n, long val) {
	//[same to:] for(int i=0;i<n;i++)Add(val);
	if(n>0){ long pos=m_size; ToSize(pos+n); for(int i=0;i<n;i++)m_data[pos+i]=val; }
}

void CIntArray::FillAll(long val) {
	for (long i=0; i<m_size; i++) { m_data[i] = val; }
}

void CIntArray::FillAllSeq(long first_val) {
	for (long i=0; i<m_size; i++) { m_data[i] = first_val+i; }
}

bool CIntArray::equals(CIntArray* other) {
	if (other==NULL) return false;
	long s=GetSize();
	if (s != other->GetSize()) return false;
	for(long i=0; i<s; i++) { if (GetAt(i) != other->GetAt(i)) return false; }
	return true;
}

bool CIntArray::equalsCommonItems(CIntArray* other) { // compare till size of shortest
	if (other==NULL) return false;
	long s=GetSize(); long s2=other->GetSize();
	if (s>s2) s=s2;
	for(long i=0; i<s; i++) { if (GetAt(i) != other->GetAt(i)) return false; }
	return true;
}

void CIntArray::exchange(long a, long b) {
	if ((a==b) || (a<0) || (b<0) || (a>=m_size) || (b>=m_size) || (m_data == NULL)) return;
	long tmp=m_data[a]; m_data[a]=m_data[b]; m_data[b]=tmp;
}

bool CIntArray::AreAllGTZero() { // are all > 0
	for (long i=0; i<m_size; i++) { if (m_data[i] <= 0) return false; }
	return true;
}

bool CIntArray::AreAllGEZero() { // are all >= 0
	for (long i=0; i<m_size; i++) { if (m_data[i] < 0) return false; }
	return true;
}

long CIntArray::SprintfTo(char* s, long len_max) {
	CEString res("<");
	for (long i=0; i<m_size; i++) {
		if (i>0) res.AddString(",");
		res.add_int(m_data[i]);
	}
	res.AddString(">");
	if ((len_max>=0) && (res.length()>len_max)) res.CutToSize(len_max);
	long l = sprintf(s,"%s",res.getData());
	return l;
}
