////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// MemoryManager.cpp: implementation of the CMemoryManager class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
//#include <stdexcpt.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CMemoryWatchCart::CMemoryWatchCart(CEObject* o, long _id) : CEObject(false) { m_obj = o; m_id = _id; m_prev = NULL; m_next = NULL; }
CMemoryWatchCart::~CMemoryWatchCart() { }
const char* CMemoryWatchCart::CLASS_NAME = "CMemoryWatchCart";
const char* CMemoryWatchCart::ClassName() const { return CLASS_NAME; }
CEObject* CMemoryWatchCart::get_obj() { return m_obj; }
long CMemoryWatchCart::get_id() { return m_id; }
CMemoryWatchCart* CMemoryWatchCart::get_prev() { return m_prev; }
CMemoryWatchCart* CMemoryWatchCart::get_next() { return m_next; }
void CMemoryWatchCart::set_prev(CMemoryWatchCart* wc) { m_prev = wc; }
void CMemoryWatchCart::set_next(CMemoryWatchCart* wc) { m_next = wc; }
bool CMemoryWatchCart::contain_vector(void * obj) { return false; } // does not contain object, but refers it only!
char* CMemoryWatchCart::toLine(int mode) {
	char* res = (char*)CMemoryManager::mmalloc(25);
	sprintf(res, "{Obj#%ld}", m_id);
	return res;
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDebug* CMemoryManager::dbg = new CDebug("CMemoryManager");

CMemoryWatchCart* CMemoryManager::m_obj_watch_first = NULL;
CMemoryWatchCart* CMemoryManager::m_obj_watch_last = NULL;
unsigned long CMemoryManager::m_watch_list_size = 0;

bool CMemoryManager::m_is_inited = false;

//CEPtrArray* CMemoryManager::obj_data = new CEPtrArray();
CEPtrArray* CMemoryManager::delete_data = new CEPtrArray();
CEPtrArray* CMemoryManager::str_data = new CEPtrArray();

CMemoryManager::CMemoryManager()
{
}

CMemoryManager::~CMemoryManager()
{

}

void* CMemoryManager::mmalloc(long length)
{
	void* res = NULL;
	try {
		res = malloc(length);
	}
	//catch(exception& msg) {
	//	printf("Error allocating memory:\n"); printf(msg.what()); printf("\nSize: %d", length); /*terminate();*/ throw msg;
	//}
	catch(...) {
		//printf("Undefined error allocating memory (%ld), Size: %ld bytes\n", GetLastError(), length);
		printf("Undefined error allocating memory, Size: %ld bytes\n", length);
		throw 0; //terminate();
	}
	if (res==NULL) {
		printf("Error allocating memory: not enough memory for %ld bytes block\n", length);
		throw 0;
	}

	return res;
}

void CMemoryManager::mmemcpy(void* to, const void* from, long length) {
	//try {
		memcpy(to, from, length);
	//}
	//catch(exception& msg) {
	//	printf("Error in memcpy():\n");
	//	printf(msg.what());
	//	//terminate();
	//	throw msg;
	//}
	//catch(...) {
	//	printf("Undefined error in memcpy()");
	//	//terminate();
	//	throw 0;
	//}
}

void CMemoryManager::mmemmove(void* to, const void* from, long length) {
	//try {
		memmove(to, from, length);
	//}
	//catch(exception& msg) {
	//	printf("Error in memmove():\n");
	//	printf(msg.what());
	//	//terminate();
	//	throw msg;
	//}
	//catch(...) {
	//	printf("Undefined error in memmove()");
	//	//terminate();
	//	throw 0;
	//}
}

void* CMemoryManager::mrealloc(void *old, long new_len)
{
	void* res = NULL;
	try {
		res = realloc(old, new_len);
	}
	//catch(exception& msg) {
	//printf("Error reallocating memory:\n"); printf(msg.what()); printf("\nSize: %d", new_len); /*terminate();*/ throw msg;
	//}
	catch(...) {
		//printf("Undefined error reallocating memory (%ld), Size: %ld bytes\n", GetLastError(), new_len);
		printf("Undefined error reallocating memory, Size: %ld bytes\n", new_len);
		throw 0; //terminate();
	}
	if (res==NULL) { printf("Error re-allocating memory: not enough memory for %ld bytes block\n", new_len); throw 0; }
	return res;
}

char* CMemoryManager::mReallocChar(char *old, long new_len)
{
	/*char* res = (char*)mmalloc(new_len);
	int len = CEUtil::length(old);
	if (len > (new_len-1)) len = new_len-1;
	long i;
	for(i=0; i<len; i++) {
		*(res + i) = *(old + i);
	}
	*(res + len) = '\0';
	mfree(old);
	return res;*/

	char* res = (char*)mrealloc(old, new_len);
	return res;
}

void CMemoryManager::mfree(void *p)
{
	if (p == NULL) return;
	try {
		free(p);
	}
	catch(...) {
		printf("Undefined memory error (free() operation fails)");
		//terminate();
		CEException::halt();
	}
}

// return false is watch is not accepted
bool CMemoryManager::add_watch(CEObject * obj) {
	if (!m_is_inited) {// do not add watch if memory manager is not initialized
		return false;
		//init();
	}
	if (dbg->get_level() > 500) { printf("a%ld ", m_watch_list_size+1); }

	CMemoryWatchCart* wc = new CMemoryWatchCart(obj, m_obj_counter);
	if (m_obj_watch_first == NULL) {
		m_obj_watch_first = wc;
		if (m_obj_watch_last != NULL) {
			m_obj_watch_last = NULL; // fix error for proper finalization
			//printf("Memory manager ERROR: invalid watches structure (NULL,XXXX)");
			dbg->log(0, "Memory manager ERROR: invalid watches structure (NULL,XXXX)");
			CEException::halt();
		}
		m_obj_watch_last = wc;
	} else {
		if (m_obj_watch_last == NULL) {
			m_obj_watch_first = NULL; // fix error for proper finalization
			//printf("Memory manager ERROR: invalid watches structure (XXXX,NULL)");
			dbg->log(0, "Memory manager ERROR: invalid watches structure (XXXX,NULL)");
			CEException::halt();
		}
		m_obj_watch_last->set_next(wc);
		wc->set_prev(m_obj_watch_last);
		m_obj_watch_last = wc; // change the last link
	}

	m_watch_list_size++;
	if (m_watch_list_size == LONG_MAX) {
		dbg->log(0, "WARNING: watch list exceeds the LONG_MAX size. No more objects will be watched");
		finalize();
	}
	m_obj_counter++;
	if (m_obj_counter == LONG_MAX) {
		dbg->log(0, "WARNING: object counter exceeds the LONG_MAX value. Objects will be encountered from 0.");
		m_obj_counter=0;
	}
	//obj_data->add(new CMemoryWatchCart(obj, m_obj_counter));
#if defined(ALTA_DEBUG)
	obj->m_memory_watch = wc;
#endif //ALTA_DEBUG
	return true;
}

long CMemoryManager::m_obj_counter = 0;
long CMemoryManager::GetObjCounter() { return m_obj_counter; }

void CMemoryManager::init() {
	m_obj_counter = 0;

	if ((m_obj_watch_first != NULL) || (m_obj_watch_last != NULL)) {
		dbg->log(0, "Error: extra Memory Manager initialization");
	}
	m_obj_watch_first = NULL;
	m_obj_watch_last = NULL;
	m_watch_list_size = 0;

	//if (obj_data == NULL) obj_data = new CEPtrArray();
	if (str_data == NULL) str_data = new CEPtrArray();
	if (delete_data == NULL) delete_data = new CEPtrArray();
	m_is_inited = true;
}

void CMemoryManager::finalize()
{
	/*if (obj_data != NULL) {
		//obj_data->clear();
		// note: we may not delete watched objects: sub-objects may appear there.
		// the object cart deleting will not delete object
		delete obj_data;
		obj_data = NULL;
	}*/

	for(CMemoryWatchCart* ws_curr = m_obj_watch_first; ws_curr != NULL;) {
		CMemoryWatchCart* ws_curr_next = ws_curr->get_next();
#if defined(ALTA_DEBUG)
		ws_curr->get_obj()->m_memory_watch = NULL; // unbind object
#endif //ALTA_DEBUG
		delete ws_curr;
		ws_curr = ws_curr_next;
	}
	m_obj_watch_first = NULL;
	m_obj_watch_last = NULL;
	m_watch_list_size = 0;

	if (str_data != NULL) {
		//str_data->clear();
		// note: some strings may be still in use.
		str_data->clearall_without_delete_objects();
		delete str_data;
		str_data = NULL;
	}
	if (delete_data != NULL) {
		delete delete_data;
		delete_data = NULL;
	}

#if defined(ALTA_DEBUG)
	printf("total %ld objects was watched\n", m_obj_counter);
#endif //ALTA_DEBUG
	m_is_inited = false;
}

void CMemoryManager::examine_watches() {
	if (!m_is_inited) {
		dbg->log(0, "examine_watches() warning: no watches data available"); return;
		//init();
	}

	if (m_obj_watch_first == NULL) { dbg->log(10, "no active watches exist"); return; }

	// log objects number
	long i = 0;
	long num_alt = CalcWatchesNum();
	if (num_alt == (long)m_watch_list_size) {
		dbg->log(45, m_watch_list_size, "Total %ld objects in memory");
	} else {
		dbg->log(8, num_alt, m_watch_list_size, "Total %ld objects in memory (%ld declared <Error>)");
		m_watch_list_size = num_alt;
	}

	//	delete extra sub-items
	//	*****************************************************	<scroll>
	CEProgressWatch* pw = NULL;
	if (m_watch_list_size >= 1000) { pw = CEProcessor::newProgressWatch("CMemoryManager::examine_watches (%ld items)", m_watch_list_size); }
	//	*****************************************************	<\scroll>
	CMemoryWatchCart* wc = m_obj_watch_first;
	for (i=0; (wc != NULL); i++) {
		CEObject* item = wc->get_obj();

		// search for upper object for item
		bool cart_found = false;
		for (CMemoryWatchCart* wc_B = m_obj_watch_first; (wc_B != NULL); wc_B = wc_B->get_next()) {
			if (wc_B == wc) continue;
			CEObject* item_B = wc_B->get_obj();
			if (item_B->contain_vector(item)) { cart_found = true; break; }
		}
		// also search in delete list
		if ((!cart_found) && (delete_data != NULL)) {
			int del_s = delete_data->size();
			long j;
			for (j=0; j<del_s; j++) {
				CEObject* item_D = (CEObject*)delete_data->GetAt(j);
				if (item_D == item) { cart_found = true; break; }
			}
		}

		CMemoryWatchCart* wc_next = wc->get_next();
		if (cart_found) { // delete object cart, but not the object
			CMemoryWatchCart* wc_prev = wc->get_prev();
			if (wc_prev != NULL) { wc_prev->set_next(wc_next); }
			else { m_obj_watch_first = wc_next; }
			if (wc_next != NULL) { wc_next->set_prev(wc_prev); }
			else { m_obj_watch_last = wc_prev; }
			delete wc;
			m_watch_list_size--;
		}
		wc = wc_next;

		//	*****************************************************	<scroll>
		if (pw != NULL) pw->step(i+1, m_watch_list_size);
		//	*****************************************************	<\scroll>
	}
	//	*****************************************************	<scroll>
	if (pw != NULL) CEProcessor::dismissProgressWatch(pw);
	//	*****************************************************	<\scroll>

	// log objects
	dbg->log(0, m_watch_list_size, "Total %ld separated objects are lost in memory");
	wc = m_obj_watch_first;
	int dbg_lev = dbg->get_level();
	if (dbg_lev >= 20) {
		for (i=0; (wc != NULL); i++) {
			if (dbg_lev >= 50) {
				dbg->log(45, i+1, wc->get_id(), "%ld: <#%ld>", wc->get_obj());
			} else {
				// log class name only (object may not be listable)
				char* s = (char*)CMemoryManager::mmalloc(50);
				sprintf(s, "%ld: <#%ld> {%s}", i+1, wc->get_id(), wc->get_obj()->ClassName());
				dbg->log(15, s);
				CMemoryManager::mfree(s);
			}
			wc = wc->get_next();
		}
	}
}

void CMemoryManager::exclude_watch(CEObject * obj) {
	if (dbg->get_level() > 500) { printf("e%ld ", m_watch_list_size); }

	if (obj == NULL) return;
	if (!m_is_inited) return; //init();

	CMemoryWatchCart* wc_del = NULL; // the watch to be deleted
#if defined(ALTA_DEBUG)
	// use obj's pointer
	CEObject* wc_obj = obj->m_memory_watch;
	if (wc_obj != NULL) { wc_del = (CMemoryWatchCart*)wc_obj; obj->m_memory_watch = NULL; }
#else
	for (CMemoryWatchCart* wc = m_obj_watch_first; (wc != NULL);) {
		if (wc->get_obj() == obj) { wc_del = wc; break; }
		wc = wc->get_next();
	}
#endif //ALTA_DEBUG

	if (wc_del != NULL) { // watch found
		// delete object cart, but not the object
		CMemoryWatchCart* wc_prev = wc_del->get_prev();
		CMemoryWatchCart* wc_next = wc_del->get_next();
		if (wc_prev != NULL) { wc_prev->set_next(wc_next); } 
		else { m_obj_watch_first = wc_next; }
		if (wc_next != NULL) { wc_next->set_prev(wc_prev); }
		else { m_obj_watch_last = wc_prev; }
		delete wc_del;
		m_watch_list_size--;
	} else {
		//printf("Memory manager ERROR: can't find watch to remove");
		dbg->log(0, "Memory manager WARNING: can't find watch to remove");
	}
}

/*
//Note: use CEProcessor::AddToDeleteList(obj);
void CMemoryManager::add_to_delete_list(CEObject *obj) {
	if (!m_is_inited) init();

	if (obj == NULL) return;
	if (delete_data == NULL) { dbg->log(0, "Internal Error: NULL in add_to_delete_list() processing");}
	delete_data->add(obj);
}
*/

long CMemoryManager::CalcWatchesNum() {
	int res = 0;
	for (CMemoryWatchCart* wc = m_obj_watch_first; wc != NULL; wc = wc->get_next()) { res++; }
	return res;
}
