////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// Pos2D.cpp: implementation of the CPos2D class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

const char* CPos2D::CLASS_NAME = "CPos2D";
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CPos2D::CPos2D(int ix, int x_max, int iy, int y_max, double dx, double dy) : CESerializable()
{
	m_ix = ix; m_iy = iy; m_x_max = x_max, m_y_max = y_max; m_dx = dx; m_dy = dy;
}

CPos2D::CPos2D(double x, double y) : CESerializable()
{
	m_dx = x; m_dy = y;

	reset_integers();
}

CPos2D::CPos2D(int x, int x_max, int y, int y_max) : CESerializable()
{
	m_ix = x; m_iy = y; m_x_max = x_max, m_y_max = y_max;

	if (x_max > 0) m_dx = ((double)x)/x_max;
	else m_dx = 0;
	if (y_max > 0) m_dy = ((double)y)/y_max;
	else m_dy = 0;
}

CPos2D::~CPos2D()
{

}

void CPos2D::reset_integers() {
	m_ix = 0; m_iy = 0; m_x_max = -1, m_y_max = -1;
}

void CPos2D::AddPos(CPos2D* other_pos) {
	if (other_pos == NULL) return;

	reset_integers();
	m_dx += other_pos->get_double_X();
	m_dy += other_pos->get_double_Y();
}

void CPos2D::SubtrPos(CPos2D* other_pos) {
	if (other_pos == NULL) return;

	reset_integers();
	m_dx -= other_pos->get_double_X();
	m_dy -= other_pos->get_double_Y();
}

void CPos2D::MultiplyFactor(double _factor, CPos2D* center_pos) {
	MultiplyFactors(_factor, _factor, center_pos);
}

void CPos2D::MultiplyFactors(double _factor_X, double _factor_Y, CPos2D* center_pos) {
	reset_integers();

	double cent_x = 0;
	double cent_y = 0;
	if (center_pos != NULL) {
		cent_x = center_pos->get_double_X();
		cent_y = center_pos->get_double_Y();
	}

	m_dx = (m_dx-cent_x)*_factor_X+cent_x;
	m_dy = (m_dy-cent_y)*_factor_Y+cent_y;
}

CPos2D* CPos2D::clone() const
{
	return new CPos2D(m_ix, m_x_max, m_iy, m_y_max, m_dx, m_dy);
}

const char* CPos2D::ClassName() const { return CLASS_NAME; }

CEString* CPos2D::toString(int mode) {
	CEString* res = new CEString("<CPos2D: (");
	res->add(get_double_X()); res->AddString(", "); res->add(get_double_Y());
	res->AddString("), (");
	res->add_int(m_ix); res->AddString(":"); res->add_int(m_x_max);
	res->AddString(", ");
	res->add_int(m_iy); res->AddString(":"); res->add_int(m_y_max);
	res->AddString(")>");
	return res;
}

void CPos2D::set_XY(double x, double y) { m_dx = x; m_dy = y; reset_integers(); }

int CPos2D::get_X(int x_max)
{
	if (x_max < 0) return m_ix;
	else return (int)(m_dx*x_max);
}

int CPos2D::get_Y(int y_max)
{
	if (y_max < 0) return m_iy;
	else return (int)(m_dy*y_max);
}

double CPos2D::get_double_X() { return m_dx; }
double CPos2D::get_double_Y() { return m_dy; }

/*
*	(x(y)_max <= 0) leaves appropriate line unaffected.
*/
void CPos2D::set_box_size(int x_max, int y_max) {

	if (x_max > 0) {
		m_x_max = x_max, 
		m_ix = (int)(m_dx*x_max);
	}

	if (y_max > 0) {
		m_y_max = y_max;
		m_iy = (int)(m_dy*y_max);
	}
}
