////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// Sequence.cpp: implementation of the CSequence class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

const char* CSequence::CLASS_NAME		= "CSequence";
const char* CSequence::CLASS_TAG_NAME	= "seq";
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSequence::CSequence() : CExpression()
{
#ifdef USE_PLAIN_ARRAY_FORSEQ // biuld sequence-expressions on CEPtrArray* basis
	m_pitems = new CEPtrArray();
#else
	m_pitems = new CAPtrSeq();
#endif //USE_PLAIN_ARRAY_FORSEQ 
}

#ifdef USE_PLAIN_ARRAY_FORSEQ // biuld sequence-expressions on CEPtrArray* basis
CSequence::CSequence(CEPtrArray* pPtrArray) : CExpression() {
	m_pitems = pPtrArray;
	if(m_pitems==NULL) m_pitems = new CEPtrArray();
}
CEPtrArray* CSequence::getMemberList() { return m_pitems; }
const CEPtrArray* CSequence::getMemberListConst() const { return m_pitems; }
#else
CSequence::CSequence(CAPtrSeq* pPtrSeq) : CExpression() {
	m_pitems = pPtrSeq;
	if(m_pitems==NULL) m_pitems = new CAPtrSeq();
}
CAPtrSeq* CSequence::getMemberList() { return m_pitems; }
const CAPtrSeq* CSequence::getMemberListConst() const { return m_pitems; }
#endif //USE_PLAIN_ARRAY_FORSEQ 

CSequence::~CSequence() {
	delete m_pitems;
}

const char* CSequence::ClassName() const { return CLASS_NAME; }

CExpression* CSequence::applyRule(CEAbstractRule* rule) {
	CERule* r = (CERule*)rule;

	long i;
	for (i=0; i<size(); i++) {
		CExpression* item = GetAt(i);
		CExpression* item_repl = r->ApplyTo(item, false);
		if ((item_repl != item) && (item_repl != NULL)) {
			SetAt(i, item_repl);
			delete item_repl;
		}
	}
	return this;
}

CExpression* CSequence::Arrange(int level) {
	//CEBasic::dbg->log(2,"CSequence::Arrange", this);
	for (long i=0; i<size(); i++) {
		CExpression* item = GetAt(i);
		CExpression* temp = CExpression::arrge(item, false);
		if (temp != item) SetAtX(i, temp);
	}
	//CEBasic::dbg->log(2,"CSequence::Arrange result", this);
	return this;
}

CExpression* CSequence::expand(bool recursive) {
	//CEBasic::dbg->log(2,"CSequence::expand", this);
	for(long i=0; i<size(); i++) {
		CExpression* item = GetAt(i);
		CExpression* temp2 = item->expand();
		CExpression* temp = arrge(temp2, (temp2 != item));
		if ((item != temp) && (temp != NULL)) SetAtX(i, temp);
	}
	return this;
}

long CSequence::addItem(CExpression *pNewItem) {
	if (pNewItem != NULL) m_pitems->add(pNewItem);
	return size();
}

long CSequence::size() const { return m_pitems->size(); }

CExpression* CSequence::GetAt(long i) {
	return ( ((i < m_pitems->size()) && (i>=0)) ? (CExpression*)m_pitems->GetAt(i) : NULL );
}

CExpression* CSequence::GetAt2(long i) {
	CExpression* res = NULL;
	if ((i < m_pitems->size()) && (i>=0)) {
#ifdef USE_PLAIN_ARRAY_FORSEQ
		res = (CExpression*)m_pitems->GetAt(i);
#else
		res = (CExpression*)m_pitems->GetAt2(i);
#endif //USE_PLAIN_ARRAY_FORSEQ
	}

	//DEBUG
	/*if (res != GetAt(i)) {
		CESum::dbg->log(0, i, "SEQ-Error: %ld", m_pitems);
		CESum::dbg->log(0, "SEQ-Error: res", res);
		CESum::dbg->log(0, "SEQ-Error: GetAt(i)", GetAt(i)); CEException::halt();
	}*/
	return res;
}

const CExpression* CSequence::GetAtConst(long i) const {
	return ( ((i < m_pitems->size()) && (i>=0)) ? (CExpression*)m_pitems->GetAtConst(i) : NULL );
}

void CSequence::SetAt(long i, CExpression *_new) {
	if ((i < size()) && (i>=0)) {
		CExpression* old = GetAt(i);
		if (old != _new) {
			m_pitems->SetAt(i, _new->clone(), false);
			delete old;
		}
	}
}

void CSequence::SetAtX(long i, CExpression* _obj_X) { // no cloning new
	if ((i < m_pitems->size()) && (i>=0)) {
		CExpression* old = (CExpression*)m_pitems->GetAt(i);
		if (old != _obj_X) { m_pitems->SetAt(i, _obj_X, false); delete old; }
	}
}

void CSequence::OverwriteAtX(long i, CExpression* _obj_X) { // no cloning new, no deleting old
	if ((i < m_pitems->size()) && (i>=0)) {
		// no (old != _obj_X) check, do it before call if required
		m_pitems->SetAt(i, _obj_X, false);
	}
}

CExpression* CSequence::RemoveAtX(long i) { // no delete old
	CExpression* res=NULL;
	if ((i < m_pitems->size()) && (i>=0)) {
		res = (CExpression*)m_pitems->GetAt(i);
		m_pitems->RemoveAt(i, false);
	}
	return res;
}

void CSequence::ClearWODelObjs() { m_pitems->clearall_without_delete_objects(); }

void CSequence::UnsafeSetAt(long i, CExpression* _obj_X) { // no delete old, no cloning new
	if ((i < m_pitems->size()) && (i>=0)) {
		m_pitems->SetAt(i, _obj_X, false);
	} else {
		delete _obj_X;
	}
}

void CSequence::InsertAtX(long i, CExpression* _obj_X) { // no cloning new
	if (i >= m_pitems->size()) addItem(_obj_X);
	else m_pitems->InsertAt(i, _obj_X);
}

/*CExpression* CSequence::UnsafeExtractAt(long i) { // return item & clears it's field
	CExpression* res = NULL;
	if ((i<m_pitems->size()) && (i>=0)) {
		res = (CExpression*)m_pitems->ExtractAt(i);
	}
	return res;
}*/

void CSequence::RemoveAt(long i) { m_pitems->RemoveAt(i, true); }

bool CSequence::move(long from, long to)
{
	//if ((from>=size()) || (from<0) || (to>=size()) || (to<0) || (from==to)) {
	//	CESum::dbg->log(0, from, to, size(), "<Warning.> {CSequence::move(%ld, %ld), %ld}"); return false;
	//}
	m_pitems->MoveFromTo(from, to);
	return true;
	// old
	// note: it was wrong -> move(n, n+1) will change nothing
	//InsertAt(to, GetAt(from));
	//RemoveAt(((from<to)?from:(from+1)));
}

void CSequence::exchange(long a, long b) {
	if ((a==b) || (a<0) || (b<0)) return;
	long s = size();
	if ((a>=s) || (b>=s)) return;

	CEObject* a_obj = m_pitems->GetAt(a);
	CEObject* b_obj = m_pitems->GetAt(b);
	m_pitems->SetAt(a, b_obj, false);
	m_pitems->SetAt(b, a_obj, false);
}

long CSequence::InsertAt(long i, CExpression *_new)
{
	if (i >= size()) addItem(_new->clone());
	else m_pitems->InsertAt(i, _new->clone());
	return size();
}

void CSequence::addSeq(CSequence* seq) { // parameter is deleted
	if (seq != NULL) {
		long s = seq->size();
		for(long i=0; i<s; i++) { addItem(seq->GetAt(i)); }
		seq->ClearWODelObjs();
		delete seq;
	}
}

bool CSequence::contain_vector(void * obj)
{
	if (m_pitems == obj) return true;
	return m_pitems->contain_vector(obj);
}

void CSequence::clear_all()
{
	m_pitems->clear_all();
}

CExpression* CSequence::clone() const
{
	CExpression* res = new CSequence(m_pitems->clone());
	copyAttributesTo(res);
	return res;
}

bool CSequence::equals(CExpression* other) {
	if (other == NULL) return false;
	if (other == this) return true;
	if (!CEUtil::equals(ClassName(), other->ClassName())) return false;

	CSequence* o = (CSequence*)other;
	long s = size();
	if (s != o->size()) return false;
	if (!attributes_equals_to(o)) return false;

	long i;
	for (i=0; i<s; i++) {
		if (!GetAt(i)->equals(o->GetAt(i))) return false;
	}
	return true;
}

CEString* CSequence::toString(int mode)
{
	CEString* res = NULL;
	if (getFlag()) res = new CEString("");
	else res = new CEString("- ");
	res->AddString("\\({");

	long s = size();
	long i;
	for (i=0; i<s; i++) {
		if (i>0) res->AddString(", ");
		res->AddString(GetAt(i)->toString(mode));
	}

	res->AddString("}\\)");
	return res;
}

/*
*	Used to create child objects similar to different-class ones.
*	Deletes all the previous data.
*/
void CSequence::copy_from(const CSequence *other)
{
	if (other == this) return;

#ifdef USE_PLAIN_ARRAY_FORSEQ // build sequence-expressions on CEPtrArray* basis
	CEPtrArray* items_old=m_pitems;
#else
	CAPtrSeq* items_old=m_pitems;
#endif //USE_PLAIN_ARRAY_FORSEQ 
	m_pitems = other->getMemberListConst()->clone();
	delete items_old;
	other->copyAttributesTo(this);
}

long CSequence::get_item_pos(CExpression *item, bool take_care_of_sign)
{
	if (item == NULL) return -1;
	long s = size();
	long i=0;
	for (i=0; i<s; i++) {
		CExpression* it_curr = GetAt(i);
		if (take_care_of_sign) {
			if (it_curr->equals(item)) return i;
		}
		else {
			if (it_curr->equal_or_opposite(item)) return i;
		}
	}
	return -1;
}

CESerializable* CSequence::restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings) {
	CSequence* res = new CSequence();
	bool loop = true;
	for(;loop;) {
		long skip_pos = CESupport::skip_item_in_buffer(data, pos, settings);
		if (skip_pos <= 0) {
			loop = false;
		}
		else {
			CESerializable* item = CESupport::read_buffer(data, pos, settings);
			pos = skip_pos;
			if (item != NULL) {
				res->addItem((CExpression*)item);
			}
			else {
				//Error
				loop = false;
				delete res;
				return NULL;
			}
		}
	}
	return res;
}

const char* CSequence::get_TAG_Name() { return CLASS_TAG_NAME; }

long CSequence::store_stream(ofstream *p_ofs, DWORD settings)
{
	long res = 0;
	long s = size();
	long i=0;
	for (i=0; i<s; i++) {
		CExpression* item = GetAt(i);
		long item_len = CESupport::store_stream(item, p_ofs, settings | CESupport::SERIAL_EXPRESSION);
		res += item_len;
		if ((item_len > 40) && (i < (s-1))) { //TODO: 40 should be defined constant
			*p_ofs << '\n';
			res += 2;
		}
	}
	return res;
}

// Removes the first equal value only (for faster processing).
bool CSequence::RemoveExpr(CExpression* val) {
	if (val == NULL) { CEBasic::dbg->log(0, "<Error> CSequence::RemoveExpr(NULL)"); return false; }

	bool res = false;
	long s = size();
	long i=0;
	for(i=0; i<s; i++) {
		CExpression* itm = GetAt(i);
		if (val->equals(itm)) { RemoveAt(i); res = true; break; } // remove the only value
		//if (val->equals(GetAt(i))) { RemoveAt(i); res = true; s--; i--; } // remove all
	}
	return res;
}

// adds expression if unique only, clones input
bool CSequence::AddExprIfUnique(CExpression* val) {
	if (val == NULL) { CEBasic::dbg->log(0, "<Error> CSequence::AddUniqueExpr(NULL)"); return false; }

	bool val_found = false;
	long s = size();
	long i=0;
	for(i=0; i<s; i++) { if (val->equals(GetAt(i))) { val_found = true; break; } }

	bool do_add = !val_found;
	if (do_add) { addItem(val->clone()); }
	return do_add;
}

//add expressions if unique only, returns N[added], ignores other's sign & level!
int CSequence::AddUniquesFrom(CSequence* other) {
	if (other == NULL) { CEBasic::dbg->log(0, "<Error> CSequence::AddUniquesFrom(NULL)"); return 0; }
	if (other == this) return 0;
	int nAdded=0;
	long s = other->size();
	for(long i=0; i<s; i++) { if(AddExprIfUnique(other->GetAt(i)))nAdded++; }
	return nAdded;
}
