////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// UXMLparcer.cpp: implementation of the CUXMLparcer class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

const char CUXMLparcer::CODE_NO_TAG_FOUND = '/';

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CUXMLparcer::CUXMLparcer()
{

}

CUXMLparcer::~CUXMLparcer()
{

}

/*
*	finds next tag
*	returns &'/' if next TAG is </xxx>
*	also returns NULL if errors
*/
char* CUXMLparcer::get_tag_id(const char *data, long pos)
{
	pos = find_next_tag_pos(data, pos);
	//pos = skip_spaces(data, pos);
	long tag_start = CEUtil::findSymbol(data, '<', pos);
	if (pos != tag_start) {
		if (tag_start < 0) {dbg->log(1, "Error: invalid XML format"); return NULL;}
		dbg->log(1, "Warning: invalid XML format, some data skipped");
	}
	long tag_title_start = tag_start+1;
	if (*(data+tag_title_start)=='/') return (char*)&CODE_NO_TAG_FOUND;
	long tag_title_end = findSymbol(data, '>', tag_title_start);
	long tag_title_end2 = findSymbolFromTo(data, ' ', tag_title_start, tag_title_end);
	if (tag_title_end2 > 0) tag_title_end = tag_title_end2;
	tag_title_end--;
	if (tag_title_start > tag_title_end) {
		dbg->log(1, "XML Error: (NULL tag header)");
		return NULL;
	}
	char* res = copyStringFromTo(data, tag_title_start, tag_title_end);
	return res;
}

long CUXMLparcer::skip_spaces(const char *data, long pos)
{
	bool loop = true;
	for (;loop;) {
		char c = *(data+pos);
		if (c == '\0') return pos;//data end exceeded
		if ((c == '\n') || (c == ' ')) pos++;
		else loop = false;
	}
	return pos;
}

long CUXMLparcer::skip_spaces_back(const char *data, long pos)
{
	for (;;) {
		char c = *(data+pos);
		if ((c == '\n') || (c == ' ')) pos--;
		else return pos;
	}
}

long CUXMLparcer::skip_tag_header(const char *data, long pos)
{
	pos = skip_spaces(data, pos);
	long tag_start = findSymbol(data, '<', pos);
	if (pos != tag_start) {
		if (tag_start < 0) {dbg->log(1, "Error: invalid XML format"); return -1;}
		dbg->log(1, "Warning: invalid XML format, some data skipped");
	}
	long tag_header_end = findSymbol(data, '>', tag_start);
	return skip_spaces(data, tag_header_end+1);
}

/*
*	if (do_skip_closing) returns position of first sym after of </tag>
*	else position first bracket in </tag>
*	returs -1 if error
*/
long CUXMLparcer::find_tag_closing(const char *data, const char *tag, long pos, bool do_skip_closing)
{
	//pos = skip_spaces(data, pos);
	bool loop = true;
	long res = -1;
	for (;loop;) {
		pos = find_next_tag_pos(data, pos);
		long pos_new = skip_tag(data, pos);
		if (pos_new < 0) return -1;	//XML error;
		if (pos_new > 0) pos = pos_new;
		else {	//0
			//	compare closing tag to 'tag'
			loop = false;
			pos = skip_spaces(data, pos);
			long tag_title_start = findSymbol(data, '<', pos) + 2;
			//	*(data+tag_title_start-1)=='/')
			long tag_title_end = findSymbol(data, '>', tag_title_start) - 1;
			if (tag_title_start > tag_title_end) {dbg->log(1, "XML Error: (NULL /tag header)"); return -1;}
			bool valid_tag = compareSubStrings(data, tag_title_start, tag_title_end, tag, 0, length(tag)-1);
			if (!valid_tag) {return -1;}
			if (do_skip_closing) res = tag_title_end+2;
			else res = tag_title_start-2;
		}
	}
	return res;
}

/*
*	skips NEXT tag
*	returns 0 if no new tag started
*	returns -1 if error;
*/
long CUXMLparcer::skip_tag(const char *data, long pos)
{
	pos = find_next_tag_pos(data, pos);
	//pos = skip_spaces(data, pos);
	char* tag = get_tag_id(data, pos);
	if (tag == NULL) return -1;
	if (*tag == '/') return 0;
	pos = skip_tag_header(data, pos);
	pos = find_tag_closing(data, tag, pos, true);
	CMemoryManager::mfree(tag);
	if (pos<0) return -1;
	return pos;
}

/*
*	To be called after get_tag_id()
*/
CEMap* CUXMLparcer::get_tag_values_map(const char *data, long pos)
{
	pos = find_next_tag_pos(data, pos);
	//pos = skip_spaces(data, pos);
	long tag_title_start = findSymbol(data, '<', pos)+1;
	long tag_header_end = findSymbol(data, '>', tag_title_start)-1;
	long tag_space = findSymbolFromTo(data, ' ', tag_title_start, tag_header_end);
	if (tag_space < 0) return NULL;

	//bool loop = true;
	CEMap* res = NULL;
	pos = skip_spaces(data, tag_space+1);
	for (;pos < tag_header_end;) {
		long eq_pos = findSymbolFromTo(data, '=', pos, tag_header_end);
		if (eq_pos < 0) {
			pos = tag_header_end;//no more parameters found
		}
		else {
			//	parse value
			char* par_id = copyStringFromTo(data, pos, skip_spaces_back(data, eq_pos-1));

			//printf("(%s=", par_id);

			tag_space = findSymbolFromTo(data, ' ', eq_pos, tag_header_end);
			long par_end = tag_header_end;
			if (tag_space > 0) par_end = tag_space-1;
			char* par_val = copyStringFromTo(data,
				skip_spaces(data, (eq_pos+1))+1,
				skip_spaces_back(data, par_end)-1);//positions of \" are extracted
			//printf("\"%s\")", par_val);

			if (res == NULL) res = CEMap::createNewMap();

			//res->put_ptr(par_id, par_val);
			CEString* par_val_str = new CEString(par_val);
			CMemoryManager::mfree(par_val);
			res->put(par_id, par_val_str);

			CMemoryManager::mfree(par_id);
			//res->put_int(par_id, atoi(par_val), false);
			//CMemoryManager::mfree(par_val);
			pos = skip_spaces(data, par_end+1);
		}
	}
	return res;
}

long CUXMLparcer::find_next_tag_pos(const char *data, long pos)
{
	for (;;) {
		char c = *(data+pos);
		if (c == '\0') return -1;
		if (c == '<') return pos;
		if (c == '>') return -1;
		pos++;
	}
}
