// VMFileUtils.cpp
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
//#include "VMLib/VMLibAll.h"

// Microsoft specific [?]
#include <direct.h>		// for file operations
#include <sys/stat.h>	// for file operations (used in IsFileExist()), Nt: the header file may be different on linux/unix/etc.
//[linux/unix]#include <unistd.h>	// for file operations ('access()' used in IsFileExist())
#include  <io.h>		// for file operations (e.g. 'access()' used in IsFileExist())

#ifdef ALHEP_WIN
//
#undef DWORD
#include <windows.h>
//#include "windows.h"
#include "windowsx.h"
//
#else //linux, unix...
//
#include <sys/types.h>
#include <dirent.h>		//linux
//
#endif //#ifdef VMLIB_WIN
//#include <errno.h>	//for 'errno' macro

///////////////////////////////////////////////////////////////////////////////////////////////	CVMFileUtil::CVMFileInfo
CVMFileUtil::CVMFileInfo::CVMFileInfo(const char* fn, bool bIsDir) : CEObject(true)
,m_name(fn), m_bIsDir(bIsDir)
{}
const char* CVMFileUtil::CVMFileInfo::CLASS_NAME = "CVMFileInfo";
const char* CVMFileUtil::CVMFileInfo::ClassName() { return CLASS_NAME; }
//CEObject* CVMFileUtil::CVMFileInfo::copy() const {
//	CVMFileUtil::CVMFileInfo* res = new CVMFileUtil::CVMFileInfo(m_name.getData(),m_bIsDir);
//	return res;
//}

///////////////////////////////////////////////////////////////////////////////////////////////	CVMFileUtil
/*
*	File system operations
*/
/*
*	Notes.
*	1. requires #include <direct.h> (Microsoft specific?)
*	2. throw file exceptions?
*/
/*
*	::CreateDirectory(), ::CreateDirectoryEx(); (Windows API, use like CreateDirectory("c:\\dir1\\dir2\\ ... \\dirn", NULL);)
*		see also http://msdn.microsoft.com/en-us/library/aa363950.aspx
*	_tmkdir(_T("c:\\ZigBeeLogs");
*	mkdir(path);
*	system("mkdir path")
*	Adv.: "If you want a portable solution then you can use Boost Filesystem library"
*	
*	TODO: add "./" prior to path if it starts from symbol?
*/

#define MAX_PATH_LENGTH						1000
char* CVMFileUtil::m_bufGetCurrentPath		= NULL;		// used by GetCurrentPath
char* CVMFileUtil::m_bufInitialPath			= NULL;		// set automatically in GetInitialPath() or ChDir()
bool CVMFileUtil::m_bInitialDirChanged		= false;

const char* CVMFileUtil::GetInitialPath() {
	if(CVMFileUtil::m_bufInitialPath==NULL){ CVMFileUtil::SetCurrPathAsInitial(); }
	return CVMFileUtil::m_bufInitialPath;
}
void CVMFileUtil::SetCurrPathAsInitial() {
	if(CVMFileUtil::m_bufInitialPath!=NULL){
		CVMFileUtil::m_bufInitialPath=NULL; //Important: don't delete previous - it's still in 'to-delete-list'
	}
	CEString s; if(!CVMFileUtil::GetCurrentPath(s)){ s.Reset(); s.AddString("Error_path_retrieving"); }
	CVMFileUtil::m_bufInitialPath=CEUtil::copyString(s.getData());
	CEProcessor::AddToDeleteList(CVMFileUtil::m_bufInitialPath);
}
bool CVMFileUtil::GetCurrentPath(CEString& str) {
	//CEException::halt("CEUtil::GetCurrentPath not implemented");

	if(CVMFileUtil::m_bufGetCurrentPath==NULL) {
		CVMFileUtil::m_bufGetCurrentPath = (char*)CMemoryManager::mmalloc(MAX_PATH_LENGTH);
		CEProcessor::AddToDeleteList(CVMFileUtil::m_bufGetCurrentPath);
	}

	//Nt: 'getcwd' is deprecated. Why? '_getcwd' has same parameters.
	char* res = _getcwd(CVMFileUtil::m_bufGetCurrentPath, MAX_PATH_LENGTH);	//Note: allocates buffer if parameter==NULL (len==0 means arbitrary there)
	if(res==NULL){ str.Reset(); return false; }

	str.Reset(); str.AddString(res); return true;
}
bool CVMFileUtil::MkDir(const char* dir) {
	if(dir==NULL)return false;
	if(mkdir(dir) == -1)return false;
	return true;
}
bool CVMFileUtil::ChDir(const char* path) {
	if(CVMFileUtil::m_bufInitialPath==NULL){ CVMFileUtil::SetCurrPathAsInitial(); } //set initial path if not set

	//CEException::halt("CEUtil::ChangeDir not implemented");
	if(path==NULL)return false;
	if(chdir(path) == -1)return false;
	CVMFileUtil::m_bInitialDirChanged=true;
	return true;
}
bool CVMFileUtil::RmDir(const char* dir) {
	if(dir==NULL)return false;
	if(rmdir(dir) == -1)return false;
	return true;
}
/*
*	Note: calling this function is a great way to introduce 'time-of-check-to-time-of-use' bugs!
*	If one needs to read/write file --> one must try open it.
*/
bool CVMFileUtil::IsFileExist(const char* fn) { // warning: undefined return if 'fn' is a directory name!
	if(fn==NULL)return false;
	bool bRes=false;

	// Method 1: to try open it, but it may lead to conflicts e.g. in multithread program
	// ifstream ifs; ifs.open(fn,ios::in|ios::nocreate); if(ifs.is_open()){ ifs.close(); bRes=true; } return bRes;

	// Method 2: use 'stat' function from <sys/stat.h>
	// The code is portable to Linux, Windows, Unix, Solaris, etc.
	// But the title of header file may be different.
	struct stat stFileInfo;
	int iFileAttrs = stat(fn,&stFileInfo); //or use 'fstat'?
	if(iFileAttrs == 0) { // 'stat' successful
		bRes=true;
	} else {
		bRes=false;
		// Note form 'http://www.techbytes.ca/techbyte103.html':
		// We were not able to get the file attributes.
		// This may mean that we don't have permission to
		// access the folder which contains this file. If you
		// need to do that level of checking, lookup the
		// return values of stat which will give you
		// more details on why stat failed.
	}
	return bRes;

	// Method 3: use 'access' function from <unistd.h> (Linux) or <io.h> (Windows)
	// Info: http://publib.boulder.ibm.com/iseries/v5r1/ic2924/index.htm?info/apis/access.htm
	// Info: http://stackoverflow.com/questions/230062/whats-the-best-way-to-check-if-a-file-exists-in-c-cross-platform
	// access modes:
	// [win] 00 Existence only, 02 Write permission, 04 Read permission, 06 Read and write permission 
	// [unix] F_OK (exists), R_OK (for reading), W_OK (for writing), X_OK (for execution), + R_OK|W_OK|...
	//bRes = (access( fn, 0 ) != -1 ); //or use '_access' for Windows?
	//return bRes;
}

bool CVMFileUtil::IsDirExist(const char* dir) {
	return CVMFileUtil::ReadDir(dir, NULL, false);
}

/*
*	Notes
*	=====
*		Win32
*			dir: "." (Current), ""(Root!!!), "/"(Error)
*			Functions used: ::FindFirstFile, ::FindNextFile, ::FindClose, ::GetFileSize, ::GetFileTime
*			sub-dirs returned by FindXXXFile: "..", "."
*		Linux
*			dir: "." (Current)
*			Functions used: opendir, readdir, closedir
*/
//#ifdef VMLIB_WIN
//#else //#ifdef VMLIB_WIN
//#endif //#ifdef VMLIB_WIN
bool CVMFileUtil::ReadDir(const char* dir, CAPtrSeq* files, bool bReadDirs) {
	if(dir==NULL)return false;

#ifdef ALHEP_WIN
	//WIN32_FIND_DATA fd; //={0};
	WIN32_FIND_DATAA fd;									//non-unicode!
	unsigned long dwSkipAttr = 0; if(!bReadDirs)dwSkipAttr|=FILE_ATTRIBUTE_DIRECTORY;	//if(!bCountHidden)dwAttr|=FILE_ATTRIBUTE_HIDDEN;
	CEString sTmp(dir); sTmp.AddString("\\*.*"); //"\\*"
	//HANDLE hFile = ::FindFirstFile(sTmp.GetBuffer(), &fd);
	HANDLE hFile = ::FindFirstFileA(sTmp.getData(), &fd);	//non-unicode!
	if (hFile == INVALID_HANDLE_VALUE) {
		//int iErr = ::GetLastError();
		return false;
	}
	if(files != NULL) {
		do {
			bool bSkipFile=false;
			if((fd.dwFileAttributes & dwSkipAttr) != 0) bSkipFile=true;
			if((fd.cFileName[0] == '.') && (!bSkipFile)) {
				//ignore ".." and "." directories
				if(CEUtil::equals(fd.cFileName,"..") || CEUtil::equals(fd.cFileName,".")) bSkipFile=true;
			}
			if(!bSkipFile) {
				files->add(new CVMFileInfo(
					fd.cFileName,
					(fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)==FILE_ATTRIBUTE_DIRECTORY
					));

				// Get file size
				//DWORD FileSizeHigh=0; DWORD FileSize=::GetFileSize(hFile,&FileSizeHigh);

				// get file times
				//FILETIME ftCreattion={0}; FILETIME ftLastAccess={0}; FILETIME ftLastWrite={0};
				//::GetFileTime(hFile,&ftCreattion,&ftLastAccess,&ftLastWrite);
			}
		//} while (::FindNextFile(hFile, &fd));
		} while (::FindNextFileA(hFile, &fd)); //non-unicode!
		::FindClose(hFile);
		DWORD dwError = ::GetLastError(); if(dwError != ERROR_NO_MORE_FILES){ return false; } //error
	}else{
		::FindClose(hFile);
	}
#else //linux, unix...
	//functions: opendir, readdir, closedir
    DIR *dp; struct dirent *dirp;
    if((dp=opendir(dir)) == NULL) {
		//cout << "Error(" << errno << ") opening " << dir << endl return errno;
		return false;
    }
	if(files != NULL) {
		while((dirp=readdir(dp)) != NULL) {
			//Note. dirp->d_type: 8 == A file, 4 == A directory
			bool bAddFile=true;
			if((!bReadDirs) && (dirp->d_type==4))bAddFile=false;
			files->Add(new CVMFileInfo(dirp->d_name,(dirp->d_type==4)));
			//
			//if(recurse){ // not tested
			//  test if dir here (or use test inside 'ReadDir')
			//	CEString sSubDir(dir); sSubDir.AddString("/"); sSubDir.AddString(dirp->d_name);
			//	ReadDir(sSubDir.GetBuffer(),files,ADD_PREFIX_HERE));
			//}
		}
	}
    closedir(dp);
#endif //#ifdef VMLIB_WIN
    return true;
}

/////////////////////////////////examples
/*
//Win32
int SearchDirectory(std::vector<std::string> &refvecFiles,
                    const std::string &refcstrRootDirectory,
                    const std::string &refcstrExtension,
                    bool bSearchSubdirectories = true) {
   std::string strFilePath; // Filepath
   std::string strPattern; // Pattern
   std::string strExtension; // Extension
   HANDLE hFile; // Handle to file
   WIN32_FIND_DATA FileInformation; // File information

   strPattern = refcstrRootDirectory + "\\*.*";
   
   hFile = ::FindFirstFile(strPattern.c_str(), &FileInformation);
   if(hFile != INVALID_HANDLE_VALUE) {
      do {
         if(FileInformation.cFileName[0] != '.') {
            strFilePath.erase();
            strFilePath = refcstrRootDirectory + "\\" + FileInformation.cFileName;
            
            if(FileInformation.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
               if(bSearchSubdirectories) { // Search subdirectory
                  int iRC = SearchDirectory(refvecFiles, strFilePath, refcstrExtension, bSearchSubdirectories);
                  if(iRC)return iRC;
               }
            } else { // Check extension
               strExtension = FileInformation.cFileName;
               strExtension = strExtension.substr(strExtension.rfind(".") + 1);
               if(strExtension == refcstrExtension) { // Save filename
					refvecFiles.push_back(strFilePath);
               }
            }
         }
      } while(::FindNextFile(hFile, &FileInformation) == TRUE);
      
      // Close handle
      ::FindClose(hFile);
      
      DWORD dwError = ::GetLastError();
      if(dwError != ERROR_NO_MORE_FILES) return dwError;
}
*/
