////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// Debug.h: interface for the CDebug class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DEBUG_H__0E04A314_5FC5_44EE_8F6E_98391B27BC69__INCLUDED_)
#define AFX_DEBUG_H__0E04A314_5FC5_44EE_8F6E_98391B27BC69__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CDebug : public CEObject
{
public:
	static const int LOG_DEFAULT;
	static const int LOG_NO_OBJECT;
	static const int LOG_TEX;
	static const char* GetDebugFileName();

private:
	static bool TEX_LOG_STARTED;

	char* m_cn; // class ID
	long m_num; // message counter
	static CEPtrArray* data;

public:
	void logperiod(int level = 0, const char* message = NULL);
	void watch_periods(bool do_watch);
	void reset_periods(int level = -1); // '-1' means reset at any debug level
	static CETimeMark* GetAbsTime();
	static char* GetAbsTimeString();
	//static void ClearAbsTime(); // deletes global time mark
	static void ClearAllTimers(); // deletes all the timer marks
	long GetMessagesCounter();
private:
	CETimeMark* update_period();
	void clear_periods_data(); // deletes time mark
	//long period;
	CETimeMark* m_time_mark;

	bool do_watch_period;
	//static long m_start_time;
	static CETimeMark* m_start_time;
	static void ResetAbsTime();

public:
	bool is_log_keeped(int level);
	int set_level(int level);
	int get_level();
	const char* GetDebugClassID();
private:
	int m_level;

public:
	void log(int level, long val, const char* message, int mode = 0);
	void log(int level, long val_1, long val_2, const char* message, int mode = 0);
	void log(int level, long val_1, long val_2, long val_3, const char* message, int mode = 0);
	void log(int level, long val, const char* message, CEObject* obj, int mode = 0);
	void log(int level, const char* message, const char* message2);
	void log(int level, const char* message, const char* message2, CEObject* obj, int mode = 0);
	void logIns(int level, const char* message, const char* messageToInsert);
	void log(int level, long val_1, long val_2, const char* message, CEObject* obj, int mode = 0);
	void log(int level, long val_1, long val_2, long val_3, const char* message, CEObject* obj, int mode = 0);
	void log(int level, const char* message, CEObject* obj, int mode = 0);
	void log(int level, const char* message, CEObject* obj_1, CEObject* obj_2, int mode = 0);
	void log(int level, const char* message, CEObject* obj_1, CEObject* obj_2, CEObject* obj_3, int mode = 0);
	void log(CEObject* obj, int mode = 0);
	void log(int level, CEObject* obj, int mode = 0);
	void log(const char* message, int mode = 0);
    void log(int level, const char* message, int mode = 0);
private:
	static bool m_busy;

private:
	CEPtrArray* m_debug_exprs;
public:
	void store_debug_expression(int level, const char* message, CEObject* expr);
	void store_debug_list();
	void add_debug_expr(CEObject* expr, bool store_immediately = false);

public:
    CDebug(const char* class_name);
    virtual ~CDebug();

	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual bool contain_vector(void * obj);

private:
	static DWORD m_debugFN_counter;
	static char* get_unique_debugFN();

public:
	static void init();
	static void finalize();
	static CDebug* GetDebugObject(const char* id);
private:
	static CEPtrArray* m_debug_objects;
	static const char* getPrefix();
	static const char* getPostfix();
	static char* m_sFullPathDebugNB;				//NULL if until path is changed
	static char* m_sFullPathDebugTeX;				//NULL if until path is changed
	// debug data acsess
	static char* CDebug::get_next(ifstream* pifs);
    static void put(const char* id, long value);
    static long get(const char* id);
};

#endif // !defined(AFX_DEBUG_H__0E04A314_5FC5_44EE_8F6E_98391B27BC69__INCLUDED_)
