////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ECharMap.h: interface for the CECharMap class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ECHARMAP_H__D74820D1_EDF0_4194_8FE3_7EA384A03A31__INCLUDED_)
#define AFX_ECHARMAP_H__D74820D1_EDF0_4194_8FE3_7EA384A03A31__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CECharMapCart : public CESerializable
{
	CECharMapCart* m_pLeft; // less than this
	CECharMapCart* m_pRight; // greather than this

	char* m_value;

public:
	long m_mark; // used for temporary mark

private:
	// clones all the input
	CECharMapCart(CECharMapCart* _pLeft, CECharMapCart* _pRight, const char* _value);
public:
	CECharMapCart(const char* _value);
	virtual ~CECharMapCart();

	const char* GetValue();
	CECharMapCart* GetLeft();
	CECharMapCart* GetRight();
	void SetLeft(CECharMapCart* _cart); // no clone or delete may be performed
	void SetRight(CECharMapCart* _cart); // no clone or delete may be performed
	bool is_ident(const char* val);
	bool is_equal(const char* val);
	int compare_value(const char* val); // 1 means 'val is greather'
	CEString* storeTree(int depth, int mode = 0);

	virtual CEString* toString(int mode = 0);
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual CEObject* copy() const;
	virtual bool contain_vector(void * obj);
};

class CECharMap : public CESerializable
{
	CECharMapCart* m_root_value;
	long m_norm_cntr; // how much are created since last normalization
public:
	CECharMap();
	virtual ~CECharMap();

	bool is_exist(const char* val);
	const char* getValueOrigin(const char* val, bool do_create);
	void put_value(const char* val);

	static CDebug* dbg;
	virtual CEString* toString(int mode = 0);
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual CEObject* copy() const;
	virtual bool contain_vector(void * obj);

private:
	CECharMap(CECharMapCart* _root, long _norm_cntr); // clones root tree
	CECharMapCart* get_value_cart(const char* val, bool do_create);
	void NormalizeMap(CECharMapCart* cart, CECharMapCart* root, bool as_left, bool size_is_marked);
	long CalcMaxMapRepth(CECharMapCart* root, bool mark_items);
	long CalcMinMapRepth(CECharMapCart* root, bool mark_items);
	long CalcMapSize(CECharMapCart* root, bool mark_items);
	long CheckSizeMarked(CECharMapCart* root); // declares error (-1) for wrong marks
};

#endif // !defined(AFX_ECHARMAP_H__D74820D1_EDF0_4194_8FE3_7EA384A03A31__INCLUDED_)
