////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EUtil.h: interface for the CEUtil class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EUTIL_H__35F12EE0_CEEC_11D5_90CB_00403393DD18__INCLUDED_)
#define AFX_EUTIL_H__35F12EE0_CEEC_11D5_90CB_00403393DD18__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define MAX_STRING_LENGTH		200000000
#define WARNING_STRING_LENGTH	50000000

//##ModelId=3E3FF2CF0399
class CEUtil : public CEObject  
{
public:
	static bool LEQU(bool a, bool b);
	static bool LXOR(bool a, bool b);
	static bool LCPY(bool a); // copies int to "true" or "false"
	static bool compareSubStrings(const char* s1, long start1, long end1, const char* s2, long start2=0, long end2=-1);
	static CDebug* dbg;

	//##ModelId=3E3FF2CF03CA
	static bool a_less_b(const char* a, const char* b);
	static int compare_ab(const char* a, const char* b); // 1 means a<b, -1 means a>b
	//##ModelId=3E3FF2CF03C7
	static char* composeId(const char* a, const char* b);
	//##ModelId=3E3FF2CF03C3
	static char* concat(const char* a, const char* b, char insert_sym = '\0');
	//##ModelId=3E3FF2CF03BF
	static char* copyStringFromTo(const char* s, long start_pos, long end_pos);
	//##ModelId=3E3FF2CF03BC
	static char* copyStringFrom(const char* s, long start_pos);
	//##ModelId=3E3FF2CF03B9
	static char* copyStringTo(const char* s, long end_pos);
	//##ModelId=3E3FF2CF03B5
	static char* copySubString(const char* s, long len, long start_pos= 0);
	static char* removeSubString(const char *s, long start_pos, long len);
	//##ModelId=3E3FF2CF03B2
	static long findSymbol(const char* s, char c);
	static long findSymbol(const char *s, char c, long start_pos, bool check_length = false);
	static long findSymbolFromTo(const char *s, char c, long start_pos, long end_pos);
	//##ModelId=3E3FF2CF03AF
	static bool equals(const char* a, const char* b);
	//##ModelId=3E3FF2CF03AC
	static long length(const char* s, char end_pos = '\0');
	//##ModelId=3E3FF2CF03AA
	static char* copyString(const char* other);

	/*
	*	SopyStringAdv note: trg should be pre-allocated to size: s_len+1 (for break symbol);
	*	nSymsToCpy is number of symbols in string including '\0' or -1
	*/
	static void CopyStringAdv(char *trg, const char *s, long nSymsToCpy); // nSymsToCpy should be length(s)+1 or -1
	static void copySymsTo(char *trg, const char *src, long len=-1, long max_len=200); // does not appends '\0'

	static char* addint(const char* a, int i);
	static long findSubString(const char *s, const char *sub);

	static int addACBNumber(char* str, long n, int f_pos); // string should be allocated, and NOT terminated before 'f_pos'

	static unsigned UTL_COPY_NUMBERS;
	static unsigned UTL_COPY_SYMBOLS;
	static unsigned UTL_COPY_NO_FIRST_NUMBER;
	static unsigned UTL_COPY_SKIP_BSLASH_CMD; // {p\_1->p1} but {p\gamma_y->pgamma_y}
	static char* copyStringSyms(const char* src, unsigned sym_flags, const char* sym_list);
	//##ModelId=3E3FF2CF03A9
	CEUtil();
	//##ModelId=3E3FF2CF039B
	virtual ~CEUtil();

	// misc. utilites
	static bool IsSymbol(char c);
	static bool IsNumber(char c);
	static bool IsAllowedFileName(const char* str);

	static bool atolSafe(const char* s, long* res);
	static bool atofSafe(const char* s, double* res);
	//static int Trunc(double val); // moved to CLMath
	//static int Round(double val); // moved to CLMath
	static int GetDigitsNum(long val); // 0 for 0, 1 for 1..9, 2 for 10...99 etc.

	// NAN float: 7fc00000, NAN double: 7ff8000000000000, +INF float: 7f800000, +INF double: 7ff0000000000000, 7->f for negatives
	/*
	*	Note1: unexpected problems spotted with inline 'IsInf' and -O2 on linux.
	*		==> make'em NOT inline
	*	Note2: 'unsigned long long' type is absent in GCC ==> the following implementation works at Win32 only:
	*		static double GetNaN() { unsigned long long raw=0x7ff8000000000000; return *(double*)&raw; }
	*		static double GetInf() { unsigned long long raw=0x7ff0000000000000; return *(double*)&raw; }
	*		static bool IsInf(double val) { return (((*(unsigned long long*)&val) & 0x7ff8000000000000) == 0x7ff8000000000000); }
	*/
	static bool IsNaN(double val);
	static double GetNaN();
	static bool IsInf(double val);
	static double GetInf();

	static void PrintExpFormSimple(char* s, double val, char cExp);//, bool bForceExp=true // to D0 always
	static void PrintExpForm(char* s, double val, char cExp);//, bool bForceExp=true // s is already allocated to at least of 32, makes errors for large powers

	// Warning messages
	static int m_WarningLongStringCnt; 
	static void WarningLongString();
	static int m_ErrorLongStringCnt;
	static void ErrorLongString();
};

#endif // !defined(AFX_EUTIL_H__35F12EE0_CEEC_11D5_90CB_00403393DD18__INCLUDED_)
