////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// Sequence.h: interface for the CSequence class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SEQUENCE_H__549F6EE0_F0D6_11D5_90CB_00403393DD18__INCLUDED_)
#define AFX_SEQUENCE_H__549F6EE0_F0D6_11D5_90CB_00403393DD18__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//##ModelId=3E3FF2CD02FD
class CSequence : public CExpression
{
public:
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual long store_stream(ofstream* p_ofs, DWORD settings);
	virtual const char* get_TAG_Name();
	static const char* CLASS_TAG_NAME;
	static CESerializable* restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings);

	virtual long get_item_pos(CExpression* item, bool take_care_of_sign = true);
	void copy_from(const CSequence* other);
	virtual CEString* toString(int mode = 0);
	virtual CExpression* clone() const;
	virtual bool equals(CExpression* other);
	virtual CExpression* applyRule(CEAbstractRule* rule);
	void clear_all();
	virtual bool contain_vector(void * obj);
	CSequence();
	virtual ~CSequence();

	virtual long addItem(CExpression* pNewItem);
	virtual long InsertAt(long i, CExpression *_new);
	bool move(long from, long to);
	virtual void RemoveAt(long i);
	virtual void SetAt(long i, CExpression* _new);
	CExpression* GetAt(long i);
	CExpression* GetAt2(long i);
	const CExpression* GetAtConst(long i) const;
	long size() const;
	void addSeq(CSequence* seq); // parameter is deleted

	virtual CExpression* Arrange(int level);
	virtual CExpression* expand(bool recursive = true);

	// non-safe operations for faster work
	virtual void SetAtX(long i, CExpression* _obj_X); // no cloning new
	virtual void OverwriteAtX(long i, CExpression* _obj_X); // no cloning new, no deleting old
	virtual CExpression* RemoveAtX(long i); // no delete old
	void UnsafeSetAt(long i, CExpression* _obj_X); // no delete old, no cloning new
	void InsertAtX(long i, CExpression* _obj_X); // no cloning new
	void ClearWODelObjs();
	//virtual CExpression* UnsafeExtractAt(long i); // return item & clears it's field

#ifdef USE_PLAIN_ARRAY_FORSEQ // biuld sequence-expressions on CAPtrArray* basis
	CEPtrArray* getMemberList();
	const CEPtrArray* getMemberListConst() const;
	CSequence(CEPtrArray* pPtrArray); // Note: deletes pPtrArray!
#else
	CAPtrSeq* getMemberList();
	CSequence(CAPtrSeq* pPtrSeq); // Note: deletes pPtrArray!
#endif //USE_PLAIN_ARRAY_FORSEQ 

	bool RemoveExpr(CExpression* val);
	bool AddExprIfUnique(CExpression* val);	// add expression if unique only
	int AddUniquesFrom(CSequence* other);	// add expressions if unique only, returns N[added], ignores other's sign & level!

private:
#ifdef USE_PLAIN_ARRAY_FORSEQ // biuld sequence-expressions on CAPtrArray* basis
	CEPtrArray* m_pitems;
#else
	CAPtrSeq* m_pitems;
#endif //USE_PLAIN_ARRAY_FORSEQ 

protected:
	void exchange(long a, long b);
};

#endif // !defined(AFX_SEQUENCE_H__549F6EE0_F0D6_11D5_90CB_00403393DD18__INCLUDED_)
