////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EFunction.cpp: implementation of the CEFunction class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
const char* CEFunction::TEMP_FUNC_TITLE = "TempFuncTitle";
CEFunction::CEFunction(const char* pName, const CEPtrArray* pPtrArray, bool _bIsCommutative, bool _bSelfConjugated)
: CEAbstractFunction(CEFunction::TEMP_FUNC_TITLE, pPtrArray) {
	// Note: function string should also be cloned/deleted for abstract function
	if (pName != NULL) m_pName = CEUtil::copyString(pName);

	m_bIsCommutative=_bIsCommutative;
	m_bSelfConjugated=_bSelfConjugated;
	//The function may become !commutative for !commutative arguments
	//TODO: make 3 cases: commutative, non-commutative & agrs-dependent!
	if(m_bIsCommutative)m_fInheritedProperties|=CEProperties::PROP_MASK_COMMUTATIVE;
	if(m_bSelfConjugated)m_fInheritedProperties|=CEProperties::PROP_MASK_SELF_CONJUGATED;
}

CEFunction::CEFunction(const char* pName, const CExpression* pPar, bool _bIsCommutative, bool _bSelfConjugated)
: CEAbstractFunction(CEFunction::TEMP_FUNC_TITLE, pPar) {
	// Note: function string should also be cloned/deleted for abstract function
	if (pName != NULL) m_pName = CEUtil::copyString(pName);

	m_bIsCommutative=_bIsCommutative;
	m_bSelfConjugated=_bSelfConjugated;
	//The function may become !commutative for !commutative arguments
	//TODO: make 3 cases: commutative, non-commutative & agrs-dependent!
	if(m_bIsCommutative)m_fInheritedProperties|=CEProperties::PROP_MASK_COMMUTATIVE;
	if(m_bSelfConjugated)m_fInheritedProperties|=CEProperties::PROP_MASK_SELF_CONJUGATED;
}

CEFunction::CEFunction(const char* pName, const CExpression* pPar1, const CExpression* pPar2, bool _bIsCommutative, bool _bSelfConjugated)
: CEAbstractFunction(CEFunction::TEMP_FUNC_TITLE, pPar1, pPar2) {
	// Note: function string should also be cloned/deleted for abstract function
	if (pName != NULL) m_pName = CEUtil::copyString(pName);

	m_bIsCommutative=_bIsCommutative;
	m_bSelfConjugated=_bSelfConjugated;
	//The function may become !commutative for !commutative arguments
	//TODO: make 3 cases: commutative, non-commutative & agrs-dependent!
	if(m_bIsCommutative)m_fInheritedProperties|=CEProperties::PROP_MASK_COMMUTATIVE;
	if(m_bSelfConjugated)m_fInheritedProperties|=CEProperties::PROP_MASK_SELF_CONJUGATED;
}

CEFunction::CEFunction(const char* pName, bool _bIsCommutative, bool _bSelfConjugated)
: CEAbstractFunction(CEFunction::TEMP_FUNC_TITLE) {
	// Note: function string should also be cloned/deleted for abstract function
	if (pName != NULL) m_pName = CEUtil::copyString(pName);

	m_bIsCommutative=_bIsCommutative;
	m_bSelfConjugated=_bSelfConjugated;
	//The function may become !commutative for !commutative arguments
	//TODO: make 3 cases: commutative, non-commutative & agrs-dependent!
	if(m_bIsCommutative)m_fInheritedProperties|=CEProperties::PROP_MASK_COMMUTATIVE;
	if(m_bSelfConjugated)m_fInheritedProperties|=CEProperties::PROP_MASK_SELF_CONJUGATED;
}

CEFunction::~CEFunction() {
	if (m_pName != CEFunction::TEMP_FUNC_TITLE) { CMemoryManager::mfree((char*)m_pName); m_pName = CEFunction::TEMP_FUNC_TITLE; }
}

const char* CEFunction::CLASS_NAME = "CEFunction";
const char* CEFunction::ClassName() const { return CLASS_NAME; }
const char* CEFunction::CLASS_TAG_NAME = "func";
const char* CEFunction::get_TAG_Name() { return CLASS_TAG_NAME; }

CExpression* CEFunction::clone() const {
	CExpression* res = new CEFunction(m_pName, GetParamsConst(),m_bIsCommutative,m_bSelfConjugated);
	copyAttributesTo(res);
	return res;
}

bool CEFunction::equals(CExpression* other) {
	if (!CEAbstractFunction::equals(other)) return false;
	// also compare commutation flag
	if (props_long() != other->props_long()) return false;
	return true;
}

void CEFunction::collectProperties(CEProperties* props, unsigned set) {
	CEAbstractFunction::collectProperties(props, set);
	//Nt: "!commutative etc." due to parameters is calculated in CEAbstractFunction::collectProperties()
	if (!m_bIsCommutative) props->setProperty(CEProperties::PROP_MASK_COMMUTATIVE, false);
	if (!m_bSelfConjugated) props->setProperty(CEProperties::PROP_MASK_SELF_CONJUGATED, false);
}

unsigned long CEFunction::props_long() {
	unsigned long res = CEAbstractFunction::props_long();
	//Nt: "!commutative etc." due to parameters is calculated in CEAbstractFunction::props_long()
	if (!m_bIsCommutative) res=CEProperties::set_property_to(res,CEProperties::PROP_MASK_COMMUTATIVE,false);
	if (!m_bSelfConjugated) res=CEProperties::set_property_to(res,CEProperties::PROP_MASK_SELF_CONJUGATED,false);
	return res;
}

long CEFunction::store_stream(ofstream* p_ofs, DWORD settings) {

	// store function name as symbol
	CESymbol* func_name = new CESymbol(getTitle());
	long res = CESupport::store_stream(func_name, p_ofs, settings);
	delete func_name;

	int s = params_num();
	res += CESupport::storeXMLLong(s,p_ofs,settings);
	int i;
	for (i=0; i<s; i++) {
		CExpression* item = get_param(i);
		res += CESupport::store_stream(item, p_ofs, settings);
	}
	long lComm=(m_bIsCommutative?1:0);
	res += CESupport::storeXMLLong(lComm,p_ofs,settings);
	long lSelfConj=(m_bSelfConjugated?1:0);
	res += CESupport::storeXMLLong(lSelfConj,p_ofs,settings);
	return res;
}

CESerializable* CEFunction::restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings) {

	// load title
	long skip_pos = CESupport::skip_item_in_buffer(data, pos, settings);
	if (skip_pos <= 0) { CEException::halt("Error restoring CEFunction : Can't load Function."); return NULL; }
	CESerializable* dd = CESupport::read_buffer(data, pos, settings);
	if ((dd == NULL) || (!CEUtil::equals(dd->ClassName(), CESymbol::CLASS_NAME))) {
		CEException::halt("Error loading TITLE for CEFunction."); if (dd != NULL) delete dd; return NULL;
	}
	CESymbol* func_title = (CESymbol*)dd;
	pos = skip_pos;

	long lParsN=-1;
	if (!CESupport::readXMLLong(data,&pos,&lParsN,settings,CLASS_NAME)) { delete func_title; return NULL; }

	// load parameters
	CEPtrArray* params_list = new CEPtrArray();
	long i=0;
	for(;i<lParsN;i++) {
		long skip_pos = CESupport::skip_item_in_buffer(data, pos, settings);
		if (skip_pos > 0) {
			CESerializable* item = CESupport::read_buffer(data, pos, settings);
			pos = skip_pos;
			if (item != NULL) { params_list->add(item); }
			else { //Error
				delete params_list; delete func_title; CEException::halt("Error loading PARAMETERS for CEFunction."); return NULL;
			}
		} else {
			delete params_list; delete func_title; CEException::halt("Error loading PARAMETERS for CEFunction."); return NULL;
		}
	}

	long lComm=1;
	if (!CESupport::readXMLLong(data, &pos, &lComm, settings, CLASS_NAME,true)) lComm=1;
	long lSelfConj=1;
	if (!CESupport::readXMLLong(data, &pos, &lSelfConj, settings, CLASS_NAME,true)) lSelfConj=1;
	if (!CESupport::checkAllRead(data, &pos, settings, CLASS_NAME)) { CEException::halt("Error loading CEFunction: unexpected parameters spotted."); }
	CEFunction* res = new CEFunction(func_title->get_name(), params_list, (lComm>0), (lSelfConj>0));
	delete func_title; delete params_list;
	return res;
}
