////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EFunction.h: interface for the CEFunction class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EFUNCTION_H__34EC0A20_B4F1_11D5_90CB_00403393DD18__INCLUDED_)
#define AFX_EFUNCTION_H__34EC0A20_B4F1_11D5_90CB_00403393DD18__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/*
*	This class is used to declare abstract (not implemented) functions
*
*	Note:
*		The two property-politics are allowed:
*			1. function is !commutative(!self-conjugated)
*		or
*			2. it depends on parameter flags.
*	TODO: add case:
*			3. function is commutative neglecting the argument flags
*/
class CEFunction : public CEAbstractFunction  
{
	bool m_bIsCommutative;
	bool m_bSelfConjugated;
public:
	CEFunction(const char* pName, const CEPtrArray* pPtrArray, bool _bIsCommutative=true, bool _bSelfConjugated=true);
	CEFunction(const char* pName, const CExpression* pPar, bool _bIsCommutative=true, bool _bSelfConjugated=true);
	CEFunction(const char* pName, const CExpression* pPar1, const CExpression* pPar2, bool _bIsCommutative=true, bool _bSelfConjugated=true);
	CEFunction(const char* pName, bool _bIsCommutative=true, bool _bSelfConjugated=true);
	virtual ~CEFunction();

	virtual CExpression* clone() const;
	virtual bool equals(CExpression* other);

	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	static const char* CLASS_TAG_NAME;
	virtual const char* get_TAG_Name();
	virtual long store_stream(ofstream* p_ofs, DWORD settings);
	static CESerializable* restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings);

	static const char* TEMP_FUNC_TITLE;

public:
	virtual unsigned long props_long();
protected:
	virtual void collectProperties(CEProperties* props, unsigned set = CEProperties::PROP_MASK_ALL);
};

#endif // !defined(AFX_EFUNCTION_H__34EC0A20_B4F1_11D5_90CB_00403393DD18__INCLUDED_)
