////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// FSp.h: interface for the CFSp class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_FSP_H__13C8B217_109F_41E7_945D_FDED1D2B97CF__INCLUDED_)
#define AFX_FSP_H__13C8B217_109F_41E7_945D_FDED1D2B97CF__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//##ModelId=3E3FF2DE005D
class CFSp : public CEAbstractFunction  
{
	static CEPtrArray* m_spur_cache;
public:
	virtual CExpression* clone() const;
	virtual CExpression* Arrange(int level); // To extract Sp[0]->0
	static CExpression* Spur(CExpression* val, bool do_delete = true);
	static CDebug* dbg;
	virtual CExpression* evaluate(int level = 0);
	CFSp(const CExpression* data);
	virtual ~CFSp();
public:
	virtual unsigned long props_long();
protected:
	virtual void collectProperties(CEProperties* props, unsigned set = CEProperties::PROP_MASK_ALL);
public:

	static const char* CLASS_NAME;
	virtual const char* ClassName() const;

	// Tr[] operations:
	bool IsOfNormalForm(); // Tr[... G..G-hat.G...G5/P/M], G5/P/M is last, thr only G and G-hat are other non-commutatives
	CExpression* ToNormalForm(); // this, cloned result(SUM of Tr) or NULL(if failed)
	int CalcGammaNum(); // Number of Gamma/G-hat objects (but no G5/+/-), -1 if unexpected object spoted (or g-5 at wrong place)
	// CExpression* GetGammaNo(int i); // Get Gamma/G-hat object
	bool SwapToGamma(int i); // Makes Gamma-i first in Gamma-sequence, false if failed (unexpected non-commutative object spotted)
	int FindGammaCoupleForIndex(CExpression* ind, bool cov); // Logical no is returned (0..CalcGammaNum()-1)
	bool CoupleGammaToMetric(CEMetric* metr); // metr is not affected, try every gamma
	CEPtrArray* GetTIndicesList(); //Array of CTIndex*/NULL objects of Gamma-indices (NULL for Gamma-X or coupled internally), to be deleted by user
	int GetLastGammaXID(); // 0/1/-1 ot INT_MIN
	CExpression* ExpandLastGammaX(bool toGpm); // this or sum(clone(G+)+clone(G-)) or ZERO(for 1-sized source)
	CExpression* RetrieveSrcSeq(bool _StraightOrder, bool _NoLastGX, bool _NoFirstG); // cloned product w/o(or with) last gamma-5 and first Gamma-i

	static DWORD m_Settings;
	static const DWORD SP_CHECK_SUCCESS;
	static bool m_TracesBondarev;
	static int m_sc_order_max; // Max order of pre-calculated traces, 0 for NO CACHE USAGE

	// cache (same objects from DataStorage, do not delete)
	static CExpression* m_gP; // gamma+
	static CExpression* m_gM; // gamma-
	static CExpression* m_g5; // gamma5

	static void fill_cache_upto(int order);

	static CExpression* TruncateZeroTraces(CExpression* src, bool del_src=true);
	static CExpression* TruncateZeroTracesRuleProc(CExpression* src, CEPtrArray* data_arr);

private:
	CExpression* SpCommon(CEProduct* src);

	CExpression* Sp(CEProduct* seq);
	CExpression* Sp(CEPtrArray* data);
	static CExpression* Sp_calculation(CEPtrArray *data, bool do_fill_cache = false);

	CExpression* SpBondarev(CExpression* src);
	CExpression* SpBondarev(CEPtrArray* data, bool _gPlus);
};

#endif // !defined(AFX_FSP_H__13C8B217_109F_41E7_945D_FDED1D2B97CF__INCLUDED_)
