////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// DCorrRule.cpp: implementation of the CDCorrRule class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDCorrRule::CDCorrRule(CExpression* _corr_expr, int _corr_kind, long _ID, CSequence* _temp_syms)
: CESerializable() {
	if (_corr_expr != NULL) {
		m_corr_expr = _corr_expr->clone();
	} else {
		CEBasic::dbg->log(0, "{Internal Error} CDCorrRule: NULL expression initialization");
		m_corr_expr = new CESymbol("<NULL data>");
	}
	m_corr_kind = _corr_kind;
	m_ID = _ID;
	m_temp_syms = (_temp_syms != NULL)?((CSequence*)_temp_syms->clone()):NULL;
}

CDCorrRule::~CDCorrRule() {
	if (m_corr_expr != NULL) delete m_corr_expr;
	if (m_temp_syms != NULL) delete m_temp_syms;
}

int CDCorrRule::GetCorrKind() { return m_corr_kind; }
CExpression* CDCorrRule::getInternalCorrExpr() { return m_corr_expr; }
long CDCorrRule::GetID() { return m_ID; }
CSequence* CDCorrRule::GetTempSyms() { return m_temp_syms; }

bool CDCorrRule::obj_equals(CEObject* other) {
	if ((other == NULL) || (!CEUtil::equals(ClassName(), other->ClassName()))) return false;
	CDCorrRule* cr = (CDCorrRule*)other;
	if (GetCorrKind() != cr->GetCorrKind()) return false;
	if (m_corr_expr != NULL) {
		if (!m_corr_expr->equals(cr->getInternalCorrExpr())) return false;
	} else if (cr->getInternalCorrExpr() != NULL) return false;
	return true;
}

bool CDCorrRule::contain_vector(void * obj) {
	if (m_corr_expr == obj) return true;
	if ((m_corr_expr != NULL) && (m_corr_expr->contain_vector(obj))) return true;
	return false;
}
