////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// DElementViewInfo.cpp: implementation of the CDElementViewInfo class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

const char* CDElementViewInfo::CLASS_NAME = "CDElementViewInfo";
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDElementViewInfo::CDElementViewInfo(CPos2D* _pos)
{
	m_pos = _pos;
	m_repetition_ID = 1; m_repetition_factor = 1;
}

CDElementViewInfo::CDElementViewInfo(double x_pos, double y_pos)
{
	m_pos = new CPos2D(x_pos, y_pos);
	m_repetition_ID = 1; m_repetition_factor = 1;
}

CDElementViewInfo::CDElementViewInfo() {
	m_pos = NULL; m_repetition_ID = 1; m_repetition_factor = 1;
}

CDElementViewInfo::CDElementViewInfo(CPos2D* _pos, int _repetition_ID, int _repetition_factor) {
	m_pos = _pos;
	m_repetition_ID = _repetition_ID; m_repetition_factor = _repetition_factor;
}

CDElementViewInfo::~CDElementViewInfo()
{
	if (m_pos != NULL) delete m_pos;
}

CPos2D* CDElementViewInfo::get_pos() {return m_pos;}
void CDElementViewInfo::set_pos(CPos2D* pos_new) {
	if (m_pos != pos_new) {
		if (m_pos != NULL) delete m_pos;
		m_pos = pos_new;
	}
}

CDElementViewInfo* CDElementViewInfo::clone() const {
	CPos2D* pos_copy = NULL;
	if (m_pos != NULL) pos_copy = m_pos->clone();
	return new CDElementViewInfo(pos_copy, m_repetition_ID, m_repetition_factor);
}

const char* CDElementViewInfo::ClassName() const { return CLASS_NAME; }

CEString* CDElementViewInfo::toString(int mode) {
	CEString* res = new CEString("< ElementViewInfo: ");
	if (m_pos == NULL) res->AddString("No position.");
	else res->AddString(m_pos->toString(mode));
	if ((m_repetition_ID != 1) || (m_repetition_factor != 1)) {
		res->AddString(", multiplicity: ");
		res->add_int(m_repetition_ID);
		res->AddString(" of ");
		res->add_int(m_repetition_factor);
	}
	res->AddString(">");
	return res;
}

void CDElementViewInfo::set_repetition(int _repetition_ID, int _repetition_factor) {
	m_repetition_ID = _repetition_ID; m_repetition_factor = _repetition_factor;
}

void CDElementViewInfo::inc_repetition_factor() { m_repetition_factor++; }
int CDElementViewInfo::get_repetition_ID() { return m_repetition_ID; }
int CDElementViewInfo::get_repetition_number() { return m_repetition_factor; }
