////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// DLine.cpp: implementation of the CDLine class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDLine::CDLine(const CDElementViewInfo* view_info) : CDElement(view_info)
{
	m_DE_kind |= CDElement::EL_LINE;
	//m_is_binded = false;	
}

CDLine::~CDLine() {
}

const char* CDLine::CLASS_NAME = "CDLine";
const char* CDLine::ClassName() const { return CLASS_NAME; }

bool CDLine::is_external() {
	if ((m_DE_kind & CDElement::EL_EXT_LINE) == CDElement::EL_EXT_LINE) return true;
	return false;
}

CEString* CDLine::GetDrawCaption(int _draw_settings) {
	CEString* res = new CEString("");
	if (_draw_settings & CFeynDiagram::DRAW_CAPS_PARTICLE_TITLES) {
		const char* cap_str = CDiagramPhysics::get_particle_symbolA(get_particle_kind(), CDiagramPhysics::SYMBOL_STYLE_TEX);
		if (cap_str != NULL) {
			if (res->length()>0) res->AddString(", ");
			res->AddString(cap_str);
		}
		if (_draw_settings & CFeynDiagram::DRAW_CAPS_PARTICLE_SIGNS) {
			CDiagramPhysics* phys = CDiagramPhysics::get_curr_physics();
			if (phys!=NULL) {
				int iC3=phys->getCharge3(get_particle_kind());
				if((iC3!=INT_MIN)&&(iC3!=0)){ if(iC3>0) res->AddString("^+"); else res->AddString("^-"); }
				//if((iC3!=INT_MIN)&&(iC3!=0)){ if(iC3>0) res->AddString("^{\\scriptscriptstyle +}"); else res->AddString("^{\\scriptscriptstyle -}"); }
			}
		}
	}
	if (_draw_settings & CFeynDiagram::DRAW_CAPS_LINE_INFO) {
		CFeynDiagram* curr_diag  = CLDrawDiagView::GetCurrentDiagram();
		if (curr_diag != NULL) {
			int line_pos = curr_diag->get_line_pos(this);
			if (res->length()>0) res->AddString(", ");
			res->add_int(line_pos);
		} else {
			const char* cap_str = "?";
			if (cap_str != NULL) {
				if (res->length()>0) res->AddString(", ");
				res->AddString(cap_str);
			}
		}
	}
	if (_draw_settings & CFeynDiagram::DRAW_CAPS_MOMENTA) {
		CExpression* m = get_internal_momentum_val();
		if (res->length()>0) res->AddString(", ");
		if (m != NULL) {
			res->AddString(m->toString(STORE_TEX));
			res->AddString("");
		} else {
			res->AddString("NULL");
		}
	}
	return res;
}


/*
bool CDLine::is_binded()
{
	return m_is_binded;
}*/
