////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// DVertexCorrRule.cpp: implementation of the CDVertexCorrRule class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDVertexCorrRule::CDVertexCorrRule(CExpression* _corr_expr,
								   int _corr_kind,
								   long _ID,
								   CSequence* _temp_syms,
								   bool need_vec_momenta, bool need_all_momenta)
								   : CDCorrRule(_corr_expr,
								   _corr_kind | CDiagCorrRules::CORR_RULE_VERTEX,
								   _ID,
								   _temp_syms)
{
	m_vec_momenta_reqiured = need_vec_momenta;
	m_all_momenta_reqiured = need_all_momenta;
}

CDVertexCorrRule::~CDVertexCorrRule() {}

bool CDVertexCorrRule::AreVecMomentaRequired() { return m_vec_momenta_reqiured; }
bool CDVertexCorrRule::AreAllMomentaRequired() { return m_all_momenta_reqiured; }
const char* CDVertexCorrRule::CLASS_NAME = "CDVertexCorrRule";
const char* CDVertexCorrRule::ClassName() const { return CLASS_NAME; }
CEString* CDVertexCorrRule::toString(int mode) {
	CEString* res = NULL;
	const char* corr_title = NULL;
	if (m_corr_kind == CDiagCorrRules::CORR_RULE_VERTEX) { corr_title = "Vert: "; }
	else { corr_title = "?: "; }
	res = new CEString(corr_title);
	res->AddString("{");res->add_int(m_ID);res->AddString("}");
	
	// log involved particles
	CDiagramPhysics* phys = CDiagramPhysics::get_curr_physics();
	if (phys == NULL) { res->AddString("{No Physics Found}"); return res; }
	CEPtrArray* vi_arr = phys->getVerticesInfo();
	if (vi_arr == NULL) { res->AddString("{No Vertex Info Array found}"); return res; }
	CDiagramVertexInfo* vi = (CDiagramVertexInfo*)vi_arr->GetAt(m_ID);
	if (vi == NULL) { res->AddString("{No Vertex Info found}"); return res; }
	if (vi->check_vert_type(CDiagramVertexInfo::VERTEX_COUNTERTERM))  { res->AddString("<CT> "); }
	int s = vi->get_tales_num();
	int i;
	for(i=1; i<=s; i++) {
		long pk = vi->get_particle_kind(i-1);
		if (pk<0) { res->AddString("-"); pk = -pk; }
		res->AddString(CDiagramPhysics::get_particle_symbolA(pk, CDiagramPhysics::SYMBOL_STYLE_NB));
		//res->AddString("("); res->add_int(pk); res->AddString(")");
		if (i<s) res->AddString(", ");
	}
	
	res->AddString(" --> ");
	res->AddString(m_corr_expr->toString(mode));
	
	if (m_temp_syms != NULL) {
		res->AddString(", ");
		res->AddString(m_temp_syms->toString(mode));
	}
	return res;
}

CEObject* CDVertexCorrRule::copy() const {
	return new CDVertexCorrRule(m_corr_expr, m_corr_kind, m_ID, m_temp_syms, m_vec_momenta_reqiured, m_all_momenta_reqiured);
}

bool CDVertexCorrRule::obj_equals(CEObject* other) {
	if (!CDCorrRule::obj_equals(other)) return false;
	return true;
}

bool CDVertexCorrRule::contain_vector(void * obj) {
	if (CDCorrRule::contain_vector(obj)) return true;
	return false;
}
