////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// DiagCorrRules.cpp: implementation of the CDiagCorrRules class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

const int CDiagCorrRules::INTERNAL_SYM_MOMENTUM			= 0x01;
const int CDiagCorrRules::INTERNAL_SYM_MASS				= 0x02;
const int CDiagCorrRules::INTERNAL_SYM_VECTOR_INDEX		= 0x04;
const int CDiagCorrRules::INTERNAL_SYM_MOMENTUM_2		= 0x08;
const int CDiagCorrRules::INTERNAL_SYM_VECTOR_INDEX_2	= 0x10;

const int CDiagCorrRules::CORR_RULE_INT_LINE				= 0x0001;
const int CDiagCorrRules::CORR_RULE_EXT_LINE				= 0x0002;
const int CDiagCorrRules::CORR_RULE_VERTEX					= 0x0004;
const int CDiagCorrRules::CORR_RULE_FLAG_CONJUGATED			= 0x0100;
const int CDiagCorrRules::CORR_RULE_COUNTER_TERM			= 0x0200;
//const int CDiagCorrRules::CORR_RULE_FLAG_ANTIPARTICLE		= 0x0400;
//const int CDiagCorrRules::CORR_RULE_FLAG_INITIAL_PARTICLE	= 0x0800;

CDebug* CDiagCorrRules::dbg = new CDebug("CDiagCorrRules");

const char* CDiagCorrRules::CLASS_NAME = "CDiagCorrRules";

/*
*	Correspondence Rule format.
*	<CExprContainer>:
*		expr: correspondence expression
*		int_info0: correspondence type (vertex/internal line/external line)
*		int_info1: correspondence ID (vertex No., ect.)
*		expr_info: list (CSequence) of internal variables, to be changed before use
*/

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CDiagCorrRules::CDiagCorrRules() : CESerializable()
{
	// fill oyt symbols data
	m_internal_symbols = new CEPtrArray();

	CESymbol* sym = NULL;
	sym = new CESymbol("p");
	m_internal_symbols->add(new CExprContainer(sym, INTERNAL_SYM_MOMENTUM, 0));
	delete sym;
	sym = new CESymbol("m");
	m_internal_symbols->add(new CExprContainer(sym, INTERNAL_SYM_MASS, 0));
	delete sym;
	sym = new CESymbol("a");
	m_internal_symbols->add(new CExprContainer(sym, INTERNAL_SYM_VECTOR_INDEX, 0));
	delete sym;
	sym = new CESymbol("q");
	m_internal_symbols->add(new CExprContainer(sym, INTERNAL_SYM_MOMENTUM_2, 0));
	delete sym;
	sym = new CESymbol("b");
	m_internal_symbols->add(new CExprContainer(sym, INTERNAL_SYM_VECTOR_INDEX_2, 0));
	delete sym;

	m_corr_data = new CEPtrArray();

	m_X_rule_no = -1; // temp
	m_X_rule_cnt = 1; // temp
}

CDiagCorrRules::~CDiagCorrRules()
{
	delete m_internal_symbols;
	delete m_corr_data;
}

const char* CDiagCorrRules::ClassName() const { return CLASS_NAME; }

CEString* CDiagCorrRules::toString(int mode) {
	if (m_corr_data == NULL) return new CEString("NULL");

	int s = m_corr_data->size();
	CEString* res = new CEString("{Correspondence Rules}");
	CEBasic::appendLineBreak(res, mode);
	int i;
	for(i=0; i<s; i++) {
		CDCorrRule* cr = (CDCorrRule*)m_corr_data->GetAt(i); //CExprContainer* cart = (CExprContainer*)m_corr_data->GetAt(i);
		if (cr == NULL) { res->AddString("???"); continue; }

		res->AddString(cr->toString(mode));
		if (i<(s-1)) CEBasic::appendLineBreak(res, mode);
	}
	res->AddString(".\n");
	return res;
}

/*CEString* CDiagCorrRules::getCorrTitle(int corr_type, int ID) {
	const char* corr_title = NULL;
	if (corr_type == CORR_RULE_VERTEX) {
		corr_title = "Vert: ";
	} else if (corr_type == CORR_RULE_INT_LINE) {
		corr_title = "Int: ";
	} else if (corr_type == CORR_RULE_EXT_LINE) {
		corr_title = "Ext: ";
	} else if (corr_type == (CORR_RULE_EXT_LINE | CORR_RULE_FLAG_CONJUGATED)) {
		corr_title = "Ext*: ";
	} else {
		corr_title = "?: ";
	}
	CEString* res = new CEString(corr_title);

	if (corr_type == CORR_RULE_VERTEX) {
		CDiagramPhysics* phys = CDiagramPhysics::get_curr_physics();
		if (phys == NULL) { res->AddString("{No Physics Found}"); return res; }
		CEPtrArray* vi_arr = phys->getVerticesInfo();
		if (vi_arr == NULL) { res->AddString("{No Vertex Info Array found}"); return res; }
		CDiagramVertexInfo* vi = (CDiagramVertexInfo*)vi_arr->GetAt(ID);
		if (vi == NULL) { res->AddString("{No Vertex Info found}"); return res; }
		
		int s = vi->get_tales_num();
		int i;
		for(i=1; i<=s; i++) {
			int pk = vi->get_particle_kind(i-1);
			if (pk<0) { res->AddString("-"); pk = -pk; }
			res->AddString(CDiagramPhysics::get_particle_symbolA(pk, CDiagramPhysics::SYMBOL_STYLE_NB));
			//res->AddString("("); res->add_int(pk); res->AddString(")");
			if (i<s) res->AddString(", ");
		}
	} else { // line
		res->AddString(CDiagramPhysics::get_particle_symbolA(ID, CDiagramPhysics::SYMBOL_STYLE_NB));
	}
	return res;
}*/

CExpression* CDiagCorrRules::get_internal_symbol(int sym_ID)
{
	int s = m_internal_symbols->size();
	int i;
	for(i=0; i<s; i++) {
		CExprContainer* item = (CExprContainer*)m_internal_symbols->GetAt(i);
		if (item->get_info() == sym_ID) return item->get_expr();
	}
	return NULL;
}

CExpression* CDiagCorrRules::get_Correspondence(int corr_type,
												long _ID,
												CSequence* new_syms,
												bool demand_vars_coinsidence)
{
	dbg->log(30, _ID, "get_Correspondence (ID == %ld), syms:", new_syms);

	CDCorrRule* cr = GetCorrRule(corr_type, _ID);
	if (cr == NULL) {
		dbg->log(2, corr_type, _ID, "{WARNING} get_Correspondence no correspondence found (corr_type: %x, ID: %x).");
		dbg->log(2, "Correspondent particle: ", CDiagramPhysics::get_particle_symbolA(_ID,CDiagramPhysics::SYMBOL_STYLE_NB));
		return NULL;
	}
	
	CExpression* res = cr->getInternalCorrExpr()->clone(); // clone result
	CSequence* temp_syms = cr->GetTempSyms();

	int s_0 = 0;
	if (temp_syms != NULL) s_0 = temp_syms->size();
	int s_1 = 0;
	if (new_syms != NULL) s_1 = new_syms->size();

	bool vars_are_OK = (s_0 <= s_1);
	if (demand_vars_coinsidence) vars_are_OK = (s_0 == s_1);
	if (!vars_are_OK) {
		// Error
		CEString sTypeID("Unknown");
		if (corr_type & CDiagCorrRules::CORR_RULE_INT_LINE) { sTypeID.Reset(); sTypeID.AddString("Int. line"); }
		else if (corr_type & CDiagCorrRules::CORR_RULE_EXT_LINE) { sTypeID.Reset(); sTypeID.AddString("Ext. line"); }
		else if (corr_type & CDiagCorrRules::CORR_RULE_VERTEX) { sTypeID.Reset(); sTypeID.AddString("Vertex"); }
		if (corr_type & CDiagCorrRules::CORR_RULE_FLAG_CONJUGATED) sTypeID.AddString("+Conjugated");
		if (corr_type & CDiagCorrRules::CORR_RULE_COUNTER_TERM) sTypeID.AddString("+CounterTerm");
		sTypeID<<" ("; sTypeID.add_int(_ID,true); sTypeID<<")";
		dbg->log(0, "CDiagCorrRules ERROR: request with invalid set of variables. Request type: ", sTypeID.getData());
		dbg->log(0, "Correspondence rule (before syms replacement): ", res);
		dbg->log(0, "Expreted replacements for:", temp_syms);
		dbg->log(0, "Symbols for replacement are:", new_syms);
		CEException::halt();
	}

	if (s_0 > 0) {
		// change variables
		CEReplList* rl = new CEReplList();
		int i;
		for(i=0; i<s_0; i++) {
			CExpression* ind_old = temp_syms->GetAt(i);
			CExpression* ind_new = new_syms->GetAt(i);
			
			if (!ind_old->equals(ind_new)) {
				rl->add(ind_old, ind_new, false);
			}
		}
		res = rl->ApplyTo(res, true);
		delete rl;
	}

	dbg->log(30, "get_Correspondence result:", res);
	return res;
}

CDCorrRule* CDiagCorrRules::GetCorrRule(int corr_type, long _ID) {
	int s = m_corr_data->size();
	int i;
	for(i=0; i<s; i++) {
		CDCorrRule* cr = (CDCorrRule*)m_corr_data->GetAt(i);
		if ((cr->GetCorrKind() == corr_type) && (cr->GetID() == _ID)) {
			return cr;
		}
	}
	return NULL;
}

/*bool CDiagCorrRules::addCorrRule(int corr_type, int _ID, CExpression* _data, CSequence* _info_seq) {
	if (_data == NULL) return false;
	m_corr_data->add(new CExprContainer(_data, corr_type, _ID, _info_seq));
	return true;
}*/

// corr_flags may be CORR_RULE_COUNTER_TERM
void CDiagCorrRules::addIntLineCorrRule(long pk, CExpression* expr, CSequence* info_seq, int corr_flags) {
	if (expr == NULL) CEException::halt();
	CDPropagatorCorrRule* cr = new CDPropagatorCorrRule(expr, corr_flags, pk, info_seq);
	m_corr_data->add(cr);
	dbg->log(45, "Corr rule created", cr);
}

// corr_flags may be CORR_RULE_COUNTER_TERM
void CDiagCorrRules::addExtLineCorrRule(long pk, CExpression* expr, CSequence* info_seq, int corr_flags) {
	if (expr == NULL) CEException::halt();
	CDExtLineCorrRule* cr = new CDExtLineCorrRule(expr, corr_flags, pk, info_seq);
	m_corr_data->add(cr);
	dbg->log(45, "Corr rule created", cr);
}

void CDiagCorrRules::addVertexCorrRule(int vertex_ID, CExpression* expr, CSequence* info_seq, bool vec_mom_used, bool all_mom_used) {
	if (expr == NULL) CEException::halt();
	CDVertexCorrRule* cr = new CDVertexCorrRule(expr, CORR_RULE_VERTEX, vertex_ID, info_seq, vec_mom_used, all_mom_used);
	m_corr_data->add(cr);
	dbg->log(45, "Corr rule created", cr);
}

CExpression* CDiagCorrRules::GetVertexCorrespondence(int vertex_ID, CSequence* new_syms) {
	CExpression* res = get_Correspondence(CORR_RULE_VERTEX, vertex_ID, new_syms, true);
	return res;
}

bool CDiagCorrRules::NeedVecMomentaForVertexCorr(int vertex_ID) {
	CDCorrRule* cr = GetCorrRule(CORR_RULE_VERTEX, vertex_ID);
	if (cr == NULL) CEException::halt(dbg, "No vertex data found");
	return ((CDVertexCorrRule*)cr)->AreVecMomentaRequired();
}

bool CDiagCorrRules::NeedAllMomentaForVertexCorr(int vertex_ID) {
	CDCorrRule* cr = GetCorrRule(CORR_RULE_VERTEX, vertex_ID);
	if (cr == NULL) CEException::halt(dbg, "No vertex data found");
	return ((CDVertexCorrRule*)cr)->AreAllMomentaRequired();
}

/*
*	Parameter usage:
*		- momentum, mass, indices{vec1[,glu1...],vec2[,glu2...]}
*/
CExpression* CDiagCorrRules::GetLineCorrespondence(int line_type, long pk, CExpression* sym_mom, CExpression* sym_mass, CSequence* indices) {
	dbg->log(70, pk, "GetLineCorrespondence(part: %ld):", sym_mom);

	CSequence* syms = NULL;
	// Syms format: {p1,m,i1[,glu1],i2[,glu2]},{p1,m},{p1},{}
	if (indices != NULL) {
		syms = (CSequence*)indices->clone();

		// first items should be momentum and mass
		if ((sym_mom == NULL) || (sym_mass == NULL)) {
			dbg->log(0, pk, "CDiagCorrRules internal ERROR: NULL momentum or mass symbols (pk == %ld).");
			throw(new CEException(CEException::INT_ERROR));
		}

		syms->InsertAt(0, sym_mom);
		syms->InsertAt(1, sym_mass);
	}
	else {
		syms = new CSequence();
		if (sym_mom != NULL) {
			syms->addItem(sym_mom->clone());
			if (sym_mass != NULL) syms->addItem(sym_mass->clone());
		}
	}

	//Note: for external photons/WZ-bosons we don't currently need the full set of indices.
	//The unused indices are attached to matrix element object and used in squaring/amplitude procedures.
	//For external fermions mass is also unused ==> it is absent in 'required indices' ==> don't check exactly
	//[old & wrong:] bool bCheckParamsNum = (syms->size() >= 3);	//check more carefully if vector/gluon indices appear
	int pkAbs=((pk>=0)?pk:-pk);
	bool bCheckParamsNum = (
		((line_type & CDiagCorrRules::CORR_RULE_INT_LINE) == CDiagCorrRules::CORR_RULE_INT_LINE) ||
		//[old & wrong] add any scalar/fermion but not nuclons in structure functions approach
		//[old & wrong]((CEParticle::is_fermion_PK(pk)) && (pkAbs != CEParticle::PARTICLE_PROTON) && (pkAbs != CEParticle::PARTICLE_NEUTRON)) ||
		//add just in case...
		(pkAbs == CEParticle::PARTICLE_GLUON)
		);
	CExpression* res = get_Correspondence(line_type, pk, syms, bCheckParamsNum);
	delete syms;
	return res;
}
