////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// DiagramVertexInfo.cpp: implementation of the CDiagramVertexInfo class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

const char* CDiagramVertexInfo::CLASS_NAME = "CDiagramVertexInfo";
const int CDiagramVertexInfo::VERTEX_COUNTERTERM	= 0x01;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDiagramVertexInfo::CDiagramVertexInfo(int vertex_ID, long part_kind_1, long part_kind_2, long part_kind_3)
: CESerializable() {
	m_vertex_ID = vertex_ID;
	m_tales_num = 3;
	
	int long_siz = sizeof(long);
	m_tales_info = (long*)CMemoryManager::mmalloc(m_tales_num*long_siz);

	m_tales_info[0] = part_kind_1; m_tales_info[1] = part_kind_2; m_tales_info[2] = part_kind_3;
	m_vertex_type = 0;
}

CDiagramVertexInfo::CDiagramVertexInfo(int vertex_ID, long part_kind_1, long part_kind_2, long part_kind_3, long part_kind_4)
: CESerializable() {
	m_vertex_ID = vertex_ID;
	m_tales_num = 4;
	
	int long_siz = sizeof(long);
	m_tales_info = (long*)CMemoryManager::mmalloc(m_tales_num*long_siz);

	m_tales_info[0] = part_kind_1; m_tales_info[1] = part_kind_2; m_tales_info[2] = part_kind_3; m_tales_info[3] = part_kind_4;
	m_vertex_type = 0;
}

CDiagramVertexInfo::CDiagramVertexInfo(int vertex_ID, long part_kind_1, long part_kind_2)
: CESerializable() {
	m_vertex_ID = vertex_ID;
	m_tales_num = 2;
	
	int long_siz = sizeof(long);
	m_tales_info = (long*)CMemoryManager::mmalloc(m_tales_num*long_siz);

	m_tales_info[0] = part_kind_1; m_tales_info[1] = part_kind_2;
	m_vertex_type = 0;
}

CDiagramVertexInfo::CDiagramVertexInfo(int vertex_ID, int _tales_num, long* _tales_info, int _vert_type)
: CESerializable() {
	m_vertex_ID = vertex_ID;
	m_tales_num = _tales_num;

	int long_siz = sizeof(long);
	m_tales_info = (long*)CMemoryManager::mmalloc(m_tales_num*long_siz);
	//m_tales_info = new int[m_tales_num];
	int i;
	for(i=0; i<m_tales_num; i++) { m_tales_info[i] = _tales_info[i]; }

	m_vertex_type = _vert_type;
}

CDiagramVertexInfo* CDiagramVertexInfo::newCountertermVertex(int vertex_ID, long part_kind_1, long part_kind_2, long part_kind_3, long part_kind_4) {
	CDiagramVertexInfo* res = new CDiagramVertexInfo(vertex_ID, part_kind_1, part_kind_2, part_kind_3, part_kind_4);
	res->set_vert_type(VERTEX_COUNTERTERM);
	return res;
}

CDiagramVertexInfo* CDiagramVertexInfo::newCountertermVertex(int vertex_ID, long part_kind_1, long part_kind_2, long part_kind_3) {
	CDiagramVertexInfo* res = new CDiagramVertexInfo(vertex_ID, part_kind_1, part_kind_2, part_kind_3);
	res->set_vert_type(VERTEX_COUNTERTERM);
	return res;
}

CDiagramVertexInfo* CDiagramVertexInfo::newCountertermVertex(int vertex_ID, long part_kind_1, long part_kind_2) {
	CDiagramVertexInfo* res = new CDiagramVertexInfo(vertex_ID, part_kind_1, part_kind_2);
	res->set_vert_type(VERTEX_COUNTERTERM);
	return res;
}

CDiagramVertexInfo::~CDiagramVertexInfo()
{
	CMemoryManager::mfree(m_tales_info);
	//delete[] m_tales_info;
}

CEObject* CDiagramVertexInfo::copy() const { return clone(); }
CDiagramVertexInfo* CDiagramVertexInfo::clone() const {
	return new CDiagramVertexInfo(m_vertex_ID, m_tales_num, m_tales_info, m_vertex_type);
}

const char* CDiagramVertexInfo::CLASS_TAG_NAME = "diagVI";
const char* CDiagramVertexInfo::get_TAG_Name() { return CLASS_TAG_NAME; }

long CDiagramVertexInfo::store_stream(ofstream* p_ofs, DWORD settings) {
	long res = CESupport::storeXMLLong(m_tales_num, p_ofs, settings);
	for(int i=0; i<m_tales_num; i++) { res += CESupport::storeXMLLong(m_tales_info[i], p_ofs, settings); }
	res += CESupport::storeXMLLong(m_vertex_ID, p_ofs, settings);
	res += CESupport::storeXMLLong(m_vertex_type, p_ofs, settings);
	return res;
}

CESerializable* CDiagramVertexInfo::restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings) {
	long l = 0;
	bool read_OK = CESupport::readXMLLong(data, &pos, &l, settings, CLASS_NAME);
	if (!read_OK) return NULL;

	int _tales_num = l;
	int long_siz = sizeof(long);
	long* _tales_info = (long*)CMemoryManager::mmalloc(_tales_num*long_siz);

	for(long i=0; i<_tales_num; i++) {
		read_OK = CESupport::readXMLLong(data, &pos, &l, settings, CLASS_NAME);
		if (!read_OK) { CMemoryManager::mfree(_tales_info); return NULL; }
		_tales_info[i]=l;
	}

	read_OK = CESupport::readXMLLong(data, &pos, &l, settings, CLASS_NAME);
	if (!read_OK) { CMemoryManager::mfree(_tales_info); return NULL; }
	int _vertex_ID = l;

	read_OK = CESupport::readXMLLong(data, &pos, &l, settings, CLASS_NAME);
	if (!read_OK) { CMemoryManager::mfree(_tales_info); return NULL; }
	int _vertex_type = l;

	read_OK = CESupport::checkAllRead(data, &pos, settings, CLASS_NAME);
	if (!read_OK) { CMemoryManager::mfree(_tales_info); return NULL; }
	
	CDiagramVertexInfo* res = new CDiagramVertexInfo(_vertex_ID, _tales_num, _tales_info, _vertex_type);
	CMemoryManager::mfree(_tales_info);
	return res;
}

void CDiagramVertexInfo::set_vert_type(int _vert_type) { m_vertex_type = _vert_type; }
bool CDiagramVertexInfo::check_vert_type(int type_mask) const { return ((m_vertex_type & type_mask) != 0); }

long CDiagramVertexInfo::CalcPartsIdentityOrder() { // NNA->2, NNMM->4, NNN->6
	long res=1;
	long n=get_tales_num(); long i; long j;
	for(i=0; i<n; i++) {
		long pki=get_particle_kind(i);
		int pkiN=1;

		// 1. is this pk aready involved?
		bool bPkInvolved=false;
		for(j=0; j<i; j++) { if (pki==get_particle_kind(j)) { bPkInvolved=true; break;} }
		if (bPkInvolved) continue;

		// 2. calculate factor for new pk
		for(j=i+1; j<n; j++) { if (pki==get_particle_kind(j)) { pkiN++; } }
		if (pkiN>1) res*=CLMath::factorial(pkiN);
	}
	return res;
}

long CDiagramVertexInfo::CalcPartsIdentityOrder(long n, long* pks) { // call: long pks[3]; ... f=CalcPartsIdentityOrder(3,pks);
	if (pks==NULL) return 1;
	long res=1; long i; long j;
	for(i=0; i<n; i++) {
		long pki=pks[i]; int pkiN=1;

		// 1. is this pk already involved?
		bool bPkInvolved=false;
		for(j=0; j<i; j++) { if (pki==pks[j]) { bPkInvolved=true; break;} }
		if (bPkInvolved) continue;

		// 2. calculate factor for new pk
		for(j=i+1; j<n; j++) { if (pki==pks[j]) { pkiN++; } }
		if (pkiN>1) res*=CLMath::factorial(pkiN);
	}
	//printf("ORDER: %ld {%ld,%ld,%ld,%ld}, res: %ld \n",n,pks[0],pks[1],pks[2],pks[3],res);
	return res;
}

long CDiagramVertexInfo::CalcPartsIdentityOrder(long pk1, long pk2) { long pks[2]={pk1,pk2}; return CalcPartsIdentityOrder(2,pks); }
long CDiagramVertexInfo::CalcPartsIdentityOrder(long pk1, long pk2, long pk3) { long pks[3]={pk1,pk2,pk3}; return CalcPartsIdentityOrder(3,pks); }
long CDiagramVertexInfo::CalcPartsIdentityOrder(long pk1, long pk2, long pk3, long pk4) { long pks[4]={pk1,pk2,pk3,pk4}; return CalcPartsIdentityOrder(4,pks); }

/*
*	Used for Class vertices only,
*	vSpec: non-class vertex to meet this.
*	Note: straight order is MANDATORY (as vertices fitting check).
*	Do not call it for non-fitting vertices.
*	orders->size() should occur CalcPartsIdentityOrder()/vSpec->CalcPartsIdentityOrder();
*/
void CDiagramVertexInfo::CollectClassFitOrders(CDiagramVertexInfo* vSpec, CEPtrArray* orders) {
	if (orders==NULL) return;
	orders->clear_all();
	if (vSpec==NULL) return;
	long n=get_tales_num(); if(vSpec->get_tales_num()!=n)return;
	int i;

	// fit-check & fill straight order
	bool bFitsOk=true;
	CIntArray o;
	for(i=0; i<n; i++) {
		if (!CEParticle::FitClassPk(vSpec->get_particle_kind(i),get_particle_kind(i))) { bFitsOk=false; break; }
		o.Add(i);
	}
	if (!bFitsOk) { CEException::halt("CDiagramVertexInfo::CollectClassFitOrders(non-fitting vertex)"); return; }

	CollectClassFitOrders(vSpec,orders,&o,0);
}

void CDiagramVertexInfo::CollectClassFitOrders(CDiagramVertexInfo* vSpec, CEPtrArray* orders, CIntArray* o, int i_next) {
	long n=get_tales_num();
	if (i_next > n) { CEException::halt("CDiagramVertexInfo::CollectClassFitOrders internal error"); }
	if (i_next == n) {
		orders->add(new CIntArray(*o));
		return;
	}

	// 1. add straight order
	CollectClassFitOrders(vSpec, orders, o, i_next+1);

	// 2. try to exchange 'i' to any other class particle
	long pkClass=get_particle_kind(i_next);
	if (CEParticle::is_class_PK(pkClass)) {
		long pkSpec=vSpec->get_particle_kind(i_next);
		for(int i=i_next+1; i<n; i++) {
			long pkClass2=get_particle_kind(i); if(pkClass2!=pkClass)continue;
			long pkSpec2=vSpec->get_particle_kind(i); if(pkSpec2==pkSpec)continue;
			CIntArray oCpy(*o);
			oCpy.exchange(i_next,i);
			CollectClassFitOrders(vSpec, orders, &oCpy, i_next+1);
		}
	}
}

bool CDiagramVertexInfo::isEqualTo(CDiagramVertexInfo* other) {
	if (other == this) return true;
	if (other == NULL) return false;

	// consider single physics only
	return (get_vertex_ID() == other->get_vertex_ID());
	// elsewhere compare all the data
}

int CDiagramVertexInfo::get_tales_num() { return m_tales_num; }
int CDiagramVertexInfo::get_vertex_ID() { return m_vertex_ID; }

int CDiagramVertexInfo::get_vertex_order() {
	int res = (m_tales_num-2);
	if (check_vert_type(VERTEX_COUNTERTERM)) res+=2;
	if (res <= 0) {
		CEBasic::dbg->log(0, res, m_tales_num, "<CDiagramVertexInfo::get_vertex_order()> ERROR: vertex of order %ld (%ld tales) spotted", this); CEException::halt();
	}
	return res;
}

/*
*	note: tale_number is of [0, max-1]
*/
long CDiagramVertexInfo::get_particle_kind(int tale_number) const
{
	if ((tale_number < 0) || (tale_number >= m_tales_num)) { CEBasic::dbg->log(0, "CDiagramVertexInfo::get_particle_kind() invlid call"); return -1; }
	return m_tales_info[tale_number];
}

CEString* CDiagramVertexInfo::toString(int mode) {
	CEString* res = new CEString("<VertexInfo#"); res->add_int(m_vertex_ID);res->AddString(":");
	if (check_vert_type(VERTEX_COUNTERTERM)) res->AddString("<CT> ");
	int s = get_tales_num();
	int i;
	for(i=0; i<s; i++) {
		long pk = get_particle_kind(i);
		res->add_int(pk,true);
		res->AddString("(");
		if (pk<0) res->AddString("-");
		res->AddString(CDiagramPhysics::get_particle_symbolA(pk,(mode!=STORE_TEX)?CDiagramPhysics::SYMBOL_STYLE_NB:CDiagramPhysics::SYMBOL_STYLE_TEX));
		res->AddString(")");
		if ((i+1) < s) res->AddString(", ");
	}
	res->AddString(">");
	return res;
}

const char* CDiagramVertexInfo::ClassName() const { return CLASS_NAME; }
bool CDiagramVertexInfo::contain_vector(void * obj) {
	if (CESerializable::contain_vector(obj)) return true;
	if (m_tales_info == obj) return true;
	return false;
}
