////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// PhysModelFA.cpp
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

CPhysModelFA::CFAGenericCplPartInfo::CFAGenericCplPartInfo() : CEObject(true) {
	m_SignSym=NULL; // si
	m_MomSym=NULL; // momi
	m_PartNumber=NULL;
	m_bIsNeutral=false;
	//CEString m_PartTitle; // F,V
	//CSequence m_Indices; // Generation etc.
	//CSequence m_VecIndices; // without 'covariance' info
}

CPhysModelFA::CFAGenericCplPartInfo::CFAGenericCplPartInfo(class CFAGenericCplPartInfo& inf) : CEObject(true),
m_PartTitle(inf.m_PartTitle)
{
	m_SignSym=(inf.m_SignSym!=NULL)?inf.m_SignSym->clone():NULL;
	m_PartNumber=(inf.m_PartNumber!=NULL)?inf.m_PartNumber->clone():NULL;
	m_MomSym=(inf.m_MomSym!=NULL)?inf.m_MomSym->clone():NULL;
	m_VecIndices.copy_from(&inf.m_VecIndices);
	m_bIsNeutral=inf.m_bIsNeutral;
}

CPhysModelFA::CFAGenericCplPartInfo::~CFAGenericCplPartInfo() {
	if (m_SignSym!=NULL) delete m_SignSym;
	if (m_MomSym!=NULL) delete m_MomSym;
	if (m_PartNumber!=NULL) delete m_PartNumber;
}

bool CPhysModelFA::CFAGenericCplPartInfo::contain_vector(void * obj) {
	if (obj==NULL) return false;
	if ((obj == &m_PartTitle) || (m_PartTitle.contain_vector(obj))) return true;
	//if ((obj == &m_Indices) || (m_Indices.contain_vector(obj))) return true;
	if (m_PartNumber!=NULL) { if ((obj == m_PartNumber) || (m_PartNumber->contain_vector(obj))) return true; }
	if (m_SignSym!=NULL) { if ((obj == m_SignSym) || (m_SignSym->contain_vector(obj))) return true; }
	if (m_MomSym!=NULL) { if ((obj == m_MomSym) || (m_MomSym->contain_vector(obj))) return true; }
	if ((obj == &m_VecIndices) || (m_VecIndices.contain_vector(obj))) return true;
	return false;
}

bool CPhysModelFA::CFAGenericCplPartInfo::equals(CFAGenericCplPartInfo* other) {
	if (other==NULL) return false;
	if (!CEUtil::LEQU(m_bIsNeutral,other->m_bIsNeutral)) return false;
	if (!m_PartTitle.equals(&other->m_PartTitle)) return false;
	if (m_SignSym != NULL) {if(!m_SignSym->equals(other->m_SignSym))return false;} else {if(other->m_SignSym!=NULL)return false;}
	if (m_PartNumber != NULL) {if(!m_PartNumber->equals(other->m_PartNumber))return false;} else {if(other->m_PartNumber!=NULL)return false;}
	if (m_MomSym != NULL) {if(!m_MomSym->equals(other->m_MomSym))return false;} else {if(other->m_MomSym!=NULL)return false;}
	if (!m_VecIndices.equals(&other->m_VecIndices)) return false;
	return true;
}

const char* CPhysModelFA::CFAGenericCplPartInfo::CLASS_NAME = "CPhysModelFA::CFAGenericCplPartInfo";
const char* CPhysModelFA::CFAGenericCplPartInfo::ClassName() const { return CLASS_NAME; }
char* CPhysModelFA::CFAGenericCplPartInfo::toLine(int mode) {
	CEString res;
	res.AddString("{FA_GenCplPartInfo: ");
	if (m_SignSym!=NULL) { res.AddString(m_SignSym->toString(mode)); res.AddString(" "); }
	res.AddString(m_PartTitle.getData());
	if (m_bIsNeutral) res.AddString("0");
	res.AddString("[");
	//res.AddString(m_Indices.toString(mode));
	if (m_PartNumber!=NULL) res.AddString(m_PartNumber->toString(mode));
	res.AddString("]");
	res.AddString("-");
	if (m_MomSym !=NULL) { res.AddString(m_MomSym->toString(mode)); } else { res.AddString("NULL"); }
	if (m_VecIndices.size()>0) {
		res.AddString("-");
		res.AddString(m_VecIndices.toString(mode));
	}
	res.AddString("}");
	char* resS=CEUtil::copyString(res.getData());
	return resS;
}

CPhysModelFA::CFAModelGenericCpl::CFAModelGenericCpl() : CEObject(true) {
	//CSequence m_partInfos; // array of CSequence{si, Fi, inds, vec_inds}
	m_bMomsSymmetric=true;
	m_Value=NULL;
}

CPhysModelFA::CFAModelGenericCpl::CFAModelGenericCpl(class CFAModelGenericCpl& inf) : CEObject(true) {
	long i; long s = inf.m_partInfos.size();
	for (i=0; i<s; i++) {
		CFAGenericCplPartInfo* pInf=(CFAGenericCplPartInfo*)inf.m_partInfos.GetAt(i);
		if (pInf!=NULL) pInf=new CFAGenericCplPartInfo(*pInf);
		m_partInfos.add(pInf);
	}
	m_bMomsSymmetric=inf.m_bMomsSymmetric;
	m_Value=(inf.m_Value!=NULL)?inf.m_Value->clone():NULL;
}

CPhysModelFA::CFAModelGenericCpl::~CFAModelGenericCpl() {
	if (m_Value!=NULL) delete m_Value;
}

void CPhysModelFA::CFAModelGenericCpl::ExchangeParticles(int i,int j) {
	int s=m_partInfos.size();
	if((i<0) || (i>=s)) { CEException::halt("CPhysModelFA::CFAModelGenericCpl::ExchangeParticles() invalid call"); }
	if((j<0) || (j>=s)) { CEException::halt("CPhysModelFA::CFAModelGenericCpl::ExchangeParticles() invalid call"); }
	CEObject* oI=m_partInfos.GetAt(i);
	CEObject* oJ=m_partInfos.GetAt(j);
	m_partInfos.SetAt(i,oJ);
	m_partInfos.SetAt(j,oI);
}

bool CPhysModelFA::CFAModelGenericCpl::contain_vector(void * obj) {
	if (obj==NULL) return false;
	if ((obj == &m_partInfos) || (m_partInfos.contain_vector(obj))) return true;
	if (m_Value!=NULL) { if ((obj == m_Value) || (m_Value->contain_vector(obj))) return true; }
	return false;
}

bool CPhysModelFA::CFAModelGenericCpl::equals(CFAModelGenericCpl* other) {
	if (other==NULL) return false;
	long s=m_partInfos.size(); if(s!=other->m_partInfos.size()) return false;
	for (long i=0; i<s; i++) {
		CFAGenericCplPartInfo* inf=(CFAGenericCplPartInfo*)m_partInfos.GetAt(i);
		CFAGenericCplPartInfo* infO=(CFAGenericCplPartInfo*)other->m_partInfos.GetAt(i);
		if ((inf==NULL) && (infO==NULL)) continue;
		if ((inf==NULL) || (infO==NULL)) return false;
		if (!inf->equals(infO)) return false;
	}
	if (!CEUtil::LEQU(m_bMomsSymmetric,other->m_bMomsSymmetric)) return false;
	if (m_Value != NULL) {if(!m_Value->equals(other->m_Value))return false;} else {if(other->m_Value!=NULL)return false;}
	return true;
}

const char* CPhysModelFA::CFAModelGenericCpl::CLASS_NAME = "CPhysModelFA::CFAModelGenericCpl";
const char* CPhysModelFA::CFAModelGenericCpl::ClassName() const { return CLASS_NAME; }
char* CPhysModelFA::CFAModelGenericCpl::toLine(int mode) {
	CEString res;
	res.AddString("{FA_GenCpl: ");
	char* line=m_partInfos.toLine(mode); res.AddString(line); CMemoryManager::mfree(line);
	res.AddString("-");
	if (m_bMomsSymmetric) res.AddString("[+1]"); else res.AddString("[-1]");
	if (m_Value!=NULL) { res.AddString("="); res.AddString(m_Value->toString(mode)); }
	res.AddString("}");
	char* resS=CEUtil::copyString(res.getData());
	return resS;
}

CPhysModelFA::CFAPatricleClassInfo::CFAPatricleClassInfo() : CEObject(true) {
	m_SymMassOrig=NULL;
	m_SymMass=NULL;
	m_iPropagatorType=0; // 0 will call default selection procedure
	m_iPropagatorArrow=0;
	m_PropagatorLabel=NULL;
	m_PropagatorLabelM=NULL;
	m_SelfConjugated=false;
	m_PK=0;
}

CPhysModelFA::CFAPatricleClassInfo::CFAPatricleClassInfo(class CFAPatricleClassInfo& inf)
: CEObject(true),
m_Indices(inf.m_Indices),
m_idClassName(inf.m_idClassName),
m_idNumber(inf.m_idNumber),
m_sPropagatorLabelTex(inf.m_sPropagatorLabelTex),
m_sPropagatorLabelNb(inf.m_sPropagatorLabelNb)
//,m_sPropagatorLabel(inf.m_sPropagatorLabel)
{
	m_SymMassOrig=(inf.m_SymMassOrig!=NULL)?inf.m_SymMassOrig->clone():NULL;
	m_PropagatorLabel=(inf.m_PropagatorLabel!=NULL)?inf.m_PropagatorLabel->clone():NULL;
	m_PropagatorLabelM=(inf.m_PropagatorLabelM!=NULL)?inf.m_PropagatorLabelM->clone():NULL;
	m_QuantumNumbers.copy_from(&inf.m_QuantumNumbers);
	m_iPropagatorType=inf.m_iPropagatorType;
	m_iPropagatorArrow=inf.m_iPropagatorArrow;
	m_SelfConjugated=inf.m_SelfConjugated;

	m_PK=inf.m_PK;
	m_SymMass=(inf.m_SymMass!=NULL)?inf.m_SymMass->clone():NULL;
}

CPhysModelFA::CFAPatricleClassInfo::~CFAPatricleClassInfo() {
	if (m_SymMassOrig!=NULL) delete m_SymMassOrig;
	if (m_SymMass!=NULL) delete m_SymMass;
	if (m_PropagatorLabel != NULL) delete m_PropagatorLabel;
	if (m_PropagatorLabelM != NULL) delete m_PropagatorLabelM;
}

bool CPhysModelFA::CFAPatricleClassInfo::contain_vector(void * obj) {
	if (obj==NULL) return false;
	if ((obj == &m_idClassName) || (m_idClassName.contain_vector(obj))) return true;
	if ((obj == &m_idNumber) || (m_idNumber.contain_vector(obj))) return true;
	if (m_SymMassOrig!=NULL) { if ((obj == m_SymMassOrig) || (m_SymMassOrig->contain_vector(obj))) return true; }
	if ((obj == &m_Indices) || (m_Indices.contain_vector(obj))) return true;
	if ((obj == &m_Indices) || (m_Indices.contain_vector(obj))) return true;
	if ((obj == &m_QuantumNumbers) || (m_QuantumNumbers.contain_vector(obj))) return true;
	if (m_PropagatorLabel!=NULL) { if ((obj == m_PropagatorLabel) || (m_PropagatorLabel->contain_vector(obj))) return true; }
	if (m_PropagatorLabelM!=NULL) { if ((obj == m_PropagatorLabelM) || (m_PropagatorLabelM->contain_vector(obj))) return true; }
	if (m_SymMass!=NULL) { if ((obj == m_SymMass) || (m_SymMass->contain_vector(obj))) return true; }
	return false;
}

const char* CPhysModelFA::CFAPatricleClassInfo::CLASS_NAME = "CFAPatricleClassInfo";
const char* CPhysModelFA::CFAPatricleClassInfo::ClassName() const { return CLASS_NAME; }
char* CPhysModelFA::CFAPatricleClassInfo::toLine(int mode) {
	CEString res;
	res.AddString("{FA_ParticleClass: ");
	res.AddString(m_idClassName.getData());
	res.AddString(m_idNumber.getData());
	if(m_SelfConjugated) { res.AddString("+"); } else { res.AddString("-"); }
	if (m_SymMassOrig!=NULL) res.AddString(m_SymMassOrig->toString(mode)); else res.AddString("NULL");
	res.AddString("-");
	res.AddString(m_QuantumNumbers.toString(mode));
	res.AddString("-");
	char* inds_line = m_Indices.toLine(mode);
	if (inds_line!=NULL) { res.AddString(inds_line); CMemoryManager::mfree(inds_line); } else { res.AddString("NULL"); }
	res.AddString("-");
	res.add_int(m_iPropagatorType);
	res.AddString("-");
	res.add_int(m_iPropagatorType);
	res.AddString("-");
	if (mode == STORE_WR_MATHEMATICA) {
		if (m_PropagatorLabelM!=NULL) res.AddString(m_PropagatorLabelM->toString(mode));
	} else { //if (mode == STORE_TEX) {
		if (m_PropagatorLabel!=NULL) res.AddString(m_PropagatorLabel->toString(mode));
	}
	res.AddString("-");
	if (m_SymMass!=NULL) res.AddString(m_SymMass->toString(mode)); else res.AddString("NULL");
	res.AddString("-");
	char* pk_data = (char*)CMemoryManager::mmalloc(20); sprintf(pk_data, "x%lx", m_PK);
	res.AddString(pk_data);
	CMemoryManager::mfree(pk_data);
	res.AddString("}");
	char* resS=CEUtil::copyString(res.getData());
	return resS;
}

/*
bool CPhysModelFA::CFAPatricleClassRangeInfo::contain_vector(void * obj) {
	if (obj==NULL) return false;
	if ((obj == &m_idClassName) || (m_idClassName.contain_vector(obj))) return true;
	if ((obj == &m_idNumber) || (m_idNumber.contain_vector(obj))) return true;
	if ((obj == &m_IndexValues) || (m_IndexValues.contain_vector(obj))) return true;
	if ((obj == &m_IndexTitles) || (m_IndexTitles.contain_vector(obj))) return true;
	return false;
}

const char* CPhysModelFA::CFAPatricleClassRangeInfo::CLASS_NAME = "CFAPatricleClassRangeInfo";
const char* CPhysModelFA::CFAPatricleClassRangeInfo::ClassName() const { return CLASS_NAME; }
char* CPhysModelFA::CFAPatricleClassRangeInfo::toLine(int mode) {
	CEString res;
	res.AddString("{FA_PartClassRangeInfo: ");
	res.AddString(m_idClassName.getData());
	res.AddString(m_idNumber.getData());
	res.AddString(":");
	if (m_IndexValues.size()>0) {
		char* inds_line = m_IndexValues.toLine(mode);
		if (inds_line!=NULL) { res.AddString(inds_line); CMemoryManager::mfree(inds_line); } else { res.AddString("NULL"); }
	} else {
		res.AddString("{}");
	}
	if (m_IndexTitles.size()>0) {
		res.AddString(",");
		res.AddString(m_IndexTitles.toString(mode));
	}
	res.AddString("}");
	char* resS=CEUtil::copyString(res.getData());
	return resS;
}
*/

////////////////////////////////////////////////////////////////////// ************ CPhysModelFA class implementation
// CPhysModelFA class implementation
//////////////////////////////////////////////////////////////////////

CPhysModelFA::CPhysModelFA() : CDiagramPhysics()
{
	dbg->log(25, "FeynArts Model Physics creation.");
	m_IsInited=false;

	m_Hamiltonian_expr = NULL;
	m_eClassesHamiltonian = NULL;
	m_eClassesHamiltonianR = NULL;
	m_Renorm_Hamiltonian = NULL;
	m_HamiltonianReplRules.RemoveAll(); //CERulesList

	// Vector indices to be used in Hamiltonian
	m_HvInd1 = CESymbol::getUnique();
	m_HvInd2 = CESymbol::getUnique();
	m_HvInd3 = CESymbol::getUnique();
	m_HvInd4 = CESymbol::getUnique();

	m_SymUnknownMass=new CESymbol("MassUnknown");

	m_numExtraParticles=0;
	m_bClassVerticesComplete=true;
}

CPhysModelFA::~CPhysModelFA() {
	if (m_Hamiltonian_expr != NULL) delete m_Hamiltonian_expr;
	if (m_eClassesHamiltonian != NULL) delete m_eClassesHamiltonian;
	if (m_eClassesHamiltonianR != NULL) delete m_eClassesHamiltonianR;
	if (m_Renorm_Hamiltonian != NULL) delete m_Renorm_Hamiltonian;
	//if (m_HamiltonianReplsRule != NULL) delete m_HamiltonianReplsRule;

	if (m_HvInd1 != NULL) delete m_HvInd1;
	if (m_HvInd2 != NULL) delete m_HvInd2;
	if (m_HvInd3 != NULL) delete m_HvInd3;
	if (m_HvInd4 != NULL) delete m_HvInd4;

	if (m_SymUnknownMass!=NULL) delete m_SymUnknownMass;
	//long i; long s=m_VertexClasses.size();
	//for(i=0; i<s; i++) { CFAModelGenericCpl* inf = (CFAModelGenericCpl*)m_VertexClasses.GetAt(i); if (inf!=NULL) delete inf; }
	//m_VertexClasses.clearall_without_delete_objects();
}

const char* CPhysModelFA::CLASS_NAME = "CPhysModelFA";
const char* CPhysModelFA::ClassName() const { return CPhysModelFA::CLASS_NAME; }

long CPhysModelFA::CreateNewExtraParticleID() {
	m_numExtraParticles++;
	if (m_numExtraParticles > 0x0fff) { CEException::halt("INTERNAL ERROR. Maximum number of particles reached."); }
	return m_numExtraParticles;
}

CPhysModelFA::CFAPatricleClassInfo* CPhysModelFA::GetParticleClassInfo(const char* cName, const char* idNumber, CIntArray* _Indices, bool bAllowIncompleteIndices) {
	if ((cName==NULL) || (idNumber==NULL) || (_Indices==NULL)) return NULL;
	int s=m_ParticleInfos.size();
	CPhysModelFA::CFAPatricleClassInfo* res=NULL;
	for(int i=0; i<s; i++) {
		CEObject* infO=m_ParticleInfos.GetAt(i); if(infO==NULL)continue;
		if (infO->ClassName() != CPhysModelFA::CFAPatricleClassInfo::CLASS_NAME)continue;
		CPhysModelFA::CFAPatricleClassInfo* inf=(CPhysModelFA::CFAPatricleClassInfo*)infO;
		if (CEUtil::equals(inf->m_idClassName.getData(),cName) &&
			CEUtil::equals(inf->m_idNumber.getData(),idNumber)) {
			bool indsOK=false;
			if (bAllowIncompleteIndices) {
				indsOK=(inf->m_Indices.equalsCommonItems(_Indices));
			} else {
				indsOK=(inf->m_Indices.equals(_Indices));
			}
			if (indsOK) { res=inf; break; }
		}
	}
	return res;
}

CPhysModelFA::CFAPatricleClassInfo* CPhysModelFA::GetNextParticleClassInfo(const char* cName, const char* idNumber, int* prev)
{ // first *prev should be==-1
	if ((cName==NULL) || (idNumber==NULL) || (prev==NULL)) return NULL;
	int s=m_ParticleInfos.size();
	int i=(*prev+1); if (i<0) i=0;
	CPhysModelFA::CFAPatricleClassInfo* res=NULL;
	for(; i<s; i++) {
		CEObject* infO=m_ParticleInfos.GetAt(i); if(infO==NULL)continue;
		if (infO->ClassName() != CPhysModelFA::CFAPatricleClassInfo::CLASS_NAME)continue;
		CPhysModelFA::CFAPatricleClassInfo* inf=(CPhysModelFA::CFAPatricleClassInfo*)infO;
		if (CEUtil::equals(inf->m_idClassName.getData(),cName) &&
			CEUtil::equals(inf->m_idNumber.getData(),idNumber)) {
			*prev=i; res=inf; break;
		}
	}
	return res;
}

CPhysModelFA::CFAPatricleClassInfo* CPhysModelFA::FindParticleClassInfo(long pk) {
	int s=m_ParticleInfos.size();
	int i=0;
	CPhysModelFA::CFAPatricleClassInfo* res=NULL;
	for(; i<s; i++) {
		CEObject* infO=m_ParticleInfos.GetAt(i); if(infO==NULL)continue;
		if (infO->ClassName() != CPhysModelFA::CFAPatricleClassInfo::CLASS_NAME)continue;
		CPhysModelFA::CFAPatricleClassInfo* inf=(CPhysModelFA::CFAPatricleClassInfo*)infO;
		if (inf->m_PK==pk) { res=inf; break; }
	}
	return res;
}

CPhysModelFA::CFAPatricleClassInfo* CPhysModelFA::GetParticleClassInfo(long i) {
	if((i<0)||(i>=m_ParticleInfos.size()))return NULL;
	CEObject* infO=m_ParticleInfos.GetAt(i); if(infO==NULL)return NULL;
	if (infO->ClassName() != CPhysModelFA::CFAPatricleClassInfo::CLASS_NAME)return NULL;
	return (CPhysModelFA::CFAPatricleClassInfo*)infO;
}
long CPhysModelFA::GetParticleClassesNum() { return m_ParticleInfos.size(); }

void CPhysModelFA::init_physics() { // invoked before physics is installed
	// Note: all the model data should already be filled
	if (!m_IsInited) {
		dbg->log(0, "ERROR: Installing of non-initianized CPhysModelFA pgysics");
		CEException::halt();
	}

	// particles data init/clear
	CEProcessor::getEnvironment()->ClearMassCalcOrders();

	// register particle kinds
	int s=m_ParticleInfos.size(); int i;
	for(i=0; i<s; i++) {
		CFAPatricleClassInfo* inf = (CFAPatricleClassInfo*)m_ParticleInfos.GetAt(i); if(inf==NULL)continue;
		CESymbol par(inf->m_idNumber.getData());
		CEFunction pk_func(inf->m_idClassName.getData(),&par);
		CScriptProcessor::addPreDefinedConstant(&pk_func,inf->m_PK);
	}
}

CDiagCorrRules* CPhysModelFA::getCorrespondenceRules() { return &m_corr_rules; }

CExpression* CPhysModelFA::getHamiltonianExpr() { return m_Hamiltonian_expr; }
CExpression* CPhysModelFA::getClassesHamiltonianE() { return (m_bClassVerticesComplete?m_eClassesHamiltonian:NULL); }
CExpression* CPhysModelFA::getClassesHamiltonianRen() { return (m_bClassVerticesComplete?m_eClassesHamiltonianR:NULL); }
CExpression* CPhysModelFA::getRenormHamiltonian() { return m_Renorm_Hamiltonian; }
CERule* CPhysModelFA::getHReplRule() {
	dbg->log(0, "WARNING: CPhysModelFA::getHReplRule() call. Hamiltonian replacement is not implemented.");
	return &m_HamiltonianReplRules;
}
DWORD CPhysModelFA::getPhysicsID() {
	// ToDo: make some Hash-code of data here.
	return 0;
}

bool CPhysModelFA::CounterTermsDeclared() { // called by CMEComposeLib
	if (m_Renorm_Hamiltonian == NULL) return false;
	return true;
}

const char* CPhysModelFA::get_particle_symbol(long _kind, int sym_style) {
	long pk_abs = CLMath::abs(_kind);
	CPhysModelFA::CFAPatricleClassInfo* inf = NULL;
	if ((pk_abs & CEParticle::PARTICLE_CLASS_FLAG) == 0) inf = FindParticleClassInfo(pk_abs);
	const char* res = NULL;
	if (inf!=NULL) {
		if (sym_style == CDiagramPhysics::SYMBOL_STYLE_TEX) {
			res=inf->m_sPropagatorLabelTex.getData();
		} else if (sym_style == CDiagramPhysics::SYMBOL_STYLE_NB) {
			res=inf->m_sPropagatorLabelNb.getData();
		} else {
			dbg->log(0, sym_style, "get_particle_symbol() invalid style parameter: %ld"); CEException::halt();
		}
	} else {
		res=GetClassPKTitle(pk_abs);
		//res="Null";
	}
	return res;
}

bool CPhysModelFA::validate_part_kind(long _kind) { // is it particle PK (or flag)
	long pk_abs = CLMath::abs(_kind);
	CPhysModelFA::CFAPatricleClassInfo* inf = FindParticleClassInfo(pk_abs);
	bool res = (inf != NULL);
	return res;
}

CExpression* CPhysModelFA::get_mass_sym(long pk) { // returns not cloned value
	long pk_abs = (pk>=0)?pk:(-pk);
	CPhysModelFA::CFAPatricleClassInfo* inf = FindParticleClassInfo(pk_abs);
	CExpression* res = NULL;
	if (inf!=NULL) {
		res=inf->m_SymMass;
	} else {
		dbg->log(0,pk,"Warning: CPhysModelFA::get_mass_sym(%x)->MassUnknown");
		res=m_SymUnknownMass;
	}
	if (res==NULL) CEException::halt("CPhysModelFA::get_mass_sym() == NULL");
	return res;
}

void CPhysModelFA::set_mass_sym(long pk, CExpression* val) {
	long pk_abs = (pk>=0)?pk:(-pk);
	CPhysModelFA::CFAPatricleClassInfo* inf = FindParticleClassInfo(pk_abs);
	if (inf!=NULL) {
		CExpression* mOld=inf->m_SymMass;
		if ((mOld != val) && (val != NULL)) {
			inf->m_SymMass=val->clone();
			delete mOld;
		} else {
			if (val != NULL) {
				dbg->log(0, pk, "WARNING: set_mass_sym(pk: %lx, value: NULL). Attempt of NULL mass value setting. Ignored.");
			}
		}
	} else {
		dbg->log(0, pk, "WARNING: set_mass_sym(pk: %lx). No particle class info found");
	}
}

int CPhysModelFA::getCharge3(long pk) { // charge * 3 (INT_MIN for unknown)
	long pk_abs = (pk>=0)?pk:(-pk);
	CPhysModelFA::CFAPatricleClassInfo* inf = FindParticleClassInfo(pk_abs);
	if (inf==NULL) return INT_MIN;
	int iCharge3=0;
	CESymbol tmpChargeSym("Charge"); // temporary
	for (long i=0;i<inf->m_QuantumNumbers.size();i++) {
		CExpression* qnItem=inf->m_QuantumNumbers.GetAt(i); if(qnItem==NULL)continue;
		if (qnItem->ClassName() == CESymbol::CLASS_NAME) {
			const char* qnInd=((CESymbol*)qnItem)->get_name();
			if (CEUtil::equals(qnInd,"LeptonNumber")) continue;
			if (CEUtil::equals(qnInd,"GhostNumber")) continue;
		}
		// charge * 3
		CExpression* tmpCharge=CExpression::reduce(qnItem->clone()->divide(tmpChargeSym.clone())->multiply(CENumber::GetThree()));
		if (tmpCharge->ClassName() == CENumber::CLASS_NAME) {
			iCharge3=((CENumber*)tmpCharge)->getLong();
			delete tmpCharge;
			break;
		} else {
			delete tmpCharge;
		}
	}
	if ((pk < 0) && (iCharge3 != INT_MIN)) iCharge3=-iCharge3;
	return iCharge3;
}

int CPhysModelFA::getDiagramLineType(long pk_abs) { // res==0 means default selection procedure
	CPhysModelFA::CFAPatricleClassInfo* inf = FindParticleClassInfo(pk_abs);
	int res = 0;
	if(inf!=NULL) {
		res=inf->m_iPropagatorType;
		if (res!=0) { if (inf->m_iPropagatorArrow <= 0) res|=CLDrawDiagView::LINE_NO_ARROW; }
	}
	return res;
}

// returns \alpha*4*pi/e^2 = 1
//CExpression* CSMPhysics::getModelContsUnitFactor() {
//	CExpression* pi = CEBasic::GetPiSym(); // not cloned
//	CExpression* res = getModelContsSym(CSMPhysics::CONST_ALPHA)->clone();
//	res = res->multiply(CENumber::GetFour())->multiply(pi->clone());
//	res = res->divide(getModelContsSym(CSMPhysics::CONST_E)->sqra());
//	return res;
//}

CERule* CPhysModelFA::getRenormConstValues() { return NULL; }
CSequence* CPhysModelFA::CollectConstants() { return NULL; }
CEPtrArray* CPhysModelFA::CollectConstantValues()  { return NULL; } // array of CExprContainer(), cloned value

long CPhysModelFA::InvolveVertexClass(CFAModelGenericCpl* inf, bool del_inf) {
	if (inf==NULL) return -1;
	long id=-1;
	long i; long s=m_VertexClasses.size();
	for(i=0; i<s; i++) {
		CFAModelGenericCpl* infI = (CFAModelGenericCpl*)m_VertexClasses.GetAt(i); if(infI==NULL)continue;
		if (infI->equals(inf)) { id=i; break; }
	}
	if (id>=0) {
		if (del_inf) delete inf;
	} else {
		if (!del_inf) inf= new CFAModelGenericCpl(*inf);
		m_VertexClasses.add(inf);
		id=m_VertexClasses.size()-1;
	}
	return id;
}

long CPhysModelFA::CollectVerticesForClass(long vnumClassVertex, CIntArray* idAltVerts) {
	if ((vnumClassVertex<0) || (idAltVerts==NULL)) return 0;
	CEPtrArray* arr_VIs=getVerticesInfo(); if (arr_VIs==NULL) return 0;
	long i=0; long idClassVertex=-1;
	long s=m_idClassVertices.GetSize(); if(s!=arr_VIs->size()) CEException::halt("Internal ERROR at CPhysModelFA::CollectVerticesForClass()");
	if (vnumClassVertex>=s) { dbg->log(0,vnumClassVertex,s,"ERROR: CPhysModelFA::CollectVerticesForClass(%ld,..) invalid call. max: %ld"); return 0; }

	CDiagramVertexInfo* viClassV=(CDiagramVertexInfo*)arr_VIs->GetAt(vnumClassVertex); if(viClassV==NULL) { dbg->log(0,"WARNING: NULL record at getVerticesInfo()"); return 0; }
	bool isCT=viClassV->check_vert_type(CDiagramVertexInfo::VERTEX_COUNTERTERM);

	// 1. find class-info object position (at m_VertexClasses array)
	idClassVertex=m_idClassVertices.GetAt(vnumClassVertex);
	if (idClassVertex<0) { // Note: next lines replace previous if m_idClassVertices.GetAt(class_vertex)==-1
		s=m_vnumClassVertices.GetSize(); for(i=0;i<s;i++) { if (m_vnumClassVertices.GetAt(i)==vnumClassVertex) { idClassVertex=i; break; } }
		if (idClassVertex<0) {
			s=m_vnCTClassVertices.GetSize();  for(i=0;i<s;i++) { if (m_vnCTClassVertices.GetAt(i)==vnumClassVertex) { idClassVertex=i; isCT=true; break; } }
		}
	}

	long res=0;
	if (idClassVertex>=0) {
		for(i=0;i<s;i++) {
			if (i>=arr_VIs->size()) { dbg->log(0,s,arr_VIs->size(),"WARNING: m_idClassVertices does not meet to getVerticesInfo() (%ld,%ld). It may lead to incomplete set of diagrams."); break; }
			if ((m_idClassVertices.GetAt(i)==idClassVertex) && (i != vnumClassVertex)) {
				CDiagramVertexInfo* vi=(CDiagramVertexInfo*)arr_VIs->GetAt(i); if(vi==NULL) { dbg->log(0,"WARNING: NULL record at getVerticesInfo()"); continue; }
				bool isViCT=vi->check_vert_type(CDiagramVertexInfo::VERTEX_COUNTERTERM);
				if (CEUtil::LEQU(isCT,isViCT)) { idAltVerts->Add(i); res++; }
			}
		}
	}
	return res;
}

bool CPhysModelFA::GetClassVertexID(long vnum, long* vnumClass, long* vnumClassCT) { // returns false of no class vertex are found
	if ((vnum<0) || (vnum>=m_idClassVertices.GetSize())) return false;
	long idCl=m_idClassVertices.GetAt(vnum);
	if (idCl<0) return false;
	bool res=true;
	long vnumC=m_vnumClassVertices.GetAt(idCl);
	if ((vnumC>=0) && (vnumClass != NULL)) { *vnumClass=vnumC; res=true; }
	long vnumC_CT=m_vnCTClassVertices.GetAt(idCl);
	if ((vnumC_CT>=0) && (vnumClassCT != NULL)) { *vnumClassCT=vnumC_CT; res=true; }
	return res;
}

// pk3,pk4 may be==0
// vIndX may be NULL
// vCorr,vCorrCT==NULL mean 0-valued vertex
// inf_const is not deleted, but may be changed (particles replaced, pk changed)
bool CPhysModelFA::addVertexCorr(long pk1, long pk2, long pk3, long pk4,
								 CExpression* vCorr, CExpression* vCorrCT,
								 CExpression* pPart1, CExpression* vInd1, CExpression* pPart2, CExpression* vInd2,
								 CExpression* pPart3, CExpression* vInd3, CExpression* pPart4, CExpression* vInd4,
								 CFAModelGenericCpl* inf_const,
								 long nVClassNo) {
	m_LastErrorMsg.Reset();
	if (nVClassNo>=0) {
		m_vnumClassVertices.SetAt(nVClassNo,-1); // before any return
		m_vnCTClassVertices.SetAt(nVClassNo,-1); // before any return
	}
	CEPtrArray* vert_info_list = getVerticesInfo(); if (vert_info_list==NULL) { m_LastErrorMsg.AddString("INTERNAL ERROR: getVerticesInfo()==NULL"); return false; }
	if ((m_HvInd1==NULL)||(m_HvInd2==NULL)||(m_HvInd3==NULL)||(m_HvInd4==NULL)) { m_LastErrorMsg.AddString("Initialization error"); return false; }
	int i;

	int partN=0; // number of particles in vertex
	if(pk1!=0){ partN=1; if(pk2!=0){ partN=2; if(pk3!=0){ partN=3; if(pk4!=0){partN=4;} } } }
	if (partN<2) { m_LastErrorMsg.AddString("Vertex with "); m_LastErrorMsg.add_int(partN); m_LastErrorMsg.AddString(" fields creation. Skipped."); return false; }

	// clear pk greather than partN (superfluous opreation)
	for(i=partN+1; i<=4; i++) { if(i==1){pk1=0;vInd1=NULL;} if(i==2){pk2=0;vInd2=NULL;} if(i==3){pk3=0;vInd3=NULL;} if(i==4){pk4=0;vInd4=NULL;} }

	bool bIsClassVertex=false;
	for(i=1; i<=partN; i++) {
		int pki=0; if(i==1){pki=pk1;} else if(i==2){pki=pk2;} else if(i==3){pki=pk3;} else if(i==4){pki=pk4;}
		if (pki==0) { m_LastErrorMsg.AddString("Internl ERROR: pk["); m_LastErrorMsg.add_int(i); m_LastErrorMsg.AddString("]==0"); return false; }
		long pki_abs=((pki>=0)?pki:-pki);
		if ((pki_abs & CEParticle::PARTICLE_CLASS_FLAG)==CEParticle::PARTICLE_CLASS_FLAG) {
			if (!bIsClassVertex) {
				if (i>1) { m_LastErrorMsg.AddString("CLASS and NON-CLASS particle mix at vertex"); return false; }
				bIsClassVertex=true;
			}
		} else {
			if (bIsClassVertex) {
				if (i>1) {
					m_LastErrorMsg.AddString("CLASS and NON-CLASS particle mix at vertex");
					char* tmp1=(char*)CMemoryManager::mmalloc(100);
					if (i>=4) { sprintf(tmp1,"<%lx %lx %lx %lx>",pk1,pk2,pk3,pk4); }
					else if (i>=3) { sprintf(tmp1,"<%lx %lx %lx>",pk1,pk2,pk3); }
					else if (i>=2) { sprintf(tmp1,"<%lx %lx>",pk1,pk2); }
					else if (i>=1) { sprintf(tmp1,"<%lx>",pk1); }
					else sprintf(tmp1,"<>");
					m_LastErrorMsg.AddString(tmp1);
					CMemoryManager::mfree(tmp1);
					return false;
				}
				bIsClassVertex=false;
			}
		}
	}

	if ((!bIsClassVertex) && (nVClassNo>=0)) {
		m_LastErrorMsg.AddString("(Internal error) nVClassNo>=0 at !bIsClassVertex"); return false;
	}
	if ((bIsClassVertex) && (nVClassNo<0)) {
		m_LastErrorMsg.AddString("(Internal error) nVClassNo<0 at bIsClassVertex"); return false;
	}

	bool bValueOK=(vCorr!=NULL); if ((bValueOK)&&(CEBasic::isZero(vCorr)))bValueOK=false;
	//if ((bIsClassVertex) && (!bValueOK)) { bValueOK=true; vCorr=NULL; }
	bool bValCtOK=(vCorrCT!=NULL); if ((bValCtOK)&&(CEBasic::isZero(vCorrCT)))bValCtOK=false;
	//if ((bIsClassVertex) && (!bValCtOK)) { bValCtOK=true; vCorrCT=NULL; }
	if ((!bValCtOK) && (!bValueOK)) return true; // Nothing to add
	
	long v_num=-1; if (bValueOK) v_num=vert_info_list->size();
	long v_nCT=-1; if (bValCtOK) { v_nCT=((bValueOK)?(v_num+1):vert_info_list->size()); }
	
	// set 'NO CLASS' by default
	if (v_num>=0) m_idClassVertices.SetAt(v_num,-1);
	if (v_nCT>=0) m_idClassVertices.SetAt(v_nCT,-1);

	// Check particles order
	// It is required (similar to CSMPhysics): Frst fermion PK should be < 0, second > 0.
	bool bIsF1=CEParticle::is_fermion_PK(pk1); bool bIsF2=CEParticle::is_fermion_PK(pk2);
	bool bIsF3=CEParticle::is_fermion_PK(pk3); bool bIsF4=CEParticle::is_fermion_PK(pk4);
	int nFermions=0; if(bIsF1)nFermions++; if(bIsF2)nFermions++; if(bIsF3)nFermions++; if(bIsF4)nFermions++;
	if ((nFermions!=0) && (nFermions!=2)) { m_LastErrorMsg.AddString("coupling with "); m_LastErrorMsg.add_int(nFermions); m_LastErrorMsg.AddString(" fermions is not allowed"); return false; }
	if (nFermions==2) {
		if (!bIsF1) {
			if (bIsF2) {
				// 1 <-> 2
				int pkX=pk1;pk1=pk2;pk2=pkX;CExpression* vIndX=vInd1;vInd1=vInd2;vInd2=vIndX;bIsF1=CEParticle::is_fermion_PK(pk1);bIsF2=CEParticle::is_fermion_PK(pk2);
				if (inf_const!=NULL) inf_const->ExchangeParticles(0,1);
			} else if (bIsF3) {
				// 1 <-> 3
				int pkX=pk1;pk1=pk3;pk3=pkX;CExpression* vIndX=vInd1;vInd1=vInd3;vInd3=vIndX;bIsF1=CEParticle::is_fermion_PK(pk1);bIsF3=CEParticle::is_fermion_PK(pk3);
				if (inf_const!=NULL) inf_const->ExchangeParticles(0,2);
			} else if (bIsF4) {
				// 1 <-> 4
				int pkX=pk1;pk1=pk4;pk4=pkX;CExpression* vIndX=vInd1;vInd1=vInd4;vInd4=vIndX;bIsF1=CEParticle::is_fermion_PK(pk1);bIsF4=CEParticle::is_fermion_PK(pk4);
				if (inf_const!=NULL) inf_const->ExchangeParticles(0,3);
			}
		}

		if (!bIsF2) {
			if (bIsF3) {
				// 2 <-> 3
				int pkX=pk2; pk2=pk3; pk3=pkX; CExpression* vIndX=vInd2; vInd2=vInd3; vInd3=vIndX; bIsF2=CEParticle::is_fermion_PK(pk2); bIsF3=CEParticle::is_fermion_PK(pk3);
				if (inf_const!=NULL) inf_const->ExchangeParticles(1,2);
			} else if (bIsF4) {
				// 2 <-> 4
				int pkX=pk2; pk2=pk4; pk4=pkX; CExpression* vIndX=vInd2; vInd2=vInd4; vInd4=vIndX; bIsF2=CEParticle::is_fermion_PK(pk2); bIsF4=CEParticle::is_fermion_PK(pk4);
				if (inf_const!=NULL) inf_const->ExchangeParticles(1,3);
			}
		}
		
		if ((pk1>=0) && (pk2>=0)) {
			//if (bIsClassVertex) { pk1=-pk1; /* "fix" it */ } else {
			m_LastErrorMsg.AddString("Non-allowed fermions set order in coupling: +F[], +F[]"); return false;
			//}
		}
		if ((pk1<0) && (pk2<0)) {
			//if (bIsClassVertex) { pk2=-pk2; /* "fix" it */  } else {
			m_LastErrorMsg.AddString("Non-allowed fermions set order in coupling: -F[], -F[]"); return false;
			//}
		}
		if ((pk1>=0) && (pk2<0)) {
			// 1 <-> 2
			int pkX=pk1;pk1=pk2;pk2=pkX;CExpression* vIndX=vInd1;vInd1=vInd2;vInd2=vIndX;bIsF1=CEParticle::is_fermion_PK(pk1);bIsF2=CEParticle::is_fermion_PK(pk2);
			if (inf_const!=NULL) inf_const->ExchangeParticles(0,1);
		}
	}

	/*
	*	Neutral particles mark
	*/
	if (inf_const!=NULL) {
		for(i=1; i<=partN; i++) {
			long pki=0; if(i==1){pki=pk1;} else if(i==2){pki=pk2;} else if(i==3){pki=pk3;} else if(i==4){pki=pk4;}
			long pkAbs=(pki>0)?pki:-pki;
			CFAGenericCplPartInfo* pInf=(CFAGenericCplPartInfo*)inf_const->m_partInfos.GetAt(i-1);
			if (pInf==NULL) CEException::halt("CPhysModelFA::addVertexCorr: inf_const->m_partInfos.GetAt()==NULL");
			if ((pkAbs & (long)CEParticle::PARTICLE_NEUTRAL_BOSON) == (long)CEParticle::PARTICLE_NEUTRAL_BOSON) {
				pInf->m_bIsNeutral=true;
			} else {
				pInf->m_bIsNeutral=false;
			}
		}
	}

	/*
	*	Select momenta usage mode
	*	Exclude non-used momenta
	*/
	bool mom_vec_used=false; bool all_mom_used=false;
	bool pP1_used=false; bool pP2_used=false; bool pP3_used=false; bool pP4_used=false;
	if (vCorr!=NULL) {
		if ((partN>=1)&&(!pP1_used)&&(pPart1!=NULL)) pP1_used=CRContainCheck::search(vCorr,pPart1);
		if ((partN>=2)&&(!pP2_used)&&(pPart2!=NULL)) pP2_used=CRContainCheck::search(vCorr,pPart2);
		if ((partN>=3)&&(!pP3_used)&&(pPart3!=NULL)) pP3_used=CRContainCheck::search(vCorr,pPart3);
		if ((partN>=4)&&(!pP4_used)&&(pPart4!=NULL)) pP4_used=CRContainCheck::search(vCorr,pPart4);
	}
	if (vCorrCT!=NULL) {
		if ((partN>=1)&&(!pP1_used)&&(pPart1!=NULL)) pP1_used=CRContainCheck::search(vCorrCT,pPart1);
		if ((partN>=2)&&(!pP2_used)&&(pPart2!=NULL)) pP2_used=CRContainCheck::search(vCorrCT,pPart2);
		if ((partN>=3)&&(!pP3_used)&&(pPart3!=NULL)) pP3_used=CRContainCheck::search(vCorrCT,pPart3);
		if ((partN>=4)&&(!pP4_used)&&(pPart4!=NULL)) pP4_used=CRContainCheck::search(vCorrCT,pPart4);
	}
	//if ((!pP1_used)&&(!pP2_used)&&(!pP3_used)&&(!pP4_used)) { pPart1=NULL; pPart2=NULL; pPart3=NULL; pPart4=NULL; }
	if (pP1_used || pP2_used || pP3_used || pP4_used) {
		mom_vec_used=true;
		if (((partN>=1)&&(pPart1==NULL)&&(vInd1!=NULL)) ||
			((partN>=2)&&(pPart2==NULL)&&(vInd2!=NULL)) ||
			((partN>=3)&&(pPart3==NULL)&&(vInd3!=NULL)) ||
			((partN>=4)&&(pPart4==NULL)&&(vInd4!=NULL))) {
			m_LastErrorMsg.AddString("Internal ERROR: Non-NULL momenta required at vector particle"); return false;
		}
	}
	if ( ((pP1_used)&&(vInd1==NULL)) || ((pP2_used)&&(vInd2==NULL)) || ((pP3_used)&&(vInd3==NULL)) || ((pP4_used)&&(vInd4==NULL)) ) {
		all_mom_used=true;
		if (((partN>=1)&&(pPart1==NULL)) || ((partN>=2)&&(pPart2==NULL)) ||
			((partN>=3)&&(pPart3==NULL)) || ((partN>=4)&&(pPart4==NULL))) {
			m_LastErrorMsg.AddString("Internal ERROR: Non-NULL momenta combined to NULLs"); return false;
		}
	}

	// Now pk: [-F,F,Any...]
	// Now index order are correspondent: vIndI (for corresp. value) <-> m_HvIndI (for H)

	// Hamiltonian indices
	CExpression* viH1=((vInd1!=NULL)?m_HvInd1:NULL);
	CExpression* viH2=((vInd2!=NULL)?m_HvInd2:NULL);
	CExpression* viH3=((vInd3!=NULL)?m_HvInd3:NULL);
	CExpression* viH4=((vInd4!=NULL)?m_HvInd4:NULL);
	CSequence listViH; // Hamiltonian vector indices without NULLs
	CIntArray listViHPKs; // PK corresponding to listViH
	//CSequence listVMoms; // Momenta symbols corresponding to vector particles (same order to listViH). USED if (mom_vec_used && !all_mom_used)
	if(viH1!=NULL) { listViH.addItem(viH1->clone()); listViHPKs.Add(pk1); }
	if(viH2!=NULL) { listViH.addItem(viH2->clone()); listViHPKs.Add(pk2); }
	if(viH3!=NULL) { listViH.addItem(viH3->clone()); listViHPKs.Add(pk3); }
	if(viH4!=NULL) { listViH.addItem(viH4->clone()); listViHPKs.Add(pk4); }
	int nViH=listViH.size();

	bool vIndCov=true;

	CExpression* ham_val = NULL;
	CExpression* ham_vCT = NULL;

	// nFermions: allowed 0 or 2
	switch(partN) {
	case 2: // 2-lines vertex
		if (bValueOK) {
			CESymbol* ham_tmp = CESymbol::getUnique(nFermions==0);
			// _VertHCore is a "gamma_{\mu}"-like expression.
			// NULL _VertHCore means UNIT factor (no vector indices, all particles are SCALAR)
			CExpression* _VertHCore = NULL;
			if (nViH==2) {
				_VertHCore = new CEMatrix(ham_tmp, listViH.GetAt(0), listViH.GetAt(1), !vIndCov, !vIndCov,
					((listViHPKs.GetAt(0)==listViHPKs.GetAt(1))?1:0));
			} else if (nViH==1) {
				_VertHCore = new CEVector(ham_tmp, listViH.GetAt(0), !vIndCov); 
			} // else NULL {UNIT factor (no vector indices, all particles are SCALAR)}
			delete ham_tmp;

			ham_val = (bIsF1?((CExpression*)(new COpFermionField(pk1, v_num))):((CExpression*)(new COpBosonField(pk1, viH1, vIndCov, v_num))));
			if (_VertHCore!=NULL) ham_val = ham_val->multiply(_VertHCore);
			ham_val = ham_val->multiply((bIsF2?((CExpression*)(new COpFermionField(pk2, v_num))):((CExpression*)(new COpBosonField(pk2, viH2, vIndCov, v_num)))));
		}
		if (bValCtOK) {
			CESymbol* ham_tmp = CESymbol::getUnique(nFermions==0);
			CExpression* _VertHCore = NULL;
			if (nViH==2) {
				_VertHCore = new CEMatrix(ham_tmp, listViH.GetAt(0), listViH.GetAt(1), !vIndCov, !vIndCov,
					((listViHPKs.GetAt(0)==listViHPKs.GetAt(1))?1:0));
			} else if (nViH==1) {
				_VertHCore = new CEVector(ham_tmp, listViH.GetAt(0), !vIndCov); 
			} // else NULL {UNIT factor (no vector indices, all particles are SCALAR)}
			delete ham_tmp;

			ham_vCT = (bIsF1?((CExpression*)(new COpFermionField(pk1, v_nCT))):((CExpression*)(new COpBosonField(pk1, viH1, vIndCov, v_nCT))));
			if (_VertHCore!=NULL) ham_vCT = ham_vCT->multiply(_VertHCore);
			ham_vCT = ham_vCT->multiply((bIsF2?((CExpression*)(new COpFermionField(pk2, v_nCT))):((CExpression*)(new COpBosonField(pk2, viH2, vIndCov, v_nCT)))));
		}
		break;
	case 3: // 3-lines vertex
		if (bValueOK) {
			CESymbol* ham_tmp = CESymbol::getUnique(nFermions==0);
			CExpression* _VertHCore = NULL;
			if (nViH==3) {
				_VertHCore = new CERank3Tensor(ham_tmp,listViH.GetAt(0),!vIndCov,listViH.GetAt(1),!vIndCov,listViH.GetAt(2),!vIndCov);
			} else if (nViH==2) {
				_VertHCore = new CEMatrix(ham_tmp, listViH.GetAt(0), listViH.GetAt(1), !vIndCov, !vIndCov,
					((listViHPKs.GetAt(0)==listViHPKs.GetAt(1))?1:0));
			} else if (nViH==1) {
				_VertHCore = new CEVector(ham_tmp, listViH.GetAt(0), !vIndCov); 
			} // else NULL {UNIT factor (no vector indices, all particles are SCALAR)}
			delete ham_tmp;

			ham_val = ((bIsF1)?((CExpression*)(new COpFermionField(pk1, v_num))):((CExpression*)(new COpBosonField(pk1, viH1, vIndCov, v_num))));
			if (_VertHCore!=NULL) ham_val = ham_val->multiply(_VertHCore);
			ham_val = ham_val->multiply(((bIsF2)?((CExpression*)(new COpFermionField(pk2, v_num))):((CExpression*)(new COpBosonField(pk2, viH2, vIndCov, v_num)))));
			ham_val = ham_val->multiply(((bIsF3)?((CExpression*)(new COpFermionField(pk3, v_num))):((CExpression*)(new COpBosonField(pk3, viH3, vIndCov, v_num)))));
		}
		if (bValCtOK) {
			CESymbol* ham_tmp = CESymbol::getUnique(nFermions==0);
			CExpression* _VertHCore = NULL;
			if (nViH==3) {
				_VertHCore = new CERank3Tensor(ham_tmp,listViH.GetAt(0),!vIndCov,listViH.GetAt(1),!vIndCov,listViH.GetAt(2),!vIndCov);
			} else if (nViH==2) {
				_VertHCore = new CEMatrix(ham_tmp, listViH.GetAt(0), listViH.GetAt(1), !vIndCov, !vIndCov,
					((listViHPKs.GetAt(0)==listViHPKs.GetAt(1))?1:0));
			} else if (nViH==1) {
				_VertHCore = new CEVector(ham_tmp, listViH.GetAt(0), !vIndCov); 
			} // else NULL {UNIT factor (no vector indices, all particles are SCALAR)}
			delete ham_tmp;

			ham_vCT = ((bIsF1)?((CExpression*)(new COpFermionField(pk1, v_nCT))):((CExpression*)(new COpBosonField(pk1, viH1, vIndCov, v_nCT))));
			if (_VertHCore !=NULL) ham_vCT = ham_vCT->multiply(_VertHCore);
			ham_vCT = ham_vCT->multiply(((bIsF2)?((CExpression*)(new COpFermionField(pk2, v_nCT))):((CExpression*)(new COpBosonField(pk2, viH2, vIndCov, v_nCT)))));
			ham_vCT = ham_vCT->multiply(((bIsF3)?((CExpression*)(new COpFermionField(pk3, v_nCT))):((CExpression*)(new COpBosonField(pk3, viH3, vIndCov, v_nCT)))));
		}
		break;
	case 4: // 4-lines vertex
		if (bValueOK) {
			CESymbol* ham_tmp = CESymbol::getUnique(nFermions==0);
			CExpression* _VertHCore = NULL;
			if (nViH==4) {
				_VertHCore = new CERank4Tensor(ham_tmp,
					listViH.GetAt(0),!vIndCov,listViH.GetAt(1),!vIndCov,listViH.GetAt(2),!vIndCov,listViH.GetAt(3),!vIndCov,
					((listViHPKs.GetAt(0)==listViHPKs.GetAt(1))?1:0),((listViHPKs.GetAt(2)==listViHPKs.GetAt(3))?1:0));
			} if (nViH==3) {
				_VertHCore = new CERank3Tensor(ham_tmp,listViH.GetAt(0),!vIndCov,listViH.GetAt(1),!vIndCov,listViH.GetAt(2),!vIndCov);
			} else if (nViH==2) {
				_VertHCore = new CEMatrix(ham_tmp, listViH.GetAt(0), listViH.GetAt(1), !vIndCov, !vIndCov,
					((listViHPKs.GetAt(0)==listViHPKs.GetAt(1))?1:0));
			} else if (nViH==1) {
				_VertHCore = new CEVector(ham_tmp, listViH.GetAt(0), !vIndCov); 
			} // else NULL {UNIT factor (no vector indices, all particles are SCALAR)}
			delete ham_tmp;

			ham_val = ((bIsF1)?((CExpression*)(new COpFermionField(pk1, v_num))):((CExpression*)(new COpBosonField(pk1, viH1, vIndCov, v_num))));
			if (_VertHCore!=NULL) ham_val = ham_val->multiply(_VertHCore);
			ham_val = ham_val->multiply(((bIsF2)?((CExpression*)(new COpFermionField(pk2, v_num))):((CExpression*)(new COpBosonField(pk2, viH2, vIndCov, v_num)))));
			ham_val = ham_val->multiply(((bIsF3)?((CExpression*)(new COpFermionField(pk3, v_num))):((CExpression*)(new COpBosonField(pk3, viH3, vIndCov, v_num)))));
			ham_val = ham_val->multiply(((bIsF4)?((CExpression*)(new COpFermionField(pk4, v_num))):((CExpression*)(new COpBosonField(pk4, viH4, vIndCov, v_num)))));
		}
		if (bValCtOK) {
			CESymbol* ham_tmp = CESymbol::getUnique(nFermions==0);
			CExpression* _VertHCore = NULL;
			if (nViH==4) {
				_VertHCore = new CERank4Tensor(ham_tmp,
					listViH.GetAt(0),!vIndCov,listViH.GetAt(1),!vIndCov,listViH.GetAt(2),!vIndCov,listViH.GetAt(3),!vIndCov,
					((listViHPKs.GetAt(0)==listViHPKs.GetAt(1))?1:0),((listViHPKs.GetAt(2)==listViHPKs.GetAt(3))?1:0));
			} if (nViH==3) {
				_VertHCore = new CERank3Tensor(ham_tmp,listViH.GetAt(0),!vIndCov,listViH.GetAt(1),!vIndCov,listViH.GetAt(2),!vIndCov);
			} else if (nViH==2) {
				_VertHCore = new CEMatrix(ham_tmp, listViH.GetAt(0), listViH.GetAt(1), !vIndCov, !vIndCov,
					((listViHPKs.GetAt(0)==listViHPKs.GetAt(1))?1:0));
			} else if (nViH==1) {
				_VertHCore = new CEVector(ham_tmp, listViH.GetAt(0), !vIndCov); 
			} // else NULL {UNIT factor (no vector indices, all particles are SCALAR)}
			delete ham_tmp;

			ham_vCT = ((bIsF1)?((CExpression*)(new COpFermionField(pk1, v_nCT))):((CExpression*)(new COpBosonField(pk1, viH1, vIndCov, v_nCT))));
			if (_VertHCore!=NULL) ham_vCT = ham_vCT->multiply(_VertHCore);
			ham_vCT = ham_vCT->multiply(((bIsF2)?((CExpression*)(new COpFermionField(pk2, v_nCT))):((CExpression*)(new COpBosonField(pk2, viH2, vIndCov, v_nCT)))));
			ham_vCT = ham_vCT->multiply(((bIsF3)?((CExpression*)(new COpFermionField(pk3, v_nCT))):((CExpression*)(new COpBosonField(pk3, viH3, vIndCov, v_nCT)))));
			ham_vCT = ham_vCT->multiply(((bIsF4)?((CExpression*)(new COpFermionField(pk4, v_nCT))):((CExpression*)(new COpBosonField(pk4, viH4, vIndCov, v_nCT)))));
		}
		break;
	}

	if ( ((bValueOK) && (ham_val==NULL)) || ((bValCtOK) && (ham_vCT==NULL)) ) {
		m_LastErrorMsg.AddString("Cant' create Hamiltonian item");
		if (ham_vCT!=NULL) delete ham_vCT; if (ham_val!=NULL) delete ham_val;
		return false;
	}

	CDiagramVertexInfo* vi = NULL; CDiagramVertexInfo* viCT = NULL;
	bool viOK=true;
	switch(partN) {
	case 2: // 2-lines vertex
		if (bValueOK) vi = new CDiagramVertexInfo(v_num, pk1, pk2);
		if (bValCtOK) viCT = CDiagramVertexInfo::newCountertermVertex(v_nCT, pk1, pk2);
		break;
	case 3: // 3-lines vertex
		if (bValueOK) vi = new CDiagramVertexInfo(v_num, pk1, pk2, pk3);
		if (bValCtOK) viCT = CDiagramVertexInfo::newCountertermVertex(v_nCT, pk1, pk2, pk3);
		break;
	case 4: // 4-lines vertex
		if (bValueOK) vi = new CDiagramVertexInfo(v_num, pk1, pk2, pk3, pk4);
		if (bValCtOK) viCT = CDiagramVertexInfo::newCountertermVertex(v_nCT, pk1, pk2, pk3, pk4);
		break;
	}

	if ((bValueOK) && (vi==NULL)) { m_LastErrorMsg.AddString("Can't create vertex info"); viOK=false; }
	if ((bValCtOK) && (viCT==NULL)) { m_LastErrorMsg.AddString("Can't create counterterm vertex info"); viOK=false; }
	if (!viOK) { if(vi!=NULL)delete vi; if(viCT!=NULL)delete viCT; if(ham_vCT!=NULL)delete ham_vCT; if(ham_val!=NULL)delete ham_val; return false; }

	// All the checks are done. Fill infos now.
	if (bValueOK) {
		if (bIsClassVertex) {
			m_eClassesHamiltonian=((m_eClassesHamiltonian!=NULL)?m_eClassesHamiltonian->add(ham_val):ham_val);
		} else {
			//dbg->log(2,"H-value:",ham_val);
			m_Hamiltonian_expr=((m_Hamiltonian_expr!=NULL)?m_Hamiltonian_expr->add(ham_val):ham_val);
			//dbg->log(2,m_Hamiltonian_expr);
		}
	}
	if (bValCtOK) {
		if (bIsClassVertex) {
			m_eClassesHamiltonianR=((m_eClassesHamiltonianR!=NULL)?m_eClassesHamiltonianR->add(ham_vCT):ham_vCT);
		} else {
			//dbg->log(2,"H-CT value:",ham_vCT);
			m_Renorm_Hamiltonian=((m_Renorm_Hamiltonian!=NULL)?m_Renorm_Hamiltonian->add(ham_vCT):ham_vCT);
			//dbg->log(2,m_Renorm_Hamiltonian);
		}
	}

	if (bValueOK) {
		dbg->log(55, v_num, "Vertex Info (no. %ld): ", vi);
		vert_info_list->add(vi);
	}
	if (bValCtOK) {
		dbg->log(55, v_nCT, "Vertex counterterm info (no. %ld): ", viCT);
		vert_info_list->add(viCT);
	}

	if (bIsClassVertex) {
		if (nVClassNo<0) { dbg->log(0,"WARNING: Invalid class-info-number for CLASS vertex. No class vertices will be available."); m_bClassVerticesComplete=false; }
		if (bValueOK) { m_idClassVertices.SetAt(v_num,nVClassNo); m_vnumClassVertices.SetAt(nVClassNo,v_num); }
		if (bValCtOK) { m_idClassVertices.SetAt(v_nCT,nVClassNo); m_vnCTClassVertices.SetAt(nVClassNo,v_nCT); }
	} else {
		long idClassVertex=-1;
		if ((bValueOK) || (bValCtOK)) {
			idClassVertex=InvolveVertexClass(inf_const,false);
			if (idClassVertex<0) m_bClassVerticesComplete=false; // Check for incomplete vertices set.
		}
		if (bValueOK) { m_idClassVertices.SetAt(v_num,idClassVertex); }
		if (bValCtOK) { m_idClassVertices.SetAt(v_nCT,idClassVertex); }
	}

	if (!bIsClassVertex) { // fill correspondence value
		CSequence syms_seq;
		if (all_mom_used) {
			mom_vec_used=true; // extra check
			if(partN>=1){ syms_seq.addItem(pPart1->clone()); if (vInd1 != NULL) { syms_seq.addItem(vInd1->clone()); } }
			if(partN>=2){ syms_seq.addItem(pPart2->clone()); if (vInd2 != NULL) { syms_seq.addItem(vInd2->clone()); } }
			if(partN>=3){ syms_seq.addItem(pPart3->clone()); if (vInd3 != NULL) { syms_seq.addItem(vInd3->clone()); } }
			if(partN>=4){ syms_seq.addItem(pPart4->clone()); if (vInd4 != NULL) { syms_seq.addItem(vInd4->clone()); } }
		} else if (mom_vec_used) {
			if(partN>=1){ if (vInd1 != NULL) { syms_seq.addItem(pPart1->clone()); syms_seq.addItem(vInd1->clone()); } }
			if(partN>=2){ if (vInd2 != NULL) { syms_seq.addItem(pPart2->clone()); syms_seq.addItem(vInd2->clone()); } }
			if(partN>=3){ if (vInd3 != NULL) { syms_seq.addItem(pPart3->clone()); syms_seq.addItem(vInd3->clone()); } }
			if(partN>=4){ if (vInd4 != NULL) { syms_seq.addItem(pPart4->clone()); syms_seq.addItem(vInd4->clone()); } }
		} else {
			if(partN>=1){ if (vInd1 != NULL) syms_seq.addItem(vInd1->clone()); }
			if(partN>=2){ if (vInd2 != NULL) syms_seq.addItem(vInd2->clone()); }
			if(partN>=3){ if (vInd3 != NULL) syms_seq.addItem(vInd3->clone()); }
			if(partN>=4){ if (vInd4 != NULL) syms_seq.addItem(vInd4->clone()); }
		}
		if (bValueOK) { m_corr_rules.addVertexCorrRule(v_num,vCorr,&syms_seq,mom_vec_used,all_mom_used); }
		if (bValCtOK) { m_corr_rules.addVertexCorrRule(v_nCT,vCorrCT,&syms_seq,mom_vec_used,all_mom_used); }
	}
	return viOK;
}

// particle INTERNAL lines
bool CPhysModelFA::addIntLineCorr(long pk, CExpression* vCorr, CExpression* mom, CExpression* m, CExpression* vInd1, CExpression* vInd2) {
	m_LastErrorMsg.Reset();
	if (pk==0) { m_LastErrorMsg.AddString("CPhysModelFA::addExtLineCorr(pk==0)"); return false; }
	if (vCorr==NULL) { m_LastErrorMsg.AddString("CPhysModelFA::addExtLineCorr(vCorr==NULL)"); return false; }
	if(pk<0) {
		//we put both for particle and conjugated one at pk>0 call
		m_LastErrorMsg.AddString("CPhysModelFA::addIntLineCorr(): pk should be positive.");
		return false;
	}
	//bool bCorrCjgtd=false; if(pk<0){bCorrCjgtd=true;pk=-pk;}

	// Syms format: {p1,m,i1,i2},{p1,m},{p1},{}
	CSequence* syms_seq=NULL;
	if (mom!=NULL) {
		syms_seq=new CSequence(); syms_seq->addItem(mom->clone());
		if (m!=NULL) {
			syms_seq->addItem(m->clone());
			if (vInd1!=NULL) {
				syms_seq->addItem(vInd1->clone());
				if (vInd2!=NULL) { syms_seq->addItem(vInd2->clone()); }
			}
		}
	}

	m_corr_rules.addIntLineCorrRule(pk, vCorr, syms_seq);
	// in SM-physics also exist:
	// m_corr_rules.addIntLineCorrRule(pk, ferm_CT, syms_seq, CDiagCorrRules::CORR_RULE_COUNTER_TERM);

	if(syms_seq!=NULL)delete syms_seq;
	return true;
}

// particle EXTERNAL lines
bool CPhysModelFA::addExtLineCorr(long pk, CExpression* vCorr, CExpression* mom, CExpression* m, CExpression* vInd1) {
	m_LastErrorMsg.Reset();
	if (pk==0) { m_LastErrorMsg.AddString("CPhysModelFA::addExtLineCorr(pk==0)"); return false; }
	if (vCorr==NULL) { m_LastErrorMsg.AddString("CPhysModelFA::addExtLineCorr(vCorr==NULL)"); return false; }
	if(pk<0) {
		//we put both for particle and conjugated one at pk>0 call
		m_LastErrorMsg.AddString("CPhysModelFA::addExtLineCorr(): pk should be positive. Both {pk} and {-pk} correspondence are created if pk>0.");
		return false;
	}
	//bool bCorrCjgtd=false; if(pk<0){bCorrCjgtd=true;pk=-pk;}

	// Syms format: {p1,m,i1,i2},{p1,m},{p1},{}
	// TODO: check mom- and mass- containment
	CSequence* syms_seq=NULL;
	if (mom!=NULL) {
		syms_seq=new CSequence(); syms_seq->addItem(mom->clone());
		if (m!=NULL) {
			syms_seq->addItem(m->clone());
			if (vInd1!=NULL) syms_seq->addItem(vInd1->clone());
		}
	}
	//if(bCorrCjgtd) m_corr_rules.addExtLineCorrRule(pk, vCorr, syms_seq, CDiagCorrRules::CORR_RULE_FLAG_CONJUGATED);
	//else m_corr_rules.addExtLineCorrRule(pk, vCorr, syms_seq);

	m_corr_rules.addExtLineCorrRule(pk, vCorr, syms_seq);
	CExpression* vCorrCjg=CEBasic::Congugate(vCorr,false); if(vCorrCjg==vCorr)vCorrCjg=vCorrCjg->clone();

	m_corr_rules.addExtLineCorrRule(pk, vCorrCjg, syms_seq, CDiagCorrRules::CORR_RULE_FLAG_CONJUGATED);
	delete vCorrCjg;

	if(syms_seq!=NULL)delete syms_seq;

	//SM-values:
	//fermions
	//CESpinor val(mom, false); m_corr_rules.addExtLineCorrRule(pk, &val, syms_seq);
	//CESpinor valC(mom, true); m_corr_rules.addExtLineCorrRule(pk, &valC, syms_seq, CDiagCorrRules::CORR_RULE_FLAG_CONJUGATED);
	//bosons
	//m_corr_rules.addExtLineCorrRule(pk, unit, NULL);....
	//Higgs
	//m_corr_rules.addExtLineCorrRule(CEParticle::PARTICLE_HIGGS, unit, NULL);
	//m_corr_rules.addExtLineCorrRule(CEParticle::PARTICLE_HIGGS, unit, NULL, CDiagCorrRules::CORR_RULE_FLAG_CONJUGATED);
	return true;
}

