////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ScriptValueDiagramSet.cpp: implementation of the CScriptValueDiagramSet class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

const char* CScriptValueDiagramSet::CLASS_NAME = "CScriptValueDiagramSet";

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CScriptValueDiagramSet::CScriptValueDiagramSet(CDiagramSet* _diagrams, bool is_cloned)
: CScriptValue()
{
	if ((is_cloned) || (_diagrams == NULL)) m_diags_val = _diagrams;
	else m_diags_val = _diagrams->clone();
}

CScriptValueDiagramSet::~CScriptValueDiagramSet()
{
	if (m_diags_val != NULL) delete m_diags_val;
}

CScriptValue* CScriptValueDiagramSet::clone() const {
	bool bKinOK=true;
	if (m_diags_val!=NULL) {
		CKinematics* kin=CEProcessor::kinematics();
		//CScriptProcessor::dbg->log(0,m_diags_val->GetKinHashID(),kin->GetHashCode(),"Dgr: %ld, Kin: %ld");
		//CScriptProcessor::dbg->log(0,m_diags_val->get_diagrams_number(),"Dgr N: %ld");
		if (kin==NULL) { bKinOK=false; }
		else if ((m_diags_val->GetKinHashID() != kin->GetHashCode())&&(m_diags_val->GetKinHashID()!=0)) { bKinOK=false; }
	}
	CScriptValue* res=NULL;
	if ((bKinOK) || (m_diags_val==NULL)) {
		res=new CScriptValueDiagramSet(m_diags_val, false);
	} else {
		CScriptProcessor::dbg->log(2,"WARNING: Operating with diagrams for different kinematics. DIAGRAMS ARE TRUNCATED!.");
		res=new CScriptValueDiagramSet(m_diags_val->getEmptyFrame(), true);
	}
	return res;
}
int CScriptValueDiagramSet::value_type() { return CScriptValue::VAL_DIAGRAM_SET;}
CEObject* CScriptValueDiagramSet::get_obj_value() { return m_diags_val; }

bool CScriptValueDiagramSet::check_value_type(int type_mask) {
	if (type_mask & CScriptValue::VAL_DIAGRAM_SET) return true;
	return false;
}

bool CScriptValueDiagramSet::check_value(int type_mask, bool check_object) {
	if (!check_value_type(type_mask)) return false;
	if (check_object) {
		if (m_diags_val == NULL) return false;
	}
	return true;
}

CEString* CScriptValueDiagramSet::toString(int mode)
{
	CEString* res = NULL;
	if (m_diags_val != NULL) res = new CEString("<Diagram Set>");
	else res = new CEString("<NULL Diagram Set>");
	return res;
}

const char* CScriptValueDiagramSet::ClassName() const { return CLASS_NAME; }
