////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// DElement.h: interface for the CDElement class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DELEMENT_H__D89C2400_6B13_42AC_A06F_C3F851D9BFDD__INCLUDED_)
#define AFX_DELEMENT_H__D89C2400_6B13_42AC_A06F_C3F851D9BFDD__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CDElement : public CESerializable  
{
	// pointers to other CDElement objects
	CEPtrArray* m_links;

	// graph position
	// Note: position is NOT stored with diagrams ==> one must call 'diags->LocateDiagramsView' again after loading
	CDElementViewInfo* m_pos_info;

protected:
	int m_DE_kind;

public:
	static CDebug* dbg;
	static const char* CLASS_NAME;

	CESerializable* m_temporary_mark; // volatile variable

	static const int EL_VERTEX;
	static const int EL_LINE;
	static const int EL_INT_LINE;
	static const int EL_EXT_LINE;
	static const int EL_VECTOR;
	static const int EL_SPINOR;
	static const int EL_LN_INITIAL;
	static const int EL_COUNTER_TERM;

	CEPtrArray* get_links();
	int get_links_number();
	CDElement* get_link(int i);
	CDElementViewInfo* getViewInfo() { return m_pos_info; }
	const CDElementViewInfo* getViewInfoConst() const { return m_pos_info; }
	void setViewInfo(CDElementViewInfo* view_info);

	// used by diagram factory
	int get_first_binded_link_number();

	CDElement(const CDElementViewInfo* view_info);
	virtual ~CDElement();
	virtual CEObject* copy() const; // overrrided for array cloning
	
	int DE_kind();
	bool checkDEKind(int _mask);
	// virtuals to override
	virtual const char* ClassName() const;
	virtual bool contain_vector(void * obj);
	virtual CDElement* clone() const = 0;
	virtual bool is_binded() = 0;
	virtual CEString* draw(int settings = 0) = 0;
	//virtual CExpression* get_ME_factor() = 0;

public:
	static bool copy_links(CEPtrArray *target, CEPtrArray *source, CEPtrArray *refs_new, CEPtrArray *refs_old);

	// old
	//CDElement* m_recent_copy;
	// old
	//virtual CExpression* get_me_factor() = 0;
	//virtual void arange_links(CDElement* old) = 0;
	//virtual bool is_binded() = 0;
};

#endif // !defined(AFX_DELEMENT_H__D89C2400_6B13_42AC_A06F_C3F851D9BFDD__INCLUDED_)
