////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// DIntLine.h: interface for the CDIntLine class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DINTLINE_H__82EB8EAF_8A76_4D10_B325_9520EF7D1197__INCLUDED_)
#define AFX_DINTLINE_H__82EB8EAF_8A76_4D10_B325_9520EF7D1197__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CDIntLine : public CDLine  
{
	CExpression* m_momentum;
	long m_particle_kind;
	bool m_is_CT_line; // is counterterm element
protected:
	CDIntLine(); // unused
	CDIntLine(long part_kind, const CExpression* _momentum, bool _is_CT_line, const CDElementViewInfo* view_info);
public:
	static const char* CLASS_NAME;
	virtual ~CDIntLine();
	virtual void ArrangeElement();

	// save/load
	virtual long store_stream(ofstream* p_ofs, DWORD settings);
	virtual const char* get_TAG_Name();
	static const char* CLASS_TAG_NAME;
	static CESerializable* restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings);

	virtual long get_particle_kind() const;
	virtual void set_particle_kind(long pk);
	virtual CDElement* clone() const;
	virtual CEString* toString(int mode = 0);
	virtual CEString* draw(int settings = 0);
	virtual const char* ClassName() const;
	virtual bool contain_vector(void * obj);
	virtual bool is_binded();
	CDElement* get_other_link(CDElement* link);

	static CDIntLine* createIntLine(int part_kind, CExpression* _momentum);
	static CDIntLine* createCounterTermLine(int part_kind, CExpression* _momentum);

	virtual CDElement* get_vertex_to();
	virtual CDElement* get_vertex_from();

	// Returns the momentum of line goes to vertex.
	// For (vertex != NULL) returns CLONED value.
	// For "NULL" vertex also returns CLONED internal momentum value
	virtual CExpression* get_momentum(CDElement* vertex_to);
	// used for cloning, returns NON-cloned value
	// can be used for check up momentum existance
	virtual CExpression* get_internal_momentum_val();
	void applyRuleToMomentum(CERule* r); // Used to change volatile variables in momentum
	// Treats momentum as OUTCOMING from "vertex_from" vertex.
	// If (vertex_from == NULL) the momentum can only be cleared (if _momentum -> NULL).
	virtual void set_momentum(CExpression* _momentum, CDElement* vertex_from);

	//void set_momentum(CDVertex* v_to, CExpression* val);
	//virtual CExpression* get_momentum(CDVertex* v_to = NULL);
	//CDVertex* get_other_vertex(CDVertex* v);
	//CDVertex* vertex_from();
	//CDVertex* vertex_to();
	//virtual void bind(CDVertex* v);
	//virtual CEString* toString(int mode = 0);CDIntLine(CDVertex* _v_to);
	//CDIntLine(CDVertex* _v_to, CDVertex* _v_from, bool _is_binded);
	//virtual ~CDIntLine();
};

#endif // !defined(AFX_DINTLINE_H__82EB8EAF_8A76_4D10_B325_9520EF7D1197__INCLUDED_)
