////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// DVertex.h: interface for the CDVertex class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DVERTEX_H__8CFF95EE_4518_429C_A1A9_A350FD1704B0__INCLUDED_)
#define AFX_DVERTEX_H__8CFF95EE_4518_429C_A1A9_A350FD1704B0__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//##ModelId=3E3FF2DC005D
class CDVertex : public CDElement  
{
	CDiagramVertexInfo* m_vertex_info;

public:

	CDVertex(const CDiagramVertexInfo* vi, const CDElementViewInfo* view_info);
	virtual ~CDVertex();
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual bool contain_vector(void * obj);

	// save/load
	virtual long store_stream(ofstream* p_ofs, DWORD settings);
	virtual const char* get_TAG_Name();
	static const char* CLASS_TAG_NAME;
	static CESerializable* restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings);

	// overrided virtuals
	//virtual const char* ClassName() const;
	virtual CDElement* clone() const;
	//virtual CExpression* get_ME_factor();
	virtual CEString* toString(int mode = 0);
	virtual CEString* draw(int settings = 0);

	CDiagramVertexInfo* get_vertex_info();
	void set_vertex_info(CDiagramVertexInfo* vi_const);
	virtual bool is_binded();
	bool has_binded_tale_line();

	CDVertex* get_vertex_linked(int n);

	//	returns -1 if no momenta can be arranged.
	//	returns 0 if all the momenta are arranged.
	//	returns 1 if momenta arranged successfully.
	// Kinematics object is used for simplification of momntum expressions.
	int ArrangeMomenta(CKinematics* kin);

	bool check_VI_validity(); // debug function to vertex self-check

	bool sortLinksOrder(); // sorts links order according to VertexInfo data
	bool sortLinksOrder(CIntArray* order); // sorts links order according to external order
public:
	long GetLogicalPKForAttachedLine(CDElement* l); // as 'incoming'
	void SetLogicalPKForAttachedLine(CDElement* l, long pk_incoming);
};

#endif // !defined(AFX_DVERTEX_H__8CFF95EE_4518_429C_A1A9_A350FD1704B0__INCLUDED_)
