////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// DiagCorrRules.h: interface for the CDiagCorrRules class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DIAGCORRRULES_H__282EF06A_DEEA_4C9A_937C_2BBDBA121381__INCLUDED_)
#define AFX_DIAGCORRRULES_H__282EF06A_DEEA_4C9A_937C_2BBDBA121381__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CDiagCorrRules : public CESerializable  
{
	// default symbols for parameter notation (almost useless)
	CEPtrArray* m_internal_symbols; // contains CExprContainer objects

	//	inetrnal/external lines (1: int/ext line ID, 2: particle ID) 
	//	vertices (1: vertex ID, 2: vertex NO)
	CEPtrArray* m_corr_data; // contains CDCorrRule objects

public:
	static const int INTERNAL_SYM_MOMENTUM;
	static const int INTERNAL_SYM_MASS;
	static const int INTERNAL_SYM_VECTOR_INDEX;
	static const int INTERNAL_SYM_MOMENTUM_2;
	static const int INTERNAL_SYM_VECTOR_INDEX_2;

	static const int CORR_RULE_INT_LINE;
	static const int CORR_RULE_EXT_LINE;
	static const int CORR_RULE_VERTEX;
	static const int CORR_RULE_COUNTER_TERM; // to be combined with internal line (CT vertices are implemented as separate vertices)

	// flags to composed with CORR_RULE_EXT_LINE
	static const int CORR_RULE_FLAG_CONJUGATED; // means XOR(Antiparticle, Final-state particle)
//	static const int CORR_RULE_FLAG_ANTIPARTICLE;
//	static const int CORR_RULE_FLAG_INITIAL_PARTICLE;

	static CDebug* dbg;

	int m_X_rule_no; // temp
	int m_X_rule_cnt; // temp

	CDiagCorrRules();
	virtual ~CDiagCorrRules();

	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual CEString* toString(int mode = 0);
private:
	//CEString* getCorrTitle(int corr_type, long ID);

public:

	CExpression* get_internal_symbol(int sym_ID);

	// corr_flags may be CORR_RULE_COUNTER_TERM
	void addIntLineCorrRule(long pk, CExpression* expr, CSequence* info_seq, int corr_flags = 0);
	// corr_flags may be CORR_RULE_FLAG_CONJUGATED
	void addExtLineCorrRule(long pk, CExpression* expr, CSequence* info_seq, int corr_flags = 0);
	void addVertexCorrRule(int vertex_ID, CExpression* expr, CSequence* info_seq, bool vec_mom_used=false, bool all_mom_used=false);

	CExpression* get_Correspondence(int corr_type, long _ID, CSequence* new_syms, bool demand_vars_coinsidence = false);
	CExpression* GetVertexCorrespondence(int vertex_ID, CSequence* new_syms);
	bool NeedVecMomentaForVertexCorr(int vertex_ID);
	bool NeedAllMomentaForVertexCorr(int vertex_ID);
	CExpression* GetLineCorrespondence(int line_type, long pk, CExpression* sym_mom, CExpression* sym_mass, CSequence* indices);

private:
	CDCorrRule* GetCorrRule(int corr_type, long _ID);
};

#endif // !defined(AFX_DIAGCORRRULES_H__282EF06A_DEEA_4C9A_937C_2BBDBA121381__INCLUDED_)
