////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// DiagramPhysics.h: interface for the CDiagramPhysics class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DIAGRAMPHYSICS_H__91893EF3_E2B3_4D69_8FBB_9ED85B5EEE8E__INCLUDED_)
#define AFX_DIAGRAMPHYSICS_H__91893EF3_E2B3_4D69_8FBB_9ED85B5EEE8E__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CDiagramPhysics : public CEObject  
{
	static CDiagramPhysics* m_physics;

	CEPtrArray* m_vertices_info; // array of (CDiagramVertexInfo*)

	int m_DR_mass_PK; // 0 means usage of abstract "mu" mass for DR
	CExpression* m_DR_mass_symbol; // may not be NULL;
//protected:
	//void add_new_vertex_info(CDiagramVertexInfo* vi);
public:
	static CDebug* dbg;
	static const char* m_SymGeV2PbConst;

	static CDiagramPhysics* get_curr_physics();
	static void install_diagram_physics(CDiagramPhysics* phys, bool delete_old = true);

	static void init();
	static void finalize();

	virtual void init_physics() = 0; // invoked before physics is installed
	virtual CDiagCorrRules* getCorrespondenceRules() = 0;
	virtual CExpression* getHamiltonianExpr() = 0;
	virtual CExpression* getRenormHamiltonian() = 0;
	virtual CExpression* getClassesHamiltonianE(); // NULL if not available
	virtual CExpression* getClassesHamiltonianRen(); // NULL if not available
	virtual CERule* getHReplRule() = 0;
	virtual DWORD getPhysicsID() = 0;
	virtual bool CounterTermsDeclared() = 0; // called by CMEComposeLib
	virtual CERule* getRenormConstValues() = 0;
	virtual CSequence* CollectConstants() = 0;
	virtual CEPtrArray* CollectConstantValues() = 0; // array of CExprContainer, cloned value
	virtual CExpression* GetGeV2PbFactor(); // returns cloned expression
	virtual bool validate_part_kind(long _kind) = 0;
	//
	static const int SYMBOL_STYLE_TEX;
	static const int SYMBOL_STYLE_NB;
	virtual const char* get_particle_symbol(long _kind, int sym_style) = 0;
	virtual int getCharge3(long pk); // charge * 3 (INT_MIN for unknown)
	static const char* get_particle_symbolA(long _kind, int sym_style);

	//virtual void initPhysics() = 0;
	CEPtrArray* getVerticesInfo();
	CDiagramPhysics();
	virtual ~CDiagramPhysics();
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual bool contain_vector(void * obj);

	virtual CExpression* get_mass_sym(long pk) = 0; // returns not cloned value
	virtual void set_mass_sym(long pk, CExpression* val) = 0;
	virtual int getDiagramLineType(long pk_abs); // res==0 means default selection procedure
	virtual long CollectVerticesForClass(long vnumClassVertex, CIntArray* idAltVerts);
	virtual bool AreCTPropagatorsVertices(); //default: true <== CT are implemented as 2-tale vertices

	void set_DR_mass_PK(long pk);
	CExpression* get_DR_mass_sym();

	virtual long GetPDGParticleCode(long pk);	//default implementation should fit most cases

private:
	/*
	*	Titles for particles by PK. (examsple: class particle with no title notation)
	*/
	CEPtrArray m_arrClassPKTitles; //array of CEString
	CIntArray m_iIdClassPKTitles; // pks attached to m_arrClassPKTitles
public:
	const char* GetClassPKTitle(long pkClass);
};

#endif // !defined(AFX_DIAGRAMPHYSICS_H__91893EF3_E2B3_4D69_8FBB_9ED85B5EEE8E__INCLUDED_)
