////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// DiargamSet.h: interface for the CDiagramSet class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DIARGAMSET_H__91561C00_1E31_11D8_9A77_97FF12A67035__INCLUDED_)
#define AFX_DIARGAMSET_H__91561C00_1E31_11D8_9A77_97FF12A67035__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CDiagramSet : public CESerializable  
{
	CDiagramPhysics* m_physics;
	// (?) set of diagrams of uniform order
	// int m_order;

	long m_kinHashID; // kinematics hash code (0: undefined (no diagrams))

	// list of valid diagrams (CFeynDiagram)
	CEPtrArray* m_diagram_list;

	int m_drawing_flags; // flags are declared in CFeynDiagram class
	
	CDiagramSet(CDiagramPhysics* phys, CEPtrArray* diagram_list, int _drawing_flags, DWORD _kinHashID);
public:
	static CDebug* dbg;

	/////////////////////////////////////////////////
	CDiagramSet(CDiagramPhysics* phys);
	virtual ~CDiagramSet();
	CDiagramSet* clone() const;
	CDiagramSet* getEmptyFrame();
	long GetKinHashID();
	virtual CEString* toString(int mode = 0);
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual CEObject* copy() const;

	// save/load
	virtual long store_stream(ofstream* p_ofs, DWORD settings);
	virtual const char* get_TAG_Name();
	static const char* CLASS_TAG_NAME;
	static CESerializable* restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings);

	/////////////////////////////////////////////////
	void add_diagram(CFeynDiagram* diag);
	void add_diagrams(CDiagramSet* diags);
	long get_diagrams_number();
	CFeynDiagram* get_diagram(long i);
	void remove_diagram(long i);

	int GetDrawingFlags();
	void SetDrawingFlags(int _drawing_flags);

	/////////////////////////////////////////////////
	bool LocateDiagramsView(int Algorithm_ID);
	virtual CEString* draw(int settings = 0);
	bool ArrangeMomenta();
	CMatrixElement* RetrieveMatrixElement();
};

#endif // !defined(AFX_DIARGAMSET_H__91561C00_1E31_11D8_9A77_97FF12A67035__INCLUDED_)
