////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// DiagramVertexInfo.h: interface for the CDiagramVertexInfo class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DIAGRAMVERTEXINFO_H__B23CA29B_1CC6_4508_B000_D1F493D32107__INCLUDED_)
#define AFX_DIAGRAMVERTEXINFO_H__B23CA29B_1CC6_4508_B000_D1F493D32107__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CDiagramVertexInfo : public CESerializable  
{
	int m_tales_num;
	long* m_tales_info; // INCOMING patricles pk

	int m_vertex_ID;

	int m_vertex_type; // usual or counterterm

	CDiagramVertexInfo(int vertex_ID, int _tales_num, long* _tales_info, int _vert_type);
	void set_vert_type(int _vert_type); // set/unset counterterm flag
public:
	static const int VERTEX_COUNTERTERM;

	CDiagramVertexInfo(int vertex_ID, long part_kind_1, long part_kind_2, long part_kind_3, long part_kind_4);
	CDiagramVertexInfo(int vertex_ID, long part_kind_1, long part_kind_2, long part_kind_3);
	CDiagramVertexInfo(int vertex_ID, long part_kind_1, long part_kind_2);
	static CDiagramVertexInfo* newCountertermVertex(int vertex_ID, long part_kind_1, long part_kind_2, long part_kind_3, long part_kind_4);
	static CDiagramVertexInfo* newCountertermVertex(int vertex_ID, long part_kind_1, long part_kind_2, long part_kind_3);
	static CDiagramVertexInfo* newCountertermVertex(int vertex_ID, long part_kind_1, long part_kind_2);
	virtual ~CDiagramVertexInfo();

	int get_tales_num();
	long get_particle_kind(int tale_number) const; // tale_number is of [0, max-1]
	int get_vertex_ID();
	int get_vertex_order();
	bool check_vert_type(int type_mask) const; // check for counterterm
	long CalcPartsIdentityOrder(); // NNA->2, NNMM->4, NNN->6
	static long CalcPartsIdentityOrder(long n, long* pks); // call: long pks[3]; ... f=CalcPartsIdentityOrder(3,pks);
	static long CalcPartsIdentityOrder(long pk1, long pk2);
	static long CalcPartsIdentityOrder(long pk1, long pk2, long pk3);
	static long CalcPartsIdentityOrder(long pk1, long pk2, long pk3, long pk4);

public:
	/*
	*	Used for Class vertices only,
	*	vSpec: non-class vertex to meet this.
	*	Note: straight order is MANDATORY (as vertices fitting check).
	*	Do not call it for non-fitting vertices.
	*	orders->size() should occur CalcPartsIdentityOrder()/vSpec->CalcPartsIdentityOrder();
	*/
	void CollectClassFitOrders(CDiagramVertexInfo* vSpec, CEPtrArray* orders);
private:
	void CollectClassFitOrders(CDiagramVertexInfo* vSpec, CEPtrArray* orders, CIntArray* o, int i_next);

public:
	virtual CEString* toString(int mode = 0);
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual bool contain_vector(void * obj);
	virtual CEObject* copy() const;
	CDiagramVertexInfo* clone() const;

	// save/load
	virtual long store_stream(ofstream* p_ofs, DWORD settings);
	virtual const char* get_TAG_Name();
	static const char* CLASS_TAG_NAME;
	static CESerializable* restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings);

	bool isEqualTo(CDiagramVertexInfo* other);
};

#endif // !defined(AFX_DIAGRAMVERTEXINFO_H__B23CA29B_1CC6_4508_B000_D1F493D32107__INCLUDED_)
