////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// FeynDiagram.h: interface for the CFeynDiagram class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_FEYNDIAGRAM_H__0A35C7BD_9975_4AA6_873B_5F0EACC6955D__INCLUDED_)
#define AFX_FEYNDIAGRAM_H__0A35C7BD_9975_4AA6_873B_5F0EACC6955D__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CFeynDiagram : public CESerializable  
{
	CDiagramPhysics* m_physics;
	long m_kinHashID; // 0 for undefined (blank object)

	// arrays of CDElement objects
	CEPtrArray* m_vertices_list;
	CEPtrArray* m_lines_list;

	// locate info
	int m_locate_algorithm;

	CEPtrArray* m_virt_momenta_list;
	CEPtrArray* m_ext_vector_indices_list;	// volatile

	CExpression* m_factor_expr; // NULL means "*1" factor

public:

	//
	//	Draw settings
	//
	static const int DRAW_NO_ENVIRONMENT; // do not create "picture" environment for diagram
	static const int DRAW_IN_DEFAULT_BOX; // unused (what should it do?)
	static const int SET_LOOP_90;
	static const int SET_LOOP_120;
	static const int DRAW_CAPS_PARTICLE_TITLES;
	static const int DRAW_CAPS_PARTICLE_SIGNS; // used rectrictrly by CDExtLine
	static const int DRAW_CAPS_MOMENTA;
	static const int DRAW_CAPS_LINE_INFO;
	static const int DRAW_CAPS_SMALL;
	static const int DRAW_DIAGRAM_NUMBERS; // no of diagram in DiagramSet
	static const int DRAW_SWAP_TALES;
	static const int DRAW_DONT_SWAP_TALES;
	

private:
	CFeynDiagram(CDiagramPhysics* _physics,
		CEPtrArray* _vertices_list,
		CEPtrArray* _lines_list,
		CEPtrArray* _virt_momenta_list,
		CExpression* _factor_expr,
		long _kinHashID,
		int locate_algorithm);

public:
	static CDebug* dbg;
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual bool contain_vector(void * obj);

	// save/load
	virtual long store_stream(ofstream* p_ofs, DWORD settings);
	virtual const char* get_TAG_Name();
	static const char* CLASS_TAG_NAME;
	static CESerializable* restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings);

	virtual ~CFeynDiagram();
	static CFeynDiagram* create_blanc(CDiagramPhysics* _physics,
		bool create_EXT_lines,
		CKinematics* kin);

	CFeynDiagram* clone() const;
	virtual CEObject* copy() const; // used for copying of diagram arrays

	// data operations
	CEPtrArray* get_lines_list();
	CEPtrArray* get_vertices_list();
	int get_lines_number();
	int get_vertices_number();
	CDVertex* get_vertex(int n);
	CDLine* get_line(int n);
	void add_vertex(CDVertex* v);
	void add_line(CDLine* l);
	int get_vertex_pos(CDVertex* v);
	int get_line_pos(CDLine* l);
	CExpression* get_factor_expr();
	void set_factor_expr(CExpression* val); // does not affects patrameter
	long GetKinHashID();

	// locate data access
	int get_locate_algorithm();
	void set_locate_algorithm(int locate_algorithm);
	void clear_locate_data(bool _lines=true, bool _vertices=true);

	// self-check
	bool is_valid(int in_order = -1);
	bool IsEqualTo(CFeynDiagram* other, CELibrary* arr_lib=NULL, CELibrary* arr_lib_other=NULL, bool ignore_factor=false);
	int GetLoopsNumber();
	int GetCTNumber(); // counter-terms number

	virtual CEString* toString(int mode = 0);
	void addBindingInfo(CEString* str, CDElement* el);
	virtual CEString* draw(unsigned width, unsigned height, int settings = 0);

	// temporary data
	void clear_temporary_data(bool do_delete);

	// Assigns integer positions to all the diagram elements.
	// Used locally by draw() function
	// Perhaps, can be used to set default size before toString()-drawing (not implemented).
	void set_diagram_size(unsigned width, unsigned height);

	// CKinematics is used for momenta expressions simplification.
	bool ArrangeMomenta(CKinematics* kin);

	///////////////////////////////////////////////////////////////////////// ME retrieving functions

	/*
	*  Do not use this function directly. Use CDiagramSet::RetrieveMatrixElement() instead.
	*  kin->reset_volatile_variables(true/false) should be calles before & after matrix element retrieving.
	*/
	CMatrixElement* RetrieveMatrixElement(); // CDiagCorrRules* rules
private:
	CExpression* RetrieveFermionTaleFactor(CDLine* l);// ext line or int line for loops
	CExpression* RetrieveFermionLineFactor(CDLine* l, CDVertex* v, bool v_is_to);
	CExpression* RetrieveVertexFactor(CDVertex* v);
	CExpression* RetrieveBosonicLineFactor(CDLine* l);
};

#endif // !defined(AFX_FEYNDIAGRAM_H__0A35C7BD_9975_4AA6_873B_5F0EACC6955D__INCLUDED_)
