////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// LDiagArrange.h: interface for the CLDiagArrange class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_LDIAGARRANGE_H__07404024_3E27_4388_996C_B621E83DBE17__INCLUDED_)
#define AFX_LDIAGARRANGE_H__07404024_3E27_4388_996C_B621E83DBE17__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CLDiagArrange : public CELibrary  
{
	int m_lines_num;
	int m_verts_num;
	int m_lines_sorted;
	int m_verts_sorted;
	CFeynDiagram* m_diag; // diagram is neither cloned no deleted
	CEPtrArray* m_lines;
	CEPtrArray* m_verts;

	bool m_arrange_OK;

	CLDiagArrange();
	bool DiagArrangeImpl(CFeynDiagram* diag);

	void DiagArrangeElementsOrder();

	// comparison
public:
	static const int CMP_A_LESS_THAN_B;
	static const int CMP_A_GREATER_THAN_B;
	static const int CMP_A_EQUAL_B;
	static const int CMP_AB_UNDEFINED;

	int LineABeforeB(int a, int b);
	int VertexABeforeB(int a, int b);

	int GetEqualsVerticesNum(int first_pos);
	int GetEqualsLinesNum(int first_pos);

	int IsDiagArrangedOK();

	static bool AreVerticesSimilar(CDVertex* v1, CFeynDiagram* d1, CDVertex* v2, CFeynDiagram* d2);
	static bool AreLinesSimilar(CDLine* l1, CFeynDiagram* d1, CDLine* l2, CFeynDiagram* d2);
private:

	// comparison data collection
	int getLineMinSortedLink(CDLine* l, int min_after = -1);
	int getVertMinSortedLink(CDVertex* v, int min_after = -1);
	int getNumLinksToSortedVert(CDLine* l, int v); // calc how many links of line points to vertex
	int getNumLinksToSortedLine(CDVertex* v, int l); // calc how many links of vertex points to line
	int getMinVertexLinked(CDVertex* v, int min_after); // search vertices linked to this
	int getNumLinksToOtherVertex(CDVertex* v, int v_other); // search vertices linked to this

	bool areAllLinksSorted(CDVertex* v);
	bool areAllLinksSorted(CDLine* l);
	bool areLinesParallel(CDLine* a, CDLine* b);
	bool areVerticesParallel(CDVertex* a, CDVertex* b); // have similar links
	bool isLineDirect(CDLine* l); // i.e. starts from vertex with lower position
public:
	virtual ~CLDiagArrange();
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;

	static CDebug* dbg;
	static void DiagArrange(CFeynDiagram* diag);
	static CLDiagArrange* DiagArrangeGetLib(CFeynDiagram* diag); // returns library instance just worked
	static void DiagArrangeLinesPK(CFeynDiagram* diag);
};

#endif // !defined(AFX_LDIAGARRANGE_H__07404024_3E27_4388_996C_B621E83DBE17__INCLUDED_)
