////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// PhysModelFA.h: interface for the CPhysModelFA class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_PHYSMODELFA_H__135CC2BE_616F_40DA_91B5_D74D72086E6F__INCLUDED_)
#define AFX_PHYSMODELFA_H__135CC2BE_616F_40DA_91B5_D74D72086E6F__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CPhysModelFA : public CDiagramPhysics  
{
public:
	class CFAGenericCplPartInfo : public CEObject {
	public:
		CExpression* m_SignSym; // si
		CEString m_PartTitle; // F,V
		CExpression* m_PartNumber; // symbol -> j1
		//CSequence m_Indices; // Generation etc.
		CExpression* m_MomSym; // momi
		CSequence m_VecIndices; // without 'covariance' info
		bool m_bIsNeutral;

		CFAGenericCplPartInfo();
		CFAGenericCplPartInfo(class CFAGenericCplPartInfo& inf);
		~CFAGenericCplPartInfo();

		virtual bool contain_vector(void * obj);
		static const char* CLASS_NAME;
		virtual const char* ClassName() const;
		virtual char* toLine(int mode = 0);
		bool equals(CFAGenericCplPartInfo* other);
	};
public:
	class CFAModelGenericCpl : public CEObject {
	public:
		CEPtrArray m_partInfos; // array of {CFAGenericCplPartInfo*}
		bool m_bMomsSymmetric;
		CExpression* m_Value;

		CFAModelGenericCpl();
		CFAModelGenericCpl(class CFAModelGenericCpl& inf);
		~CFAModelGenericCpl();
		void ExchangeParticles(int i,int j);

		virtual bool contain_vector(void * obj);
		static const char* CLASS_NAME;
		virtual const char* ClassName() const;
		virtual char* toLine(int mode = 0);

		bool equals(CFAModelGenericCpl* other);
	};
public:
	class CFAPatricleClassInfo : public CEObject {
	public:
		CEString m_idClassName; // F,S,...
		CEString m_idNumber; // [1],[2],...
		CExpression* m_SymMassOrig; // 0, ME, ...
		CIntArray m_Indices; // [geneation-ID],[color-ID]
		CSequence m_QuantumNumbers;
		int m_iPropagatorType; // CLDrawDiagView::LINE_FERMION || CLDrawDiagView::LINE_BOSON || CLDrawDiagView::LINE_SCALAR
		int m_iPropagatorArrow; // 1 or 0 --> leads to CLDrawDiagView::LINE_NO_ARROW flag
		//CEString m_sPropagatorLabel; // \\nu_1
		CExpression* m_PropagatorLabel; // \\nu_1
		CExpression* m_PropagatorLabelM; // Mathematica format
		CEString m_sPropagatorLabelTex;
		CEString m_sPropagatorLabelNb;
		bool m_SelfConjugated;

		CExpression* m_SymMass; // 0, m_e, m_F2...
		long m_PK;

		CFAPatricleClassInfo();
		CFAPatricleClassInfo(class CFAPatricleClassInfo& inf);
		~CFAPatricleClassInfo();

		virtual bool contain_vector(void * obj);
		static const char* CLASS_NAME;
		virtual const char* ClassName() const;
		virtual char* toLine(int mode = 0);
	};

public:
	/*
	class CFAPatricleClassRangeInfo : public CEObject {
	public:
		CEString m_idClassName; // F,S,...
		CEString m_idNumber; // [1],[2],...
		CEPtrArray m_IndexValues; // array of CIntArray
		CSequence m_IndexTitles; // for debug/logging use

		virtual bool contain_vector(void * obj);
		static const char* CLASS_NAME;
		virtual const char* ClassName() const;
		virtual char* toLine(int mode = 0);
	};
	*/

public:
	// model constants list
	//CEPtrArray* m_model_consts;
	//void initModelConstsList();
	//void involveModelConst(int id, CExpression* val); // deletes value
	// renormalization constants bank
	//CEPtrArray* m_renorm_consts;
	//void initRenormConstsList();
	//void involveRenormConst(int id, CExpression* val); // deletes value
	//void addVertexInfo(int v_num, int pk1, int pk2, int pk3, bool is_renorm_CT = false);
	//void addHamExprVFF(int v_num,
	//	CExpression* vec_core, CExpression* vect_index,
	//	int pk_a, int pk_f1, int pk_f2, bool is_renorm_CT = false);
	//void addCorrRuleAFF(int v_num, CExpression* ind);
	//void addCorrRuleGFF_CT(int v_num, CExpression* ind);
	//void Add3BosonVertex(int v_num,
	//	CExpression* vec_core,
	//	CExpression* vect_index_a, CExpression* vect_index_v1, CExpression* vect_index_v2,
	//	int pk_a, int pk_v1, int pk_v2, bool is_renorm_CT = false);
	//void Add4BosonVertex(int v_num, CExpression* vec_core,
	//	CExpression* vect_index_v1, CExpression* vect_index_v2,
	//	CExpression* vect_index_v3, CExpression* vect_index_v4,
	//	int pk_v1, int pk_v2, int pk_v3, int pk_v4, bool is_renorm_CT = false);
	//void addVertex4Info(int v_num, int pk1, int pk2, int pk3, int pk4, bool is_renorm_CT = false);
	//void addHamExprASV(int v_num,
	//	CExpression* vec_core, CExpression* vect_index_a, CExpression* vect_index_v,
	//	int pk_a, int pk_s, int pk_v, bool is_renorm_CT = false);

	//CExpression* GetFermCTDeclaration(int pk, CExpression* p, CExpression* m);
	//CExpression* getModelContsUnitFactor(); // returns \alpha*4*pi/e^2 = 1

public:

	CEPtrArray m_ParticleInfos; // array of CFAPatricleClassInfo
	CFAPatricleClassInfo* GetParticleClassInfo(const char* cName, const char* idNumber, CIntArray* _Indices, bool bAllowIncompleteIndices=false);
	CFAPatricleClassInfo* GetNextParticleClassInfo(const char* cName, const char* idNumber, int* prev); // first *prev should be==-1
	CFAPatricleClassInfo* FindParticleClassInfo(long pk);
	CFAPatricleClassInfo* GetParticleClassInfo(long i);
	long GetParticleClassesNum();

public:
	CPhysModelFA(); //CSModelLoader* _loader
	virtual ~CPhysModelFA();
	CEString m_LastErrorMsg;

	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	//virtual bool contain_vector(void * obj);

	long m_numExtraParticles; // Unique pk counter (next pk will be FLAGS|m_numExtraParticles)
	long CreateNewExtraParticleID(); // To unique pk (new pk will be FLAGS|CreateNewExtraParticleID())

	// correspondence rules
	CDiagCorrRules m_corr_rules;
	CExpression* m_Hamiltonian_expr;
	CExpression* m_eClassesHamiltonian;
	CExpression* m_eClassesHamiltonianR;
	CExpression* m_Renorm_Hamiltonian;
	CERulesList m_HamiltonianReplRules; // Unused (unfilled) now! H-m can't be converted to standard form!

public:
	/*
	*	m_VertexClasses: array of class-info objects
	*	m_idClassVertices[0,getVerticesInfo()->size()]: position of corresponding CLASS-vertex info in m_VertexClasses, -1 for CLASS vertex
	*	m_vnumClassVertices[0,m_VertexClasses.size()]: reverse for m_idClassVertices for class-vertices
	*	m_vnCTClassVertices[0,m_VertexClasses.size()]: reverse for m_idClassVertices for class-vertices
	*/
	CEPtrArray m_VertexClasses; // arr of CFAModelGenericCpl with "+/-" signs specified, used to fill m_eClassesHamiltonianX
	CIntArray m_idClassVertices; // pos corresponding CLASS-vertex info in m_VertexClasses (or -1)
	CIntArray m_vnumClassVertices; // vnum for every item of m_VertexClasses (or -1 if vertex is CT-only)
	CIntArray m_vnCTClassVertices; // vnCT for every item of m_VertexClasses (or -1 if vertex is non-CT-only)
	bool m_bClassVerticesComplete;

public:
	bool addVertexCorr(long pk1, long pk2, long pk3, long pk4,
		CExpression* vCorr, CExpression* vCorrCT,
		CExpression* pPart1, CExpression* vInd1, CExpression* pPart2, CExpression* vInd2,
		CExpression* pPart3, CExpression* vInd3, CExpression* pPart4, CExpression* vInd4,
		CFAModelGenericCpl* inf_const=NULL, // not deleted, but may be changed (particles replaced)
		long nVClassNo=-1 // position in m_VertexClasses (for class vertex)
		);
	bool addIntLineCorr(long pk, CExpression* vCorr, CExpression* mom, CExpression* m, CExpression* vInd1, CExpression* vInd2);
	bool addExtLineCorr(long pk, CExpression* vCorr, CExpression* mom, CExpression* m, CExpression* vInd1);
	long InvolveVertexClass(CFAModelGenericCpl* inf, bool del_inf=true);

private:
	// Vector indices to be used in Hamiltonian
	CExpression* m_HvInd1;
	CExpression* m_HvInd2;
	CExpression* m_HvInd3;
	CExpression* m_HvInd4;

	// Unknown mass symbol, returned by get_mass_sym(wrong PK)
	CESymbol* m_SymUnknownMass;
public:

	bool m_IsInited;
	virtual void init_physics(); // invoked before physics is installed
	virtual CDiagCorrRules* getCorrespondenceRules();
	virtual CExpression* getHamiltonianExpr();
	virtual CExpression* getClassesHamiltonianE(); // NULL if not available
	virtual CExpression* getClassesHamiltonianRen(); // NULL if not available
	virtual CExpression* getRenormHamiltonian();
	virtual CERule* getHReplRule();
	virtual DWORD getPhysicsID();
	virtual bool CounterTermsDeclared(); // called by CMEComposeLib

	virtual CERule* getRenormConstValues();
	virtual CSequence* CollectConstants();
	virtual CEPtrArray* CollectConstantValues(); // array of CExprContainer(), cloned value

	virtual bool validate_part_kind(long _kind);
	virtual const char* get_particle_symbol(long _kind, int sym_style);
	virtual int getCharge3(long pk); // charge * 3 (INT_MIN for unknown)
	virtual CExpression* get_mass_sym(long pk);
	virtual void set_mass_sym(long pk, CExpression* val);
	virtual int getDiagramLineType(long pk_abs); // res==0 means default selection procedure
	virtual long CollectVerticesForClass(long vnumClassVertex, CIntArray* idAltVerts);
	bool GetClassVertexID(long vnum, long* vnumClass, long* vnumClassCT); // returns false of no class vertex are found
};

#endif // !defined(AFX_PHYSMODELFA_H__135CC2BE_616F_40DA_91B5_D74D72086E6F__INCLUDED_)
