////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// SMPhysics.h: interface for the CSMPhysics class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SMPHYSICS_H__5E2D6F3B_6871_4D7F_8A7A_364610F91434__INCLUDED_)
#define AFX_SMPHYSICS_H__5E2D6F3B_6871_4D7F_8A7A_364610F91434__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CSMPhysics : public CDiagramPhysics {
public:
	class CSMPhysClassCplInfo : public CEObject { // used for class-couplings keeping
	public:
		int m_pkConfigID; // 1,2,3,4: pks configurations
		bool m_bIsCt;
		int m_iVecSymmFlags; // 0, 1 (1-2), 2(3-4) or 1|2
		CIntArray m_partPKs;
		CSMPhysClassCplInfo(int _pkConfigID, bool _bIsCt, int iVecSymmFlags=0);
		CSMPhysClassCplInfo(class CSMPhysClassCplInfo& inf);
		~CSMPhysClassCplInfo();
		void SetClassCouplingFor(long pk1,long pk2,long pk3=0,long pk4=0);

		virtual bool contain_vector(void * obj);
		static const char* CLASS_NAME;
		virtual const char* ClassName() const;
		virtual char* toLine(int mode = 0);
		bool equals(CSMPhysClassCplInfo* other);
	};

private:
	// info about particles (int array?)
	// info about vertices (list of empty vertices)

	// correspondence rules
	CDiagCorrRules* m_SM_corr_rules;

	CExpression* m_Hamiltonian_expr;
	CExpression* m_Renorm_Hamiltonian;
	CERule* m_HamiltonianReplsRule;

	DWORD m_phys_ID;

	// indices for Hamiltoian creation (cached to be uniform)
	CESymbol* m_H_vect_index;
	CESymbol* m_H_vect_index_2;
	CESymbol* m_H_vect_index_3;
	CESymbol* m_H_vect_index_4;

	// model constants list
	CEPtrArray* m_model_consts;
	void initModelConstsList();
	void involveModelConst(int id, CExpression* val); // deletes value

	// renormalization constants bank
	CEPtrArray* m_renorm_consts;
	void initRenormConstsList();
	void involveRenormConst(int id, CExpression* val); // deletes value

	void addVertexInfo(int v_num, int pk1, int pk2, int pk3, bool is_renorm_CT = false);

	void addHamExprVFF(int v_num,
		CExpression* vec_core, CExpression* vect_index,
		int pk_a, int pk_f1, int pk_f2, bool is_renorm_CT = false);
	void addCorrRuleAFF(int v_num, CExpression* ind);
	void addCorrRuleGFF_CT(int v_num, CExpression* ind);

	void Add3BosonVertex(int v_num,
		CExpression* vec_core,
		CExpression* vect_index_a, CExpression* vect_index_v1, CExpression* vect_index_v2,
		int pk_a, int pk_v1, int pk_v2, bool is_renorm_CT = false);

	void Add4BosonVertex(int v_num, CExpression* vec_core,
		CExpression* vect_index_v1, CExpression* vect_index_v2,
		CExpression* vect_index_v3, CExpression* vect_index_v4,
		int pk_v1, int pk_v2, int pk_v3, int pk_v4, bool is_renorm_CT = false);
	void addVertex4Info(int v_num, int pk1, int pk2, int pk3, int pk4, bool is_renorm_CT = false);

	void addHamExprASV(int v_num,
		CExpression* vec_core,
		CExpression* vect_index_a, CExpression* vect_index_v,
		int pk_a, int pk_s, int pk_v, bool is_renorm_CT = false);

	CExpression* GetFermCTDeclaration(int pk, CExpression* p, CExpression* m);
public:
	static const DWORD PHYS_PURE_QED;
	static const DWORD PHYS_ELW_FEYN;		// SM(Feyn gauge) with no rare vertices
	static const DWORD PHYS_QCD;
	static const DWORD PHYS_Z_VERTICES;
	static const DWORD PHYS_W_VERTICES;
	static const DWORD PHYS_MUON;
	static const DWORD PHYS_TAU_LEPTON;
	static const DWORD PHYS_SCALARS; // including ghosts
	static const DWORD PHYS_GHOSTS;
	static const DWORD PHYS_ANOMALOUS_CONST;
	static const DWORD PHYS_ANOMALOUS_CONST_NOLCH; // No Levi-Chevita part in anomalous term
	static const DWORD PHYS_ANOMALOUS_CONST_ONLY; // Only anomalous part in XXXX-vertices.
	static const DWORD PHYS_QUARKS_1GEN;
	static const DWORD PHYS_QUARKS_2GEN;
	static const DWORD PHYS_QUARKS_3GEN;
	static const DWORD PHYS_UNITARY_GAUGE;
	static const DWORD PHYS_RENORM_COUNTERTERMS;
	static const DWORD PHYS_NOQED_COUNTERTERMS;
	static const DWORD PHYS_RARE2_VERICES; // unused now! (see RARE2_ALLOWED=true; at init_corr_rules())
	static const DWORD PHYS_RARE3_VERICES; // equivalent to script operand "PHYS_RARE_VERICES"
	static const DWORD PHYS_SM;
	static const DWORD PHYS_EONLY;
	static const DWORD PHYS_NOQUARKS;
	static const DWORD PHYS_Q1GEN_ONLY; // 1 generation of quarks only (cancels PHYS_QUARKS_3GEN, leaves PHYS_QUARKS_1GEN)
	static const DWORD PHYS_Q2GEN_ONLY; // 2 generations of quarks only (cancels PHYS_QUARKS_3GEN, sets PHYS_QUARKS_2GEN)
	static const DWORD PHYS_NO_RARE_VERTICES; // cancels PHYS_RARE3_VERICES (but leaves PHYS_RARE2_VERICES flag)
	static const DWORD PHYS_PROTON_STRUCTFUNC; // including proton & structure functions
	//static const DWORD PHYS_NO_RARE3_VERTICES;
	//static const DWORD PHYS_NO_RARE2_VERTICES;

	//////////////////// model constants
	static const int CONST_E;
	static const int CONST_ALPHA;
	static const int CONST_CW;
	static const int CONST_SW;
	static const int CONST_CTGW;
	static const int CONST_GP; // constant in Z-vertex (-e*Qe*tgW = e tgW)
	static const int CONST_GM; // constant in Z-vertex 
	static const int CONST_NEU; // constant in neutrino vertex (1/(sqrt(2)*sinW))
	static const int CONST_QU; // -1/3 {u,c,t}
	static const int CONST_QD; // 2/3 {d,s,b}
	static const int CONST_GU_P;
	static const int CONST_GU_M;
	static const int CONST_GD_P;
	static const int CONST_GD_M;
	static const int CONST_G_S; // strong interaction
	CExpression* getModelContsSym(int id);
	CExpression* getModelContsUnitFactor(); // returns \alpha*4*pi/e^2 = 1

	//////////////////// renormalization constants
	static const int DZ_AA;
	static const int DZ_ZA;
	static const int DZ_AZ;
	static const int DZ_ZZ;
	static const int DZ_E;
	static const int DZ_F_EL;
	static const int DZ_F_MU;
	static const int DZ_F_TAU;
	static const int DM_F_EL;
	static const int DM_F_MU;
	static const int DM_F_TAU;
	static const int DZ_F_EL_P;
	static const int DZ_F_EL_M;
	static const int DZ_F_MU_P;
	static const int DZ_F_MU_M;
	static const int DZ_F_TAU_P;
	static const int DZ_F_TAU_M;
	static const int DM_F_EL_P;
	static const int DM_F_EL_M;
	static const int DM_F_MU_P;
	static const int DM_F_MU_M;
	static const int DM_F_TAU_P;
	static const int DM_F_TAU_M;
	CExpression* getRenormContsSym(int id);
	virtual CERule* getRenormConstValues();
	virtual CSequence* CollectConstants();
	virtual CEPtrArray* CollectConstantValues(); // array of CExprContainer(), cloned value

	CSMPhysics(DWORD phys_ID);
	virtual ~CSMPhysics();

	virtual void init_physics(); // invoked before physics is installed
	virtual CDiagCorrRules* getCorrespondenceRules();
	virtual CExpression* getHamiltonianExpr();
	virtual CExpression* getRenormHamiltonian();
	virtual CERule* getHReplRule();
	virtual DWORD getPhysicsID();
	virtual bool CounterTermsDeclared(); // called by CMEComposeLib
	//virtual void initPhysics();

	virtual bool validate_part_kind(long _kind);
	virtual const char* get_particle_symbol(long _kind, int sym_style);
	virtual int getCharge3(long pk); // charge * 3 (INT_MIN for unknown)

private:
	CEPtrArray m_particle_mass_symbols;
	CExprContainer* getMassSymbolCart(long pk_abs);
public:
	virtual CExpression* get_mass_sym(long pk);
	virtual void set_mass_sym(long pk, CExpression* val);

public:
	static CERule* createVecReplRule(CEVector* vec, CExpression* subst_expr);
	static CExpression* ReplVectorsRuleProc(CExpression* src, CEPtrArray* data_arr);

	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
private:
	void init_corr_rules();
	void init_H_data();

public:
	// correspondence rules
	CExpression* m_eClassesHamiltonian;
	CExpression* m_eClassesHamiltonianR;
	virtual CExpression* getClassesHamiltonianE(); // NULL if not available
	virtual CExpression* getClassesHamiltonianRen(); // NULL if not available

public:
	/*
	*	m_VertexClasses: array of class-info objects
	*	m_idClassVertices[0,getVerticesInfo()->size()]: position of corresponding CLASS-vertex info in m_VertexClasses, -1 for CLASS vertex
	*	m_vnumClassVertices[0,m_VertexClasses.size()]: reverse for m_idClassVertices for class-vertices
	*	m_vnCTClassVertices[0,m_VertexClasses.size()]: reverse for m_idClassVertices for class-vertices
	*/
	CEPtrArray m_VertexClasses; // arr of CSMPhysClassCplInfo with "+/-" signs specified, used to fill m_eClassesHamiltonianX
	CIntArray m_idClassVertices; // pos corresponding CLASS-vertex info in m_VertexClasses (or -1)
	CIntArray m_vnumClassVertices; // vnum for every item of m_VertexClasses (or -1 if vertex is CT-only)
	CIntArray m_vnCTClassVertices; // vnCT for every item of m_VertexClasses (or -1 if vertex is non-CT-only)
	bool m_bClassVerticesComplete;
	virtual long CollectVerticesForClass(long vnumClassVertex, CIntArray* idAltVerts);
	virtual bool AreCTPropagatorsVertices(); //default: true <== CT are implemented as 2-tale vertices

protected:
	bool InitCouplingsClasses();
	long InvolveVertexClass(long pk1, long pk2, long pk3, long pk4, bool bIsCT, long nVClassNo, int pkConfID, int iVecSymmFlags=0); //pkConfID is caller-ID
	long InvolveVertexClass(CSMPhysClassCplInfo* inf); // inf is unaffected
};

#endif // !defined(AFX_SMPHYSICS_H__5E2D6F3B_6871_4D7F_8A7A_364610F91434__INCLUDED_)
