////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// SModelLoader.h: interface for the CSModelLoader class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SMODELLOADER_H__9BA3748B_C980_4885_9FC0_24DDE9ADA747__INCLUDED_)
#define AFX_SMODELLOADER_H__9BA3748B_C980_4885_9FC0_24DDE9ADA747__INCLUDED_

//Do not use next mechanism! Inf vertex * 1/Inf propagator is allowed! TODO: fix it!
//#define LOADMODEL_DROP_ZERO_PROPS

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CSModelLoader : public CEAbstractSession
{
	CMathFileParser* m_ParserGen;
	CMathFileParser* m_ParserModel;
	CSModelLoader(const char* _GenBuf, const char* _ModelBuf);

	bool m_bGenericPropagatorsFilling;
	bool m_bGenericCouplingsFilling;
	bool m_bCouplingMatricesFilling;
	bool m_bClassesDescriptionFilling;
	bool m_bFormatParameterFilling;

	CPhysModelFA* m_phys; // volatile: physics to be filed & installed
public:
	const char* m_GenBuf;
	const char* m_ModelBuf;

public:
	// used externally by CPhysModelFA
	CEPtrArray m_KinIndices; // (CESymbol*):{{F}{V,Lorentz},{S},{SV,Lorentz},{U}}
	CSequence m_GenericPropagators; // {1,X,Val}:internal, {0,X,Val}:external
	CEPtrArray m_GenericCouplings; // array of CFAModelGenericCpl
	CSequence m_CouplingMatrices;
	CSequence m_ClassesDescription;
	CSequence m_IndexRanges; // arr of CSequence { Index[Generation],{1,2,3}}

#ifdef LOADMODEL_DROP_ZERO_PROPS //Do not use this mechanism! Inf vertex * 1/Inf propagator is allowed! TODO: fix it!
	CIntArray m_ZeroPropagatorPKs; // particles with ZERO propagator -> to drop out adjacent vertices
	bool bIsZeroPropagatorPK(long pk);
#endif //LOADMODEL_DROP_ZERO_PROPS

	// TODO: unify m_Variables and m_MathEquRules!
	CEPtrArray m_Variables; // arr of CPtrArray {var, CScriptValue*}
	CEPtrArray m_MathEquRules; // arr of CEMathEquRule*
	//CESimpleMap m_mapVariables; // map of CPtrArray {var, CScriptValue*}
public:
	virtual ~CSModelLoader();
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	static CDebug* dbg;

public:
	static bool InstallModelFile(const char* _GenFile, const char* _ModelFile);
	static CSModelLoader* LoadModel(const char* _GenBuf, const char* _ModelBuf);
	CEString m_MsgLastError;

public:
	static CEPtrArray* m_PreDefValues; // array of {(expr)par, (expr)value}, deleted by CEProcessor::AddToDeleteList() engine
	static void setModelParameter(const char* par, CExpression* val_const);

private:
	bool DoLoadModel();
	bool DoInitPhysics(); // m_phys is inited here

private:
	///////////////////////////////////////////////////////////// parser
	// string execute variables
	CScriptValue* m_whole_value;
	CEPtrArray* m_LeftValues; // a=b=... -> {a,b}, array of (CScriptValue*)
	CScriptValue* m_curr_operand_value;
	//CEPtrArray* m_last_func_values;
	//CEMap* m_map_locals;

	DWORD m_line_brake_flags; // volatile variable, indicates line break conditions
	static const DWORD LINE_BRAKE_EOF;
	static const DWORD LINE_BRAKE_RIGHT_BRACE;
	static const DWORD LINE_BRAKE_PRIORITY;
	static const DWORD LINE_BRAKE_ANY_FLAG;

	int m_line_priority; // 0 for usual line, >0 for operations, sets by DoLoadModelLine(X,...) call
	/*
	*	The right bracket will be skipped for every (m_line_priority==0) lines ecxept of m_bDontSkipRBrace flag is set.
	*/
	bool m_bDontSkipRBrace;

	void declare_script_error(int err_code = 0, int i_param = -1, const char* c_param = NULL);
	CMathFileParser* m_parser; // current parser
	CScriptValue* DoLoadModelLine(int line_priority = 0, CScriptValue* _whole_val = NULL);

	void do_operation_involve(const char* c_op, int op_type);
	void update_string_value();
	CExpression* expr_operation_binary(const char* c_op, CExpression* _whole, CExpression* _current);
	CExpression* expr_operation_unitary(const char* c_op, CExpression* _current);
	int getOperandPriority(const char* c_op, int op_type);
	void clear_line_break_flag();
	void lefthand_operation_execute();
	bool sysEquOpExec(CScriptValue* left, CScriptValue* right);
	void do_variable_involve(const char* c_op, int op_type);
	//bool do_function_cast(const char* c_op, int op_type);

	int getFuncSpecInfo(const char* c_op); // params num, -1 for unknown function, INT_MAX for any
	int getFuncSpecInfo2(const char* c_op); // second-[] params num, INT_MAX for any, INT_MIN if unused
	bool bNeedEqualsConvert(const char* c_op); // do not treat == as = in parameters
	bool bIsFuncCommutative(const char* c_op); // do not treat == as = in parameters

	CScriptValue* getFuncSpecValue(const char* c_op, CSequence* params, CSequence* params2=NULL); // params are unaffected

	bool AddFieldInfo(CPhysModelFA::CFAModelGenericCpl* inf, CExpression* src);

	bool InitParticleList(); // fills m_phys, called in DoInitPhysics()
	bool InitPropagatorsList(); // fills m_phys, called in DoInitPhysics()
	bool InitCouplingsClasses(); // fills m_phys, called in DoInitPhysics()
	bool InitCouplingsList(); // fills m_phys, called in DoInitPhysics()

private:
	CExpression* ExtractPolVector(CExpression* ext_prop,
		const char* vec_id, CExpression* num, CExpression* mom, CExpression* ind); // deletes source, check parameters, NULL if failed
	bool ComparePolVecParams(CEFunction* pol_vec,
		const char* vec_id, CExpression* num, CExpression* mom, CExpression* ind); // not affects source, func_name is already checked

private:
	void SetVariableValue(CExpression* var, CScriptValue* val); // var should be "+"-signed and leveled, does not affects input
	CScriptValue* GetVariableValue(CExpression* var); // var should be "+"-signed and leveled
	CExpression* GetMathRuleValueCloned(CExpression* var); // any signed input
	CExpression* GetVarRuleValueECloned(CExpression* var); // any signed input

	CSequence* FindClassDescriptionSeq(const char* idName, int idNumber);
	// ParseDescSequence:deletes previous (*eMassOrig) and (*ePropLabel) values if changed
	bool ParseDescSequence(CSequence* descSeq, CSequence* sqQNumbers, CSequence* sqIndices, CExpression** ePropLabel, CExpression** eMassOrig, int* iPropArrow, int* iPropType, int* iSelfConj);
	bool InitParticleInfo(CPhysModelFA::CFAPatricleClassInfo* inf); // init pk and mass symbol

public:
	static CExpression* ExtractConditionsRProc(CExpression* src, CEPtrArray* data_arr);
	static CExpression* EvaluateComposedChar(CExpression* src, int mode, bool del_src);
	static CExpression* EvaluateComposedCharRProc(CExpression* src, CEPtrArray* data_arr);
	static CExpression* EvaluateIndexDeltaRProc(CExpression* src, CEPtrArray* data_arr);
	static CExpression* EvaluateCouplRProcs(CExpression* src, CEPtrArray* data_arr);
	static CExpression* EvaluateSubstRules(CExpression* src, CEPtrArray* data_arr); // m_Variables & m_MathEquRules
};

#endif // !defined(AFX_SMODELLOADER_H__9BA3748B_C980_4885_9FC0_24DDE9ADA747__INCLUDED_)
