////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EFieldOperator.h: interface for the CEFieldOperator class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EFIELDOPERATOR_H__3C26C308_4F2A_4037_AB93_E843ABDDCCD8__INCLUDED_)
#define AFX_EFIELDOPERATOR_H__3C26C308_4F2A_4037_AB93_E843ABDDCCD8__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CEFieldOperator : public CEParticleFunction
{
protected:
	int m_coupling_no; // coupled to operator with same number
	bool m_is_CT_coupling; // is one-loop counterterm coupling (ignored for uncoupled field)
	bool m_is_conjugated; // unused for neutral particles

protected:
	CEFieldOperator(long _particle_kind,
		bool _is_conjugated,
		int _vertex_ID,
		unsigned long _point_ID,
		int _coupling_no, // 0 means operator is not part of a coupling
		bool _is_CT_coupling // truncated for uncoupled field
		);

	CEFieldOperator(long _particle_kind,
		bool _is_conjugated,
		int _vertex_ID,
		unsigned long _point_ID
		);
public:
	static CDebug* dbg;
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual CEString* toString(int mode = 0);
	virtual void collectProperties(CEProperties* props, unsigned set = CEProperties::PROP_MASK_ALL);
	virtual unsigned long props_long();
	virtual CExpression* conjugated(int mode = 0);
	virtual bool equals(CExpression *other);
	virtual ~CEFieldOperator();

	virtual long getLogicalPK();
	virtual bool isConjugated();
	virtual bool MayBeCoupledWith(CEFieldOperator* fo);
	int getCouplingNumber();
	bool isCoupled();
	bool isCounterTermCoupling();
	void setCounterTermCoupling(bool _CT_coupling);
	void setCouplingNo(int _coupling_no); // requires check MayBeCoupledWith() before use
	virtual bool CanBeCoupledExternal(CEParticle* part, bool is_initial, bool bCheckClassesOnly) = 0;
	virtual CExpression* GetExtParticleCoupling(CEParticle* part, bool is_initial, bool bClassesSource) = 0;
};

#endif // !defined(AFX_EFIELDOPERATOR_H__3C26C308_4F2A_4037_AB93_E843ABDDCCD8__INCLUDED_)
