////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// OpBosonField.h: interface for the COpBosonField class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_OPBOSONFIELD_H__0F01CF82_6E4E_4E2C_B1C8_619CB83234F7__INCLUDED_)
#define AFX_OPBOSONFIELD_H__0F01CF82_6E4E_4E2C_B1C8_619CB83234F7__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class COpBosonField : public CEFieldOperator  
{
	CTIndex* m_vector_index;

protected:
	// Note: NULL vector index means scalar particle wave function
	COpBosonField(long _particle_kind,
		bool _is_conjugated,
		CTIndex* _vector_index,
		int _vertex_ID,
		unsigned long _point_ID,
		int _coupling_no, // 0 means operator is not part of a coupling
		bool _is_CT_coupling // truncated for uncoupled field
		);

public:
	// Note: NULL vector index means scalar particle wave function
	COpBosonField(long _particle_kind,
		CExpression* _vector_index, // does not affect the input expression
		bool _vector_index_cov,
		int _vertex_ID,
		unsigned long _point_ID = 0 // default for the abstract hamiltonian
		);
	virtual ~COpBosonField();

	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual bool contain_vector(void * obj);
	virtual CEString* toString(int mode = 0);
	virtual void collectProperties(CEProperties* props, unsigned set = CEProperties::PROP_MASK_ALL);
	virtual unsigned long props_long();
	virtual CExpression* clone() const;
	virtual bool equals(CExpression *other);
#ifdef PF_LESS_THAN
	virtual bool less_than(CExpression* other);
#endif //PF_LESS_THAN
	virtual CExpression* applyRule(CEAbstractRule* rule);

	CTIndex* getVectorIndex(); // Note: NULL vector index means scalar particle wave function
	bool isVectorBoson(); // (getVectorIndex() != NULL)
	virtual bool MayBeCoupledWith(CEFieldOperator* fo);
	virtual bool CanBeCoupledExternal(CEParticle* part, bool is_initial, bool bCheckClassesOnly);
	virtual CExpression* GetExtParticleCoupling(CEParticle* part, bool is_initial, bool bClassesSource);
};

#endif // !defined(AFX_OPBOSONFIELD_H__0F01CF82_6E4E_4E2C_B1C8_619CB83234F7__INCLUDED_)
