////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// FeynDiagramFactory.h: interface for the CFeynDiagramFactory class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_FEYNDIAGRAMFACTORY_H__78E0D8E9_2FDE_4904_BE47_4769B63341CD__INCLUDED_)
#define AFX_FEYNDIAGRAMFACTORY_H__78E0D8E9_2FDE_4904_BE47_4769B63341CD__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CFeynDiagramFactory : public CELibrary  
{
	CDiagramPhysics* m_phys;
	int m_order;
	CKinematics* m_kin;
	CDiagramSet* m_diagrams;

	int m_COMPOSE_SETTINGS;
public:
	static const int COMPOSE_SETTINGS_BIND_TO_NEXT;
	static const int COMPOSE_SETTINGS_BIND_TO_FIRST;
	static const int COMPOSE_SETTINGS_VERTEX_ORDER;
	static const int COMPOSE_SETTINGS_NO_UNBINDED;

private:
	CFeynDiagramFactory(CDiagramPhysics* _phys, int _order, CKinematics* _kin, int alg_setts = 0);
	virtual ~CFeynDiagramFactory();

	CDiagramSet* compose_diagrams_impl();

	// engine functions
	void develop_diagram(CFeynDiagram* diag);
	void develop_last_vertex(CFeynDiagram* diag);

	// utilites
	int find_first_unbinded_line(CFeynDiagram* diag, int part_kind);
	bool can_vertex_be_binded(CFeynDiagram* diag, CDiagramVertexInfo* vi);
	bool check_setting(int setting);

public:
	static CDebug* dbg;

public:
	/*
	*	Filter: CLDiagTopologyInspector::"NON_EULER_CIRCUIT | LOOPS_IN_EXTERNAL_LINES | ODD_LINEAR_FERMION_LOOP"
	*/
	static CDiagramSet* ComposeAndFilterDiagrams(int order, int alg_setts = 0);
	static CDiagramSet* compose_diagrams(int order, int alg_setts = 0);
	static CDiagramSet* compose_diagrams(CDiagramPhysics* phys, int order, int alg_setts = 0);
};

#endif // !defined(AFX_FEYNDIAGRAMFACTORY_H__78E0D8E9_2FDE_4904_BE47_4769B63341CD__INCLUDED_)
