////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// HEPLib.cpp: implementation of the CHEPLib class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

CDebug* CHEPLib::dbg = new CDebug("CHEPLib");
bool CHEPLib::m_do_ur_reduce = true;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CHEPLib::CHEPLib()
{

}

CHEPLib::~CHEPLib()
{

}

/*
* me_b == NULL will result in squaring |&me_a|.
*/
/*//[unused]
CExpression* CHEPLib::couple_me(CExpression *me_a, CExpression *me_b, bool do_delete_input)
{
	if (me_a == NULL) {
		if (do_delete_input) {
			if (me_a != NULL) delete me_a;
			if (me_b != NULL) delete me_b;
		}
		return NULL;
	}

	if (me_b == NULL) me_b = me_a->clone();

	CExpression* b_conj = NULL;//CMESquareLib::change_polarization_indices(CEBasic::Congugate(me_b->clone()));
	CExpression* res = me_a->clone()->multiply(b_conj);

	if (do_delete_input) {
		if (me_a != NULL) delete me_a;
		if (me_b != NULL) delete me_b;
	}

	dbg->log(20, "ME coupling is", res);
	res = CExpression::reduce(res);
	dbg->log(30, "reduced", res);

	CExpression* res3 = CDiracAlgebra::simplifyGammaSequence(res, NULL, NULL);
	delete res;
	res = res3;
	dbg->log(30, "simplified", res);

	sum_spins_adv(res);
	dbg->log(30, "spins summarized", res);

	//res = CExpression::reduce(res);
	//dbg->log(10, "reduced", res);

	////CExpression* res2 = CDiracAlgebra::simplifyGammaSequence(res, NULL, NULL);
	////delete res;
	////res = res2;
	////dbg->log(10, "simplified", res);

	CExpression* res2 = res->evaluate();
	if (res2 != res) delete res;
	res = res2;
	dbg->log(30, "spurs evaluated", res);

	res = CExpression::reduce(res);
	//res = CExpression::reduce(res);
	dbg->log(10, "reduced", res);

	return res;
}
*/

//CExpression* CHEPLib::sum_fermion_spins(CEProduct *seq)
void CHEPLib::sum_fermion_spins(CEProduct *seq)
{
	//Deprecated!
	dbg->log(0,"[Error] deprecated code involved: CHEPLib::sum_fermion_spins()");
	dbg->log(0,"Use CLFermionSpinSum::sum_fermion_spins() instead!");
	CEException::halt();
	//
	dbg->log(45, "sum spins", seq);
	long i;
	for(i=0; i<(seq->size()-1); i++) {
		CExpression* item = seq->GetAt(i);
		if (item->getProperty(CEProperties::PROP_MASK_IS_SPINOR)) {
			CESpinor* sp = (CESpinor*)item;
			long j;
			for(j=(i+1); j<seq->size(); j++) {
				item = seq->GetAt(j);
				if (item->getProperty(CEProperties::PROP_MASK_IS_SPINOR)) {
					CESpinor* sp2 = (CESpinor*)item;

					if ((sp->base()->equals(sp2->base())) &
						(sp->is_conjugated()) &
						(!sp2->is_conjugated())) {

						CExpression* sub_seq = seq->extract_items(i+1, j);

						//if (sub_seq == NULL) {}
						sub_seq = sub_seq->multiply(sp->clone());
						sub_seq = CExpression::reduce(sub_seq);

						CExpression* sp_sub_seq = new CFSp(sub_seq);
						seq->SetAt(i, sp_sub_seq);

						delete sp_sub_seq;
						delete sub_seq;

						//dbg->log(30, "seq", seq);

						j = seq->size();
					}
				}
			}
		}
	}
	dbg->log(45, "sum spins result", seq);
	//return seq;
}

void CHEPLib::sum_spins_adv(CExpression *source) {
	dbg->log(45, "sum spins (adv)", source);
	if (source->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
		sum_fermion_spins((CEProduct*)source);
	}
	else if (source->getProperty(CEProperties::PROP_MASK_IS_SUM)) {
		CESum* sum = (CESum*)source;

		long s = sum->size();
		long i;
		for (i=0; i<s; i++) { sum_spins_adv(sum->GetAt(i)); }
	}
}

/*
//[unused]
CExpression* CHEPLib::Calc_V_term(CExpression *born_me)
{
	CExpression* box_me = NULL;
	CExpression* box_dens = NULL;
	CEPtrArray* diagrams = NULL;//CFeynDiagramFactory::create_diagrams(4);
	long i;
	for(i=0; i<diagrams->size(); i++) {
		CFeynDiagram* diag = (CFeynDiagram*)diagrams->get(i);
		CESupport::savetex(diag);

		CExpression* me = NULL;//diag->get_me();
		dbg->log(10, "me", me);
		CExpression* dens = ((CEProduct*)me)->extract_denominator();
		dbg->log(10, "me", me);
		dbg->log(10, "dens", dens);

		//if (i == 0) CDiracAlgebra::dbg->set_level(40);
		me = CDiracAlgebra::simplify(me);
		//if (i == 0) CDiracAlgebra::dbg->set_level(1);
		dbg->log(10, "me simplified", me);

		me = to_ultrarev(me);
		dbg->log(10, "ultra-rev", me);


		if (i == 3) {
			box_me = me;
			box_dens = dens;
		}
		else {
			delete me;
			delete dens;
		}
	}
	delete diagrams;

	CExpression* res = Calc_Box(box_me, box_dens, born_me);
	delete box_me;
	return res;
}
*/

CExpression* CHEPLib::to_ultrarev(CExpression *source, bool log_reducing)
{
	CESymbol* m = new CESymbol("m\\_f");
	CENumber* z = CENumber::GetZero();
	CEReplList* rule = new CEReplList(m, z, false);
	source->applyRule(rule);
	delete rule;
	delete m;
	delete z;

	dbg->log(30, "U-rev (before reduction)", source);

	if (log_reducing) {
		CEProduct::dbg->set_level(50);
		CESum::dbg->set_level(50);
	}
	CExpression* res = CExpression::reduce(source);
	if (log_reducing) {
		CEProduct::dbg->set_level(1);
		CESum::dbg->set_level(1);
	}
	//dbg->log(20, "ultrarev", res);
	return res;
}

CExpression* CHEPLib::repl_kin_invariants(CExpression *source, bool do_reduce)
{
	CEReplList* rule = CEProcessor::kinematics()->getReplacements(CKinematics::REPL_MANDELSTAM_VARIABLES);
	if (rule != NULL) {
		source->applyRule(rule);
		dbg->log(20, "Rules involved:", rule);
		delete rule;
		if (do_reduce) {
			dbg->log(45, "Result (after rule involved):", source);
			CExpression* res = CExpression::reduce(source);
			dbg->log(45, "Reduced (after rule involved):", res);
			return res;
		}
		
		return source;
	}
	dbg->log(1, "Failed to involve kinematic invariants.");
	return source;
}

CExpression* CHEPLib::repl_similar_kin(CExpression* source, bool do_reduce)
{
	CEReplList* rule = CEProcessor::kinematics()->getReplacements(CKinematics::REPL_COUPLINGS_SIMPLIFY);
	if (rule == NULL) {
		dbg->log(0, "Failed to involve kinematic invariants.");
		return source;
	}
	
	dbg->log(45, "Result (before rule involved):", source);
	source = rule->ApplyTo(source);
	//source->applyRule(rule);
	dbg->log(20, "Rules involved:", rule);
	delete rule;
	if (do_reduce) {
		dbg->log(45, "Result (after rule involved):", source);
		source = CExpression::reduce(source);
		dbg->log(45, "Reduced (after rule involved):", source);
	}
	
	return source;
}

/*
//[unused]
CExpression* CHEPLib::Calc_Born_Me() {
	
	CExpression* res = NULL;
	CEPtrArray* diagrams = NULL;//CFeynDiagramFactory::create_diagrams(2);
	long i;
	for(i=0; i<diagrams->size(); i++) {
		CFeynDiagram* diag = (CFeynDiagram*)diagrams->get(i);
		//CESupport::Save(diag->analyze());
		CESupport::savetex(diag);

		CExpression* me = NULL;//diag->get_me();
		//CESupport::SaveResults(me);

		if (res == NULL) res = me;
		//else res = res->add(me);//*******************************************************!!!!!!!
	}
	delete diagrams;

	dbg->log(20, "Born-level ME calculated", res);
	return res;
}
*/

/*
//[unused]
CExpression* CHEPLib::Calc_Born(CExpression* born_me) {

	if (born_me == NULL) return NULL;

	CExpression* me_simpl = CDiracAlgebra::simplifyGammaSequence(born_me, NULL, NULL);
	CExpression* res = couple_me(me_simpl, NULL);

	dbg->log(10, "Born-level ME squared", res);

	res = repl_kin_invariants(res, false);
	dbg->log(10, "invariants substituted", res);

	res = to_ultrarev(res);
	dbg->log(20, "ultrarev", res);
	return res;
}
*/

CExpression* CHEPLib::ur_reduce(CExpression *val)
{
	if (m_do_ur_reduce) val = to_ultrarev(val);
	val = repl_kin_invariants(val, false);
	if (m_do_ur_reduce) val = to_ultrarev(val);
	return val;
}

/*
//[unused]
CExpression* CHEPLib::Calc_Box(CExpression *box_me, CExpression *box_dens, CExpression *born_me)
{
	CExpression* box = couple_me(box_me, born_me, false);
	box = to_ultrarev(box);
	box = repl_kin_invariants(box, false);
	box = to_ultrarev(box);
	box = CExpression::reduce(box);
	dbg->log(10, "box", box);
	box = box->multiply(box_dens);

	dbg->log(10, "box*dens", box);

	CESymbol* k = new CESymbol("k");
	CExpression* res = CVIComposeLib::composeVI(box, k);
	delete k;

	dbg->log(10, "V-term res", res);
	delete box;
	return res;
}
*/

bool CHEPLib::ur_reduce_perform(bool do_UR)
{
	bool old = m_do_ur_reduce;
	m_do_ur_reduce = do_UR;
	return old;
}

CExpression* CHEPLib::get_mass_correction(CExpression *source)
{
	CExpression* source_UR = to_ultrarev(source->clone());
	CExpression* res = source->clone()->minus(source_UR);
	res = CExpression::reduce(res);
	return res;
}

CExpression* CHEPLib::get_rule_difference(CExpression *source, CERule *rule)
{
	CExpression* res = source->clone();
	res = rule->ApplyTo(res);
	res = res->minus(source);
	res = CExpression::reduce(res);
	return res;
}
