////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// MESeqFORoute.cpp: implementation of the CMESeqFORoute class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CMESeqFORoute::CMESeqFORoute() : CEPtrArray() {}
CMESeqFORoute::~CMESeqFORoute() {
	int route_len = size();
	int i;
	for(i=0; i<route_len; i++) {
		CEPtrArray* fo_pair = (CEPtrArray*)GetAt(i);
		fo_pair->clearall_without_delete_objects();
	}
	// TODO: check up if basic class dectructor is properly called
}

const char* CMESeqFORoute::CLASS_NAME = "CMESeqFORoute";
const char* CMESeqFORoute::ClassName() const { return CLASS_NAME; }
CEObject* CMESeqFORoute::copy() const { return cloneRoute(); }

// note: FO-objects may not be cloned
CMESeqFORoute* CMESeqFORoute::cloneRoute() const {
	CMESeqFORoute* res = new CMESeqFORoute(); // note: iterator-engine is stopped here
	int route_len = size();
	int i;
	for(i=0; i<route_len; i++) {
		//Note: UNSAFE conversion is used here
		CEPtrArray* fo_pair = (CEPtrArray*)GetAtConst(i);
		CEPtrArray* fo_pair_copy = new CEPtrArray();
		fo_pair_copy->add(fo_pair->GetAt(0));
		fo_pair_copy->add(fo_pair->GetAt(1));
		res->add(fo_pair_copy);
	}
	return res;
}

void CMESeqFORoute::addPair(CEFieldOperator* fo_start, CEFieldOperator* fo_end) {
	if ((fo_start == NULL) || (fo_end == NULL)) CEException::halt("CMESeqFORoute::addPair(NULL)");
	CEPtrArray* fo_pair = new CEPtrArray();
	fo_pair->add(fo_start);
	fo_pair->add(fo_end);
	add(fo_pair);
}

void CMESeqFORoute::setPair(int i, CEFieldOperator* fo_start, CEFieldOperator* fo_end) {
	if ((fo_start == NULL) || (fo_end == NULL)) CEException::halt("CMESeqFORoute::addPair(NULL)");
	if ((i<0) || (i>=size())) CEException::halt("CMESeqFORoute::setPair(invalid position)");

	CEPtrArray* fo_pair = (CEPtrArray*)GetAt(i);
	if (fo_pair->size() != 2) CEException::halt("CMESeqFORoute structure ERROR");
	fo_pair->SetAt(0, fo_start, false);
	fo_pair->SetAt(1, fo_end, false);
}

bool CMESeqFORoute::equalPairs(int i1, CEPtrArray* fo_pair2) {
	CEPtrArray* fo_pair1 = (CEPtrArray*)GetAt(i1);
	if ((fo_pair1 == NULL) || (fo_pair2 == NULL)) { CEBasic::dbg->log(0, "{from CMESeqFORoute} WARNING: equalPairs(NULL)"); return false; }

	CEFieldOperator* fo1 = (CEFieldOperator*)fo_pair1->GetAt(0);
	CEFieldOperator* fo2 = (CEFieldOperator*)fo_pair2->GetAt(0);
	if (!fo1->equals(fo2)) return false;

	fo1 = (CEFieldOperator*)fo_pair1->GetAt(1);
	fo2 = (CEFieldOperator*)fo_pair2->GetAt(1);
	if (!fo1->equals(fo2)) return false;
	return true;
}

void CMESeqFORoute::addPairCopyFrom(CMESeqFORoute* r, int pos, bool swap_pair) {
	if (r == NULL) CEException::halt("CMESeqFORoute::addPairCopyFrom(NULL)");
	CEPtrArray* fo_pair = (CEPtrArray*)r->GetAt(pos);
	if (fo_pair == NULL) CEException::halt("CMESeqFORoute::addPairCopyFrom(NULL pair)");
	if (fo_pair->size() != 2) CEException::halt("CMESeqFORoute structure ERROR");

	if (swap_pair) {
		addPair((CEFieldOperator*)fo_pair->GetAt(1), (CEFieldOperator*)fo_pair->GetAt(0));
	} else {
		addPair((CEFieldOperator*)fo_pair->GetAt(0), (CEFieldOperator*)fo_pair->GetAt(1));
	}
}

bool CMESeqFORoute::haveCommonLineWith(CMESeqFORoute* other) {
	if (other == NULL) CEException::halt("CMESeqFORoute::haveCommonLineWith(NULL)");
	int s = size();
	int os = other->size();
	int i;
	for(i=0; i<s; i++) {
		CEPtrArray* pair_I = (CEPtrArray*)GetAt(i);
		if (pair_I->size() != 2) CEException::halt("CMESeqFORoute structure ERROR");
		CEFieldOperator* fo_I1 = (CEFieldOperator*)pair_I->GetAt(0);
		CEFieldOperator* fo_I2 = (CEFieldOperator*)pair_I->GetAt(1);
		int j;
		for(j=0; j<os; j++) {
			CEPtrArray* pair_J = (CEPtrArray*)other->GetAt(j);
			if (pair_J->size() != 2) CEException::halt("CMESeqFORoute structure ERROR");
			CEFieldOperator* fo_J1 = (CEFieldOperator*)pair_J->GetAt(0);
			CEFieldOperator* fo_J2 = (CEFieldOperator*)pair_J->GetAt(1);

			if ((fo_I1->equals(fo_J1)) || (fo_I2->equals(fo_J2)) ||
				(fo_I1->equals(fo_J2)) || (fo_I2->equals(fo_J1))) return true;
		}
	}
	return false;
}

// no == 0,1
CEFieldOperator* CMESeqFORoute::getFO(int i, int no) {
	if ((i<0) || (i>=size()) || (no < 0) || (no > 1)) CEException::halt("CMESeqFORoute::getFO(Invalid params)");

	CEPtrArray* pair = (CEPtrArray*)GetAt(i);
	if (pair->size() != 2) CEException::halt("CMESeqFORoute structure ERROR");
	CEFieldOperator* res  = (CEFieldOperator*)pair->GetAt(no);
	return res;
}
