////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// VIDensData.cpp: implementation of the CVIDensData class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//#define DENSDATA_OLD_FORMAT_READ

const char* CVIDensData::CLASS_NAME		= "CVIDensData";
const char* CVIDensData::CLASS_TAG_NAME	= "dd";

CDebug* CVIDensData::dbg = new CDebug("CVIDensData");
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CVIDensData::CVIDensData(CSequence* ddata, int _dens_num) : CSequence() {
	copy_from(ddata);
	//m_is_invariant = false;
	m_dens_num = _dens_num;
}

#ifdef USE_PLAIN_ARRAY_FORSEQ // biuld sequence-expressions on CAPtrArray* basis
CVIDensData::CVIDensData(CEPtrArray* items, int _dens_num) : CSequence(items) {
	//m_is_invariant = _is_invariant;
	m_dens_num = _dens_num;
}
#else
CVIDensData::CVIDensData(CAPtrSeq* items, int _dens_num) : CSequence(items) {
	//m_is_invariant = _is_invariant;
	m_dens_num = _dens_num;
}
#endif //USE_PLAIN_ARRAY_FORSEQ

CVIDensData::~CVIDensData() {}
const char* CVIDensData::ClassName() const { return CLASS_NAME; }

CExpression* CVIDensData::clone() const {
	CVIDensData* res = new CVIDensData(getMemberListConst()->clone(), m_dens_num);
	copyAttributesTo(res);
	return res;
}

bool CVIDensData::contain_vector(void * obj) {
	if (CSequence::contain_vector(obj)) return true;
	return false;
}

CExpression* CVIDensData::applyRule(CEAbstractRule* rule) {
	CERule* r = (CERule*)rule;

	bool allow_rule = true;
	// do not apply m-->0 rules
	if (CEUtil::equals(r->ClassName(), CEReplasement::CLASS_NAME)) {
		if(CEBasic::isZero(((CEReplasement*)r)->b())) { allow_rule=false; }
	}
	
	CExpression* res = this;
	if(allow_rule) { res=CSequence::applyRule(r); }
	return res;
}

/*
*	Creates Veltmann-like representation for DENSDATA object.
*/
CEString* CVIDensData::toString(int mode) {
	//CEString* res = new CEString("\\((");
	CEString* res = new CEString("{");
	int s = size();
	int i=0;

	if (isInvariant()) {
		for(i=0; i<s; i++) {
			if (i > 0) res->AddString(", ");
			CExpression* item = GetAt(i);
			res->AddString(item->toString(mode));
		}
	} else {
		bool prev_exist = false;
		for(i=0; i<s; i++) {
			if (prev_exist) res->AddString(", ");
			CExpression* mom = momentum(i);
			if (i==0) {
				if (!CEBasic::isZero(mom)) {
					res->AddString(mom->toString(mode));
					prev_exist = true;
				}
			}
			else {
				res->AddString(mom->toString(mode));
				prev_exist = true;
			}
		}
		
		for(i=0; i<s; i++) {
			if (prev_exist) res->AddString(", ");
			res->AddString(mass(i)->toString(mode));
			prev_exist = true;
		}
	}

	//res->AddString(")\\)");
	res->AddString("}");
	return res;
}

const char* CVIDensData::get_TAG_Name() { return CLASS_TAG_NAME; }
long CVIDensData::store_stream(ofstream *p_ofs, DWORD settings) {
	long res = 0;

	// store invariance data
	CENumber* dn = new CENumber(m_dens_num);
	res += CESupport::store_stream(dn, p_ofs, settings);
	delete dn;
	
	int s = size();
	int i;
	for (i=0; i<s; i++) {
		res += CESupport::store_stream(GetAt(i), p_ofs, settings);
	}
	return res;
}

CESerializable* CVIDensData::restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings) {

	long skip_pos = pos;
	CESerializable* item = NULL;
	int _dens_num = -1;

#ifndef	DENSDATA_OLD_FORMAT_READ
	// comment next for old-format read
	skip_pos = CESupport::skip_item_in_buffer(data, pos, settings);
	if (skip_pos <= 0) { dbg->log(0, "Error restoring CVIDensData: invalid do data found"); return NULL; }
	item = CESupport::read_buffer(data, pos, settings);
	pos = skip_pos;
	if (item == NULL) { dbg->log(0, "Error restoring CVIDensData from buffer: NULL data."); return NULL; }
	if (CEUtil::equals(item->ClassName(), CENumber::CLASS_NAME)) {
		CENumber* dn = (CENumber*)item;
		if (!dn->is_integer()) { dbg->log(0, "Error restoring CVIDensData from buffer: integer data expexted", dn); return NULL; }
		_dens_num = dn->getLong();
		delete dn;

	} else { dbg->log(0, "Error restoring CVIDensData from buffer: integer data expexted", item); return NULL; }
	// <END> comment next for old-format read
#endif

	bool inv_load = (_dens_num > 0);
	CSequence* res_seq = new CSequence();
	for(;;) {
		skip_pos = CESupport::skip_item_in_buffer(data, pos, settings);
		if (skip_pos <= 0) break;
		
		item = CESupport::read_buffer(data, pos, settings);
		pos = skip_pos;

		if (inv_load) {
			if (!CEBasic::isExpression(item)) {
				dbg->log(0, "Error restoring CVIDensData from buffer: expression data expected", item);
				if (item != NULL) delete item;
				delete res_seq; return NULL;
			}
			res_seq->addItem((CExpression*)item);
		} else {
			if ((item == NULL) || (!CEUtil::equals(item->ClassName(), CSequence::CLASS_NAME))) {
				dbg->log(0, "Error restoring CVIDensData from buffer: invalid data", item);
				if (item != NULL) delete item;
				delete res_seq; return NULL;
			}

			// check up sequence
			CSequence* item_seq = (CSequence*)item;
			if (item_seq->size() != 2) {
				dbg->log(0, "Error restoring CVIDensData: invalid (p, m) sequence loaded:", item_seq);
				delete item_seq; delete res_seq; return NULL;
			}
			
			res_seq->addItem(item_seq);
		}
	}

	if (inv_load) { // size check up
		if (2*res_seq->size() != _dens_num*(_dens_num+1)) {
			dbg->log(0, res_seq->size(), _dens_num*(_dens_num+1)/2, "DENSDATA loading ERROR: illegal size (%ld, required size is %ld)");
			delete res_seq; return NULL;
		}
	}

	CVIDensData* res = new CVIDensData(res_seq, _dens_num);
	delete res_seq;
	return res;
}

//////////////////////////////////////////////////////////////////////
// Data Access Operations
//////////////////////////////////////////////////////////////////////
bool CVIDensData::isInvariant() { return (m_dens_num > 0); }
int CVIDensData::getDensNumber() {
	if (m_dens_num > 0) return m_dens_num; // for invariant form
	return size(); // for non-invariant form
}

CExpression* CVIDensData::momentum(int i) {
	if (isInvariant()) { dbg->log(0, "ERROR momentum() call for invariant DENSDATA"); CEException::halt(); }

	CSequence* seq = (CSequence*)GetAt(i);
	if (seq == NULL) return NULL;
	return seq->GetAt(0);
}

CExpression* CVIDensData::mass(int i) {
	CExpression* res = NULL;
	if (isInvariant()) {
		if ((i < 0) || (i >= m_dens_num)) { dbg->log(0, i, "ERROR mass(%ld) invalid parameter", this); CEException::halt(); }
		int mass_i_offset = (size()-m_dens_num+i);
		res = GetAt(mass_i_offset);
	} else {
		CSequence* seq = (CSequence*)GetAt(i);
		if (seq == NULL) return NULL;
		res = seq->GetAt(1);
	}
	return res;
}

CSequence* CVIDensData::get_PM_cart(int i) {
	if (isInvariant()) { dbg->log(0, "ERROR get_PM_cart() call for invariant DENSDATA"); CEException::halt(); }

	return (CSequence*)GetAt(i);
}

/*
*	values number is s*(s+1)/2 where s = getDensNumber();
*/
CExpression* CVIDensData::getInvValue(int i) {
	if (!isInvariant()) { dbg->log(0, "ERROR getInvValue() call for non-invariant DENSDATA"); CEException::halt(); }

	int i_max = (m_dens_num*(m_dens_num+1))/2-1;

	// (intarnal check) compare i_max to size()
	if (i_max != (size()-1)) { dbg->log(0, size(), i_max+1, "<ERROR> check failed in getInvValue(): invlaid DD size: %ld, expected value: %ld", this); CEException::halt(); }

	if ((i < 0) || (i > i_max)) { dbg->log(0, i, i_max, "ERROR getInvValue(%ld) invalid parameter, (0 <= expected value <= %ld)"); CEException::halt(); }

	return GetAt(i);
}

int CVIDensData::getFirstMassPos() { // first pos of mass parameter
	if (isInvariant()) return (size()-m_dens_num);
	return -1;
}

//////////////////////////////////////////////////////////////////////
// Sort/Normalize Operations
//////////////////////////////////////////////////////////////////////

/*
*	Sorted DENSDATA order is the following:
*	{0,0},{0,m},{p,0},{p,m},{p+q,0},{p+q,m},{p+q+g,0},{p+q+g,m}.
*/
//#define CVIDensData_DEBUG
#ifdef CVIDensData_DEBUG
CExpression* xTmpKK1=NULL;
CExpression* xTmpKK2=NULL;
CExpression* xTmpK1K2=NULL;
CExpression* xTmpP1P2=NULL;
#endif //#ifdef CVIDensData_DEBUG
void CVIDensData::sort() {
	if (isInvariant()) return; // do not sort invariant DENSDATA
	dbg->log(45, "sort()", this);

	int s = size();
	for(int i=1; i<s; i++) {
		CSequence* seq=get_PM_cart(i); CExpression* p=seq->GetAt(0); CExpression* m=seq->GetAt(1); if((p==NULL)||(m==NULL))CEException::halt(dbg, "Invalid DENSDATA sorting.");

		for(int j=0; j<i; j++) {
			// if {p,m} should be before 'j', move it here.
			CSequence* seqJ=get_PM_cart(j);
			bool move_before_J=is_right_order(p,m,seqJ->GetAt(0),seqJ->GetAt(1));

			// moving
			if (move_before_J) {
				seq=(CSequence*)(seq->clone());
				RemoveAt(i); InsertAt(j,seq); delete seq; // j<i here
				break;
			}
		}
	}

	if(CVirtIntegral::PAR_LOOPTOOLS_COMPARTIBLE){ //asdfg: debug cases for BH process
		//Changed order for D-integrals to fix dens order in BH process: {0,0},{k1,m},{sum,m},{k2,m}
		//TODO: remove next lines after LT problems fixing
		if(s==3){
			//To last pos: k1-k, k1-k2, k2+k
#ifdef CVIDensData_DEBUG
			if(xTmpKK1==NULL){
				CKinematics* kin=CEProcessor::kinematics();
				xTmpKK1=CExpression::reduce(kin->get_momentum(2)->clone()->minus(kin->get_momentum(-3)->clone()));
				xTmpKK2=CExpression::reduce(kin->get_momentum(-2)->clone()->add(kin->get_momentum(-3)->clone()));
				xTmpK1K2=CExpression::reduce(kin->get_momentum(2)->clone()->minus(kin->get_momentum(-2)->clone()));
				xTmpP1P2=CExpression::reduce(kin->get_momentum(1)->clone()->minus(kin->get_momentum(-1)->clone()));
			}
#endif //#ifdef CVIDensData_DEBUG
			CSequence* seq1=get_PM_cart(1); CExpression* p1=seq1->GetAt(0);
			CSequence* seq2=get_PM_cart(2); CExpression* p2=seq2->GetAt(0);
#ifdef CVIDensData_DEBUG
			if((p1->equal_or_opposite(xTmpKK1)) || (p1->equal_or_opposite(xTmpKK2)) || (p1->equal_or_opposite(xTmpK1K2))){
				move(1,2);
			}
			else if((p2->equal_or_opposite(xTmpKK1)) || (p2->equal_or_opposite(xTmpKK2)) || (p2->equal_or_opposite(xTmpK1K2))){
				//move(2,2);
				//It's at 'right' place
			}
#endif //#ifdef CVIDensData_DEBUG
			seq1=get_PM_cart(1);
#ifdef CVIDensData_DEBUG
			if(seq1->GetAt(0)->equals(xTmpP1P2)){ //p1-p2-->-q
				CESymbol mQ("q"); mQ.inverseFlag(); seq1->SetAt(0,&mQ);
			}
#endif //#ifdef CVIDensData_DEBUG
		}else if(s==4){
			CSequence* seq2=get_PM_cart(2); CExpression* p2=seq2->GetAt(0); bool bSumP2=p2->getProperty(CEProperties::PROP_MASK_IS_SUM);
			CSequence* seq3=get_PM_cart(3); CExpression* p3=seq3->GetAt(0); bool bSumP3=p3->getProperty(CEProperties::PROP_MASK_IS_SUM);
			if(bSumP3 && (!bSumP2)) {
				move(3,2);
				//exchange(2,3);
			}else{ CEException::halt("Internal error in CVIDensData::sort()"); }
		}
	}

	dbg->log(55, "sort() result:", this);
}

/*
*	Sorted DENSDATA order is the following:
*	{0,0},{0,m},{p,0},{p,m},{p+q,0},{p+q,m},{p+q+g,0},{p+q+g,m}.
*	1 if before 2 ==> true result (right order)
*	Note: undefined order is right order
*/
bool CVIDensData::is_right_order(CExpression* p1, CExpression* m1, CExpression* p2, CExpression* m2) {
	bool ZeroP1=CEBasic::isZero(p1); bool ZeroM1=CEBasic::isZero(m1);
	bool ZeroP2=CEBasic::isZero(p2); bool ZeroM2=CEBasic::isZero(m2);

	if (ZeroP1 && ZeroM1 && ((!ZeroP2) || (!ZeroM2))) return true; // for fast compare

	// comparison
	if (ZeroP1) {
		if (!ZeroP2) return true;

		// compare masses
		if (ZeroM1) return true;
		if ((ZeroM2) && (!ZeroM1)) return false;

		bool m2_less_m1 = m2->less_than(m1);
		if (m2_less_m1) return false;
		return true; // undefined order is right order
	}

	if (ZeroP2) return false;

	// both P1 & P2 != 0 here
	bool SumP1 = p1->getProperty(CEProperties::PROP_MASK_IS_SUM);
	bool SumP2 = p2->getProperty(CEProperties::PROP_MASK_IS_SUM);

	int Sum_Size1 = 0; if (SumP1) Sum_Size1 = ((CESum*)p1)->size();
	int Sum_Size2 = 0; if (SumP2) Sum_Size2 = ((CESum*)p2)->size();
	if (Sum_Size1 != Sum_Size2) return (Sum_Size1 < Sum_Size2);

	// compare masses
	if ((ZeroM1) && (!ZeroM2)) return true;
	if ((ZeroM2) && (!ZeroM1)) return false;
	
	bool p2_less_p1 = p2->less_than(p1);
	if (p2_less_p1) return false;
	return true; // undefined order is right order
}

/*CExpression* CVIDensData::SortAndGetNormPDel() {
	sort();
	return get_norm_p_del();
}*/

/*
*	Works properly with sorted DENSDATA only.
*	returns not-cloned value (one of the internal momenta).
*/
CExpression* CVIDensData::get_norm_p_del() {
	if (isInvariant()) { dbg->log(0, "ERROR get_norm_p_del() call for invariant DENSDATA"); CEException::halt(); }
	dbg->log(45, "get_norm_p_del()", this);

	int s = size();
	if (s == 0) return NULL;
	CExpression* p_del = NULL;
	
	//sort();//TEST // should be already sorted here

	// search for {p,0} at any position
	int i;
	for(i=0; i<s; i++) {
		if (CEBasic::isZero(mass(i))) { p_del = momentum(i); break; }
	}
	if (p_del != NULL) {
		if (!CEBasic::isZero(p_del)) return p_del;
	}

	// return {p,m} at first position
	p_del = momentum(0);
	if (!CEBasic::isZero(p_del)) return p_del;
	
	// DENSDATA structure is normalized.
	return NULL;
}


// return CLONED!!! value
// DD should be SORTED & NORMALIZED
// {0,m0}{...}{p1,m1}{pX+pY,m2} --> {0,m1}{-p1,m0}{pZ,m2} if (pX+pY-p1 == single pZ)
// the ONLY single-p1 momentum is allowed at initial state
CExpression* CVIDensData::get_simpl_p_del() {
	if (isInvariant()) { dbg->log(0, "WARNING: get_simpl_p_del() call for invariant DENSDATA"); return NULL; }

	// No vector shift for LoopTools-compartible mode
	if(CVirtIntegral::PAR_LOOPTOOLS_COMPARTIBLE)return NULL;

	int s = size(); if(s<3)return NULL;
	CExpression* p_expr = NULL;
	//CExpression* psum_expr = NULL;
	CExpression* res = NULL;
	int i;
	if (!CEBasic::isZero(momentum(0))) { dbg->log(0, "WARNING: get_simpl_p_del() call for UNSORTED DENSDATA", this); return NULL; }

	// find SUM and SINGLE expression
	for(i=1; i<s; i++) {
		CExpression* pI = momentum(i);
		if (CEBasic::isZero(pI)) continue;
		if (pI->getProperty(CEProperties::PROP_MASK_IS_SUM)) {
			if (res != NULL) continue; // simplification is already found, check next momenta
			if (p_expr == NULL) {
				// try this SUM as "single expression" to simplify {p+q,m}{p+q+x,m} case
				p_expr = pI;
				continue;
			}

			CExpression* tmp = pI->clone()->minus(p_expr->clone());
			dbg->log(75, "DENSDATA simplification check", this, tmp);
			tmp = CLHEPBasics::KinSimplify(tmp,
				CLHEPBasics::KS_MASS_EXACT|CLHEPBasics::KS_MOMENTA_SUBST|CLHEPBasics::KS_NO_COUPLINGS_SUBST); // MASS EXACT for faster work
			//dbg->log(2, "DENSDATA simplification res", tmp);
			if (tmp->getProperty(CEProperties::PROP_MASK_IS_SUM)) {
				delete tmp;
				continue; // search another SUM
			}

			delete tmp;
			res = p_expr->clone();
			dbg->log(75, "DENSDATA simplification found", res);
			continue; // simplification found, check next momenta are NOT SINGLE
		}

		if (res != NULL) {
			// SINGLE item spotted after SUM
			delete res;
			dbg->log(0, "WARNING in get_simpl_p_del(): unexpected order in DENSDATA", this);
			return NULL;
		}
		
		if (p_expr != NULL) {
			if (p_expr->equals(pI)) continue; //{pI,m1}{pI,m2} -- allowed case
			
			// too many momenta, no simplification is obvious
			return NULL;
		}
		p_expr = pI;
	}

	return res;
}

/*
*	Changes variable k->k-p and sorts DENSDATA afterwards.
*	Impl. Note: "p_del" object may be part of DENSDATA structure ==>
*	Clone "p_del" before changing DENSDATA.
*/
void CVIDensData::change_var_minus_pdel(CExpression *p_del) {
	if (isInvariant()) { dbg->log(0, "ERROR change_var_minus_pdel() call for invariant DENSDATA"); CEException::halt(); }
	dbg->log(30, "changing variable (k->k-p)", p_del);
	if ((p_del == NULL) || (CEBasic::isZero(p_del))) return;
	p_del = p_del->clone();

	CENumber zero((long)0);
	int s = size();
	int i;
	for(i=0; i<s; i++) {
		CSequence* pm_seq = get_PM_cart(i);
		CExpression* p = pm_seq->GetAt(0);

		if (p->equals(p_del)) {
			pm_seq->SetAt(0, &zero);
		} else {
			CExpression* p_new = p->clone()->minus(p_del->clone());
			p_new = CEProcessor::kinematics()->get_simpliest(CExpression::reduce(p_new));
			pm_seq->SetAt(0, p_new);
			delete p_new;
		}
	}
	delete p_del;

	sort();
	dbg->log(55, "variable changed, DD sorted:", this);
}

void CVIDensData::markInvariant(int _dens_num) {
	dbg->log(0, "WARNING markInvariant() call");
	m_dens_num = _dens_num;
}

void CVIDensData::convertInvariant() {
	if (isInvariant()) { dbg->log(0, "WARNING convertInvariant() call for invariant DENSDATA"); return; }

	int s = size();
	if (s == 0) { dbg->log(0, "ERROR 0-sized DENSDATA"); CEException::halt(); }
	if (!CEBasic::isZero(momentum(0))) { dbg->log(0, "ERROR: Casting unarranged DENSDATA to invariant form."); CEException::halt(); }

	dbg->log(15, "DENSDATA invariant converting", this);
	CSequence* new_seq = new CSequence();

	/*
	//Old
	int i;
	for (i=1; i<s; i++) {
		CExpression* k_i = momentum(i);
		CExpression* val = CLHEPBasics::KinArrange(new CESProduct(k_i, k_i),
			CLHEPBasics::KA_INVOLVE_MANDELSTAM_VARS | CLHEPBasics::KA_MASS_EXACT);
		new_seq->addItem(val);
	}

	for (i=1; i<s; i++) {
		CExpression* k_i = momentum(i);
		int j;
		for (j=i+1; j<s; j++) {
			CExpression* k_j = momentum(j);
			
			CExpression* val = new CESProduct(k_i, k_j);
			CExpression* val_cpy = val->clone();

			// try first mass limit simplification
			val = CLHEPBasics::KinArrange(val, CLHEPBasics::KA_INVOLVE_MANDELSTAM_VARS);
			val = CLHEPBasics::KinSimplify(val, CLHEPBasics::KS_INVOLVE_MANDELSTAM_VARS);
			if (CEBasic::isZero(val)) {
				delete val;
				val = CLHEPBasics::KinArrange(val_cpy,
					CLHEPBasics::KA_INVOLVE_MANDELSTAM_VARS | CLHEPBasics::KA_MASS_EXACT);
			} else { // val is valid
				delete val_cpy;
			}

			new_seq->addItem(val);
		}
	}*/

	int i;
	// add (p_N)^2 couplings
	// p_N == k_{N}-k_{N-1}
	for (i=1; i<s; i++) {
		CExpression* k_i = momentum(i);
		CExpression* p_i = (i>1)?(k_i->clone()->minus(momentum(i-1)->clone())):(k_i->clone());
		CExpression* val_src = CExpression::reduce(new CESProduct(p_i, p_i)); delete p_i;

		// try first mass limit simplification
		CExpression* val=CLHEPBasics::KinArrange(val_src->clone(),CLHEPBasics::KA_INVOLVE_MANDELSTAM_VARS);
		val=CLHEPBasics::KinSimplify(val, CLHEPBasics::KS_INVOLVE_MANDELSTAM_VARS);
		if (CEBasic::isZero(val)) {
			delete val; val=CLHEPBasics::KinArrange(val_src,CLHEPBasics::KA_INVOLVE_MANDELSTAM_VARS|CLHEPBasics::KA_MASS_EXACT);
			if (val->getProperty(CEProperties::PROP_MASK_IS_SUM)) { val=CLHEPBasics::KinSimplify(val, CLHEPBasics::KS_INVOLVE_MANDELSTAM_VARS|CLHEPBasics::KS_MASS_EXACT); }
		} else { // val is valid
			delete val_src;
		}
		new_seq->addItem(val);
	}

	if (s <= 4) { // for C and D integrals
		if (s == 3) { // C
			// LoopTools[2.4]-like implementation.
			// Add (p_2+p_1)^2 couplings, p_{N+1}+p_N=k_{N+1}-k_{N-1} ==> p1+p2=k2;
			CExpression* P2pP1 = momentum(2)->clone();
			CExpression* val_src = CExpression::reduce(new CESProduct(P2pP1, P2pP1)); delete P2pP1;
			CExpression* val = CLHEPBasics::KinArrange(val_src->clone(), CLHEPBasics::KA_INVOLVE_MANDELSTAM_VARS);
			if (val->getProperty(CEProperties::PROP_MASK_IS_SUM)) { val = CLHEPBasics::KinSimplify(val, CLHEPBasics::KS_INVOLVE_MANDELSTAM_VARS); }
			if (CEBasic::isZero(val)) {
				delete val;
				val = CLHEPBasics::KinArrange(val_src,CLHEPBasics::KA_INVOLVE_MANDELSTAM_VARS|CLHEPBasics::KA_MASS_EXACT);
				if (val->getProperty(CEProperties::PROP_MASK_IS_SUM)) { val = CLHEPBasics::KinSimplify(val, CLHEPBasics::KS_INVOLVE_MANDELSTAM_VARS|CLHEPBasics::KS_MASS_EXACT); }
			} else { // val is valid
				delete val_src;
			}
			new_seq->addItem(val);
		} else if (s == 4) { // D
			// LoopTools[2.4]-like implementation.
			// Add (p_4)^2 coupling, p_4^2=(p_1+p_2+p_3)^2=(k_3)^2
			CExpression* P4m = momentum(3)->clone();
			CExpression* val_src = CExpression::reduce(new CESProduct(P4m, P4m)); delete P4m;
			CExpression* val=CLHEPBasics::KinArrange(val_src->clone(), CLHEPBasics::KA_INVOLVE_MANDELSTAM_VARS);
			if (val->getProperty(CEProperties::PROP_MASK_IS_SUM)){ val=CLHEPBasics::KinSimplify(val, CLHEPBasics::KS_INVOLVE_MANDELSTAM_VARS); }
			if (CEBasic::isZero(val)) {
				delete val; val=CLHEPBasics::KinArrange(val_src,CLHEPBasics::KA_INVOLVE_MANDELSTAM_VARS|CLHEPBasics::KA_MASS_EXACT);
				if(val->getProperty(CEProperties::PROP_MASK_IS_SUM)){ val=CLHEPBasics::KinSimplify(val, CLHEPBasics::KS_INVOLVE_MANDELSTAM_VARS|CLHEPBasics::KS_MASS_EXACT); }
			} else { // val is valid
				delete val_src;
			}
			new_seq->addItem(val);
			// Add (p_{N+1}+p_N)^2 couplings [p1+2,p2+3], p_{N+1}+p_N=k_{N+1}-k_{N-1}, k_1=0;
			for (i=2; i<s; i++) {
				CExpression* PipPl=(i>2)?(momentum(i)->clone()->minus(momentum(i-2)->clone())):(momentum(i)->clone());
				CExpression* val_src=CExpression::reduce(new CESProduct(PipPl, PipPl)); delete PipPl;
				val=CLHEPBasics::KinArrange(val_src->clone(), CLHEPBasics::KA_INVOLVE_MANDELSTAM_VARS);
				if (val->getProperty(CEProperties::PROP_MASK_IS_SUM)) { val=CLHEPBasics::KinSimplify(val, CLHEPBasics::KS_INVOLVE_MANDELSTAM_VARS); }
				if (CEBasic::isZero(val)) {
					delete val; val=CLHEPBasics::KinArrange(val_src,CLHEPBasics::KA_INVOLVE_MANDELSTAM_VARS|CLHEPBasics::KA_MASS_EXACT);
					if(val->getProperty(CEProperties::PROP_MASK_IS_SUM)){ val=CLHEPBasics::KinSimplify(val, CLHEPBasics::KS_INVOLVE_MANDELSTAM_VARS|CLHEPBasics::KS_MASS_EXACT); }
				} else { // val is valid
					delete val_src;
				}
				new_seq->addItem(val);
			}
		}
	} else { // common case
		// add p_i*p_j couplings
		// Warning: this case is NOT uniformed with LoopTools.
		// TODO: uniform it!
		for (i=1; i<s; i++) {
			CExpression* k_i = momentum(i)->clone();
			CExpression* p_i = (i>1)?(k_i->minus(momentum(i-1)->clone())):(k_i);
			
			int j;
			for (j=i+1; j<s; j++) {
				CExpression* k_j = momentum(j)->clone();
				CExpression* p_j = (j>1)?(k_j->minus(momentum(j-1)->clone())):(k_j);
				CExpression* val = CExpression::reduce(new CESProduct(p_i, p_j));
				delete p_j;
				CExpression* val_cpy = val->clone();
				
				// try first mass limit simplification
				val = CLHEPBasics::KinArrange(val, CLHEPBasics::KA_INVOLVE_MANDELSTAM_VARS);
				if (val->getProperty(CEProperties::PROP_MASK_IS_SUM)) { val = CLHEPBasics::KinSimplify(val, CLHEPBasics::KS_INVOLVE_MANDELSTAM_VARS); }
				if (CEBasic::isZero(val)) {
					delete val;
					val = CLHEPBasics::KinArrange(val_cpy,CLHEPBasics::KA_INVOLVE_MANDELSTAM_VARS|CLHEPBasics::KA_MASS_EXACT);
					if (val->getProperty(CEProperties::PROP_MASK_IS_SUM)) { val = CLHEPBasics::KinSimplify(val, CLHEPBasics::KS_INVOLVE_MANDELSTAM_VARS|CLHEPBasics::KS_MASS_EXACT); }
				} else { // val is valid
					delete val_cpy;
				}
				new_seq->addItem(val);
			}
			delete p_i;
		}
	}

	// add mass data
	for (i=0; i<s; i++) { new_seq->addItem(mass(i)->clone()); }

	// replace old data with new
	copyAttributesTo(new_seq); // to preserve attributes in next operation
	copy_from(new_seq);
	delete new_seq;

	m_dens_num = s;
	dbg->log(15, "DENSDATA invariant converting result", this);
}

///////////////////////////////////////////////////////////////////////////////////////////////////
/*CSequence* CVIDensData::get_vector_basis(int rank)
{
	if (rank <=0) return NULL;
	if (rank > 2) {
		dbg->log(1, "Error: vector basis of rank >2 is not supported.");
		return NULL;
	}

	int s = size();
	if ((s > 3) || ((rank == 2) && (s != 3))) {
		dbg->log(1, "Error: (get_vector_basis) not supported case.");
		return NULL;
	}
	sort();
	if (!is_normalized()) {
		dbg->log(1, "Error: request for vector basis of non-normalized denominators.");
		return NULL;
	}

	int i=0;
	CSequence* res = new CSequence();
	switch (rank)
	{
	case 1:
		for (i=1; i<s; i++) {
			res->addItem(momentum(i)->clone());
		}
		break;
	case 2:
		//s == 3

		//index set should be involved...
		{
			CExpression* p1 = momentum(1);
			CExpression* p2 = momentum(2);
		}
		//res->addItem(momentum(i)->clone());
		break;
	}
	return res;
}

bool CVIDensData::is_normalized() {
	//check up for sorting should also be here
	if (get_norm_p_del() != NULL) return false;
	return true;
}
*/
