////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// VirtInt0.cpp: implementation of the CVirtInt0 class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

const char* CVirtInt0::CLASS_NAME		= "CVirtInt0";
const char* CVirtInt0::CLASS_TAG_NAME	= "vi0";

CExpression* CVirtInt0::m_a0_to_m = NULL;
CExpression* CVirtInt0::get_a0m_cpy() {
	if (m_a0_to_m == NULL) m_a0_to_m = new CESymbol("a\\_0"); // init
	return m_a0_to_m->clone();
}
void CVirtInt0::finalize() { if (m_a0_to_m != NULL) { delete m_a0_to_m; m_a0_to_m=NULL;} }

// does no affect parameter
CVirtInt0* CVirtInt0::createInvVI(CVIDensData* inv_dd) {
	if (inv_dd == NULL) return NULL;
	return new CVirtInt0(inv_dd, true, true);
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CVirtInt0::CVirtInt0(const CVIDensData* ddata) : CVirtIntegral(ddata) {
	m_Invariant_Dens = false;
	m_isNormalized = false;
}

CVirtInt0::CVirtInt0(const CVIDensData* ddata, bool _Invariant_Dens, bool _isNormalized) : CVirtIntegral(ddata) {
	m_Invariant_Dens = _Invariant_Dens;
	m_isNormalized = _isNormalized;
}

CVirtInt0::~CVirtInt0() { }

CEString* CVirtInt0::GetHEPId() { return new CEString("0"); }

CExpression* CVirtInt0::clone() const {
	CExpression* res = new CVirtInt0(get_dens_data_const(), m_Invariant_Dens, m_isNormalized);
	copyAttributesTo(res);
	return res;
}

bool CVirtInt0::equals(CExpression* other) {
	if (!CVirtIntegral::equals(other)) return false;
	return true;
}

CEString* CVirtInt0::getTitleAdv(unsigned conventions, int mode) {
	CEString* res = NULL;

	if (conventions & CLFileManager::FT_LOOPTOOLS) {
		res = new CEString("");
	} else {
		res = new CEString("VirtIntegral");
	}

	switch (get_dens_data()->getDensNumber()) {
	case 1: res->AddString("A0"); break;
	case 2: res->AddString("B0"); break;
	case 3: res->AddString("C0"); break;
	case 4: res->AddString("D0"); break;
	case 5: res->AddString("E0"); break;
	case 6: res->AddString("F0"); break;
	default: res->AddString("X0");
	}

	return res;
}

CEString* CVirtInt0::toString(int mode) {
	CEString* res = NULL;
	if (getFlag()) res = new CEString("\\(");
	else res = new CEString("- \\(");

	switch (get_dens_data()->getDensNumber()) {
	case 1: res->AddString("A"); break;
	case 2: res->AddString("B"); break;
	case 3: res->AddString("C"); break;
	case 4: res->AddString("D"); break;
	case 5: res->AddString("E"); break;
	case 6: res->AddString("F"); break;
	default:
		res->AddString("X");
	}
	res->AddString("\\_0\\)");

	//res->AddString("[");
	res->AddString("\\&");
	res->AddString(get_dens_data()->toString(mode));
	//res->AddString("]");
	return res;
}

CExpression* CVirtInt0::evaluate(int level) {
	CExpression* res = CVirtIntegralLib::get_stored(this);
	if (res != NULL) return res;

	if (m_Invariant_Dens) return this;
	Arrange(AL_INLINE);


	/*CVIDensData* dd = get_dens_data();
	int s = dd->size();
	switch(s) {
	case 1://A_0(m)
		{
			CExpression* m = dd->mass(0);
			res = get_a0m_cpy()->multiply(m->sqra());
			copyAttributesTo(res, true);
			CVirtIntegralLib::store_vi(this, res);
			return res;
		}
		break;
	}*/

	return this;
}

CExpression* CVirtInt0::Arrange(int level) {
#if defined(D0_ONLY_CALC)
	if ((level == AL_EXTRACT) && (get_dens_data()->getDensNumber() < 4)) return CENumber::GetZero();
#endif //D0_ONLY_CALC
	
	if (m_isNormalized) return this;

	CExpression* norm_res = NormalizeVI(level);
	return norm_res;
}

/*
*	Slow ARRANGE including denominators sorting
*/
CExpression* CVirtInt0::NormalizeVI(int arr_level) {
	if (m_Invariant_Dens) return this;

	CVIDensData* dd = get_dens_data();
	dd->sort();
	
	CExpression* p_del = dd->get_norm_p_del();  // p_del is NON-CLONED here
	if (p_del != NULL) {
		p_del = p_del->clone();
		dd->change_var_minus_pdel(p_del); // including sorting
		delete p_del;
		dbg->log(45, "denominators normalized", dd);
	}

	// find DD-simplification
	p_del = dd->get_simpl_p_del(); // p_del is CLONED here
	if (p_del != NULL) {
		dd->change_var_minus_pdel(p_del); // including sorting
		delete p_del;
		dbg->log(45, "denominators simplified", dd);
	}

	if (dd->size() <= 2) toInvariantDens();
	dbg->log(75, "normalized", this);

	m_isNormalized = true;
	return this;
}

/*
*	Denominators should be normalized.
*/
void CVirtInt0::toInvariantDens() {
	CVIDensData* dd = get_dens_data();
	if (dd->isInvariant()) return;

	dd->convertInvariant();
	m_Invariant_Dens = true;

	m_isNormalized = true;

	/*int s = dd->size();
	if (s != 2) { m_Invariant_Dens = true; return; }

	// check up if DENSDATA is arranged.
	CExpression* p = dd->momentum(0);
	if (!CEBasic::isZero(p)) {
		dbg->log(0, "Warning: Casting VI-0 to invariant form while DENSDATA is unarranded.");
		return;
	}

	CSequence* seq1 = dd->get_PM_cart(1);
	p = seq1->GetAt(0);
	CExpression* inv_val = CLHEPBasics::KinArrange(new CESProduct(p, p),
		CLHEPBasics::KA_INVOLVE_MANDELSTAM_VARS | CLHEPBasics::KA_MASS_EXACT);
	seq1->SetAt(0, inv_val);
	delete inv_val;

	m_Invariant_Dens = true;*/
}

/*
*	for external calls
*/
void CVirtInt0::ConvertInvariant() {
	// internal check
	if (!CEUtil::LEQU(m_Invariant_Dens, get_dens_data()->isInvariant())) {
		dbg->log(0, m_Invariant_Dens, get_dens_data()->isInvariant(), "<Warning> VI-0 invalid invariance flag: %ld, actual value is: %ld. Corrected.");
		m_Invariant_Dens = get_dens_data()->isInvariant();
	}

	if (m_Invariant_Dens) return;
	Arrange(AL_INLINE); // normalize denominators
	toInvariantDens();
}

CExpression* CVirtInt0::GetSingularTermFactor() { // NULL for unknown
	CExpression* res = NULL;
	CVIDensData* dd = get_dens_data();

	/*
	*	The result means: Integral==Xi*Result+0(n-4)
	*	Note: Xi*(n-4) = -2 + 0(n-4)
	*	A0->m^2
	*	B0->1
	*/
	switch (dd->getDensNumber()) {
	case 1:
		res = dd->mass(0)->sqra();
		break;
	case 2:
		res = CENumber::GetUnit();
		break;
	case 3: // C0
		//res = NULL;
		// TODO: check for possible errors due to IR-singular C0-integrals.
		res = CENumber::GetZero();
		break;
	case 4: // D0
		res = CENumber::GetZero();
		break;
	default:
		res = NULL;
	}

	if (res != NULL) copyAttributesTo(res, true);
	return res;
}

const char* CVirtInt0::ClassName() const { return CLASS_NAME; }
const char* CVirtInt0::get_TAG_Name() { return CLASS_TAG_NAME; }

long CVirtInt0::store_stream(ofstream *p_ofs, DWORD settings) {
	long res = CESupport::store_stream(get_dens_data(), p_ofs, settings);
	CENumber* inv_obj = new CENumber((m_Invariant_Dens)?1:0);
	res += CESupport::store_stream(inv_obj, p_ofs, settings);
	delete inv_obj;
	return res;
}

CESerializable* CVirtInt0::restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings) {
	// load DENSDATA
	long skip_pos = CESupport::skip_item_in_buffer(data, pos, settings);
	if (skip_pos <= 0) { dbg->log(0, "Error restoring CVirtInt0 : Can't load DENSDATA."); return NULL; }
	CESerializable* dd = CESupport::read_buffer(data, pos, settings);
	if ((dd == NULL) || (!CEUtil::equals(dd->ClassName(), CVIDensData::CLASS_NAME))) {
		dbg->log(0, "Error loading DENSDATA for CVirtInt0.");
		if (dd != NULL) delete dd;
		return NULL;
	}
	CVIDensData* viDD = (CVIDensData*)dd;

	// load invariance flag
	pos = skip_pos;
	skip_pos = CESupport::skip_item_in_buffer(data, pos, settings);
	if (skip_pos <= 0) { dbg->log(0, "Error restoring CVirtInt0 : Can't load invariance flag."); return NULL; }
	CESerializable* num_void = CESupport::read_buffer(data, pos, settings);
	if ((num_void == NULL) || (!CEUtil::equals(num_void->ClassName(), CENumber::CLASS_NAME))) {
		dbg->log(0, "Error loading invariance flag for CVirtInt0.");
		delete viDD;
		if (num_void != NULL) delete num_void;
		return NULL;
	}

	// check up number value
	CENumber* num_obj = (CENumber*)num_void;
	int inv_flag = num_obj->getNum();
	if ((inv_flag < 0) || (num_obj->getDen() != 1) || (num_obj->getNumIm() != 0)) {
		dbg->log(0, "Invalid invariance flag loaded for CVirtInt0:", num_obj);
		delete viDD; delete num_obj; return NULL;
	}
	delete num_obj;

	pos = skip_pos;
	skip_pos = CESupport::skip_item_in_buffer(data, pos, settings);
	if (skip_pos > 0) { dbg->log(0, "Error restoring CVirtInt0: Extra data found."); delete viDD; return NULL; }

	// for old-format DENSDATA convertions
	if ((inv_flag != 0) && (!viDD->isInvariant())) {
		if (viDD->size() <= 2) viDD->markInvariant(viDD->size());
		else inv_flag = 0;
	}

	CVirtInt0* res = new CVirtInt0(viDD, (inv_flag != 0), false);
	delete viDD;
	return res;
}






/*
//////////////////////////////////////////////////////////	old code from "evaluate()"
	CEData* data = CEProcessor::getDataStorage();
	CExpression* zero = data->gete(CConst::DV_ZERO);
	CExpression* unit = data->gete(CConst::DV_UNIT);
	CExpression* two = data->gete(CConst::DV_TWO);
	CExpression* half = data->gete(CConst::DV_HALF);
	CExpression* del = data->gete(CConst::GL_DELTA);
	CExpression* phi = data->gete(CConst::GL_PHI);

	CVIDensData* dd = get_dens_data();
	int s = dd->size();
	switch (s)
	{
	case 1:
		{
			CExpression* m = dd->mass(0);
			res = del->clone()->add(unit->clone())->multiply(m->sqra());
		}
		break;
	case 2:
		{
			CExpression* m0 = dd->mass(0);
			CExpression* m1 = dd->mass(1);
			CExpression* m1
			CExpression* p = dd->momentum(1);
			CExpression* p2 = temp = 
				CHEPLib::repl_kin_invariants(CExpression::reduce(CEVector::square(mom)));

			if (zero->equals(p2)) {
				if (m0->equals(m1)) res = del->clone();
			}
			else if (m1->equals(p2)) {
			}
		}
		break;
	}

	//B(0,m,m)
	CSequence* ts2 = new CSequence();
	ts2->addItem(p1->clone());
	ts2->addItem(m->clone());
	ds->addItem(ts2);

	vi = CVirtIntegral::createVirtIntegral(ds, NULL);
	CVirtIntegral::store_vi(vi, del);
	delete vi;

	//B'(0,m,m)
	vi = new CVirtInt1(new CVIDensData(ds), 1, 1);
	vi_val = del->clone()->multiply(half->clone());
	vi_val->inverseFlag();
	CVirtIntegral::store_vi(vi, vi_val);
	delete vi_val;
	delete vi;

	//B(0,m,m)
	ts2->SetAt(0, p2);
	ds->addItem(ts2);
	vi = CVirtIntegral::createVirtIntegral(ds, NULL);
	CVirtIntegral::store_vi(vi, del);
	delete vi;

	//B'(0,m,m)
	vi = new CVirtInt1(new CVIDensData(ds), 1, 1);
	vi_val = del->clone()->multiply(half->clone());
	vi_val->inverseFlag();
	CVirtIntegral::store_vi(vi, vi_val);
	delete vi_val;
	delete vi;

	//B(t,0,m)
	ts2->SetAt(0, p11d1);
	ts1->SetAt(1, zero);
	vi = CVirtIntegral::createVirtIntegral(ds, NULL);
	vi_val = del->clone()->add(two->clone())->minus(phi->clone());
	CVirtIntegral::store_vi(vi, vi_val);
	//delete vi_val;
	delete vi;

	//B'(t,0,m)
	vi = new CVirtInt1(new CVIDensData(ds), 1, 1);
	vi_val = vi_val->multiply(half->clone());
	vi_val->inverseFlag();
	CVirtIntegral::store_vi(vi, vi_val);
	delete vi_val;
	delete vi;

	//B(m^2,0,m)
	ts2->SetAt(0, p11);
	vi = CVirtIntegral::createVirtIntegral(ds, NULL);
	vi_val = del->clone()->add(two->clone());
	CVirtIntegral::store_vi(vi, vi_val);
	delete vi_val;
	delete vi;

	//B'(m^2,0,m)
	vi = new CVirtInt1(new CVIDensData(ds), 1, 1);
	vi_val = del->clone()->add(unit->clone())->multiply(half->clone());
	vi_val->inverseFlag();
	CVirtIntegral::store_vi(vi, vi_val);
	delete vi_val;
	delete vi;

	//B(m^2,0,m)
	ts2->SetAt(0, p21);
	vi = CVirtIntegral::createVirtIntegral(ds, NULL);
	vi_val = del->clone()->add(two->clone());
	CVirtIntegral::store_vi(vi, vi_val);
	delete vi_val;
	delete vi;

	//B'(m^2,0,m)
	vi = new CVirtInt1(new CVIDensData(ds), 1, 1);
	vi_val = del->clone()->add(unit->clone())->multiply(half->clone());
	vi_val->inverseFlag();
	CVirtIntegral::store_vi(vi, vi_val);
	delete vi_val;
	delete vi;

	//-B(m^2,0,m)
	ts2->SetAt(0, p11m);
	vi = CVirtIntegral::createVirtIntegral(ds, NULL);
	vi_val = del->clone()->add(two->clone());
	vi_val->inverseFlag();
	CVirtIntegral::store_vi(vi, vi_val);
	delete vi_val;
	delete vi;

*/
