////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// VirtInt1.cpp: implementation of the CVirtInt1 class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

const char* CVirtInt1::CLASS_NAME		= "CVirtInt1";
const char* CVirtInt1::CLASS_TAG_NAME	= "vi1";
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CVirtInt1::CVirtInt1(const CVIDensData* ddata, int _order, int _num) : CVirtIntegral(ddata) { m_num=_num; m_order=_order; }
CVirtInt1::~CVirtInt1() { }

CEString* CVirtInt1::toString(int mode) {
	CEString* res = NULL;
	if (getFlag()) res = new CEString("\\(");
	else res = new CEString("- \\(");

	switch (get_dens_data()->getDensNumber())
	{
	case 1: res->AddString("A"); break;
	case 2: res->AddString("B"); break;
	case 3: res->AddString("C"); break;
	case 4: res->AddString("D"); break;
	case 5: res->AddString("E"); break;
	case 6: res->AddString("F"); break;
	default:
		res->AddString("X");
	}

	res->AddString("\\_\\(");
	res->AddString(GetHEPId());
	res->AddString("\\)\\)");
	
	//res->AddString("[");
	res->AddString("\\&");
	res->AddString(get_dens_data()->toString(mode));
	//res->AddString("]");
	return res;
}

int CVirtInt1::GetVI1Num() { return m_num; }
int CVirtInt1::GetVI1Order() { return m_order; }

bool CVirtInt1::equals(CExpression* other) {
	if (!CVirtIntegral::equals(other)) return false; // check type & densdata
	if (m_num != ((CVirtInt1*)other)->GetVI1Num()) return false;
	if (m_order != ((CVirtInt1*)other)->GetVI1Order()) return false;
	return true;
}

CEString* CVirtInt1::getTitleAdv(unsigned conventions, int mode) {
	// Note: LoopTools noteation  for Cxx integrals is COi(id, ...)
	CEString* res = NULL;
	if (conventions & CLFileManager::FT_LOOPTOOLS) {
		res = new CEString("");
	} else {
		res = new CEString("VirtIntegral");
	}

	int d_num = get_dens_data()->getDensNumber();
	switch (d_num) {
	case 1: res->AddString("A"); break;
	case 2: res->AddString("B"); break;
	case 3: res->AddString("C"); break;
	case 4: res->AddString("D"); break;
	case 5: res->AddString("E"); break;
	case 6: res->AddString("F"); break;
	default: res->AddString("X");
	}

	bool add_hepID = false;
	if (((conventions & CLFileManager::FT_LOOPTOOLS)==0) || (d_num == 2)) {
		res->AddString(GetHEPId());
	} else { // LoopTools C,D
		res->AddString("0i");
	}

	return res;
}

CExpression* CVirtInt1::clone() const {
	CExpression* res = new CVirtInt1(get_dens_data_const(), m_order, m_num);
	copyAttributesTo(res);
	return res;
}

CExpression* CVirtInt1::Arrange(int level) {
#if defined(D0_ONLY_CALC)
	if ((level == AL_EXTRACT) && (get_dens_data()->getDensNumber() < 4)) return CENumber::GetZero();
#endif //D0_ONLY_CALC
	
	return this;
}

CExpression* CVirtInt1::evaluate(int level) {
#if defined(D0_ONLY_CALC)
	if (get_dens_data()->getDensNumber() < 4) return CENumber::GetZero();
#endif //D0_ONLY_CALC

	CExpression* res = CVirtIntegralLib::get_stored(this);
	if (res != NULL) return res;

	if (get_dens_data()->isInvariant()) {
		return this; // no evaluation for invariant VI-X are currently available
	}

	switch(m_order) {
	case 1:
		res = CVirtIntegralLib::Evaluate_VI1(this);
		break;
	case 2:
		res = CVirtIntegralLib::Evaluate_VI2(this);
		break;
	}

	if (res != NULL) return res;
	return this;
}

CExpression* CVirtInt1::GetSingularTermFactor() { // NULL for unknown
	CExpression* res = NULL;
	CVIDensData* dd = get_dens_data();

	/*
	*	The result means: Integral==Xi*Result+0(n-4)
	*	Note: Xi*(n-4) = -2 + 0(n-4)
	*	A0->m^2
	*	B0->1
	*	B1->-1/2
	*	B00->-1/12(p^2-3m1^2-3m2^2)
	*	B11->1/3
	*	C00->1/4
	*	C001->-1/12
	*	C002->-1/12
	*	D0000->1/24
	*	For others --> 0
	*/
	switch(m_order) {
	case 1: // X1
		switch (dd->getDensNumber()) {
		case 2: res = CENumber::GetHalf(); res->inverseFlag(); break; //B1
		default: res = CENumber::GetZero();
		}
		break;
	case 2: // X11
		switch (dd->getDensNumber()) {
		case 2: // B2x
			if (m_num == 1) { // B11
				res = CENumber::GetFraction(1,3);
			} else if (m_num == 2) { // B00
				CExpression* p2=NULL;
				if(dd->isInvariant()){ p2=dd->getInvValue(0)->clone(); }else{ CExpression* p0=dd->momentum(1); p2=new CESProduct(p0,p0); }
				CExpression* m12_sum_t3 = (dd->mass(0)->sqra()->add(dd->mass(1)->sqra()))->multiply(CENumber::GetNumber(3));
				res = (CENumber::GetFraction(1,12))->multiply( m12_sum_t3->minus(p2) );
			}
			break;
		case 3: // C2x
			if (m_num == 4) { //C_00
				res = CENumber::GetFraction(1,4);
			} else {
				res = CENumber::GetZero();
			}
			break;
		default: res = CENumber::GetZero();
		}
		break;
	case 3: // X111
		if (dd->getDensNumber() == 3) {
			if ((m_num == 1) || (m_num == 2)) { //C_001 and C_002
				res = CENumber::GetFraction(1,12); res->inverseFlag();
			} else {
				res = CENumber::GetZero();
			}
		} else {
			res = CENumber::GetZero();
		}
		break;
	case 4: // for D0000
		if (dd->getDensNumber() == 4) {
			// leave the result unknown
			dbg->log(0,"WARNING: D-XXXX integrals are not fully implemented",this);
		} else {
			res = CENumber::GetZero();
		}
		break;
	}

	if (res != NULL) copyAttributesTo(res, true);
	return res;
}

CEString* CVirtInt1::GetHEPId() {
	CEString* res = NULL;

	switch(m_order) {
	case 1: // X1
		res = new CEString(""); res->add_int(m_num);
		break;
	case 2: // X11
		{
			const char* _id = NULL;
			switch (get_dens_data()->getDensNumber()) {
			case 1: break;
			case 2:
				switch (m_num) {
				case 1: _id = "11"; break;
				case 2: _id = "00"; break;
				}
				break;
			case 3:
				switch (m_num) {
				case 1: _id = "11"; break;
				case 2: _id = "22"; break;
				case 3: _id = "12"; break;
				case 4: _id = "00"; break;
				}
				break;
			case 4:
				switch (m_num) {
				case 1: _id = "11"; break;
				case 2: _id = "12"; break;
				case 3: _id = "13"; break;
				case 4: _id = "22"; break;
				case 5: _id = "23"; break;
				case 6: _id = "33"; break;
				case 7: _id = "00"; break;
				}
				break;
			}
			
			if (_id != NULL) {
				res = new CEString(_id);
			} else {
				res = new CEString("x"); res->add_int(m_order); res->add_int(m_num); res->add_int(get_dens_data()->getDensNumber());
			}
		}
		break;
	case 3: // X111
		{
			const char* _id = NULL;
			switch (get_dens_data()->getDensNumber()) {
			case 1: break;
			case 2: break;
			case 3:
				switch (m_num) {
				case 1: _id = "001"; break; //C_001
				case 2: _id = "002"; break; //C_002
				case 3: _id = "111"; break; //C_111
				case 4: _id = "222"; break; //C_222
				case 5: _id = "112"; break; //C_112
				case 6: _id = "122"; break; //C_122
				}
				break;
			case 4:
				switch (m_num) {
				case 1: _id = "001"; break; //D_001
				case 2: _id = "002"; break; //D_002
				case 3: _id = "003"; break; //D_003
				case 4: _id = "111"; break; //D_111
				case 5: _id = "222"; break; //D_222
				case 6: _id = "333"; break; //D_333
				case 7: _id = "112"; break; //D_112
				case 8: _id = "113"; break; //D_113
				case 9: _id = "122"; break; //D_122
				case 10: _id = "123"; break; //D_123
				case 11: _id = "133"; break; //D_133
				case 12: _id = "223"; break; //D_223
				case 13: _id = "233"; break; //D_233
				}
				break;
			}

			if (_id != NULL) {
				res = new CEString(_id);
			} else {
				res = new CEString("x"); res->add_int(m_order); res->add_int(m_num); res->add_int(get_dens_data()->getDensNumber());
			}
		}
		break;
	}

	if (res == NULL) { res = new CEString("x"); res->add_int(m_order); res->add_int(m_num); res->add_int(get_dens_data()->getDensNumber()); }
	return res;
}

void CVirtInt1::ConvertInvariant() {
	if (get_dens_data()->isInvariant()) return;

	Arrange(AL_INLINE); // normalize denominators here like VI-0 (not implemented currently)

	get_dens_data()->convertInvariant();
}

const char* CVirtInt1::ClassName() const { return CLASS_NAME; }
const char* CVirtInt1::get_TAG_Name() { return CLASS_TAG_NAME; }

long CVirtInt1::store_stream(ofstream *p_ofs, DWORD settings) {
	long res = CESupport::store_stream(get_dens_data(), p_ofs, settings);
	CENumber num_order(m_order);
	res += CESupport::store_stream(&num_order, p_ofs, settings);
	CENumber num_obj(m_num);
	res += CESupport::store_stream(&num_obj, p_ofs, settings);
	return res;
}

CESerializable* CVirtInt1::restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings) {
	// load DENSDATA
	long skip_pos = CESupport::skip_item_in_buffer(data, pos, settings);
	if (skip_pos <= 0) { dbg->log(0, "Error restoring CVirtInt1 : Can't load DENSDATA."); return NULL; }
	CESerializable* dd = CESupport::read_buffer(data, pos, settings);
	if ((dd == NULL) || (!CEUtil::equals(dd->ClassName(), CVIDensData::CLASS_NAME))) {
		dbg->log(0, "Error loading DENSDATA for CVirtInt1.");
		if (dd != NULL) delete dd;
		return NULL;
	}
	CVIDensData* viDD = (CVIDensData*)dd;

	// load order number
	pos = skip_pos;
	skip_pos = CESupport::skip_item_in_buffer(data, pos, settings);
	if (skip_pos <= 0) { dbg->log(0, "Error restoring CVirtInt1 : Can't load order number."); return NULL; }
	CESerializable* num_void_obj1 = CESupport::read_buffer(data, pos, settings);
	if ((num_void_obj1 == NULL) || (!CEUtil::equals(num_void_obj1->ClassName(), CENumber::CLASS_NAME))) {
		dbg->log(0, "Error loading ID number for CVirtInt1.");
		delete viDD;
		if (num_void_obj1 != NULL) delete num_void_obj1;
		return NULL;
	}
	int num_order = ((CENumber*)num_void_obj1)->getNum();
	if ((num_order <= 0) || (((CENumber*)num_void_obj1)->getDen() != 1) || (((CENumber*)num_void_obj1)->getNumIm() != 0)) {
		dbg->log(0, "Invalid order number loaded for CVirtInt1:", num_void_obj1);
		delete viDD; delete num_void_obj1; return NULL;
	}
	delete num_void_obj1;

	// load ID number
	pos = skip_pos;
	skip_pos = CESupport::skip_item_in_buffer(data, pos, settings);
	if (skip_pos <= 0) { dbg->log(0, "Error restoring CVirtInt1 : Can't load ID number."); return NULL; }
	CESerializable* num_void_obj2 = CESupport::read_buffer(data, pos, settings);
	if ((num_void_obj2 == NULL) || (!CEUtil::equals(num_void_obj2->ClassName(), CENumber::CLASS_NAME))) {
		dbg->log(0, "Error loading ID number for CVirtInt1.");
		delete viDD;
		if (num_void_obj2 != NULL) delete num_void_obj2;
		return NULL;
	}

	// check up number value
	CENumber* num_obj2 = (CENumber*)num_void_obj2;
	int num_val = num_obj2->getNum();
	if ((num_val <= 0) || (num_obj2->getDen() != 1) || (num_obj2->getNumIm() != 0)) {
		dbg->log(0, "Invalid ID number loaded for CVirtInt1:", num_obj2);
		delete viDD; delete num_obj2; return NULL;
	}
	delete num_obj2;

	pos = skip_pos;
	skip_pos = CESupport::skip_item_in_buffer(data, pos, settings);
	if (skip_pos > 0) { dbg->log(0, "Error restoring CVirtInt1: Extra data found."); delete viDD; return NULL; }

	CVirtInt1* res = new CVirtInt1(viDD, num_order, num_val);
	delete viDD;
	return res;
}
