////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// VirtInt11.cpp: implementation of the CVirtInt11 class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

const char* CVirtInt11::CLASS_NAME		= "CVirtInt11";
const char* CVirtInt11::CLASS_TAG_NAME	= "vi2";
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CVirtInt11::CVirtInt11(const CVIDensData* ddata, int n) : CVirtIntegral(ddata) {
	m_num = n;
	dbg->log(0, "Warning! CVirtInt11 oblect constructor involved");
	CEException::halt("Deprecated object (CVirtInt11) usage!");
}

CVirtInt11::~CVirtInt11() { }

CEString* CVirtInt11::toString(int mode) {
	CEString* res = NULL;
	if (getFlag()) res = new CEString("\\(");
	else res = new CEString("- \\(");

	switch (get_dens_data()->size())
	{
	case 1: res->AddString("A"); break;
	case 2: res->AddString("B"); break;
	case 3: res->AddString("C"); break;
	case 4: res->AddString("D"); break;
	case 5: res->AddString("E"); break;
	case 6: res->AddString("F"); break;
	default:
		res->AddString("X");
	}

	res->AddString("\\_\\(");
	res->AddString(GetHEPId());
	res->AddString("\\)\\)");
	
	//res->AddString("[");
	res->AddString("\\&");
	res->AddString(get_dens_data()->toString(mode));
	//res->AddString("]");
	return res;
}

CExpression* CVirtInt11::clone() const {
	CExpression* res = new CVirtInt11(get_dens_data_const(), m_num);
	copyAttributesTo(res);
	return res;
}

bool CVirtInt11::equals(CExpression* other) { return CExpression::equals(other); } // check toStrings()

CEString* CVirtInt11::GetHEPId() { // "?" for undefined, "0","1","2",11,21,111 ...
	const char* _id = NULL;
	switch (get_dens_data()->size())
	{
	case 1:
		break;
	case 2:
		switch (m_num) {
		case 1: _id = "11"; break;
		case 2: _id = "00"; break;
		}
		break;
	case 3:
		switch (m_num) {
		case 1: _id = "11"; break;
		case 2: _id = "22"; break;
		case 3: _id = "12"; break;
		case 4: _id = "00"; break;
		}
		break;
	case 4:
		break;
	}

	CEString* res = NULL;
	if (_id != NULL) {
		res = new CEString(_id);
	} else {
		res = new CEString("x");
		res->add_int(m_num);
	}
	return res;
}

CExpression* CVirtInt11::GetSingularTermFactor() { // NULL for unknown
	CExpression* res = NULL;
	CVIDensData* dd = get_dens_data();

	switch (dd->getDensNumber()) {
	case 1: // A2x (Error?)
		break;
	case 2: // B2x
		if (m_num == 1) {
			res = CENumber::GetFraction(1,3);
		} else if (m_num == 2) {
			CExpression* p2 = NULL;
			if (dd->isInvariant()) {
				p2 = dd->getInvValue(0)->clone();
			} else {
				CExpression* p0 = dd->momentum(0);
				p2 = new CESProduct(p0, p0);
			}

			CExpression* m12_sum = dd->mass(0)->sqra()->add(dd->mass(1)->sqra());
			
			res = (CENumber::GetFraction(1,12))->multiply( (CENumber::GetNumber(3))->multiply(m12_sum)->minus(p2) );
		}
		break;
	case 3: // C2x
		if (m_num == 4) {
			res = CENumber::GetFraction(1,4);
		} else {
			res = CENumber::GetZero();
		}
		break;
	case 4: // D2x
		res = CENumber::GetZero();
		break;
	default:
		res = NULL;
	}

	if (res != NULL) copyAttributesTo(res, true);
	return res;
}

CExpression* CVirtInt11::evaluate(int level) {
#if defined(D0_ONLY_CALC)
	if (get_dens_data()->getDensNumber() < 4) return CENumber::GetZero();
#endif //D0_ONLY_CALC

	CExpression* res = CVirtIntegralLib::get_stored(this);
	if (res != NULL) return res;

	// old evaluate, curent call is from CVirtInt1::evaluate(...)
	//res = CVirtIntegralLib::Evaluate_VI2(this);
	//if (res != NULL) return res;
	return this;
}

const char* CVirtInt11::ClassName() const { return CLASS_NAME; }
const char* CVirtInt11::get_TAG_Name() { return CLASS_TAG_NAME; }

long CVirtInt11::store_stream(ofstream *p_ofs, DWORD settings) {
	long res = CESupport::store_stream(get_dens_data(), p_ofs, settings);
	CENumber num_obj(m_num);
	res += CESupport::store_stream(&num_obj, p_ofs, settings);
	return res;
}

CESerializable* CVirtInt11::restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings) {
	// load DENSDATA
	long skip_pos = CESupport::skip_item_in_buffer(data, pos, settings);
	if (skip_pos <= 0) { dbg->log(0, "Error restoring CVirtInt1 : Can't load DENSDATA."); return NULL; }
	CESerializable* dd = CESupport::read_buffer(data, pos, settings);
	if ((dd == NULL) || (!CEUtil::equals(dd->ClassName(), CVIDensData::CLASS_NAME))) {
		dbg->log(0, "Error loading DENSDATA for CVirtInt11.");
		if (dd != NULL) delete dd;
		return NULL;
	}
	CVIDensData* viDD = (CVIDensData*)dd;

	// load ID number
	pos = skip_pos;
	skip_pos = CESupport::skip_item_in_buffer(data, pos, settings);
	if (skip_pos <= 0) { dbg->log(0, "Error restoring CVirtInt11 : Can't load ID number."); return NULL; }
	CESerializable* num_void_obj = CESupport::read_buffer(data, pos, settings);
	if ((num_void_obj == NULL) || (!CEUtil::equals(num_void_obj->ClassName(), CENumber::CLASS_NAME))) {
		dbg->log(0, "Error loading ID number for CVirtInt11.");
		delete viDD;
		if (num_void_obj != NULL) delete num_void_obj;
		return NULL;
	}

	// check up number value
	CENumber* num_obj = (CENumber*)num_void_obj;
	int num_val = num_obj->getNum();
	if ((num_val <= 0) || (num_obj->getDen() != 1) || (num_obj->getNumIm() != 0)) {
		dbg->log(0, "Invalid ID number loaded for CVirtInt11:", num_obj);
		delete viDD; delete num_obj; return NULL;
	}
	delete num_obj;

	pos = skip_pos;
	skip_pos = CESupport::skip_item_in_buffer(data, pos, settings);
	if (skip_pos > 0) { dbg->log(0, "Error restoring CVirtInt11: Extra data found."); delete viDD; return NULL; }

	CVirtInt11* res = new CVirtInt11(viDD, num_val);
	delete viDD;
	return res;
}
