////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// VirtIntegral.cpp: implementation of the CVirtIntegral class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

bool CVirtIntegral::EVALUATE_ALL_POSSBILE_VI	= false; //incl. C_ij-values --> may be very slow for massive vectors
bool CVirtIntegral::PAR_LOOPTOOLS_COMPARTIBLE	= false; //Forbid integrals not supported by LoopTools

CDebug* CVirtIntegral::dbg = new CDebug("CVirtIntegral");
const char* CVirtIntegral::CLASS_NAME = "CVirtIntegral";
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CVirtIntegral::CVirtIntegral(const CVIDensData* ddata) :CEAbstractFunction("VirtualIntegral") {
	if (ddata == NULL) {//Error.
		dbg->log(0, "Error: dens_data == NULL."); throw(new CEException(CEException::INT_ERROR));
	}
	m_dens_data = (CVIDensData*)ddata->clone();
}

CVirtIntegral::~CVirtIntegral() {
	if (m_dens_data != NULL) delete m_dens_data;
}

CEString* CVirtIntegral::GetHEPId() { return new CEString("?"); }

void CVirtIntegral::collectProperties(CEProperties* props, unsigned set) {
	CEAbstractFunction::collectProperties(props, set);
	props->setProperty(CEProperties::PROP_MASK_IS_VIRT_INTEGRAL, true);
}

unsigned long CVirtIntegral::props_long() {
	unsigned long res = CEAbstractFunction::props_long();
	res = CEProperties::set_property_to(res, CEProperties::PROP_MASK_IS_VIRT_INTEGRAL, true);
	return res;
}

bool CVirtIntegral::contain_vector(void * obj) {
	if (CEAbstractFunction::contain_vector(obj)) return true;
	if (m_dens_data != NULL) {
		if (m_dens_data == obj) return true;
		if (m_dens_data->contain_vector(obj)) return true;
	}
	return false;
}

bool CVirtIntegral::equals(CExpression* other) {
	//if (!other->getProperty(CEProperties::PROP_MASK_IS_VIRT_INTEGRAL)) return false;
	if (!CEUtil::equals(ClassName(), other->ClassName())) return false;
	if (!attributes_equals_to(other)) return false;
	if (!m_dens_data->equals(((CVirtIntegral*)other)->get_dens_data())) return false;
	return true;
}

CExpression* CVirtIntegral::expand(bool recursive) {
	CExpression* sub=CEAbstractFunction::expand(recursive);
	if((sub!=this) && (sub!=NULL)){ CExpression* res=sub->expand(recursive); if(res!=sub)delete sub; return res; }

	CExpression* dd2=m_dens_data->expand(recursive);

	if((dd2!=m_dens_data) && (dd2!=NULL)){
		if(dd2->ClassName()==CVIDensData::CLASS_NAME){ delete m_dens_data; m_dens_data=(CVIDensData*)dd2; }
		else{ dbg->log(0,"Internal error in CVirtIntegral::expand(). DD-expand result is wrong",dd2); CEException::halt(); }
	}
	return this;
}

CExpression* CVirtIntegral::applyRule(CEAbstractRule* rule)
{
	CERule* r = (CERule*)rule;
	if (m_dens_data != NULL) {
		CExpression* dd_new = r->ApplyTo(m_dens_data, false);
		if ((dd_new != m_dens_data) && (dd_new != NULL)) {
			if (CEUtil::equals(dd_new->ClassName(), CVIDensData::CLASS_NAME)) {
				delete m_dens_data;
				m_dens_data = (CVIDensData*)dd_new;
			}
			else dbg->log(0, "ERROR: rule->ApplyTo(DENSDATA) != DENSDATA", m_dens_data, dd_new);
		}
	}
	return this;
}

const char* CVirtIntegral::ClassName() const { return CLASS_NAME; }

CExpression* CVirtIntegral::GetSingularTermFactor() { return NULL; } // NULL for unknown
DWORD CVirtIntegral::GetResultType() {  return CEAbstractFunction::PAR_COMPLEX; }
