////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// VirtIntegralLib.cpp: implementation of the CVirtIntegralLib class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

CDebug* CVirtIntegralLib::dbg = new CDebug("CVirtIntegralLib");

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CVirtIntegralLib::CVirtIntegralLib() : CELibrary() { }
CVirtIntegralLib::~CVirtIntegralLib() { }

//////////////////////////////////////////////////////////////////////
// Static VI engine initialization
//////////////////////////////////////////////////////////////////////
void CVirtIntegralLib::fill_data_storage()
{
	dbg->log(0, "WARNING: DEPRECATED CODE INVOLVED, VI predefined data initialization.");
	CEException::halt("DEPRECATED CODE INVOLVED");
	//integrals
	CEData* data = CEProcessor::getDataStorage();
	CExpression* a0 = CVirtInt0::get_a0m_cpy();
	CExpression* zero = data->gete(CConst::DV_ZERO);
	CExpression* unit = data->gete(CConst::DV_UNIT);
	CExpression* two = data->gete(CConst::DV_TWO);
	CExpression* half = data->gete(CConst::DV_HALF);
	//CExpression* del = data->gete(CConst::GL_DELTA);
	CExpression* phi = data->gete(CConst::GL_PHI);
	CExpression* phu = new CESymbol("f\\_u");
	//CExpression* psi = data->gete(CConst::GL_PSI);
	CExpression* ln_ts = new CESymbol("l\\_t");
	CExpression* ln_tu = new CESymbol("l\\_u");

	CExpression* p1 = new CESymbol("p\\_1");
	CExpression* p1m = p1->clone(); p1m->inverseFlag();
	CExpression* p2 = new CESymbol("p\\_2");
	CExpression* p2m = p2->clone(); p2m->inverseFlag();

	CExpression* p11 = new CESymbol("f\\_1");
	CExpression* p11m = p11->clone(); p11m->inverseFlag();
	CExpression* p21 = new CESymbol("f\\_2");

	CExpression* m = new CESymbol("m\\_e");
	CExpression* mZ = new CESymbol("m\\_Z");

	//CExpression* s = new CESymbol("s");
	//CExpression* t = new CESymbol("t");
	//CExpression* u = new CESymbol("u");

	CExpression* p11d1 = p11->clone()->minus(p1->clone());
	CExpression* p1d11 = p1->clone()->minus(p11->clone());
	CExpression* p21d2 = p21->clone()->minus(p2->clone());
	CExpression* p2d21 = p2->clone()->minus(p21->clone());

	CExpression* p11d2 = p11->clone()->minus(p2->clone());
	CExpression* p2d11 = p2->clone()->minus(p11->clone());
	CExpression* p21d1 = p21->clone()->minus(p1->clone());
	CExpression* p1d21 = p1->clone()->minus(p21->clone());
	CExpression* p1p2 = p1->clone()->add(p2->clone());
	CExpression* p11p21 = p11->clone()->add(p21->clone());

	CExpression* vi_B00mm = a0->clone()->minus(unit->clone());
	CExpression* vi_B10mm = vi_B00mm->clone()->multiply(half->clone());
	vi_B10mm->inverseFlag();
	CExpression* vi_B0t0m = a0->clone()->add(unit->clone())->minus(phi->clone());
	CExpression* vi_B0u0m = a0->clone()->add(unit->clone())->minus(phu->clone());
	CExpression* vi_B0m0m = a0->clone()->add(unit->clone());
	CExpression* vi_B0smm = vi_B0t0m->clone()->add(ln_ts->clone());
	
	CSequence* ds = new CSequence();
	//CSequence* ns = new CSequence();
	CSequence* ts1 = new CSequence();
	ts1->addItem(zero->clone());
	ts1->addItem(m->clone());
	ds->addItem(ts1);

	// A(m)
	CExpression* a_val = a0->clone()->multiply(m->sqra());
	batch_vi(CVIComposeLib::createVISeq(ds, NULL, NULL), a_val);
	delete a_val;

	ts1->SetAt(1, mZ); // A(m_Z)
	a_val = a0->clone()->multiply(mZ->sqra());
	batch_vi(CVIComposeLib::createVISeq(ds, NULL, NULL), a_val); delete a_val;

	ts1->SetAt(1, zero); // A(0)
	batch_vi(CVIComposeLib::createVISeq(ds, NULL, NULL), zero);
	ts1->SetAt(1, m);

	//B(0,m,m)
	CSequence* ts2 = new CSequence();
	ts2->addItem(p1->clone());
	ts2->addItem(m->clone());
	ds->addItem(ts2);

	CVIDensData* vidd = NULL;

	//batch_vi(CVIComposeLib::createVISeq(ds, NULL, NULL), vi_B00mm);
	batch_vi((CVirtIntegral*)CExpression::arrge(CVIComposeLib::createVISeq(ds, NULL, NULL)), vi_B00mm);
	//B'(0,m,m)
	vidd = new CVIDensData(ds); batch_vi(new CVirtInt1(vidd, 1, 1), vi_B10mm); delete vidd;

	//B(-p1,m,m)
	ts2->SetAt(0, p1m);
	batch_vi(CVIComposeLib::createVISeq(ds, NULL, NULL), vi_B00mm);
	vidd = new CVIDensData(ds); batch_vi(new CVirtInt1(vidd, 1, 1), vi_B10mm); delete vidd;

	//B(p2,m,m)
	ts2->SetAt(0, p2);
	batch_vi(CVIComposeLib::createVISeq(ds, NULL, NULL), vi_B00mm);
	vidd = new CVIDensData(ds); batch_vi(new CVirtInt1(vidd, 1, 1), vi_B10mm); delete vidd;

	ts2->SetAt(0, p2m);
	vidd = new CVIDensData(ds); batch_vi(new CVirtInt1(vidd, 1, 1), vi_B10mm); delete vidd;

	//B(p1+p2,m,m)
	ts2->SetAt(0, p1p2);
	//batch_vi(CVIComposeLib::createVISeq(ds, NULL, NULL), vi_B0smm);
	batch_vi((CVirtIntegral*)CExpression::arrge(CVIComposeLib::createVISeq(ds, NULL, NULL)), vi_B0smm);

	//B(p11+p21,m,m)
	ts2->SetAt(0, p11p21);
	//batch_vi(CVIComposeLib::createVISeq(ds, NULL, NULL), vi_B0smm);

	//B(t,0,m)
	ts2->SetAt(0, p11d1);
	ts1->SetAt(1, zero);
	//batch_vi(CVIComposeLib::createVISeq(ds, NULL, NULL), vi_B0t0m);
	batch_vi((CVirtIntegral*)CExpression::arrge(CVIComposeLib::createVISeq(ds, NULL, NULL)), vi_B0t0m);
	//B(t,0,m)
	ts2->SetAt(0, p1d11);
	//batch_vi(CVIComposeLib::createVISeq(ds, NULL, NULL), vi_B0t0m);
	ts2->SetAt(0, p21d2);
	//batch_vi(CVIComposeLib::createVISeq(ds, NULL, NULL), vi_B0t0m);
	ts2->SetAt(0, p2d21);
	//batch_vi(CVIComposeLib::createVISeq(ds, NULL, NULL), vi_B0t0m);

	//B(u,0,m)
	ts2->SetAt(0, p21d1);
	ts1->SetAt(1, zero);
	//batch_vi(CVIComposeLib::createVISeq(ds, NULL, NULL), vi_B0u0m);
	batch_vi((CVirtIntegral*)CExpression::arrge(CVIComposeLib::createVISeq(ds, NULL, NULL)), vi_B0u0m);
	ts2->SetAt(0, p1d21);
	//batch_vi(CVIComposeLib::createVISeq(ds, NULL, NULL), vi_B0u0m);
	ts2->SetAt(0, p11d2);
	//batch_vi(CVIComposeLib::createVISeq(ds, NULL, NULL), vi_B0u0m);
	ts2->SetAt(0, p2d11);
	//batch_vi(CVIComposeLib::createVISeq(ds, NULL, NULL), vi_B0u0m);

	//B(m^2,0,m)
	ts2->SetAt(0, p11);
	//batch_vi(CVIComposeLib::createVISeq(ds, NULL, NULL), vi_B0m0m);
	batch_vi((CVirtIntegral*)CExpression::arrge(CVIComposeLib::createVISeq(ds, NULL, NULL)), vi_B0m0m);
	//B(-p11,0,m)
	ts2->SetAt(0, p11m);
	//batch_vi(CVIComposeLib::createVISeq(ds, NULL, NULL), vi_B0m0m);
	//B(p21,0,m)
	ts2->SetAt(0, p21);
	//batch_vi(CVIComposeLib::createVISeq(ds, NULL, NULL), vi_B0m0m);

	delete p1;
	delete p1m;
	delete p2;
	delete p2m;
	delete p11;
	delete p11m;
	delete p21;
	delete p11d1;
	delete p1d11;
	delete p21d2;
	delete p2d21;
	delete p11d2;
	delete p2d11;
	delete p21d1;
	delete p1d21;
	delete p1p2;
	delete p11p21;
	delete m;
	delete mZ;
	delete a0;
	delete ln_ts;
	delete ln_tu;
	delete ds;
	delete phu;

	delete vi_B00mm;
	delete vi_B10mm;
	delete vi_B0t0m;
	delete vi_B0u0m;
	delete vi_B0m0m;
	delete vi_B0smm;

	dbg->log(40, "VI data storage filled.");
}

void CVirtIntegralLib::batch_vi(CVirtIntegral *vi, CExpression *vi_val, bool do_delete_vi) {

	// arrange VI befire storage
	CVirtIntegral* vi_cpy  = (CVirtIntegral*)vi->clone();
	vi_cpy->Arrange(AL_INLINE);

	dbg->log(2, "VI batching", vi_cpy); dbg->log(2, "VI val", vi_val);
	store_vi(vi_cpy, vi_val);
	delete vi_cpy;
	if (do_delete_vi) delete vi;
}

//////////////////////////////////////////////////////////////////////
// VI Utilites
//////////////////////////////////////////////////////////////////////

/*
*	1. Calls DENSDATA sorting
*	2. Changes virtual variable to cast first momentum in DENSDATA to zero.
*	3. Sorts DENSDATA again.
*	4. Performs the same variable change in numerator expression.
*	5. Returns the new numerator expression.
*	Returns NULL and does not delete input if error occurs.
*
*	Kills 'num' object and returns the new 'num' value (or original object).
*	Changes 'dens_data' object inline.
*/
CExpression* CVirtIntegralLib::normalize_integral(CVIDensData* dens_data,
												  CExpression* num,
												  CExpression* virt_sym) {
	dbg->log(40, "normalize_integral()", dens_data);
	if ((dens_data == NULL) || (num == NULL) || (virt_sym == NULL)) return NULL;

	dens_data->sort();
	CExpression* p_del = dens_data->get_norm_p_del();
	//CExpression* p_del = dens_data->SortAndGetNormPDel();
	if (p_del == NULL) {
		// DENSDATA structure is normalized.
		return num;
	}
	p_del = p_del->clone();

	//changing variable k->k-p in DENSDATA;
	dens_data->change_var_minus_pdel(p_del);

	//changing variable k->k-p in the numerator expression;
	CExpression* k_new = CExpression::reduce(virt_sym->clone()->minus(p_del));
	dbg->log(50, "changing variable during normalizing (k_new)", k_new);
	CEReplList* rule = new CEReplList(virt_sym, k_new, false);
	num = CExpression::reduce(rule->ApplyTo(num));
	delete rule;
	delete k_new;
	dbg->log(50, "Virtual variable changed in numerator :", num);
	return num;
}

//////////////////////////////////////////////////////////////////////
// VI Evaluation Cache
//////////////////////////////////////////////////////////////////////

/*
*	Returns cloned value or NULL.
*/
CExpression* CVirtIntegralLib::get_stored(CVirtIntegral* vi) {
	if (vi == NULL) return NULL;

	bool sign = vi->getFlag();
	if (!sign) { vi=(CVirtIntegral*)vi->clone(); vi->setFlag(); }

	CEString* id_str = vi->toString();
	CExpression* res = CEProcessor::getDataStorage()->gete(id_str->getData());
	delete id_str;

	if (res != NULL) res = res->clone();
	if (!sign) {
		vi->inverseFlag(); delete vi;
		if (res != NULL) res->inverseFlag();
	}
	return res;
}

/*
*	Does not affect input parameters.
*/
void CVirtIntegralLib::store_vi(CVirtIntegral* vi, CExpression* vi_val) {
	if ((vi == NULL) || (vi_val == NULL)) return;

	CExpression* vi_val_copy = vi_val->clone();

	bool sign = vi->getFlag();
	if (!sign) { vi = (CVirtIntegral*)vi->clone(); vi->setFlag(); vi_val_copy->inverseFlag(); }

	CEString* id_str = vi->toString();
	CEProcessor::getDataStorage()->pute(id_str->getData(), vi_val_copy);
	delete id_str;

	if (!sign) { vi->inverseFlag(); delete vi; }
}

//////////////////////////////////////////////////////////////////////
// VI Evaluation Procedures
//////////////////////////////////////////////////////////////////////
CExpression* CVirtIntegralLib::Evaluate_VI1(CVirtInt1* vi) {
	dbg->log(20, "Evaluate_VI1:", vi);
	if (vi == NULL) return NULL;

	if (CVirtIntegral::EVALUATE_ALL_POSSBILE_VI) {
		bool eval_OK = EvaluateVISet1(vi->get_dens_data());
		if (eval_OK) {
			CExpression* res = get_stored(vi);
			if (res != NULL) return res;
			dbg->log(0, "Warning: can'f find VI evaluated after evaluation performed.");
		}
	}
	store_vi(vi, vi); //Store self to avoid further evaluation attempts
	return NULL;
}

CExpression* CVirtIntegralLib::Evaluate_VI2(CVirtInt1* vi) {
	dbg->log(20, "Evaluate_VI11:", vi);
	if (vi == NULL) return NULL;
	CExpression* res = NULL;

	CVIDensData* dd = vi->get_dens_data();
	int d_num = dd->size();
	if (d_num == 3) { // Implemented for C_xy only
		if (CVirtIntegral::EVALUATE_ALL_POSSBILE_VI) { //C_ij is 'complicated' in fully-massive case
			bool env_dim_is_N = CETensor::isNDimensionSpace();
			if(!env_dim_is_N){ CETensor::setNDimensionSpace(true); }
			
			bool eval_OK = EvaluateVISet2(vi->get_dens_data());
			if (eval_OK) {
				res = get_stored(vi);
				if (res == NULL) dbg->log(0, "Warning: can'f find VI evaluated after evaluation performed.");
			}
			if (!env_dim_is_N) CETensor::setNDimensionSpace(env_dim_is_N); // restore dimension
		}
		if(res==NULL) store_vi(vi, vi); //Store self to avoid further evaluation attempts
	}

	return res;
}

/*
*	//Do not evaluate B'(x,x,x)  (?)
*	Does not evaluates B'(0,x,x)
*/
bool CVirtIntegralLib::EvaluateVISet1(CVIDensData* dens_data) {
	dbg->log(20, "evaluating VI-1 coefficients set, DENSDATA:", dens_data);
	if (dens_data->isInvariant()) { dbg->log(0, "<ERROR> EvaluateVISet1() for invariant DENSDATA", dens_data); return false; }
	//CERule* r_mass = CLHEPBasics::get_mass_order_rule();
	int s = dens_data->size();

	CSequence* arr = new CSequence();
	CSequence* col = new CSequence();

	//CEProcessor::getEnvironment()->SetMassExactCalc();
	int i;
	for (i=1; i<s; i++) {
		CSequence* sub_col = new CSequence();
		int j;
		for (j=1; j<s; j++) {
			CExpression* item = new CESProduct(dens_data->momentum(i), dens_data->momentum(j));
			item = CExpression::reduce(item);
			item = CEProcessor::kinematics()->get_simpliest(item);
			sub_col->addItem(item);
		}
		arr->addItem(sub_col);
		dbg->log(40, "sub_col filled", sub_col);

		CSequence* seq_tmp = new CSequence();
		seq_tmp->addItem(dens_data->momentum(i)->clone());
		CExpression* c_int = CVIComposeLib::createVI(dens_data, seq_tmp, NULL);
		delete seq_tmp;
		CExpression* c_int_ev = c_int->evaluate(); if (c_int_ev != c_int) delete c_int;
		//CEProcessor::getEnvironment()->RestoreMassCalcOrders();
		c_int_ev = CVIScalarCalcLib::CalcScalarVI(c_int_ev); // FORBIDDED CALL DURING SetMassExactCalc().
		//CEProcessor::getEnvironment()->SetMassExactCalc();
		dbg->log(40, "sub_res filled", c_int_ev);
		c_int_ev = CExpression::reduce(c_int_ev);
		col->addItem(c_int_ev);
	}
	//CEProcessor::getEnvironment()->RestoreMassCalcOrders();

	dbg->log(30, "arr filled", arr);
	dbg->log(30, "col filled", col);

	//check out for B(0,x,x)
	if (s == 2) {
		CExpression* p_squared = ((CSequence*)(arr->GetAt(0)))->GetAt(0);
		if (CEBasic::isZero(p_squared)) {
			// special case: B(0,m,m)
			CExpression* m1 = dens_data->mass(0);
			CExpression* m2 = dens_data->mass(1);
			if ((m1->equals(m2)) && (!CEBasic::isZero(m1))) {
				// B1(0,m,m) = -(1/2)*B0(0,m,m)
				CVirtInt1* vi = new CVirtInt1(dens_data, 1, 1);
				CExpression* vi_val = new CVirtInt0(dens_data);
				vi_val = vi_val->multiply(CENumber::GetHalf());
				vi_val->inverseFlag();

				// standard (common) simplification & storage
				vi_val = CLHEPBasics::KinArrange(vi_val, CLHEPBasics::KA_MASS_EXACT);
				vi_val = CLHEPBasics::KinSimplify(vi_val, CLHEPBasics::KS_MASS_EXACT | CLHEPBasics::KS_FACTORIZE);
				CExpression* val_evaltd = vi_val->evaluate();
				if (val_evaltd != vi_val) { delete vi_val; vi_val = val_evaltd; }
				vi_val = CVIScalarCalcLib::CalcScalarVI(vi_val);
				// simplify including mass rule
				vi_val = CLHEPBasics::KinArrange(vi_val, CLHEPBasics::KA_INVOLVE_MANDELSTAM_VARS);
				vi_val = CLHEPBasics::KinSimplify(vi_val, CLHEPBasics::KS_INVOLVE_MANDELSTAM_VARS);
				store_vi(vi, vi_val);
				delete vi_val;
				delete vi;
				delete arr; delete col; return true;
			}

			dbg->log(20, "Unsolvable B\\_1(0,x,x)", dens_data);
			//CVirtIntegral::store_vi(this, this);
			delete arr;
			delete col;
			return false;
		}

		// U-rev check
		bool AZ_case = false; // X/0 limit check
		CExpression* tmp_val = CLHEPBasics::ApplyMassOrder(p_squared->clone());
		if (CEBasic::isZero(tmp_val)) { AZ_case = true; }
		delete tmp_val;

		bool ZA_case = false; // 0/X limit check
		tmp_val = CLHEPBasics::ApplyMassOrder(col->GetAt(0)->clone(), CLHEPBasics::MO_EXPAND);
		if (CEBasic::isZero(tmp_val)) { ZA_case = true; }
		delete tmp_val;

		bool allow_case = true;
		if (AZ_case) {
			allow_case = false;

			if (ZA_case) {
				// ignore ZZ_case for common case (to avoid 0/(0^2) cases)
				CExpression* tmp_m2 = CExpression::reduce(dens_data->mass(1)->sqra());
				if ((CEBasic::isZero(dens_data->mass(0))) && (p_squared->equals(tmp_m2))) {
					// But allow 0/0 case for B1(m^2, 0, m)
					allow_case = true;
				}
				delete tmp_m2;
			}
		}

		if (!allow_case) {
			dbg->log(20, "Zero-limit in B\\_1() resolution, evaluation skipped", dens_data);
			delete arr;
			delete col;
			return false;
		}
	}

	CSequence* res = CLMath::linear_system_solve(arr, col);
	delete arr;
	delete col;
	if (res == NULL) return false;

	//reducing res
	for (i=0; i<res->size(); i++) {
		CExpression* item = CExpression::reduce(res->GetAt(i)->clone());
		res->SetAt(i, item);
		delete item;
	}
	dbg->log(30, "result col", res);

	for (i=0; i<res->size(); i++) {
		CVirtInt1* vi = new CVirtInt1(dens_data, 1, i+1);
		CExpression* vi_val = res->GetAt(i)->clone();

		vi_val = CLHEPBasics::KinArrange(vi_val, CLHEPBasics::KA_MASS_EXACT);
		vi_val = CLHEPBasics::KinSimplify(vi_val,
			CLHEPBasics::KS_MASS_EXACT | CLHEPBasics::KS_FACTORIZE);
		CExpression* val_evaltd = vi_val->evaluate();
		if (val_evaltd != vi_val) { delete vi_val; vi_val = val_evaltd; }
		vi_val = CVIScalarCalcLib::CalcScalarVI(vi_val);

		//vi_val = CHEPLib::ur_reduce(vi_val);
		// simplify including mass rule
		vi_val = CLHEPBasics::KinArrange(vi_val, CLHEPBasics::KA_INVOLVE_MANDELSTAM_VARS);
		vi_val = CLHEPBasics::KinSimplify(vi_val, CLHEPBasics::KS_INVOLVE_MANDELSTAM_VARS);
		store_vi(vi, vi_val);
		delete vi_val;
		delete vi;
	}
	delete res;
	dbg->log(20, "VI-1 coefficients set evaluated successfull.");
	return true;
}

// Evaluates C_xy oefficients only
bool CVirtIntegralLib::EvaluateVISet2(CVIDensData* dens_data) {
	dbg->log(20, "evaluating VI-2 coefficients set, DENSDATA:", dens_data);
	if (dens_data->isInvariant()) { dbg->log(0, "<ERROR> EvaluateVISet2() for invariant DENSDATA", dens_data); return false; }
	//CERule* r_mass = CLHEPBasics::get_mass_order_rule();
	int s = dens_data->size();
	if (s != 3) return false;

	CSequence* arr = new CSequence();
	CSequence* col = new CSequence();

	CExpression* p1 = dens_data->momentum(1);
	CExpression* p2 = dens_data->momentum(2);
	CExpression* p1p2 = CEProcessor::kinematics()->get_simpliest(
		CExpression::reduce(new CESProduct(p1, p2)));
	CExpression* p12 = CEProcessor::kinematics()->get_simpliest(
		CExpression::reduce(new CESProduct(p1, p1)));
	CExpression* p22 = CEProcessor::kinematics()->get_simpliest(
		CExpression::reduce(new CESProduct(p2, p2)));
	CExpression* two = CENumber::GetTwo();

	//1.
	CSequence* sub_col = new CSequence();
	sub_col->addItem(p12->sqra());
	sub_col->addItem(p1p2->sqra());
	sub_col->addItem(p12->clone()->multiply(p1p2->clone()));
	sub_col->addItem(p12->clone());
	arr->addItem(sub_col);
	
	CSequence* seq_tmp = new CSequence();
	seq_tmp->addItem(p1->clone());
	seq_tmp->addItem(p1->clone());
	CExpression* c_int = CVIComposeLib::createVI(dens_data, seq_tmp, NULL);
	delete seq_tmp;
	CExpression* c_int_ev = c_int->evaluate();
	if (c_int_ev != c_int) delete c_int;
	c_int_ev = CVIScalarCalcLib::CalcScalarVI(c_int_ev);
	c_int_ev = CExpression::reduce(c_int_ev);
	col->addItem(c_int_ev);
	
	//2.
	sub_col = new CSequence();
	sub_col->addItem(p1p2->sqra());
	sub_col->addItem(p22->sqra());
	sub_col->addItem(p22->clone()->multiply(p1p2->clone()));
	sub_col->addItem(p22->clone());
	arr->addItem(sub_col);

	seq_tmp = new CSequence();
	seq_tmp->addItem(p2->clone());
	seq_tmp->addItem(p2->clone());
	c_int = CVIComposeLib::createVI(dens_data, seq_tmp, NULL);
	delete seq_tmp;
	c_int_ev = c_int->evaluate();
	if (c_int_ev != c_int) delete c_int;
	c_int_ev = CVIScalarCalcLib::CalcScalarVI(c_int_ev);
	c_int_ev = CExpression::reduce(c_int_ev);
	col->addItem(c_int_ev);
	
	//3.
	sub_col = new CSequence();
	sub_col->addItem(p12->clone()->multiply(p1p2->clone())->multiply(two->clone()));
	sub_col->addItem(p22->clone()->multiply(p1p2->clone())->multiply(two->clone()));
	sub_col->addItem(p1p2->sqra()->add(p12->clone()->multiply(p22->clone())));
	sub_col->addItem(p1p2->clone()->multiply(two->clone()));
	arr->addItem(sub_col);

	seq_tmp = new CSequence();
	seq_tmp->addItem(p1->clone());
	seq_tmp->addItem(p2->clone());
	c_int = CVIComposeLib::createVI(dens_data, seq_tmp, NULL);
	delete seq_tmp;
	c_int_ev = c_int->evaluate();
	if (c_int_ev != c_int) delete c_int;
	c_int_ev = CVIScalarCalcLib::CalcScalarVI(c_int_ev);
	c_int_ev = CExpression::reduce(c_int_ev);
	col->addItem(c_int_ev);
	
	//4.
	sub_col = new CSequence();
	sub_col->addItem(p12->clone());
	sub_col->addItem(p22->clone());
	sub_col->addItem(p1p2->clone());
	//Note: manual N-dim correction is performed further (by adding 1/2 etc., e.g. see Kahane)!
	sub_col->addItem(CENumber::GetFour()); //sub_col->addItem(CETensor::getDimensionVal());
	arr->addItem(sub_col);
	
	//////////////////////////////////////////////
	/// C_mn*g^mn ==> (k^2 - M^2) + M^2 ==> B_0 + M^2*C_0
	CVIDensData* dd_new = (CVIDensData*)dens_data->clone();
	CExpression* m02 = dd_new->mass(0)->sqra();
	dd_new->RemoveAt(0);
	CExpression* int4 = CVIComposeLib::createVI(dd_new, NULL, NULL); delete dd_new; // B_0
	CExpression* int4_temp = CVIComposeLib::createVI(dens_data, NULL, NULL); // C_0
	int4 = int4->add(m02->multiply(int4_temp));
	c_int_ev = int4->evaluate();
	if (c_int_ev != int4) delete int4;
	c_int_ev = CVIScalarCalcLib::CalcScalarVI(c_int_ev);
	c_int_ev = CExpression::reduce(c_int_ev);
	// manual N-dim correction:
	c_int_ev = c_int_ev->add(CENumber::GetHalf());
	col->addItem(c_int_ev);

	delete p1p2;
	delete p12;
	delete p22;
	delete two;

	dbg->log(30, "arr filled", arr);
	dbg->log(30, "col filled", col);

	CSequence* res = CLMath::linear_system_solve(arr, col);
	delete arr;
	delete col;
	//reducing res
	/*for (i=0; i<res->size(); i++) {
		CExpression* item = CExpression::reduce(res->GetAt(i)->clone());
		res->SetAt(i, item);
		delete item;
	}*/

	dbg->log(30, "result col", res);

	if (res == NULL) return false;
	int i;
	for (i=0; i<res->size(); i++) {
		CVirtInt1* vi = new CVirtInt1(dens_data, 2, i+1);
		//dbg->log(30, i, "vi calc(%ld)", vi); //TEMP

		//if (i==1) CLHEPBasics::dbg->set_level(100); //TEMP
		CExpression* res_i = res->GetAt(i)->clone();
		res_i=CExpression::arrge(res_i);

		CExpression* vi_val = CLHEPBasics::KinSimplify(res_i,
			CLHEPBasics::KS_MASS_EXACT | CLHEPBasics::KS_FACTORIZE);
		//if (i==1) CLHEPBasics::dbg->set_level(5); //TEMP
		//dbg->log(30, "vi value", vi_val); //TEMP

		CExpression* val_evaltd = vi_val->evaluate(); // calc B_1 - integrals
		//dbg->log(30, "vi val_evaltd", val_evaltd); //TEMP
		if (val_evaltd != vi_val) { delete vi_val; vi_val = val_evaltd; }
		vi_val = CVIScalarCalcLib::CalcScalarVI(vi_val); // calc X_0 - integrals

		vi_val = CLHEPBasics::KinArrange(vi_val, CLHEPBasics::KA_INVOLVE_MANDELSTAM_VARS);
		dbg->log(30, "vi value to be stored", vi, vi_val);
		store_vi(vi, vi_val);
		delete vi_val;
		delete vi;
	}
	delete res;
	dbg->log(30, "Evaluate COMPLETED"); //TEMP
	return true;
}

CExpression* CVirtIntegralLib::ReduceTensorVI(CExpression* src) {
	CExpression* res = src->evaluate();
	if(res!=src)delete src;
	return res;
}

CExpression* CVirtIntegralLib::ConvertVI0ToInvForm(CExpression* src) {
	dbg->log(15, "ConvertVI0ToInvForm()", src);
	if (src == NULL) return NULL;

	//////////////////////////////////////////
	CERule* r = new CESocketRule((void*)CVirtIntegralLib::ConvertVI0Inv_ruleImpl);
	src = r->ApplyTo(src);
	delete r;
	//////////////////////////////////////////

	dbg->log(25, "ConvertVI0ToInvForm() result", src);
	return src;
}

CExpression* CVirtIntegralLib::ConvertVI0Inv_ruleImpl(CExpression* src, CEPtrArray* data_arr) {
	if (src == NULL) return src;
	if (CEUtil::equals(src->ClassName(), CVirtInt0::CLASS_NAME)) {
		CVirtInt0* vi = (CVirtInt0*)src;
		vi->ConvertInvariant(); // is-converted check is therein
		return vi; // return source
	} else if (CEUtil::equals(src->ClassName(), CVirtInt1::CLASS_NAME)) {
		CVirtInt1* vi = (CVirtInt1*)src;
		vi->ConvertInvariant(); // is-converted check is therein
		return vi; // return source
	}

	return src;
}
