////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// AmplCalcLib.h: interface for the CAmplCalcLib class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_AMPLCALCLIB_H__AFC7CFAD_CB8E_49CF_8AE6_7C5B0BFFC7DA__INCLUDED_)
#define AFX_AMPLCALCLIB_H__AFC7CFAD_CB8E_49CF_8AE6_7C5B0BFFC7DA__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CAmplCalcLib : public CELibrary  
{
	DWORD m_settings;
	CMatrixElement* m_me; // cloned object

	// Values for calculation
	CESymbol* m_CalcBasisVecSym;
	CESymbol* m_CalcBasisMassSym; // NULL for massless (for faster check on massless)
	CESymbol* m_CalcBasisVectorE; // (polarization) NULL for +/- polarized?
	CExpression* m_BasisSpinorDM;
	CERule* m_BS_rule; // basis spinor properties // (to set pp->m, ect.)

	CECache* m_DMCache; // unused now, old: CSequence* m_DMCache; // SEQ{...SEQ{p,DM}...} 
	CDenCacheLib* m_DensCache;

	CKinematics* m_kin;

	CERule* m_mass_rule;
	CEReplList* m_kin_simp_rules;
	CEReplList* m_basic_coupling_vals;
	CEReplList* m_mand_vars_rule;

public:
	static CDebug* dbg;

public:
	static const DWORD A_MANDELSTAMS;
	static const DWORD A_NO_CROSSINGS;
	static const DWORD A_USE_FIERZ_TRANSFORM;
	static const DWORD A_SIMPLE_BASIS; // unpolarized & massless

public:
	static CExpression* CalcAmplitude(CMatrixElement* me, DWORD settings); // me not affected
	virtual ~CAmplCalcLib();

	static CESymbol* m_BasisVecSymM;
	static CESymbol* m_BasisVecSym0;
	static CESymbol* m_BasisVecPol;
	static CESymbol* m_BasisMassSym;
	static CESymbol* GetBasisVecSym(bool _mom, bool _massive); // bool do_init=true
	static CESymbol* GetBasisMassSym(); //bool do_init=true
	static void InitBasisSyms();

private:
	CAmplCalcLib();

	// basic procedure, result is internal ME value
	void doCalcAmplitude(CMatrixElement* me, DWORD settings);
	CMatrixElement* ExtractME(); // to retrieve ME result
	CExpression* GetMEExpr(); // to retrieve Expr result

	void calcWholeME(); // parses sum & processes me item-by-item
	CExpression* calcSingleME(CExpression* me); // may change src, but not delete

	void calcFermionSeqs(CExpression* me_expr);
	void calcFermionSeqsPr(CEProduct* prod);
	CExpression* calcSeqSubst(CEProduct* uu_seq); // source may be changed, but not deleted, result is cloned
	CExpression* findFerionDM(CExpression* p);
	CExpression* kin_reduce(CExpression* val, bool do_expand_res=true, bool do_expand_src=true, bool del_src=true);

	CExpression* getCachedDM(CExpression* p);
	void addDMToCache(CExpression* p, CExpression* dm);

	bool existMasslessFermion(); // check if some of in/out fermions are massless ==> massless calc
};

#endif // !defined(AFX_AMPLCALCLIB_H__AFC7CFAD_CB8E_49CF_8AE6_7C5B0BFFC7DA__INCLUDED_)
