////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// LHEPBasics.h: interface for the CLHEPBasics class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_LHEPBASICS_H__181D4182_17EE_4768_A8FB_D16C414F4F48__INCLUDED_)
#define AFX_LHEPBASICS_H__181D4182_17EE_4768_A8FB_D16C414F4F48__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CLHEPBasics : public CELibrary  
{
	CLHEPBasics(); // unused
public:
	virtual ~CLHEPBasics(); // unused
	static CDebug* dbg;
	static void Finalize();

	/////////////////////////////////////////////////////////// Mass Order
	static CERule* get_mass_order_rule();
	static const int MO_EXPAND;
	static const int MO_NO_ZERO;
	static CExpression* ApplyMassOrder(CExpression* source, int settings = 0);

	/////////////////////////////////////////////////////////// Kin-Arrange
	static const int KA_MASS_EXACT;
	static const int KA_INVOLVE_MANDELSTAM_VARS;
	static const int KA_NO_EXPAND;
	static CExpression* KinArrange(CExpression* source, int settings = 0);

	/////////////////////////////////////////////////////////// Kin-Simplify
	static const int KS_MASS_EXACT;
	static const int KS_INVOLVE_MANDELSTAM_VARS;
	static const int KS_NO_EXPAND;
	static const int KS_FACTORIZE; // functions & all denominators
	static const int KS_FACTORIZE_FUNCTIONS;
	static const int KS_FACTORIZE_DENOMINATORS;
	static const int KS_FACTORIZE_ALL_DENOMINATORS;
	static const int KS_MOMENTA_SUBST;
	static const int KS_NO_COUPLINGS_SUBST;
	static const int KS_FAST; //simplify small sub-objects only
	static CSequence* m_ConstantsList;
	static void InitConstantsList(); // fill 'm_ConstantsList' if NULL
	static CExpression* KinSimplify(CExpression* source, int settings = 0);
	//static CExpression* KinSimplifyFast(CExpression* source, int settings = 0); // to be implemented

	// Avoid last vector in kinematics ==> uniform vectors set
	static CExpression* KinToUniformVectors(CExpression* source);

	/////////////////////////////////////////////////////////// IR-limit of Loops
	static void GetIRLimit(CMatrixElement* source);
	static CExpression* IRLimitRuleProc(CExpression* src, CEPtrArray* data_arr);

	/////////////////////////////////////////////////////////// UV-arrange
	static CExpression* SingularArrange(CExpression* source);
	static CExpression* GetUVTerm(CExpression* source);

private:
	static CExpression* TruncateDeltaPower(CExpression* source, int iStartFromPow); // set delta^x-->0s if(x>=iStartFromPow)
	static CExpression* SingularArrangeImpl(CExpression* src, CExpression* dimDeltaSym, CExpression* singSym, CExpression* singIRSym);
	static CExpression* SingularArrangeProduct(CEProduct* src, CExpression* dimDeltaSym, CExpression* singSym, CExpression* singIRSym);

public:
	/////////////////////////////////////////////////////////// Comparison Procedures
	static bool IsAGGthanB_abs(CExpression* a, CExpression* b); // A >> B check (abs only)
	static bool IsALLthanB_abs(CExpression* a, CExpression* b); // A << B check (abs only)
	static int GetExprSign(CExpression* src); // 0 means unknown result

	/////////////////////////////////////////////////////////// me2-->diffCS
	static CExpression* GetDiffCS(CExpression* me2, bool bGeV2Pb, bool bPostArrange); // destroys source

	/////////////////////////////////////////////////////////// Symbol booking
	static CSequence* m_BookedSymsList;
	static bool BookSymbol(CESymbol* sym); // true is booked, source unaffected
	static bool isSymbolBooked(CESymbol* sym); // source unaffected
	static CESymbol* BookSymbolOrUnique(const char* sym, const char* alt=NULL, const char* alt2=NULL); // returns cloned result

	/////////////////////////////////////////////////////////// Basis vector symbols info
	static int isBasisSymbol(CESymbol* sym);

	/////////////////////////////////////////////////////////// Soft bremsstrahlung factor (IR-limit)
	// Note: cross_section_soft = GetSoftBremsstrahlungFactor()*cross_section_born
	static CExpression* GetSoftBremsstrahlungFactor();
};

#endif // !defined(AFX_LHEPBASICS_H__181D4182_17EE_4768_A8FB_D16C414F4F48__INCLUDED_)
