////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// MEComposeLib.h: interface for the CMEComposeLib class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_MECOMPOSELIB_H__0B4FF5D2_B9CE_4C0E_9EFC_46FF2E0E9A68__INCLUDED_)
#define AFX_MECOMPOSELIB_H__0B4FF5D2_B9CE_4C0E_9EFC_46FF2E0E9A68__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CMEComposeLib : public CELibrary  
{
	int m_order;
	//CSequence* m_ME_seq_list; // sequence of CMEOperatorSeq objects
	CAPtrSeq* m_ME_seq_list; // sequence of CMEOperatorSeq objects
	//CExpression* m_UnitFlags; // to keep flags of m_ME_seq_list, no common flag are required!

	CMatrixElement* m_ME_result;
	CDiagramSet* m_Diags_result;

	CDiagramPhysics* m_phys;
	CKinematics* m_kin;
	bool m_init_OK;
	bool m_do_involve_CT; // counterterms creation flag
	bool m_bClassesInvolved;

	DWORD m_settings;
	CEPtrArray* m_uniform_index_list; // vector indices list
public:
	static CDebug* dbg;
	CMEComposeLib();
	virtual ~CMEComposeLib();
	
	/**************************************************************************
	//	library usage:
	//	CMEComposeLib* lib = new CMEComposeLib();
	//	// lib->setCounterTermsProcessing(true); ==> counterterms (only) will be created
	//	bool run_OK = lib->ComposeME(order);
	//	if (!run_OK) { delete lib; throw; }
	//	CMatrixElement* res_me = (lib->GetCreatedME())->clone();
	//	CDiagramSet* res_diags = (lib->GetDiagramsCreated())->clone();
	//	delete lib;
	**************************************************************************/
	bool ComposeME(int order); // library uses current physics & kinematics
	void setCounterTermsProcessing(bool _do_involve_CT);
	static const DWORD COMPOSE_ME_NO_CROSSINGS;
	CMatrixElement* GetCreatedME();
	CDiagramSet* GetDiagramsCreated();

	static CExpression* ChangePointID(CExpression* src, int ID_old, int ID_new);
	static CExpression* ChangePointIDRuleProc(CExpression* src, CEPtrArray* data_arr);

private:
	void FillMESeqList();
	void FillMESeqList2();
	bool PreAcceptSeq(CMEOperatorSeq* seq);
	void FilterSeqOrder();
	bool CheckSeqOrder(CMEOperatorSeq* seq, bool allow_CT_gap = false); // useless parameter if (!m_do_involve_CT)
	void WickTheoremExpand();
	void WickTheoremLoop(CEPtrArray* res_arr, CMEOperatorSeq* seq, long first_pos);
	void InsertSeqToArray(CEPtrArray* arr, CMEOperatorSeq* seq); // deletes seq
	void CoupleExternals();
	void CoupleExternalsLoop(CEPtrArray* res_arr, CMEOperatorSeq* seq, int _PID, CEPtrArray* crossings_data);
	void TopologyFilter();
	void SortSequences();
	void ArrangeCrossings();
	void TransformStandardSpinorOrder();
	void UnifyLoops();
	void ArrangeCommonSign();
	void RestoreDiagrams();
	void RestoreClassesToDiagrams();
	void RestoreClassToDiagrams(CFeynDiagram* dClass, long nNextVert);
	long DropZeroDiagrams(); // returns number of diagrams dropped out

	long GetLastExtCopulePos(CEPtrArray* crossings_data, int pk);
	void SetLastExtCopulePos(CEPtrArray* crossings_data, int pk, long pos);
};

#endif // !defined(AFX_MECOMPOSELIB_H__0B4FF5D2_B9CE_4C0E_9EFC_46FF2E0E9A68__INCLUDED_)
