////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// MEOperatorSeq.h: interface for the CMEOperatorSeq class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_MEOPERATORSEQ_H__A7079ECF_A95A_4A2A_BFCA_1E22FD01B63C__INCLUDED_)
#define AFX_MEOPERATORSEQ_H__A7079ECF_A95A_4A2A_BFCA_1E22FD01B63C__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CMEOperatorSeq : public CEProduct  
{
	int m_multiplicity; // used to combine similar topologies
	int m_max_coupling_no;

	bool m_ferm_sign;// true means '+'

	int m_curr_sort_point; // temporary
public:
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	//virtual bool contain_vector(void * obj); // same to super class
	virtual CEString* toString(int mode = 0);
	virtual CExpression* Arrange(int level);
	virtual CExpression* expand(bool recursive = true);
	virtual bool equals(CExpression* other);
	//virtual bool less_than(CExpression* other); // used for sorting objects
	virtual CExpression* clone() const;

	CMEOperatorSeq(const CEProduct* _seq, int _multiplicity, int _max_coupling_no, bool _ferm_sign = true);
	virtual ~CMEOperatorSeq();
	static CDebug* dbg;
	static DWORD m_log_settings;
	static const DWORD ME_SEQ_SORT_LOG;
	static const DWORD ME_SEQ_SORT_ADVANCED_LOG;
	static const DWORD ME_SEQ_SORT_COMPLETE_LOG;

	void sort_internal(CEPtrArray* uniform_index_list);
	bool equals_topology(CMEOperatorSeq* other);
	bool is_kin_vanishing(CKinematics* kin, bool allow_uncoupled, bool classes_only); // checks if sequence vanishes after the kinematics is applied
	int calcSeqOrder(CDiagramPhysics* _phys, bool search_CT = true); // if no CT are expected ==> fast processing
	int calcCounterTermsNum(CDiagramPhysics* _phys);
	bool have_uncoupled_field_op();
	int get_multiplicity();
	void inc_multiplicity(int val = 1);
	void set_multiplicity(int val);
	int getMaxCouplingNo();
	int incMaxCouplingNo(); // increases coupling No and returns new value
	CMatrixElement* ConvertToME();
	bool get_ferm_sign();
	bool getStandardOrderSign();
	void inverse_ferm_sign();

private:
	bool calcFOCouplingsSeqSign(); // additional sign factor (when diagram is retrieved)

public:
	int GetSpinorCouplingsNum();

	CEParticleFunction* GetFermPFNoX(int sp_seq_no, bool get_start_op);
	//COpFermionField* GetFermFieldOp(int sp_seq_no, bool get_start_op);
	//COpFermionField* GetFermFOAtPoint(int pointID, bool get_start_op);
	long GetNextPFPosAtPoint(int pointID, long start_search_pos, long PK_abs); // PK == 0 means any PK, the abs(PK) is only compared
	//int GetNextFOPosAtPoint(int pointID, int start_search_pos, long PK); // PK == 0 means any PK
	CEParticleFunction* GetPFAtPos(int pos); // throws for wrong type
	//CEFieldOperator* GetFOAtPos(int pos); // throws for wrong type

	//int GetNumFOAtPoint(int pointID, long PK); // PK == 0 means any PK
	int getMaxPointID();
	int GetOpCouplePK(int couple_no);
	CEFieldOperator* GetFieldOperatorCoupled(int couple_no, int op_no); // op_no == 0 or 1
	CParticleWaveFunction* GetWaveFunction(CEParticle* part);

	/*
	*	Transforms crossed diagrams (sequence order) to normal ones.
	*	Takes care of the sign for every crossing transformation.
	*	Returns true is SEQ is changed.
	*/
	bool NormalizeCrossings(CKinematics* kin);
	void ToStandardFOCouplingsOrder();

public:
	CEFieldOperator* GetCoupledFO(CEFieldOperator* fo);

private:
	int getVertexIDAtPoint(int pt);

	void SortOpCouplsNumsByPK(); // unused

	// internal sort functions
	void SortPointIDs();
	int ComparePointData(int a, int b);
	int ComparePointFunctions(CEParticleFunction* f_a, CEParticleFunction* f_b);
	int FindNextMinPKAtPoint(long pk_prev, int pID, bool fermions_first, bool wf_only=false); // 0 result means no pk are found
	CEParticleFunction* FindNextMinFunctionPointPK(CEParticleFunction* f_prev, int pID, long pk, bool wf_only=false);
	//int CompareFieldOperators(CEFieldOperator* fo_a, CEFieldOperator* fo_b);
	int GetCoupleOtherOpPointID(CEFieldOperator* fo);
	//int GetOpPos(CEFieldOperator* fo);
	
	void SortOpCouplingNumbers();
	int CompareOpCouplingNumbers(int a, int b);
	void ExchangeOpCouplingNumbers(int a, int b);
	
	void ArrangeSpinorCouplingsByPoint();

	//void SortSpinorSeqs(); // unused // sort by coupling numbers
	//bool CheckSpinorCouplsOrder(int a, int b); // unused // true means b should follows a

	int GetSpinorCouplingPos(int i, bool get_start_pos); // false means end pos
	void MoveSpinorCoupling(int i, int new_pos); // positions are logical
	int GetFermOperatorsNumBetween(int pos_a, int pos_b);

	//void ArrangePointIDs();
	void ExchangePointIDs(int a, int b);

	void UnifyVectorIndices(CEPtrArray* uniform_index_list);

	int NormCrossings(bool in_inits, CParticleWaveFunction* pf_route_close,
		CParticleWaveFunction* pf_route_far, CKinematics* kin);
public:
	// returns number of vertices between functions (0 for equals, -1 for unbinded operators)
	int CalcLinksNum(CEParticleFunction* f_a, CEParticleFunction* f_b);
private:
	void CalcLinksNumProc(int* _vals, int step_no, CEParticleFunction* f, int last_point);
	void CalcLinksNumProc(int* _vals, int step_no, int pID, CEParticleFunction* f_last);
	void ExchangeCrossingParticles(long pk, CParticleWaveFunction* wf_A, CParticleWaveFunction* wf_B);
};

#endif // !defined(AFX_MEOPERATORSEQ_H__A7079ECF_A95A_4A2A_BFCA_1E22FD01B63C__INCLUDED_)
