////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// MESquareLib.h: interface for the CMESquareLib class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_MESQUARELIB_H__4FC7882A_060C_4945_8666_C49D5BB5863D__INCLUDED_)
#define AFX_MESQUARELIB_H__4FC7882A_060C_4945_8666_C49D5BB5863D__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CMESquareLib : public CELibrary  
{
	bool m_ME_SQUARE;
	bool m_NDimCalc; // (m_loops_no > 0) flag
	int m_loops_no;
	DWORD m_settings;
	CMatrixElement* m_me_a;
	CMatrixElement* m_me_b;
	CKinematics* m_kin;
	CEPtrArray* m_vector_parts_DMs; // to be coupled during squaring (at N-dim)
	CEPtrArray* m_vector_PostProcDMs; // to be coupled after squaring (at 4-dim)

	CERule* m_mass_rule;
	CEReplList* m_kin_simp_rules;
	CEReplList* m_basic_coupling_vals;
	CEReplList* m_mand_vars_rule;

	CEPtrArray* m_denominators_cache;

	// Proton structure functions reconstruction data
	bool m_bRestoreProtonStrFunc;
	CIntArray m_arrProtonID;
	CSequence m_SFTmpIndices; // temporary indices used in m_bRestoreProtonStrFunc mode

public:
	static CDebug* dbg;

	static const DWORD SQRME_DEFAULT;
	static const DWORD SQRME_SIMPLE;
	static const DWORD SQRME_SPLIT_SUM;
	static const DWORD SQRME_EXPAND_AND_SPLIT;
	static const DWORD SQRME_PRESIMPLIFY_ME;
	static const DWORD SQRME_CACHE_DENOMINATORS;
	static const DWORD SQRME_CMS_BEAMS;
	static const DWORD SQRME_SKIP_VI_CALC;
	static const DWORD SQRME_NO_CROSSINGS_FOR_ME1;
	static const DWORD SQRME_NO_CROSSINGS_FOR_ME2;
	static const DWORD SQRME_ALLOW_MANDELSTAMS;
	static const DWORD SQRME_BERENDS_HELICITIES;
	static const DWORD SQRME_BERENDS_HELICITIES_PRECISE;
	static const DWORD SQRME_IR_LIMIT;
	static const DWORD SQRME_EARLY_PHOTON_HELICITIES; // early involve photon polarization vectors
	static const DWORD SQRME_LATE_PHOTON_HELICITIES; // late involve photon polarization vectors
	static const DWORD SQRME_FORCE_4DIM;
	static const DWORD SQRME_4DIM_BOSON_POLARIZATION; // unused
	static const DWORD SQRME_NO_POSTARRANGE;
	static const DWORD SQRME_NO_SUB_ARRANGE;
	static const DWORD SQRME_SAVE_MEMORY;
//private:
//	static const SQRME_EXPAND_ME;

public:
	static CExpression* SquareME(CMatrixElement* me_a, CMatrixElement* me_b, DWORD settings);
	virtual ~CMESquareLib();

private:
	CMESquareLib(DWORD settings);
	CExpression* run_task(CMatrixElement* me_a, CMatrixElement* me_b);
	void AppendCrossings();
	bool PrepareMatrixElementsExprs();
	CExpression* PrepareME(CExpression* me_expr, CEPtrArray* inds, int loops_no);
	bool prepareVectorPolarizationData();
	CExpression* coupleMESplitting();
	CExpression* coupleExprs(CExpression* a, CExpression* b);
	CExpression* InvolvePostProcDMs(CExpression* expr);

	CExpression* kinematic_reduce(CExpression* val);
	CExpression* virt_integrate(CExpression* val);

	// denominators cache functions
	void init_denominators_cache();
	CExpression* involve_denominators_cache(CExpression* val, bool allow_mass_simp);
	void simplify_cached_denominators();
	CExpression* restore_denominators(CExpression* val);
	static const DWORD D_CACHE_ALLOW_MASS_SIMPLIFICATION;
};

#endif // !defined(AFX_MESQUARELIB_H__4FC7882A_060C_4945_8666_C49D5BB5863D__INCLUDED_)
