////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// VIDensData.h: interface for the CVIDensData class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_VIDENSDATA_H__031EDDA5_D772_400F_95B2_895A77EB3301__INCLUDED_)
#define AFX_VIDENSDATA_H__031EDDA5_D772_400F_95B2_895A77EB3301__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//	Note: VI-dens definition is: 1/{ ((k+p1)^2-m1^2)*((k+p2)^2-m2^2)*...}
//##ModelId=3E3FF2E1001F
class CVIDensData : public CSequence  
{
public:
	// operations
	void sort();
private:
	bool is_right_order(CExpression* p1, CExpression* m1, CExpression* p2, CExpression* m2);
public:
	CExpression* get_norm_p_del(); // return NON-CLONED value
	//CExpression* SortAndGetNormPDel();
	void change_var_minus_pdel(CExpression* p_del);
	void convertInvariant();
	CExpression* get_simpl_p_del(); // return CLONED!!! value

	// others
	//bool is_normalized();
	//CSequence* get_vector_basis(int rank);

	// data mode switches
private:
	//bool m_is_invariant;
	int m_dens_num; // -1 for non-invariant DENSDATA (use size() instead)
public:
	bool isInvariant();
	void markInvariant(int _dens_num); // debug function

	// data access
	CExpression* momentum(int i); // for non-invariant only
	CExpression* mass(int i); // for non-invariant only
	CSequence* get_PM_cart(int i); // for non-invariant only
	int getDensNumber(); // equals to size() for non-invariant DENSDATA
	CExpression* getInvValue(int i); // values number is s*(s+1)/2 where s = getDensNumber();
	int getFirstMassPos(); // first pos of mass parameter

	// statics
	static CDebug* dbg;
	static const char* CLASS_NAME;

	// virtuals
	virtual const char* ClassName() const;
	virtual CEString* toString(int mode = 0);
	virtual CExpression* clone() const;
	virtual bool contain_vector(void * obj);
	virtual CExpression* applyRule(CEAbstractRule* rule);

	// construction/destruction
	CVIDensData(CSequence* ddata, int _dens_num = -1);
	virtual ~CVIDensData();

	static const char* CLASS_TAG_NAME;
	virtual const char* get_TAG_Name();
	virtual long store_stream(ofstream* p_ofs, DWORD settings);
	static CESerializable* restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings);

private:
#ifdef USE_PLAIN_ARRAY_FORSEQ // biuld sequence-expressions on CAPtrArray* basis
	CVIDensData(CEPtrArray* items, int _dens_num);
#else
	CVIDensData(CAPtrSeq* items, int _dens_num);
#endif //USE_PLAIN_ARRAY_FORSEQ
};

#endif // !defined(AFX_VIDENSDATA_H__031EDDA5_D772_400F_95B2_895A77EB3301__INCLUDED_)
