////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// VirtIntC.h: interface for the CVirtIntC class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_VIRTINTC_H__B50CFE81_A59C_46FA_BACC_166C2756AE4A__INCLUDED_)
#define AFX_VIRTINTC_H__B50CFE81_A59C_46FA_BACC_166C2756AE4A__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CVirtIntC : public IETensor, public CVirtIntegral
{
	CSequence* m_couplings;
public:
	virtual CExpression* Arrange(int level);
	virtual CExpression* expand(bool recursive = true);
	virtual CExpression* evaluate(int level = 0);
	virtual CExpression* applyRule(CEAbstractRule* rule);
	virtual CEString* toString(int mode = 0);
	virtual CExpression* clone() const;
	virtual bool equals(CExpression* other);
	virtual bool contain_vector(void * obj);
	CVirtIntC(const CVIDensData* ddata, CSequence* _couplings, CEPtrArray* _indices=NULL);
	virtual ~CVirtIntC();

	CExpression* normalize_denominators();
	CExpression* tensor_reduction();

public:
	CSequence* GetCouplings();

	// TENSOR part
protected:
	//CExpression* m_base; // unused?
	CEPtrArray* m_indices;
protected:
	virtual void collectProperties(CEProperties* props, unsigned set = CEProperties::PROP_MASK_ALL);
	virtual unsigned long props_long();
public:
	virtual void* GetInterface(long interface_ID);
	// IETensor interface <BEGIN>
	virtual CExpression* ThisExpr();	 // returns this as (CExpression*)
	//virtual IETensor* ThisITensor();	 // returns this as (IETensor*)
	virtual CExpression* base();
	virtual const CExpression* GetBase() const;
	virtual bool setBase(CExpression* _new);
	virtual int rank() const;
	virtual CExpression* index(int i);	// 1..rank
	virtual const CExpression* GetIndex(int i) const;
	virtual CTIndex* GetTIndex(int i);	// 1..rank
	virtual const CTIndex* GetTIndexConst(int i) const; // 1..rank
	virtual bool covariance(int i) const;
	virtual bool changeIndex(int i, CExpression* index, bool covariance);
	virtual bool liftIndex(int i, bool to_covariance);
	virtual bool setIndex(int i, CExpression* index);
	virtual CExpression* exchange_subst(IETensor* other, bool other_is_at_right, bool bCalledByOther=false);
	virtual CExpression* unite_with(IETensor* other, bool other_is_at_right=true, bool bCalledByOther=false);
	// IETensor interface <END>

	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	static const char* CLASS_TAG_NAME;
	virtual const char* get_TAG_Name();
	virtual long store_stream(ofstream* p_ofs, DWORD settings);
	static CESerializable* restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings);
private:
	CExpression* expand_to_vi_basis();
	CExpression* GetExpandCpl(CExpression* base, int i); // used by expand_to_vi_basis to get p_p or p^i
	CExpression* GetExpandCpl(int i, int j); // used by expand_to_vi_basis to get p_p, p^i or g^ik
	static void XCplRemoveAt(CSequence* _couplings, CEPtrArray* _indices, int i);
	CExpression* tens_reduce_combines();
	CExpression* tens_reduce_combines(int d_pos, int n_pos, CExpression* sum);
	CExpression* tens_reduce_similar();
	CExpression* tens_reduce_similar(int i, int j, bool sign);
};

#endif // !defined(AFX_VIRTINTC_H__B50CFE81_A59C_46FA_BACC_166C2756AE4A__INCLUDED_)
