////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// VirtIntegral.h: interface for the CVirtIntegral class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_VIRTINTEGRAL_H__3E242B42_E0CC_4558_9A12_6F791A267615__INCLUDED_)
#define AFX_VIRTINTEGRAL_H__3E242B42_E0CC_4558_9A12_6F791A267615__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//##ModelId=3E3FF2D60203
class CVirtIntegral : public CEAbstractFunction  
{
	//	Note: VI-dens definition is: 1/{ ((k+p1)^2-m1^2)*((k+p2)^2-m2^2)*...}
	CVIDensData* m_dens_data;
public:
	//Parameter:
	static bool EVALUATE_ALL_POSSBILE_VI;	//(def: false) incl. C_ij-values --> may be very slow for massive vectors
	static bool PAR_LOOPTOOLS_COMPARTIBLE;	//(def: false) forbid integrals not supported by LoopTools
public:
	static CDebug* dbg;
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual unsigned long props_long();
	virtual bool contain_vector(void * obj);
	virtual CExpression* applyRule(CEAbstractRule* rule);
	virtual bool equals(CExpression* other);
	virtual CExpression* expand(bool recursive = true);
	
	CVIDensData* get_dens_data() { return m_dens_data; }
	const CVIDensData* get_dens_data_const() const { return m_dens_data; }
	
	CVirtIntegral(const CVIDensData* ddata);
	virtual ~CVirtIntegral();
	virtual CEString* GetHEPId(); // "?" for undefined, "0","1","2",11,21,111 ...
	//virtual CEString* GetParamsStrSpecial(); // NULL if plain sequence is OK

	virtual CExpression* GetSingularTermFactor(); //The result means: Integral==Xi*Result+0(n-4), Xi*(n-4)=-2+0(n-4), NULL for unknown
	virtual DWORD GetResultType(); // complex!
protected:
	virtual void collectProperties(CEProperties* props, unsigned set = CEProperties::PROP_MASK_ALL);
};

#endif // !defined(AFX_VIRTINTEGRAL_H__3E242B42_E0CC_4558_9A12_6F791A267615__INCLUDED_)
