////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EParticleFunction.cpp: implementation of the CEParticleFunction class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CEParticleFunction::CEParticleFunction(long _particle_kind,
									   int _vertex_ID,
									   unsigned long _point_ID)
									   : CExpression()
{
	m_particle_kind = _particle_kind;
	m_point_ID = _point_ID;
	m_vertex_ID = _vertex_ID;
}

CEParticleFunction::~CEParticleFunction() { }

long CEParticleFunction::getParticleKind() { return m_particle_kind; }
int CEParticleFunction::getPointID() { return m_point_ID; }
int CEParticleFunction::getVertexID() { return m_vertex_ID; }
void CEParticleFunction::setPointID(int _point_ID) { m_point_ID = _point_ID; }

const char* CEParticleFunction::CLASS_NAME = "CEParticleFunction";
const char* CEParticleFunction::ClassName() const { return CLASS_NAME; }
bool CEParticleFunction::contain_vector(void * obj) { return CExpression::contain_vector(obj); }

void CEParticleFunction::collectProperties(CEProperties* props, unsigned set) {
	CExpression::collectProperties(props, set);
	props->setProperty(CEProperties::PROP_MASK_IS_PARTICLE_FUNCTION, true);
	props->setProperty(CEProperties::PROP_MASK_SELF_CONJUGATED, false);
}

unsigned long CEParticleFunction::props_long() {
	unsigned long res = CExpression::props_long();
	res = CEProperties::set_property_to(res, CEProperties::PROP_MASK_IS_PARTICLE_FUNCTION, true);
	res = CEProperties::set_property_to(res, CEProperties::PROP_MASK_SELF_CONJUGATED, false);
	return res;
}

bool CEParticleFunction::equals(CExpression *other) {
	if (other == NULL) return false;
	if (!CEUtil::equals(ClassName(), other->ClassName())) return false;
	if (!attributes_equals_to(other)) return false;

	CEParticleFunction* pf_other = (CEParticleFunction*)other;
	bool res = ((getParticleKind() == pf_other->getParticleKind()) &&
		(getPointID() == pf_other->getPointID()));
	return res;
}

/*bool CEParticleFunction::less_than(CExpression* other) {
	if (!other->getProperty(CEProperties::PROP_MASK_IS_PARTICLE_FUNCTION)) return CExpression::less_than(other);

	if (m_particle_kind < ((CEParticleFunction*)other)->getParticleKind()) return true;
	if (m_point_ID < ((CEParticleFunction*)other)->getPointID()) return true;
	if (m_vertex_ID < ((CEParticleFunction*)other)->getVertexID()) return true;
}*/
