////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// PAbstractBoson.cpp: implementation of the CPAbstractBoson class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

int CPAbstractBoson::m_MassiveVecMatrixType = 1; //0(g_ab), 1(g_ab-kk/M2)(default)

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CPAbstractBoson::CPAbstractBoson(long _pk,
								 CExpression* _momentum_sym,
								 CExpression* _mass_sym,
								 CExpression* _pol_vec_sym,
								 int _ID_kin,
								 CEValue* _mass)
: CEParticle(_momentum_sym, _mass_sym, _pol_vec_sym, _ID_kin, _mass)
{
	m_boson_pk = _pk;
	m_vector_index = NULL;
}

CPAbstractBoson::~CPAbstractBoson()
{
	if (m_vector_index != NULL) delete m_vector_index;
}

long CPAbstractBoson::get_PK() { return m_boson_pk; }
CExpression* CPAbstractBoson::get_vector_index() { return m_vector_index; }

void CPAbstractBoson::reset_vector_index(bool do_create) {
	if (m_vector_index != NULL) { delete m_vector_index; m_vector_index = NULL; }
	if (do_create) m_vector_index = CESymbol::getUnique();
}

bool CPAbstractBoson::SetMassiveVecMatrixType(int val) {
	if ((val < 0) || (val > 1)) return false;
	m_MassiveVecMatrixType = val;
	return true;
}

/*
*	vars should be vector indices
*/
CExpression* CPAbstractBoson::getDensityMatrix(CEPtrArray* vars) {
	if (vars->size() < 2) return NULL;
	CExpression* i_1 = (CExpression*)vars->GetAt(0);
	CExpression* i_2 = (CExpression*)vars->GetAt(1);

	int _pol_info = get_polarization_info();
	CExpression* res = NULL;
	if (_pol_info == 0) { //////////////////////////////// unpolarized
		res = new CEMetric(i_1, i_2, false, false);
		CExpression* m=get_sym_mass();
		if (m == NULL) { dbg->log(0, "ERROR: NULL particle mass", this); CEException::halt(); }

		if (!CEBasic::isZero(m)) { // add current part
			if (m_MassiveVecMatrixType > 0) {
				CExpression* p = get_momentum_symbol();
				if (p == NULL) { dbg->log(0, "ERROR: NULL particle momentum", this); CEException::halt(); }
				res = res->minus( (new CEVector(p,i_1,false))->multiply(new CEVector(p,i_2,false))->divide(m->sqra()) );
				
				// TODO: remove next test if no promlems
				if ((get_PK() & CEParticle::PARTICLE_PHOTON) == CEParticle::PARTICLE_PHOTON) { dbg->log(0, "ERROR: Non-zero mass of PHOTON spotted", this, m); }
			}
		} else {
		}

		if (is_initial()) {
			CExpression* ave_factor = NULL;
			if (CEBasic::isZero(m)) { // 2 eigenstates
				ave_factor = CENumber::GetFraction(1,2);
			} else {  // 3 eigenstates
				ave_factor = CENumber::GetFraction(1,3);
			}
			res = res->multiply(ave_factor);
		}
		res->inverseFlag();
	} else { ///////////////////////////////////////////// polarized
		CExpression* pol_sym = get_pol_vector_symbol();
		CExpression* pol_sym_cjg = get_pol_vector_symbol_cjg();

		res = (new CEVector(pol_sym, i_1, false))->multiply(new CEVector(pol_sym_cjg, i_2, false));
	}

	return res;
}

CExpression* CPAbstractBoson::getHelicityVector(CExpression* vector_index) { return NULL; }
