////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// PPhoton.cpp: implementation of the CPPhoton class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

const int CPPhoton::PH_VECTOR_NOTATION = 0x00;
const int CPPhoton::PH_BERENDS_NOTATION = 0x01;

const char* CPPhoton::dm_data_node = "ph_matrices";
const char* CPPhoton::PH_REPL_DATA_ID = "ph_polarization_repl";

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

/*CPPhoton::CPPhoton(CExpression* id,
				   CExpression* _vector_index,
				   int helicity,
				   CExpression* h_base_1,
				   CExpression* h_base_2)
:CEParticle(id, new CENumber(0), new CERealValue(0))
{
	m_vector_index = _vector_index;
	if (m_vector_index != NULL) m_vector_index = m_vector_index->clone();
	m_vector_index_alt = CESymbol::getUnique();
	m_DM_node_data_filled = false;
	m_Vector_Polarization_Simpl_list_added = false;
	m_pol_vec_sym = NULL;
	m_pol_vec_cjg_sym = NULL;
	m_pol_vec_stub = NULL;
	m_pol_vec_cjg_stub = NULL;
	m_helicity = helicity;
	if (m_helicity != 0) {
		if ((h_base_1 == NULL) || (h_base_2 == NULL)) {
			printf("Error PHOTON data initialization.");
			//terminate();
		}
		m_h_base_1 = h_base_1->clone();
		m_h_base_2 = h_base_2->clone();
	}
	else {
		m_h_base_1 = NULL;
		m_h_base_2 = NULL;
	}
	
	create_polarization_vectors_symbols();
}*/

CPPhoton::CPPhoton(CExpression* _momentum_sym, CExpression* _pol_vec_sym,
				   int _ID_kin, CEPtrArray *params)
:CPAbstractBoson(CEParticle::PARTICLE_PHOTON,_momentum_sym, new CENumber(0), _pol_vec_sym, _ID_kin, new CERealValue(0))
{
	m_helicity = 0; // unknown
	m_h_base_1 = NULL;
	m_h_base_2 = NULL;
	m_stoks_e_basis1=NULL; m_stoks_e_basis2=NULL;
	m_DM_base = NULL;

	m_UseStoksParams = false; m_StoksParams = NULL;

	if (init_photon_data(params)) m_init_OK = true;
}

bool CPPhoton::init_photon_data(CEPtrArray *params)
{
	//m_pol_vec_sym = NULL;
	//m_pol_vec_cjg_sym = NULL;
	m_pol_vec_stub = NULL;
	m_pol_vec_cjg_stub = NULL;

	return true;
}

CPPhoton::~CPPhoton()
{
	if (m_h_base_1 != NULL) delete m_h_base_1;
	if (m_h_base_2 != NULL) delete m_h_base_2;

	if (m_DM_base != NULL) delete m_DM_base;

	if (m_stoks_e_basis1 != NULL) delete m_stoks_e_basis1;
	if (m_stoks_e_basis2 != NULL) delete m_stoks_e_basis2;

	//if (m_pol_vec_sym != NULL) delete m_pol_vec_sym;
	//if (m_pol_vec_cjg_sym != NULL) delete m_pol_vec_cjg_sym;
	if (m_pol_vec_stub != NULL) delete m_pol_vec_stub;
	if (m_pol_vec_cjg_stub != NULL) delete m_pol_vec_cjg_stub;

	if (m_StoksParams != NULL) delete m_StoksParams;
}

/*
*	vars should be vector indices
*/
CExpression* CPPhoton::getDensityMatrix(CEPtrArray* vars) {

	if ((m_UseStoksParams) && (get_polarization_info() != 0)) { // Stoks & polarized
		CKinematics* kin = CEProcessor::kinematics();
		if ((kin != NULL) && (vars->size() >= 2)){

			if ((m_stoks_e_basis1 != NULL) && (m_stoks_e_basis2 != NULL) && (m_StoksParams != NULL)) {
				CExpression* i_1 = (CExpression*)vars->GetAt(0);
				CExpression* i_2 = (CExpression*)vars->GetAt(1);

				CExpression* res = CPPhoton::phot_DM_Stoks(m_StoksParams, get_momentum_symbol(), m_stoks_e_basis1, m_stoks_e_basis2, i_1, false, i_2, false);
				if (res != NULL) return res;
				dbg->log(0, "Can't create Stoks-parametrized DM");
			} else {
				if (m_stoks_e_basis1 == NULL) dbg->log(0, "Can't involve Stoks parameters, no m_stoks_e_basis1 vector found");
				if (m_stoks_e_basis2 == NULL) dbg->log(0, "Can't involve Stoks parameters, no base2 vector found");
				if (m_StoksParams == NULL) dbg->log(0, "Can't involve Stoks parameters, Stoks parameters not initialized (Internal Error)");
			}
		} else {
			dbg->log(0, "Can't involve Stoks parameters, No kinematics or Internal Error");
		}
	}

	if (m_DM_base != NULL) {
		if ((get_polarization_info() == 0) && (CEBasic::isZero(get_sym_mass()))) { // for massless unpolarized photon
			if (vars->size() < 2) return NULL;
			CExpression* i_1 = (CExpression*)vars->GetAt(0);
			CExpression* i_2 = (CExpression*)vars->GetAt(1);
			CExpression* p = get_momentum_symbol(); if(p== NULL){dbg->log(0, "ERROR: NULL photon momentum",this);CEException::halt();}
			
			CExpression* res = NULL;
			// g_mn - p_n k_m/pk - k_n p_m/pk + k^2 p_m p_n /pk^2
			res = new CEMetric(i_1, i_2, false, false);
			res = res->minus( (new CEVector(p,i_1,false))->multiply(new CEVector(m_DM_base,i_2,false))->divide(new CESProduct(p,m_DM_base)) );
			res = res->minus( (new CEVector(m_DM_base,i_1,false))->multiply(new CEVector(p,i_2,false))->divide(new CESProduct(p,m_DM_base)) );

			// m_DM_base may be massive
			CExpression* pB2 = CExpression::reduce(new CESProduct(m_DM_base,m_DM_base));
			if (!CEBasic::isZero(pB2)) {
				res = res->add( pB2->multiply(new CEVector(p,i_1,false))->multiply(new CEVector(p,i_2,false))->divide( (new CESProduct(p,m_DM_base))->sqra() ) );
			} else {
				delete pB2;
			}

			if (is_initial()) {
				CExpression* ave_factor = CENumber::GetHalf();
				res = res->multiply(ave_factor);
			} // 2 eigenstates
			res->inverseFlag();
			return res;
		}
	}

	return CPAbstractBoson::getDensityMatrix(vars);
}

CExpression* CPPhoton::getHelicityVector(CExpression* vector_index) {
	if (m_helicity == 0) return NULL; // no helicity vector available
	if (vector_index == NULL) return NULL; // invalid parameter
	if ((m_h_base_1 == NULL) || (m_h_base_2 == NULL)) return NULL; // invalid internal data
	if (m_UseStoksParams) return NULL; // use DM insstead of h-vectors

	CExpression* res = phot_helisity(
		get_momentum_symbol(),
		m_h_base_1,
		m_h_base_2,
		(m_helicity > 0),
		vector_index,
		false);
	return res;
}

void CPPhoton::setHelicityData(int _val, CExpression* _base_1, CExpression* _base_2) {
	// delete old
	if (m_h_base_1 != NULL) { delete m_h_base_1; m_h_base_1 = NULL; }
	if (m_h_base_2 != NULL) { delete m_h_base_2; m_h_base_2 = NULL; }
	m_helicity = 0;
	
	// "_val" == 0 means clear helicity data
	if (_val != 0) {
		if ((_base_1 == NULL) || (_base_2 == NULL)) {
			dbg->log(0, "Error: CPPhoton::setHelicityData invalid parameters.");
			return;
		}

		m_h_base_1 = _base_1->clone();
		m_h_base_2 = _base_2->clone();
		m_helicity = _val;

		// also set particle as 'polarized'
		if (get_polarization_info() == 0) CEParticle::set_polarization_info(1);
	}
}

void CPPhoton::setUnpolarizedDMBase(CExpression* DM_base) { // does not sets particle unpolarized
	dbg->log(55, "CPPhoton::setUnpolarizedDMBase()", DM_base);
	if (m_DM_base != DM_base) { if (m_DM_base != NULL) delete m_DM_base; }
	m_DM_base = ((DM_base!=NULL)?(DM_base->clone()):NULL);
}

int CPPhoton::get_helicity_value() { return m_helicity; } // for debug only

// for debug only (base_no = 1,2)
CExpression* CPPhoton::get_helicity_base(int base_no) {
	if (base_no == 1) return m_h_base_1;
	else if (base_no == 2) return m_h_base_2;
	return NULL;
}

void CPPhoton::set_polarization_info(int _pol_val) {
	CEParticle::set_polarization_info(_pol_val);
	if (_pol_val == 0) {
		setHelicityData(0, NULL, NULL);

		// unset "use stoks" flag
		m_UseStoksParams = false;
	}
}

void CPPhoton::setStoksParamsUse(bool _use_Stoks, CExpression* _stoks_e_basis1, CExpression* _stoks_e_basis2) {

	// delete previous data
	if (m_stoks_e_basis1 != NULL) { delete m_stoks_e_basis1; m_stoks_e_basis1 = NULL; }
	if (m_stoks_e_basis2 != NULL) { delete m_stoks_e_basis2; m_stoks_e_basis2 = NULL; }
	m_UseStoksParams = false;

	if (_use_Stoks) {
		if ((_stoks_e_basis1 == NULL) || (_stoks_e_basis2 == NULL)) { dbg->log(0, "Error: CPPhoton::setStoksParamsUse invalid parameters."); return; }

		set_polarization_info(1);
		m_stoks_e_basis1 = _stoks_e_basis1->clone();
		m_stoks_e_basis2 = _stoks_e_basis2->clone();
		m_UseStoksParams = true;

		// init Stoks parameters if required
		if (m_StoksParams == NULL) {
			m_StoksParams = new CSequence();
			char* tmp_s = (char*)CMemoryManager::mmalloc(100);

			int pID = get_ID();
			bool is_init = (pID<0);
			if (pID<0) pID=-pID;

			int i;
			for(i=0; i<3; i++) {
				// generate symbols for Stoks parameters
				sprintf(tmp_s, "\\(c\\&\\(%s%ld\\)\\)\\+%ld", ((is_init)?"i":"f"), pID, i+1);
				m_StoksParams->addItem(new CESymbol(tmp_s));
			}
			CMemoryManager::mfree(tmp_s);
		}
	}
}

CSequence* CPPhoton::GetStoksParamsDeclared() { // NULL if Stoks params not declared
	if (get_polarization_info() == 0) return NULL;
	if (!m_UseStoksParams) return NULL;
	return m_StoksParams;
}

/*
*	Returns the helicity expression coupled with gamma matrix.
*	The gamma-5 term is dropped out.
*/
CExpression* CPPhoton::getBerendsHelicity(bool calc_precise) {
	if (m_helicity == 0) return NULL; // no helicity information is set.
	if ((m_h_base_1 == NULL) || (m_h_base_2 == NULL)) return NULL; // invalid internal data
	if (m_UseStoksParams) return NULL; // use stoks

	CExpression* res = phot_berends_helisity(
		get_momentum_symbol(),
		m_h_base_1,
		m_h_base_2,
		(m_helicity > 0),
		calc_precise);
	return res;
}

/*
*	m_h_base_1 is P-
*	m_h_base_2 is P+
*
*	Unpolarized: if (!is_initial) result *2 due to summarizing over helicities.
*	Polarized: return symbol for density and store its's value in DATA.
*	This should work because the indices are unique.
*/
/*CExpression* CPPhoton::get_density_matrix(bool is_initial)
{
	if (m_helicity == 0) {
		CExpression* res = new CEMetric(get_vector_index(), get_vector_index_alt(), false, false);
		if (is_initial) {
			res = res->multiply(CENumber::GetHalf());
		}
		res->inverseFlag();
		return res;
	}

	// helicity is +/-1.
	bool h_sign = (m_helicity == 1);

	int ph_h_notation = CEProcessor::getEnvironment()->m_photon_helisity_notation;
	if (ph_h_notation == CPPhoton::PH_VECTOR_NOTATION) {
		if ((m_pol_vec_sym == NULL) || (m_pol_vec_cjg_sym == NULL))
			create_polarization_vectors_symbols();
		
		CEVector* ph_ret = new CEVector(m_pol_vec_sym, get_vector_index(), false);
		CEVector* ph_ret_cjg = new CEVector(m_pol_vec_cjg_sym, get_vector_index_alt(), false);

		if (!m_Vector_Polarization_Simpl_list_added) add_vector_polarization_simpl_list();
		
		return ph_ret->multiply(ph_ret_cjg);
	}
	else if (ph_h_notation == CPPhoton::PH_BERENDS_NOTATION) {
		// returns nothing and stores polarization vectors into storage for further use
		if (!m_DM_node_data_filled) {
			CExpression* ph_1 = phot_helisity(get_momentum_symbol(), m_h_base_1, m_h_base_2, h_sign, get_vector_index(), false);
			CExpression* ph_2 = phot_helisity(get_momentum_symbol(), m_h_base_1, m_h_base_2, h_sign, get_vector_index_alt(), false);
			CEProcessor::getDataStorage()->pute_to_node(dm_data_node, CESymbol::getUnique(), ph_1, true);
			CEProcessor::getDataStorage()->pute_to_node(dm_data_node, CESymbol::getUnique(), ph_2, true);
			m_DM_node_data_filled = true;
		}
		return CENumber::GetUnit();
	}

	// Error: undefined photon notation !
	return NULL;
}*/

/*
*	base vectors == NULL means no vectors should be changed.
*/
/*int CPPhoton::set_helicity(int h_val, CExpression *h_base_1, CExpression *h_base_2)
{
	int old = m_helicity;
	m_helicity = h_val;

	if ((h_base_1 != m_h_base_1) && (h_base_1 != NULL)) {
		if (m_h_base_1 != NULL) delete m_h_base_1;
		m_h_base_1 = h_base_1->clone();
	}

	if ((h_base_2 != m_h_base_2) && (h_base_2 != NULL)) {
		if (m_h_base_2 != NULL) delete m_h_base_2;
		m_h_base_2 = h_base_2->clone();
	}
	
	if (m_helicity != 0) {
		if ((h_base_1 == NULL) || (h_base_2 == NULL)) {
			printf("Error PHOTON data re-initialization.\n");
			if (old != m_helicity) printf("Can't change helicity to <>0 without base vectors initialization.\n.");
			//terminate();
		}
	}
	return old;
}*/

/*
*	expression from BERENDS'81, p4.
* creates first !- vector (with no E-tesor)
*/
CExpression* CPPhoton::CreateHelicityL(CExpression *base, CExpression *vP, CExpression *vM, CExpression *ind, bool cov) {
	CExpression* f = CreateHFactor(base, vP, vM); if(f==NULL) return NULL;
	CExpression* ph_L = (new CESProduct(base, vP))->multiply(new CEVector(vM, ind, cov))->
		minus((new CESProduct(base, vM))->multiply(new CEVector(vP, ind, cov)));
	CExpression* res = f->multiply(ph_L);
	dbg->log(7, "CPPhoton::CreateHelicityL", res);
	return res;
}

/*
*	expression from BERENDS'81, p4.
* creates first !- vector (with E-tesor)
*/
CExpression* CPPhoton::CreateHelicityT(CExpression *base, CExpression *vP, CExpression *vM, CExpression *ind, bool cov) {
	CExpression* f = CreateHFactor(base, vP, vM); if(f==NULL) return NULL;
	CExpression* i1 = CESymbol::getUnique();
	CExpression* i2 = CESymbol::getUnique();
	CExpression* i3 = CESymbol::getUnique();
	CExpression* ph_T = (new CEAntisymmetric(ind, cov, i1, true, i2, true, i3, true))->
		multiply(new CEVector(vP, i1, false))->
		multiply(new CEVector(vM, i2, false))->
		multiply(new CEVector(base, i3, false));
	delete i1; delete i2; delete i3;
	CExpression* res = f->multiply(ph_T);
	dbg->log(7, "CPPhoton::CreateHelicityT", res);
	return res;
}

/*
*	expression from BERENDS'81, p4.
*	factor for CreateHelicityL & CreateHelicityT
*	WARNING: differs from phot_helisity_factor() by numeric factor & masses
*/
CExpression* CPPhoton::CreateHFactor(CExpression *base, CExpression *vP, CExpression *vM) { // incl. massisve calc
	if ((base == NULL) || (vP == NULL) || (vM == NULL)) return NULL;
	CExpression* m2P = new CESProduct(vP, vP);
	CExpression* m2M = new CESProduct(vM, vM);
	m2P = CLHEPBasics::KinArrange(m2P);
	m2M = CLHEPBasics::KinArrange(m2M);
	if (!m2P->equals(m2M)) { dbg->log(0, "Error: sqr(p^+) != sqr(p^-) in photon helicity declararion", m2P, m2M); delete m2P; delete m2M; return NULL; }
	
	CExpression* _den = (new CESProduct(vP, vM))->multiply(new CESProduct(base, vP))->multiply(new CESProduct(base, vM));
	_den = _den->multiply(CENumber::GetTwo());
	_den = _den->minus(m2P->multiply(CEPower::square(new CESProduct(base, vP))));
	_den = _den->minus(m2M->multiply(CEPower::square(new CESProduct(base, vM))));
	_den = CEPower::sqrt(_den);
	CExpression* res = (CENumber::GetUnit())->divide(_den);
	return res;
}

/*
* v1 is P-
* v2 is P+
* (CreateHelicityL +/- i*CreateHelicityT)/sqrt(2)
*/
CExpression* CPPhoton::phot_helisity(CExpression *base, CExpression *v1, CExpression *v2, bool sign, CExpression *ind, bool cov)
{
	CExpression* ph_L = (new CESProduct(base, v2))->multiply(new CEVector(v1, ind, cov))->
		minus((new CESProduct(base, v1))->multiply(new CEVector(v2, ind, cov)));
	CExpression* i1 = CESymbol::getUnique();
	CExpression* i2 = CESymbol::getUnique();
	CExpression* i3 = CESymbol::getUnique();
	CExpression* ph_T = (new CEAntisymmetric(ind, cov, i1, true, i2, true, i3, true))->
		multiply(new CEVector(v2, i1, false))->
		multiply(new CEVector(v1, i2, false))->
		multiply(new CEVector(base, i3, false));
	ph_T = ph_T->multiply(CENumber::GetImUnit());
	delete i1;
	delete i2;
	delete i3;
	if (!sign) ph_T->inverseFlag();
	CExpression* res = ph_L->add(ph_T);
	
	res = phot_helisity_factor(base, v1, v2)->multiply(CENumber::GetTwo())->multiply(res);
	res = CExpression::arrge(res);
	dbg->log(20, "phot_helisity() composed: ", res);
	return res;
}

/*
*	v1 is P-
*	v2 is P+
*/
CExpression* CPPhoton::phot_berends_helisity(CExpression *base, CExpression *v1, CExpression *v2, bool sign, bool calc_precise) {

	int gamma_P_value = (sign)?1:(-1);
	CExpression* gP = new CEGammaSpecial(gamma_P_value);
	CExpression* gM = new CEGammaSpecial(-gamma_P_value);

	// if (!sign) gM & gP are inversed here.
	CExpression* a = (new CESGammaProduct(v1))->
		multiply(new CESGammaProduct(v2))->
		multiply(new CESGammaProduct(base));
	CExpression* b = (new CESGammaProduct(base))->
		multiply(new CESGammaProduct(v1))->
		multiply(new CESGammaProduct(v2));
	CExpression* res = a->multiply(gM)->minus(b->multiply(gP));
	res = res->multiply(CENumber::GetTwo());

	if (calc_precise) {
		CExpression* c = (new CESProduct(v1, v2))->multiply(new CESGammaProduct(base))->
			multiply(new CEGammaSpecial(0))->multiply(CENumber::GetTwo());
		if (!sign) c->inverseFlag();
		res = res->add(c);
	}

	res = phot_helisity_factor(base, v1, v2)->multiply(res);
	res = CExpression::arrge(res);
	dbg->log(20, sign, "phot_berends_helisity(sign == %ld) composed: ", res);
	return res;
}

/*
*	1/sqrt(2*{p1,p2})
*	Note: massless calculation is supposed.
*/
CExpression* CPPhoton::phot_helisity_factor(CExpression *base, CExpression *v1, CExpression *v2) {
	CExpression* _den = (new CESProduct(v1, v2))->
		multiply(new CESProduct(base, v1))->
		multiply(new CESProduct(base, v2));
	_den = CEPower::sqrt(_den);
	CExpression* res = (CENumber::GetFraction(1,4))->divide(_den);
	return res;
}

CExpression* CPPhoton::phot_DM_Stoks(CSequence* s_params,
									 CExpression* k,
									 CExpression* vP, CExpression* vM,
									 CExpression *ind1, bool cov1,
									 CExpression *ind2, bool cov2) {
	if ((s_params==NULL) || (k==NULL) || (vP==NULL) || (vM==NULL) || (ind1==NULL) || (ind2==NULL)) return NULL;
	if (s_params->size() != 3) { dbg->log(0, "CPPhoton::phot_DM_Stoks Error: invalid Stoks parameters", s_params); return NULL; }

	// create helicity basis using base-vectors
	CExpression* v11 = CreateHelicityL(k, vP, vM, ind1, cov1);
	CExpression* v12 = CreateHelicityL(k, vP, vM, ind2, cov2);
	CExpression* v21 = CreateHelicityT(k, vP, vM, ind1, cov1);
	CExpression* v22 = CreateHelicityT(k, vP, vM, ind2, cov2);

	//CExpression* temp = v11->clone()->multiply(v12->clone());
	//temp = temp->add(v21->clone()->multiply(v22->clone()));
	CExpression* temp = new CEMetric(ind1, ind2, cov1, cov2);
	temp->inverseFlag();
	CExpression* res = CENumber::GetHalf()->multiply(temp);

	temp = v11->clone()->multiply(v22->clone());
	temp = temp->add(v21->clone()->multiply(v12->clone()));
	res = res->add(temp->multiply(s_params->GetAt(0)->clone())->multiply(CENumber::GetHalf()));

	temp = v11->clone()->multiply(v22->clone());
	temp = temp->minus(v21->clone()->multiply(v12->clone()));
	res = res->minus(temp->multiply(s_params->GetAt(1)->clone())->multiply(CENumber::GetImUnit())->multiply(CENumber::GetHalf()));

	temp = v11->clone()->multiply(v12->clone());
	temp = temp->minus(v21->clone()->multiply(v22->clone()));
	res = res->minus(temp->multiply(s_params->GetAt(2)->clone())->multiply(CENumber::GetHalf()));

	delete v11; delete v21; delete v12; delete v22;
	dbg->log(15, "CPPhoton::phot_DM_Stoks(): DM created", res);
	return res;
}

/*
*	Note: works for "p\\_x" momenta
*	Fails for "k".
*/
/*void CPPhoton::create_polarization_vectors_symbols()
{
	if ((m_pol_vec_sym != NULL) && (m_pol_vec_cjg_sym != NULL)) return;

	CEString* str_mom = get_momentum_symbol()->toString();
	if (str_mom->length() < 4) {delete str_mom; return;}
	char* mom_id = CEUtil::copyStringFrom(str_mom->getData(), 3);
	delete str_mom;

	if (m_pol_vec_sym == NULL) {
		char* e_str = CEUtil::concat("e\\_", mom_id);
		m_pol_vec_sym = new CESymbol(e_str);
		CMemoryManager::mfree(e_str);
	}

	if (m_pol_vec_cjg_sym == NULL) {
		char* e_str_cjg = CEUtil::concat("ec\\_", mom_id);
		m_pol_vec_cjg_sym = new CESymbol(e_str_cjg);
		CMemoryManager::mfree(e_str_cjg);
	}
	
	CMemoryManager::mfree(mom_id);
}*/

/*void CPPhoton::add_vector_polarization_simpl_list()
{
	if (m_Vector_Polarization_Simpl_list_added) return;
	
	CEObject* rep_data = CEProcessor::getDataStorage()->get(PH_REPL_DATA_ID);
	CEReplList* reps = NULL;
	if (rep_data == NULL) {
		reps = new CEReplList();
		CEProcessor::getDataStorage()->put(PH_REPL_DATA_ID, reps);
	}
	else reps = (CEReplList*)rep_data;
	
	CENumber zero((long)0);
	CENumber unit((long)1);
	CExpression* munit = unit.clone(); munit->inverseFlag();
	CExpression* p = get_momentum_symbol();

	CExpression* a = new CESProduct(p, m_pol_vec_sym);
	reps->add(a, &zero);
	delete a;

	a = new CESProduct(m_pol_vec_sym, p);
	reps->add(a, &zero);
	delete a;
	
	a = new CESProduct(p, m_pol_vec_cjg_sym);
	reps->add(a, &zero);
	delete a;

	a = new CESProduct(m_pol_vec_cjg_sym, p);
	reps->add(a, &zero);
	delete a;

	a = new CESProduct(m_pol_vec_sym, m_pol_vec_sym);
	reps->add(a, &zero);
	delete a;

	a = new CESProduct(m_pol_vec_cjg_sym, m_pol_vec_cjg_sym);
	reps->add(a, &zero);
	delete a;

	a = new CESProduct(m_pol_vec_sym, m_pol_vec_cjg_sym);
	reps->add(a, munit);
	delete a;

	a = new CESProduct(m_pol_vec_cjg_sym, m_pol_vec_sym);
	reps->add(a, munit);
	delete a;

	delete munit;
	m_Vector_Polarization_Simpl_list_added = true;
}*/

/*
*	vec_sym should be  "+"-signed.
*/
/*CENumVector* CPPhoton::get_vector_stub(CExpression *vec_sym)
{
	if (vec_sym == NULL) return NULL;
	CENumVector* res = CEParticle::get_vector_stub(vec_sym);
	if (res != NULL) return res;
	
	// compare vec_sym to helisity vectors
	if (m_pol_vec_sym != NULL) {
		if (vec_sym->equals(m_pol_vec_sym)) {
			//printf("stub: %s", m_pol_vec_stub->toLine());
			return m_pol_vec_stub;
		}
	}
	if (m_pol_vec_cjg_sym != NULL) {
		if (vec_sym->equals(m_pol_vec_cjg_sym)) {
			return m_pol_vec_cjg_stub;
		}
	}

	return NULL;
}*/

/*void CPPhoton::fill_pol_vector_stubs()
{
	if (m_pol_vec_stub != NULL) delete m_pol_vec_stub;
	if (m_pol_vec_cjg_stub != NULL) delete m_pol_vec_cjg_stub;

	CEValue* zv = new CERealValue(0);
	CEValue* unit = new CERealValue(1);
	CEValue* min_unit = new CERealValue(-1);

	if (m_helicity == 0) {
		m_pol_vec_stub = new CENumVector((CEValue*)zv->clone());
		m_pol_vec_stub->addItem(zv->clone());
		m_pol_vec_stub->addItem(zv->clone());
		m_pol_vec_cjg_stub = new CENumVector((CEValue*)zv->clone());
		m_pol_vec_cjg_stub->addItem(zv->clone());
		m_pol_vec_cjg_stub->addItem(zv->clone());

		m_pol_vec_stub->addItem(zv->clone());
		m_pol_vec_stub->addItem(zv->clone());
		m_pol_vec_cjg_stub->addItem(zv->clone());
		m_pol_vec_cjg_stub->addItem(zv->clone());
		delete zv;
		delete unit;
		delete min_unit;
		return;
	}

	if ((m_helicity != 1) && (m_helicity != -1)) {
		printf("Error: unexpected photon polarization identifier.\n");
		delete zv;
		delete unit;
		delete min_unit;
		return;
	}

	m_pol_vec_stub = new CENumVector((CEValue*)min_unit->clone());
	m_pol_vec_stub->addItem(zv->clone());
	m_pol_vec_stub->addItem(zv->clone());
	m_pol_vec_cjg_stub = new CENumVector((CEValue*)min_unit->clone());
	m_pol_vec_cjg_stub->addItem(zv->clone());
	m_pol_vec_cjg_stub->addItem(zv->clone());

	CEValue* sqrt_2 = CEValue::Sqrt(new CERealValue(2));
	CExpression* u_to_sqrt_2 = unit->clone()->divide(sqrt_2->clone());
	CExpression* min_u_to_sqrt_2 = min_unit->clone()->divide(sqrt_2->clone());

	CEValue* c_unit = new CEComplexValue(0, 1);
	CEValue* min_c_unit = new CEComplexValue(0, -1);
	CExpression* cu_to_sqrt_2 = c_unit->divide(sqrt_2->clone());
	CExpression* min_cu_to_sqrt_2 = min_c_unit->divide(sqrt_2);
	//to delete : zv; unit; min_unit; u_to_sqrt_2; min_u_to_sqrt_2;cu_to_sqrt_2; min_cu_to_sqrt_2

	bool mom_sign = (((CEValue*)(get_stub()->GetAt(1)))->to_double() > 0);
	bool im_part_sign = (m_helicity > 0);
	if (!mom_sign) im_part_sign = !im_part_sign;

	//	<2>: 1/sqrt(2)
	m_pol_vec_stub->addItem(u_to_sqrt_2->clone());
	m_pol_vec_cjg_stub->addItem(u_to_sqrt_2->clone());

	//	<3>: (+/-)i/sqrt(2)
	if (im_part_sign) {
		m_pol_vec_stub->addItem(cu_to_sqrt_2->clone());
		m_pol_vec_cjg_stub->addItem(min_cu_to_sqrt_2->clone());
	}
	else {
		m_pol_vec_stub->addItem(min_cu_to_sqrt_2->clone());
		m_pol_vec_cjg_stub->addItem(cu_to_sqrt_2->clone());
	}

	//printf("stub created: %s\n", m_pol_vec_stub->toLine());
	//printf("stub created: %s\n", m_pol_vec_cjg_stub->toLine());

	delete zv;
	delete unit;
	delete min_unit;
	delete u_to_sqrt_2;
	delete min_u_to_sqrt_2;
	delete cu_to_sqrt_2;
	delete min_cu_to_sqrt_2;
}*/

/*CESymbol* CPPhoton::get_pol_vec_symbol()
{
	return m_pol_vec_sym;
}*/

/*CESymbol* CPPhoton::get_pol_vec_cjg_symbol()
{
	return m_pol_vec_cjg_sym;
}*/
