////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EParticleFunction.h: interface for the CEParticleFunction class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EPARTICLEFUNCTION_H__CE340D34_2C27_4F90_8D01_69C38522EF28__INCLUDED_)
#define AFX_EPARTICLEFUNCTION_H__CE340D34_2C27_4F90_8D01_69C38522EF28__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CEParticleFunction : public CExpression  
{
protected:
	long m_particle_kind;
	int m_vertex_ID; // attached vertex data
	unsigned long m_point_ID; // substitute for x-coordinate (attached vertex No).

protected:
	// PK < 0 means antiparticle for fermions
	CEParticleFunction(long _particle_kind, int _vertex_ID, unsigned long _point_ID);

public:
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual bool contain_vector(void * obj);
	virtual void collectProperties(CEProperties* props, unsigned set = CEProperties::PROP_MASK_ALL);
	virtual unsigned long props_long();
	virtual bool equals(CExpression *other);
	//virtual bool less_than(CExpression* other);

	/*
	*	Nore: field operator's PK is always > 0 (they use 'conjugated flag')
	*	wave function's pk is signed value.
	*	Use getLogicalPK() to obtain signed value for every object
	*/
	virtual long getParticleKind();
	virtual long getLogicalPK() = 0;
	int getPointID();
	int getVertexID();
	void setPointID(int _point_ID);

public:
	virtual ~CEParticleFunction();
};

#endif // !defined(AFX_EPARTICLEFUNCTION_H__CE340D34_2C27_4F90_8D01_69C38522EF28__INCLUDED_)
