////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// PFermion.h: interface for the CPFermion class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_PFERMION_H__69BFE554_71AC_4223_B731_952519235933__INCLUDED_)
#define AFX_PFERMION_H__69BFE554_71AC_4223_B731_952519235933__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CPFermion : public CEParticle  
{
	long m_fermion_PK;
	
	int m_helicity_value; // 0 value means no integer helicity to be involved, but does not mean unpolarized fermion
	CExpression* m_helicity_symbol; // NULL does not mean unpolarized fermion

	//
	class CPFermionColorIndexContainer : public CEObject {
		class CPFermionColorIndexContainerC : public CEObject {
			int m_idKin;
			CESymbol* m_ClrIndex;
		public:
			CPFermionColorIndexContainerC(int _idKin, CESymbol* _ClrIndex);
			virtual ~CPFermionColorIndexContainerC();
			int IdKin() { return m_idKin; }
			CESymbol* ClrIndex() { return m_ClrIndex; }
		};
		CEPtrArray m_info;
	public:
		CPFermionColorIndexContainer();
		virtual ~CPFermionColorIndexContainer();
		CESymbol* GetColorIndex(int kinID);				//returns non-cloned value, NULL if error
	};
	CESymbol* m_eColorIndex;
	static CPFermionColorIndexContainer* m_ColorIndexCache;	//used to uniform color indices in various sub-processes & avoid extra unique symbols
public:
	virtual long get_PK();
	virtual CExpression* getDensityMatrix(CEPtrArray* vars);

	virtual CExpression* GetColorIndex(); //NULL if no color index is applicable

	CExpression* get_helicity_symbol();
	int get_helicity_value();

	void setHelicityData(int _val);
	void setHelicityData(CExpression* _val); // does not affects parameter

	//CPFermion(bool _is_antiparticle, CExpression* id, CExpression* sym_mass, CENumVector* _stub);
	//CPFermion(CExpression* _momentum_sym, CExpression* _mass_sym, bool _is_antiparticle, CEPtrArray* params);

	virtual ~CPFermion();

public:
	//********************* Density Matrix ********************************
	// m==NULL means massless case
	// e==NULL means unpolarized
	// f_charge: true for fermion, false for anti-fermion
	//
	// Important note: use 'particle->getDensityMatrix()' instead of these calls!
	// Use 'CPFermion::createDM()' for non-real-particle objects only (e.g. used in amplitude calculation).
	//
	static CExpression* createDM(CExpression* p, CExpression* m, CExpression* e, bool f_charge);
	static CExpression* createDM(CExpression* p, int h, bool f_charge, CExpression* m=NULL);
	// common case
private:
	static CExpression* CreateDensityMatrixImpl(
		CExpression* pAbs, CExpression* m, bool bMomSign,
		bool is_polarized,
		CExpression* e,						//set polarization using \hat{eSym}
		CExpression* eVecIndex,				//if !=NULL: set polarization using gamma^{index} * e [=={...}_{index}]
		CExpression* h_sym, int h_val,
		bool bPKgt0,						//true(fermion), false(antifermion)
		bool bAverageSpins,					//otherwise sum spins for unpolarized particle
		CExpression* clrIndex1, CExpression* clrIndex2
											//set NULL for non-quark particle
		);

	//[old] static CExpression* createDMimpl(CExpression* p,CExpression* m, bool is_polarized,CExpression* e, CExpression* h_sym, int h_val,bool is_particle, bool _average);

public:
	// Constructor: (old) CEValue* _mass = NULL; NULL mass means no numerical calculations are expected
	// Constructor: does NOT clone CExpressions!
	CPFermion(long _fermion_PK,
		CExpression* id, CExpression* sym_mass, CExpression* _pol_vec_sym,
		int _ID_kin, CEValue* _mass_val = NULL);
private:
	//bool init_fermion_data(CEPtrArray* params);
};

///////////////////////////////////////////////////////// CPProtonSF - proton for Structure functions approach
class CPProtonSF: public CPFermion {
	CSequence m_TmpIndices;
public:
	virtual CExpression* getDensityMatrix(CEPtrArray* vars);
	//TODO check possible errors of CPFermion::get_helicity_symbol(),CPFermion::get_helicity_value() & others
public:
	// Constructor: (old) CEValue* _mass = NULL; NULL mass means no numerical calculations are expected
	CPProtonSF(long _fermion_PK, //always=CEParticle::PARTICLE_PROTON
		CExpression* id, CExpression* sym_mass, CExpression* _pol_vec_sym,
		int _ID_kin, CEValue* _mass_val = NULL);
	virtual ~CPProtonSF();
	CExpression* CreateHadronicTensor(CExpression* Q, CExpression* ind1, CExpression* ind2, bool cov1, bool cov2);
public:
	static CExpression* GetGammaVertexSym(CExpression* momQ,CExpression* mom1,CExpression* mom2);
	//protonIDs: to cache IDs collection
	//tmpIndices: to uniform temporary indices in sub-results
	static bool ReconstructSF(CSequence* me, CSequence* factor, CIntArray* protonIDs, CSequence* tmpIndices);
	static CExpression* ExtractSFVectorRuleImpl(CExpression* src, CEPtrArray* data_arr);
private:
	static CExpression* AddSFVecAndGetSubstIndex(CExpression* sfVecFunc, CEPtrArray* data_arr);
};

#endif // !defined(AFX_PFERMION_H__69BFE554_71AC_4223_B731_952519235933__INCLUDED_)
