////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// PPhoton.h: interface for the CPPhoton class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_PPHOTON_H__39E8A2C7_DCB8_4F1D_92AF_041643C85727__INCLUDED_)
#define AFX_PPHOTON_H__39E8A2C7_DCB8_4F1D_92AF_041643C85727__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//##ModelId=3E3FF2DE00FA
class CPPhoton : public CPAbstractBoson  
{
	int m_helicity;
	CExpression* m_h_base_1; // P-
	CExpression* m_h_base_2; // P+

	CExpression* m_DM_base; // base for unpolarized DM

	bool m_UseStoksParams;

	CSequence* m_StoksParams; // array[3] or NULL
	CExpression* m_stoks_e_basis1; // P- to construct stoks basis
	CExpression* m_stoks_e_basis2; // P+ to construct stoks basis

	// old variables (BEGIN)
	//CESymbol* m_pol_vec_sym;
	//CESymbol* m_pol_vec_cjg_sym;
	CENumVector* m_pol_vec_stub;
	CENumVector* m_pol_vec_cjg_stub;
	//bool m_DM_node_data_filled;
	//bool m_Vector_Polarization_Simpl_list_added;
	// old variables (END)

public:
	///////////////////////////////////////////////////////////// construction/destruction
	CPPhoton(CExpression* _momentum_sym, CExpression* _pol_vec_sym,
		int _ID_kin, CEPtrArray *params);
	virtual ~CPPhoton();

	///////////////////////////////////////////////////////////// get data
	virtual CExpression* getDensityMatrix(CEPtrArray* vars);
	virtual CExpression* getHelicityVector(CExpression* vector_index);
	void setHelicityData(int _val, CExpression* _base_1, CExpression* _base_2);
	void setUnpolarizedDMBase(CExpression* DM_base); // does not sets particle unpolarized
	int get_helicity_value(); // for debug only
	CExpression* get_helicity_base(int base_no); // for debug only (base_no = 1,2)
	virtual void set_polarization_info(int _pol_val);
	void setStoksParamsUse(bool _use_Stoks, CExpression* _stoks_e_basis1, CExpression* _stoks_e_basis2);
	CSequence* GetStoksParamsDeclared(); // NULL if Stoks params not declared
	
	CExpression* getBerendsHelicity(bool calc_precise);

	///////////////////////////////////////////////////////////// constants
	static const int PH_VECTOR_NOTATION;
	static const int PH_BERENDS_NOTATION;

	static const char* dm_data_node;
	static const char* PH_REPL_DATA_ID;

	///////////////////////////////////////////////////////////// statics
	static CExpression* CreateHelicityL(CExpression *base, CExpression *vP, CExpression *vM, CExpression *ind, bool cov);
	static CExpression* CreateHelicityT(CExpression *base, CExpression *vP, CExpression *vM, CExpression *ind, bool cov);
	static CExpression* CreateHFactor(CExpression *base, CExpression *vP, CExpression *vM); // incl. massisve calc

	static CExpression* phot_helisity(CExpression *base, CExpression *v1, CExpression *v2, bool sign, CExpression *ind, bool cov);
	static CExpression* phot_berends_helisity(CExpression *base, CExpression *v1, CExpression *v2, bool sign, bool calc_precise);
	static CExpression* phot_helisity_factor(CExpression *base, CExpression *v1, CExpression *v2);
	static CExpression* phot_DM_Stoks(CSequence* s_params, CExpression* k, CExpression* vP, CExpression* vM, CExpression *ind1, bool cov1, CExpression *ind2, bool cov2);

	///////////////////////////////////////////////////////////// old
	/*CExpression* get_density_matrix(bool is_initial);
	int set_helicity(int h_val, CExpression* h_base_1, CExpression* h_base_2);
	CESymbol* get_pol_vec_cjg_symbol();
	CESymbol* get_pol_vec_symbol();
	virtual void fill_pol_vector_stubs();
	virtual CENumVector* get_vector_stub(CExpression* vec_sym);
	void add_vector_polarization_simpl_list();
	void create_polarization_vectors_symbols();*/

private:
	bool init_photon_data(CEPtrArray *params);
};

#endif // !defined(AFX_PPHOTON_H__39E8A2C7_DCB8_4F1D_92AF_041643C85727__INCLUDED_)
