////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// QuarkMixMatrix.cpp: implementation of the CQuarkMixMatrix class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CQuarkMixMatrix::CQuarkMixMatrix(long quark_PK_1, long quark_PK_2) : CExpression()
{
	m_quark_PK_1 = quark_PK_1; m_quark_PK_2 = quark_PK_2;
	// apply simmetry
	if (m_quark_PK_1 > m_quark_PK_2) { long temp = m_quark_PK_1; m_quark_PK_1=m_quark_PK_2; m_quark_PK_2=temp; }
}

CQuarkMixMatrix::~CQuarkMixMatrix()
{
}

const char* CQuarkMixMatrix::CLASS_NAME = "CQuarkMixMatrix";
const char* CQuarkMixMatrix::ClassName() const { return CLASS_NAME; }

bool CQuarkMixMatrix::contain_vector(void * obj) { return false; }

CExpression* CQuarkMixMatrix::clone() const {
	CExpression* res = new CQuarkMixMatrix(m_quark_PK_1, m_quark_PK_2);
	copyAttributesTo(res);
	return res;
}

long CQuarkMixMatrix::GetQuarkPK1() { return m_quark_PK_1; }
long CQuarkMixMatrix::GetQuarkPK2() { return m_quark_PK_2; }

CEString* CQuarkMixMatrix::toString(int mode) {
	CEString* res = NULL;
	switch (mode) {
	case STORE_WR_MATHEMATICA:
		res = new CEString("CKM");
		res->AddString(CDiagramPhysics::get_particle_symbolA(m_quark_PK_1, CDiagramPhysics::SYMBOL_STYLE_NB));
		res->AddString(CDiagramPhysics::get_particle_symbolA(m_quark_PK_2, CDiagramPhysics::SYMBOL_STYLE_NB));

		if (!getFlag()) { res->insertString("- \\(", 0); res->AddString("\\)"); }
		break;
	case STORE_TEX:
	case STORE_CPP:
	case STORE_FORTRAN_SYMBOL:
	case STORE_PLAIN_TEXT:
		res = new CEString("V");
		res->AddString(CDiagramPhysics::get_particle_symbolA(m_quark_PK_1, CDiagramPhysics::SYMBOL_STYLE_TEX));
		res->AddString(CDiagramPhysics::get_particle_symbolA(m_quark_PK_2, CDiagramPhysics::SYMBOL_STYLE_TEX));
		if (!getFlag()) { res->insertString("- ", 0); }
		break;
	default:
		res = new CEString("<Not supported data format.>");
	}
	return res;
}

bool CQuarkMixMatrix::equals(CExpression* other) {
	if (other == NULL) return false;
	if (!CEUtil::equals(other->ClassName(), CLASS_NAME)) return false;
	CQuarkMixMatrix* o = (CQuarkMixMatrix*)other;
	if ((m_quark_PK_1 == o->GetQuarkPK1()) && (m_quark_PK_2 == o->GetQuarkPK2()) && (attributes_equals_to(o))) return true;
	return false;
}

const char* CQuarkMixMatrix::CLASS_TAG_NAME = "CKM";
const char* CQuarkMixMatrix::get_TAG_Name() { return CLASS_TAG_NAME; }

long CQuarkMixMatrix::store_stream(ofstream* p_ofs, DWORD settings) {
	CENumber* pk1 = new CENumber(m_quark_PK_1);
	CENumber* pk2 = new CENumber(m_quark_PK_2);
	long res = CESupport::store_stream(pk1, p_ofs, settings);
	res += CESupport::store_stream(pk2, p_ofs, settings);
	delete pk1; delete pk2;
	return res;
}

CESerializable* CQuarkMixMatrix::restore_from_buffer(const char* data, long pos, CEMap* params, DWORD settings) {
	long quark_PK_1 = 0;
	bool read_OK = CESupport::readXMLLong(data, &pos, &quark_PK_1, settings, CLASS_NAME);
	if (!read_OK) return NULL;

	long quark_PK_2 = 0;
	read_OK = CESupport::readXMLLong(data, &pos, &quark_PK_2, settings, CLASS_NAME);
	if (!read_OK) return NULL;

	return new CQuarkMixMatrix(quark_PK_1, quark_PK_2);
}

CQuarkMixMatrix* CQuarkMixMatrix::CreateDU() { return new CQuarkMixMatrix(CEParticle::PARTICLE_QUARK_D, CEParticle::PARTICLE_QUARK_U); }
CQuarkMixMatrix* CQuarkMixMatrix::CreateDC() { return new CQuarkMixMatrix(CEParticle::PARTICLE_QUARK_D, CEParticle::PARTICLE_QUARK_C); }
CQuarkMixMatrix* CQuarkMixMatrix::CreateDT() { return new CQuarkMixMatrix(CEParticle::PARTICLE_QUARK_D, CEParticle::PARTICLE_QUARK_T); }
CQuarkMixMatrix* CQuarkMixMatrix::CreateSU() { return new CQuarkMixMatrix(CEParticle::PARTICLE_QUARK_S, CEParticle::PARTICLE_QUARK_U); }
CQuarkMixMatrix* CQuarkMixMatrix::CreateSC() { return new CQuarkMixMatrix(CEParticle::PARTICLE_QUARK_S, CEParticle::PARTICLE_QUARK_C); }
CQuarkMixMatrix* CQuarkMixMatrix::CreateST() { return new CQuarkMixMatrix(CEParticle::PARTICLE_QUARK_S, CEParticle::PARTICLE_QUARK_T); }
CQuarkMixMatrix* CQuarkMixMatrix::CreateBU() { return new CQuarkMixMatrix(CEParticle::PARTICLE_QUARK_B, CEParticle::PARTICLE_QUARK_U); }
CQuarkMixMatrix* CQuarkMixMatrix::CreateBC() { return new CQuarkMixMatrix(CEParticle::PARTICLE_QUARK_B, CEParticle::PARTICLE_QUARK_C); }
CQuarkMixMatrix* CQuarkMixMatrix::CreateBT() { return new CQuarkMixMatrix(CEParticle::PARTICLE_QUARK_B, CEParticle::PARTICLE_QUARK_T); }

CExpression* CQuarkMixMatrix::ToSymbolForm() {
	char* tmp = (char*)CMemoryManager::mmalloc(4);
	*tmp = 'V';
	*(tmp+1) = *(CDiagramPhysics::get_particle_symbolA(m_quark_PK_1, CDiagramPhysics::SYMBOL_STYLE_TEX));
	*(tmp+2) = *(CDiagramPhysics::get_particle_symbolA(m_quark_PK_2, CDiagramPhysics::SYMBOL_STYLE_TEX));
	*(tmp+3) = '\0';
	CESymbol* res = new CESymbol(tmp);
	CMemoryManager::mfree(tmp);
	copyAttributesTo(res, true);
	return res;
}

CExpression* CQuarkMixMatrix::QMMConvertToSymbols(CExpression* src, bool del_src) {
	if (src == NULL) return NULL;

	CERule* r = new CESocketRule((void*)CQuarkMixMatrix::QMMConvertSym_ruleImpl);
	CExpression* res = r->ApplyTo(src);
	delete r;
	if ((del_src) && (res != src)) { delete src; }
	return res;
}

CExpression* CQuarkMixMatrix::QMMConvertSym_ruleImpl(CExpression* src, CEPtrArray* data_arr) {
	if (!(CEUtil::equals(src->ClassName(), CQuarkMixMatrix::CLASS_NAME))) return src;
	CExpression* res = ((CQuarkMixMatrix*)src)->ToSymbolForm(); // attributes are copied
	return res;
}
