////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ParticleWaveFunction.cpp: implementation of the CParticleWaveFunction class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

CDebug* CParticleWaveFunction::dbg = new CDebug("CParticleWaveFunction");
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CParticleWaveFunction::CParticleWaveFunction(long _particle_kind,
											 int _vertex_ID,
											 unsigned long _point_ID,
											 CExpression* _momentum_sym)
: CEParticleFunction(_particle_kind, _vertex_ID, _point_ID)
{
	if (_momentum_sym != NULL) m_momentum_sym = _momentum_sym->clone();
	else {
		m_momentum_sym = NULL;
		dbg->log(0, "Error: NULL particle momenta in wave function. Warning ignored.");
	}
}

CParticleWaveFunction::~CParticleWaveFunction() {
	if (m_momentum_sym != NULL) delete m_momentum_sym;
}

bool CParticleWaveFunction::contain_vector(void * obj) {
	if (m_momentum_sym == obj) return true;
	if (m_momentum_sym != NULL) { if (m_momentum_sym->contain_vector(obj)) return true; }
	return false;
}

const char* CParticleWaveFunction::CLASS_NAME = "CParticleWaveFunction";
const char* CParticleWaveFunction::ClassName() const { return CLASS_NAME; }
long CParticleWaveFunction::getLogicalPK() { return m_particle_kind; }
CExpression* CParticleWaveFunction::getMomentumSym() { return m_momentum_sym; }
void CParticleWaveFunction::setMomentumSym(CExpression* mom_sym) {
	if (mom_sym == NULL) { dbg->log(0, "Error: setMomentumSym(NULL) call"); CEException::halt(); }
	if (mom_sym != m_momentum_sym) {
		if (m_momentum_sym != NULL) delete m_momentum_sym;
		m_momentum_sym = mom_sym->clone();
	}
}

CEString* CParticleWaveFunction::toString(int mode) {
	CEString* res = new CEString("UnknownParticleWaveFunction");
	if (!getFlag()) res->insertString("- ");
	return res;
}

void CParticleWaveFunction::collectProperties(CEProperties* props, unsigned set) {
	CEParticleFunction::collectProperties(props, set);
	props->setProperty(CEProperties::PROP_MASK_IS_WAVE_FUNCTION, true);
}

unsigned long CParticleWaveFunction::props_long() {
	unsigned long res = CEParticleFunction::props_long();
	res = CEProperties::set_property_to(res, CEProperties::PROP_MASK_IS_WAVE_FUNCTION, true);
	return res;
}

CExpression* CParticleWaveFunction::conjugated(int mode) {
	CEException::halt("Illegal Wave Function conjugation involved."); return NULL;
}

bool CParticleWaveFunction::equals(CExpression *other) {
	if (!CEParticleFunction::equals(other)) return false;

	// specific comparison
	CExpression* o_mom = ((CParticleWaveFunction*)other)->getMomentumSym();
	if ((m_momentum_sym != NULL) && (o_mom != NULL)) {
		if (!m_momentum_sym->equals(o_mom)) return false;
	} else {
		if (m_momentum_sym != o_mom) return false; // the only NULL
	}
	return true;
}

/*bool CParticleWaveFunction::less_than(CExpression* other) {
	if (!other->getProperty(CEProperties::PROP_MASK_IS_WAVE_FUNCTION)) return CEParticleFunction::less_than(other);

	if (CEParticleFunction::less_than(other)) return true;
	...
}*/

CExpression* CParticleWaveFunction::applyRule(CEAbstractRule* rule) {
	CERule* r = (CERule*)rule;
	if (m_momentum_sym != NULL) {
		CExpression* mom_repl = r->ApplyTo(m_momentum_sym, false);
		if ((mom_repl != m_momentum_sym) && (mom_repl != NULL)) {
			delete m_momentum_sym;
			m_momentum_sym = mom_repl;
		}
	}
	return this;
}
