////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// WaveFuncFermion.cpp: implementation of the CWaveFuncFermion class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CWaveFuncFermion::CWaveFuncFermion(long _particle_kind,
								   int _vertex_ID,
								   unsigned long _point_ID,
								   CExpression* _momentum_sym)
: CParticleWaveFunction(_particle_kind, _vertex_ID, _point_ID, _momentum_sym)
{
	if (!CEParticle::is_fermion_PK(m_particle_kind)) { CEException::halt("Fermion operator construction for non-fermion field"); }
}

CWaveFuncFermion::~CWaveFuncFermion() {}

const char* CWaveFuncFermion::CLASS_NAME = "CWaveFuncFermion";
const char* CWaveFuncFermion::ClassName() const { return CLASS_NAME; }
bool CWaveFuncFermion::isConjugated() { return (getParticleKind() < 0); }

CEString* CWaveFuncFermion::toString(int mode) {
	CEString* res = new CEString("\\(\\[Psi]"); // first \\( -- for ... \\)\\+ m_point_ID
	if (isConjugated()) { res->insertString("\\("); res->AddString("\\&_\\)"); }

	res->AddString("\\_\\(");
	if (m_momentum_sym != NULL) { res->AddString(m_momentum_sym->toString(mode)); }
	else { res->AddString("?"); }
	res->AddString("\\)");

	res->AddString("\\)\\+");
	res->add_int(m_point_ID);

	// sign
	if (getFlag()) res->insertString("\\(");
	else res->insertString("- \\(");
	res->AddString("\\)");
	return res;
}

#ifdef PF_LESS_THAN
bool CWaveFuncFermion::less_than(CExpression* other) {
	if (other == NULL) return false;
	if (other->getProperty(CEProperties::PROP_MASK_IS_PARTICLE_FUNCTION)) {
		if (!CEUtil::LEQU(getFlag(), other->getFlag())) return other->getFlag(); // compare sign: "-"sym is < than "sym"
		const char* o_cn = other->ClassName();
		// compare to COpFermionField, COpBosonField, CWaveFuncFermion, CWaveFuncBoson
		// A < Psi < S, Wf < Field
		if (o_cn == COpFermionField::CLASS_NAME) {
			return true;
		}
		if (o_cn == COpBosonField::CLASS_NAME) {
			bool is_Scalar_b = (((COpBosonField*)other)->getVectorIndex() == NULL);
			return (is_Scalar_b);
		}
		if (o_cn == CWaveFuncFermion::CLASS_NAME) {
			CWaveFuncFermion* o = (CWaveFuncFermion*)other;
			if (!CEUtil::LEQU(isConjugated(), o->isConjugated())) return isConjugated(); // u_bar < u
			CExpression* m_b = o->getMomentumSym();
			if (!CEUtil::LEQU((m_momentum_sym==NULL), (m_b==NULL))) return (m_momentum_sym != NULL);
			if (m_momentum_sym != NULL) { if (m_momentum_sym->less_than(m_b)) return true; if (!m_momentum_sym->equals(m_b)) return false; }
			if (m_point_ID != o->getPointID()) return ((long)m_point_ID < o->getPointID());
			return false;
		}
		if (o_cn == CWaveFuncBoson::CLASS_NAME) {
			bool is_Scalar_b = (((CWaveFuncBoson*)other)->getVectorIndex() == NULL);
			return (is_Scalar_b);
		}
	}
	return CExpression::less_than(other);
}
#endif //PF_LESS_THAN

void CWaveFuncFermion::collectProperties(CEProperties* props, unsigned set) {
	CParticleWaveFunction::collectProperties(props, set);
	props->setCommutative(false);
}

unsigned long CWaveFuncFermion::props_long() {
	unsigned long res = CParticleWaveFunction::props_long();
	res = CEProperties::set_property_to(res, CEProperties::PROP_MASK_COMMUTATIVE, false);
	return res;
}

CExpression* CWaveFuncFermion::clone() const {
	CExpression* res = new CWaveFuncFermion(m_particle_kind, m_vertex_ID, m_point_ID, m_momentum_sym);
	copyAttributesTo(res);
	return res;
}
