////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// AbstractGenerator.cpp: implementation of the CAbstractGenerator class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <stdlib.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDebug* CAbstractGenerator::dbg = new CDebug("CAbstractGenerator");

CAbstractGenerator::CAbstractGenerator():CEAbstractTask()
{
	//randomize();
	srand(1234);
	m_smin = NULL;
	m_s2min = NULL;
	m_p_in = NULL;
	m_ph_f = NULL;
	m_s = NULL;
	m_rand_prev = new CERealValue(0);

	zero = new CERealValue(0);
	one = new CERealValue(1);
	two = new CERealValue(2);
	half = new CERealValue(0.5);
	pi = new CERealValue(3.1415926535897932385);
	two_pi = (CEValue*)pi->clone()->multiply(two->clone());
	cfn = ((CEValue*)two_pi->clone())->inverse(false)->Sqrx();
}

CAbstractGenerator::~CAbstractGenerator()
{
	delete zero;
	delete one;
	delete two;
	delete half;
	delete pi;
	delete two_pi;
	delete cfn;
	delete m_rand_prev;
	if (m_p_in != NULL) delete m_p_in;
	if (m_s != NULL) delete m_s;
	if (m_smin != NULL) {
		//m_smin->clear();
		delete m_smin;
	}
	if (m_s2min != NULL) delete m_s2min;
	if (m_ph_f != NULL) delete m_ph_f;
}

CEValue* CAbstractGenerator::rand_next(CEValue * prev)
{
	//CEValue* res = new CEValue((double)URAND8(prev->get_double()));
	delete prev;
	//return res;
	//return prev;
	return new CERealValue(((double)rand()/RAND_MAX));
}

CEObject* CAbstractGenerator::doTask()
{
	//CKinematics* kin = CEProcessor::kinematics();
	//kin->log_vectors();
	//dbg->log(5, "initial vector #1", kin->vector(0, true));
	//dbg->log(5, "initial vector #2", kin->vector(1, true));

	dbg->log(20, "initalized");
	
	CExpression* cs1 = CFeynLib::test();
	//CExpression* cs2 = CLMath::temp_func0();
	//CEHistogram* hs = new CEHistogram(100, -1, 1);
	long num_events = 1000;
	CEProcessor::setKinematics(new CElasticKinematics(),true);
	
	unsigned num_points = 10;
	double en_step = 1000/num_points;
	double cms_en = en_step/2;
	CEHistogram* hs = new CEHistogram(num_points, 0, 1000);

	double m = 0.0005;

	CKinematics* kin = CEProcessor::kinematics();
	unsigned j;
	for (j = 0; j<num_points; j++) {
		kin->set_CMS_energy(cms_en);

		//total c/s
		double cs_res = mid(cs1, num_events, false);
		//printf("res: %f \n", cs_res);
		//log_item(10, new CESymbol("s"));
		//log_item(10, new CESymbol("m"));
		hs->add_value(cms_en, CFeynLib::total_cs(cs_res, cms_en, 0.0005), 0);
		cms_en += en_step;
	}

	//CEProcessor::kinematics()->log_vectors();
	//fill_hs(cs2, num_events, hs, 0, false);//CLMath::temp_func0()
	//fill_hs(cs1, num_events, hs, 0, false);
	//CEProcessor::kinematics()->log_vectors();
	//hs->visualize(0);

	/*for (i = 1; i<10; i++) {
		CESum* cs1_sum = (CESum*)cs1;
		fill_hs(cs1_sum->get_n_items(i), num_events, hs, i);
	}*/

	//CEProcessor::setKinematics(new CInelasticKinematics(), true);
	//fill_hs(CLMath::temp_func2(), num_events, hs, 2);
	//hs->visualize(1);
	//fill_hs(CLMath::temp_func3(), num_events, hs, 2);

	//CEProcessor::kinematics()->log_vectors();

	dbg->log(20, "generation performed");
	hs->normalize_all();

	//hs->add_bank(2, 1);
	//CESupport::SaveResults(cs2);
	//hs->visualize(0);

	//CESupport::SaveResults(cs1);
	/*for (i = 1; i<10; i++) {
		hs->visualize(i);
	}*/

	//CEProcessor::kinematics()->log_vectors();
	hs->visualize(0);
	//CEProcessor::kinematics()->log_vectors();

	//hs->visualize(2);
	//hs->visualize(3);
	//hs->visualize(2);
	dbg->log(20, "histogram stored");

	//CEProcessor::kinematics()->log_vectors();
	delete hs;
	dbg->log(30, "histogram deleted");

	//CEProcessor::kinematics()->log_vectors();
	dbg->log(30, "cs1:", cs1);
	//delete cs2;
	delete cs1;
	dbg->log(20, "finished");
	//CEProcessor::kinematics()->log_vectors();
	return NULL;
}

bool CAbstractGenerator::generate_event()
{
	CKinematics* kin = CEProcessor::kinematics();
	int nr = kin->num_final_particles();
	dbg->log(20, nr, "generate_event for %d paricles");
	

	///////////////////////////////////////////////
	//SUBROUTINE PH_EVT(NF, MASS, P_IN, S_IN, Q, PH_FACTOR)

	/*
      IMPLICIT NONE

      INTEGER	  NF,NR,N_MAX
      PARAMETER   (N_MAX = 20)
      REAL*8	  MASS(1),P_IN(1),S_IN,Q(4,1),PH_FACTOR

      REAL*8	  SMIN(N_MAX),S2MIN
      INTEGER	  I,K
      LOGICAL	  INIT

      REAL*8	  P_PREV(4),P_NEXT(4),S_PREV,S_NEXT,SS_PREV,SS_NEXT
      REAL*8	  S_MAX,SS_MAX,DS,LAMBDA,COST,SINT,FI,XI,ETA
      REAL*8	  RAN_NEXT

      REAL*8	  ZERO,HALF,ONE,TWO,PI,TWO_PI,CFN
      PARAMETER   (ZERO = 0D0,
     *		   ONE	= 1D0,
     *		   TWO	= 2D0,
     *		   HALF = ONE/TWO,
     *		   PI = 3.1415926535897932385D0,
     *		   TWO_PI = TWO*PI,
     *		   CFN = ONE/(TWO*TWO_PI)**2)

      REAL*8	  PT(4),QT(4),PM

  */

	/*
      DATA	  INIT/.TRUE./,NR/0/
      SAVE	  INIT,NR,S2MIN,SMIN

      K = ABS (NF)
	  */

	CENumVector* p_next = (CENumVector*)m_p_in->clone();
	CENumVector* p_prev;
	CEValue* s_next = (CEValue*)m_s->clone();
	CEValue* s_prev;
	CEValue* ss_next = s_next->Absx(true)->Sqrtx();
	CEValue* ss_prev;
	//CEValue* ph_f = (CEValue*)two_pi->clone();
	if (m_ph_f != NULL) delete m_ph_f;
	m_ph_f = (CEValue*)two_pi->clone();

	dbg->log(20, "p_next", p_next);
	dbg->log(20, "s_next", s_next);
	dbg->log(20, "ss_next", ss_next);

/*C     Prepare to start

      DO I=1,4
	 P_NEXT(I)=P_IN(I)
      END DO
      S_NEXT = S_IN
      SS_NEXT = SQRT( ABS (S_NEXT))
      PH_FACTOR = TWO_PI*/

	int k;
	for (k = nr; k>=2; k--) {

		dbg->log(20, nr, "k = %d");

		p_prev = (CENumVector*)p_next;//->clone();
		s_prev = (CEValue*)s_next;//->clone();
		ss_prev = (CEValue*)ss_next;//->clone();

		dbg->log(20, "p_prev", p_prev);
		dbg->log(20, "s_prev", s_prev);
		dbg->log(20, "ss_prev", ss_prev);

/*C     Start main cycle

      DO K=NR,2,-1
	 DO I=1,4
	    P_PREV(I) = P_NEXT(I)
	 END DO
	 S_PREV = S_NEXT
	 SS_PREV = SS_NEXT*/

		CEValue* eta;
		CEValue* lambda = NULL;
		
		if (k>2) {
			CEValue* ss_max = maxx(
				(CEValue*)ss_prev->clone()->minus(kin->mass(k-1)),
				_zero());
			CEValue* s_max = ss_max->Sqrx();
			CEValue* ds = maxx((CEValue*)s_max->clone()->minus(get_smin(k-1)), _zero());
			m_ph_f->multiply(ds->clone());
			CEValue* xi = rand_next();
			eta = (CEValue*)ds->clone()->multiply(
				maxx((CEValue*)one->minus(xi, true), (CEValue*)zero->clone()));
			s_next = minx((CEValue*)get_smin(k-1)->add(ds->multiply(xi)),
				s_max);
			ss_next = s_next->Absx(true)->Sqrtx();
			lambda = (CEValue*)((CEValue*)eta->clone()->multiply(
				kin->mass(k-1)->multiply(4)->multiply(ss_prev->clone())
				->add(eta->clone())
				))->Absx()->Sqrtx()->divide(s_prev->clone());
			delete eta;
		}
		else {
			s_next = kin->mass(0);
			lambda = (CEValue*)
				((CEValue*)maxx((CEValue*)s_prev->clone()->minus(get_smin(2)), _zero())
				->multiply(s_prev->clone()->minus(m_s2min->clone())))->Sqrtx()
				->divide(s_prev->clone());
		}
		m_ph_f = (CEValue*)m_ph_f->multiply(cfn->clone())->multiply(lambda->clone());
		
		dbg->log(20, "m_ph_f", m_ph_f);
		dbg->log(20, "lambda", lambda);
		
		/*
		C     Start main cycle

      DO K=NR,2,-1
         P_PREV = P_NEXT
D        print 3,'P_PREV=',P_PREV            !!! Debug
         S_PREV = S_NEXT
D        print 3,'S_PREV=',S_PREV            !!! Debug
         SS_PREV = SS_NEXT
D        print 3,'SS_PREV=',SS_PREV          !!! Debug
         IF(K.GT.2)THEN
            SS_MAX = MAX(SS_PREV-MASS(K),ZERO)
D           print 3,'SS_MAX=',SS_MAX         !!! Debug
            S_MAX = SS_MAX ** 2
D           print 3,'S_MAX=',S_MAX           !!! Debug
            DS = MAX(S_MAX - SMIN(K-1) , ZERO)
D           print 3,'DS=',DS                 !!! Debug
            PH_FACTOR = PH_FACTOR * DS
D           print 3,'PH_FACTOR=',PH_FACTOR   !!! Debug
            XI = RAN_NEXT()
D           print 3,'XI=',XI                 !!! Debug
            ETA = DS*MAX(ONE-XI,ZERO)
D           print 3,'ETA=',ETA               !!! Debug
            S_NEXT = MIN(SMIN(K-1)+DS*XI, S_MAX)
D           print 3,'S_NEXT=',S_NEXT         !!! Debug
            SS_NEXT = SQRT(S_NEXT)
D           print 3,'SS_NEXT=',SS_NEXT       !!! Debug
            LAMBDA=SQRT(ETA*(4.*MASS(K)*SS_PREV+ETA))/S_PREV
D           print 3,'LAMBDA=',LAMBDA         !!! Debug
         ELSE
            S_NEXT = MASS(1)**2 ! The last particle
D           print 3,'S_NEXT=',S_NEXT         !!! Debug
            LAMBDA=SQRT(MAX(S_PREV-SMIN(2),ZERO)*(S_PREV-S2MIN))/S_PREV
D           print 3,'LAMBDA=',LAMBDA         !!! Debug
         END IF

         PH_FACTOR=CFN * LAMBDA * PH_FACTOR
D        print 3,'PH_FACTOR=',PH_FACTOR      !!! Debug
		*/

		//old
	 /*IF(K.GT.2)THEN
	    SS_MAX = MAX(SS_PREV-MASS(K),ZERO)
	    S_MAX = SS_MAX ** 2
	    DS = S_MAX - SMIN(K-1)
	    XI = RAN_NEXT()
	    ETA = DS*MAX(ONE-XI,ZERO)
	    S_NEXT = MIN(SMIN(K-1)+DS*XI, S_MAX)
	    SS_NEXT = SQRT( ABS (S_NEXT))
	    LAMBDA = SQRT( ABS (ETA*(4D0*MASS(K)*SS_PREV+ETA)))/S_PREV
	    PH_FACTOR=CFN * LAMBDA * PH_FACTOR
	 END IF*/

		eta = rand_next();
		dbg->log(20, "eta", eta);

		CEValue* cost = maxx(
			minx((CEValue*)_two()->multiply(eta->clone())->minus(_one()), _one()),
			_one()->multiply(-1));
		dbg->log(20, "cost", cost);
		CEValue* sint = minx(
			(CEValue*)_two()->multiply(((CEValue*)eta->clone()->multiply(
				maxx((CEValue*)_one()->minus(eta->clone()), _zero())
				))->Absx()->Sqrtx()),
			_one());
		delete eta;
		dbg->log(20, "sint", sint);
		CEValue* fi = (CEValue*)two_pi->clone()->multiply(rand_next());
		dbg->log(20, "fi", fi);

		CEValue* pm;
		pm = (CEValue*)half->clone()->multiply(ss_prev->clone())->multiply(lambda);
		/*if (lambda == NULL) {
			pm = (CEValue*)half->clone()->multiply(ss_prev);
		}
		else {
			pm = (CEValue*)half->clone()->multiply(ss_prev)->multiply(lambda);
		}*/
		dbg->log(20, "pm", pm);


/*	 ETA = RAN_NEXT()
	 COST = MAX(MIN(TWO*ETA-ONE,ONE),-ONE)
	 SINT = MIN(TWO*SQRT( ABS (ETA*MAX(ONE-ETA,ZERO))),ONE)

	 FI = TWO_PI*RAN_NEXT()
	 PM = HALF * SS_PREV * LAMBDA

C ================= Debug ===========================
C	 S_P(K) = S_NEXT
C	 C_T(K) = COST
C	 FI_T(K) = FI
C ================= Debug ===========================
*/

		CENumVector* pt = new CENumVector();
		CENumVector* qt = new CENumVector();
		if (k > 2) {
			pt->addItem(
				((CEValue*)pm->Sqrx(true)->add(s_next->clone()))
					->Absx()->Sqrtx());
		}
		else {
			pt->addItem(
				((CEValue*)pm->Sqrx(true)->add(kin->mass(0)->Sqrx()))
					->Absx()->Sqrtx());
		}
		qt->addItem(
			((CEValue*)pm->Sqrx(true)->add(kin->mass(k-1)->Sqrx()))
				->Absx()->Sqrtx());
		dbg->log(20, "qt", qt);
		dbg->log(20, "pt", pt);
		pt->addItem(pm->clone()->multiply(sint->clone())->multiply(
			CLMath::Cos(fi)
			));
		pt->addItem(pm->clone()->multiply(sint)->multiply(
			CLMath::Sin(fi)
			));
		dbg->log(20, "qt", qt);
		dbg->log(20, "pt", pt);
		delete fi;
		pt->addItem(pm->multiply(cost));

		dbg->log(20, "qt", qt);
		dbg->log(20, "pt", pt);

		qt->addItem(((CEValue*)pt->GetAt(1)->clone())->multiply(-1));
		qt->addItem(((CEValue*)pt->GetAt(2)->clone())->multiply(-1));
		qt->addItem(((CEValue*)pt->GetAt(3)->clone())->multiply(-1));

		dbg->log(20, "qt (res)", qt);
		dbg->log(20, "pt (res)", pt);

	 /*IF(K.GT.2)THEN
	    PT(4) = SQRT( ABS (PM**2 + S_NEXT))
	 ELSE
	    PT(4) = SQRT( ABS (PM**2 + MASS(1)**2))
	 END IF
	 QT(4) = SQRT( ABS(PM**2 + MASS(K)**2))
	 PT(3) = PM*COST
	 PT(2) = PM*SINT*SIN(FI)
	 PT(1) = PM*SINT*COS(FI)
	 QT(3) = -PT(3)
	 QT(2) = -PT(2)
	 QT(1) = -PT(1)*/

		////
		////
		p_prev->set_mass(ss_prev);
		qt->lor0k(p_prev);
		pt->lor0k(p_prev);
		kin->vector(k-1, false)->copy_values_from(qt);
		delete qt;
		p_next = pt;

	 /*CALL LOR0K(QT,Q(1,K),P_PREV,SS_PREV)
	 CALL LOR0K(PT,P_NEXT,P_PREV,SS_PREV)*/
		delete p_prev;
		delete s_prev;
	}
      //END DO
	
	kin->vector(0, false)->copy_values_from(p_next);

/*C     Main cyle finished

      DO I=1,4
	 Q(I,1)=P_NEXT(I)
      END DO*/

	delete p_next;
	delete s_next;

	return true;
/*
      END
	*/
}

/*
* Note: S2MIN is m_s2min;
*/
void CAbstractGenerator::init_smin()
{
	CKinematics* kin = CEProcessor::kinematics();
	if (m_smin != NULL) {
		//m_smin->clear();
		delete m_smin;
	}
	m_smin = new CEPtrArray();
	int nr = kin->num_final_particles();

	CEValue* s2min = kin->vector(1, false)->mass();
	s2min->inverseFlag();
	s2min->add(kin->vector(0, false)->mass());
	m_s2min = s2min->Sqrx();

	s2min = kin->vector(0, false)->mass();
	int i;
	for (i=1; i<nr; i++) {
		s2min = (CEValue*)s2min->add(kin->vector(i, false)->mass());
		m_smin->add(s2min->Sqrx(true));
	}
	delete s2min;

	m_p_in = kin->get_sum_initials();
	m_s = m_p_in->square();

	/*
	 NR = K        ! Initialization begins....
	 S2MIN = MASS(1)
	 DO I=2,NR
	    S2MIN = S2MIN + MASS(I)
	    SMIN(I) = S2MIN**2
	 END DO
	 S2MIN = (MASS(1) - MASS(2))**2
	 INIT = .FALSE.
      END IF
	*/
}

CEValue* CAbstractGenerator::rand_next()
{
	m_rand_prev = rand_next(m_rand_prev);
	return (CEValue*)m_rand_prev->clone();
}

CEValue* CAbstractGenerator::maxx(CEValue * a, CEValue * b, bool do_delete)
{
	if (a->less_than(b)) {
		if (do_delete) {
			delete a;
			return b;
		}
		return (CEValue*)b->clone();
	}
	if (do_delete) {
		delete b;
		return a;
	}
	return (CEValue*)a->clone();
}

CEValue* CAbstractGenerator::minx(CEValue * a, CEValue * b, bool do_delete)
{
	if (a->less_than(b)) {
		if (do_delete) {
			delete b;
			return a;
		}
		return (CEValue*)a->clone();
	}
	if (do_delete) {
		delete a;
		return b;
	}
	return (CEValue*)b->clone();
}

CEValue* CAbstractGenerator::get_smin(int i)
{
	if (i<2) {
		dbg->log(1, "Invalid get_smin request.");
		return NULL;
	}
	return (CEValue*)((CEValue*)m_smin->GetAt(i-2))->clone();
}

CEValue* CAbstractGenerator::_zero()
{
	return (CEValue*)zero->clone();
}

CEValue* CAbstractGenerator::_one()
{
	return (CEValue*)one->clone();
}

CEValue* CAbstractGenerator::_two()
{
	return (CEValue*)two->clone();
}

double CAbstractGenerator::mid(CExpression *cs, long num_events, bool do_delete_cs)
{
	if (cs == NULL) return 0;
	double res = 0;
	init_smin();
	CKinematics* kin = CEProcessor::kinematics();

	bool do_scroll = (num_events > 100);
	if (do_scroll) {
		printf("Generating %d events\n", num_events);
		printf("0%%........................................100%%\n  ");
	}
	long scr_step = num_events/40;
	long scr_curr = 0;
	long i;
	for (i=0; i<num_events; i++) {
		//dbg->log(5, i, "generating event #%d");
		generate_event();
		kin->fill_data_storage();

		double val = 0;

		if (!cut_event(kin)) {
			//CEProcessor::init_values_cache();
			CExpression* val_obj = cs->numeric();
			//CEProcessor::disable_values_cache();
			if (!val_obj->getProperty(CEProperties::PROP_MASK_IS_NUMERIC_VALUE)) {
				dbg->log(1, "can't calculate numeric value", cs);
				dbg->log(1, "result obtained", val_obj);
				delete val_obj;
				if (do_delete_cs) delete cs;
				return 0;
			}
			val = ((CEValue*)val_obj)->to_double();//*m_ph_f->to_double()
			delete val_obj;
		}

		if (do_scroll) {
			if (scr_curr < i) {
				scr_curr += scr_step;
				printf("#");
			}
		}

		res+=val;
	}
	if (do_scroll) {
		printf("\nComplete.\n");
	}
	res /= num_events;
	if (do_delete_cs) delete cs;
	return res;
}

void CAbstractGenerator::fill_hs(CExpression *cs, long num_events, CEHistogram *hs, int bank, bool do_delete_cs)
{
	if ((hs == NULL) || (cs == NULL)) return;
	init_smin();
	CKinematics* kin = CEProcessor::kinematics();

	bool do_scroll = (num_events > 100);
	if (do_scroll) {
		printf("Generating %d events\n", num_events);
		printf("0%%........................................100%%\n  ");
	}
	long scr_step = num_events/40;
	long scr_curr = 0;
	long i;
	for (i=0; i<num_events; i++) {
		//dbg->log(5, i, "generating event #%d");
		generate_event();
		kin->fill_data_storage();

		double val = 0;

		if (!cut_event(kin)) {
			//CEProcessor::init_values_cache();
			CExpression* val_obj = cs->numeric();
			//CEProcessor::disable_values_cache();
			if (!val_obj->getProperty(CEProperties::PROP_MASK_IS_NUMERIC_VALUE)) {
				//delete hs;
				dbg->log(1, "can't calculate numeric value", cs);
				dbg->log(1, "result obtained", val_obj);
				delete val_obj;
				if (do_delete_cs) delete cs;
				return;
			}
			val = ((CEValue*)val_obj)->to_double();//*m_ph_f->to_double()
			delete val_obj;
		}

		if (do_scroll) {
			if (scr_curr < i) {
				scr_curr += scr_step;
				printf("#");
			}
		}

		///*
		//testing
		//log_item(10, new CESymbol("s"));
		//log_item(10, new CESymbol("m"));
		//log_item(10, new CESymbol("t"));
		//log_item(10, new CESymbol("u"));
		//log_item(10, cs, false);
		//*/
		
		double angle = kin->vfin(1)->angle_cos(kin->vin(1));

		hs->add_value(angle, val, bank);
		//hs->add_value(kin->vfin(1)->angle_cos(kin->vin(1)), m_ph_f->to_double(), bank+1);
		//dbg->log(5, "final vector #1", kin->vector(0, false));
		//dbg->log(5, "final vector #2", kin->vector(1, false));
		//dbg->log(5, "final vector #3", kin->vector(2, false));
	}
	if (do_scroll) {
		printf("\nComplete.\n");
	}
	if (do_delete_cs) delete cs;
}

void CAbstractGenerator::log_item(int p, CExpression *item, bool do_delete)
{
	if (item == NULL) return;
	CExpression* temp = data->getvalue(item);
	if (temp != NULL) {
		CEString* s = item->toString();
		dbg->log(p, s->getData(), temp);
		delete s;
	}
	else {
		temp = item->numeric();
		CEString* s = temp->toString();
		dbg->log(p, s->getData(), item);
		delete s;
		delete temp;
	}
	if (do_delete) delete item;
}

bool CAbstractGenerator::cut_event(CKinematics* kin)
{
	double angle = kin->vfin(1)->angle_cos(kin->vin(1));
	if ((angle > 0.8) || (angle < -0.8)) return true;
	return false;
}

void CAbstractGenerator::compare_cs(CEPtrArray *p_cs, int num_evnts)
{
	if (p_cs == NULL) return;
	long s = p_cs->size();
	if (s == 0) return;
	if (num_evnts <= 0) return;

	printf("comparison\n");
	CKinematics* kin = CEProcessor::kinematics();
	if (kin == NULL) {printf("Error: no kinematic installed\n");return;}

	kin->set_CMS_energy(1);
	init_smin();
	long i;
	for (i = 0; i<num_evnts; i++) {
		printf("event %ld\n", i);

		if (i==0) load_event();
		else generate_event();
		//printf(".");
		kin->fill_data_storage();
		//printf(".");
		log_event();
		//CEProcessor::disable_values_cache();

		long j;
		for (j=0; j<s; j++) {
			CExpression* cs = (CExpression*)p_cs->GetAt(j);
			CExpression* cs_val_obj = cs->numeric();
			if (cs_val_obj->getProperty(CEProperties::PROP_MASK_IS_NUMERIC_VALUE)) {
				double cs_val = ((CEValue*)cs_val_obj)->to_double();
				//printf("val %ld: %.2e\n", j, cs_val);
				printf("val %ld: %f\n", j, cs_val);
				if (((CEValue*)cs_val_obj)->get_val_type() & CEValue::VTYPE_COMPLEX) {
					CEComplexValue* cv = (CEComplexValue*)cs_val_obj;
					char* l = cv->toLine();
					printf("cmplx: %s\n", l);
					CMemoryManager::mfree(l);
				}
			}
			else {
				dbg->log(1, j, "can't calculate numeric value fo cs # %ld");
				dbg->log(1, "result obtained", cs_val_obj);
			}
			delete cs_val_obj;
		}
		printf("\n");
	}
	printf("comparison complete.\n");
}

void CAbstractGenerator::log_event()
{
	CKinematics* kin = CEProcessor::kinematics();
	if (kin == NULL) return;
	dbg->log(10, "event log");
	printf("event log\n");
	int i_nr = kin->num_init_particles();
	int i;
	for (i=0; i<i_nr; i++) {
		CENumVector* vec = kin->vector(i, true);
		if (vec != NULL) {
			dbg->log(10, vec);
			char* line = vec->toLine();
			printf("%s\n", line);
			CMemoryManager::mfree(line);
		}
	}
	int f_nr = kin->num_final_particles();
	for (i=0; i<f_nr; i++) {
		CENumVector* vec = kin->vector(i, false);
		if (vec != NULL) {
			dbg->log(10, vec);
			char* line = vec->toLine();
			printf("%s\n", line);
			CMemoryManager::mfree(line);
		}
	}
}

void CAbstractGenerator::load_event()
{
	//generate_event();
	CKinematics* kin = CEProcessor::kinematics();
	if (kin == NULL) return;

	CENumVector* f;
	/*f = new CENumVector();
	f->addItem(new CERealValue(50.0000000000000));
	f->addItem(new CERealValue(38.0670848304134));
	f->addItem(new CERealValue(8.94758342532275));
	f->addItem(new CERealValue(-31.1582702241475));
	kin->vector(0, false)->copy_values_from(f);

	f = new CENumVector();
	f->addItem(new CERealValue(50.0000000000000));
	f->addItem(new CERealValue(-38.0670848304134));
	f->addItem(new CERealValue(-8.94758342532275));
	f->addItem(new CERealValue(31.1582702241475));
	kin->vector(1, false)->copy_values_from(f);*/

	//-0.297439701844507     0.294096906269155       8.762314456274076E-002  0.427365396322047     
	f = new CENumVector();
	f->addItem(new CERealValue(0.427365396322047));
	f->addItem(new CERealValue(0.08762314456274076));
	f->addItem(new CERealValue(-0.297439701844507));
	f->addItem(new CERealValue(0.294096906269155));
	kin->vector(0, false)->copy_values_from(f);

	//-2.767128668160793E-002 -9.421856916844716E-002  0.106162442944668    0.144614325620752
	f = new CENumVector();
	f->addItem(new CERealValue(0.144614325620752));
	f->addItem(new CERealValue(0.106162442944668));
	f->addItem(new CERealValue(-0.02767128668160793));
	f->addItem(new CERealValue(-0.09421856916844716));
	kin->vector(1, false)->copy_values_from(f);
	
	//0.325110988526115      -0.199878337100708    -0.193785587507409       0.428020278057201
	f = new CENumVector();
	f->addItem(new CERealValue(0.428020278057201));
	f->addItem(new CERealValue(-0.193785587507409));
	f->addItem(new CERealValue(0.325110988526115));
	f->addItem(new CERealValue(-0.199878337100708));
	kin->vector(2, false)->copy_values_from(f);
}
