////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// DiracAlgebra.cpp: implementation of the CDiracAlgebra class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDebug* CDiracAlgebra::dbg = new CDebug("CDiracAlgebra");

CDiracAlgebra::CDiracAlgebra():CELibrary()
{

}

CDiracAlgebra::~CDiracAlgebra()
{

}

/*
* Not implemented yet.
*/
CExpression* CDiracAlgebra::Sp(CExpression *seq)
{
	CEAbstractFunction* f_sp = new CFSp(seq);
	CExpression* res = f_sp->evaluate();
	delete f_sp;
	return res;
}

/*
* Not implemented yet.
*/
CExpression* CDiracAlgebra::simplifyGammaSequence(CExpression* seq, CExpression *left, CExpression *right)
{
	CExpression* res = doSimplifyGS(seq, left, right);
	//CESupport::Save("doSimplifyGS ==>>");
	//CESupport::SaveResults(res);
	CDiracAlgebra::dbg->log(10, "doSimplifyGS ==>>", res);
	res = CExpression::reduce(res);
	CDiracAlgebra::dbg->log(10, "Result:", res);
	return res;
}

bool CDiracAlgebra::replacePairs(CEProduct *source)
{
	bool res = false;
	long fmp = source->getFirstMatrixPos();
	if ((fmp >= 0) && (source->size()>(1+fmp))) {
		CExpression* expr_a = source->GetAt(fmp);
		CExpression* expr_b = NULL;
		long i=fmp+1;
		while (i<source->size()) {
			expr_b = source->GetAt(i);
			if (expr_a->equals(expr_b)) {
				if (CEUtil::equals(expr_a->ClassName(), "CESGammaProduct")) {
					CExpression* base = ((CESGammaProduct*)expr_a)->getBaseExpression();
					CExpression* sqr = CEVector::square(base);
					source->SetAt(i-1, sqr);
					delete sqr;
					source->RemoveAt(i);
					res = true;
					i++;
				}
				else {
					log("<Warning><All non-tensor items should be previously removed.>");
					i+=2;
				}
				if (i<source->size()) expr_a = source->GetAt(i-1);
			}
			else {
				i++;
				expr_a = expr_b;
			}
		}
	}
	return res;
}

/*
* To add skipping commutative items, when required.
*/
bool CDiracAlgebra::checkupBounds(CEProduct *seq, CExpression *left, CExpression *right)
{
	bool res = false;
	long size = seq->size();
	long fmp = seq->getFirstMatrixPos();
	if (fmp >= 0) {
		CExpression* bound = NULL;
		CExpression* temp = seq->GetAt(fmp);
		if ((left != NULL) && (CEUtil::equals(temp->ClassName(), "CESGammaProduct"))) {
			bound = left->clone();
			bool sign = bound->getFlag();
			bound->setFlag();
			if(((CESGammaProduct*)temp)->getBaseExpression()->equals(bound)) {
				//log("Left side coinsides");
				CExpression* mass = CEVector::module(bound);
				if (mass != NULL) {
					CExpression* mass_temp = mass->clone();
					if (!sign) mass_temp->inverseFlag();
					seq->SetAt(fmp, mass_temp);
					delete mass_temp;
					res = true;
				}
			}
			delete bound;
		}
		temp = seq->GetAt(size-1);
		if ((right != NULL) && (CEUtil::equals(temp->ClassName(), "CESGammaProduct"))) {
			bound = right->clone();
			bool sign = bound->getFlag();
			bound->setFlag();
			if(((CESGammaProduct*)temp)->getBaseExpression()->equals(bound)) {
				//log("Right side coinsides");
				CExpression* mass = CEVector::module(bound);
				if (mass != NULL) {
					CExpression* mass_temp = mass->clone();
					if (!sign) mass_temp->inverseFlag();
					seq->SetAt(size-1, mass_temp);
					delete mass_temp;
					res = true;
				}
			}
			delete bound;
		}
	}
	return res;
}

/*
* Not used.
* No equal items in *seq are presumed.
*/
bool CDiracAlgebra::expandTriples(CEProduct *source, CExpression *left, CExpression *right)
{
	//bool res = false;
	long size = source->size();
	long fmp = source->getFirstMatrixPos();
	if (fmp>0) {//old: if(size>1)
		CExpression* expr_a = left;
		CExpression* expr_b = source->GetAt(fmp);
		CExpression* expr_c = NULL;
		long i=fmp;
		while (i<size) {
			expr_c = ((size>(i+1))?source->GetAt(i+1):right);
			if (CEUtil::equals(expr_b->ClassName(), "CESGammaProduct")) {
				CExpression* base_b = ((CESGammaProduct*)expr_b)->getBaseExpression();
				if (expr_a->equals(expr_c)) {
					return excangePair(source, i-1);
				}
				else {
					if ((CEUtil::equals(expr_a->ClassName(), "CESGammaProduct")) &
						(CEUtil::equals(expr_c->ClassName(), "CESGammaProduct"))) {
						/*CExpression* subst_base = CEProcessor::kinematics()->getEqual(base_b);
						if (subst_base != NULL) {
							//CESGammaProduct* subst = new CESGammaProduct(subst_base);
							source->SetAt(i, new CESGammaProduct(subst_base));
							return true;
						}*/
					}
					else {
						if (CEUtil::equals(expr_c->ClassName(), "CESGammaProduct")) {
							if (expr_a != NULL) {
								CExpression* base_c = ((CESGammaProduct*)expr_c)->getBaseExpression();
								if (!expr_a->equals(base_c)) {
									if ((!base_b->equals(right)) && (!base_c->equals(right)) & (right != NULL)) {
										CExpression* subst_base = CEProcessor::kinematics()->getEqual(base_b);
										if (subst_base != NULL) {
											source->SetAt(i, new CESGammaProduct(subst_base));
											return true;
											//return false;
										}
									}
									else if (base_b->equals(right)) {
										return excangePair(source, i);
									}
								} else {
									return excangePair(source, i);
								}
							}
						}
						else if (CEUtil::equals(expr_a->ClassName(), "CESGammaProduct")) {
							if (expr_c != NULL) {
								CExpression* base_a = ((CESGammaProduct*)expr_a)->getBaseExpression();
								if (!expr_c->equals(base_a)) {
									if ((!base_b->equals(left)) && (!base_a->equals(left)) & (left != NULL)) {
										CExpression* subst_base = CEProcessor::kinematics()->getEqual(base_b);
										if (subst_base != NULL) {
											source->SetAt(i, new CESGammaProduct(subst_base));
											return true;
											//return false;
										}
									}
									else if (base_b->equals(left)) {
										return excangePair(source, i-1);
									}
								} else {
									return excangePair(source, i-1);
								}
							}
						}
					}
				}
			}
			i++;
			expr_a = expr_b;
			expr_b = expr_c;
		}
	}
	return false;
}

bool CDiracAlgebra::excangePair(CEProduct* source, long first_pos)
{
	dbg->log(35, "excangePair");
	long size = source->size();
	if ((first_pos < source->getFirstMatrixPos()) || ((first_pos+1) >= size)) {
		dbg->log(0, "excangePair : invalid position", source);
		return false;
	}
	CExpression* expr_a = source->GetAt(first_pos);
	CExpression* expr_b = source->GetAt(first_pos+1);
	CExpression* subst = getPairSubst(expr_a, expr_b);
	if (subst != NULL) {
		source->SetAt(first_pos, subst);
		delete subst;
		source->RemoveAt(first_pos+1);
		return true;
	}
	return false;
}

CExpression* CDiracAlgebra::doSimplifyGS(CExpression *source, CExpression *left, CExpression *right)
{
	if (source == NULL) {
		log("<Error><CDiracAlgebra::doSimplifyGS : NULL input data.>");
		return NULL;
	}
	dbg->log(10, "doSimplifyGS()", source);
	source = CExpression::reduce(source->clone());
	dbg->log(22, "doSimplifyGS(): after redusing", source);
	CEProperties* props = source->getProperties();
	//dbg->log(29, "properties gathered");
	if (props->isSum()) {
		CESum* _sum = (CESum*)source;
		long _size = _sum->size();
		long i;
		for (i=0; i<_size; i++) {
			CExpression* sub_res = doSimplifyGS(_sum->GetAt(i), left, right);
			_sum->SetAt(i, sub_res);
			delete sub_res;
		}
		//delete props;
		//return _sum;
	}
	else if (props->isProduct()) {
		CEProduct* seq = (CEProduct*)source;
		if (expandPairs(seq, left, right)) {
			source = doSimplifyGS(seq, left, right);
			delete seq;
		}
		//if (expandTriples(seq, left, right))
		//	return doSimplifyGS(seq, left, right);
		//delete props;
		//return seq;
	}
	delete props;
	dbg->log(20, "doSimplifyGS(): result", source);
	return source;
}

bool CDiracAlgebra::expandPairs(CEProduct *source, CExpression *left, CExpression *right)
{
	dbg->log(15, "expandPairs:", source);
	if (replacePairs(source)) return true;
	dbg->log(29, "expandPairs: no pairs replased");
	if (checkupBounds(source, left, right)) return true;
	dbg->log(29, "expandPairs: no bounds simplified");

	long fmp = source->getFirstMatrixPos();
	if ((fmp >= 0) && (source->size()>(1+fmp))) {
		//moving left.
		long s = source->size();
		long i;
		for (i = fmp+1; i<s; i++) {
			if (is_to_move_left(i, source, left)) {
				if (excangePair(source, i-1)) return true;
				//stop moving left.
				i = s;
			}
		}
		//moving right.
		for (i = s-2; i>=fmp; i--) {
			if (is_to_move_right(i, source, right)) {
				if (excangePair(source, i)) return true;
				//stop moving right.
				i = fmp;
			}
		}
		dbg->log(29, "expandPairs: no items moved");
		//replasing kinematically-dependent vectors
		//if there is nothing to move.
		for (i = fmp; i<s; i++) {
			CExpression* expr = source->GetAt(i);
			if (CEUtil::equals(expr->ClassName(), "CESGammaProduct")) {
				CExpression* base = ((CESGammaProduct*)expr)->getBaseExpression();
				if (!CEProcessor::kinematics()->isPrimary(base)) {
					CExpression* subst_base = CEProcessor::kinematics()->getEqual(base);
					if (subst_base != NULL) {
						CExpression* subst = new CESGammaProduct(subst_base);
						delete subst_base;
						source->SetAt(i, subst);
						delete subst;
						return true;
					}
				}
			}
		}
	}
	return false;
}

/*
* ???
* Gamma-5 object suspected at expr_b only
* They shold be moved to the last position of sequence.
*/
CExpression* CDiracAlgebra::getPairSubst(CExpression *expr_a, CExpression *expr_b)
{
	if ((expr_a->getProperty(CEProperties::PROP_MASK_IS_TENSOR)) &&
		(expr_b->getProperty(CEProperties::PROP_MASK_IS_TENSOR))) {
			return (IETensor::GetIETensor(expr_a))->exchange_subst(IETensor::GetIETensor(expr_b),true);
	}
	/*
	*	Old implementation:
	*	if (expr_a->getProperty(CEProperties::PROP_MASK_IS_TENSOR))
	*		return (IETensor::GetIETensor(expr_a))->exchange_subst(expr_b, true);
	*	else if (expr_b->getProperty(CEProperties::PROP_MASK_IS_TENSOR))
	*		return (IETensor::GetIETensor(expr_b))->exchange_subst(expr_a, false);
	*/
	return NULL;
}

bool CDiracAlgebra::expandGammaProducts(CEProduct *data)
{
	long fmp = data->getFirstMatrixPos();
	if (fmp < 0) return false;
	bool res = false;
	long i;
	for (i=fmp; i<data->size(); i++) {
		CExpression* expr = data->GetAt(i);
		if (CEUtil::equals(expr->ClassName(), "CESGammaProduct")) {
			CExpression* base = ((CESGammaProduct*)expr)->getBaseExpression();
			CExpression* index = CESymbol::getUnique();
			CExpression* temp = new CEVector(base, index, false);
			data->InsertAt(i, temp);
			delete temp;
			i++;
			temp = new CEGamma(index, true);
			data->SetAt(i, temp);
			delete temp;
			delete index;
			//delete ...index, vector, gamma;
			res = true;
		}
	}
	return res;
}

/*
* Such an arranging is also included in simplify-gamma-sequence mechanism.
*/
bool CDiracAlgebra::arrangeGammaSpecial(CEProduct *seq)
{
	log("CDiracAlgebra::arrangeGammaSpecial()...");
	log(seq);
	bool res = false;
	long fmp = seq->getFirstMatrixPos();
	if (fmp < 0) return false;
	long _l = seq->size()-1;
	long i;
	for (i=fmp; i<_l; i++) {
		CExpression* item = seq->GetAt(i);
		CExpression* next = seq->GetAt(i+1);
		if (!item->getProperty(CEProperties::PROP_MASK_IS_TENSOR)) { //Warning: non-commutative object is NOT a tensor
			dbg->log(2,"Warning: non-commutative object is NOT a tensor",next);
			continue;
		}
		if (CEUtil::equals(CEGammaSpecial::CLASS_NAME, item->ClassName())) {
			CEGammaSpecial* g = (CEGammaSpecial*)item;
			item = g->exchange_subst(IETensor::GetIETensor(next), true);
			if (item != NULL) {
				if (item->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
					CEProduct* p = (CEProduct*)item;
					long s = p->size();
					seq->RemoveAt(i);
					seq->RemoveAt(i);
					long j;
					for (j=s-1; j>=0; j--) {
						seq->InsertAt(i, p->GetAt(j));
					}
					i = _l;
					res = true;
				}
				else {
					seq->SetAt(i, item);
					seq->RemoveAt(i+1);
					i = _l;
					res = true;
				}
				//delete item;
			}
		}
	}
	if (res) {
		seq->Arrange(AL_INLINE);
		arrangeGammaSpecial(seq);
	}
	return res;
}

bool CDiracAlgebra::is_to_move_left(long i, CEProduct *source, CExpression *border)
{
	dbg->log(35, "is_to_move_left");
	//log("CDiracAlgebra::is_to_move_left()");
	//log(source);
	//log(border);
	CExpression* expr = source->GetAt(i);
	if ((border != NULL) && (CEUtil::equals(expr->ClassName(), CESGammaProduct::CLASS_NAME))) {
		CExpression* base = ((CESGammaProduct*)expr)->getBaseExpression();
		CExpression* bound_value = border->clone();
		bound_value->setFlag();
		if (base->equals(bound_value)) {
			delete bound_value;
			return true;
		}
		delete bound_value;
	}
	long f = source->getFirstMatrixPos();
	long j;
	for (j=i-1; j>=f; j--) {
		CExpression* other = source->GetAt(j);
		if (expr->equals(other)) return true;
		if ((expr->getProperty(CEProperties::PROP_MASK_IS_TENSOR))
			| (other->getProperty(CEProperties::PROP_MASK_IS_TENSOR))) {
			CExpression* temp = (IETensor::GetIETensor(expr))->unite_with(IETensor::GetIETensor(other), false);
			if (temp != NULL) {
				delete temp;
				return true;
			}
		}
	}
	return false;
}

bool CDiracAlgebra::is_to_move_right(long i, CEProduct *source, CExpression *border)
{
	dbg->log(35, "is_to_move_right");
	CExpression* expr = source->GetAt(i);
	if (CEUtil::equals(expr->ClassName(), CEGammaSpecial::CLASS_NAME)) return true;
	if ((border != NULL) && (CEUtil::equals(expr->ClassName(), CESGammaProduct::CLASS_NAME))) {
		CExpression* base = ((CESGammaProduct*)expr)->getBaseExpression();
		CExpression* bound_value = border->clone();
		bound_value->setFlag();
		if (base->equals(bound_value)) {
			delete bound_value;
			return true;
		}
		delete bound_value;
	}
	long s = source->size();
	long j;
	for (j=i+1; j<s; j++) {
		CExpression* other = source->GetAt(j);
		if (expr->equals(other)) return true;
		if ((expr->getProperty(CEProperties::PROP_MASK_IS_TENSOR))
			| (other->getProperty(CEProperties::PROP_MASK_IS_TENSOR))) {
			CExpression* temp = (IETensor::GetIETensor(expr))->unite_with(IETensor::GetIETensor(other), true);
			if (temp != NULL) {
				delete temp;
				return true;
			}
		}
	}
	return false;
}

CExpression* CDiracAlgebra::get_3Gamma_Gamma5_subst(CExpression *ind1, bool cov1, CExpression *ind2, bool cov2, CExpression *ind3, bool cov3)
{
	CESum* res = new CESum();
	CEProduct* prod = new CEProduct();
	prod->addItem(data->gete(CConst::DV_IMAGINARY)->clone());
	CESymbol* u = CESymbol::getUnique();
	prod->addItem(new CEGamma(u, false));
	prod->addItem(new CEAntisymmetric(u, true,
		ind1, cov1,
		ind2, cov2,
		ind3, cov3));
	res->addItem(prod->clone());
	delete prod;
	delete u;
	prod = new CEProduct();
	prod->addItem(new CEGamma(ind1, cov1));
	prod->addItem(new CEMetric(ind2, ind3, cov2, cov3));
	prod->addItem(data->gete(CConst::DV_GAMMA5)->clone());
	res->addItem(prod->clone());
	delete prod;
	prod = new CEProduct();
	prod->addItem(new CEGamma(ind2, cov2));
	prod->addItem(new CEMetric(ind1, ind3, cov1, cov3));
	prod->addItem(data->gete(CConst::DV_GAMMA5)->clone());
	prod->unsetFlag();
	res->addItem(prod->clone());
	delete prod;
	prod = new CEProduct();
	prod->addItem(new CEGamma(ind3, cov3));
	prod->addItem(new CEMetric(ind1, ind2, cov1, cov2));
	prod->addItem(data->gete(CConst::DV_GAMMA5)->clone());
	res->addItem(prod->clone());
	delete prod;
	//CESupport::Save("CDiracAlgebra::get_3Gamma_Gamma5_subst() result:");
	//CESupport::SaveResults(res);
	return res;
}

CExpression* CDiracAlgebra::gammaPM(bool sign)
{
	CEProduct* res = new CEProduct();
	res->addItem(data->gete(CConst::DV_HALF)->clone());
	CESum* s = new CESum();
	s->addItem(data->gete(CConst::DV_UNIT)->clone());
	CExpression* t = data->gete(CConst::DV_GAMMA5)->clone();
	t->putFlag(sign);
	s->addItem(t);
	res->addItem(s);
	return res;
}

CExpression* CDiracAlgebra::simplify(CExpression *source)
{
	CExpression* res = simplifyGammaSequence(source, NULL, NULL);
	delete source;
	return res;
}
