////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// FeynLib.cpp: implementation of the CFeynLib class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#include <math.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDebug* CFeynLib::dbg = new CDebug("CFeynLib");

CFeynLib::CFeynLib()
{

}

CFeynLib::~CFeynLib()
{

}

/*
* We should probably first clone base and truncate it's sign.
*/
CExpression* CFeynLib::ferm_propagator(CExpression *base, CExpression* m)
{
	//CExpression* b = base->clone();
	//b->setFlag();
	/*CExpression* m = CEVector::module(b);
	if (m == NULL) {
		log("<Error><CFeynLib::ferm_propagator() : Can't find vector's module>");
		log(base);
		delete b;
		return NULL;
	}*/
	CEProduct* res = new CEProduct();
	CESum* s = new CESum();
	s->addItem(new CESGammaProduct(base));
	s->addItem(m->clone());
	res->addItem(s);
	s = new CESum();
	s->addItem(new CESProduct(base, base));
	CExpression* t = new CEPower(m, (long)2);
	t->unsetFlag();
	s->addItem(t);
	s->unsetFlag(LEVEL_FLAG);
	res->addItem(s);
	return res;
}

CExpression* CFeynLib::test()
{
	//dbg->watch_periods(true);
	dbg->logperiod(5);
	//CESupport::Save("CFeynLib::test() started...");
	dbg->log(10, "test started.");

	CExpression* m = data->getv("p\\_11", CConst::DNV_MODULE);
	CExpression* p1 = data->getv("p\\_1");
	CExpression* p2 = data->getv("p\\_2");
	CExpression* p11 = data->getv("p\\_11");
	CExpression* p21 = data->getv("p\\_21");
	CExpression* pm21 = p21->clone();
	pm21->unsetFlag();
	CExpression* temp = NULL;
	CExpression* ind1 = CESymbol::getUnique();
	CExpression* ind2 = CESymbol::getUnique();
	CExpression* ind3 = CESymbol::getUnique();
	CExpression* ind4 = CESymbol::getUnique();

	dbg->log(30, "momenta initialized.");

	CExpression* work = compose_matrix_element(ind1, ind2);
	dbg->log(10, "Matrix element composed:", work);
	work = CExpression::reduce(work);
	dbg->log(15, "Reduced:", work);

	//long old_s_level = CESum::dbg->set_level(40);
	CExpression* work2 = CDiracAlgebra::simplifyGammaSequence(work, p11, pm21);
	//CESum::dbg->set_level(old_s_level);
	delete work;
	work = work2;
	dbg->log(15, "Simplified:", work);
	temp = CEBasic::Congugate(work->clone(),true);
	dbg->log(15, "Conjugated:", temp);
	
	CEReplList* rule = new CEReplList(ind1, ind3, false);
	rule->add(ind2, ind4, false);
	temp->applyRule(rule);
	delete rule;

	CEProduct* seq = new CEProduct();
	////////// Unpolarized case
	seq->addItem(new CEMetric(ind1, ind3, false, false));
	seq->addItem(new CEMetric(ind2, ind4, false, false));
	////////// Polarized case
	//seq->addItem(phot_helisity(p1, p11, p21, true, ind1, false));
	//seq->addItem(phot_helisity(p1, p11, p21, true, ind3, false));
	//seq->addItem(phot_helisity(p2, p11, p21, true, ind2, false));
	//seq->addItem(phot_helisity(p2, p11, p21, true, ind4, false));

	seq->addItem(work);
	seq->addItem(ferm_matrix(pm21));
	seq->addItem(temp);
	seq->addItem(ferm_matrix(p11));
	dbg->log(10, "Total expression:", seq);
	dbg->logperiod(5);
	//CESupport::SaveResults(seq->expand());

	//temp = sub_seq->expand();
	//CESupport::SaveResults(temp);
	//temp->Arrange();
	//CESupport::SaveResults(temp);

	//work = CExpression::reduce(seq);
	//dbg->log(15, "Simplified (1):", work);
	
	temp = CDiracAlgebra::simplifyGammaSequence(seq, NULL, NULL);
	delete seq;
	dbg->log(15, "Simplified:", temp);
	dbg->logperiod(5);
	//if (temp != seq) delete seq;
	//CESupport::SaveResults(temp);
	
	temp = CFSp::Spur(temp);
	dbg->log(10, "Trace calculated:", temp);
	dbg->logperiod(5);

	//temp = temp->multiply(phot_helisity(p1, p11, p21, true, ind1, false));
	//temp = temp->multiply(phot_helisity(p1, p11, p21, true, ind3, false));
	//temp = temp->multiply(phot_helisity(p2, p11, p21, true, ind2, false));
	//temp = temp->multiply(phot_helisity(p2, p11, p21, true, ind4, false));

	//temp = CExpression::reduce(temp);
	//CESupport::Save("Photon helisities substituted:");
	//CESupport::SaveResults(temp);

	work = temp;
	temp = new CEPower(m ,(long)2);
	temp->unsetFlag();
	temp = temp->add(new CESProduct(p1, p2));
	CExpression* pxpx = new CESProduct(p11, p21);
	rule = new CEReplList(pxpx, temp);
	delete pxpx;
	delete temp;
	pxpx = new CESProduct(p2, p21);
	temp = new CESProduct(p1, p11);
	rule->add(pxpx, temp);
	delete pxpx;
	delete temp;
	pxpx = new CESProduct(p1, p21);
	temp = new CESProduct(p11, p2);
	rule->add(pxpx, temp);
	delete pxpx;
	delete temp;

	work->applyRule(rule);
	delete rule;
	work = CExpression::reduce(work);
	dbg->log(15, "Kinematic used", work);

	temp = new CESymbol("s");
	CExpression* t = new CESymbol("t");
	t->unsetFlag();
	CExpression* u = new CESymbol("u");
	u->unsetFlag();
	pxpx = new CESProduct(p1, p2);
	CExpression* pxx2 = temp->multiply(data->gete(CConst::DV_HALF)->clone());
	rule = new CEReplList(pxpx, pxx2);
	delete pxpx;
	delete pxx2;
	pxpx = new CESProduct(p1, p11);
	pxx2 = t->add(new CEPower(m ,(long)2))->multiply(data->gete(CConst::DV_HALF)->clone());
	rule->add(pxpx, pxx2);
	delete pxpx;
	delete pxx2;
	pxpx = new CESProduct(p11, p2);
	pxx2 = u->add(new CEPower(m ,(long)2))->multiply(data->gete(CConst::DV_HALF)->clone());
	rule->add(pxpx, pxx2);
	delete pxpx;
	delete pxx2;
	
	CESum* s = new CESum();
	CExpression* temp2 = p1->clone();
	temp2->unsetFlag();
	s->addItem(temp2);
	s->addItem(p11->clone());
	pxpx = new CESProduct(s, s);
	delete s;
	pxx2 = new CESymbol("t");
	rule->add(pxpx, pxx2);
	delete pxpx;
	delete pxx2;

	s = new CESum();
	temp2 = p2->clone();
	temp2->unsetFlag();
	s->addItem(temp2);
	s->addItem(p11->clone());
	pxpx = new CESProduct(s, s);
	delete s;
	pxx2 = new CESymbol("u");
	rule->add(pxpx, pxx2);
	delete pxpx;
	delete pxx2;

	dbg->log(20, "Substitute rules composed.");
	
	work->applyRule(rule);
	delete rule;
	dbg->log(15, "Invariants substituted (#1) :", work);

	s = new CESum();
	s->addItem((CENumber::GetTwo())->multiply(new CEPower(m ,(long)2)));
	t = new CESymbol("t");
	t->unsetFlag();
	u = new CESymbol("u");
	u->unsetFlag();
	s->addItem(t);
	s->addItem(u);
	pxpx = new CESymbol("s");
	rule = new CEReplList(pxpx, s);
	delete pxpx;
	delete s;
	work->applyRule(rule);
	delete rule;
	dbg->log(15, "Invariants substituted (#2) :", work);

	//long old_p_level = CEProduct::dbg->set_level(40);
	//long old_s_level = CESum::dbg->set_level(40);

	work = CExpression::reduce(work);

	//CEProduct::dbg->set_level(old_p_level);
	//CESum::dbg->set_level(old_s_level);

	dbg->log(15, "Reduced:", work);
	
	/*work = work->multiply(phot_helisity(p2, p11, p21, true, ind2, false));
	work = work->multiply(phot_helisity(p2, p11, p21, true, ind4, false));
	dbg->log(25, "work", work);
	work = work->expand()->Arrange();
	work = work->arrangeTensors();
	dbg->log(25, "work", work);*/

	//CESupport::Save("...CFeynLib::test() finished.");
	//CESupport::Save("...CFeynLib::test() result:");
	//CESupport::SaveResults(work);
	//delete work;
	
	delete pm21;
	delete ind1;
	delete ind2;
	delete ind3;
	delete ind4;
	return work;
}

CExpression* CFeynLib::phot_helisity_gamma(CExpression *base, CExpression *v1, CExpression *v2, bool sign)
{
	CEProduct* res = new CEProduct();
	res->addItem(CENumber::GetTwo());
	res->addItem(new CESymbol("N\\_x"));
	CESum* s = new CESum();
	CEProduct* p = new CEProduct();
	p->addItem(new CESGammaProduct(v1));
	p->addItem(new CESGammaProduct(v2));
	p->addItem(new CESGammaProduct(base));
	p->addItem(CDiracAlgebra::gammaPM(sign));
	s->addItem(p);
	p = new CEProduct();
	p->addItem(new CESGammaProduct(base));
	p->addItem(new CESGammaProduct(v1));
	p->addItem(new CESGammaProduct(v2));
	p->addItem(CDiracAlgebra::gammaPM(!sign));
	p->unsetFlag();
	s->addItem(p);
	res->addItem(s);
	return res;
}

CExpression* CFeynLib::ferm_matrix(CExpression *base, int helisity)
{
	CExpression* b = base->clone();
	bool sign = b->getFlag();
	b->setFlag();
	CExpression* m = CEVector::module(b);
	if (m == NULL) {
		log("<Error><CFeynLib::ferm_matrix() : Can't find vector's module>");
		log(base);
		delete b;
		return NULL;
	}
	CEProduct* res = new CEProduct();
	if (helisity != 0) res->addItem(CDiracAlgebra::gammaPM(helisity>0));
	else res->addItem(data->gete(CConst::DV_HALF)->clone());
	CESum* s = new CESum();
	s->addItem(new CESGammaProduct(b));
	delete b;
	CExpression* m_local = m->clone();
	if (!sign) m_local->inverseFlag();
	s->addItem(m_local);
	res->addItem(s);
	return res;
}

/*
* V1 is P-
* V2 is P+
*/
CExpression* CFeynLib::phot_helisity(CExpression *base, CExpression *v1, CExpression *v2, bool sign, CExpression *ind, bool cov)
{
	//CExpression* factor = new CESymbol("N\\_x");
	CExpression* ph_L = (new CESProduct(base, v2))->multiply(new CEVector(v1, ind, cov))->
		minus((new CESProduct(base, v1))->multiply(new CEVector(v2, ind, cov)));
	CExpression* i1 = CESymbol::getUnique();
	CExpression* i2 = CESymbol::getUnique();
	CExpression* i3 = CESymbol::getUnique();
	CExpression* ph_T = (new CEAntisymmetric(ind, cov, i1, true, i2, true, i3, true))->
		multiply(new CEVector(v2, i1, false))->
		multiply(new CEVector(v1, i2, false))->
		multiply(new CEVector(base, i3, false));
	ph_T = ph_T->multiply(data->gete(CConst::DV_IMAGINARY)->clone());
	delete i1;
	delete i2;
	delete i3;
	if (!sign) ph_T->inverseFlag();
	CExpression* res = ph_L->add(ph_T);//->multiply(factor);
	dbg->log(20, "phot_helisity() composed: ", res);
	return res;
}

CExpression* CFeynLib::compose_matrix_element(CExpression* inda, CExpression* indb)
{
	CExpression* m = data->getv("p\\_11", CConst::DNV_MODULE);
	CExpression* p1 = data->getv("p\\_1");
	CExpression* p2 = data->getv("p\\_2");
	CExpression* p11 = data->getv("p\\_11");
	CExpression* p21 = data->getv("p\\_21");
	CExpression* temp = NULL;

	CEProduct* sub_seq = new CEProduct();
	//sub_seq->addItem(phot_helisity_gamma(p1, p11, p21, true));
	sub_seq->addItem(new CEGamma(inda, true));
	CESum* s = new CESum();
	s->addItem(p11->clone());
	temp = p1->clone();
	temp->unsetFlag();
	s->addItem(temp);
	sub_seq->addItem(ferm_propagator(s, m));
	delete s;
	//sub_seq->addItem(phot_helisity_gamma(p2, p11, p21, true));
	sub_seq->addItem(new CEGamma(indb, true));

	CEReplList* rule = new CEReplList(p1, p2, true);
	//rule->add(ind1, ind2, true);
	temp = sub_seq->clone();
	temp->applyRule(rule);
	delete rule;
	rule = new CEReplList(inda, indb, true);
	temp->applyRule(rule);
	delete rule;
	
	return sub_seq->add(temp);
	//return sub_seq;
}

CExpression* CFeynLib::test2()
{
	dbg->logperiod(5);
	CExpression* ind1a = CESymbol::getUnique();
	CExpression* ind1b = CESymbol::getUnique();
	CExpression* ind2a = CESymbol::getUnique();
	CExpression* ind2b = CESymbol::getUnique();
	CExpression* ind3a = CESymbol::getUnique();
	CExpression* ind3b = CESymbol::getUnique();

	CExpression* p11 = data->getv("p\\_11");
	CExpression* p21 = data->getv("p\\_21");
	CExpression* pm21 = p21->clone();
	pm21->unsetFlag();

	dbg->logperiod(5);

	/*CExpression* me = compose_matrix_element(data->gete(CConst::GL_ALPHA),
		data->gete(CConst::GL_BETA),
		data->gete(CConst::GL_LAMBDA));*/
	CExpression* me = compose_matrix_element(ind1a, ind2a, ind3a);

	dbg->logperiod(5);

	CExpression* mec = CEBasic::Congugate(me->clone(),true);

	dbg->logperiod(5);
	CEReplList* rule = new CEReplList(ind1a, ind1b, false);
	rule->add(ind2a, ind2b, false);
	rule->add(ind3a, ind3b, false);
	mec->applyRule(rule);
	delete rule;
	dbg->logperiod(5);

	CEProduct* seq = new CEProduct();
	////////// Unpolarized case
	seq->addItem(new CEMetric(ind1a, ind1b, false, false));
	seq->addItem(new CEMetric(ind2a, ind2b, false, false));
	seq->addItem(new CEMetric(ind3a, ind3b, false, false));
	////////// Polarized case
	//seq->addItem(phot_helisity(p1, p11, p21, true, ind1, false));
	//seq->addItem(phot_helisity(p1, p11, p21, true, ind3, false));
	//seq->addItem(phot_helisity(p2, p11, p21, true, ind2, false));
	//seq->addItem(phot_helisity(p2, p11, p21, true, ind4, false));

	seq->addItem(me);
	seq->addItem(ferm_matrix(pm21));
	seq->addItem(mec);
	seq->addItem(ferm_matrix(p11));
	dbg->log(10, "Total expression:", seq);
	//CESupport::SaveResults(seq->expand());
	//CESupport::SaveResults(seq->expand()->Arrange());

	//temp = sub_seq->expand();
	//CESupport::SaveResults(temp);
	//temp->Arrange();
	//CESupport::SaveResults(temp);

	//CExpression* work = CExpression::reduce(seq);

	dbg->logperiod(5);
	
	CExpression* work = CDiracAlgebra::simplifyGammaSequence(seq, NULL, NULL);
	//delete seq;
	dbg->logperiod(5);
	dbg->log(15, "Simplified:", work);
	dbg->logperiod(5);
	work = CFSp::Spur(work);
	dbg->logperiod(5);
	dbg->log(10, "Trace calculated:", work);
	dbg->logperiod(5);

	delete work;
	delete ind1a;
	delete ind1b;
	delete ind2a;
	delete ind2b;
	delete ind3a;
	delete ind3b;
	delete pm21;
	dbg->logperiod(5);
	return NULL;
}

CExpression* CFeynLib::compose_matrix_element(CExpression * inda, CExpression * indb, CExpression * indk)
{
	CExpression* m = data->getv("p\\_11", CConst::DNV_MODULE);
	CExpression* p1 = data->getv("p\\_1");
	CExpression* p2 = data->getv("p\\_2");
	CExpression* p11 = data->getv("p\\_11");
	CExpression* p21 = data->getv("p\\_21");
	CExpression* pm21 = p21->clone();
	pm21->unsetFlag();
	CExpression* k = data->getv("k");
	
	CESum* res = new CESum();
	CEProduct* sub_seq = new CEProduct();
	sub_seq->addItem(new CEGamma(indk, true));
	CExpression* temp = p11->plus(k, true);
	sub_seq->addItem(ferm_propagator(temp, m));
	delete temp;
	sub_seq->addItem(new CEGamma(inda, true));
	temp = p2->minus(p21, true);
	sub_seq->addItem(ferm_propagator(temp, m));
	delete temp;
	sub_seq->addItem(new CEGamma(indb, true));
	res->addItem(sub_seq);

	/*sub_seq = new CEProduct();
	sub_seq->addItem(new CEGamma(inda, true));
	temp = p11->minus(p1, true);
	sub_seq->addItem(ferm_propagator(temp, m));
	delete temp;
	sub_seq->addItem(new CEGamma(indk, true));
	temp = p2->minus(p21, true);
	sub_seq->addItem(ferm_propagator(temp, m));
	delete temp;
	sub_seq->addItem(new CEGamma(indb, true));
	res->addItem(sub_seq);

	sub_seq = new CEProduct();
	sub_seq->addItem(new CEGamma(inda, true));
	temp = p11->minus(p1, true);
	sub_seq->addItem(ferm_propagator(temp, m));
	delete temp;
	sub_seq->addItem(new CEGamma(indb, true));
	temp = p21->plus(k, true);
	temp->inverseFlag();
	sub_seq->addItem(ferm_propagator(temp, m));
	delete temp;
	sub_seq->addItem(new CEGamma(indk, true));
	res->addItem(sub_seq);*/

	dbg->log(30, "Half of matrix element", res);
	
	CExpression* work = CDiracAlgebra::simplifyGammaSequence(res, p11, pm21);
	delete res;

	dbg->log(30, "Half of matrix element expanded", work);

	//CESum* work = res;
	CEReplList* rule = new CEReplList(p1, p2, true);
	rule->add(inda, indb, true);
	temp = work->clone();
	temp->applyRule(rule);
	delete rule;
	
	work = work->add(temp);
	delete pm21;
	
	dbg->log(30, "Matrix element", work);
	return work;
}

CExpression* CFeynLib::matr_el(CExpression *in1, CExpression *in2, CExpression *f1, CExpression *f2, CExpression *f3)
{
	CESymbol* k1 = new CESymbol("k\\_1");
	CESymbol* k2 = new CESymbol("k\\_2");
	CESymbol* k3 = new CESymbol("k\\_3");
	CESymbol* pp = new CESymbol("w\\_1");
	CESymbol* pm = new CESymbol("w\\_2");
	CESymbol* m = new CESymbol("m");
	CExpression* u1 = CESymbol::getUnique();
	CExpression* u2 = CESymbol::getUnique();
	CExpression* u3 = CESymbol::getUnique();
	CExpression* u4 = CESymbol::getUnique();
	//CExpression* u5 = CESymbol::getUnique();
	
	CExpression* res = boson_vertex(pp, k3, f1, false, f3, false, u1, false);
	CExpression* temp = pp->clone()->add(k3->clone());
	res = res->multiply(boson_propagator(temp, m, u1, true, u2, true));
	k1->inverseFlag();
	res = res->multiply(boson_vertex(k1, temp, in1, false, u2, false, u3, false));
	k1->inverseFlag();
	delete temp;
	res = CExpression::reduce(res);
	dbg->log(10, "matrix-> ", res);
	temp = pm->clone()->minus(k2->clone());
	res = res->multiply(boson_propagator(temp, m, u3, true, u4, true));
	delete temp;
	res = CExpression::reduce(res);
	pm->inverseFlag();
	res = res->multiply(boson_vertex(pm, k2, f2, false, in2, false, u4, false));
	res = CExpression::reduce(res);

	//res = res ->expand();
	//dbg->log(10, "matrix-> ", res);
	dbg->log(10, "matrix-> ", res);

	dbg->log(10, "res-> ", res);
	return res;
}

CExpression* CFeynLib::boson_propagator(CExpression *base, CExpression *m,
										CExpression *ind1, bool c1,
										CExpression *ind2, bool c2)
{
	CExpression* res = new CEMetric(ind1, ind2, c1, c2);
	res = res->minus(
		(new CEVector(base, ind1, c1))->multiply(new CEVector(base, ind2, c2))
		->divide(new CEPower(m, (long)2))
		);
	/*res = res->divide(
		(new CEPower(base, (long)2))->minus(new CEPower(m, (long)2))
		);*/
	dbg->log(10, "boson_propagator -> ", res);
	return res;
}

CExpression* CFeynLib::boson_vertex(CExpression *b1, CExpression *b2,
									CExpression *i1, bool c1,
									CExpression *i2, bool c2,
									CExpression *i3, bool c3)
{
	CExpression* res = (new CEVector(b1, i1, c1))->multiply(CENumber::GetTwo());
	res = res->add(new CEVector(b2, i1, c1));
	res = res->multiply(new CEMetric(i2, i3, c2, c3));
	CExpression* temp = (new CEVector(b2, i2, c2))->multiply(CENumber::GetTwo());
	temp = temp->add(new CEVector(b1, i2, c2));
	temp = temp->multiply(new CEMetric(i1, i3, c1, c3));
	res = res->minus(temp);
	temp = (new CEVector(b2, i3, c3))->minus(new CEVector(b1, i3, c3));
	temp = temp->multiply(new CEMetric(i1, i2, c1, c2));
	res = res->minus(temp);
	dbg->log(10, "boson_vertex -> ", res);
	return res;
}

double CFeynLib::total_cs(double mid_me_sqr, double csm_en, double m)
{
	double res = (sqrt(1-4*m*m/(csm_en*csm_en)))/0.7;
	res *= (500/csm_en)*(500/csm_en);
	res *= 0.175;
	res *= mid_me_sqr;
	return res;
}
