////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// LMath.cpp: implementation of the CLMath class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <math.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDebug* CLMath::dbg = new CDebug("CLMath");
char* CLMath::m_TempBuffer100 = NULL; // created at first call, deleted by CEProcessor::AddToDeleteList();

CLMath::CLMath()
{

}

CLMath::~CLMath()
{

}

CEValue* CLMath::Sin(CEValue * angle)
{
	//angle->normalize();
	CEValue* res = new CERealValue(sin(angle->to_double()));
	//angle->copyAttributesTo(res, true);
	return res;
}
double CLMath::Sin(double angle, bool in_degree) {
	if (in_degree) angle *= (3.14159/180.);
	return sin(angle);
}

CEValue* CLMath::Cos(CEValue * angle)
{
	//angle->normalize();
	CEValue* res = new CERealValue(cos(angle->to_double()));
	//angle->copyAttributesTo(res, true);
	return res;
}
double CLMath::Cos(double angle, bool in_degree) {
	if (in_degree) angle *= (3.14159/180.);
	return cos(angle);
}

/*
* f(x)=x^3+x^2+x;
*/
CExpression* CLMath::temp_func1(CExpression * x)
{
	CExpression* res = new CEPower(x, (long)3);
	res->add(new CEPower(x, (long)2));
	res->add(x);
	return res;
}

CExpression* CLMath::temp_func2()
{
	CESymbol* w = new CESymbol("w");
	CESymbol* v = new CESymbol("v");
	CExpression* res = new CEPower(w, (long)2);
	res = res->add(new CEPower(v, (long)2));
	delete w;
	delete v;
	res = res->multiply(new CESymbol("s1"))->multiply(CENumber::GetFour());
	res = res->divide(new CESymbol("t"))->divide(new CESymbol("u"))
		->divide(new CESymbol("t1"))->divide(new CESymbol("u1"));
	res = res->multiply(new CESymbol("a2pi"));
	res = CExpression::reduce(res);
	dbg->log(10, "Generated expression", res);
	return res;
}

CExpression* CLMath::temp_func3()
{
	CExpression* s = new CESymbol("s");
	CExpression* t = new CESymbol("t");
	CExpression* t1 = new CESymbol("t1");
	CExpression* u = new CESymbol("u");
	CExpression* u1 = new CESymbol("u1");
	CExpression* w = new CESymbol("w");
	CExpression* v = new CESymbol("v");
	CExpression* z = new CESymbol("z");
	CExpression* z1 = new CESymbol("z1");

	CExpression* x = (new CEPower(t1, (long)2))->add(new CEPower(u, (long)2))->divide(t->clone())->divide(u1->clone());
	x = x->add((new CEPower(t, (long)2))->add(new CEPower(u1, (long)2))->divide(t1->clone())->divide(u->clone()));
	x = x->clone()->divide(w->clone())->add(x->divide(v->clone()));
	x->inverseFlag();
	CExpression* res = x;

	x = t->clone()->divide(t1->clone())->divide(u->clone());
	x = x->add(u->clone()->divide(t->clone())->divide(t1->clone()));
	x = x->minus(t->clone()->add(t1->clone())->divide(t->clone())->divide(u1->clone()));
	CExpression* f = w->clone()->minus(z)->divide(w->clone())->divide(v->clone());
	x = x->multiply(s->clone())->multiply(f);

	res = res->add(x);

	x = u->clone()->divide(t->clone())->divide(u1->clone());
	x = x->add(t->clone()->divide(u->clone())->divide(u1->clone()));
	x = x->minus(u->clone()->add(u1->clone())->divide(t1->clone())->divide(u->clone()));
	f = w->clone()->minus(z1)->divide(w->clone())->divide(v->clone());
	x = x->multiply(s->clone())->multiply(f);

	res = res->add(x);

	res = res->multiply(CENumber::GetFour())->multiply(new CESymbol("a2pi"));
	delete s;
	delete t;
	delete t1;
	delete u;
	delete u1;
	delete w;
	delete v;
	//res = CExpression::reduce(res);
	dbg->log(10, "Generated expression", res);
	return res;
}

CExpression* CLMath::temp_func0()
{
	/*CExpression* res = (new CESymbol("t"))->divide(new CESymbol("u"));
	res = res->add((new CESymbol("u"))->divide(new CESymbol("t")));
	res = res->multiply(CENumber::GetTwo());
	res = CExpression::reduce(res);
	dbg->log(10, "Generated expression", res);
	return res;*/
	CExpression* res = (new CESymbol("t"))->divide(
		(new CEPower("m", (long)2))->minus(new CESymbol("u"))
		);
	res = res->add((new CESymbol("u"))->divide(
		(new CEPower("m", (long)2))->minus(new CESymbol("t")))
		);
	res = res->multiply(CENumber::GetTwo());
	res->inverseFlag();
	res = CExpression::reduce(res);
	dbg->log(10, "Generated expression", res);
	return res;
}

int CLMath::get_power(double val)
{
	if (val<0) return get_power(-val);
	if (val==0) return 0;
	int res = 0;
	if (val<1) {
		for (;val<1;) {
			val *= 10;
			res--;
		}
	}
	else {
		for (;val>=10;) {
			val /= 10;
			res++;
		}
	}
	return res;
}

/*
*	Note: this procedure is very slow
*	Gets the mantissa and DECIMAL exponent of a floating-point number
*	x == result*10^(*expptr)
*	result: [1..10)
*	result >=10 means error (CLMath_frexp10_ERROR)
*/
#define CLMath_frexp10_ERROR	100
double CLMath::frexp10(double x, int *expptr) {
	if (m_TempBuffer100==NULL) { m_TempBuffer100=(char*)CMemoryManager::mmalloc(100); CEProcessor::AddToDeleteList(m_TempBuffer100); }
	//double m=frexp(x,&p2);
	char* buf = m_TempBuffer100;
	bool bMinus=false; if(x<0){ x=-x; bMinus=true; } // Not-obligatory check
	int l = sprintf(buf, "%.18e", x);
	int iE=CEUtil::findSymbol(buf, 'e');
	if ((iE<0) || (iE>=l)) { *expptr=0; return CLMath_frexp10_ERROR; }
	*(buf+iE)='\0';
	double m=0; long pw=0;
	bool val_ok = CEUtil::atofSafe(buf, &m); if (!val_ok) { *expptr=0; return CLMath_frexp10_ERROR; }
	val_ok = CEUtil::atolSafe(buf+iE+1, &pw); if (!val_ok) { *expptr=0; return CLMath_frexp10_ERROR; }
	*expptr=pw;
	if (bMinus) m=-m;
	// test: printf("((%e->%e %ld))",x,m,pw);
	return m;
}

int CLMath::get_digits_num(long val) {
	if (val<0) return get_digits_num(-val);
	int res = 1;
	for (;val>=10;) { val /= 10; res++; }
	return res;
}

double CLMath::length(double x1, double x2, double y1, double y2)
{
	return sqrt(sqr(x2-x1)+sqr(y2-y1));
}

int CLMath::LengthApprox(int x1, int y1, int x2, int y2) {
	int dx=x2-x1; if(dx<0)dx=-dx; int dy=y2-y1; if(dy<0)dy=-dy;
	return ((dx>dy)?(dx+dy/2):(dy+dx/2));
}

long CLMath::Log2Int(long val) { // val = 2^(res), -1 if res is NOT integer
	if (val<=0) return -1;
	long res = 0;
	for (;val>1;) {
		if (val%2 != 0) return -1;
		val = val/2;
		res++;
	}
	return res;
}

double CLMath::sqr(double x)
{
	return x*x;
}

double CLMath::angle(double x, double y, bool in_degrees)
{
	double res = 0;

	double precision_gap = 0.00000001;
	if ((x == 0) || ((x > 0) && (x < precision_gap)) || ((x < 0) && (x > -precision_gap))) {
		if (y == 0) {
			dbg->log(1, "Error: angle(0,0) request.");
			return 0;
		}
		else {
			res = (3.14159)/2;

			// check up quarter
			// (see p.35(below) for details)
			if (y<0) { res+= 3.14159; } // += 180 degrees
		}
	}
	else {
		res = atan(y/x);

		// check up quarter
		// atan reults are (-90,+90)
		// (see p.35(below) for details)
		if (x<0) {
			res+= 3.14159;// += 180 degrees
		}
		//else if (y<0) { res+= 2.*3.14159; } // += 360 degrees
	}

	if (in_degrees) res = res*(180/3.14159);
	return res;
}

CExpression* CLMath::det(CSequence *arr, int col_pos, CSequence *col)
{
	if (col == NULL) return det(arr);

	if (arr == NULL) {
		dbg->log(0, "Error: det{NULL, x}");
		return NULL;
	}

	int r = arr->size();
	if (r < 1) {
		dbg->log(1, "Error: det{0:0}");
		return NULL;
	}

	if ((col_pos < 0) && (col_pos >= r)) {
		dbg->log(0, "Warning: (det(,,)) extra column replaces nothing in array");
		return det(arr);
	}

	CSequence* arr_copy = (CSequence*)arr->clone();
	arr_copy->SetAt(col_pos, col);
	CExpression* res = det(arr_copy);
	delete arr_copy;
	return res;
}

CExpression* CLMath::det(CSequence *arr)
{
	if (arr == NULL) {
		dbg->log(0, "Error: det{NULL}");
		return NULL;
	}

	int r = arr->size();
	if (r < 1) {
		dbg->log(0, "Error: det{0:0}");
		//delete arr;
		return NULL;
	}

	if (r == 1) {
		CSequence* col0 = (CSequence*)arr->GetAt(0);
		CExpression* res = col0->GetAt(0)->clone();
		//delete arr;
		return res;
	}

	CESum* res = new CESum();
	bool sign = true;
	int i;
	for(i=0; i<r; i++) {
		CSequence* col_i = (CSequence*)arr->GetAt(i);
		CExpression* item_i0 = col_i->GetAt(0)->clone();

		CSequence* arr_copy = (CSequence*)arr->clone();
		arr_copy->RemoveAt(i);
		int j;
		for(j=0; j<(r-1); j++) {
			CSequence* col = (CSequence*)arr_copy->GetAt(j);
			if (col->size() != r) {
				dbg->log(0, "Error det() of non-square matrix.");
				delete arr_copy;
				delete res;
				//delete arr;
				return NULL;
			}
			col->RemoveAt(0);
		}
		CExpression* item = det(arr_copy);
		if (item == NULL) {
			delete res;
			delete arr_copy;
			delete item_i0;
			//delete arr;
			return NULL;
		}
		delete arr_copy;

		item = item_i0->multiply(item);
		if (!sign) item->inverseFlag();
		res->addItem(item);
		sign = !sign;
	}
	//delete arr;
	return res;
}

CSequence* CLMath::linear_system_solve(CSequence *arr, CSequence *col)
{
	dbg->log(20, "Linear system solving (arr)", arr);
	dbg->log(20, "(col)", col);

	CExpression* den = det(arr);
	//*****************************
	//den = CExpression::reduce(den);
	if (den == NULL) return NULL;
	den = CExpression::reduce(den);
	if (CEBasic::isZero(den)) {
		dbg->log(10, "Unsolvable linear system");
		delete den;
		return NULL;
	}

	CSequence* res = new CSequence();
	long s = arr->size();
	long i;
	for (i=0; i<s; i++) {
		CExpression* item = det(arr, i, col);
		//*****************************
		//item = CExpression::reduce(item);
		if (item == NULL) {
			dbg->log(10, "Error solving linear system");
			delete res;
			delete den;
			return NULL;
		}
		res->addItem(item->divide(den->clone()));
	}
	delete den;
	dbg->log(20, "Linear system solution", res);
	return res;
}

bool CLMath::is_even(long num)
{
	return (num%2 == 0);
}

long CLMath::least_common_multiple(long a, long b)
{
	if (a<0) a=-a;
	if (b<0) b=-b;

	if ((a==0) || (b==0)) return 0;
	if (a==1) return b;
	if (b==1) return a;

	long g_c_d = greatest_common_divisor(a, b);
	long res = a*((long)(b/g_c_d));
	return res;
}

long CLMath::greatest_common_divisor(long a, long b)
{
	if (a<0) a=-a;
	if (b<0) b=-b;

	if ((a==0) && (b==0)) return 1;
	if (a==0) return b;
	if (b==0) return a;

	long res = 1;

	long i;
	for(i=2; ((i<=a) && (i<=b)); i++) {
		if ((a%i == 0) && (b%i == 0)) {
			a = (long)(a/i);
			b = (long)(b/i);
			res *= i;
			i--;
		}
	}
	return res;
}

long CLMath::abs(long val) {
	return ((val >= 0)?val:(-val));
}

double CLMath::Abs(double val) {
	return ((val >= 0)?val:(-val));
}

long CLMath::factorial(int val) {
	if (val < 0) { dbg->log(0, val, "Error: CLMath::factorial(%ld) call, negative argument. 0 returned"); return 0; }
	//if (val == 0) return 1;
	long res = 1;
	for(int i=2; i<=val; i++) { res *= i; }
	return res;
}

long CLMath::factorial_frac(int num, int den) { // num!/den!
	if ((num<0) || (den<0)) { dbg->log(0, num, den, "Error: CLMath::factorial_frac(%ld,%ld) call, negative argument. 1 returned"); return 1; }
	if (num<den) { dbg->log(0, num, den, "Error: CLMath::factorial_frac(%ld,%ld) call, non-integer result truncated. 1 returned"); return 1; }

	long res = 1;
	for(long i=(den+1); i<=num; i++) { res *= i; }
	return res;
}

long CLMath::CNOfK(int n, int k) { // (n,k) = n!/(k! (n-k)!)
	if ((n<0) || (k<0)) { dbg->log(0, n, k, "Error: CLMath::CNOfK(%ld,%ld) call, negative argument. Halted"); CEException::halt(); }
	if (n<k) { dbg->log(0, n, k, "Error: CLMath::CNOfK(%ld,%ld) call, non-integer result truncated. Halted"); CEException::halt(); }
	//TODO: add long-MAX exceeding check for result

	if ((n-k)>k) k=n-k;
	long res = 1;
	long i;
	for(i=(k+1); i<=n; i++) { res *= i; } // n! / k!
	for(i=1; i<=(n-k); i++) { res = res/i; } // res / (n-k)!
	return res;
}

long CLMath::Pow(long a, long n) {
	if (n<0) CEException::halt("CLMath::Pow() ERROR: negative power");
	long res = 1;
	for(long i=0;i<n;i++) { res *= a; }
	return res;
}

long CLMath::lmin(long a, long b) { return (a<=b)?a:b; }
long CLMath::lmax(long a, long b) { return (a>=b)?a:b; }
long CLMath::step(long a, long b) { if(a<b)return 1; if(a>b)return -1; return 0; }

int CLMath::Round(double val) {
	int res = (int)val; double gap = val-res;
	if (Abs(gap) > 1) {printf("CLMath::Round INTERNAL ERROR.\n"); throw(new CEException(CEException::INT_ERROR));}
	if (gap > 0.5) { res++; } else if (gap < (-0.5)) { res--; }
	return res;
}

int CLMath::Trunc(double val) {
	int res = (int)val; double gap = val-res;
	if (Abs(gap) > 1) {printf("CLMath::Trunc INTERNAL ERROR.\n"); throw(new CEException(CEException::INT_ERROR));}
	// TODO: res--; should be here, should it???!!!
	if (gap < 0) { res++; }
	return res;
}
