////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// CodeProcessImpl.cpp: implementation of the CCodeProcessImpl class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////	CCodeProcessImpl
//
const char* CCodeProcessImpl::CN_CHEPProcessLite	= "CHEPProcessLite";		//basic class for processes
const char* CCodeProcessImpl::CN_CHEPProcessLiteSet	= "CHEPProcessLiteSet";		//class for process set
const char* CCodeProcessImpl::CN_CHEPEvtLiteCut		= "CHEPEvtLiteCut";			//basic class for cuts
const char* CCodeProcessImpl::CN_CHEPEvtLiteCutSet	= "CHEPEvtLiteCutSet";		//class for cut set
const char* CCodeProcessImpl::CN_CHEPEvtLiteCutE	= "CHEPEvtLiteCutE";		//class for cut on energy
const char* CCodeProcessImpl::CN_CHEPEvtLiteCutPt	= "CHEPEvtLiteCutPt";		//class for cut on pt
const char* CCodeProcessImpl::CN_CHEPEvtLiteCutEta	= "CHEPEvtLiteCutEta";		//class for cut on eta
const char* CCodeProcessImpl::CN_CHEPEvtLiteCutDR	= "CHEPEvtLiteCutDR";		//class for DeltaR cut
const char* CCodeProcessImpl::CN_CVMGen				= "CVMGen";					//basic class for MC
const char* CCodeProcessImpl::CN_CVMRnd				= "CVMRnd";					//basic class for Rnd
const char* CCodeProcessImpl::CN_CVMPhaseSpaceGen	= "CVMPhaseSpaceGen";		//PH-reco class
const char* CCodeProcessImpl::CN_CVMGenFunc			= "CVMGen::CVMGenFunc";		//MC source function
const char* CCodeProcessImpl::CN_CVMGenFuncMulti	= "CVMPhaseSpaceGen::CVMFuncMultiChannel";
																				//MC source function for parton-based process
const char* CCodeProcessImpl::CN_CHEPEvtLiteLHE		= "CHEPEvtLiteLHE";			//HEPEvtLite -> LHE converter class
const char* CCodeProcessImpl::CN_CHEPParticleOrder	= "CHEPParticleOrderLite";	//particle order structure
const char* CCodeProcessImpl::CN_CVMHistLite		= "CVMHistLite";			//
const char* CCodeProcessImpl::CN_CVMHistLiteView	= "CVMHistLiteView";		//
const char* CCodeProcessImpl::CN_CVMHistLiteViewDef	= "CVMHistLiteViewDefaults";//
const char* CCodeProcessImpl::CN_CVMOutputSettings	= "CVMOutputSettings";		//

const char* CCodeProcessImpl::CN_CPP_4VEC_CLASS		= "CVM4Vec";
const char* CCodeProcessImpl::CN_CPP_EVT_CLASS		= "CHEPEvtLite";			//event class
const char* CCodeProcessImpl::CN_CPP_LIB_CLASS		= "CAlHepLib";				//library class
const char* CCodeProcessImpl::CN_CPP_PDF			= "CPDFLite";				//basic PDF interface

const char* CCodeProcessImpl::FN_CPP_LIB_FILE		= "vmlib.h";
const char* CCodeProcessImpl::FN_CPP_LIB_PATH		= "VMLib/include/";

const char* CCodeProcessImpl::Flag_Real8			= "BHCODE_8";				//used if defined in 'vmlib.h'
const char* CCodeProcessImpl::Flag_Real16			= "BHCODE_16";				//used if defined in 'vmlib.h'
const char* CCodeProcessImpl::Flag_Real32			= "BHCODE_32";				//used if defined in 'vmlib.h'

////////////////////////////////////////////////////////////////////// CCodeProcessAdv
//
CCodeProcessAdv::CBeamInfo::CBeamInfo() : CEObject(false) { pk=0; bPartonsUsed=false; /*pdfId1=-1; pdfId2=-1;*/ }
void CCodeProcessAdv::CBeamInfo::Reset() { pk=0; bPartonsUsed=false; /*pdfId1=-1; pdfId2=-1;*/ }
CCodeProcessAdv::CBeamInfo::~CBeamInfo() {}
CCodeProcessAdv::CSubProcInfo::CSubProcInfo() : CEObject(false) {
	m_iFirstFinal=2; m_nFinals=0; pk.FillAll(0); m_iPosInitials[0]=0; m_iPosInitials[1]=1;
}
CCodeProcessAdv::CSubProcInfo::~CSubProcInfo() {}
//
CCodeProcessAdv::CCodeProcessAdv(const char* _sProcTitle) : CEObject() {
	m_bEstimatedIsStatic=false; m_bUseEvtNuclonEntries=true;
	m_bAutoSwapPartons=false; //repeated in 'Reset()'
	if(_sProcTitle!=NULL){ sProcTitle.AddString(_sProcTitle); }else{ sProcTitle.AddString("UnknownProcess"); }
}
CCodeProcessAdv::~CCodeProcessAdv() {}
int CCodeProcessAdv::GetNSubsActual() { //including quark-swap channels
	int szSubs=m_Subs.size();
	int nSubsActual=szSubs;
	if(m_bAutoSwapPartons && m_beam1.bPartonsUsed && m_beam2.bPartonsUsed){
		for(int i=0;i<szSubs;i++){ if(SubProc(i)->pk.GetAt(0)!=SubProc(i)->pk.GetAt(1)){ nSubsActual++; } }
	}
	return nSubsActual;
}
void CCodeProcessAdv::Reset() {
	m_Subs.clear_all(); sConstFileName.Reset(); m_beam1.Reset(); m_beam2.Reset();
	m_bAutoSwapPartons=false;
}
bool CCodeProcessAdv::AddSubProcess(
									const char* subId,			//sub-process name (implemented as 'C+id' class)
									int subFileId,				//id of C++ file in CLFileManager with simple implementation already saved by CFFortran
									const char* subId_B,		//(==subId by default) name for Born term (implemented as 'C+id' class)
									const char* subId_V			//name for V-term for sub-process (implemented as 'C+id' class)
									)
{
	/*
	*	Notes
	*	=====
	*		1. Implementation of the process must be alreaady done as 'C+id' class
	*		2. Entry in 'C+id' class is 'tReal Sub000(CHEPEvtLite* evt [, long kQOrig])'
	*/
	int i;

	//////////////////////////////////////////////////////////////////////// 1.Init
	//
	CKinematics* kin = CEProcessor::kinematics(); if(kin==NULL)return false;
	CDiagramPhysics* phys = CDiagramPhysics::get_curr_physics(); if(phys==NULL)return false;
	if (kin->num_init_particles() != 2) { CLFileManager::dbg->log(0,"[ERROR] CCodeProcessAdv: Implemented for 2->N processes only"); return false; }

	CSubProcInfo* proc=new CSubProcInfo(); if(proc==NULL)return false;
	proc->m_nFinals=kin->num_final_particles();
	int iFirstFinalPos=2; //id of the first final particle in the event
	int iPosInitials[]={0,1}; //IDs of initial particles (or PARTONS) in the event
	// Examples:
	//		p1, p2, q1, q2
	//		p1, i2, q1             <--- 'first' parton after 'second' lepton!
	//		i1, i2
	// Note: the same order must be presumed 'CVMPhaseSpaceGen::Init' { 'align.posIn[..]=' }
	if(m_bUseEvtNuclonEntries) {
		if(m_beam1.bPartonsUsed){ iPosInitials[0]=iFirstFinalPos; iFirstFinalPos++; }
		if(m_beam2.bPartonsUsed){ iPosInitials[1]=iFirstFinalPos; iFirstFinalPos++; }
	}
	proc->m_iFirstFinal=iFirstFinalPos; proc->m_iPosInitials[0]=iPosInitials[0]; proc->m_iPosInitials[1]=iPosInitials[1];
	//
	proc->sImplFileName<<CLFileManager::GetFileName(subFileId);
	proc->sImplFileNameHeader<<CLFileManager::GetHeaderFileName(subFileId);
	const char* fn_const = CLFileManager::GetConstIncludeFileName(subFileId);
	if((fn_const==NULL) || (CEUtil::length(fn_const)<=0)) { CLFileManager::dbg->log(0,"CCodeProcessAdv::AddSubProcess() Initialization failed (fn_const==NULL)"); delete proc; return false; }
	if(sConstFileName.length()<=0) { sConstFileName<<fn_const; } else{ if(!CEUtil::equals(fn_const,sConstFileName.GetBuffer())){ CLFileManager::dbg->log(0,"CCodeProcessAdv::AddSubProcess() ERROR: different constant files appeared"); delete proc; return false; } }
	if((proc->sImplFileName.length()<=0)||(proc->sImplFileNameHeader.length()==NULL)) { CLFileManager::dbg->log(0,"CCodeProcessAdv::AddSubProcess() Initialization failed"); delete proc; return false; }

	//TODO:
	//	1. Test 'id' for code compilance (sybmols + '_' + not-first-numbers)
	if(subId_B==NULL){ subId_B=subId; }

	proc->sImplClassB<<"C"<<subId_B;
	proc->sImplCallB<<subId_B;
	if(subId_V != NULL) {
		proc->sImplClassV<<"C"<<subId_V;
		proc->sImplCallV<<subId_V;
	}
	proc->sId<<subId;

	//Fill 'proc->pk' array 'as in kinamatics'
	//Nt: in kinematics N[initials]==2 always (and may be != iFirstFinal)
	for(i=0; i<(proc->m_nFinals+2); i++) {
		CEParticle* p=((i<2)?(kin->pin(i)):(kin->pfin(i-2))); if(p==NULL){ CLFileManager::dbg->log(0,"CCodeProcessAdv::AddSubProcess() NULL particle in kinematics"); delete proc; return false; }
		proc->pk.Add(p->get_PK());
	}
	if(proc->pk.GetSize() != (kin->num_init_particles()+kin->num_final_particles())){ CLFileManager::dbg->log(0,"CCodeProcessAdv::AddSubProcess() internal error"); delete proc; return false; }

	//
	m_Subs.add(proc);
	return true;
}

//
//	Important: don't use current kinematics in this function!
//	(e.g. R-term is saved while V-kin is installed)
//
bool CCodeProcessAdv::SaveProcessClass(
									   const char* id,				//process name
									   int file_id,					//id of C++ file in CLFileManager with simple implementation already saved by CFFortran
									   const char* proc_class_name,	//process class name ('CProc+id' if NULL)
									   bool bIncSubHdrs				//(never used!) include headers of sub-process classes
									   )
{
	int i,j;
	int szSubs=m_Subs.size();
	if(szSubs<=0){ CLFileManager::dbg->log(0,"CCodeProcessAdv::SaveProcessClass() ERROR: 0 sub-processes."); return false; }
	//

	//local options:
	//	m_bEstimatedIsStatic

	//////////////////////////////////////////////////////////////////////// 1.Init
	//
	CDiagramPhysics* phys = CDiagramPhysics::get_curr_physics(); if(phys==NULL)return false;
	int nFinals=SubProc(0)->m_nFinals;			//Must be equal for all sub-processes
	int iFirstFinal=SubProc(0)->m_iFirstFinal;	//Must be equal for all sub-processes
	int iPosInitials[]={SubProc(0)->m_iPosInitials[0],SubProc(0)->m_iPosInitials[1]};
												//Must be equal for all sub-processes
	for(i=1;i<szSubs;i++){ //self-check
		if ((nFinals!=SubProc(i)->m_nFinals) ||
			(iFirstFinal!=SubProc(i)->m_iFirstFinal) ||
			(iPosInitials[0]!=SubProc(i)->m_iPosInitials[0]) ||
			(iPosInitials[1]!=SubProc(i)->m_iPosInitials[1]))
		{
			CLFileManager::dbg->log(0,"CCodeProcessAdv::SaveProcessClass() ERROR: sub-processes are INCOMPARTIBLE"); return false;
		}
	}
	int nSubsActual=GetNSubsActual();

	CCodeLanguage* codeLang = CLFileManager::GetCodeLanguageRef(file_id);
	const char* fn = CLFileManager::GetFileName(file_id);
	const char* fn_hdr = CLFileManager::GetHeaderFileName(file_id);
	//Nt: 'sConstFileName' must be initialized before
	if(sConstFileName.length()<=0) { CLFileManager::dbg->log(0,"CCodeProcessAdv::SaveProcessClass() Initialization failed (sConstFileName.length()<=0)"); return false; }
	if((codeLang==NULL)||(fn==NULL)||(fn_hdr==NULL)) { CLFileManager::dbg->log(0,"CCodeProcessAdv::SaveProcessClass() Initialization failed"); return false; }
	CEString sClassProc; if(proc_class_name!=NULL){ sClassProc<<proc_class_name; }else{ sClassProc<<"CProc"<<id; }
	CEString sClassProcGenFunc(sClassProc.getData()); sClassProcGenFunc<<"GenFunc";
	CEString ln; //CEString lnTmp;
	const char* varMassFin="_MassesFin"; const char* varPIDsFin="_IDsFin"; const char* varI="i";
	const char* varSubWgt="m_subWgt"; const char* varSubWgtSum="m_subWgtSum";
	ofstream ofs;

	//////////////////////////////////////////////////////////////////////// 2.Header file
	//
	//	class CProcMy : public CHEPProcessLite {
	//	private:
	//		class CProcMyGenFunc : public CVMGen::CVMGenFunc {
	//			CProcMy* m_proc;
	//			CVMPhaseSpaceGen* m_ph;
	//		public:
	//			CProcMyGenFunc(CProcMy* proc, CVMPhaseSpaceGen* ph);
	//			virtual tReal Call(tReal* params);
	//			virtual CVMGen::CVMGenFunc* Clone() const;
	//			virtual int SelectChannel(tReal* params);	//for multichannel process only
	//		};
	//	private:
	//		CClassImpl m_proc_impl;					//implementation of diff cross section
	//		CHEPEvtLiteCutSet m_cuts;
	//		CVMGen* m_gen;							//contains cross section
	//		CVMPhaseSpaceGen* m_ph;					//constains 'm_gen' as external generator
	//	public:
	//		tReal CalcDiffCS(CHEPEvtLite* evt);		//differential c/s calculation function
	//		CProcMy();
	//		virtual ~CProcMy();
	//		void SetPDFs(const CPDFLite* _pdf1, const CPDFLite* _pdf2); //set PDFs instead of default ones
	//	public:
	//		//process interface
	//		virtual void AddCut(const CHEPEvtLiteCut* cut);
	//		virtual bool Prepare(const CVM4Vec* p1, const CVM4Vec* p2);
	//		virtual bool GetCrossSection(tReal& cs, tReal* err=NULL);
	//		virtual bool GenEvent(CHEPEvtLite* evt);
	//	};
	//
	ofs.open(fn_hdr, ios::out|ios::app);
	if (!ofs) { CLFileManager::dbg->logIns(0,"<Error opening output header file (%s).>", fn_hdr); return false; }
	ofs<<"\n";
	codeLang->StartCommentLine(&ofs); ofs << "\n";
	codeLang->StartCommentLine(&ofs); ofs << "\tProcess class "<<sClassProc.getData()<<"\n";
	codeLang->StartCommentLine(&ofs); ofs << "\n";
	ofs<<"#if !defined(_ALHEP_CLASS_"<<sClassProc.GetBuffer()<<")\n#define _ALHEP_CLASS_"<<sClassProc.GetBuffer()<<"\n";
	ln<<"class "<<sClassProc.getData()<<" : public "<<CCodeProcessImpl::CN_CHEPProcessLite<<" {"; codeLang->OutCodeLine(&ofs,&ln,-1,true);
	ln<<"private:"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	ln<<"class "<<sClassProcGenFunc.getData()<<" : public "
		<<((m_beam1.bPartonsUsed || m_beam2.bPartonsUsed)?(CCodeProcessImpl::CN_CVMGenFuncMulti):(CCodeProcessImpl::CN_CVMGenFunc))
		<<" {"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	ln<<sClassProc.getData()<<"* m_proc"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<CCodeProcessImpl::CN_CVMPhaseSpaceGen<<"* m_ph"; codeLang->OutCodeLine(&ofs,&ln);
	if(nSubsActual>1){
		codeLang->DecIndent(); ln<<"public:"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
		ln<<codeLang->GetTypePrefixReal()<<" "<<varSubWgt<<"["<<nSubsActual<<"]"; codeLang->OutCodeLine(&ofs,&ln,-1,false,"weights of sub-processes");
		ln<<codeLang->GetTypePrefixReal()<<" "<<varSubWgtSum; codeLang->OutCodeLine(&ofs,&ln);
	}
	ln<<codeLang->GetTypePrefixInt()<<" m_nCalls"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<codeLang->GetTypePrefixInt()<<" m_nLogCalls"; codeLang->OutCodeLine(&ofs,&ln,-1,false,"[debug] number of events to log");
	ln<<codeLang->GetTypePrefixInt()<<" m_iLogPrecision"; codeLang->OutCodeLine(&ofs,&ln,-1,false,"[debug] number of digits to log");
	codeLang->DecIndent(); ln<<"public:"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	ln<<sClassProcGenFunc.getData()<<"("<<sClassProc.getData()<<"* proc, "<<CCodeProcessImpl::CN_CVMPhaseSpaceGen<<"* ph)"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<"virtual "<<codeLang->GetTypePrefixReal()<<" Call("<<codeLang->GetTypePrefixReal()<<"* params)"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<"virtual "<<CCodeProcessImpl::CN_CVMGenFunc<<"* Clone() const"; codeLang->OutCodeLine(&ofs,&ln);
	if(nSubsActual>1){
		// GetPartonCodes: function is not currently used
		//ln<<"void GetPartonCodes("<<codeLang->GetTypePrefixInt()<<" iSub,"
		//	<<codeLang->GetTypePrefixInt()<<" &iPart1"<<<<codeLang->GetTypePrefixInt()<<" &iPart2)"; codeLang->OutCodeLine(&ofs,&ln);
		ln<<"virtual "<<codeLang->GetTypePrefixInt()<<" SelectChannel("<<codeLang->GetTypePrefixReal()<<"* params)"; codeLang->OutCodeLine(&ofs,&ln);
	}
	codeLang->DecIndent(); ln<<codeLang->GetOpEndProc(); codeLang->OutCodeLine(&ofs,&ln);
	codeLang->DecIndent(); ln<<"private:"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	for(i=0; i<szSubs; i++){
		ln<<SubProc(i)->sImplClassB.getData()<<" m_proc_impl"; if(szSubs>1){ln<<"_"<<i;} codeLang->OutCodeLine(&ofs,&ln);
	}
	ln<<CCodeProcessImpl::CN_CHEPEvtLiteCutSet<<" m_cuts"; codeLang->OutCodeLine(&ofs,&ln);			//externally set
	ln<<CCodeProcessImpl::CN_CHEPEvtLiteCutSet<<" m_cutsLocal"; codeLang->OutCodeLine(&ofs,&ln,-1,false,"local copy with different particle order");
	ln<<sClassProcGenFunc.getData()<<"* m_func"; codeLang->OutCodeLine(&ofs,&ln);
	//ln<<CCodeProcessImpl::CN_CVMGen<<"* m_gen"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<CCodeProcessImpl::CN_CVMPhaseSpaceGen<<"* m_ph"; codeLang->OutCodeLine(&ofs,&ln);
	// beams
	ln<<CCodeProcessImpl::CN_CVMPhaseSpaceGen<<"::CBeamInfo beams[2]"; codeLang->OutCodeLine(&ofs,&ln);
	//[unused]if(m_beam1.bPartonsUsed){ ln<<CCodeProcessImpl::CN_CPP_PDF<<"* pdf1"; codeLang->OutCodeLine(&ofs,&ln); }
	//[unused]if(m_beam2.bPartonsUsed){ ln<<CCodeProcessImpl::CN_CPP_PDF<<"* pdf2"; codeLang->OutCodeLine(&ofs,&ln); }
	//
	ln<<CCodeProcessImpl::CN_CHEPParticleOrder<<"* m_InternalPartOrder"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<codeLang->GetTypePrefixBool()<<" m_bInternalPartOrderUsed"; codeLang->OutCodeLine(&ofs,&ln);
	codeLang->DecIndent(); ln<<"public:"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	ln<<(m_bEstimatedIsStatic?"static ":"")<<codeLang->GetTypePrefixReal()<<" m_fCrossSectionExpected_pb"; codeLang->OutCodeLine(&ofs,&ln,-1,false,"estimate for MC integration");
	ln<<codeLang->GetTypePrefixInt()<<" m_idProc"; codeLang->OutCodeLine(&ofs,&ln);
	codeLang->DecIndent(); ln<<"public:"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	ln<<codeLang->GetTypePrefixReal()<<" CalcDiffCS("; if(nSubsActual>1){ ln<<codeLang->GetTypePrefixInt()<<" iSub, "; }
	ln<<CCodeProcessImpl::CN_CPP_EVT_CLASS<<"* evt";
	if(m_beam1.bPartonsUsed) { ln<<", "<<codeLang->GetTypePrefixReal()<<" x1"; }
	if(m_beam2.bPartonsUsed) { ln<<", "<<codeLang->GetTypePrefixReal()<<" x2"; }
	ln<<")"; codeLang->OutCodeLine(&ofs,&ln);
	if(m_beam1.bPartonsUsed || m_beam2.bPartonsUsed) {
		ln<<codeLang->GetTypePrefixBool()<<" GetChannelInfo("<<codeLang->GetTypePrefixInt()<<" iSub, "<<
			codeLang->GetTypePrefixInt()<<"& id1, "<<codeLang->GetTypePrefixInt()<<"& id2)"; codeLang->OutCodeLine(&ofs,&ln);
	}
	ln<<sClassProc.getData()<<"("<<codeLang->GetTypePrefixInt()<<" idProc)"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<"virtual ~"<<sClassProc.getData()<<"()"; codeLang->OutCodeLine(&ofs,&ln);
	if(m_beam1.bPartonsUsed || m_beam2.bPartonsUsed){
		ln<<"void SetPDFs("; bool bComma=false; if(m_beam1.bPartonsUsed){ ln<<"const "<<CCodeProcessImpl::CN_CPP_PDF<<"* _pdf1"; bComma=true;}
		if(m_beam2.bPartonsUsed){ if(bComma){ ln<<", "; } ln<<"const "<<CCodeProcessImpl::CN_CPP_PDF<<"* _pdf2"; } ln<<")";
		codeLang->OutCodeLine(&ofs,&ln,-1,false,"Override default PDFs, to be called before 'Prepare'");
	}
	codeLang->DecIndent(); ln<<"public:"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	codeLang->StartCommentLine(&ofs); ofs << "process interface\n";
	ln<<"virtual void AddCut(const "<<CCodeProcessImpl::CN_CHEPEvtLiteCut<<"* cut)"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<"virtual "<<codeLang->GetTypePrefixBool()<<" Prepare(const "<<CCodeProcessImpl::CN_CPP_4VEC_CLASS<<"* p1, const "
		<<CCodeProcessImpl::CN_CPP_4VEC_CLASS<<"* p2, double fPrecision)"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<"virtual "<<codeLang->GetTypePrefixBool()<<" GetCrossSection("
		<<codeLang->GetTypePrefixReal()<<"& cs, "<<codeLang->GetTypePrefixReal()<<"* err=NULL) const"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<"virtual "<<codeLang->GetTypePrefixBool()<<" GenEvent("<<CCodeProcessImpl::CN_CPP_EVT_CLASS<<"* evt)"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<"static const char* CLASS_NAME"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<"virtual const char* ProcName() const"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<"virtual int ProcID() const"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<"void PrintGeneratorStatistics()"; codeLang->OutCodeLine(&ofs,&ln);
	codeLang->DecIndent(); ln<<codeLang->GetOpEndProc(); codeLang->OutCodeLine(&ofs,&ln);
	ofs<<"#endif // !defined(_ALHEP_CLASS_"<<sClassProc.GetBuffer()<<")\n";
	ofs<<"\n";
	ofs.close();

	//////////////////////////////////////////////////////////////////////// 3.Implementation file
	//
	//	CProcMy::CProcMyGenFunc::CProcMyGenFunc(CProcMy* proc,CVMPhaseSpaceGen* ph) : CVMGen::CVMGenFunc() { m_proc=proc; m_ph=ph; }
	//	tReal CProcMy::CProcMyGenFunc::Call(tReal* params) {
	//		if ((m_ph!=NULL) && (m_proc!=NULL)) {
	//			if (m_ph->FillEvent(params)) {
	//				tReal wgt=m_ph->m_PhSpaceWgt; //Note: Wgt==0 means e.g. cuts specified at FillEvent-step
	//				if (wgt>0.) { wgt*=m_proc->CalcDiffCS(m_ph->GetLastEvent()); } // Don't call function for Wgt=0 event
	//				return wgt;
	//			}
	//		}
	//		if(m_ph!=NULL)m_ph->IncEvtsFailed();
	//		return (tReal)0.;
	//	}
	ofs.open(fn, ios::out|ios::app);
	if (!ofs) { CLFileManager::dbg->logIns(0,"<Error opening output implementation file (%s).>", fn); return false; }
	ofs<<"\n";
	codeLang->StartCommentLine(&ofs); ofs << "\n";
	codeLang->StartCommentLine(&ofs); ofs << "\tProcess class "<<sClassProc.getData()<<"\n";
	codeLang->StartCommentLine(&ofs); ofs << "\n";
	if(bIncSubHdrs){
		//Note: this option may not be used due to:
		//	sub-headers MUST be included BEFORE main process header (i.e. in 'new CEFortran')
		CLFileManager::dbg->log(0,"WARNING in 'CCodeProcessAdv::SaveProcessClass': deprecated option used 'bIncSubHdrs'");
		int nHdrs=0;
		for(i=0; i<szSubs; i++) {
			const char* fnHdr=SubProc(i)->sImplFileNameHeader.GetBuffer(); long lHdr=SubProc(i)->sImplFileNameHeader.length();
			if((fnHdr!=NULL)||(lHdr>0)){ ln<<"#include \""<<fnHdr<<"\""; codeLang->OutCodeLine(&ofs,&ln,-1,true); nHdrs++; }
		}
		if(nHdrs>0){ codeLang->StartCommentLine(&ofs); ofs << "\n"; }
	}
	//
	ln<<sClassProc.getData()<<"::"<<sClassProcGenFunc.getData()<<"::"<<sClassProcGenFunc.getData()<<"("<<sClassProc.getData()<<"* proc, ";
	ln<<CCodeProcessImpl::CN_CVMPhaseSpaceGen<<"* ph) : "
		<<((m_beam1.bPartonsUsed || m_beam2.bPartonsUsed)?(CCodeProcessImpl::CN_CVMGenFuncMulti):(CCodeProcessImpl::CN_CVMGenFunc))
		<<"() {"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	ln<<"m_proc=proc; m_ph=ph; m_nCalls=0; m_nLogCalls=-1; m_iLogPrecision=6"; codeLang->OutCodeLine(&ofs,&ln);
	if(nSubsActual>1){ codeLang->AddOpForP1(&ln,varI,0,nSubsActual,-1,true); ln<<" "<<varSubWgt<<"["<<varI<<"]=0.; "; codeLang->AddOpEndFor(&ln); ln<<" "<<varSubWgtSum<<"=0.;"; codeLang->OutCodeLine(&ofs,&ln,-1,true); }
	codeLang->DecIndent(); ln<<codeLang->GetOpEndProc(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
	//
	if(m_bEstimatedIsStatic){ ln<<codeLang->GetTypePrefixReal()<<" "<<sClassProc.getData()<<"::m_fCrossSectionExpected_pb = -1."; codeLang->OutCodeLine(&ofs,&ln); }
	//
	ln<<codeLang->GetTypePrefixReal()<<" "<<sClassProc.getData()<<"::"<<sClassProcGenFunc.getData()<<"::Call(";
	ln<<codeLang->GetTypePrefixReal()<<"* params) {"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	ln<<"m_nCalls++"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<codeLang->GetOpIf()<<" ((m_ph!=NULL) && (m_proc!=NULL)) {"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	if(nSubsActual>1){
		//	m_subWgtSum=0.;
		//	for (long i=0; i<4; i++) {
		//		if (m_ph->FillEvent(params)) {
		//			tReal wgt=m_ph->m_PhSpaceWgt;
		//			if (wgt>0.) { wgt*=m_proc->CalcDiffCS(i, m_ph->GetLastEvent()); }
		//			m_subWgt[i]=wgt; m_subWgtSum+=wgt;
		//		} else { m_subWgt[i]=0.; }
		//	}
		ln<<varSubWgtSum<<"=0."; codeLang->OutCodeLine(&ofs,&ln);
		codeLang->StartCommentLine(&ofs); ofs << "\tNote: we don't need to fill channel-specific info here => use common 'FillEvent' call"<<"\n";
		ln<<codeLang->GetOpIf()<<" (m_ph->FillEvent(params)) "<<codeLang->GetOpThen(); codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
		codeLang->AddOpForP1(&ln,varI,0,nSubsActual,-1,true); codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
		ln<<codeLang->GetTypePrefixReal()<<" wgt=m_ph->m_PhSpaceWgt"; codeLang->OutCodeLine(&ofs,&ln);
		codeLang->StartCommentLine(&ofs,true); ofs << "Note: parton density is already included into 'ph.weight'\n";
		ln<<codeLang->GetOpIf()<<" (wgt>0.) "<<codeLang->GetOpThen()<<" "<<"wgt*=m_proc->CalcDiffCS("<<varI<<", m_ph->GetLastEvent()";
		int iDimX=0; if(m_beam1.bPartonsUsed){ ln<<", params["<<iDimX<<"]"; iDimX++; } if(m_beam2.bPartonsUsed){ ln<<", params["<<iDimX<<"]"; }
		ln<<"); "<<codeLang->GetOpEndIf();
		codeLang->OutCodeLine(&ofs,&ln,-1,true);
		ln<<varSubWgt<<"["<<varI<<"]=wgt; "<<varSubWgtSum<<"+=wgt"; codeLang->OutCodeLine(&ofs,&ln);
		codeLang->DecIndent(); codeLang->AddOpEndFor(&ln); codeLang->OutCodeLine(&ofs,&ln,-1,true);
		codeLang->DecIndent(); ln<<codeLang->GetOpElse(); codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
		codeLang->AddOpForP1(&ln,varI,0,nSubsActual,-1,true); ln<<" "<<varSubWgt<<"["<<varI<<"]=0.; }";codeLang->OutCodeLine(&ofs,&ln,-1,true);
		codeLang->DecIndent(); ln<<codeLang->GetOpEndIf(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
		ln<<codeLang->GetOpIf()<<" (m_nCalls<m_nLogCalls) "<<codeLang->GetOpThen(); codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
		ln<<"int precOld=std::cout.precision(m_iLogPrecision)"; codeLang->OutCodeLine(&ofs,&ln);
		codeLang->AddOpForP1(&ln,varI,0,nSubsActual,-1,true); ln<<" std::cout<<\"Sub-weight#\"<<"<<varI<<"<<\": \"; CVMUtil::Cout(&std::cout,"<<varSubWgt<<"["<<varI<<"]); std::cout<<std::endl; }";
		codeLang->OutCodeLine(&ofs,&ln,-1,true);
		ln<<"std::cout<<\"Event #\"<<m_nCalls<<\" weight: \"; CVMUtil::Cout(&std::cout,"<<varSubWgtSum<<"); std::cout<<std::endl"; codeLang->OutCodeLine(&ofs,&ln);
		ln<<"std::cout.precision(precOld)"; codeLang->OutCodeLine(&ofs,&ln);
		codeLang->DecIndent(); ln<<codeLang->GetOpEndIf(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
		ln<<"return "<<varSubWgtSum; codeLang->OutCodeLine(&ofs,&ln);
	}else{
		ln<<codeLang->GetOpIf()<<" (m_ph->FillEvent(params)) "<<codeLang->GetOpThen(); codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
		ln<<codeLang->GetTypePrefixReal()<<" wgt=m_ph->m_PhSpaceWgt"; codeLang->OutCodeLine(&ofs,&ln);
		ln<<codeLang->GetOpIf()<<" (wgt>0.) { wgt*=m_proc->CalcDiffCS(m_ph->GetLastEvent()";
		int iDimX=0; if(m_beam1.bPartonsUsed){ ln<<", params["<<iDimX<<"]"; iDimX++; } if(m_beam2.bPartonsUsed){ ln<<", params["<<iDimX<<"]"; }
		ln<<"); }"; codeLang->OutCodeLine(&ofs,&ln,-1,true);
		ln<<codeLang->GetOpIf()<<" (m_nCalls<m_nLogCalls) "<<codeLang->GetOpThen(); codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
		ln<<"int precOld=std::cout.precision(m_iLogPrecision)"; codeLang->OutCodeLine(&ofs,&ln);
		ln<<"std::cout<<\"Event #\"<<m_nCalls<<\" weight: \"<<wgt<<std::endl"; codeLang->OutCodeLine(&ofs,&ln);
		ln<<"std::cout.precision(precOld)"; codeLang->OutCodeLine(&ofs,&ln);
		codeLang->DecIndent(); ln<<codeLang->GetOpEndIf(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
		ln<<"return wgt"; codeLang->OutCodeLine(&ofs,&ln);
		codeLang->DecIndent(); ln<<codeLang->GetOpEndIf(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
	}
	codeLang->DecIndent(); ln<<codeLang->GetOpElse(); codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	ln<<"std::cout<<\"Error: ph: \"<<(m_ph!=NULL)<<\", proc: \"<<(m_proc!=NULL)<<std::endl"; codeLang->OutCodeLine(&ofs,&ln);
	codeLang->DecIndent(); ln<<codeLang->GetOpEndIf(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
	ln<<codeLang->GetOpIf()<<" (m_ph!=NULL) m_ph->IncEvtsFailed()"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<"return 0."; codeLang->OutCodeLine(&ofs,&ln);
	codeLang->DecIndent(); ln<<codeLang->GetOpEndProc(); codeLang->OutCodeLine(&ofs,&ln,-1,true);

	//	CVMGen::CVMGenFunc* CProcMy::CProcMyGenFunc::Clone() const { return new CProcMyGenFunc(m_proc,m_ph); }
	ln<<CCodeProcessImpl::CN_CVMGenFunc<<"* "<<sClassProc.getData()<<"::"<<sClassProcGenFunc.getData()<<"::Clone() const { return new ";
	ln<<sClassProcGenFunc.getData()<<"(m_proc,m_ph); }"; codeLang->OutCodeLine(&ofs,&ln,-1,true);

	if(nSubsActual>1){
		//	void CProcMy::GetPartonCodes(int iSub, int &iPart1, int &iPart2); //for multichannel process only
		// function is not currently used
		/*
		ln<<"void "<<sClassProc.getData()<<"::"<<sClassProcGenFunc.getData()<<"GetPartonCodes("<<codeLang->GetTypePrefixInt()<<" iSub,"
			<<codeLang->GetTypePrefixInt()<<" &iPart1"<<<<codeLang->GetTypePrefixInt()<<" &iPart2) {"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
		//
		ln<<"switch(iSub) {"; codeLang->OutCodeLine(&ofs,&ln,-1,true);
		int iSub=0;
		for(i=0;i<szSubs;i++){
			int pk1=phys->GetPDGParticleCode(SubProc(i)->pk(0)); int pk2=phys->GetPDGParticleCode(SubProc(i)->pk(1));
			//direct order
			ln<<"case "<<iSub<<": iPart1="<<pk1<<"; iPart2="<<pk2; codeLang->OutCodeLine(&ofs,&ln);
			iSub++;
			//backward order (if 'auto-swap' is allowed + if differs from direct order)
			if(m_bAutoSwapPartons && m_beam1.bPartonsUsed && m_beam2.bPartonsUsed && (pk1!=pk2)) {
				ln<<"case "<<iSub<<": iPart1="<<pk2<<"; iPart2="<<pk1; codeLang->OutCodeLine(&ofs,&ln);
				iSub++;
			}
		}
		ln<<"default: iPart1=0; iPart2=0"; codeLang->OutCodeLine(&ofs,&ln);
		ln<<"}"; codeLang->OutCodeLine(&ofs,&ln,-1,true);
		codeLang->DecIndent(); ln<<codeLang->GetOpEndProc(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
		*/

		//	int CProcMy::SelectChannel(tReal* params); //for multichannel process only
		ln<<codeLang->GetTypePrefixInt()<<" "<<sClassProc.getData()<<"::"<<sClassProcGenFunc.getData()<<"::SelectChannel("<<codeLang->GetTypePrefixReal()<<"* params) {";
		codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
		codeLang->StartCommentLine(&ofs); ofs << "\tTo be called by 'ph->GenEvent' procedure"<<"\n";
		ln<<codeLang->GetOpIf()<<" ((m_ph!=NULL) && (params!=NULL)) {"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
		ln<<codeLang->GetOpIf()<<" (!m_ph->AreLastParams(params)) { Call(params); }"; codeLang->OutCodeLine(&ofs,&ln,-1,true);
		ln<<codeLang->GetOpIf()<<" ("<<varSubWgtSum<<" > 0.) {"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
		ln<<CCodeProcessImpl::CN_CVMRnd<<"* rnd = "<<CCodeProcessImpl::CN_CVMRnd<<"::GetRndRef()"; codeLang->OutCodeLine(&ofs,&ln);
		ln<<codeLang->GetOpIf()<<" (rnd != NULL) {"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
		if(nSubsActual>2){
			ln<<codeLang->GetTypePrefixReal()<<" tmp=0.; "<<codeLang->GetTypePrefixReal()<<" tmpThr="<<varSubWgtSum<<"*(rnd->Rnd())"; codeLang->OutCodeLine(&ofs,&ln);
			codeLang->AddOpForP1(&ln,varI,0,nSubsActual-1,-1,true); codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
			ln<<"tmp+="<<varSubWgt<<"["<<varI<<"]"; codeLang->OutCodeLine(&ofs,&ln);
			ln<<codeLang->GetOpIf()<<" (tmp >= tmpThr) { return "<<varI<<"; }"; codeLang->OutCodeLine(&ofs,&ln,-1,true);
			codeLang->DecIndent(); codeLang->AddOpEndFor(&ln); codeLang->OutCodeLine(&ofs,&ln,-1,true);
			ln<<"return "<<nSubsActual-1; codeLang->OutCodeLine(&ofs,&ln);
		}else{ //nSubsActual==2
			ln<<"return (("<<varSubWgt<<"[0] >= "<<varSubWgtSum<<"*(rnd->Rnd()))?0:1)"; codeLang->OutCodeLine(&ofs,&ln);
		}
		codeLang->DecIndent(); ln<<codeLang->GetOpEndIf(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
		codeLang->DecIndent(); ln<<codeLang->GetOpEndIf(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
		codeLang->DecIndent(); ln<<codeLang->GetOpEndIf(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
		ln<<"return -1"; codeLang->OutCodeLine(&ofs,&ln);
		codeLang->DecIndent(); ln<<codeLang->GetOpEndProc(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
	}

	//	CProcMy::CProcMy():CHEPProcessLite() {
	//		m_gen=NULL;
	//		m_ph=NULL;
	//	}
	ln<<sClassProc.getData()<<"::"<<sClassProc.getData()<<"("<<codeLang->GetTypePrefixInt()<<" idProc) : "<<CCodeProcessImpl::CN_CHEPProcessLite<<"() {"; codeLang->OutCodeLine(&ofs,&ln,-1,true);
	codeLang->IncIndent();
	//ln<<"m_gen=NULL"; codeLang->OutCodeLine(&ofs,&ln); //Note: we use m_ph-internal generator
	ln<<"m_idProc = idProc"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<"m_ph=NULL"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<"m_func=NULL"; codeLang->OutCodeLine(&ofs,&ln);
	//[unused] if(m_beam1.bPartonsUsed) ln<<"pdf1=NULL"; codeLang->OutCodeLine(&ofs,&ln);
	//[unused] if(m_beam2.bPartonsUsed) ln<<"pdf2=NULL"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<"m_InternalPartOrder=new "<<CCodeProcessImpl::CN_CHEPParticleOrder<<"("<<nFinals+iFirstFinal<<"); m_bInternalPartOrderUsed=false"; codeLang->OutCodeLine(&ofs,&ln);
	codeLang->DecIndent(); ln<<codeLang->GetOpEndProc(); codeLang->OutCodeLine(&ofs,&ln,-1,true);

	//	CProcMy::~CProcMy() {
	//		if(m_ph!=NULL)delete m_ph;
	//		if(m_gen!=NULL)delete m_gen;
	//	}
	ln<<sClassProc.getData()<<"::~"<<sClassProc.getData()<<"() {"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	ln<<codeLang->GetOpIf()<<"(m_ph!=NULL)delete m_ph"; codeLang->OutCodeLine(&ofs,&ln);
	//ln<<codeLang->GetOpIf()<<"(m_gen!=NULL)delete m_gen"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<codeLang->GetOpIf()<<"(m_func!=NULL)delete m_func"; codeLang->OutCodeLine(&ofs,&ln);
	//[unused] if(m_beam1.bPartonsUsed) { ln<<codeLang->GetOpIf()<<"(pdf1!=NULL)delete pdf1"; codeLang->OutCodeLine(&ofs,&ln); }
	//[unused] if(m_beam2.bPartonsUsed) { ln<<codeLang->GetOpIf()<<"(pdf2!=NULL)delete pdf2"; codeLang->OutCodeLine(&ofs,&ln); }
	ln<<codeLang->GetOpIf()<<"(m_InternalPartOrder!=NULL)delete m_InternalPartOrder"; codeLang->OutCodeLine(&ofs,&ln);
	codeLang->DecIndent(); ln<<codeLang->GetOpEndProc(); codeLang->OutCodeLine(&ofs,&ln,-1,true);

	// void SetPDFs()
	if(m_beam1.bPartonsUsed || m_beam2.bPartonsUsed){
		ln<<"void "<<sClassProc.getData()<<"::SetPDFs("; bool bComma=false; if(m_beam1.bPartonsUsed){ ln<<"const "<<CCodeProcessImpl::CN_CPP_PDF<<"* _pdf1"; bComma=true;}
		if(m_beam2.bPartonsUsed){ if(bComma){ ln<<", "; } ln<<"const "<<CCodeProcessImpl::CN_CPP_PDF<<"* _pdf2"; } ln<<") {"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
		if(m_beam1.bPartonsUsed) {
			ln<<codeLang->GetOpIf()<<"(_pdf1!=NULL) beams[0].SetPDFConst(_pdf1)"; codeLang->OutCodeLine(&ofs,&ln);
		}
		if(m_beam2.bPartonsUsed) {
			ln<<codeLang->GetOpIf()<<"(_pdf2!=NULL) beams[1].SetPDFConst(_pdf2)"; codeLang->OutCodeLine(&ofs,&ln);
		}
		codeLang->DecIndent(); ln<<codeLang->GetOpEndProc(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
	}

	//	tReal CalcDiffCS(CHEPEvtLite* evt) {
	//		tReal wgt = m_cuts.GetScale(evt);
	//		if(wgt>0.)wgt*=m_proc_impl.sClassImplCall(evt);
	//		return wgt;
	//	}
	ln<<codeLang->GetTypePrefixReal()<<" "<<sClassProc.getData()<<"::CalcDiffCS("; if(nSubsActual>1){ ln<<codeLang->GetTypePrefixInt()<<" iSub, "; }
	ln<<CCodeProcessImpl::CN_CPP_EVT_CLASS<<"* evt";
	if(m_beam1.bPartonsUsed) { ln<<", "<<codeLang->GetTypePrefixReal()<<" x1"; }
	if(m_beam2.bPartonsUsed) { ln<<", "<<codeLang->GetTypePrefixReal()<<" x2"; }
	ln<<") {"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	//
	// 1. Check cuts
	int iStep=1; codeLang->StartCommentLine(&ofs); ofs<<iStep<<". Check cuts\n"; iStep++;
	ln<<codeLang->GetTypePrefixReal()<<" wgt = m_cutsLocal.GetScale(evt)"; codeLang->OutCodeLine(&ofs,&ln);
	ofs<<"\n";
	//
	// 2. Apply PDFs
	if(m_beam1.bPartonsUsed || m_beam2.bPartonsUsed) {
		codeLang->StartCommentLine(&ofs); ofs<<iStep<<". Apply PDFs\n"; iStep++;
		ln<<codeLang->GetOpIf()<<" (wgt>0.) "<<codeLang->GetOpThen(); codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
		ln<<codeLang->GetTypePrefixInt()<<" id1, id2"; codeLang->OutCodeLine(&ofs,&ln);
		ln<<codeLang->GetOpIf()<<"(GetChannelInfo("<<((nSubsActual>1)?"iSub":"0")<<", id1, id2)) "<<codeLang->GetOpThen();
		codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
		CEString sR8_Rconv1; CEString sR8_Rconv2;
		if(!codeLang->GetTypeConversion(CEAbstractFunction::PAR_REAL8,codeLang->GetDefaultRealTypeId(),&sR8_Rconv1,&sR8_Rconv2)) {
			std::cerr<<"[Internal error] Can't find type conversion header"<<std::endl; sR8_Rconv1.Reset(); sR8_Rconv1<<"("; sR8_Rconv2.Reset(); sR8_Rconv1<<")";
		}
		if(m_beam1.bPartonsUsed) {
			ln<<codeLang->GetOpIf()<<"(id1==PDG_KF_GLUON) id1=0"; codeLang->OutCodeLine(&ofs,&ln);
			ln<<codeLang->GetOpIf()<<"(beams[0].pdf != NULL) wgt*="<<sR8_Rconv1.GetBuffer()
				<<"beams[0].pdf->GetDensity(id1, x1, beams[0].scaleQ2)"<<sR8_Rconv2.GetBuffer(); codeLang->OutCodeLine(&ofs,&ln);
		}
		if(m_beam2.bPartonsUsed) {
			ln<<codeLang->GetOpIf()<<"(id2==PDG_KF_GLUON) id2=0"; codeLang->OutCodeLine(&ofs,&ln);
			ln<<codeLang->GetOpIf()<<"(beams[1].pdf != NULL) wgt*="<<sR8_Rconv1.GetBuffer()
				<<"beams[1].pdf->GetDensity(id2, x2, beams[1].scaleQ2)"<<sR8_Rconv2.GetBuffer(); codeLang->OutCodeLine(&ofs,&ln);
		}
		codeLang->DecIndent(); ln<<codeLang->GetOpElse(); codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
		ln<<"wgt=0."; codeLang->OutCodeLine(&ofs,&ln);
		codeLang->DecIndent(); ln<<codeLang->GetOpEndIf(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
		codeLang->DecIndent(); ln<<codeLang->GetOpEndIf(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
		ofs<<"\n";
	}
	//
	//3. Calculate cross section / Calculate parton-level cross section
	codeLang->StartCommentLine(&ofs); ofs<<iStep<<". Calculate "<<((m_beam1.bPartonsUsed||m_beam2.bPartonsUsed)?"parton-level ":"")<<"cross section\n"; iStep++;
	if(nSubsActual>1){
		ln<<codeLang->GetOpIf()<<" (wgt>0.) "<<codeLang->GetOpThen(); codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
		ln<<"switch(iSub) {"; codeLang->OutCodeLine(&ofs,&ln,-1,true);
		int iSub=0;
		for(i=0;i<szSubs;i++){
			if(m_bAutoSwapPartons && m_beam1.bPartonsUsed && m_beam2.bPartonsUsed){
				if(SubProc(i)->pk.GetAt(0)!=SubProc(i)->pk.GetAt(1)) {
					//direct order
					ln<<"case "<<iSub<<": wgt*=m_proc_impl_"<<i<<"."<<SubProc(i)->sImplCallB.getData()<<"(evt,1); break"; codeLang->OutCodeLine(&ofs,&ln);
					iSub++;
					//backward order
					ln<<"case "<<iSub<<": wgt*=m_proc_impl_"<<i<<"."<<SubProc(i)->sImplCallB.getData()<<"(evt,2); break"; codeLang->OutCodeLine(&ofs,&ln);
					iSub++;
				}else{
					//direct order == backward order
					ln<<"case "<<iSub<<": wgt*=m_proc_impl_"<<i<<"."<<SubProc(i)->sImplCallB.getData()<<"(evt); break"; codeLang->OutCodeLine(&ofs,&ln);
					iSub++;
				}
			}else{
				ln<<"case "<<iSub<<": wgt*=m_proc_impl_"<<i<<"."<<SubProc(i)->sImplCallB.getData()<<"(evt); break"; codeLang->OutCodeLine(&ofs,&ln);
				iSub++;
			}
		}
		ln<<"default: wgt=0."; codeLang->OutCodeLine(&ofs,&ln);
		ln<<"}"; codeLang->OutCodeLine(&ofs,&ln,-1,true);
		codeLang->DecIndent(); ln<<codeLang->GetOpEndIf(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
	}else{
		ln<<codeLang->GetOpIf()<<"(wgt>0.) wgt*=m_proc_impl."<<SubProc(0)->sImplCallB.getData()<<"(evt)"; codeLang->OutCodeLine(&ofs,&ln);
	}
	ln<<"return wgt"; codeLang->OutCodeLine(&ofs,&ln);
	codeLang->DecIndent(); ln<<codeLang->GetOpEndProc(); codeLang->OutCodeLine(&ofs,&ln,-1,true);

	if(m_beam1.bPartonsUsed || m_beam2.bPartonsUsed) {
		ln<<codeLang->GetTypePrefixBool()<<" "<<sClassProc.getData()<<"::GetChannelInfo("<<codeLang->GetTypePrefixInt()<<" iSub, "<<
			codeLang->GetTypePrefixInt()<<"& id1, "<<codeLang->GetTypePrefixInt()<<"& id2) {"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
		ln<<codeLang->GetTypePrefixBool()<<" res = true"; codeLang->OutCodeLine(&ofs,&ln);
		ln<<"switch(iSub) {"; codeLang->OutCodeLine(&ofs,&ln,-1,true);
		int iSub=0;
		for(i=0;i<szSubs;i++){
			int pk1=phys->GetPDGParticleCode(SubProc(i)->pk(0)); int pk2=phys->GetPDGParticleCode(SubProc(i)->pk(1));
			//direct order
			ln<<"case "<<iSub<<": id1="<<pk1<<"; id2="<<pk2<<"; break"; codeLang->OutCodeLine(&ofs,&ln); iSub++;
			//backward order (if 'auto-swap' is allowed + if differs from direct order)
			if(m_bAutoSwapPartons && m_beam1.bPartonsUsed && m_beam2.bPartonsUsed && (pk1!=pk2)) {
				ln<<"case "<<iSub<<": id1="<<pk2<<"; id2="<<pk1<<"; break"; codeLang->OutCodeLine(&ofs,&ln); iSub++;
			}
		}
		ln<<"default: res=false"; codeLang->OutCodeLine(&ofs,&ln); ln<<"}"; codeLang->OutCodeLine(&ofs,&ln,-1,true);
		ln<<"return res"; codeLang->OutCodeLine(&ofs,&ln);
		codeLang->DecIndent(); ln<<codeLang->GetOpEndProc(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
	}

	//	void CHEPProcessLiteInst::AddCut(const CHEPEvtLiteCut* cut) {
	//		m_cuts.AddCut(cut);
	//	}
	ln<<"void "<<sClassProc.getData()<<"::AddCut(const "<<CCodeProcessImpl::CN_CHEPEvtLiteCut<<"* cut) { m_cuts.AddCut(cut); }";
	codeLang->OutCodeLine(&ofs,&ln,-1,true);

	//	bool CHEPProcessLiteInst::Prepare(const CVM4Vec* p1, const CVM4Vec* p2) {
	//		//set beams
	//		//temporary mass/id arrays
	//		//select particle order in ph
	//		//call ph->Init()
	//		return false;
	//	}
	ln<<codeLang->GetTypePrefixBool()<<" "<<sClassProc.getData()<<"::Prepare(const "<<CCodeProcessImpl::CN_CPP_4VEC_CLASS<<"* p1, const "
		<<CCodeProcessImpl::CN_CPP_4VEC_CLASS<<"* p2, double fPrecision) {"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	ofs<<"#include \""<<sConstFileName.GetBuffer()<<"\"\n"; //no indent for include line
	ln<<codeLang->GetOpIf()<<"(m_ph!=NULL)delete m_ph"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<"m_ph = new "<<CCodeProcessImpl::CN_CVMPhaseSpaceGen<<"()"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<codeLang->GetOpIf()<<"((p1==NULL)||(p2==NULL)||(m_ph==NULL))return false"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<codeLang->GetOpIf()<<"(m_func==NULL)m_func=new "<<sClassProcGenFunc.getData()<<"(this,m_ph)";
	codeLang->OutCodeLine(&ofs,&ln); ln<<codeLang->GetOpIf()<<"(m_func==NULL)return false"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<codeLang->GetOpIf()<<"(m_InternalPartOrder==NULL)return false"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<codeLang->GetTypePrefixInt()<<" "<<varI; codeLang->OutCodeLine(&ofs,&ln);
	//
	ofs<<"\n";

	ln<<codeLang->GetOpIf()<<"((fPrecision<=0.)||(fPrecision>0.01)) fPrecision=0.01"; codeLang->OutCodeLine(&ofs,&ln);
	ofs<<"\n";

	// algorithm object
	ln<<CCodeProcessImpl::CN_CVMPhaseSpaceGen<<"::CAlgorithmInfo alg"; codeLang->OutCodeLine(&ofs,&ln);
	//codeLang->StartCommentLine(&ofs,true); ofs<<CCodeProcessImpl::CN_CVMPhaseSpaceGen<<"::CAlgorithmInfo alg = "
	//	<<CCodeProcessImpl::CN_CVMPhaseSpaceGen<<"::CAlgorithmInfo::Alg_UseLastThetaLimits(-1.,1.);"<<"\n";
	ln<<"alg.m_bAddNuclonsToEvent="<<(m_bUseEvtNuclonEntries?"true":"false"); codeLang->OutCodeLine(&ofs,&ln);
	ofs<<"\n";

	// Select particle order used in proc-class
	//	- No cuts on last-generated particle
	//	- [TODO] probably exchange initials to fit Q=InLast-FinLast condition
	//	Note: SetOfCuts{,SetOfCuts{,SetOfCuts,},} is allowed but not parsed for PH-optimization
	//
	codeLang->StartCommentLine(&ofs,false); ofs<<"Fill 'alg.fPartFinF0eMinCuts' and select particle order for internal use\n";
	ln<<"m_bInternalPartOrderUsed=false"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<"m_InternalPartOrder->ResetOrder()"; codeLang->OutCodeLine(&ofs,&ln);
	//Nt:
	// old: local 'eMinCut','eMaxCut' and 'bUseECuts'
	// new: 'alg.fPartFinF0eMinCuts', 'alg.fPartFinF0eMaxCuts' and 'alg.bUsePartFinF0eCuts' with no pre-init
	//[old]ln<<codeLang->GetTypePrefixReal()<<" eMinCut["<<nFinals<<"],eMaxCut["<<nFinals<<"];"<<codeLang->GetTypePrefixBool()<<" bUseECuts=false";codeLang->OutCodeLine(&ofs,&ln);
	//[old]codeLang->AddOpFor(&ln,varI,0,nFinals-1); ln<<" eMinCut["<<varI<<"]=-1.; eMaxCut["<<varI<<"]=-1.; "; codeLang->AddOpEndFor(&ln);
	codeLang->OutCodeLine(&ofs,&ln,-1,true);
	ln<<"m_cutsLocal.CopyFrom(&m_cuts)"; codeLang->OutCodeLine(&ofs,&ln);
	codeLang->AddOpForP1(&ln,varI,0,"m_cutsLocal.GetNCuts()"); codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	ln<<CCodeProcessImpl::CN_CHEPEvtLiteCut<<"* cut=m_cutsLocal.GetCut("<<varI<<"); "
		<<codeLang->GetOpIf()<<"(cut==NULL)continue"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<codeLang->GetOpIf()<<"(cut->ClassName()=="<<CCodeProcessImpl::CN_CHEPEvtLiteCutE<<"::CLASS_NAME) {";
	codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	ln<<codeLang->GetTypePrefixInt()<<" idFin=(("<<CCodeProcessImpl::CN_CHEPEvtLiteCutE<<"*)cut)->GetId()-"<<iFirstFinal; codeLang->OutCodeLine(&ofs,&ln);
	ln<<codeLang->GetOpIf()<<"((idFin<0)||(idFin>="<<nFinals<<"))continue"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<codeLang->GetTypePrefixReal()<<" eMin=(("<<CCodeProcessImpl::CN_CHEPEvtLiteCutE<<"*)cut)->GetEMin()"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<codeLang->GetTypePrefixReal()<<" eMax=(("<<CCodeProcessImpl::CN_CHEPEvtLiteCutE<<"*)cut)->GetEMax()"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<codeLang->GetOpIf()<<"((eMin>0.)&&(eMin>alg.fPartFinF0eMinCuts[idFin])) {"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	ln<<"alg.fPartFinF0eMinCuts[idFin]=eMin; alg.bUsePartFinF0eCuts=true"; codeLang->OutCodeLine(&ofs,&ln);
	codeLang->DecIndent(); ln<<codeLang->GetOpEndIf(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
	ln<<codeLang->GetOpIf()
		<<"((eMax>0.)&&((alg.fPartFinF0eMaxCuts[idFin]<0.)||(eMax<alg.fPartFinF0eMaxCuts[idFin]))) {"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	ln<<"alg.fPartFinF0eMaxCuts[idFin]=eMax; alg.bUsePartFinF0eCuts=true"; codeLang->OutCodeLine(&ofs,&ln);
	codeLang->DecIndent(); ln<<codeLang->GetOpEndIf(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
	codeLang->OutCodeLine(&ofs,&ln,-1,true);
	codeLang->DecIndent(); ln<<codeLang->GetOpEndIf(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
	codeLang->DecIndent(); codeLang->AddOpEndFor(&ln); codeLang->OutCodeLine(&ofs,&ln,-1,true);
	ln<<codeLang->GetOpIf()<<"((alg.fPartFinF0eMinCuts[0]>0.)||(alg.fPartFinF0eMaxCuts[0]>0.)) {"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	codeLang->StartCommentLine(&ofs,false); ofs<<"More efficient is: to avoid energy cuts on the 'first' final particle\n";
	//
	codeLang->AddOpFor(&ln,varI,1,nFinals-1); codeLang->OutCodeLine(&ofs,&ln,-1,true,"search for other particle without E-cuts"); codeLang->IncIndent();
	ln<<codeLang->GetOpIf()<<"((alg.fPartFinF0eMinCuts["<<varI<<"]<0.)&&(alg.fPartFinF0eMaxCuts["<<varI<<"]<0.)) {";
	codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	ln<<codeLang->GetTypePrefixReal()<<" tmpE=alg.fPartFinF0eMinCuts[0]; alg.fPartFinF0eMinCuts[0]=alg.fPartFinF0eMinCuts["<<varI<<"]; alg.fPartFinF0eMinCuts["<<varI<<"]=tmpE";
	codeLang->OutCodeLine(&ofs,&ln);
	ln<<"tmpE=alg.fPartFinF0eMaxCuts[0]; alg.fPartFinF0eMaxCuts[0]=alg.fPartFinF0eMaxCuts["<<varI<<"]; alg.fPartFinF0eMaxCuts["<<varI<<"]=tmpE"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<"m_InternalPartOrder->Swap("<<iFirstFinal<<","<<iFirstFinal<<"+"<<varI<<")"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<"m_bInternalPartOrderUsed=true"; codeLang->OutCodeLine(&ofs,&ln);
	codeLang->DecIndent(); ln<<codeLang->GetOpEndIf(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
	codeLang->DecIndent(); codeLang->AddOpEndFor(&ln); codeLang->OutCodeLine(&ofs,&ln,-1,true);
	codeLang->DecIndent(); ln<<codeLang->GetOpEndIf(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
	//
	ln<<codeLang->GetOpIf()<<"(m_bInternalPartOrderUsed) "<<codeLang->GetOpThen(); codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	ln<<"m_cutsLocal.ChangeParticleOrder(m_InternalPartOrder)"; codeLang->OutCodeLine(&ofs,&ln);
	//log internal order if changed
	ln<<"std::cout<<\"[\"<<ProcName()<<\"] Internal particle order:\""; codeLang->OutCodeLine(&ofs,&ln);
	codeLang->AddOpFor(&ln,varI,0,nFinals-1); codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	ln<<"std::cout<<\" \"<<"<<varI<<"<<\"->\"<<m_InternalPartOrder->Get("<<iFirstFinal<<"+"<<varI<<")"; codeLang->OutCodeLine(&ofs,&ln);
	codeLang->DecIndent(); codeLang->AddOpEndFor(&ln); codeLang->OutCodeLine(&ofs,&ln,-1,true);
	ln<<"std::cout<<std::endl"; codeLang->OutCodeLine(&ofs,&ln);
	codeLang->DecIndent(); ln<<codeLang->GetOpElse(); codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	ln<<"std::cout<<\"[\"<<ProcName()<<\"] Internal particle order unchanged\"<<std::endl"; codeLang->OutCodeLine(&ofs,&ln);
	codeLang->DecIndent(); ln<<codeLang->GetOpEndIf(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
	ofs<<"\n";

	// Update order in sub-processes
	// - if 'm_bUseEvtNuclonEntries'
	// - if order is changed
	// - but NOT for initial parton swap cases
	bool bUpdOrderIf=((!m_bUseEvtNuclonEntries) && (iFirstFinal==2)); //else update always; last condition is ambigous (self-check)
	if(bUpdOrderIf) { ln<<codeLang->GetOpIf()<<"(m_bInternalPartOrderUsed) "<<codeLang->GetOpThen(); codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent(); }
	codeLang->StartCommentLine(&ofs,false); ofs<<"Update particle positions in implementation classes"<<(bUpdOrderIf?" (if required)":"")<<"\n";
	for(i=0;i<szSubs;i++) {
		if(nSubsActual>1){ ln<<"m_proc_impl_"<<i; }else{ ln<<"m_proc_impl"; } ln<<".SetPartOrder(";
		ln<<iPosInitials[0]<<", "<<iPosInitials[1]<<","; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
		for(j=0;j<nFinals;j++) {
			if(j>0){ ln<<","; codeLang->OutCodeLine(&ofs,&ln,-1,true); }
			ln<<"m_InternalPartOrder->Get("<<(iFirstFinal+j)<<")";
		}
		ln<<")"; codeLang->OutCodeLine(&ofs,&ln); codeLang->DecIndent();
	}
	if(bUpdOrderIf) { codeLang->DecIndent(); ln<<codeLang->GetOpEndIf(); codeLang->OutCodeLine(&ofs,&ln,-1,true); }
	ofs<<"\n";

	//codeLang->StartCommentLine(&ofs,false); ofs<<"Fill 'alg.fPartFinF0eMinCuts' and select particle order for internal use\n";
	//if(m_bUseEvtNuclonEntries) { if(m_beam1.bPartonsUsed)iFirstFinalPos++; if(m_beam2.bPartonsUsed)iFirstFinalPos++; }
	//proc->m_iFirstFinal=iFirstFinalPos;
	//	//SetPartOrder

	//
	ln<<codeLang->GetTypePrefixReal()<<" "<<varMassFin<<"["<<nFinals<<"] = {";
	for(i=0; i<(nFinals); i++) { //Nt: in kinematics and 'proc->pk' N[initials]==2 always (and may be != iFirstFinal)
		if(i>0)ln<<", "; long pk = SubProc(0)->pk(2+i);
		if(CEBasic::isZero(phys->get_mass_sym(pk))) {
			ln<<"0";
		}else{ //for massive particles only
			const char* mSym=CLFileManager::RequestParticleMassIncluded(file_id,pk);
			ln<<((mSym!=NULL)?mSym:"ERROR2");
		}
	}
	ln<<"}"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<codeLang->GetTypePrefixInt()<<" "<<varPIDsFin<<"["<<nFinals<<"] = {";
	for(i=0; i<(nFinals); i++) { //Nt: in kinematics N[initials]==2 always (and may be != iFirstFinal)
		if(i>0)ln<<", "; ln<<phys->GetPDGParticleCode(SubProc(0)->pk(2+i));
	}
	ln<<"}"; codeLang->OutCodeLine(&ofs,&ln);
	// beams init
	ofs<<"\n";
	for(i=0; i<2; i++){
		codeLang->StartCommentLine(&ofs,false); ofs<<"Beam #"<<i<<"\n";
		int pkBeam=((i==0)?(m_beam1.pk):(m_beam2.pk)); //don't use 'SubProc(0)->pk(i)' here
		ln<<"beams["<<i<<"].id = "<<phys->GetPDGParticleCode(m_beam1.pk); codeLang->OutCodeLine(&ofs,&ln);
		ln<<"beams["<<i<<"].mass = ";
		if(CEBasic::isZero(phys->get_mass_sym(pkBeam))) {
			ln<<"0";
		}else{
			const char* mSym=CLFileManager::RequestParticleMassIncluded(file_id,pkBeam); ln<<((mSym!=NULL)?mSym:"ERROR2");
		}
		codeLang->OutCodeLine(&ofs,&ln);
		ln<<"beams["<<i<<"].p = "<<((i==0)?"*p1":"*p2"); codeLang->OutCodeLine(&ofs,&ln);
		//
		if(((i==0) && m_beam1.bPartonsUsed) || ((i==1) && m_beam2.bPartonsUsed)) {
			ln<<"beams["<<i<<"].scaleQ2 = 100."; codeLang->OutCodeLine(&ofs,&ln);
			ln<<"beams["<<i<<"].bUsePartons = true"; codeLang->OutCodeLine(&ofs,&ln);
			ln<<codeLang->GetOpIf()<<"(beams["<<i<<"].pdf==NULL) beams["<<i<<"].pdf = "<<CCodeProcessImpl::CN_CPP_PDF
				<<"::Create(4, 46, beams["<<i<<"].id)"; codeLang->OutCodeLine(&ofs,&ln,-1,false,"use CTEQ5L from PYTHIA8");
			ln<<codeLang->GetOpIf()<<"(beams["<<i<<"].pdf==NULL) return false"; codeLang->OutCodeLine(&ofs,&ln,-1,false,"Internal error");
		}
		ofs<<"\n";
	}
	//
	if(m_beam1.bPartonsUsed || m_beam2.bPartonsUsed) {
		ln<<codeLang->GetTypePrefixBool()<<" bRes = m_ph->InitMultiChannel("
			<<"beams[0],beams[1],"<<nFinals<<","<<varMassFin<<","<<varPIDsFin<<",alg,m_func)";
		codeLang->OutCodeLine(&ofs,&ln);

		codeLang->AddOpForP1(&ln,varI,0,nSubsActual,-1); codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
		ln<<codeLang->GetTypePrefixInt()<<" id1=0; "<<codeLang->GetTypePrefixInt()<<" id2=0; ";
		ln<<"GetChannelInfo("<<varI<<", id1, id2); ";
		ln<<"m_ph->AddChannel("<<varI<<", id1, id2)"; codeLang->OutCodeLine(&ofs,&ln);
		codeLang->DecIndent(); codeLang->AddOpEndFor(&ln); codeLang->OutCodeLine(&ofs,&ln,-1,true);

		/*[old]
		int iSub=0;
		for(i=0;i<szSubs;i++){
			int pk1=phys->GetPDGParticleCode(SubProc(i)->pk(0)); int pk2=phys->GetPDGParticleCode(SubProc(i)->pk(1));
			//direct order
			ln<<"m_ph->AddChannel("<<iSub<<", "<<pk1<<", "<<pk2<<")"; codeLang->OutCodeLine(&ofs,&ln); iSub++;
			//backward order (if 'auto-swap' is allowed + if differs from direct order)
			if(m_bAutoSwapPartons && m_beam1.bPartonsUsed && m_beam2.bPartonsUsed && (pk1!=pk2)) {
				ln<<"m_ph->AddChannel("<<iSub<<", "<<pk2<<", "<<pk1<<")"; codeLang->OutCodeLine(&ofs,&ln); iSub++;
			}
		}
		if(iSub<=0) { ofs<<"******* Internal ERROR! *******\n"; }
		*/
		ofs<<"\n";
	}else{
		ln<<codeLang->GetTypePrefixBool()<<" bRes = m_ph->Init("
			<<"beams[0],beams[1],"<<nFinals<<","<<varMassFin<<","<<varPIDsFin<<",alg,m_func)";
		codeLang->OutCodeLine(&ofs,&ln);
	}

	//
	ln<<CCodeProcessImpl::CN_CVMGen<<"* gen = m_ph->GetGenRef()"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<codeLang->GetOpIf()<<"(gen==NULL) bRes=false"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<codeLang->GetOpIf()<<"(bRes) {"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	ln<<codeLang->GetOpIf()<<"(m_fCrossSectionExpected_pb>0.)gen->SetExpectedIntegral(m_fCrossSectionExpected_pb)";
	codeLang->OutCodeLine(&ofs,&ln);
	ln<<"m_ph->m_nLogNFirstEvents = 10";
	codeLang->OutCodeLine(&ofs,&ln); //debug
	//[to code]codeLang->StartCommentLine(&ofs,true); ofs<<ln.GetBuffer()<<";\n"; ln.Reset();
	ln<<"m_ph->m_iLogEventsPrecision = 6";
	codeLang->OutCodeLine(&ofs,&ln); //debug
	//[to code]codeLang->StartCommentLine(&ofs,true); ofs<<ln.GetBuffer()<<";\n"; ln.Reset();
	ln<<"m_func->m_nLogCalls = m_ph->m_nLogNFirstEvents"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<"m_func->m_iLogPrecision = m_ph->m_iLogEventsPrecision"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<"m_ph->Integrate(fPrecision)"; codeLang->OutCodeLine(&ofs,&ln);
	codeLang->DecIndent(); ln<<codeLang->GetOpEndIf(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
	ln<<"m_ph->SetEventRefExternal(NULL)"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<"return bRes"; codeLang->OutCodeLine(&ofs,&ln);
	codeLang->DecIndent(); ln<<codeLang->GetOpEndProc(); codeLang->OutCodeLine(&ofs,&ln,-1,true);

	//	bool CHEPProcessLiteInst::GetCrossSection(tReal& cs) {
	//		if(m_gen==NULL)return false;
	//		sc=m_gen->GetIntegral();
	//		return (sc>=0.);
	//	}
	ln<<codeLang->GetTypePrefixBool()<<" "<<sClassProc.getData()<<"::GetCrossSection("<<codeLang->GetTypePrefixReal()<<"& cs, "
		<<codeLang->GetTypePrefixReal()<<"* err) const {"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	//ln<<codeLang->GetOpIf()<<"(m_gen==NULL)return false"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<codeLang->GetOpIf()<<"(m_ph==NULL)return false"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<CCodeProcessImpl::CN_CVMGen<<"* gen = m_ph->GetGenRef()"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<codeLang->GetOpIf()<<"(gen==NULL)return false"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<"cs=gen->GetIntegral()"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<codeLang->GetOpIf()<<"(err!=NULL) {"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	ln<<"*err=gen->GetIntegralError()"; codeLang->OutCodeLine(&ofs,&ln);
	codeLang->DecIndent(); ln<<codeLang->GetOpEndIf(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
	ln<<"return (cs>=0.)"; codeLang->OutCodeLine(&ofs,&ln);
	codeLang->DecIndent(); ln<<codeLang->GetOpEndProc(); codeLang->OutCodeLine(&ofs,&ln,-1,true);

	//	bool CHEPProcessLiteInst::GenEvent(CHEPEvtLite* evt) {
	//	}
	ln<<codeLang->GetTypePrefixBool()<<" "<<sClassProc.getData()<<"::GenEvent("<<CCodeProcessImpl::CN_CPP_EVT_CLASS<<"* evt) {";
	codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	//ln<<codeLang->GetOpIf()<<"((m_ph==NULL)||(m_gen==NULL))return false"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<codeLang->GetOpIf()<<"(m_ph==NULL)return false"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<"m_ph->SetEventRefExternal(evt)"; codeLang->OutCodeLine(&ofs,&ln,-1,false,"set event");
	ln<<codeLang->GetTypePrefixBool()<<" bRes = m_ph->GenEvent()"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<"m_ph->SetEventRefExternal(NULL)"; codeLang->OutCodeLine(&ofs,&ln,-1,false,"clear event ref");
	ln<<codeLang->GetOpIf()<<"(bRes) {"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	ln<<codeLang->GetOpIf()<<"(m_bInternalPartOrderUsed) {"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	ln<<"evt->ChangeOrder(m_InternalPartOrder)"; codeLang->OutCodeLine(&ofs,&ln);
	codeLang->DecIndent(); ln<<codeLang->GetOpEndIf(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
	ln<<"evt->idProc = m_idProc"; codeLang->OutCodeLine(&ofs,&ln);
	codeLang->DecIndent(); ln<<codeLang->GetOpEndIf(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
	ln<<"return bRes"; codeLang->OutCodeLine(&ofs,&ln);
	codeLang->DecIndent(); ln<<codeLang->GetOpEndProc(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
	//
	ln<<"const char* "<<sClassProc.getData()<<"::CLASS_NAME = \""<<sClassProc.getData()<<"\""; codeLang->OutCodeLine(&ofs,&ln);
	ln<<"const char* "<<sClassProc.getData()<<"::ProcName() const { return CLASS_NAME; }"; codeLang->OutCodeLine(&ofs,&ln,-1,true);
	ln<<"int "<<sClassProc.getData()<<"::ProcID() const { return m_idProc; }"; codeLang->OutCodeLine(&ofs,&ln,-1,true);
	ln<<"void "<<sClassProc.getData()<<"::PrintGeneratorStatistics() {"; codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	ln<<CCodeProcessImpl::CN_CVMGen<<"* gen = m_ph->GetGenRef()"; codeLang->OutCodeLine(&ofs,&ln);
	ln<<codeLang->GetOpIf()<<"(gen!=NULL)"<<codeLang->GetOpThen(); codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	ln<<"std::cout<<\"Event generation efficiency: \"<<gen->GetGenEventEfficiency()<<std::endl"; codeLang->OutCodeLine(&ofs,&ln);
	codeLang->DecIndent(); ln<<codeLang->GetOpElse(); codeLang->OutCodeLine(&ofs,&ln,-1,true); codeLang->IncIndent();
	ln<<"std::cout<<\"No generator statistics available\"<<std::endl"; codeLang->OutCodeLine(&ofs,&ln);
	codeLang->DecIndent(); ln<<codeLang->GetOpEndIf(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
	codeLang->DecIndent(); ln<<codeLang->GetOpEndProc(); codeLang->OutCodeLine(&ofs,&ln,-1,true);
	//
	ofs.close();

	return true;
}
