////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// MathFileParser.cpp: implementation of the CMathFileParser class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CMathFileParser::CMathFileParser(const char* _source) : CFileParserAdv(_source) {
	m_CommentsType=CFileParserAdv::COMMENTS_MMOD;
	m_bDontSkipLnBreaksIfNoSpacesFollows=true;

	m_operand_types_data->add(CParserSymbolsList::create_list(
		CFileParserAdv::OP_VARIABLE,
		CParserSymbolsList::LIST_SYMBOLS, "$_",
		CParserSymbolsList::LIST_NUMBERS, NULL, // numbers may be posterior
		CParserSymbolsList::READ_BY_STRING));
	m_operand_types_data->add(CParserSymbolsList::create_list(
		CFileParserAdv::OP_VALUE_NUMBER, CParserSymbolsList::LIST_NUMBERS, CParserSymbolsList::READ_BY_STRING));
	m_operand_types_data->add(CParserSymbolsList::create_list(CFileParserAdv::OP_END_OF_LINE, ";,", CParserSymbolsList::READ_BY_SYMBOL));
	m_operand_types_data->add(CParserSymbolsList::create_list(CFileParserAdv::OP_BRACKET_LEFT, "[({", CParserSymbolsList::READ_BY_SYMBOL));
	m_operand_types_data->add(CParserSymbolsList::create_list(CFileParserAdv::OP_BRACKET_RIGHT, ")]}", CParserSymbolsList::READ_BY_SYMBOL));

	//m_operand_types_data->add(CParserSymbolsList::create_list(CFileParserAdv::OP_OPERATION_LEFT, "=", CParserSymbolsList::READ_BY_SYMBOL));
	m_operand_types_data->add(CParserSymbolsList::create_list(CFileParserAdv::OP_OPERATION_LEFT, "=!", CParserSymbolsList::READ_BY_STRING));

	m_operand_types_data->add(CParserSymbolsList::create_list(CFileParserAdv::OP_OPERATION_RIGHT, "+-*.^/|&:", CParserSymbolsList::READ_BY_SYMBOL));
	// Note: '/'-operation appears as 'OP_OPERATION_RIGHT' after comments separation
	m_operand_types_data->add(CParserSymbolsList::create_list(CFileParserAdv::OP_VALUE_STRING, "\"", CParserSymbolsList::READ_BY_SYMBOL));
	//m_operand_types_data->add(CParserSymbolsList::create_list(CFileParserAdv::OP_COLON, ":", CParserSymbolsList::READ_BY_SYMBOL));
	//m_operand_types_data->add(CParserSymbolsList::create_list(CFileParserAdv::OP_SLASH, "/", CParserSymbolsList::READ_BY_SYMBOL));

	m_operand_types_data->add(CParserSymbolsList::create_list(CFileParserAdv::OP_SPECIAL_1, ">", CParserSymbolsList::READ_BY_SYMBOL));

	m_bUseRHand2Equs=false; // involve "==" as right-handed operation
}

CMathFileParser::~CMathFileParser()
{
}

/*
*	Used to modify read value in childs, called in read_next_operand().
*	Returns 'false' to drop operand and call read_next_operand() again.
*/
bool CMathFileParser::OnNewOperandRead(CENamedItem* item) {
	if (item!=NULL) {
		long type = item->get_int();
		const char* id=item->getId();
		if (type == CFileParserAdv::OP_OPERATION_LEFT) {
			if ((m_bUseRHand2Equs) && (CEUtil::equals(id,"=="))) {
				item->set_int(CFileParserAdv::OP_OPERATION_RIGHT);
			} else if ((CEUtil::findSymbol(id,'!')>0) || (CEUtil::length(id)==3)) { // "=!=" or "==="
				item->set_int(CFileParserAdv::OP_OPERATION_RIGHT);
			}
		} else if (type == CFileParserAdv::OP_OPERATION_RIGHT) {
			if ((*id=='/') && (CEUtil::length(id)==1)) {
				char cNext=peak_next_sym();
				if (cNext == ';') { // change "/;" -> single special symbol
					skip_symbol();
					item->setId("/;");
				} else if (cNext == ':') { // change "/:" -> single special symbol
					skip_symbol();
					item->setId("/:");
					item->set_int(CFileParserAdv::OP_OPERATION_LEFT);
				}
			} else if (*id==':') {
				skip_spaces();
				char cNext=peak_next_sym();
				if (cNext == '=') { //  drop ":" before "="
					return false;
				} else if (cNext == '>') { //  change ":>" --> "->"
					item->setId("-");
				}
			} else if (CEUtil::equals(id,"|")) {
				char cNext=peak_next_sym();
				if (cNext == '|') { //  combine two "|" to single operation "||"
					skip_symbol();
					item->setId("||");
				}
			} else if (CEUtil::equals(id,"&")) {
				char cNext=peak_next_sym();
				if (cNext == '&') { //  combine two "&" to single operation "&&"
					skip_symbol();
					item->setId("&&");
				}
			}
		}
		//skip_spaces();
	}
	return true; // accept operand
}
