////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ParserSymbolsList.cpp: implementation of the CParserSymbolsList class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

const int CParserSymbolsList::LIST_ORDINARY		= 0x01;
const int CParserSymbolsList::LIST_SYMBOLS		= 0x02;
const int CParserSymbolsList::LIST_NUMBERS		= 0x04;

const int CParserSymbolsList::READ_BY_STRING	= 0x00;
const int CParserSymbolsList::READ_BY_SYMBOL	= 0x01;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CParserSymbolsList::CParserSymbolsList(int _type, CEString* _symbols, int _spec,
									   CEString* _syms_posterior, int _spec_posterior,
									   int read_mode)
									   : CEObject(true)
{
	m_symbols_type = _type;
	m_symbols_list = _symbols;
	m_list_spec = _spec;
	m_spec_posterior = _spec_posterior;
	m_symbols_posterior = _syms_posterior;
	m_read_mode = read_mode;
}

CParserSymbolsList::~CParserSymbolsList()
{
	if (m_symbols_list != NULL) delete m_symbols_list;
}

const char* CParserSymbolsList::CLASS_NAME = "CParserSymbolsList";
const char* CParserSymbolsList::ClassName() const { return CLASS_NAME; }
bool CParserSymbolsList::contain_vector(void * obj) {
	if (m_symbols_list == obj) return true;
	if (m_symbols_list->contain_vector(obj)) return true;
	return false;
}

bool CParserSymbolsList::contain_symbol(char sym, bool is_posterior_sym)
{
	if (m_list_spec & CParserSymbolsList::LIST_ORDINARY) {
		if (m_symbols_list != NULL) {
			long s = m_symbols_list->length();
			const char* syms_list_data = m_symbols_list->getData();
			int i;
			for(i=0; i<s; i++) {
				if (*(syms_list_data + i) == sym) return true;
			}
		}
	}
	
	if (m_list_spec & CParserSymbolsList::LIST_SYMBOLS) {
		if (((sym >= 'a') && (sym <= 'z')) || ((sym >= 'A') && (sym <= 'Z'))) {
			return true;
		}
	}
	
	if (m_list_spec & CParserSymbolsList::LIST_NUMBERS) {
		if ((sym >= '0') && (sym <= '9')) {
			return true;
		}
	}

	if ((is_posterior_sym) && (m_spec_posterior != 0)) {
		// search posteriors if they are defined

		if (m_spec_posterior & CParserSymbolsList::LIST_ORDINARY) {
			if (m_symbols_posterior != NULL) {
				long s = m_symbols_posterior->length();
				const char* syms_data_post = m_symbols_posterior->getData();
				int i;
				for(i=0; i<s; i++) { if (*(syms_data_post + i) == sym) return true; }
			}
		}
		
		if (m_spec_posterior & CParserSymbolsList::LIST_SYMBOLS) {
			if (((sym >= 'a') && (sym <= 'z')) || ((sym >= 'A') && (sym <= 'Z'))) { return true; }
		}

		if (m_spec_posterior & CParserSymbolsList::LIST_NUMBERS) {
			if ((sym >= '0') && (sym <= '9')) { return true; }
		}
	}

	return false;
}

int CParserSymbolsList::get_symbols_type()
{
	return m_symbols_type;
}

int CParserSymbolsList::get_read_mode()
{
	return m_read_mode;
}

CParserSymbolsList* CParserSymbolsList::create_list(int type, const char *syms_string, int read_mode) {
	return create_list(type, CParserSymbolsList::LIST_ORDINARY, syms_string, read_mode);
}

CParserSymbolsList* CParserSymbolsList::create_list(int type, int spec, int read_mode) {
	return create_list(type, spec, NULL, read_mode);
}

CParserSymbolsList* CParserSymbolsList::create_list(int type, int spec, const char* syms_string, int read_mode) {
	return create_list(type, spec, syms_string, 0, NULL, read_mode);
}

/*
*	Note: posterior symbols are symbols that may occur in string, but may not start it.
*/
CParserSymbolsList* CParserSymbolsList::create_list(int type,
													int spec, const char* syms_string,
													int spec_posterior, const char* syms_posterior,
													int read_mode) {
	CEString* syms_list = NULL;
	if (syms_string != NULL) {
		syms_list = new CEString(syms_string);
		spec |= CParserSymbolsList::LIST_ORDINARY;
	} else {
		if (spec & CParserSymbolsList::LIST_ORDINARY) spec ^= CParserSymbolsList::LIST_ORDINARY;
	}

	CEString* syms_posterior_str = NULL;
	if (syms_posterior != NULL) {
		syms_posterior_str = new CEString(syms_posterior);
		spec_posterior |= CParserSymbolsList::LIST_ORDINARY;
	} else {
		if (spec_posterior & CParserSymbolsList::LIST_ORDINARY) spec_posterior ^= CParserSymbolsList::LIST_ORDINARY;
	}

	CParserSymbolsList* res = new CParserSymbolsList(type,
		syms_list, spec,
		syms_posterior_str, spec_posterior,
		read_mode);
	return res;
}
