////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// AbstractFileParser.h: interface for the CAbstractFileParser class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ABSTRACTFILEPARSER_H__92DD8ED5_210F_433B_829E_641FA6E7F3F4__INCLUDED_)
#define AFX_ABSTRACTFILEPARSER_H__92DD8ED5_210F_433B_829E_641FA6E7F3F4__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CAbstractFileParser : public CEUtil  
{
	// source is not deleted after parsing
	// because it can be the part of commonly allocated buffer
	const char* m_source;
	long m_buffer_length; // if >0 used instead of == '\0' check, *(m_source+m_buffer_length) should be '\0'
	
	// parsing info
	long m_curr_pos;
	int m_last_item_type;
	long m_curr_line_num;
	long m_curr_line_pos;

	CParserSymbolsList* m_last_sym_pool;
	CParserSymbolsList* m_curr_sym_pool;
protected:
	CEPtrArray* m_operand_types_data;
public:
	static const int SYM_SPACE;
	static const int SYM_LINE_BREAK;
	static const int SYM_END_OF_BUFFER;
	static const int SYM_TABULATOR;
	static const int SYM_UNDEFINED;

	static const int LOAD_AS_BINARY;
	static const int LOAD_FORCE_CREATE;
	static const int LOAD_FULL_LINE_BREAK;
	static char* LoadFileToMemory(const char* fn, long* len, int load_opts);
	static char* LoadFileToMemory(const char* fn);

	// parsing parameters
public:
	bool m_bDontSkipLnBreaksIfNoSpacesFollows; // false by defaults

public:
	CAbstractFileParser(const char* _source, long len = -1);
	virtual ~CAbstractFileParser();

	int get_curr_line_num();
	int get_curr_line_pos();
protected:
	char* get_next_item();
	int get_last_item_type();
	
	char* read_string_until(int break_symbols, int error_symbols, bool skip_BS);
	char* read_string_untilC(char sym, int error_symbols, bool skip_BS);
	char peak_sym(int offset); // return *(m_source + m_curr_pos + offset);
	char peak_next_sym(); // current is NOT skipped or next if current is "skipped"
	char peak_prev_sym(); // previous after current is "skipped"
	char skip_symbol(); // returns symbol
	int get_curr_sym_type(); // current is NOT skipped yet
	void skip_spaces();

private:
	int get_curr_sym_read_mode();

public:
	// for binary access (fast float-integer save/load)
	void skip_N_symbols(int N);

};

#endif // !defined(AFX_ABSTRACTFILEPARSER_H__92DD8ED5_210F_433B_829E_641FA6E7F3F4__INCLUDED_)
