////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// CCodeProcessImpl.h: interface for the CCodeProcessImpl class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CODEPROCESSIMPL_H__A40BEA63_F277_4B4B_91F8_1D8A645AEA9A__INCLUDED_)
#define AFX_CODEPROCESSIMPL_H__A40BEA63_F277_4B4B_91F8_1D8A645AEA9A__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/*
*	Engine for CHEPProcessLite -dependent class creation.
*	TODO: to be replaced by 'CCodeProcessAdv'
*/
class CCodeProcessImpl
{
public:
	//Constants
	static const char* CN_CPP_4VEC_CLASS;		//CVM4Vec
	static const char* CN_CPP_EVT_CLASS;		//event class (CHEPEvtLite)
	static const char* CN_CPP_LIB_CLASS;		//library class (CAlHepLib)
	static const char* CN_CPP_PDF;				//basic PDF interface (CPDFLite)
	//
	static const char* CN_CHEPProcessLite;		//basic class for processes
	static const char* CN_CHEPProcessLiteSet;	//class for process set
	static const char* CN_CHEPEvtLiteCut;		//basic class for cuts
	static const char* CN_CHEPEvtLiteCutSet;	//class for cut set
	static const char* CN_CHEPEvtLiteCutE;		//class for cut on energy
	static const char* CN_CHEPEvtLiteCutPt;		//class for cut on pt
	static const char* CN_CHEPEvtLiteCutEta;	//class for cut on eta
	static const char* CN_CHEPEvtLiteCutDR;		//class for DeltaR cut
	static const char* CN_CVMGen;				//basic class for MC
	static const char* CN_CVMRnd;				//basic class for Rnd
	static const char* CN_CVMPhaseSpaceGen;		//PH-reco class
	static const char* CN_CVMGenFunc;			//MC source function
	static const char* CN_CVMGenFuncMulti;		//MC source function for parton-based process
	static const char* CN_CHEPEvtLiteLHE;		//HEPEvtLite -> LHE converter class
	static const char* CN_CHEPParticleOrder;	//particle order structure
	static const char* CN_CVMHistLite;			//
	static const char* CN_CVMHistLiteView;		//
	static const char* CN_CVMHistLiteViewDef;	//
	static const char* CN_CVMOutputSettings;	//
	//
	static const char* FN_CPP_LIB_FILE;			//"vmlib.h"
	static const char* FN_CPP_LIB_PATH;			//"VMLib/include/"
	//
	static const char* Flag_Real8;				//used if defined in 'vmlib.h'
	static const char* Flag_Real16;				//used if defined in 'vmlib.h'
	static const char* Flag_Real32;				//used if defined in 'vmlib.h'
};

////////////////////////////////////////////////////////////////////// CCodeProcessAdv
//
class CCodeProcessAdv : public CEObject
{
	class CBeamInfo : public CEObject {
	public:
		bool bPartonsUsed;
		long pk;
		//[unused]int pdfId1,pdfId2;
		CBeamInfo();
		virtual ~CBeamInfo();
		void Reset();
	};
	class CSubProcInfo : public CEObject {
	public:
		CEString sId;
		CEString sImplClassB;
		CEString sImplCallB;
		CEString sImplClassV;
		CEString sImplCallV;
		CEString sImplFileName;
		CEString sImplFileNameHeader;
		int m_nFinals;							//'N' for 2->N process
		int m_iFirstFinal;						//'2' for 2->N process
		int m_iPosInitials[2];					//positions of initial particles in event, {0,1) by default
		CIntArray pk;							//size: 2+m_nFinals (no beams!) <-- as in kinamatics
		//CEPtrArray arrPartMom;				//array of 'CEString*'
		bool bVTermExist() { return (sImplClassV.length()>0); }
		CSubProcInfo();
		virtual ~CSubProcInfo();
	};
private:
	CEString sProcTitle;						//common title for all sub-processes
	CEString sConstFileName;
	CEPtrArray m_Subs;							//array of 'CSubProcInfo*'
public:
	CBeamInfo m_beam1, m_beam2;

public:
	bool m_bEstimatedIsStatic;					//def: false, 'true' must be deprecated
	bool m_bAutoSwapPartons;					//use same proc for ('ud' and 'du' configurations)
	bool m_bUseEvtNuclonEntries;				//inherited from 'run->m_bUseEvtNuclonEntries'

public:
	CCodeProcessAdv(const char* _sProcTitle);
	virtual ~CCodeProcessAdv();
	const char* GetProcTitle() const { return sProcTitle.GetBuffer(); }
	int GetNSubs() { return m_Subs.size(); }	//number of sub-processes
	int GetNSubsActual();						//including quark-swap channels
	CSubProcInfo* SubProc(int i) { return ((CSubProcInfo*)m_Subs.GetAt(i)); }	//never NULL
	void Reset();

	bool AddSubProcess(
		const char* subId,						//sub-process name (implemented as 'C+id' class)
		int subFileId,							//id of C++ file in CLFileManager with simple implementation already saved by CFFortran
		const char* subId_B = NULL,				//(==subId by default) name for Born term (implemented as 'C+id' class)
		const char* subId_V = NULL				//name for V-term for sub-process (implemented as 'C+id' class)
		);
	bool SaveProcessClass(
		const char* id,							//process name
		int file_id,							//id of C++ file in CLFileManager with simple implementation already saved by CFFortran
		const char* proc_class_name,			//process class name ('CProc+id' if NULL)
		bool bIncSubHdrs						//(never used!) include headers of sub-process classes
		);
};

#endif // !defined(AFX_CODEPROCESSIMPL_H__A40BEA63_F277_4B4B_91F8_1D8A645AEA9A__INCLUDED_)
