////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// FFortran.h: interface for the CFFortran class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_FFORTRAN_H__C66DC13E_652F_4000_BDC3_E83F3219599E__INCLUDED_)
#define AFX_FFORTRAN_H__C66DC13E_652F_4000_BDC3_E83F3219599E__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CFortranFunctionInfo: public CEObject  
{
public:
	CFortranFunctionInfo(const char* _func_name,
		CEString* _res_value,
		int iResType,
		CEPtrArray* _par_list,
		CEPtrArray* _consts_list,
		CEPtrArray* _precalc_list,
		CEPtrArray* _funct_used_list,
		CEPtrArray* _temp_var_list,
		DWORD _conventions);
	CFortranFunctionInfo(DWORD _conventions);
	CFortranFunctionInfo(const char* _func_name, DWORD _conventions);
	virtual ~CFortranFunctionInfo();
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual bool contain_vector(void * obj);

	const char* func_name() {return m_func_name;}
	CEString* res_value() {return m_res_value;}
	int res_type() {return m_iResType;}
	CEPtrArray* par_list() {return m_par_list;}
	CEPtrArray* consts_list() {return m_consts_list;}
	CEPtrArray* precalc_list() {return m_precalc_list;}
	CEPtrArray* temp_var_list() {return m_temp_var_list;}
	CEPtrArray* funct_used_list() {return m_funct_used_list;}
	DWORD conventions() {return m_conventions;}

	void init_pre_calc_list();

	void set_res_value(CEString* res, int iResType);
	CEString* extract_res_value(); // replaces res-object with NULL, no cloning
	CEPtrArray* extract_precalc_list();
	CEPtrArray* extract_funct_used_list();
	CEPtrArray* extract_temp_var_list();
	void set_func_name(const char* _func_name);
	void add_to_precalc_list(CEObject* obj);
	void add_to_func_used_list(int iType, CEString* obj, bool check_previous = false);
	void add_to_temp_var_list(CEString* sym, int iType);

	void merge_with_other(CFortranFunctionInfo* fi);
private:
	char* m_func_name;
	CEString* m_res_value;
	int m_iResType;
	CEPtrArray* m_par_list; //array of CExprContainer{param,type}
	CEPtrArray* m_consts_list; // filled if ((convs & CLFileManager::FT_DONOT_INVOLVE_CONSTANTS)==0), else pushed to m_par_list
	CEPtrArray* m_precalc_list;
	CEPtrArray* m_funct_used_list;	//array of CEObjIdCart{str{name},type}
	CEPtrArray* m_temp_var_list;	//array of CEObjIdCart{str{name},type}
	DWORD m_conventions;
};

//////////////////////////////////////////////////// CFFortran
class CFFortran : public CEObject {
public:
	////////////////////////////////////////////// CFuncRegistry
	class CFuncRegistry : public CEObject {
	public:
		class CFuncRegistryInfo : public CEObject {
		public:
			CEString m_sNameAlhep;		//Internal name in ALHEP
			CEString m_sNameCpp;		//[NULL/0-size] ==> unavailable in C++
			CEString m_sNameFortran;	//[NULL/0-size] ==> unavailable in FORTRAN
			CEString m_sLibCpp;			//C++ namespace/class for function ([NULL/0-size] ==> function is global)
			int m_iReturnType;			//0:default
			CIntArray m_iParTypes;		//m_iParTypes->size()-->num-of-params, [0-value]-->default type
		public:
			CFuncRegistryInfo(const char* sId,
				int iReturnType, int nParams,
				const char* sIdCpp, const char* sIdFortran, const char* sLibCpp,
				int iParamsType=0);
			int GetParsNum() { return m_iParTypes.GetSize(); }
		};
	private:
		CEPtrArray m_Infos;
	public:
		CFuncRegistry(); //fills 'm_Infos'
		void AddFunctionInfo(CFuncRegistryInfo* inf);
		CFuncRegistryInfo* GetInfo(const char* func_id, int nParams);
		int GetReturnType(CEAbstractFunction* func);			//searches both function & registry info
		int GetParameterType(CEAbstractFunction* func, int i);	//searches both function & registry info
		virtual char* toLine(int mode = 0);
	public:
		static const char* F_STRUCT_F1;
		static const char* F_STRUCT_F2;
		static const char* F_STRUCT_F3;
		static const char* F_STRUCT_F4;
		static const char* F_SOFT_BREMS_INT_1;
		static const char* F_SOFT_BREMS_INT_2;
	};

public:
	////////////////////////////////////////////// CConstRegistry
	class CConstRegistry : public CEObject {
		bool m_bInited;
	public:
		class CConstInfo : public CEObject {
		public:
			CEString m_sName;		//Name in current language
			int m_iType;			//0:default
			int m_iPath;			//C_INTERNAL/C_INCLUDED/C_LOOPTOOLS
			CEString m_sLibCpp;		//C++ namespace/class for constant
			CEValue* m_Value;
		public:
			CConstInfo(const char* sName, int iType, int iPath, const char* sLibCpp=NULL);
			virtual ~CConstInfo();
		};
	public:
		CEPtrArray m_Infos;
	public:
		CConstRegistry();
		/*
		*	Fill basic constants from current model.
		*	Uses 'conventions' to determine constants types.
		*/
		void Init(DWORD conventions);
		bool IsInited(){ return m_bInited; }
		void AddConstInfo(CConstInfo* inf);
		void AddConstInfo(const char* sName, int iType, int iPath, const char* sLibCpp=NULL);
		CConstInfo* GetInfo(const char* id);
		virtual char* toLine(int mode = 0);
	public:
		static const int C_INTERNAL;
		static const int C_INCLUDED;
		static const int C_LOOPTOOLS;
	};

private:
	class CIncludeFlags {
	public:
		bool m_bLoopToolsIncluded;
		CIncludeFlags() { m_bLoopToolsIncluded=false; }
	};
	CIncludeFlags m_fIncludes;

private:
	// 'environs' in to_fortran_string()
	static const unsigned ENV_BRACKETS_NEVER;
	static const unsigned ENV_BRACKETS_FLAG;
	static const unsigned ENV_BRACKETS_SUM;
	static const unsigned ENV_BRACKETS_PROD;
	static const unsigned ENV_BRACKETS_SEQ;
	static const unsigned ENV_NUMBERS_INTEGER;

	static long MAX_LINES_IN_FUNC_DEFAULT;

private:
	CFuncRegistry m_fRegistry;
	static CEPtrArray* m_InternalFunctionsList; // DLOG, LOG, DSIN etc.
	static CEPtrArray* m_LoopToolsFunctionsList; // B0, C0i etc.
	bool isFuncInternal(CEString* func_name); //if (!internal)==>add it to 'FunctionsUsedList'

	CConstRegistry m_constRegistry;
	/*
	*	Note: may be (*iObjType != (*pConstInfo)->m_iType) for powers of constants.
	*/
	bool isSymbolConstant(CExpression* sym, CConstRegistry::CConstInfo** pConstInfo=NULL, long* iObjType=NULL);
	CEValue* getConstantValue(CExpression* sym); // non-cloned result
	CSequence m_ConstantsPowersList; //filled with Seq{Cnst,nMinPwr,MinPwrSym} if(m_bAllowConstantsPowers) by ReplPowersBySymbols()
	/*
	*	GetBasicConstSym. NULL: not constant, ==_SymOrPower: symbol, non-cloned sym: power
	*	iObjType: type of '_SymOrPower'. May be different to original constsnt type form 'pConstInfo'.
	*/
	CExpression* GetBasicConstSym(CExpression* _SymOrPower, long* pow=NULL, CConstRegistry::CConstInfo** pConstInfo=NULL, long* iObjType=NULL);
	
	char* m_file_name;
	char* m_header_file_name;
	CCodeLanguage* m_codeLang;
public:
	const char* GetFileName() { return m_file_name; }
	const char* GetHeaderFileName() { return m_header_file_name; }
	CCodeLanguage* GetCodeLanguage() { return m_codeLang; }

private:
	/*
	*	Note: Inculde file is overwrited at any add_function(File,Expr) call.
	*	The overwritten file will include both old and additional new constants.
	*/
	CEString m_sIncludeFileName;						// empty (length()==0) if no file are created
	bool m_bNeedSaveIncludeFile;
	CSequence m_IncludedConstantsList;					// values to appear in include file, seq{fortran_expr}
	CExpression* AddConstantToIncludeList(CExpression* var);	// add 'var' to 'm_IncludedConstantsList', returns 'const' ref to object
	static CSequence* m_IncludeConstsSaved;				// set of constants already saved to include file
	static CEPtrArray* m_IncludeUsedBy;					// set of .cpp/.f77 files that use include file
public:
	const char* GetConstIncludeFileName() { return m_sIncludeFileName.getData(); }
private:
	void DeclareIncludeFileRequired();
	static bool IsConstFileCreatedFor(const char* fn);	// if 'include.h' is already created for file (uses 'm_IncludeUsedBy')
public:
	const char* RequestParticleMassIncluded(long pk);
private:
	bool SaveIncludeConstantsFile();

	//bool AddClassHeaderFile();

	// Options & settings
	char* m_COMMONBLOCK_NAME;
	const char* m_COMMON_ARRAY_TITLE;
	char* m_CppClassName;
	int m_STORE_EXPR_AS_FLAG; //STORE_FORTRAN_SYMBOL,STORE_CPP etc.
	DWORD m_conventions; // valid during function freation
	DWORD m_general_conventions; // inited at file creation (F90/F77/CPP)
	bool m_bAllowConstantsPowers;

	int m_ParsingMode; // 0: collect commons only, 1: create functions

	CEPtrArray* m_sub_functions;
	long m_last_temp_num;
	CEPtrArray* m_vec_couplings; // used to fix (a.b)==>a_b replacements
	CEPtrArray* m_syms_replaced; // used to fix a\_i ==> a_i replacements
	CEPtrArray* m_Denominators; // used to check out 0 in dens
	CEPtrArray* m_Dens_functions;
	//[unused]CFortranFunctionInfo* m_ME2ToDiffCSFunc;

	CSequence* m_CommonBlockSyms;				// Symbols, Scalar-produts etc.
	long m_CommonArraySize;						// 0 if no array are declared
	void AddToCommonBlock(CExpression* sym, bool allow_flags=false);
	long FindInCommonBlock(CExpression* sym, bool allow_flags=false);
	void DeclareCommonArray();
	void SortCommonBlockItems();				// Note: to be called BEFORE first saving
	CExpression* GetCommonSym(int i);

	long m_string_terminate_pos;
	long m_num_strings_in_sum;
public:
	static CDebug* dbg;
	static bool m_PAR_CODE_GEN_FAST;			//non-approved mode, to be set as default after debug

	static CEString* m_TEMP_str_Predefined;		//may be NULL
	CEString m_TEMP_str;						//local copy of m_TEMP_str_Predefined
	static void SetTEMPstr(const char* _str);

	static bool m_bInited;
	static void init();
	static void finalize();

	void add_function(const char* func_name, CExpression* expr, DWORD conventions = 0); // changes source, but does NOT delete
	CFFortran(
		const char* file_name,
		DWORD conventions,
		bool bAppend,
		CEPtrArray* arrPreIncludes=NULL			//array of CEString*
		);
	virtual ~CFFortran();

	static void add_to_par_list(CExpression* par_new, int type, CEPtrArray* par_list);
	static void add_to_par_list(CExpression* par_new, CSequence* par_list);
	static void merge_list_with_other(CEPtrArray* par_list, CEPtrArray* other);

	virtual char* toLine(int mode = 0);
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual bool contain_vector(void * obj);

private:
	void AddParticleOrderList(ofstream& ofs); //"p1, p2, k1, k2..."
	void AddSetPartOrderFunction(ofstream& ofs, bool bHeaderOnly); //"void SetPartOrder(int id_p1, int id_p2...)"

private:
	char* m_TmpBuf1kb; //allocated to 1kb
	CEString* GetSubFuncCallString(CFortranFunctionInfo* func_inf); //For internal sub-functions
	void add_function(ofstream* p_ofs, CFortranFunctionInfo* func_inf, bool bMainEntry=false);//bMainEntry used for debug features only
	void add_function_header(ofstream* p_ofs, CFortranFunctionInfo* func_inf); //C++ only
	void add_CommonBlock(ofstream* p_ofs, CEString* f_line, const char* type_ID, int num_params_in_line);
	CEString* get_ItemStr(CExpression* val, bool chk_COMMON);  // possible COMMON value or toString(), never NULL, no sign allowed
	CEString* get_CommonItemStr(long i);
	// a_mode: 0--src is square,1--amplitude,2--amplitude2
	void add_vec_entry(ofstream* p_ofs, const char* _EntryTitle, CEPtrArray* cpl_data, CFortranFunctionInfo* func_inf, bool bHeaderOnly=false);
	CFortranFunctionInfo* to_fortran_string(
		CFortranFunctionInfo* fi,
		CExpression* expr,
		DWORD environs);
	CEString* get_new_temp_symbol();

	static void collect_syms(CExpression* expr, CSequence* par_list, unsigned objects_to_collect);
	static CExpression* validate_objects(CExpression* expr); // does not deletes input, but may return & modify it
	static CExpression* validate_syms(CExpression* expr, CEPtrArray* _sym_origins); // does not deletes input, may return & modify it
	static CESymbol* validate_sym(CESymbol* sym, CEPtrArray* _sym_origins); // does not affect input, but may return it
	static CEString* toFortranSyms(CEString* str);
	CExpression* scalar_couplings_to_syms(CExpression* expr, CEPtrArray* _vec_couplings);
	static void addExprString(ofstream* p_ofs, CExpression* val); // no check are performed trerein
	CExpression* ReplPowersBySymbols(CExpression* src); // changes powers to symbols; does not deletes input, but may return it

	CExpression* GetSymOrigin(CExpression* _sym);

	// Library procedures (may use 'm_conventions', 'm_STORE_EXPR_AS_FLAG' etc.)
public:
	CExpression* VIToUsualFunctions(CExpression* src); //may delete input, uses 'm_conventions' and 'm_STORE_EXPR_AS_FLAG'
private:
	static CExpression* VIToUsualFunctionsImpl(CExpression* src, CEPtrArray* data_arr);
};

#endif // !defined(AFX_FFORTRAN_H__C66DC13E_652F_4000_BDC3_E83F3219599E__INCLUDED_)
