////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// FileParserAdv.h: interface for the CFileParserAdv class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_FILEPARSERADV_H__E0537D97_B18C_4D71_9483_F85B53D0EE32__INCLUDED_)
#define AFX_FILEPARSERADV_H__E0537D97_B18C_4D71_9483_F85B53D0EE32__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CFileParserAdv : public CAbstractFileParser  
{
private:
	CEPtrArray* m_operands_list;
	CEPtrStack* m_list_stack; // used for storage line data when subline processed;
	CEPtrStack* m_push_back_stack; // storage of pushed back data;
	CENamedItem* m_next_operand;

public:
	static const char* STR_EOF_OPERAND;
	static const char* STR_INVALID_OPERAND;

public:
	static const int OP_INVALID;
	static const int OP_NO_OPERAND;
	static const int OP_END_OF_LINE;

	static const int OP_OPERAND_ANY;
	static const int OP_VARIABLE;
	static const int OP_FUNCTION_CALL;

	static const int OP_OPERATION_ANY;
	static const int OP_OPERATION_RIGHT;
	static const int OP_OPERATION_LEFT;

	static const int OP_BRACKET_ANY;
	static const int OP_BRACKET_LEFT;
	static const int OP_BRACKET_RIGHT;
	static const int OP_BRACKET_LEFT_2;
	static const int OP_BRACKET_RIGHT_2;

	static const int OP_VALUE_ANY;
	static const int OP_VALUE_NUMBER;
	static const int OP_VALUE_STRING;

	static const int OP_SLASH;
	static const int OP_COLON;

	static const int OP_SPECIAL_1;

public:
	// comments notation
	static int COMMENTS_C;		// comments: /*...*/
	static int COMMENTS_MMOD;	// comments: (*...*)
	int m_CommentsType;

public:
	static CDebug* dbg;

public:
	void finish_operands_list();
	void start_new_operands_list();
	virtual int get_operands_num();
	virtual int get_operand_type(int offset, bool from_start = false);
	virtual const char* get_operand(int offset, bool from_start = false);
	virtual bool skip_next_operand(bool add_to_list = true);
	virtual void insert_operand_as_current(const char* c_op, int type); // before current
	virtual void push_back_last_operand();
	int get_operation_offset(int type, bool from_start, int last_found_pos=INT_MIN);

	virtual int get_next_op_type();
	virtual const char* get_next_operand(bool skip_next = true);
	CFileParserAdv(const char* _source);
	virtual ~CFileParserAdv();

private:
	CENamedItem* get_operand_cart(int offset, bool from_start);

protected:
	bool ReadPushedBackOperand(); // to be used BEFORE instead of CAbstractFileParser::get_next_item()
	virtual bool read_next_operand(); // may call itself recursively
	/*
	*	Used to modify read value in childs, called in read_next_operand().
	*	Returns 'false' to drop operand and call read_next_operand() again.
	*/
	virtual bool OnNewOperandRead(CENamedItem* item);

public:
	void LogReadOperandsList(int level);
};

#endif // !defined(AFX_FILEPARSERADV_H__E0537D97_B18C_4D71_9483_F85B53D0EE32__INCLUDED_)
