////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// LFileManager.h: interface for the CLFileManager class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_LFILEMANAGER_H__A40BEA63_F277_4B4B_91F8_1D8A645AEA9A__INCLUDED_)
#define AFX_LFILEMANAGER_H__A40BEA63_F277_4B4B_91F8_1D8A645AEA9A__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//////////////////////////////////////////////////// CCodeLanguage
//
// Notes
//	1. CFFortran::CFuncRegistry contains HEP-specific internal functions. Port it here?
//	2. Port CFFortran::CConstRegistry here!
//	3. m_InternalFunctionsList // DLOG, LOG, DSIN etc.
//	4. m_LoopToolsFunctionsList // B0, C0i etc.
//	5. Port 'm_TEMP_str' here?
//	6. For more languages support make different classes (switched in 'CCodeLanguage::CreateLanguage()'):
//		[CCodeLanguage] <-- [CCodeCpp]
//		                <-- [CCodeF77]
//		                <-- [CCodeF90] etc.
//	8. required: add 'abstract' flag after creation
//	9. 'val' seems not required
//
class CCodeLanguage : public CEObject {
public:
	// Constants
	static const char* TYPE_REAL8_F90;
	static const char* TYPE_COMPLEX16_F90;
	static const char* TYPE_REAL8_F77;
	static const char* TYPE_COMPLEX16_F77;
	static const char* TYPE_INTEGER;
	static const char* TYPE_REAL8_CPP;
	static const char* TYPE_COMPLEX16_CPP;
	static const char* TYPE_INTEGER_CPP;
	static const char* TYPE_REAL8_CPP_ABS;
	static const char* TYPE_COMPLEX16_CPP_ABS;
	static const char* TYPE_4VEC_CPP;
	static const char* TYPE_BOOL_CPP;

	static const char* CPP_POW_DEF_SYM;
	static const char* CPP_REAL8_CONV_ADV;
	static const char* CPP_CMPLX_C16_CONV;
	static const char* CPP_CMPL16_R8_CONV;
	static const char* CPP_CMPLX_RX_CONV;

protected:
	// Options & settings
	DWORD m_conventions;					// language selector (c++/f77/f90) only
	bool m_bIsAbstractAvailable;			// are 'tReal'/'tComplex' types defined
	int m_IndentSize;
	static const char* m_LINEBREAK_SYMS;	// used to autosplit output lines

protected:
	CCodeLanguage(DWORD conventions);

public:
	static CCodeLanguage* CreateLanguage(DWORD conventions); // must be deleted by user
	virtual ~CCodeLanguage();
	virtual char* toLine(int mode = 0);
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual bool contain_vector(void * obj);
	static CDebug* dbg;

public:
	//indent operations
	int GetIndent() { return m_IndentSize; }
	void IncIndent() { m_IndentSize++; }
	void DecIndent() { if(m_IndentSize>0)m_IndentSize--; }
	void ClearIndent() { m_IndentSize=0; }
	void SetIndent(int indent) { m_IndentSize=indent; }

	//code-write procedures
	void StartCommentLine(ofstream* p_ofs, bool isHiddenCode=false); //'hidden' is a line to be uncommented by user in future
	void OutCodeLine(ofstream* p_ofs, CEString* c_line, int _label=-1, bool bTruncateLine=false, const char* comment=NULL); // also resets 'c_line'
	void OutCodeLine(ofstream* p_ofs, const char* _line, int _label=-1, bool bTruncateLine=false, const char* comment=NULL);
	void OutConsoleWriteLine(ofstream* p_ofs, CEString* c_line, bool bAddEOL=true); //don't need quotes in 'c_line', resets 'c_line'

	//type functions
	bool IsAbstractAvailable() { return m_bIsAbstractAvailable; }
	void SetAbstractAvailable(bool bUsed) { m_bIsAbstractAvailable=bUsed; }
	int GetDefaultTypeId(DWORD type_convs);					//uses 'type_convs' to select default type
	int GetDefaultRealTypeId();								//returns 'real8' or 'real*X'
	//
	const char* GetTypePrefixAdv(DWORD type_convs);			//uses GetTypePrefix(GetDefaultTypeId(type_convs))!!!
	const char* GetTypePrefix(int type);					//
	const char* GetTypePrefixReal();						//abstract if available
	const char* GetTypePrefixComplex();						//abstract if available
	const char* GetTypePrefixInt();							//
	const char* GetTypePrefixBool();						//
protected:
	const char* GetTypePrefixImpl(int type);				//cancels 'abstract' type if not allowed by 'IsAbstractAvailable()'
public:
	bool GetTypeConversion(int tFrom, int tTo, CEString* sBefore, CEString* sAfter,
		DWORD type_convs=0,		//used if ((tFrom==DEFAULT) || (tTo==DEFAULT)), may be 0
		CExpression* val=NULL,	//currently unused!
		bool bForce=false);
	int GetOperatingType(int type);							// real8==>realX, cmpl16==>cmplX

	//operations
	//TODO: change these operations to 'WriteOp...(ofstream* p_ofs)' -> to controle ';'-symbols at end
	const char* GetOpIf();
	const char* GetOpEqu();				//".EQ." or "=="
	const char* GetOpThen();			//"THEN" or "{"
	const char* GetOpElse();			//"ELSE" or "} else {"
	const char* GetOpEndIf();			//"ENDIF" of "}"
	const char* GetOpEndProc();			//"END" of "}" (indent not affected)
	void AddOpFor(CEString* ln, const char* iStr, int iMin, int iMax, int iLabel=-1, bool bDeclareI=false); //"DO iSrt=iMin,iMax" of "for(i=iMin; i<=iMax; i++){"
	void AddOpForP1(CEString* ln, const char* iStr, int iMin, int iMaxP1, int iLabel=-1, bool bDeclareI=false); //"DO iSrt=iMin,(iMaxP1-1)" of "for(i=iMin; i<iMaxP1; i++){"
	void AddOpForP1(CEString* ln, const char* iStr, int iMin, const char* sMaxP1, int iLabel=-1, bool bDeclareI=false); //"DO iSrt=iMin,(sMax-1)" of "for(i=iMin; i<sMax; i++){"
	void AddOpEndFor(CEString* ln);		//"CONTINUE" or "}"
	void AddFuncCallName(CEString* ln, const char* name);// C++ function call if "FName()" to separate from return variable name

protected:
	void StartCodeLine(ofstream* p_ofs, int _label=-1);	// 6-sym gap for F77
	void StartContinuationLine(ofstream* p_ofs);		// 5-sym gap for F77 + "&"
	int GetCodeLengthMax();								// LINE-length - GAP - indent
	bool IsLineBreakAllowed(const char* _line, long pos, long len);
};

//////////////////////////////////////////////////// CLFileManager
//
class CLFileManager : public CELibrary  
{
	static CEPtrArray* m_files;
public:
	static CDebug* dbg;

	static const DWORD FT_F90;
	static const DWORD FT_F77;
	static const DWORD FT_CPP;
	static const DWORD FT_REAL_8BIT;
	static const DWORD FT_COMPLEX_16BIT;
	static const DWORD FT_DONOT_CREATE_4VECTOR_ENTRY;
	static const DWORD FT_DONOT_SPLIT_FUNC;
	static const DWORD FT_SEPARATE_VIRT_INTEGRALS;
	static const DWORD FT_NO_COMMON_PARAMETERS;
	static const DWORD FT_NO_COMMON_ARRAYS;
	static const DWORD FT_LOOPTOOLS;
	static const DWORD FT_DONOT_INVOLVE_CONSTANTS;
	static const DWORD FT_POWER_PARAMS;
	static const DWORD FT_CHECK_DENOMINATORS;
	static const DWORD FT_PYTHIA_VECTORS;		// For Fortran only
	static const DWORD FT_SRC_IS_AMPLITUDE;
	static const DWORD FT_SRC_IS_DIFFCS;
	static const DWORD FT_SRC_SAVE_AS_IS;		//same to FT_SRC_IS_DIFFCS
	static const DWORD FT_NO_INCLUDES;			// do not separate constants to include file
	static const DWORD FT_FULL_INCLUDE;			// create include file with all possible constants. WARNING: no names check!
	static const DWORD FT_CPP_TYPE_ABSTRACT;	// (C++ only) abstract 'tReal'/'tComplex' types instead of 'double'/'complex'
	static const DWORD FT_FORCE_REAL;			//truncate imaginary part of every function/variable
	static const DWORD FT_ALLOW_SWAP_INITIALS;	//add initial particles swap mechanism
	static const DWORD FT_NON_STATIC_IMPL;		// (C++ only) (default: false) create non-static functions

	static int new_file(const char* file_name, DWORD conventions, CEPtrArray* sPreIncludes=NULL) {
		return open_file(file_name,conventions,false,sPreIncludes);
	}
	static int open_file(const char* file_name, DWORD conventions, bool bAppend, CEPtrArray* sPreIncludes=NULL);
	static CCodeLanguage* GetCodeLanguageRef(int file_id);
	static const char* GetFileName(int file_id);
	static const char* GetHeaderFileName(int file_id);
	static const char* GetConstIncludeFileName(int file_id);
	static const char* RequestParticleMassIncluded(int file_id, long pk);
	static void add_to_file(int file_id, const char* func_name, CExpression* expr, DWORD conventions = 0);
	static void close_file(int file_id);		//not required -- releases a bit of memory only
	static void finalize();

	CLFileManager();
	virtual ~CLFileManager();
};

#endif // !defined(AFX_LFILEMANAGER_H__A40BEA63_F277_4B4B_91F8_1D8A645AEA9A__INCLUDED_)
