////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EHistogram.h: interface for the CEHistogram class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EHISTOGRAM_H__B1155460_5D45_11D6_9CE8_82C16C289B19__INCLUDED_)
#define AFX_EHISTOGRAM_H__B1155460_5D45_11D6_9CE8_82C16C289B19__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//##ModelId=3E3FF2DE02EE
class CHBank : public CEObject
{
public:
	//##ModelId=3E3FF2DE0317
	CHBank(unsigned _size, double l, double r);
	//##ModelId=3E3FF2DE0316
	virtual ~CHBank();
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	//##ModelId=3E3FF2DE0313
	void add_value(double val, double pow);
	//##ModelId=3E3FF2DE0312
	void normalize();
	//##ModelId=3E3FF2DE0310
	void add_values_from(CHBank* other);
	//##ModelId=3E3FF2DE030F
	bool is_normalized();
	//##ModelId=3E3FF2DE030E
	CHBank* create_frame();
	//##ModelId=3E3FF2DE030D
	unsigned size();
	//##ModelId=3E3FF2DE0308
	double* margines();
	//##ModelId=3E3FF2DE0307
	double max_val();
	//##ModelId=3E3FF2DE0306
	double average();
	//##ModelId=3E3FF2DE0304
	double value(unsigned i);
	//##ModelId=3E3FF2DE0303
	unsigned long num_ivents();
	//##ModelId=3E3FF2DE0302
	unsigned long num_outmargins();
private:
	//##ModelId=3E3FF2DE02FF
	unsigned m_size;
	//##ModelId=3E3FF2DE02F6
	double* m_values;
	//##ModelId=3E3FF2DE02F5
	double* m_margines;
	//##ModelId=3E3FF2DE02F4
	double m_average;
	//##ModelId=3E3FF2DE02F3
	unsigned long* m_cell_ivents;
	//##ModelId=3E3FF2DE02F2
	unsigned long m_num_outmargins;
	//##ModelId=3E3FF2DE02F1
	unsigned long m_num_ivents;
	//##ModelId=3E3FF2DE02F0
	bool m_is_normalized;
};

//##ModelId=3E3FF2CE032C
class CEHistogram : public CESerializable  
{
	//double m_left_border;
	//double m_right_border;

	//long m_max_height;
	//unsigned m_bar_num;
	//unsigned long m_num_ivents;

	//##ModelId=3E3FF2CE034D
	CEPtrArray* m_a_values;
	//double* m_values;
	//double* m_margines;

	//long m_num_outmargins;

	//##ModelId=3E3FF2CE0346
	double m_aspect_ratio;
public:
	//##ModelId=3E3FF2CE0343
	static const unsigned G_PLOT_HISTO;
	//##ModelId=3E3FF2CE033E
	static const unsigned G_LOG_AXIS_Y;
	//##ModelId=3E3FF2CE03C8
	void normalize_all();
	//##ModelId=3E3FF2CE03BC
	void add_bank(int bank, int dest_bank);
	//##ModelId=3E3FF2CE0331
	static CDebug* dbg;
	//##ModelId=3E3FF2CE03B9
	CHBank* get_bank(int num=0, bool create_bank = false);
	//##ModelId=3E3FF2CE03AE
	CEString* to_postscript(int bank = 0);
	//double max_val();
	//##ModelId=3E3FF2CE03AC
	void visualize(int bank = 0);
	//##ModelId=3E3FF2CE03AB
	virtual const char* ClassName() const;
	//##ModelId=3E3FF2CE039E
	void add_value(double val, double pow = 1, int bank = 0);
	//void set_borders(double l, double r);
	//##ModelId=3E3FF2CE039C
	virtual CEString* toString(int mode);
	//##ModelId=3E3FF2CE0390
	CEHistogram(unsigned bar_num = 20, double l = 0, double r = 1);
	//##ModelId=3E3FF2CE038F
	virtual ~CEHistogram();

private:
	//##ModelId=3E3FF2CE038E
	double max_value_in_banks();
	//##ModelId=3E3FF2CE037D
	void add_histo(CEString* res, double bottom, double height, double left, double mv, CHBank* bank);
	//##ModelId=3E3FF2CE036D
	void add_graph(CEString* res, double bottom, double height, double left, double mv, CHBank* bank);
	//##ModelId=3E3FF2CE032E
	int m_active_bank;
	//double* values(int bank = 0, bool create_bank = false);
	//##ModelId=3E3FF2CE035D
	void add_subscriptions(CEString* res,
		double bottom, double height, double left, double mv, int bank);
	//##ModelId=3E3FF2CE0353
	void add_val2(CEString* s, double val);
	//void log_values();
	//##ModelId=3E3FF2CE0350
	void add_val(CEString* s, double val);
};

#endif // !defined(AFX_EHISTOGRAM_H__B1155460_5D45_11D6_9CE8_82C16C289B19__INCLUDED_)
