////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EMathEquRule.h: interface for the CEMathEquRule class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EMATHEQURULE_H__09BF6FCD_60DE_4FFA_8254_6B8F1F10BD51__INCLUDED_)
#define AFX_EMATHEQURULE_H__09BF6FCD_60DE_4FFA_8254_6B8F1F10BD51__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CRuleCondition : public CEObject {
public:
	CRuleCondition();
	static const char* CLASS_NAME;
    virtual const char* ClassName() const;
	virtual bool contain_vector(void * obj);
    virtual char* toLine(int mode=0)=0;
	virtual CEObject* copy() const;
	virtual CRuleCondition* clone() const = 0;
	virtual bool FitsCondition(CERule* r) = 0;
};

class CRuleConditionEquZero : public CRuleCondition {
private:
	CExpression* m_var;
	bool m_bEquZero;
public:
	CRuleConditionEquZero(bool bEquZero, CExpression* var);
	virtual ~CRuleConditionEquZero();
	static const char* CLASS_NAME;
    virtual const char* ClassName() const;
	virtual bool contain_vector(void * obj);
    virtual char* toLine(int mode=0);
	virtual CRuleCondition* clone() const;
	virtual bool FitsCondition(CERule* r);
};

class CRuleConditionAndOr: public CRuleCondition {
private:
	CEPtrArray m_Conditions;
	int m_type;
public:
	static const int COND_AND;
	static const int COND_OR;
	CRuleConditionAndOr(int type);
	virtual ~CRuleConditionAndOr();
	void AddCondition(const CRuleCondition* c);
	int GetType();
	static const char* CLASS_NAME;
    virtual const char* ClassName() const;
	virtual bool contain_vector(void * obj);
    virtual char* toLine(int mode=0);
	virtual CRuleCondition* clone() const;
	virtual bool FitsCondition(CERule* r);
};

class CEMathEquRule : public CEObject
{
private:
	CExpression* m_Left;
	CExpression* m_Value;
	CSequence m_TempSyms;
	CRuleConditionAndOr* m_Conditions;
public:
	CEMathEquRule(CExpression* left, CExpression* val);
	virtual ~CEMathEquRule();
	void AddCondition(const CRuleCondition* c);
	bool IsSimilarTo(CExpression* left); // slow procedure! equals()

	static const char* CLASS_NAME;
    virtual const char* ClassName() const;
	virtual bool contain_vector(void * obj);
    virtual char* toLine(int mode = 0);

	static bool IsMathTemplateLeft(CExpression* left);

private:
	/*
	*	Templ[a_, b____, c, a___] -> Templ[a,b,c,a] + {a,b}
	*/
	static CExpression* ConvertTemplateLeft(CExpression* left, CSequence* _TempSyms);

public:
	bool IsApplicableTo(CExpression* src); // unaffects source

	/*
	*	deletes or unaffects source
	*	returns src or cloned result
	*/
	CExpression* ApplyTo(CExpression* src, bool del_src=true);

private:
	bool FitsLeft(CExpression* left, CExpression* src, CSequence* repls);

public:
	//static CExpression* CollectEvaluateComposedCharRProc(CExpression* src, CEPtrArray* data_arr);
};

#endif // !defined(AFX_EMATHEQURULE_H__09BF6FCD_60DE_4FFA_8254_6B8F1F10BD51__INCLUDED_)
