////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// EParticle.h: interface for the CEParticle class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EPARTICLE_H__AE0D71F2_3B4F_46F2_A6A9_284C768C923C__INCLUDED_)
#define AFX_EPARTICLE_H__AE0D71F2_3B4F_46F2_A6A9_284C768C923C__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//##ModelId=3E3FF2D7006D
class CEParticle : public CESerializable
{
	CExpression* m_id; // momentum
	CExpression* m_sym_mass; // mass
	CExpression* m_pol_vec_sym; // polarization vector symbol
	CExpression* m_pol_vec_sym_cjg; // conjugated pol symbol
	int m_polarization_info; // 0 means unpolarized, 1(default) involves polarization vectors

	// polarization vector complicated expression
	CExpression* m_pol_vec_predefined;
	CExpression* m_pol_vec_predef_index; // vector index in m_pol_vec_predefined, covariance should be "false"

	int m_ID_in_kin; // ID of particle in kinematics

	CEValue* m_mass_value; // used in numerical calcs
	CENumVector* m_stub; // used in numerical calcs

protected:
	bool m_init_OK; // used to check up success of overriden constructors

public:
	static CDebug* dbg;

	static const long PARTICLE_FERMION;
	static const long PARTICLE_LEPTON;
	static const long PARTICLE_ELECTRON;
	static const long PARTICLE_MUON;
	static const long PARTICLE_TAU_LEPTON;
	static const long PARTICLE_NEUTRINO;
	static const long PARTICLE_NEUTRINO_ELECTRON;
	static const long PARTICLE_NEUTRINO_MUON;
	static const long PARTICLE_NEUTRINO_TAU;
	static const long PARTICLE_QUARK;
	static const long PARTICLE_QUARK_CONJUGETED;
	static const long PARTICLE_QUARK_D; //1
	static const long PARTICLE_QUARK_U; //2
	static const long PARTICLE_QUARK_S; //3
	static const long PARTICLE_QUARK_C; //4
	static const long PARTICLE_QUARK_B; //5
	static const long PARTICLE_QUARK_T; //6
	static const long PARTICLE_GLUON;

	static const long PARTICLE_PROTON;	//For srtucture functions approach
	static const long PARTICLE_NEUTRON;	//For srtucture functions approach
	
	static const long PARTICLE_MODEL_FERMION;
	
	static const long PARTICLE_BOSON;
	static const long PARTICLE_NEUTRAL_BOSON;
	static const long PARTICLE_VECTOR_BOSON;
	static const long PARTICLE_NEUTRAL_VEC_BOSON;
	static const long PARTICLE_PHOTON;
	static const long PARTICLE_Z_BOSON;
	static const long PARTICLE_W_BOSON; // Charge(W)==-1

	static const long PARTICLE_SCALAR;
	static const long PARTICLE_HIGGS;
	static const long PARTICLE_SCALAR_PHI; // Charge(Phi)==Charge(W)==-1
	static const long PARTICLE_SCALAR_CHI;
	static const long PARTICLE_FPGHOST; // flag
	static const long PARTICLE_FPGHOST_UA;
	static const long PARTICLE_FPGHOST_UZ;
	static const long PARTICLE_FPGHOST_UWp;
	static const long PARTICLE_FPGHOST_UWm;

	static const long PARTICLE_MODEL_BOSON;

	static const long PARTICLE_CLASS_FLAG; // "abstract" particle
	static const long PARTICLE_KIND_MAX;

	/*
	* to compare if 'usual' particle fits 'class', as "UsualPK & FitFlagPk | PARTICLE_CLASS_FLAG == ClassPK"
	*/
	static long GetClassFitFlagPk(long pkClass);
	//static long CLASS_DIFFERENT_FLAG; // to compare if 'usual' particle fits 'class'
	static bool FitClassPk(long pk, long pkClass);

	//static const long PARTICLE_ENUM_MASK;

public:
	// static entries
	static CEParticle* create_particle(long _kind, int _ID_kin,
		CExpression* _momentum_sym,										// NOT cloned!
		CExpression* _mass_sym,											// NOT cloned!
		CExpression* _pol_vec_sym,										// NOT cloned!
		CEPtrArray* params);
	static int compare_part_kind(long a, long b, bool fermions_first);	// {-1 for b<a (wrong order)}
	static bool is_fermion_PK(long pk);
	static bool is_bosonic_PK(long pk);
	static bool is_vec_bosonic_PK(long pk);
	static bool is_scalar_PK(long pk);
	static bool is_neutral_bosonic_PK(long pk);
	static bool is_class_PK(long pk);

	// check function
	bool check_PK(long pk, bool check_flag_only);

	// basic get functions
	CExpression* get_sym_mass();
	CExpression* get_momentum_symbol();
	CExpression* get_pol_vector_symbol();
	CExpression* get_pol_vector_symbol_cjg();
	CExpression* get_pol_vec_predifined();
	CExpression* get_pol_vec_predef_index();
	int get_polarization_info();

	// set functions
	void set_sym_mass(CExpression* _m_sym); // no checks for symbol usage inside
	virtual void set_polarization_info(int _pol_val);
	void predefine_pol_vector(CExpression* vec, CExpression* ind, bool ind_cov);
	
	int GetPolarizationsNumber(); // 3 for massive boson partice, 2 for massless & fermion partices

	virtual long get_PK() = 0; // particle kind
	virtual bool is_init_OK();
	int get_ID(); // ID of particle in kinematics
	void set_ID(int pID); // ID of particle in kinematics (e.g. if changed)
	bool is_initial();

	virtual CExpression* getDensityMatrix(CEPtrArray* vars)=0;// 'vars' are the particle-specific parameters

	// numeric gets
	CEValue* mass();
	CENumVector* get_stub();
	// used to find abstract vector in kinematics
	// compares vec_sym to any vector related to this particle (momentum and polarization).
	virtual CENumVector* get_vector_stub(CExpression* vec_sym);

protected:
	// _mass = NULL means no numerical calculations are supposed
	CEParticle(CExpression* id, CExpression* sym_mass, CExpression* _pol_vec_sym, int _ID_kin, CEValue* _mass = NULL);
	//CEParticle(CExpression* id, CExpression* sym_mass, CENumVector* _stub);
public:
	virtual ~CEParticle();

	virtual const char* ClassName() const;
	virtual CEString* toString(int mode = 0);
};

#endif // !defined(AFX_EPARTICLE_H__AE0D71F2_3B4F_46F2_A6A9_284C768C923C__INCLUDED_)
