////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// Kinematics.h: interface for the CKinematics class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_KINEMATICS_H__61FFCD40_D921_11D5_90CB_00403393DD18__INCLUDED_)
#define AFX_KINEMATICS_H__61FFCD40_D921_11D5_90CB_00403393DD18__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//##ModelId=3E3FF2E002AF
class CKinematics : public CEObject
{
protected:
	static CEData* data;// fast DataStorage acsess

	// particles data
	CEPtrArray* m_init_particles;
	CEPtrArray* m_final_particles;

	//vector basis
	CSequence* m_BasisVecs; // array[4] of symbol-bases for basis, intialized at first use

	//	for non-covariant simplification
	bool m_Collinear_Simpl_list_added;
public:
	static CDebug* dbg;

public:
	// data acsess
	CENumVector* get_vector_stub(CExpression* h_sym);
	int find_momentum(CExpression* expr);
	int find_polarization_vec(CExpression* expr); // check also CJG vector
	CExpression* get_momentum(int particle_num);
	CEParticle* get_particle(int i0, bool from_initial);
	CEParticle* get_particle(CExpression* momentum);
	CExpression* get_mass(CExpression* momentum);
	CExpression* GetVectorSquare(CExpression* vec_base); // returns cloned value or NULL

	//substitute particle function, used for {p->quarks} substitutions, clears all pre-inited secondary variables
	CEParticle* ReplaceParticle(int pId, CEParticle* part);	//returns previous, that is NOT deleted
	int AddFinalParticle(CEParticle* part);					//returns PID, gets ownership of 'part'
	CEParticle* RemoveLastFinalParticle();					//returns previous, that is NOT deleted
	//Note: every particle contains pID ==> be careful with removing them!

protected:
	void ClearVolatileVariables();							// clear seconrdary variables (e.g. if kinematics is changed)

public:
	CEParticle* pfin(int i0) { return get_particle(i0, false); }
	CEParticle* pin(int i0) { return get_particle(i0, true); }
	virtual void set_CMS_energy(double energy) = 0;
	CENumVector* vin(int i);
	CENumVector* vfin(int i);
	static int PID(int i0,  bool from_initial) { return (from_initial?(i0+1):(-i0-1)); }
	CEValue* mass(int i, bool from_initial = false);
	CENumVector* get_sum_initials();
	CENumVector* vector(int i, bool from_initial);

	int num_init_particles() { return m_init_particles->size(); }
	int num_final_particles() { return m_final_particles->size(); }

	// extended
	CExpression* get_simpliest(CExpression* expr);
	void add_collinear_simpl_list();
	virtual CEReplList* getReplacements(int type);
	CEPtrArray* getCrossingsTransformations();
	CEReplList* getBasicCouplingsValues(bool CM_beams = false);
	virtual bool isPrimary(CExpression* expr);
	virtual CExpression* getEqual(CExpression* expr);
	CESum* get_kin_equal(int particle_num);
	virtual int checkExprSign(CExpression* expr); // 0 for unknown result
	bool isSymbolUsed(CESymbol* sym);

	void DeclareBasisVectors(); // init symbols for basis
	int GetBasisVecNo(CExpression* vec_base); // -1 if not found
	CSequence* GetBasis(); // do not change it!

	// identity
	virtual long GetHashCode();

	// utils
	static void add_inv_repl_to_list(CEParticle* p1, CEParticle* p2, bool sum_sqr, bool plus, CExpression* inv_sym, CEReplList* list);

	// system functions
protected:
	CKinematics();
public:
	virtual ~CKinematics();
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual bool contain_vector(void * obj);
	virtual char* toLine(int mode = 0);
	static void init();
	virtual void fill_data_storage();
	virtual void InitVectors(double energy) = 0; // is it used?

	//	parameters for getReplacements()
	static int REPL_SIMILAR_COUPLINGS;		// p1.k2 --> k1.p2 etc.
	static int REPL_BASIS_COUPLINGS;		// e1.e2 for basis vectors
	static int REPL_COUPLINGS_SIMPLIFY;		// REPL_SIMILAR_COUPLINGS|REPL_BASIS_COUPLINGS
	static int REPL_MANDELSTAM_VARIABLES;
	static int REPL_ALL_MANDELSTAM_X;		// Used for simplification.
	static int REPL_ALL_COUPLINGS_X;		// Used for simplification.
	static int REPL_ALL_KIN_DEPENDENCIES;	// Used for simplification.
	static int REPL_ALL_MOMENTA_SUBST;		// Used for simplification.

	CEParticle* get_particle_by_ID(int id);
	CENumVector* get_vector_by_ID(int id);
	int get_particle_ID(CEParticle* part);

	// To be called before matrix element retrieving
	// reset vector indices
	// flase means indices to be cleared
	void reset_volatile_variables(bool do_create);

	// Log procedures
	char* toLineAdv(bool bInitials, bool bFinals, bool bIDs, int mode = 0);

	// debug function
	void log_vectors();
};

#endif // !defined(AFX_KINEMATICS_H__61FFCD40_D921_11D5_90CB_00403393DD18__INCLUDED_)
