////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// LMath.h: interface for the CLMath class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_LMATH_H__CC7F1804_53EA_4152_82FB_E04185081D20__INCLUDED_)
#define AFX_LMATH_H__CC7F1804_53EA_4152_82FB_E04185081D20__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

//##ModelId=3E3FF2CA004E
class CLMath : public CELibrary
{
public:
	static int Trunc(double val);
	static int Round(double val);
	static long abs(long val);
	static long step(long a, long b); // 1 for a<b; -1 for a>b; 0 for equals
	static long lmin(long a, long b); // returns first if equals
	static long lmax(long a, long b); // returns first if equals
	static double Abs(double val);
	static long factorial(int val);
	static long factorial_frac(int num, int den); // num!/den!
	static long CNOfK(int n, int k); // (n,k) = n!/(k! (n-k)!)
	static long Pow(long a, long n);
	static long greatest_common_divisor(long a, long b);
	static long least_common_multiple(long a, long b);
	static bool is_even(long num);
	static CSequence* linear_system_solve(CSequence *arr, CSequence *col);
	static CExpression* det(CSequence* arr);
	static CExpression* det(CSequence* arr, int col_pos, CSequence* col = NULL);
	static double angle(double x, double y, bool in_degrees = false);
	static double sqr(double x);
	static double length(double x1, double x2, double y1, double y2);
	static int LengthApprox(int x1, int y1, int x2, int y2);
	static long Log2Int(long val); // val = 2^(res), -1 if res is NOT integer

	static CDebug* dbg;
	static char* m_TempBuffer100; // created at first call, deleted by CEProcessor::AddToDeleteList();

	static int get_power(double val); // +/-X.XXXE(result power)
	static double frexp10(double x, int *expptr); // very slow operation
	static int get_digits_num(long val);

	static CExpression* temp_func2();
	static CExpression* temp_func3();
	static CExpression* temp_func0();
	static CExpression* temp_func1(CExpression* x);

	static CEValue* Cos(CEValue * angle);
	static double Cos(double angle, bool in_degree = false);

	static CEValue* Sin(CEValue* angle);
	static double Sin(double angle, bool in_degree = false);

	CLMath();
	virtual ~CLMath();
};

#endif // !defined(AFX_LMATH_H__CC7F1804_53EA_4152_82FB_E04185081D20__INCLUDED_)
