////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ElasticKinematics.cpp: implementation of the CElasticKinematics class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CElasticKinematics::CElasticKinematics():CKinematics()
{
	InitVectors(500);
}

CElasticKinematics::~CElasticKinematics()
{

}

bool CElasticKinematics::isPrimary(CExpression *expr)
{
	/*if (expr == NULL) return false;
	if (expr->equals(data->getv("p\\_1"))) return true;
	if (expr->equals(data->getv("p\\_11"))) return true;
	if (expr->equals(data->getv("p\\_21"))) return true;
	return false;*/
	return true;
}

CExpression* CElasticKinematics::getEqual(CExpression *expr)
{
	return CKinematics::getEqual(expr);
}

void CElasticKinematics::InitVectors(double energy)
{
	/*CERealValue* ferm_mass = new CERealValue(0);
	dbg->log(10, "Elastic kinematic initializaion");
	CENumber* zn = CENumber::GetZero();
	CEValue* zv = new CERealValue((long)0);
	CEValue* cm_energy = new CERealValue(energy/2);

	CENumVector* temp_stub = new CENumVector(zv);
	temp_stub->addItem(cm_energy->clone());
	temp_stub->addItem(cm_energy->clone());
	temp_stub->addItem(zv->clone());
	temp_stub->addItem(zv->clone());
	CPPhoton* ph1 = new CPPhoton(
		new CESymbol("p\\_1"),
		CEProcessor::getDataStorage()->gete(CConst::GL_ALPHA),
		temp_stub);
	CEParticle* temp_part = ph1;
	m_init_particles->add(temp_part);

	dbg->log(20, "initial vector: ", temp_part);

	temp_stub = new CENumVector((CEValue*)zv->clone());
	temp_stub->addItem(cm_energy->clone());
	cm_energy->inverseFlag();
	temp_stub->addItem(cm_energy);
	temp_stub->addItem(zv->clone());
	temp_stub->addItem(zv->clone());
	CPPhoton* ph2 = new CPPhoton(
		new CESymbol("p\\_2"),
		CEProcessor::getDataStorage()->gete(CConst::GL_BETA),
		temp_stub);
	temp_part = ph2;
	m_init_particles->add(temp_part);
	dbg->log(20, "initial vector: ", temp_part);

	CESymbol* f11 = new CESymbol("p\\_11");
	m_final_particles->add(new CPFermion(
		false,
		f11,
		new CESymbol("m"),
		(CEValue*)ferm_mass->clone()));//(long)175

	CESymbol* f21 = new CESymbol("p\\_21");
	m_final_particles->add(new CPFermion(
		true,
		f21,
		new CESymbol("m"),
		(CEValue*)ferm_mass->clone()));//(long)175

	delete zn;
	delete ferm_mass;

	ph1->set_helicity(1, f11, f21);
	ph2->set_helicity(-1, f11, f21);

	ph1->create_polarization_vectors_symbols();
	ph1->fill_pol_vector_stubs();
	ph2->create_polarization_vectors_symbols();
	ph2->fill_pol_vector_stubs();
	dbg->log(20, "Kinematic initialized.");*/
}

void CElasticKinematics::fill_data_storage()
{
	//s
	CENumVector* v = get_sum_initials();
	//CExpression* inv = v->square();
	data->putvalue(new CESymbol("s"), v->square(), true);

	/*CExpression* val = v->square();
	dbg->log(10, "s", val);
	delete val;*/
	delete v;

	//t
	v = vin(1)->num_stub();
	v->substvector(vfin(1), false);
	data->putvalue(new CESymbol("t"), v->square(), true);

	/*val = v->square();
	dbg->log(10, "t", val);
	delete val;*/
	delete v;

	//u
	v = vin(0)->num_stub();
	v->substvector(vfin(1), false);
	data->putvalue(new CESymbol("u"), v->square(), true);

	/*val = v->square();
	dbg->log(10, "t", val);
	delete val;*/
	delete v;

	//m
	data->putvalue(new CESymbol("m"), vfin(1)->mass(), true);

	/*val = vfin(1)->mass();
	dbg->log(10, "m", val);
	delete val;*/
}

void CElasticKinematics::set_CMS_energy(double energy)
{
	dbg->log(10, "CMS energy initializaion");
	CEValue* cm_energy = new CERealValue(energy/2);
	vin(0)->SetAt(0, cm_energy);
	vin(0)->SetAt(1, cm_energy);

	vin(1)->SetAt(0, cm_energy);
	cm_energy->inverseFlag();
	vin(1)->SetAt(1, cm_energy);
	delete cm_energy;
}

CEReplList* CElasticKinematics::getReplacements(int type)
{
	dbg->log(15, type, "CElasticKinematics::getReplacements (%ld)");
	CEReplList* res = CKinematics::getReplacements(type);

	//temporary data
	CExpression* half = data->gete(CConst::DV_HALF);
	CExpression* a = NULL;
	CExpression* b = NULL;
	CExpression* temp = NULL;

	CExpression* mi0 = pin(0)->get_sym_mass();
	CExpression* mi1 = pin(1)->get_sym_mass();
	CExpression* mf0 = pfin(0)->get_sym_mass();
	CExpression* mf1 = pfin(1)->get_sym_mass();

	CExpression* pi0 = pin(0)->get_momentum_symbol();
	CExpression* pi1 = pin(1)->get_momentum_symbol();
	CExpression* pf0 = pfin(0)->get_momentum_symbol();
	CExpression* pf1 = pfin(1)->get_momentum_symbol();

	if (type & CKinematics::REPL_SIMILAR_COUPLINGS) {

		a = CExpression::reduce(new CESProduct(pf0, pf1));
		b = new CESProduct(pi0, pi1);
		temp = new CEPower(mi0, (long)2);
		temp = temp->add(new CEPower(mi1, (long)2));
		temp = temp->minus(new CEPower(mf0, (long)2));
		temp = temp->minus(new CEPower(mf1, (long)2));
		temp = temp->multiply(half->clone());
		b = CExpression::reduce(b->add(temp));
		res->add(a, b, false);
		delete a;
		delete b;

		a = CExpression::reduce(new CESProduct(pf1, pi1));
		b = new CESProduct(pf0, pi0);
		temp = new CEPower(mf1, (long)2);
		temp = temp->add(new CEPower(mi1, (long)2));
		temp = temp->minus(new CEPower(mf0, (long)2));
		temp = temp->minus(new CEPower(mi0, (long)2));
		temp = temp->multiply(half->clone());
		b = CExpression::reduce(b->add(temp));
		//res = new CEReplList(a, b, false);
		res->add(a, b, false);
		delete a;
		delete b;

		a = CExpression::reduce(new CESProduct(pf1, pi0));
		b = new CESProduct(pf0, pi1);
		temp = new CEPower(mf1, (long)2);
		temp = temp->add(new CEPower(mi0, (long)2));
		temp = temp->minus(new CEPower(mf0, (long)2));
		temp = temp->minus(new CEPower(mi1, (long)2));
		temp = temp->multiply(half->clone());
		b = CExpression::reduce(b->add(temp));
		res->add(a, b, false);
		delete a;
		delete b;
	}
	
	if (type & CKinematics::REPL_MANDELSTAM_VARIABLES) {
		dbg->log(15, "creating MANDELSTAM replacements");
		CExpression* t = new CESymbol("t");
		CExpression* u = new CESymbol("u");

		CExpression* s = new CESymbol("s");
		//CExpression* s = CExpression::reduce(
		//	mi0->sqra()->add(mi1->sqra())->add(mf0->sqra())->add(mf1->sqra())
		//	->minus(t->clone())->minus(u->clone()));

		CKinematics::add_inv_repl_to_list(pin(0), pin(1), true, true, s, res);
		CKinematics::add_inv_repl_to_list(pfin(0), pfin(1), true, true, s, res);
		CKinematics::add_inv_repl_to_list(pin(0), pfin(0), true, false, t, res);
		CKinematics::add_inv_repl_to_list(pin(1), pfin(1), true, false, t, res);
		CKinematics::add_inv_repl_to_list(pfin(0), pin(1), true, false, u, res);
		CKinematics::add_inv_repl_to_list(pin(0), pfin(1), true, false, u, res);

		delete s;
		delete t;
		delete u;

		dbg->log(15, "MANDELSTAM replacements created");
	}
	
	if (type & CKinematics::REPL_ALL_MANDELSTAM_X) {
		CExpression* t = new CESymbol("t");
		CExpression* u = new CESymbol("u");
		CExpression* s = new CESymbol("s");

		CExpression* b = CExpression::reduce(
			mi0->sqra()->add(mi1->sqra())->add(mf0->sqra())->add(mf1->sqra())
			->minus(t->clone())->minus(u->clone()));
		res->add(s, b, false);
		delete b;
		
		b = CExpression::reduce(
			mi0->sqra()->add(mi1->sqra())->add(mf0->sqra())->add(mf1->sqra())
			->minus(s->clone())->minus(u->clone()));
		res->add(t, b, false);
		delete b;

		b = CExpression::reduce(
			mi0->sqra()->add(mi1->sqra())->add(mf0->sqra())->add(mf1->sqra())
			->minus(s->clone())->minus(t->clone()));
		res->add(u, b, false);
		delete b;

		delete s;
		delete t;
		delete u;
	}

	return res;
}
