////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// FourParticleKinematics.cpp: implementation of the CFourParticleKinematics class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFourParticleKinematics::CFourParticleKinematics()
{
	InitVectors(120);
}

CFourParticleKinematics::~CFourParticleKinematics()
{

}

bool CFourParticleKinematics::isPrimary(CExpression *expr)
{
	return true;
}

CExpression* CFourParticleKinematics::getEqual(CExpression *expr)
{
	return CKinematics::getEqual(expr);
}

void CFourParticleKinematics::InitVectors(double energy)
{
	/*dbg->log(10, "4-particle kinematic initializaion");
	CEValue* ferm_mass = new CERealValue(0);

	CENumber* zn = CENumber::GetZero();
	CEValue* zv = new CERealValue(0);
	CEValue* cm_energy = new CERealValue(energy/2);

	///////////////////////////////////////////	in 1
	CENumVector* temp_stub = new CENumVector(zv);
	temp_stub->addItem(cm_energy->clone());
	temp_stub->addItem(cm_energy->clone());
	temp_stub->addItem(zv->clone());
	temp_stub->addItem(zv->clone());
	CPPhoton* ph1 = new CPPhoton(
		new CESymbol("k\\_1"),
		CEProcessor::getDataStorage()->gete(CConst::GL_ALPHA),
		temp_stub);
	CEParticle* temp_part = ph1;
	m_init_particles->add(temp_part);
	dbg->log(20, "initial vector: ", temp_part);

	///////////////////////////////////////////	in 2
	temp_stub = new CENumVector((CEValue*)zv->clone());
	temp_stub->addItem(cm_energy->clone());
	cm_energy->inverseFlag();
	temp_stub->addItem(cm_energy);
	temp_stub->addItem(zv->clone());
	temp_stub->addItem(zv->clone());
	CPPhoton* ph2 = new CPPhoton(
		new CESymbol("k\\_2"),
		CEProcessor::getDataStorage()->gete(CConst::GL_BETA),
		temp_stub);
	temp_part = ph2;
	m_init_particles->add(temp_part);
	dbg->log(20, "initial vector: ", temp_part);

	///////////////////////////////////////////	fin 1
	CESymbol* f1 = new CESymbol("p\\_1");
	m_final_particles->add(new CPFermion(
		false,
		f1,
		new CESymbol("m"),
		(CEValue*)ferm_mass->clone()));

	///////////////////////////////////////////	fin 2
	CESymbol* f2 = new CESymbol("p\\_2");
	m_final_particles->add(new CPFermion(
		true,
		f2,
		new CESymbol("m"),
		(CEValue*)ferm_mass->clone()));

	///////////////////////////////////////////	fin 3
	CESymbol* f3 = new CESymbol("p\\_3");
	m_final_particles->add(new CPFermion(
		false,
		f3,
		new CESymbol("m"),
		(CEValue*)ferm_mass->clone()));

	///////////////////////////////////////////	fin 4
	CESymbol* f4 = new CESymbol("p\\_4");
	m_final_particles->add(new CPFermion(
		true,
		f4,
		new CESymbol("m"),
		(CEValue*)ferm_mass->clone()));

	delete zn;
	delete ferm_mass;

	///////////////////////////////////////////	photon internal initailization
	ph1->set_helicity(0, NULL, NULL);
	ph1->create_polarization_vectors_symbols();
	ph1->fill_pol_vector_stubs();

	ph2->set_helicity(0, NULL, NULL);
	ph2->create_polarization_vectors_symbols();
	ph2->fill_pol_vector_stubs();

	dbg->log(20, "Kinematic initialized.");*/
}

void CFourParticleKinematics::fill_data_storage()
{
	//m
	data->putvalue(new CESymbol("m"), vfin(1)->mass(), true);
}

void CFourParticleKinematics::set_CMS_energy(double energy)
{
	dbg->log(10, "CMS energy initializaion");
	CEValue* cm_energy = new CERealValue(energy/2);
	vin(0)->SetAt(0, cm_energy);
	vin(0)->SetAt(1, cm_energy);

	vin(1)->SetAt(0, cm_energy);
	cm_energy->inverseFlag();
	vin(1)->SetAt(1, cm_energy);
	delete cm_energy;
}

CEReplList* CFourParticleKinematics::getReplacements(int type)
{
	CEReplList* res = CKinematics::getReplacements(type);

	//CExpression* half = data->gete(CConst::DV_HALF);

	//CExpression* mi0 = pin(0)->get_sym_mass();
	//CExpression* mi1 = pin(1)->get_sym_mass();
	//CExpression* mf0 = pfin(0)->get_sym_mass();
	//CExpression* mf1 = pfin(1)->get_sym_mass();
	//CExpression* mf2 = pfin(2)->get_sym_mass();

	//CExpression* pi0 = pin(0)->get_momentum_symbol();
	//CExpression* pi1 = pin(1)->get_momentum_symbol();
	//CExpression* pf0 = pfin(0)->get_momentum_symbol();
	//CExpression* pf1 = pfin(1)->get_momentum_symbol();
	//CExpression* pf2 = pfin(2)->get_momentum_symbol();

	if (type & CKinematics::REPL_SIMILAR_COUPLINGS) {
	}
	
	if (type & CKinematics::REPL_MANDELSTAM_VARIABLES) {
	}
	
	if (type & CKinematics::REPL_ALL_MANDELSTAM_X) {
	}

	return res;
}
