////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// InelasticKinematics.cpp: implementation of the CInelasticKinematics class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CInelasticKinematics::CInelasticKinematics()
{
	InitVectors(500);
}

CInelasticKinematics::~CInelasticKinematics()
{

}

bool CInelasticKinematics::isPrimary(CExpression *expr)
{
	return true;
}

CExpression* CInelasticKinematics::getEqual(CExpression *expr)
{
	return CKinematics::getEqual(expr);
}

void CInelasticKinematics::InitVectors(double energy)
{
	/*dbg->log(10, "Inelastic kinematic initializaion");
	CEValue* ferm_mass = new CERealValue(0);

	CENumber* zn = CENumber::GetZero(0);
	CEValue* zv = new CERealValue(0);
	CEValue* cm_energy = new CERealValue(energy/2);

	CENumVector* temp_stub = new CENumVector(zv);
	temp_stub->addItem(cm_energy->clone());
	temp_stub->addItem(cm_energy->clone());
	temp_stub->addItem(zv->clone());
	temp_stub->addItem(zv->clone());
	CPPhoton* ph1 = new CPPhoton(
		new CESymbol("p\\_1"),
		CEProcessor::getDataStorage()->gete(CConst::GL_ALPHA),
		temp_stub);
	CEParticle* temp_part = ph1;
	m_init_particles->add(temp_part);
	dbg->log(20, "initial vector: ", temp_part);

	temp_stub = new CENumVector((CEValue*)zv->clone());
	temp_stub->addItem(cm_energy->clone());
	cm_energy->inverseFlag();
	temp_stub->addItem(cm_energy);
	temp_stub->addItem(zv->clone());
	temp_stub->addItem(zv->clone());
	CPPhoton* ph2 = new CPPhoton(
		new CESymbol("p\\_2"),
		CEProcessor::getDataStorage()->gete(CConst::GL_BETA),
		temp_stub);
	temp_part = ph2;
	m_init_particles->add(temp_part);
	dbg->log(20, "initial vector: ", temp_part);
	
	CESymbol* f11 = new CESymbol("p\\_11");
	m_final_particles->add(new CPFermion(
		false,
		f11,
		new CESymbol("m"),
		(CEValue*)ferm_mass->clone()));

	CESymbol* f21 = new CESymbol("p\\_21");
	m_final_particles->add(new CPFermion(
		true,
		f21,
		new CESymbol("m"),
		(CEValue*)ferm_mass->clone()));

	m_final_particles->add(new CPPhoton(
		new CESymbol("k"),
		CEProcessor::getDataStorage()->gete(CConst::GL_CHI)));

	delete zn;
	delete ferm_mass;

	ph1->set_helicity(1, f11, f21);
	ph2->set_helicity(1, f11, f21);

	ph1->create_polarization_vectors_symbols();
	ph1->fill_pol_vector_stubs();
	ph2->create_polarization_vectors_symbols();
	ph2->fill_pol_vector_stubs();
	dbg->log(20, "Kinematic initialized.");*/
}

void CInelasticKinematics::fill_data_storage()
{
	//s
	CENumVector* v = get_sum_initials();
	//CExpression* inv = v->square();
	data->putvalue(new CESymbol("s"), v->square(), true);
	delete v;

	//s'
	v = vfin(0)->num_stub();
	v->addvector(vfin(1), false);
	data->putvalue(new CESymbol("s1"), v->square(), true);
	delete v;

	//t
	v = vin(1)->num_stub();
	v->substvector(vfin(1), false);
	data->putvalue(new CESymbol("t"), v->square(), true);
	delete v;

	//t'
	v = vin(0)->num_stub();
	v->substvector(vfin(0), false);
	data->putvalue(new CESymbol("t1"), v->square(), true);
	delete v;

	//u
	v = vin(0)->num_stub();
	v->substvector(vfin(1), false);
	data->putvalue(new CESymbol("u"), v->square(), true);
	delete v;

	//u'
	v = vin(1)->num_stub();
	v->substvector(vfin(0), false);
	data->putvalue(new CESymbol("u1"), v->square(), true);
	delete v;

	//z
	v = vin(0)->num_stub();
	v->addvector(vfin(2), false);
	data->putvalue(new CESymbol("z"), v->square(), true);
	delete v;

	//z'
	v = vin(1)->num_stub();
	v->addvector(vfin(2), false);
	data->putvalue(new CESymbol("z1"), v->square(), true);
	delete v;

	//w
	v = vfin(0)->num_stub();
	data->putvalue(new CESymbol("w"), v->couple(vfin(2), false), true);
	delete v;

	//v
	v = vfin(1)->num_stub();
	data->putvalue(new CESymbol("v"), v->couple(vfin(2), false), true);
	delete v;

	//m
	data->putvalue(new CESymbol("m"), vfin(1)->mass(), true);
}

void CInelasticKinematics::set_CMS_energy(double energy)
{
	dbg->log(10, "CMS energy initializaion");
	CEValue* cm_energy = new CERealValue(energy/2);
	vin(0)->SetAt(0, cm_energy);
	vin(0)->SetAt(1, cm_energy);

	vin(1)->SetAt(0, cm_energy);
	cm_energy->inverseFlag();
	vin(1)->SetAt(1, cm_energy);
	delete cm_energy;
}

CEReplList* CInelasticKinematics::getReplacements(int type)
{
	CEReplList* res = CKinematics::getReplacements(type);

	//temporary data
	//CExpression* half = data->gete(CConst::DV_HALF);
	//CExpression* a = NULL;
	//CExpression* b = NULL;
	//CExpression* temp = NULL;

	//CExpression* mi0 = pin(0)->get_sym_mass();
	//CExpression* mi1 = pin(1)->get_sym_mass();
	//CExpression* mf0 = pfin(0)->get_sym_mass();
	//CExpression* mf1 = pfin(1)->get_sym_mass();
	//CExpression* mf2 = pfin(2)->get_sym_mass();

	CExpression* pi0 = pin(0)->get_momentum_symbol();
	CExpression* pi1 = pin(1)->get_momentum_symbol();
	CExpression* pf0 = pfin(0)->get_momentum_symbol();
	CExpression* pf1 = pfin(1)->get_momentum_symbol();
	CExpression* pf2 = pfin(2)->get_momentum_symbol();

	if (type & CKinematics::REPL_SIMILAR_COUPLINGS) {
	}
	
	if (type & CKinematics::REPL_MANDELSTAM_VARIABLES) {
		CExpression* t = new CESymbol("t");
		CExpression* u = new CESymbol("u");
		CExpression* s = new CESymbol("s");

		CExpression* t1 = new CESymbol("t1");
		CExpression* u1 = new CESymbol("u1");
		CExpression* s1 = new CESymbol("s1");

		CExpression* v = new CESymbol("v");
		CExpression* w = new CESymbol("w");
		CExpression* z = new CESymbol("z");
		CExpression* z1 = new CESymbol("z1");

		CKinematics::add_inv_repl_to_list(pin(0), pin(1), true, true, s, res);
		CKinematics::add_inv_repl_to_list(pfin(0), pfin(1), true, true, s1, res);
		CKinematics::add_inv_repl_to_list(pin(0), pfin(0), true, false, t1, res);
		CKinematics::add_inv_repl_to_list(pin(1), pfin(1), true, false, t, res);
		CKinematics::add_inv_repl_to_list(pfin(0), pin(1), true, false, u1, res);
		CKinematics::add_inv_repl_to_list(pin(0), pfin(1), true, false, u, res);

		CKinematics::add_inv_repl_to_list(pin(0), pfin(2), false, false, z, res);
		CKinematics::add_inv_repl_to_list(pin(1), pfin(2), false, false, z1, res);
		CKinematics::add_inv_repl_to_list(pfin(0), pfin(2), false, false, w, res);
		CKinematics::add_inv_repl_to_list(pfin(1), pfin(2), false, false, v, res);

		delete s;
		delete t;
		delete u;

		delete s1;
		delete t1;
		delete u1;

		delete z;
		delete z1;
		delete v;
		delete w;
	}
	
	if (type & CKinematics::REPL_ALL_MANDELSTAM_X) {
	}

	return res;
}
