////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// DenCacheLib.cpp: implementation of the CDenCacheLib class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

CDebug* CDenCacheLib::dbg = new CDebug("CDenCacheLib");

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CDenCacheLib::CDenCacheLib() : CELibrary()
{
	m_denominators_cache = NULL;
	m_simpl_rule = NULL;
	m_isSimplified=false;
}

CDenCacheLib::~CDenCacheLib()
{
	if (m_denominators_cache != NULL) delete m_denominators_cache;
	if (m_simpl_rule != NULL) delete m_simpl_rule;
}

const char* CDenCacheLib::CLASS_NAME = "CDenCacheLib";
const char* CDenCacheLib::ClassName() const { return CLASS_NAME; }
bool CDenCacheLib::contain_vector(void * obj) {
	if (m_denominators_cache == obj) return true;
	if (m_denominators_cache != NULL) { if(m_denominators_cache->contain_vector(obj)) return true; }
	return false;
}

void CDenCacheLib::AddSimplRule(CERule* r) { // rule object is unaffected
	if (r == NULL) return;

	m_isSimplified=false;
	if (m_simpl_rule == NULL) {
		m_simpl_rule = r->GetRuleCopy();
	} else {
		m_simpl_rule = m_simpl_rule->combine_with(r->GetRuleCopy());
	}
}

CExpression* CDenCacheLib::InvolveDenominatorsCache(CExpression* val) {
	dbg->log(25, "InvolveDenominatorsCache()", val);
	if (val == NULL) return NULL;
	if (m_denominators_cache == NULL) m_denominators_cache = new CEPtrArray();

	int mass_policy_flag = 0; //((allow_mass_simp)?D_CACHE_ALLOW_MASS_SIMPLIFICATION:0);
	CRReplaceDenominators* r = new CRReplaceDenominators(
		CRReplaceDenominators::REPL_DENS_DEFAULT, m_denominators_cache, mass_policy_flag);
	CExpression* res = r->ApplyTo(val, true);
	CEPtrArray* new_dens_arr = r->getReplacedValues();
	if (new_dens_arr->size() != m_denominators_cache->size()) {
		// update denominators cache array
		delete m_denominators_cache; m_denominators_cache = new_dens_arr->clone();
		m_isSimplified=false;
		dbg->log(45, "denominators cache updated:", m_denominators_cache);
	}
	delete r;
	dbg->log(45, "expression denominator cached: ", res);
	return res;
}

void CDenCacheLib::SimplifyCachedDenominators() {
	dbg->log(35, "SimplifyCachedDenominators()");
	if (m_isSimplified) return;
	dbg->log(75, "dens cache before simplification", m_denominators_cache);
	
	long s = m_denominators_cache->size();
	long i;
	for(i=0; i<s; i++) {
		CExprContainer* cart = (CExprContainer*)m_denominators_cache->GetAt(i);
		CExpression* d = cart->get_expr()->clone();
		int setts = cart->get_info();
		
		// 1. simplify
		d = CExpression::reduce(d);

		// 2. involve kinematics identities & other rules
		if (m_simpl_rule != NULL) {
			d = m_simpl_rule->ApplyTo(d);
			d = CExpression::reduce(d);
		}

		cart->set_expr(d);
		delete d;
	}

	m_isSimplified=true;
	dbg->log(35, "denominators simplified:", m_denominators_cache);
}

CExpression* CDenCacheLib::RestoreDenominators(CExpression* val) {
	dbg->log(25, "restore_denominators()");
	if ((m_denominators_cache == NULL) || (m_denominators_cache->size() == 0)) return val;

	long s = m_denominators_cache->size();
	CEReplList* r = new CEReplList();
	long i;
	for(i=0; i<s; i++) {
		CExprContainer* cart = (CExprContainer*)m_denominators_cache->GetAt(i);
		r->add(cart->get_info_expr(), cart->get_expr());
	}
	val = r->ApplyTo(val);
	delete r;

	//val = CExpression::arrge(val);
	dbg->logperiod(25, "Denominators restored.");
	return val;
}
