////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// LFactor.cpp: implementation of the CLFactor class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDebug* CLFactor::dbg = new CDebug("CLFactor");

const DWORD CLFactor::DONOT_FACT_NUMBERS		= 0x01;
const DWORD CLFactor::DONOT_FACT_FRACTIONS		= 0x02; // affects numeric fractions only
const DWORD CLFactor::FACT_ALL_DENOMINATORS		= 0x04;
const DWORD CLFactor::FACT_ALL_SINGLE_DENS		= 0x08;
const DWORD CLFactor::FACT_VERIFY				= 0x10;
const DWORD CLFactor::NO_EXPAND					= 0x20; // forced & not checked

CLFactor::CLFactor()
{

}

CLFactor::~CLFactor()
{

}

/*
*	source is supposed to be reduced.
*	      .
*/
CExpression* CLFactor::factor_simple(CExpression *source) //OLD
{
	dbg->log(0,"WARNING: OLD code involved {CLFactor::factor_simple()}");
	if (source == NULL) return NULL;
	dbg->log(20, "factor_simple", source);

	if (!(source->getProperty(CEProperties::PROP_MASK_IS_SUM))) {
		return source;
		//	Further it should recursively factorize items in products, powers, etc.
	}

	CESum* sum = (CESum*)source;
	CExpression* unit = data->gete(CConst::DV_UNIT);
	CENumber* num_factor = NULL;
	CEProduct* it_factor = NULL;
	long s = sum->size();
	//bool it_f_have_numerator = true;// or (it_factor == NULL)
	//bool do_num_factor_calc = true;//((num_factor == NULL) || (!(unit->equals(num_factor))))

	//	*****************************************************	<scroll>
	bool do_scroll = (s > 1000);
	if (do_scroll) {printf("Factor searching in long sum (%d items)\n", s);
		printf("0%%........................................100%%\n  ");}
	long scr_step = 1; long scr_curr = 0;
	//	*****************************************************	<\scroll>

	// (factor == NULL) means no factor involved yet;
	// (factor->size() == 0) means no factiorization available
	long i;
	for (i=0; i<s; i++) {
		CExpression* item = sum->GetAt(i);
		if (item->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
			//CEProduct* it_f_new = new CEProduct();
			CEProduct* pr = (CEProduct*)item;
			int pr_s = pr->size();
			bool first_loop = false;

			//	1. it_factor;
			//	deletes from it_factor numerators that are absent in current item;
			if (it_factor != NULL) {
				long j;
				for(j=0; j<it_factor->size(); j++) {
					CExpression* it_f_item = it_factor->GetAt(j);
					if (it_f_item->getLevel()) {
						int pos = pr->get_item_pos(it_f_item, false);
						if (pos < 0) {it_factor->RemoveAt(j);j--;}
					}
				}
				//if (it_factor->size() == 0) it_f_have_numerator = false;
			}
			else {
				it_factor = new CEProduct();
				first_loop = true;
			}

			//	2. num_factor && (it_factor == NULL);
			int j;
			for(j=0; j<pr_s; j++) {
				CExpression* pr_item = pr->GetAt(j);
				if (pr_item->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) {
					num_factor = calc_num_factor(num_factor, (CENumber*)pr_item);
					//if ((num_factor != NULL) && (unit->equals(num_factor))) do_num_factor_calc = false;
				}
				else {
					if (first_loop) {
						CExpression* pr_it_cln = pr_item->clone();
						pr_it_cln->setFlag();
						it_factor->addItem(pr_it_cln);
					}
					else {
						if (!pr_item->getLevel()) {
							if ((it_factor->get_item_pos(pr_item, false)) < 0) {
								CExpression* pr_it_cln = pr_item->clone();
								pr_it_cln->setFlag();
								it_factor->addItem(pr_it_cln);
							}
						}
					}
				}
			}
		}
		else {
			if (item->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) {
				//if (do_num_factor_calc) {
				num_factor = calc_num_factor(num_factor, (CENumber*)item);
				//if ((num_factor != NULL) && (unit->equals(num_factor))) do_num_factor_calc = false;
				//}
			}
			else {
				if (it_factor == NULL) {
					it_factor = new CEProduct();
					CExpression* it_cln = item->clone();
					it_cln->setFlag();
					it_factor->addItem(it_cln);
				}
				else {
					int pos = it_factor->get_item_pos(item, false);
					// if factor equals item do nothing.
					if (!((pos == 0) && (it_factor->size() == 1))) {
						// else
						//clear all numerators from it_factor except (j == pos)
						int j;
						for(j=(it_factor->size()-1); j>=0; j--) {
							CExpression* it_f_item = it_factor->GetAt(j);
							if ((j != pos) && (it_f_item->getLevel())) {it_factor->RemoveAt(j);}
						}

						// if item is denominator && not in factor -- add it.
						if ((pos < 0) && (!item->getLevel())) {
							CExpression* it_cln = item->clone();
							it_cln->setFlag();
							it_factor->addItem(it_cln);
						}
					}
				}
			}
		}
		
		///if ((!do_it_factor_calc) && (!do_num_factor_calc)) i=s;// exit loops.
		// no exits from loop: in any item may occur the unique denominator!

		//	*****************************************************	<scroll>
		if (do_scroll) {if (scr_curr < (((i+1)*40)/s))  {scr_curr += scr_step; printf("#");}}
		//	*****************************************************	<\scroll>
	}

	//	*****************************************************	<scroll>
	if (do_scroll) {printf("\n");}
	//	*****************************************************	<\scroll>

	dbg->log(20, "it_factor", it_factor);
	dbg->log(20, "num_factor", num_factor);

	if (num_factor != NULL) {
		if (unit->equals(num_factor)) {delete num_factor; num_factor = NULL;}
	}
	if (it_factor != NULL) {
		if ((unit->equals(it_factor)) || (it_factor->size() == 0))
		{delete it_factor; it_factor = NULL;}
	}

	if ((num_factor == NULL) && (it_factor == NULL)) return source;

	CEProduct* total_factor = NULL;
	if (it_factor != NULL) total_factor = it_factor;
	else total_factor = new CEProduct();
	if (num_factor != NULL) {
		total_factor->InsertAt(0,num_factor);
		delete num_factor;
	}

	bool lev = source->getLevel();
	if (!lev) source->setLevel(true);
	//	*****************************************************	<scroll>
	if (do_scroll) {printf("Calculating rest sum\n");}
	//	*****************************************************	<\scroll>
	CExpression* sum_new = source->divide(total_factor->clone());
	sum_new = CExpression::reduce(sum_new);
	total_factor->addItem(sum_new);
	if (!lev) total_factor->inverseLevel();//setLevel(false);

	//	*****************************************************	<scroll>
	if (do_scroll) {printf("Factorization complete.\n");}
	//	*****************************************************	<\scroll>

	dbg->log(20, "factor_simple result:", total_factor);
	return total_factor;
}

/*
*	1. What about sign?
*	2. What about complex numbers factorization?
*/
CENumber* CLFactor::calc_num_factor(CENumber *num_factor_old, CENumber *num)//OLD
{
	if (num == NULL) return num_factor_old;
	if (num_factor_old == NULL) {
		CENumber* res = (CENumber*)num->clone();
		return res;
	}

	CExpression* unit = data->gete(CConst::DV_UNIT);
	if (unit->equal_or_opposite(num_factor_old)) return num_factor_old;
	if (unit->equal_or_opposite(num)) {
		delete num_factor_old;
		return (CENumber*)unit->clone();
	}

	long den_1 = num_factor_old->getDen();
	long num_1 = num_factor_old->getNum();
	long im_1 = num_factor_old->getNumIm();//0
	if (im_1 != 0) {printf("Error in <CLFactor::calc_num_factor>");delete num_factor_old; return (CENumber*)unit->clone();}

	long den_2 = num->getDen();
	long num_2 = num->getNum();
	long im_2 = num->getNumIm();

	long num_fact_2 = CLMath::greatest_common_divisor(num_2, im_2);//=num_2 if (im_2==0);

	long den_res = CLMath::least_common_multiple(den_1, den_2);
	long num_res = CLMath::greatest_common_divisor(num_1, num_fact_2);
	long im_res = 0;//0
	CENumber* res = new CENumber(num_res, im_res, den_res);
	res = (CENumber*)CExpression::arrge(res);
	delete num_factor_old;
	return res;
}

CExpression* CLFactor::factorize(CExpression *source, DWORD settings)
{
	if (source == NULL) return NULL;
	dbg->log(10, settings, "factorize (%ld)", source);

	CExpression* source_copy = NULL;
	if (settings & FACT_VERIFY) source_copy = source->clone();

	// do not verify during the calculation -- there will be the common verification
	DWORD setts_to_impl = ((settings | FACT_VERIFY) ^ FACT_VERIFY);
	CExpression* res = factor_impl(source, setts_to_impl);
	
	// verification
	if (settings & FACT_VERIFY) {
		dbg->log(25, "Factorize Verification");
		//printf("Factorize Verification... ");
		if (res == NULL) {
			printf("<Error> Factorize Verification fails: NULL result.\n");
			dbg->log(0, "<Error> Factorize Verification fails: NULL result.");
			dbg->log(0, "<Error> Source was:", source_copy);
		}
		else {
			CExpression* res_reduced = CExpression::reduce(res->clone());
			bool verify_OK = source_copy->equals(res_reduced);
			if (!verify_OK) {
				// perhaps sourse is not reduced
				source_copy = CExpression::reduce(source_copy);
				verify_OK = source_copy->equals(res_reduced);
			}
			delete res_reduced;
			if (verify_OK) {
				dbg->log(55, "Verification successful");
			} else {
				printf("<Error> Factorize Verification fails.\n");
				dbg->log(0, "<Error> Factorize Verification fails.");
				dbg->log(0, "<Error> Source was:", source_copy);
				dbg->log(0, "<Error> Result:", res);
			}
		}
		delete source_copy;
	}

	dbg->log(10, "factorize result", res);
	return res;
}

CExpression* CLFactor::factor_impl(CExpression *source, DWORD settings) {
	dbg->log(55, settings, "factor_impl(%ld)", source);
	if (source == NULL) return NULL;

	//	0. if source is not sum return (do not deep to subobjects)
	if (!source->getProperty(CEProperties::PROP_MASK_IS_SUM)) return source;

	//  1. special cases
	CESum* sum = (CESum*)source;
	if (sum->size() == 1) {
		CExpression* res = sum->GetAt(0)->clone();
		sum->copyAttributesTo(res, true);
		delete sum;
		return factor_impl(res, settings);	// process item #0
	}
	else if (sum->size() == 0) {	//error: how can it occur?
		dbg->log(0, "<Error> sum size == 0");
		delete sum;
		return NULL;
	}

	//  2. factor determination
	CEProduct* factor = NULL;
	//bool stop_if_null_factor = ((settings & DONOT_FACT_FRACTIONS) != 0);
	bool stop_if_null_factor = ((settings & (FACT_ALL_DENOMINATORS|FACT_ALL_SINGLE_DENS)) == 0);
	long s = sum->size();
	dbg->log(20, s, "sum is valid, total %ld items");
	//dbg->log(50, stop_if_null_factor, "stop_if_null_factor == %ld");
	long i;
	for(i=0; i<s; i++) {
		CExpression* item = sum->GetAt(i);
		factor = update_factor(item, factor, settings);

		dbg->log(30, i, "factor after %ld items", factor);
		if (stop_if_null_factor) {
			if (factor->size() == 0) i = s;
		}
	}

	if (CEBasic::isZeroProd(factor)) { if (factor!=NULL) { delete factor; factor=NULL; } }
	dbg->log(20, "factor calculated", factor);

	//  3. extracting factor IF IT EXIST
	CExpression* res = NULL;
	if ((factor != NULL) && (factor->size() > 0)) {
		bool lev = source->getLevel();
		if (!lev) source->setLevel(true);

		// OLD factorization:
		// if source is sum & denominators this may fail the factorization.
		// CExpression* sum_rest = source->divide(factor->clone());
		// sum_rest = CExpression::reduce(sum_rest);
		
		s = sum->size();
		for(i=0; i<s; i++) {
			CExpression* item = sum->GetAt(i);
			CExpression* item_new = item->clone()->divide(factor->clone());
			item_new = CExpression::arrge(item_new);
			sum->SetAt(i, item_new);
			delete item_new;
		}
		CExpression* sum_rest=sum;
		//*************************************** NO_EXPAND option is forced
		//if ((settings & NO_EXPAND) == 0) {
		//	sum_rest = CExpression::reduce(sum_rest);
		//} else {
		//	sum_rest = CExpression::arrge(sum_rest);
		//}
		sum_rest = CExpression::arrge(sum_rest);

		factor->addItem(sum_rest);
		if (!lev) factor->inverseLevel();//setLevel(false);
		res = factor;
	}
	else {
		if (factor != NULL) delete factor;
		res = source;
	}

	return res;
}

/*
*	factor == NULL means no factor exist yet (parsing first item)
*	do not affect item
*/
CEProduct* CLFactor::update_factor(CExpression *item, CEProduct *factor, DWORD settings)
{
	dbg->log(85, "update_factor ()", item);
	if (factor == NULL) {
		CExpression* first_item = item->clone();
		// some arrangment
		first_item->Arrange(AL_INLINE);
		if (!first_item->getFlag()) first_item->setFlag();
		// we should return product
		CEProduct* first_item_prod = NULL;
		if (first_item->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
			first_item_prod = (CEProduct*)first_item;
		}
		else {
			first_item_prod = new CEProduct();
			first_item_prod->addItem(first_item);
		}
		return first_item_prod;
	}

	// extract common factor (also and numerical)
	CEProduct* prod_common = NULL;
	CExpression* item_left = NULL;
	//bool item_is_cloned = false;
	bool item_left_to_delete = false;

	if (factor->size() > 0) {
		prod_common = new CEProduct();
		item = item->clone();
		item_left = extract_common_factors(item->clone(), factor,
			prod_common, settings);
		//item_is_cloned = true;// ??
		if (item_left != item) delete item;
		item_left_to_delete = true;
	}
	else {
		prod_common = factor;
		item_left = item;
	}

	// parsing rest for denominators
	if (settings & (FACT_ALL_DENOMINATORS|FACT_ALL_SINGLE_DENS)) {
		if (prod_common != factor) {
			long s = factor->size();
			long i;
			for(i=0; i<s; i++) {
				CExpression* f_item = factor->GetAt(i);
				if (!f_item->getLevel()) {
					if(allow_fact(f_item, settings)) prod_common->addItem(f_item->clone());
				}
			}
		}
		
		if (item_left != NULL) {
			bool it_left_level = item_left->getLevel();
			if (item_left->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
				CEProduct* pr_it_left = (CEProduct*)item_left;
				long s = pr_it_left->size();
				long i;
				for(i=0; i<s; i++) {
					CExpression* l_item = pr_it_left->GetAt(i);
					bool itl_item_lev = l_item->getLevel();
					if (CEUtil::LXOR(it_left_level, itl_item_lev)) {
						if(allow_fact(l_item, settings)) {
							l_item = l_item->clone();
							if (itl_item_lev) l_item->inverseLevel(); // item should be "-"-leveled.
							prod_common->addItem(l_item);
						}
					}
				}
			}
			else if(!it_left_level) {
				if (allow_fact(item_left, settings)) {
					if (!item_left_to_delete) item_left = item_left->clone();
					else item_left_to_delete = false;
					prod_common->addItem(item_left);
				}
			}
		}
		dbg->log(85, "denominators appended", prod_common);
	}

	if ((item_left_to_delete) && (item_left != NULL)) {
		delete item_left;
	}

	if (prod_common != factor) delete factor;

	// arrange result
	prod_common->Arrange(AL_INLINE);
	return prod_common;
}

// note: settings should have FACT_ALL_SINGLE_DENS or FACT_ALL_DENOMINATORS flag
bool CLFactor::allow_fact(CExpression *expr, DWORD settings) {
	if(settings & FACT_ALL_DENOMINATORS) return true;

	//FACT_ALL_SINGLE_DENS elswhere
	if(expr->getProperty(CEProperties::PROP_MASK_IS_SUM)) return false;
	if(CEUtil::equals(expr->ClassName(), CEPower::CLASS_NAME)) {
		if (((CEPower*)expr)->basis()->getProperty(CEProperties::PROP_MASK_IS_SUM)) return false;
	}
	return true;
}

/*
*	deletes similar factors in "expr_a" and "prod_b"
*	the rest of "expr_a" is returned.
*	NULL if nothing remains
*	"prod_b" product becomes shorter.
*	common factors are extracted to "prod_common" product
*	if prod_common == NULL it is not filled
*	initial expr_a is deleted
*
*	factor is always reduced (i.e. one can use arrangement of sequence)
*
*	//WARNING: NUMERIC VALUES ARE NOT EXTRACTED BUT NOT ALWAYS DELETED IN SOURCES
*	//example: 1/2, 1 ==> (factor)1/2, 1, 1 instead of (1/2), 1, 2
*	//(see "// !!!!!" line heredown)
*	//I.e. this acn be used for DETECTING factor only, but not for extraction
*	
*	IT'S OK NOW!
*/
CExpression* CLFactor::extract_common_factors(CExpression *expr_a,
											  CEProduct *prod_b,
											  CEProduct *prod_common,
											  DWORD settings)
{
	if ((expr_a == NULL) || (prod_b == NULL)) {
		if (expr_a != NULL) delete expr_a;
		return NULL;
	}

	if (dbg->get_level() >= 80) {
		dbg->log(85, "extract_common_factors, expr_a", expr_a);
		dbg->log(85, "extract_common_factors, prod_b", prod_b);
		dbg->log(85, "extract_common_factors, prod_common", prod_common);
	}
	
	CExpression* res = extract_common_factors_impl(expr_a, prod_b, prod_common, settings);

	if (dbg->get_level() >= 80) {
		dbg->log(85, "after extract_common_factors, prod_common", prod_common);
		dbg->log(85, "after extract_common_factors, prod_b", prod_b);
		dbg->log(85, "extract_common_factors, result", res);
	}

	return res;
}

CExpression* CLFactor::extract_common_factors_impl(CExpression *expr_a,
												   CEProduct *prod_b,
												   CEProduct *prod_common,
												   DWORD settings)
{
	dbg->log(85, "extract_common_factors_impl", expr_a);
	if (prod_b->size() < 1) return expr_a;

	if (expr_a->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
		CEProduct* prod_a = (CEProduct*)expr_a;
		long i;
		for(i=0; i<prod_a->size(); i++) {
			CExpression* prod_a_item = prod_a->GetAt(i)->clone();
			CExpression* extr_res = extract_common_factors_impl(prod_a_item,
				prod_b, prod_common, settings);

			dbg->log(85, "extr_res", extr_res);
			if (extr_res != prod_a_item) { // prod_a_item should be already deleted
				if (extr_res == NULL) {// means unit
					prod_a->RemoveAt(i);
					i--;
				} else {
					prod_a->SetAt(i, extr_res);
					delete extr_res;
				}
			} else {
				// extr_res may not coinside to prod_a->GetAt(i) (e.g. reduced power)
				if (!(prod_a->GetAt(i)->equals(extr_res))) {
					prod_a->SetAt(i, extr_res);
					//dbg->log(0, "Test point #1", extr_res);
					//dbg->log(0, "Test point #1", prod_a->GetAt(i));
				}
				delete extr_res;
			}

			if (prod_b->size() < 1) break;
		}

		// simplify remains
		if (prod_a->size() == 0) {
			delete prod_a;
			return NULL;
		}// else if (prod_a->size() == 1) {... + copyAttributes()}		?
		return prod_a;
	}

	// numbers
	if (((settings & DONOT_FACT_NUMBERS) == 0) &&
		(expr_a->getProperty(CEProperties::PROP_MASK_IS_NUMBER))) {

		// copmare number to first item in prod_b
		CENumber* num_a = (CENumber*)expr_a;
		CENumber* num_b = NULL;

		CExpression* pb_0 = prod_b->GetAt(0);
		if (pb_0->getProperty(CEProperties::PROP_MASK_IS_NUMBER))
			num_b = (CENumber*)pb_0;

		CENumber* num_res = get_greatest_common_divisor(num_a, num_b, settings, true);

		if (num_res != NULL) {
			CExpression* ret_val = NULL;

			// there exist common divisor != 1
			if (prod_common != NULL) {
				prod_common->InsertAt(0, num_res);
			}

			// b-item rest
			if (num_b != NULL) {
				CExpression* b_rest = num_b->clone()->divide(num_res->clone());
				b_rest->Arrange(AL_INLINE);
				prod_b->SetAt(0, b_rest);
				delete b_rest;
			}
			else {
				// if 1/2 extracted ==> 1 should be changed with 2
				CExpression* b_rest = num_res->clone();
				b_rest->inverseLevel();
				b_rest->Arrange(AL_INLINE);
				prod_b->InsertAt(0, b_rest);
				delete b_rest;
			}

			// a-item rest
			if (num_a->equals(num_res)) {
				ret_val = NULL; delete num_res; delete num_a;
			} else {
				ret_val = num_a->divide(num_res);
				ret_val->Arrange(AL_INLINE);
			}

			return ret_val;
		}
		return expr_a;
	}

	if (expr_a->getProperty(CEProperties::PROP_MASK_IS_NUMERIC_VALUE)) return expr_a;

	//	items and powers are processed together

	//	WE TRY TO AVOID ANY CLONING HERE!

	//	coolect info about expr_a item
	bool is_power_a = false;
	long pow_a = 1;
	CEPower* expr_pow_a = NULL;
	CExpression* cmp_a = expr_a;
	bool mean_inversed_a = false;
	if ((CEUtil::equals(expr_a->ClassName(), CEPower::CLASS_NAME)) &&
		(((CEPower*)expr_a)->power()->getProperty(CEProperties::PROP_MASK_IS_NUMBER))) {
		expr_pow_a = (CEPower*)expr_a;
		pow_a = expr_pow_a->getLongPower();
		if ((pow_a != LONG_MIN)&&(pow_a != 0)) {//else non-arranged power
			cmp_a = expr_pow_a->basis();
			is_power_a = true;
			bool a_lev = expr_pow_a->getLevel();
			if (CEUtil::LXOR((pow_a > 0), a_lev)) mean_inversed_a = true;
			if (pow_a < 0) pow_a = -pow_a;
		}
	}

	// search for item in prod_b
	long i;
	for(i=0; i<prod_b->size(); i++) {
		//	comparing items and powers
		CExpression* f_item = prod_b->GetAt(i);
		
		//	coolect info about prod_b product
		bool is_power_b = false;
		long pow_b = 1;
		CEPower* expr_pow_b = NULL;
		CExpression* cmp_b = f_item;
		bool mean_inversed_b = false;
		if ((CEUtil::equals(f_item->ClassName(), CEPower::CLASS_NAME)) &&
			(((CEPower*)f_item)->power()->getProperty(CEProperties::PROP_MASK_IS_NUMBER))) {
			
			expr_pow_b = (CEPower*)f_item;
			pow_b = expr_pow_b->getLongPower();
			if ((pow_b != LONG_MIN)&&(pow_b != 0)) {//else non-arranged power
				cmp_b = expr_pow_b->basis();
				is_power_b = true;
				bool b_lev = expr_pow_b->getLevel();
				if (CEUtil::LXOR((pow_b > 0), b_lev)) mean_inversed_b = true;
				if (pow_b < 0) pow_b = -pow_b;
			}
		}

		/////// copmparison
		bool a_equ_b = false;
		bool mean_inv_a_equ_b = CEUtil::LEQU(mean_inversed_a, mean_inversed_b);
		bool levels_a_equ_b = CEUtil::LEQU(cmp_a->getLevel(), cmp_b->getLevel());

		if (CEUtil::LEQU(mean_inv_a_equ_b, levels_a_equ_b)) {
			if (!mean_inv_a_equ_b) cmp_b->inverseLevel();
			if (cmp_a->equal_or_opposite(cmp_b)) a_equ_b = true;
			if (!mean_inv_a_equ_b) cmp_b->inverseLevel();
		}

		if (a_equ_b) {
			dbg->log(95, "a_equ_b");

			// can be reduced
			CExpression* res = NULL;
			
			long order = 1;
			if (is_power_a && is_power_b) {
				// find out max common power
				// abs(pow_a-pow_b)
				if (pow_a > pow_b) order = pow_b;
				else order = pow_a;
			}
			
			// self check up
			if (order <= 0) {dbg->log(0, "Error CLFactor: (order <= 0) in factor reduction."); return expr_a;}
			
			// store common factor
			if (prod_common != NULL) {
				CExpression* f = NULL;
				// next condition should be ample to mean the power reduction
				if (order != 1) f = new CEPower(cmp_a, order);
				else f = cmp_a->clone();
				if (mean_inversed_a) f->inverseLevel();
				prod_common->addItem(f);
			}
			
			if (is_power_a) res = lower_power_on(expr_pow_a, mean_inversed_a, order);
			
			if (is_power_b) {
				// nt: expr_pow_b is prod_b-contained object
				CExpression* f_new = lower_power_on(expr_pow_b, mean_inversed_b, order);
				if (f_new == NULL) prod_b->RemoveAt(i);
				else if (f_new != expr_pow_b) { // this may not occur (currently)
					dbg->log(0, "Internal Warning: lower_power_on() changes source");
					prod_b->SetAt(i, f_new);
					delete f_new;
				}
			} else prod_b->RemoveAt(i);
			// no other fits are supposed (prod_b is arranged)

			if (res != expr_a) { // same to expr_pow_a for powers
				delete expr_a; // same to "delete expr_pow_a;" for powers
			}
			return res;
		}
	}
	return expr_a;
}

/*
*	returns same objects with adjusted power
*	or NULL if new power is zero (for avoiding extra cloning)
*	expr is not deleted
*	"is_inversed" is not used
*	Note: expr is always INTEGER power, non-integer are NOT TREATED AS POWERS (see is_power_a/b definition)
*/
CExpression* CLFactor::lower_power_on(CEPower* expr, bool is_inversed, long order)
{
	long lPowOld = expr->getLongPower();
	if (lPowOld==LONG_MIN) { dbg->log(0, "Internal ERROR. CLFactor::lower_power_on(non-integer power)", expr); CEException::halt(); }
	long lPowNew = lPowOld;
	if (lPowOld > 0) lPowNew -= order; else lPowNew += order;

	if (lPowNew == 0) return NULL;
	
	expr->setPower(lPowNew);
	return expr;
}

/*
*	nevermind the signs
*	works with real fractions only
*/
CENumber* CLFactor::get_greatest_common_divisor(CENumber* a,
												CENumber* b,
												DWORD settings,
												bool treat_NULL_as_unit)
{
	if (!treat_NULL_as_unit) {
		// check up NULLs
		if ((a == NULL) || (b == NULL)) {
			printf("<Error> unexpected NULL CENumber::get_greatest_common_divisor()\n");
			return NULL;
		}
	}

	// NULL means 1
	long num_a = 1;
	long den_a = 1;
	if (a != NULL) {
		if (a->getNumIm() != 0) return NULL;
		num_a = CLMath::abs(a->getNum());
		den_a = CLMath::abs(a->getDen());
	}
	long num_b = 1;
	long den_b = 1;
	if (b != NULL) {
		if (b->getNumIm() != 0) return NULL;
		num_b = CLMath::abs(b->getNum());
		den_b = CLMath::abs(b->getDen());
	}

	long num_res = CLMath::greatest_common_divisor(num_a, num_b);
	long den_res = 1;
	if ((settings & DONOT_FACT_FRACTIONS) == 0) {
		den_res = CLMath::least_common_multiple(den_a, den_b);
	}

	if ((treat_NULL_as_unit) && (num_res == 1) && (den_res == 1)) {
		return NULL;// means unit
	}
	CENumber* res = CENumber::GetFraction(num_res,den_res);
	//res->Arrange(AL_INLINE);
	return res;
}
