////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// LSimplifyMisc.cpp: implementation of the CLSimplifyMisc class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CLSimplifyMisc::CLSimplifyMisc() {}
CLSimplifyMisc::~CLSimplifyMisc() { }

CExpression* CLSimplifyMisc::get_Re(CExpression *source, bool delete_source)
{
	if (source == NULL) return NULL;

	CExpression* res = source->Re();
	if (delete_source && (res != source)) delete source;
	if (res == NULL) res = data->gete(CConst::DV_ZERO)->clone();
	return res;
}

CExpression* CLSimplifyMisc::get_Im(CExpression *source, bool delete_source)
{
	if (source == NULL) return NULL;

	CExpression* res = source->Im();
	if (delete_source && (res != source)) delete source;
	if (res == NULL) res = data->gete(CConst::DV_ZERO)->clone();
	return res;
}

/*
*	clones source
*/
CExpression* CLSimplifyMisc::TruncateSign(CExpression* source) {
	if (source == NULL) return NULL;

	CExpression* res = source->clone();
	if (!res->getFlag()) res->setFlag();
	return res;
}

const DWORD CLSimplifyMisc::MISC_EXPAND_SAVE_MEMORY	= 0x01;
CExpression* CLSimplifyMisc::ExpandSaveMemory(CExpression* source) {
	if (source==NULL) return NULL;
	return CLSimplifyMisc::ExpandSaveMemoryImpl(source, true);
}

CExpression* CLSimplifyMisc::ExpandSaveMemoryImpl(CExpression* source, bool del_src) {
	CExpression* res=NULL;
	if (source->getProperty(CEProperties::PROP_MASK_IS_SUM)) {
		// do it for basic sum only
		CESum* sum_src = (CESum*)source;
		long s = sum_src->size();
		if (s>0) {
			long reduce_i=0;
			res = sum_src->GetAt(0)->clone();
			for(long i=1; i<s; i++) {
				res = res->add(sum_src->GetAt(i)->clone());
				reduce_i++;
				if (reduce_i>10) { res = CExpression::reduce(res); reduce_i=0; }
			}
			source->copyAttributesTo(res,true);
			if (del_src) delete source;
			return res;
		}
	} else if (source->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
		// do it for every sum in product
		CEProduct* prod_src = (CEProduct*)source;
		long s = prod_src->size();
		if (s>0) {
			CEProduct* pr_res = new CEProduct();
			CExpression* item = pr_res->GetAt(0);
			if (item->getProperty(CEProperties::PROP_MASK_IS_SUM)) { item = ExpandSaveMemoryImpl(item->clone(), true); }
			pr_res->addItem(item->clone());
			for(long i=1; i<s; i++) {
				if (pr_res==NULL) { pr_res=new CEProduct(); pr_res->addItem(res); res=NULL; }

				item = pr_res->GetAt(i);
				if (item->getProperty(CEProperties::PROP_MASK_IS_SUM)) {
					item = ExpandSaveMemoryImpl(item->clone(), true);
					pr_res->addItem(item);
					res = CExpression::reduce(pr_res); pr_res=NULL;
				} else {
					pr_res->addItem(item->clone());
				}
			}
			if (pr_res != NULL) { res = CExpression::reduce(pr_res); pr_res=NULL; }
			source->copyAttributesTo(res,true);
			if (del_src) delete source;
			return res;
		}

	}
	if (res==NULL) res=CExpression::reduce(source, del_src);
	return res;
}
