////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ViewSimplify.cpp: implementation of the CViewSimplify class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

CDebug* CViewSimplify::dbg = new CDebug("CViewSimplify");

const DWORD CViewSimplify::PART_FACT_VERIFY				= 0x10;

const DWORD CViewSimplify::PART_FACT_NUMBERS			= 0x01;
const DWORD CViewSimplify::PART_FACT_FUNCTIONS			= 0x02;
const DWORD CViewSimplify::PART_FACT_DENOMINATORS		= 0x04;
const DWORD CViewSimplify::PART_FACT_NUMERATORS			= 0x08;
const DWORD CViewSimplify::PART_FACT_ALL_DENOMINATORS	= 0x20;
const DWORD CViewSimplify::PART_FACT_ALL_SINGLE_DENS	= 0x40;

const DWORD CViewSimplify::PART_FACT_DEFAULT			= 0x0e;// no numbers
const DWORD CViewSimplify::PART_FACT_ALL				= 0x0f;

CSequence* CViewSimplify::m_FactSyms=NULL; // (volatile link)

CExpression* CViewSimplify::m_Unit = NULL;

//////////////////////////////////////////////////////////////////////
//                          CUExprCounter
//////////////////////////////////////////////////////////////////////

class CUExprCounter : public CEObject  
{
private:
	CExpression* m_expr;
	long m_val;
	bool m_updated;
public:
	long inc_val();
	long get_val();
	void reset_val();

	bool is_updated();
	void reset_update_status();
	
	CExpression* get_expr();

	CUExprCounter(CExpression* _expr, bool truncate_sign = true);
	virtual ~CUExprCounter();
	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
	virtual bool contain_vector(void * obj);
};

long CUExprCounter::inc_val() {m_val++; m_updated = true; return m_val;}
long CUExprCounter::get_val() {return m_val;}
void CUExprCounter::reset_val() {m_val = 0;}
bool CUExprCounter::is_updated() {return m_updated;}
void CUExprCounter::reset_update_status() {m_updated = false;}
CExpression* CUExprCounter::get_expr() {return m_expr;}
CUExprCounter::CUExprCounter(CExpression* _expr, bool truncate_sign) : CEObject(true)
{
	if (_expr != NULL) {
		m_expr = _expr->clone();
		if (truncate_sign) m_expr->setFlag();
	}
	else m_expr = NULL;
	reset_val();
	reset_update_status();
};

CUExprCounter::~CUExprCounter()
{
	if (m_expr != NULL) delete m_expr;
}

const char* CUExprCounter::CLASS_NAME = "CUExprCounter";
const char* CUExprCounter::ClassName() const { return CLASS_NAME; }
bool CUExprCounter::contain_vector(void * obj) {
	if (m_expr == obj) return true;
	if (m_expr != NULL) { if(m_expr->contain_vector(obj)) return true; }
	return false;
}

//////////////////////////////////////////////////////////////////////
//                          CViewSimplify
//////////////////////////////////////////////////////////////////////

CViewSimplify::CViewSimplify() { }
CViewSimplify::~CViewSimplify() { }

/*
*	a*func1+b*func1+c ==> (a+b)*func1+c;
*	(a+b) and c will be kinematically reduced;
*/
CExpression* CViewSimplify::simp_view(CExpression *old, bool u_rev, bool do_reduce_old)
{
	dbg->log(20, "simp_view:", old);
	//	Inline all the sum sub-items.
	CExpression* res = NULL;
	if (do_reduce_old) res = CExpression::reduce(old);
	else res = old;
	res = simp_vi_view(res, true, u_rev);

	dbg->log(20, "simp_view result:", res);
	return res;
}

/*
*	a*func1+b*func1+c ==> (a+b)*func1+c;
*	(a+b) and c will be kinematically reduced;
*
*	To do: change the VIs into Abstract Functions.
*	Warning: Commutation rules for VI-s are not checked out.
*/
CExpression* CViewSimplify::simp_vi_view(CExpression *source, bool del_src, bool u_rev)
{
	dbg->log(0, "WARNING: old code involved");
	//	Move all the functions ahead the product
	//	Should it be included into reducing?
	//	Or all the functions should be at the end -- and it can be included into reducing.
	if (source->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
		CEProduct* pr = (CEProduct*)source;
		int func_pos = 0;
		long i;
		for(i=1; i<pr->size(); i++) {
			CExpression* item = pr->GetAt(i);
			if (item->getProperty(CEProperties::PROP_MASK_IS_VIRT_INTEGRAL)) {
				func_pos++;
				if (i > func_pos) pr->move(i, func_pos);
			}
		}
		return source;
	}

	if (!source->getProperty(CEProperties::PROP_MASK_IS_SUM)) {
		return source;
	}

	//	Simplifying the sub-items
	CESum* sum = (CESum*)source;
	long i;
	for(i=0; i<sum->size(); i++) {
		CExpression* item = sum->GetAt(i);
		CExpression* it_simp = simp_vi_view(item, false, u_rev);
		//if (it_simp->getProperty(CEProperties::PROP_MASK_IS_VIRT_INTEGRAL)) do_arrange_vis = true;
		if (it_simp != item) {
			sum->SetAt(i, it_simp);
			delete it_simp;
		}
	}

	//	test: should extract VI-objects in this sum
	bool do_arrange_vis = false;
	int test_size = sum->size();
	for (i=0; ((i<test_size) && (!do_arrange_vis)); i++) {
		CExpression* item = sum->GetAt(i);
		if (item->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
			CEProduct* pr = (CEProduct*)item;
			CExpression* it0 = pr->GetAt(0);
			if (it0 == NULL) {
				printf("<CViewSimplify::simp_vi_view> Invalid expression after reduction.\n");
				return NULL;
			}
			
			if (it0->getProperty(CEProperties::PROP_MASK_IS_VIRT_INTEGRAL)) {
				do_arrange_vis = true;
			}
		}
		else if (item->getProperty(CEProperties::PROP_MASK_IS_VIRT_INTEGRAL)) {
			do_arrange_vis = true;
		}
	}

	if (!do_arrange_vis) {
		CExpression* res = NULL;
		if (del_src) res = CSubstSimplify::simp_kin(source, u_rev);
		else res = CSubstSimplify::simp_kin(source->clone(), u_rev);
		return res;
	}

	//	VI exptracting, storage and arranging.
	CExpression* unit = data->gete(CConst::DV_UNIT);
	CSequence* seq_vi = new CSequence();
	for (i=0; i<sum->size(); i++) {
		CExpression* item = sum->GetAt(i);
		CExpression* vi = NULL;
		if (item->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
			CEProduct* pr = (CEProduct*)item;
			CExpression* it0 = pr->GetAt(0);
			if (it0->getProperty(CEProperties::PROP_MASK_IS_VIRT_INTEGRAL)) {
				vi = it0->clone();
				pr->RemoveAt(0);
			}
		}
		else if (item->getProperty(CEProperties::PROP_MASK_IS_VIRT_INTEGRAL)) {
			vi = item->clone();
			sum->SetAt(i, unit);
			if (!vi->getFlag()) {
				vi->setFlag();
				sum->GetAt(i)->inverseFlag();
			}

		}
		if (vi == NULL) vi = unit->clone();
		//	Item is no longer sum->GetAt(i);

		//	Comparing vi to previous ones.
		bool compare_success = false;
		long j;
		for(j=0; j<i; j++) {
			CExpression* vi_old = seq_vi->GetAt(j);
			if (vi_old == NULL) {
				printf("<CViewSimplify::simp_vi_view> Error in function cashe.\n");
				return NULL;
			}
			
			if (vi->equals(vi_old)) {
				CExpression* rest = sum->GetAt(i)->clone();
				CExpression* it_j = sum->GetAt(j)->clone();
				it_j = it_j->add(rest);
				sum->SetAt(j, it_j);
				sum->RemoveAt(i);
				delete vi;
				i--;
				j = i;
				compare_success = true;
			}
		}
		if (!compare_success) {
			seq_vi->addItem(vi);
		}
	}

	//	Simplifying rests and multiplying to functions.
	for (i=0; i<sum->size(); i++) {
		CExpression* item_cln = sum->GetAt(i)->clone();
		CExpression* vi_cln = seq_vi->GetAt(i)->clone();
		item_cln = CSubstSimplify::simp_kin(item_cln, u_rev);
		item_cln = vi_cln->multiply(item_cln);
		sum->SetAt(i, item_cln);
		delete item_cln;
	}

	delete seq_vi;
	return sum;
}

/*////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
////                                                                      ////
////                      Partial Factorization                           ////
////              Minimization of sum/multiply operations                 ////
////                                                                      ////
////   Destroys or returns source.                                        ////
////   Usage: value=CViewSimplify::factorize_partial(value,...);          ////
////                                                                      ////
//////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////*/
CExpression* CViewSimplify::factorize_partial(CExpression *source, DWORD settings, CSequence* f_syms)
{
	if (source == NULL) return NULL;
	dbg->log(10, settings, "partial factorization(%ld)", source);

	// init
	if (m_Unit == NULL) m_Unit = data->gete(CConst::DV_UNIT);

	// set default settings if no settings are set
	if ( (settings == 0) || (settings == PART_FACT_VERIFY) ) {
		settings = settings | PART_FACT_DEFAULT;
		dbg->log(0, "WARNING: ZERO factorize_partial() settings. Defaults are set.");
	}

	// store link to factorization symbols
	m_FactSyms=f_syms;

	CExpression* source_copy = NULL;
	if ((settings & PART_FACT_VERIFY) == PART_FACT_VERIFY) {
		// get a copy of source
		source_copy = source->clone();
	}

	CExpression* res = fact_part_impl(source, ((settings | PART_FACT_VERIFY) ^ PART_FACT_VERIFY));

	// verification
	if (((settings & PART_FACT_VERIFY) == PART_FACT_VERIFY) && (source_copy != NULL)) {
		//printf("Part Factorize Verification... ");
		if (res == NULL) printf("NULL result\n");
		else {
			CExpression* res_reduced = CExpression::reduce(res->clone());
			bool verify_OK = source_copy->equals(res_reduced);
			if (!verify_OK) {
				// perhaps sourse is not reduced
				source_copy = CExpression::reduce(source_copy);
				verify_OK = source_copy->equals(res_reduced);
			}
			delete res_reduced;
			if (!verify_OK) {
				dbg->log(2, "<Error> Partial Factor Verification fails.", source_copy, res);
				//dbg->log(2, "<Error> Source:", source_copy); dbg->log(2, "<Error> Result:", res);
				// Nothing to do here: it might be OK.
			}
		}
		delete source_copy;
	}

	m_FactSyms=NULL;
	dbg->log(10, "partial factorization result", res);
	return res;
}

CExpression* CViewSimplify::fact_part_impl(CExpression *source, DWORD settings) {
	dbg->log(55, settings, "fact_part_impl(%ld)", source);
	if (source == NULL) return NULL;

	//CExpression* source_cpy = source->clone();

	//	0. if source is not sum return (do not deep to subobjects)
	unsigned long src_props = source->props_long();
	if (src_props & CEProperties::PROP_MASK_IS_SUM) {
		return fact_part_sum((CESum*)source, settings);
	} else if (src_props & CEProperties::PROP_MASK_IS_PRODUCT) {
		return fact_part_prod((CEProduct*)source, settings);
	}
	return source;
}

CExpression* CViewSimplify::fact_part_prod(CEProduct* pr, DWORD settings) {
	long s = pr->size();
	long i;
	for(i=0; i<s; i++) {
		CExpression* item = pr->GetAt(i);
		if (!item->getLevel()) continue;
		CExpression* res_i = fact_part_impl(item, settings); // item may be destroyed
		if (res_i != item) pr->OverwriteAtX(i, res_i);
	}
	return pr;
}

CExpression* CViewSimplify::fact_part_sum(CESum *ssum, DWORD settings) {

	//	1. call CLFactor::factorize()
	//	if success process subitems
	DWORD _factor_setts = CLFactor::NO_EXPAND;
	if ((settings & PART_FACT_NUMBERS) == 0) _factor_setts |= CLFactor::DONOT_FACT_NUMBERS;

	// denominator options (I'm not sure which one to choose) <???>
	// nt: common denominator will currently be extracted even if no PART_FACT_DENOMINATORS flag is set
	//if ((settings & PART_FACT_DENOMINATORS) == 0) _factor_setts |= CLFactor::DONOT_FACT_FRACTIONS;
	_factor_setts |= CLFactor::DONOT_FACT_FRACTIONS; // do not factorize numeric fractions
	if (settings & PART_FACT_ALL_DENOMINATORS) _factor_setts |= CLFactor::FACT_ALL_DENOMINATORS;
	if (settings & PART_FACT_ALL_SINGLE_DENS) _factor_setts |= CLFactor::FACT_ALL_SINGLE_DENS;

	CExpression* factorize_result = CLFactor::factorize(ssum, _factor_setts);
	if ((factorize_result != ssum) && (factorize_result != NULL)) {
		// source is already deleted
		if (factorize_result->getProperty(CEProperties::PROP_MASK_IS_SUM)) {
			ssum = (CESum*)factorize_result;
		} else {
			if (factorize_result->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
				factorize_result = fact_part_prod((CEProduct*)factorize_result, settings);

				/*CEProduct* pr_res = (CEProduct*)factorize_result;
				long ii;
				for(ii=0; ii<pr_res->size(); ii++) {
					CExpression* item_ii = pr_res->GetAt(ii);
					if (!item_ii->getProperty(CEProperties::PROP_MASK_IS_SUM)) continue;
					CExpression* res_ii = fact_part_impl(item_ii->clone(), ((settings | PART_FACT_VERIFY) ^ PART_FACT_VERIFY));
					pr_res->SetAt(ii, res_ii);
					delete res_ii;
				}*/
			}
			dbg->log(75, "factorization found:", factorize_result);
			return factorize_result;
		}
	}
	else if (factorize_result == NULL) {
		//error? // isn't source deleted?
		dbg->log(0, "<Warning> factorize rezult == NULL: somthing wrong?");
		//printf("<Warning> (partial factorization) factorize rezult == NULL: somthing wrong?\n");
		return NULL;
	}

	long ssum_size = ssum->size();
	dbg->log(30, ssum_size, "partial factorization: sum of size %ld.", ssum);

	if (ssum_size == 1) {
		CExpression* res = ssum->GetAt(0)->clone();
		ssum->copyAttributesTo(res, true);
		delete ssum;
		// factorize sub-items
		res = fact_part_impl(res, settings);
		return res;
	}
	else if (ssum_size == 0) {//error: how can it occur?
		dbg->log(0, "<Error> sum size == 0");
		delete ssum;
		return NULL;
	}

	//  2. factor list creation
	CEPtrArray* factors = new CEPtrArray();
	long s = ssum->size();
	long i;
	for(i=0; i<s; i++) {
		CExpression* item = ssum->GetAt(i);

		// reset "updated" flags
		// actually the "updated" flags are required for non-reduced expressions simplification
		// e.g. a*b*a*c*a^3 => Na++; Nb++; Nc++; but not Na+=3
		int fact_sz = factors->size();
		long j;
		for(j=0; j<fact_sz; j++) {
			((CUExprCounter*)factors->GetAt(j))->reset_update_status();
		}

		// mean the sum as numerator whatever it is.
		update_factors_list(item, factors, true, settings);
	}

	// <debug>
	dbg->log(30, factors->size(), "factor list created, size: %ld.");
	if (dbg->get_level() >= 40) {
		// log factor list
		long tmp_s = factors->size();
		long tmp_i;
		for (tmp_i=0; tmp_i<tmp_s; tmp_i++) {
			CUExprCounter* tmp_cart = (CUExprCounter*)factors->GetAt(tmp_i);
			CExpression* tmp_expr = tmp_cart->get_expr();
			if (tmp_expr->getLevel()) dbg->log(40, tmp_cart->get_val(), "(fr == %ld, level = 1):", tmp_expr);
			else dbg->log(40, tmp_cart->get_val(), "(fr == %ld, level = -1):", tmp_expr);
		}
	}
	// </debug>

	// 3. factor selection
	long max_func_pos = -1;
	long max_func_cnt = -1;
	long max_item_pos = -1;
	long max_item_cnt = -1;
	for (i=0; i<factors->size(); i++) {
		CUExprCounter* cart = (CUExprCounter*)factors->GetAt(i);
		long cnt = cart->get_val();
		if (cnt > max_item_cnt) {max_item_pos = i; max_item_cnt = cnt;}

		// also calculate functions popularity
		CExpression* item = cart->get_expr();
		if (item->getProperty(CEProperties::PROP_MASK_IS_FUNCTION)) {
			if (cnt > max_func_cnt) {max_func_pos = i; max_func_cnt = cnt;}
		}
	}

	long factor_pos = -1;
	// select functions primarily
	if (max_func_cnt > 1) factor_pos = max_func_pos;
	else if (max_item_cnt > 1) factor_pos = max_item_pos;
	if (factor_pos < 0) {//factor item not found
		delete factors;
		dbg->log(40, "no partial factor found");

		// factorize sub-items
		s = ssum->size();
		for(i=0; i<s; i++) {
			CExpression* item = ssum->GetAt(i);
			CExpression* res_i = fact_part_impl(item, settings); // item may be destroyed
			if (res_i != item) ssum->OverwriteAtX(i, res_i);
		}
		//res_B = fact_part_impl(ssum, settings);
		return ssum;
	}

	// cache flags
	bool res_sign = ssum->getFlag();
	if (!res_sign) { ssum->setFlag(); dbg->log(40, "sign flag cached"); }
	bool res_lev = ssum->getLevel();
	if (!res_lev) { ssum->setLevel(true); dbg->log(40, "level flag cached"); }

	CUExprCounter* fact_cart = (CUExprCounter*)factors->GetAt(factor_pos);
	CExpression* fact_expr = fact_cart->get_expr()->clone();
	dbg->log(30, fact_cart->get_val(), "part factor selected (frequency == %ld):", fact_expr);
	delete factors;

	// 4. factorization:  factor*(sum_A)+sum_B;
	CESum* sum_A = new CESum();
	for (i=0; i<ssum->size(); i++) {
		CExpression* item = ssum->GetAt(i);
		//dbg->log(30, i, "extracting factor from item %ld:", item);
		CExpression* extr_res = extract_factor_from(item, fact_expr, true, settings);
		if (extr_res == m_Unit) extr_res = extr_res->clone();
		//dbg->log(30, "extract_factor_from result:", extr_res);
		if (extr_res != NULL) {	// add to sum_A
			if (extr_res == item) extr_res = extr_res->clone();
			sum_A->addItem(extr_res);
			ssum->RemoveAt(i);
			i--;
		}
		// else no factor to be extracted in current item.
	}
	// ssum is now sum_B
	dbg->log(95, "sum_A:", sum_A);
	dbg->log(95, "ssum rest:", ssum);

	CEProduct* prod_A = new CEProduct();
	prod_A->addItem(fact_expr);
	// the sum_A should be added here after recurrent factorization

	// 5. recurrent call to sum_A and sum_B and combining result
	CExpression* res_A = fact_part_impl(CExpression::arrge(sum_A), settings);
	if (!res_A->getFlag()) {
		res_A->setFlag();
		prod_A->inverseFlag();
	}
	prod_A->addItemInline(res_A);//the products will be inserted inline!
	prod_A->Arrange(AL_INLINE);
	// prod_A is the "f*sum_A" result

	// if sum_B is empty ==> warning: full factorization is not expected
	if (ssum->size() == 0) {
		dbg->log(0, "<Warning> full factorization instead of partial after CLFactor failed", prod_A);
		dbg->log(0, "<Warning> The source was probably unarranged");
		//dbg->store_debug_expression(2, "source expression", source_cpy);
		//dbg->store_debug_expression(2, "factorized expression", prod_A);
		ssum->addItem(CENumber::GetZero());
	}

	CExpression* res_B = fact_part_impl(ssum, settings);
	CESum* res = NULL;
	if ((res_B->getProperty(CEProperties::PROP_MASK_IS_SUM)) &&
		(res_B->getFlag()) &&
		(res_B->getLevel())) {
		res = (CESum*)res_B;
	} else {
		res = new CESum();
		res->addItem(res_B);
	}
	res->InsertAt(0, prod_A);
	delete prod_A;

	if (!res_sign) res->inverseFlag();
	if (!res_lev) res->inverseLevel();

	dbg->log(75, "part factor sub-result:", res);
	return res;
}

void CViewSimplify::update_factors_list(CExpression *item, CEPtrArray *factors, bool item_lev, DWORD settings)
{
	if ((item == NULL) || (factors == NULL)) return; //Error;
	if (item->getProperty(CEProperties::PROP_MASK_IS_SUM)) {
		//<old> //dbg->log(0, "<Warning> CViewSimplify incorrect use: factorizing of not-expanded sum.");
		//<old> //return;

		// <to do> partial factorize the
		// try factorize sum as it is
	}

	if (item->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
		CEProduct* prod = (CEProduct*)item;
		long s = prod->size();
		bool prod_lev = (prod->getLevel())?(item_lev):(!item_lev);
		long i;
		for(i=0; i<s; i++) {
			update_factors_list(prod->GetAt(i), factors, prod_lev, settings);
		}
		return;
	}

	//	powers
	if (CEUtil::equals(item->ClassName(), CEPower::CLASS_NAME)) {
		CEPower* pow = (CEPower*)item;
		//if (!power->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) { return; } //internal error
		//if (pow->power()->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) { 
		if (pow->isPowerInteger()) {

			long lPower = pow->getLongPower();
			if ((lPower==LONG_MIN)||(lPower == 0)) return;//source expression is not reduced or internal error
			CExpression* basis = pow->basis();

			bool pow_lev = item_lev;
			// try to extract from basis (in appropriate level)
			if (CEUtil::LXOR((lPower > 0), pow->getLevel())) pow_lev = !pow_lev;
			update_factors_list(basis, factors, pow_lev, settings);

			// the lower powers of basis are not added ==> the folllowing case may occur:
			// a^2+a^3->a(a+a^2) -- it's OK, the further factorize() will fix it.
			return;
		}
		//if power is abstract expression ==> do try factorize power as it is
	}

	// Note: the 'item_lev' flag will now indicates if item is in numerator.
	if (!item->getLevel()) item_lev = !item_lev;
	//////////////////////////////////////////////////////////////////////// <settings use>
	// check settings for add (or not) this item to factor list
	////////////////////////////////////////////////////////////////////////;

	// this check is required for PART_FACT_NUMERATORS does not deal with numbers
	if ((settings & PART_FACT_NUMBERS) == 0) {
		if (item->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) return;
		if (item->getProperty(CEProperties::PROP_MASK_IS_NUMERIC_VALUE)) return;
	}

	bool do_add = false;
	if ((settings & PART_FACT_NUMERATORS) && (item_lev)) do_add = true;
	else if ((settings & PART_FACT_DENOMINATORS) && (!item_lev)) do_add = true;
	else if ((settings & PART_FACT_FUNCTIONS) &&
		(item->getProperty(CEProperties::PROP_MASK_IS_FUNCTION))) do_add = true;
	else if (settings & PART_FACT_NUMBERS) {
		if (item->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) do_add = true;
		else if (item->getProperty(CEProperties::PROP_MASK_IS_NUMERIC_VALUE)) do_add = true;
	}

	if (!do_add) {
		if(m_FactSyms != NULL) {
			int fs_i; int fs_s=m_FactSyms->size();
			for(fs_i=0; fs_i<fs_s; fs_i++) { if(item->equals(m_FactSyms->GetAt(fs_i))) { do_add=true; break; } }
		}
	}

	if (!do_add) return;
	//////////////////////////////////////////////////////////////////////// </settings use>

	CExpression* item_alt = item; // item with appropriate level.
	if (!CEUtil::LEQU(item->getLevel(), item_lev)) {
		item_alt = item_alt->clone(); item_alt->inverseLevel();
	}
	
	// search for factor in factors list
	long fact_size = factors->size();
	long i;
	for(i=0; i<fact_size; i++) {
		CUExprCounter* cart = (CUExprCounter*)factors->GetAt(i);
		CExpression* cart_expr = cart->get_expr();
		if ((CEUtil::LEQU(cart_expr->getLevel(), item_lev))
			&& (cart_expr->equal_or_opposite(item_alt))) {
			if (!cart->is_updated()) cart->inc_val();
			if (item_alt != item) delete item_alt;
			return;
		}
	}

	CUExprCounter* cart_new = new CUExprCounter(item_alt);
	cart_new->inc_val();
	factors->add(cart_new);
	if (item_alt != item) delete item_alt;
}

/*
*	returns NULL if can not extract factor.
*	if extracted returns rest value (without cloning products)
*	In arranging factor is supposed to be "+"-signed.
*	Factorization will work at any sign of factor
*
*	item_lev means the level of item-container.
*	it will be XOR-ed with item's level.
*
*	can be fastered by returning non-cloned units
*	together with non-deleting ones
*	due to all the units are deleted for the inline arrangement
*/
CExpression* CViewSimplify::extract_factor_from(CExpression *item,
												CExpression *factor,
												bool item_lev,
												DWORD settings)
{
	if ((item == NULL) || (factor == NULL)) return NULL;
	//dbg->log(105, "extract_factor_from (item):", item);
	//dbg->log(105, "extract_factor_from (factor):", factor);

	if (item->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
		CEProduct* prod = (CEProduct*)item;
		long s = prod->size();
		bool prod_lev = (prod->getLevel())?(item_lev):(!item_lev);
		long i;
		for(i=0; i<s; i++) {
			CExpression* pr_it = prod->GetAt(i);
			CExpression* extr_res = extract_factor_from(pr_it, factor, prod_lev, settings);
			//dbg->log(105, "extr_res (sub-res):", extr_res);

			if (extr_res != NULL) {
				if (extr_res != pr_it) {	//	quick inline arrangement
					prod->SetAt(i, extr_res);
					if (extr_res != m_Unit) delete extr_res;
					//CExpression* prod_arranged = CExpression::arrge(prod, false);
					//dbg->log(105, "extract_factor_from (res):", prod_arranged);
					//return prod_arranged;
					return CExpression::arrge(prod, false);
				}
				//dbg->log(105, "extract_factor_from (res):", prod);
				return prod;
			}
		}
		return NULL;
	}

	if ((settings & PART_FACT_NUMBERS) == 0) {
		if (item->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) return NULL;
		if (item->getProperty(CEProperties::PROP_MASK_IS_NUMERIC_VALUE)) return NULL;
	}

	//	powers
	if (CEUtil::equals(item->ClassName(), CEPower::CLASS_NAME)) {
		CEPower* pow = (CEPower*)item;
		if (pow->isPowerInteger()) { //if (pow->power()->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) {
			CExpression* basis = pow->basis();
			long lPower = pow->getLongPower(); if ((lPower==LONG_MIN)||(lPower == 0)) { dbg->log(0,"InetnalWarning: 0 power spotted",pow); return NULL; }

			bool basis_in_den = CEUtil::LXOR((lPower > 0), pow->getLevel());
			bool pow_lev = (basis_in_den)?(!item_lev):item_lev;

			// Note: we can't call extract_factor_from(basis), because 'basis' can be changed then
			CExpression* basis_cpy = basis->clone();
			CExpression* extr_res = extract_factor_from(basis_cpy, factor, pow_lev, settings);
			if (extr_res != NULL) {
				if (extr_res != basis_cpy) { delete basis_cpy; }
				if (lPower > 0) lPower--; else lPower++;
				
				// clone power: result != power in general and can't be processed inline.
				CExpression* res = pow->clone();
				((CEPower*)res)->setPower(lPower);
				if (extr_res != m_Unit) {
					// if basis was in denominator --> inverse extr_res
					if (basis_in_den) extr_res->inverseLevel();
					res = res->multiply(extr_res);
				}
				res = CExpression::arrge(res);
				//dbg->log(105, "extract_factor_from (power) (res):", res);
				return res;
			} else {
				delete basis_cpy;
			}

			return NULL;
		}
		//if power is abstract expression ==> do try extract power as it is
	}

	// single items
	bool it_flag = item->getFlag();
	if (!it_flag) item->setFlag();
	bool f_flag = factor->getFlag();
	if (!f_flag) factor->setFlag();

	if (!item_lev) item->inverseLevel();

	CExpression* res = NULL;
	if (item->equals(factor)) { // equals only if levels coinside
		res = m_Unit;
		if (CEUtil::LXOR(it_flag, f_flag)) {
			res = res->clone();
			res->inverseFlag();
		}
	}

	if (!it_flag) item->unsetFlag(); // restore sources
	if (!f_flag) factor->unsetFlag(); // restore sources

	if (!item_lev) item->inverseLevel(); // restore sources

	if (res != NULL) res = CExpression::arrge(res);
	//dbg->log(105, "single extract res (arranged):", res);
	return res;
}
