////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ScriptErrors.cpp: implementation of the CScriptErrors class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Error codes
//////////////////////////////////////////////////////////////////////
const int CScriptErrors::UNCOUNTED_ERROR			= 0;
const int CScriptErrors::ILLEGAL_OPERATION			= 1;
const int CScriptErrors::NULL_OBJECT				= 2;
const int CScriptErrors::INTERNAL_ERROR				= 3;
const int CScriptErrors::UNDEFINED_OBJECT			= 4;
const int CScriptErrors::UNKNOWN_FUNCTION			= 5;
const int CScriptErrors::STATEMENT_ILLEGAL			= 6;
const int CScriptErrors::PARAMETER_INVALID			= 7;
const int CScriptErrors::PARAMETERS_NUMBER_INVALID	= 8;
const int CScriptErrors::HALT_COMMAND				= 9;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CScriptErrors::CScriptErrors() {}
CScriptErrors::~CScriptErrors() {}

// breaks are on errors & halt commands
bool CScriptErrors::isBreakError(int err_id) {
	if (err_id == HALT_COMMAND) return false;
	return true;
}

CEString* CScriptErrors::getStringMessage(int err_id, int i_param) {
	const char* mess_text = NULL;
	switch(err_id) {
	case UNCOUNTED_ERROR:
		mess_text = "Undefined error";
		break;
	case ILLEGAL_OPERATION:
		mess_text = "Illegal operation";
		break;
	case NULL_OBJECT:
		mess_text = "Null object encountered";
		break;
	case INTERNAL_ERROR:
		mess_text = "Internal error";
		break;
	case UNDEFINED_OBJECT:
		mess_text = "Undefined object";
		break;
	case UNKNOWN_FUNCTION:
		mess_text = "Unknown function";
		break;
	case STATEMENT_ILLEGAL:
		mess_text = "Illegal statement";
		break;
	case PARAMETER_INVALID:
		mess_text = "Invalid parameter no. %ld";
		break;
	case PARAMETERS_NUMBER_INVALID:
		mess_text = "Function does not take %ld parameters";
		break;
	case HALT_COMMAND:
		mess_text = "Script processing halted";
		break;
	}

	if (mess_text == NULL) mess_text = "Unknown error code";
	char* s = (char*)CMemoryManager::mmalloc(
		CEUtil::length(mess_text) + 20);
	sprintf(s, mess_text, i_param);
	CEString* res = new CEString(s);
	CMemoryManager::mfree(s);
	return res;
}
