////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ScriptException.cpp: implementation of the CScriptException class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//#define MESSAGE_LOCATION;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CScriptException::CScriptException(int id, int line, int line_pos, int i_param, const char* c_param)
: CEException()
{
	m_id = id; m_line = line; m_line_pos = line_pos; m_i_param = i_param;
	if (c_param != NULL) { // copy text parameter
		m_c_param = CEUtil::copyString(c_param);
	} else {
		m_c_param = NULL;
	}

	// string message creation
	m_msg = new CEString("");

#if defined(MESSAGE_LOCATION)
	m_msg->AddString("line "); m_msg->add_int(m_line);
	m_msg->AddString(", pos "); m_msg->add_int(m_line_pos);
	m_msg->AddString(": ");
#endif //MESSAGE_LOCATION

	if (m_c_param != NULL) { m_msg->AddString("{"); m_msg->AddString(m_c_param); m_msg->AddString("} "); }
	m_msg->AddString(CScriptErrors::getStringMessage(m_id, m_i_param));
	//m_msg = new CEString("CScriptException: #");
	//m_msg->add_int(m_id);
}

CScriptException::~CScriptException() {
	if (m_c_param != NULL) CMemoryManager::mfree(m_c_param);
}

const char* CScriptException::CLASS_NAME = "CScriptException";
const char* CScriptException::ClassName() const { return CLASS_NAME; }
bool CScriptException::contain_vector(void * obj) {
	if (obj == NULL) return false;
	if (CEException::contain_vector(obj)) return true;
	if (m_c_param == obj) return true;
	return false;
}

int CScriptException::getErrCode() { return m_id; }
int CScriptException::get_iParam() { return m_i_param; }
const char* CScriptException::get_cParam() { return m_c_param; }

bool CScriptException::isErrorBreak() {
	return CScriptErrors::isBreakError(m_id);
}
