////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ScriptObjectValue.cpp: implementation of the CScriptObjectValue class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

const char* CScriptObjectValue::CLASS_NAME = "CScriptObjectValue";

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CScriptObjectValue::CScriptObjectValue(int obj_type, CESerializable* obj, bool is_cloned)
: CScriptValue()
{
	m_obj = NULL;
	if (obj != NULL) {
		if (is_cloned) m_obj = obj;
		else m_obj = (CESerializable*)obj->copy();
	}
	m_obj_type = obj_type;
}

CScriptObjectValue::~CScriptObjectValue()
{
	if (m_obj != NULL) delete m_obj;
}

CScriptValue* CScriptObjectValue::clone() const {
	return new CScriptObjectValue(m_obj_type, m_obj, false);
}

int CScriptObjectValue::value_type() { return m_obj_type; }
CEObject* CScriptObjectValue::get_obj_value() { return m_obj; }
const char* CScriptObjectValue::ClassName() const { return CLASS_NAME; }

bool CScriptObjectValue::check_value_type(int type_mask) {
	if (m_obj_type & type_mask) return true;
	return false;
}

bool CScriptObjectValue::check_value(int type_mask, bool check_object) {
	if (!check_value_type(type_mask)) return false;
	if (check_object) {
		if (m_obj == NULL) return false;
	}
	return true;
}

CEString* CScriptObjectValue::toString(int mode) {
	CEString* res = new CEString("{(");
	res->add_int(m_obj_type);
	if (m_obj == NULL) {
		res->AddString("): NULL }");
	}
	else {
		res->AddString(",");
		res->AddString(m_obj->ClassName());
		res->AddString("): ");
		res->AddString(m_obj->toString(mode));
		res->AddString(" }");
	}
	return res;
}
