////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ScriptParser.cpp: implementation of the CScriptParser class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CScriptParser::CScriptParser(const char* _source) : CFileParserAdv(_source) {
	m_CommentsType=CFileParserAdv::COMMENTS_C;

	m_operand_types_data->add(CParserSymbolsList::create_list(
		CFileParserAdv::OP_VARIABLE,
		CParserSymbolsList::LIST_SYMBOLS, "_",
		CParserSymbolsList::LIST_NUMBERS, NULL, // numbers may be posterior
		CParserSymbolsList::READ_BY_STRING));

	m_operand_types_data->add(CParserSymbolsList::create_list(
		CFileParserAdv::OP_VALUE_NUMBER, CParserSymbolsList::LIST_NUMBERS, CParserSymbolsList::READ_BY_STRING));

	m_operand_types_data->add(CParserSymbolsList::create_list(CFileParserAdv::OP_END_OF_LINE, ";,", CParserSymbolsList::READ_BY_SYMBOL));
	m_operand_types_data->add(CParserSymbolsList::create_list(CFileParserAdv::OP_BRACKET_LEFT, "(", CParserSymbolsList::READ_BY_SYMBOL));
	m_operand_types_data->add(CParserSymbolsList::create_list(CFileParserAdv::OP_BRACKET_RIGHT, ")", CParserSymbolsList::READ_BY_SYMBOL));
	m_operand_types_data->add(CParserSymbolsList::create_list(CFileParserAdv::OP_BRACKET_LEFT_2, "[", CParserSymbolsList::READ_BY_SYMBOL));
	m_operand_types_data->add(CParserSymbolsList::create_list(CFileParserAdv::OP_BRACKET_RIGHT_2, "]", CParserSymbolsList::READ_BY_SYMBOL));
	m_operand_types_data->add(CParserSymbolsList::create_list(CFileParserAdv::OP_OPERATION_LEFT, "=", CParserSymbolsList::READ_BY_SYMBOL));
	m_operand_types_data->add(CParserSymbolsList::create_list(CFileParserAdv::OP_OPERATION_RIGHT, "+-|*", CParserSymbolsList::READ_BY_SYMBOL));
	// Note: '/'-operation appears as 'OP_OPERATION_RIGHT' after comments separation
	m_operand_types_data->add(CParserSymbolsList::create_list(CFileParserAdv::OP_VALUE_STRING, "\"", CParserSymbolsList::READ_BY_SYMBOL));
	m_operand_types_data->add(CParserSymbolsList::create_list(CFileParserAdv::OP_SLASH, "/", CParserSymbolsList::READ_BY_SYMBOL));
}

CScriptParser::~CScriptParser() {}

int CScriptParser::get_left_operation_offset(bool from_start) {
	return CFileParserAdv::get_operation_offset(CFileParserAdv::OP_OPERATION_LEFT,from_start);
}

// may call itself recursively
//bool CScriptParser::read_next_operand() { return CFileParserAdv::read_next_operand(); }
