////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ScriptValue.cpp: implementation of the CScriptValue class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

// Note: CScriptValueImpl::VAL_DEFAULT_IMPL (==0x8000) have the same segment
const int CScriptValue::VAL_EXPRESSION		= 0x0001;
const int CScriptValue::VAL_INT				= 0x0002;
const int CScriptValue::VAL_STRING			= 0x0004;
const int CScriptValue::VAL_UNDEFINED		= 0x0008;
const int CScriptValue::VAL_DIAGRAM_SET		= 0x0010;
const int CScriptValue::VAL_MATRIX_ELEMENT	= 0x0020; // S-Matrix element
const int CScriptValue::VAL_ARRAY			= 0x0080; // array of other script-values
const int CScriptValue::VAL_ARRAY_ELEMENT	= 0x0100; // Abstract A[x,y][j,k,l] construction
//const int CScriptValue::VAL_SYMBOL			= 0x0041; // Expression | Symbol-check

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CScriptValue::CScriptValue() : CESerializable()
{
}

CScriptValue::~CScriptValue()
{
}

CScriptValue* CScriptValue::newValue(CExpression *val, bool is_cloned)
{
	return new CScriptValueImpl(val, is_cloned);
}

CScriptValue* CScriptValue::newValue(CEString* str, bool is_cloned)
{
	return new CScriptValueImpl(str, is_cloned);
}

CScriptValue* CScriptValue::newValue(const char* str) { return new CScriptValueImpl(new CEString(str), true); }

CScriptValue* CScriptValue::newValue(long value)
{
	return new CScriptValueImpl(value);
}

CScriptValue* CScriptValue::newUndefinedValue()
{
	return new CScriptValueImpl();
}

CScriptValue* CScriptValue::newValue(CDiagramSet* diags, bool is_cloned)
{
	return new CScriptValueDiagramSet(diags, is_cloned);
}

CScriptValue* CScriptValue::newObjectValue(int type, CESerializable* obj, bool is_cloned)
{
	// check up type:
	// expression, string and diagram set objects should be kept in specific containers.
	if (type & VAL_EXPRESSION) {
		return newValue((CExpression*)obj, is_cloned);
	} else if (type & VAL_STRING) {
		return newValue((CEString*)obj, is_cloned);
	} else if (type & VAL_DIAGRAM_SET) {
		return newValue((CDiagramSet*)obj, is_cloned);
	} else if (type & VAL_INT) {
		type = VAL_UNDEFINED;
	}

	return new CScriptObjectValue(type, obj, is_cloned);
}

long CScriptValue::get_int_value() { throw(new CScriptException(CScriptErrors::INTERNAL_ERROR, -1, -1)); }
CEString* CScriptValue::get_str_value() { throw(new CScriptException(CScriptErrors::INTERNAL_ERROR, -1, -1)); }
CExpression* CScriptValue::get_expr_value() { throw(new CScriptException(CScriptErrors::INTERNAL_ERROR, -1, -1)); }
CExpression* CScriptValue::TurnToExprCloned() { return NULL; }
bool CScriptValue::equals(CScriptValue* other) { return (other == this); }

CScriptValue* CScriptValue::newArray(CScriptValue* val, bool is_cloned) {
	CScriptValueArray* res=new CScriptValueArray(); res->addToArray(val,is_cloned); return res;
}

void CScriptValue::addToArray(CScriptValue* val, bool is_cloned) { CEException::halt("CScriptValue::addToArray(): value is NOT an array"); }
int CScriptValue::getArraySize() const {  CEException::halt("CScriptValue::getArraySize(): value is NOT an array"); return 0; }
CScriptValue* CScriptValue::getArrayItem(int i) {  CEException::halt("CScriptValue::getArrayItem(): value is NOT an array"); return NULL; }
void CScriptValue::setArrayItem(int i, CScriptValue* val, bool is_cloned) {  CEException::halt("CScriptValue::setArrayItem(): value is NOT an array"); }
void CScriptValue::deleteArrayItem(int i) {  CEException::halt("CScriptValue::deleteArrayItem(): value is NOT an array"); }

CESymbol* CScriptValue::GetValidExprSymbol() { // can be casted ((CESymbol*)get_expr_value()) and (res!=NULL)
	CESymbol* res=NULL;
	if (check_value(CScriptValue::VAL_EXPRESSION,true)) {
		CExpression* expr=get_expr_value(); if (expr->ClassName() == CESymbol::CLASS_NAME) res=((CESymbol*)expr);
	}
	return res;
}
