////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ScriptValueImpl.cpp: implementation of the CScriptValueImpl class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

const char* CScriptValueImpl::CLASS_NAME = "CScriptValueImpl";
const int CScriptValueImpl::VAL_DEFAULT_IMPL	= 0x8000;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CScriptValueImpl::CScriptValueImpl(CExpression* val, bool is_cloned)
: CScriptValue()
{
	m_type = (CScriptValue::VAL_EXPRESSION | VAL_DEFAULT_IMPL);
	m_e_val = NULL;
	m_s_val = NULL;
	m_i_val = 0;
	if (is_cloned) m_e_val = val;
	else if (val != NULL) m_e_val = val->clone();
}

CScriptValueImpl::CScriptValueImpl(CEString* str, bool is_cloned)
: CScriptValue()
{
	m_type = (CScriptValue::VAL_STRING | VAL_DEFAULT_IMPL);
	m_e_val = NULL;
	m_s_val = NULL;
	m_i_val = 0;
	if (is_cloned) m_s_val = str;
	else if (str != NULL) m_s_val = str->clone();
}

CScriptValueImpl::CScriptValueImpl(long value)
: CScriptValue()
{
	m_type = (CScriptValue::VAL_INT | VAL_DEFAULT_IMPL);
	m_e_val = NULL;
	m_s_val = NULL;
	m_i_val = value;
}

CScriptValueImpl::CScriptValueImpl() : CScriptValue()
{
	m_type = (CScriptValue::VAL_UNDEFINED | VAL_DEFAULT_IMPL);
	m_e_val = NULL;
	m_s_val = NULL;
	m_i_val = LONG_MIN;
}

/*
*	private constructor for cloning
*	input parameters supposed as not cloned yet
*/
CScriptValueImpl::CScriptValueImpl(int _type, CExpression* val, CEString* str, long value)
: CScriptValue()
{
	m_type = _type;
	m_e_val = NULL;
	if (val != NULL) m_e_val = val->clone();
	m_s_val = NULL;
	if (str != NULL) m_s_val = str->clone();
	m_i_val = value;
}

CScriptValueImpl::~CScriptValueImpl()
{
	if (m_e_val != NULL) delete m_e_val;
	if (m_s_val != NULL) delete m_s_val;
}

int CScriptValueImpl::value_type()
{
	return m_type;
}

bool CScriptValueImpl::check_value_type(int type_mask)
{
	if (m_type & type_mask) return true;
	return false;
}

bool CScriptValueImpl::check_value(int type_mask, bool check_object)
{
	if (!check_value_type(type_mask)) return false;
	if (check_object) {
		if (check_value_type(CScriptValue::VAL_EXPRESSION)) {
			if (get_expr_value() == NULL) return false;
		}
		if (check_value_type(CScriptValue::VAL_STRING)) {
			if (get_str_value() == NULL) return false;
		}
	}
	return true;
}

CEString* CScriptValueImpl::toString(int mode)
{
	CEString* res = NULL;
	if (m_type & CScriptValue::VAL_EXPRESSION) {
		if (m_e_val != NULL) res = m_e_val->toString(mode);
		else res = new CEString("<NULL>");
	}
	else if (m_type & CScriptValue::VAL_STRING) {
		if (m_s_val != NULL) {
			res = new CEString("\"");
			res->AddString(m_s_val, false);
			res->AddString("\"");
		}
		else res = new CEString("<Null string>");
	}
	else if (m_type & CScriptValue::VAL_INT) {
		res = new CEString("");
		res->add_int(m_i_val);
	}
	else if (m_type & CScriptValue::VAL_UNDEFINED) {
		res = new CEString("<Undefined>");
	}
	if (res == NULL) res = new CEString("<Invalid Script Value>");
	return res;
}

bool CScriptValueImpl::equals(CScriptValue* other)
{
	return (this == other);
}

CScriptValue* CScriptValueImpl::clone() const
{
	return new CScriptValueImpl(m_type, m_e_val, m_s_val, m_i_val);
}

long CScriptValueImpl::get_int_value()
{
	return m_i_val;
}

CEString* CScriptValueImpl::get_str_value()
{
	return m_s_val;
}

CExpression* CScriptValueImpl::get_expr_value()
{
	return m_e_val;
}

CEObject* CScriptValueImpl::get_obj_value()
{
	if (m_type & CScriptValue::VAL_EXPRESSION) return m_e_val;
	else if (m_type & CScriptValue::VAL_STRING) return m_s_val;
	else if (m_type & CScriptValue::VAL_INT) return NULL;

	throw(new CScriptException(CScriptErrors::INTERNAL_ERROR, -1, -1));
	//return NULL;
}

CExpression* CScriptValueImpl::TurnToExprCloned() { // expr, arr->seq, numbers->CENumber, arr-elem->AbstrFunct
	CExpression* res=NULL;
	if (m_type & CScriptValue::VAL_EXPRESSION) {
		res=((m_e_val!=NULL)?m_e_val->clone():NULL);
	} else if (m_type & CScriptValue::VAL_INT) {
		res=new CENumber(m_i_val);
	} else if (m_type & CScriptValue::VAL_STRING) {
		if (m_s_val!=NULL) res=new CESymbol(m_s_val->getData());
	}
	return res;
}

const char* CScriptValueImpl::ClassName() const { return CLASS_NAME; }

//////////////////////////////////////////////////////////////////////
//
// implementation of the CScriptValueArray class.
//
//////////////////////////////////////////////////////////////////////
CScriptValueArray::CScriptValueArray() : CScriptValue() {
}

CScriptValueArray::~CScriptValueArray() {
}

CScriptValue* CScriptValueArray::clone() const {
	long s=m_vals.size(); //=getArraySize();
	CScriptValueArray* res=new CScriptValueArray();
	for(long i=0; i<s; i++) {
		const CScriptValue* item=(const CScriptValue*)m_vals.GetAtConst(i);
		if(item!=NULL)res->addToArray(item->clone(),true);
	}
	return res;
}

int CScriptValueArray::value_type() { return CScriptValue::VAL_ARRAY; }
bool CScriptValueArray::check_value_type(int type_mask) { return ((type_mask & CScriptValue::VAL_ARRAY)==VAL_ARRAY); }
bool CScriptValueArray::check_value(int type_mask, bool check_object) { return check_value_type(type_mask); }

CEString* CScriptValueArray::toString(int mode) {
	CEString* res = new CEString("<Array of size "); res->add_int(m_vals.size()); res->AddString(">"); return res;
}

CEObject* CScriptValueArray::get_obj_value() { return NULL; }

void CScriptValueArray::addToArray(CScriptValue* val, bool is_cloned) {
	if (val!=NULL) {
		if (!is_cloned) val=val->clone();
		m_vals.add(val);
	}
}

CExpression* CScriptValueArray::TurnToExprCloned() { // expr, arr->seq, numbers->CENumber, arr-elem->AbstrFunct
	CSequence* res=new CSequence();
	long s=getArraySize();
	for(long i=0; i<s; i++) {
		CScriptValue* item = getArrayItem(i);
		if(item==NULL)continue; // TODO: check if return NULL is better here
		CExpression* itemE=item->TurnToExprCloned();
		if(itemE==NULL)continue; // TODO: check if return NULL is better here
		res->addItem(itemE);
	}
	return res;
}

int CScriptValueArray::getArraySize() const { return m_vals.size(); }

CScriptValue* CScriptValueArray::getArrayItem(int i) {
	if ((i<0) || (i>=m_vals.size())) return NULL;
	return (CScriptValue*)m_vals.GetAt(i);
}

void CScriptValueArray::setArrayItem(int i, CScriptValue* val, bool is_cloned) {
	if (!is_cloned) val=val->clone();
	if ((i<0) || (i>=m_vals.size())) { delete val; return; }
	m_vals.SetAt(i, val, true);
}

void CScriptValueArray::deleteArrayItem(int i) {
	if ((i>=0) && (i<m_vals.size())) { m_vals.RemoveAt(i,true); }
}

const char* CScriptValueArray::CLASS_NAME = "CScriptValueArray";
const char* CScriptValueArray::ClassName() const { return CLASS_NAME; }

//////////////////////////////////////////////////////////////////////
//
// implementation of the CScriptValueArrayElement class.
//
//////////////////////////////////////////////////////////////////////
CScriptValueArrayElement::CScriptValueArrayElement(CScriptValue* _Title, CScriptValue* _Indices) : CScriptValue() {
	m_ArrTitle = NULL; if (_Title!=NULL) m_ArrTitle=_Title->clone();
	m_Indices = NULL; if (_Indices!=NULL) m_Indices=_Indices->clone();
}

CScriptValueArrayElement::~CScriptValueArrayElement() {
	if (m_ArrTitle!=NULL) delete m_ArrTitle;
	if (m_Indices!=NULL) delete m_Indices;
}

CScriptValue* CScriptValueArrayElement::clone() const { return new CScriptValueArrayElement(m_ArrTitle, m_Indices); }
int CScriptValueArrayElement::value_type() { return CScriptValue::VAL_ARRAY_ELEMENT; }
bool CScriptValueArrayElement::check_value_type(int type_mask) { return ((type_mask & CScriptValue::VAL_ARRAY)==VAL_ARRAY); }
bool CScriptValueArrayElement::check_value(int type_mask, bool check_object) { return check_value_type(type_mask); }

CEString* CScriptValueArrayElement::toString(int mode) {
	CEString* res = new CEString("<");
	if (m_ArrTitle!=NULL) res->AddString(m_ArrTitle->toString(mode)); else res->AddString("?");
	res->AddString("[");
	if (m_Indices!=NULL) res->AddString(m_Indices->toString(mode)); else res->AddString("?");
	res->AddString("]>");
	return res;
}
CEObject* CScriptValueArrayElement::get_obj_value() { return NULL; }

CExpression* CScriptValueArrayElement::TurnToExprCloned() { // expr, arr->seq, numbers->CENumber, arr-elem->AbstrFunct
	CScriptValue* ttle = GetTitle();
	CScriptValue* inds = GetIndices();
	if (ttle==NULL)return NULL;
	if (inds==NULL) {
		return ttle->TurnToExprCloned();
	}

	CExpression* ttleE=ttle->TurnToExprCloned();
	if (ttleE==NULL) return NULL;
	if (ttleE->ClassName() != CESymbol::CLASS_NAME) {
		delete ttleE;
		return NULL;
	}

	CExpression* indsE=inds->TurnToExprCloned();
	if (indsE==NULL) return ttleE; // TODO: check if return NULL is better here

	CEAbstractFunction* res=new CEFunction(((CESymbol*)ttleE)->get_name(),indsE);
	delete indsE;
	return res;
}

CScriptValue* CScriptValueArrayElement::GetTitle() { return m_ArrTitle; }
CScriptValue* CScriptValueArrayElement::GetIndices() { return m_Indices; }

const char* CScriptValueArrayElement::CLASS_NAME = "CScriptValueArrayElement";
const char* CScriptValueArrayElement::ClassName() const { return CLASS_NAME; }
