////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// SriptFunctionLib.h: interface for the CScriptFunctionLib class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SRIPTFUNCTIONLIB_H__41C7A45B_4803_4A1D_8F02_7E0FAEFC5B2B__INCLUDED_)
#define AFX_SRIPTFUNCTIONLIB_H__41C7A45B_4803_4A1D_8F02_7E0FAEFC5B2B__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CScriptFunctionLib : public CELibrary  
{
	CScriptFunctionLib();
public:
	static CDebug* dbg;

	//static CEPtrArray* FReturnError(CEPtrArray* arg_list, int invalid_argument_num);
	//static int FParamsCheck(CEPtrArray* arg_list, int par_num, int* par_types);

	// local utils
	static CExpression* GetExpressionCpy(CScriptValue* val); //val: str, int or expr

	// script functions
	// basic operations
	static CEPtrArray* SFuncExpand(CEPtrArray* arg_list);
	static CEPtrArray* SFuncArrange(CEPtrArray* arg_list);
	static CEPtrArray* SFuncLoadExprFromFile(CEPtrArray* arg_list);
	static CEPtrArray* SFuncStoreValueToFile(CEPtrArray* arg_list);
	static CEPtrArray* SFuncStoreValueToNB(CEPtrArray* arg_list);
	static CEPtrArray* SFuncStoreValueToM(CEPtrArray* arg_list);
	static CEPtrArray* SFuncStoreTextToNB(CEPtrArray* arg_list);
	static CEPtrArray* SFuncStoreTextToTeX(CEPtrArray* arg_list);
	static CEPtrArray* SFuncNewFortranFile(CEPtrArray* arg_list);		// deprecated
	static CEPtrArray* SFuncCreateFortranProc(CEPtrArray* arg_list);	// deprecated
	static CEPtrArray* SFuncNewCodeFile(CEPtrArray* arg_list);
	static CEPtrArray* SFuncCreateCodeProc(CEPtrArray* arg_list);
	static CEPtrArray* SFuncMinimize(CEPtrArray* arg_list);
	static CEPtrArray* SFuncFactor(CEPtrArray* arg_list);
	static CEPtrArray* SFuncEquals(CEPtrArray* arg_list);
	static CEPtrArray* SFuncEvaluate(CEPtrArray* arg_list);
	static CEPtrArray* SFuncSimplify(CEPtrArray* arg_list);

	// feyn art / feyn calc (basic HEP operations)
	static CEPtrArray* SFuncComposeDiagrams(CEPtrArray* arg_list);
	static CEPtrArray* SFuncDrawDiagrams(CEPtrArray* arg_list);
	static CEPtrArray* SFuncRetrieveME(CEPtrArray* arg_list);
	static CEPtrArray* SFuncComposeDiagsWick(CEPtrArray* arg_list);
	static CEPtrArray* SFuncCreateME(CEPtrArray* arg_list);
	static CEPtrArray* SFuncSquareME(CEPtrArray* arg_list);
	static CEPtrArray* SFuncCalcAmplitude(CEPtrArray* arg_list);
	static CEPtrArray* SFuncToDiffCS(CEPtrArray* arg_list);
	static CEPtrArray* SFuncLoadModel(CEPtrArray* arg_list);
	static CEPtrArray* SFuncGetSoftBremsstrahlung(CEPtrArray* arg_list);
	static CEPtrArray* SFuncRun(CEPtrArray* arg_list);

	// environment sets
	static CEPtrArray* SFuncSetParameter(CEPtrArray* arg_list);
	static CEPtrArray* SFuncSetParameterS(CEPtrArray* arg_list);
	static CEPtrArray* SFuncSetModelParameter(CEPtrArray* arg_list); // used by LoadModel() function
	static CEPtrArray* SetKinematics(CEPtrArray* arg_list);
	static CEPtrArray* SFuncSetMassCalcOrder(CEPtrArray* arg_list);
	static const int SelectDiagrams_LOOPS;
	static const int SelectDiagrams_CT;
	static CEPtrArray* SFuncSelectDiagrams(CEPtrArray* arg_list);
	static CEPtrArray* SFuncCollectDiffDiagrams(CEPtrArray* arg_list);
	static CEPtrArray* SFuncCollectEqualDiagrams(CEPtrArray* arg_list);
	static CEPtrArray* SFuncSetPhotonHelicity(CEPtrArray* arg_list);
	static CEPtrArray* SFuncSetPhotonDMBase(CEPtrArray* arg_list);
	static CEPtrArray* SFuncSetStoksParams(CEPtrArray* arg_list);
	static CEPtrArray* SFuncSetFermionHelicity(CEPtrArray* arg_list);
	static CEPtrArray* SFuncSetPolarized(CEPtrArray* arg_list);
	static CEPtrArray* SFuncSetMassSym(CEPtrArray* arg_list); // for pID
	static CEPtrArray* SFuncSetHelicityExpr(CEPtrArray* arg_list);
	static CEPtrArray* SFuncSetBondarevBaseSyms(CEPtrArray* arg_list);

	static CEPtrArray* SFuncSetDiagramPhysics(CEPtrArray* arg_list);
	static CEPtrArray* SFuncSetVirtIntegralValues(CEPtrArray* arg_list);
	static CEPtrArray* SFuncSetDRMassPK(CEPtrArray* arg_list);
	static CEPtrArray* SFuncSetNDimensionSpace(CEPtrArray* arg_list);
	static CEPtrArray* SFuncReduceTensorVI(CEPtrArray* arg_list);
	static CEPtrArray* SFuncConvertInvariantVI(CEPtrArray* arg_list);
	static CEPtrArray* SFuncCalcScalarVI(CEPtrArray* arg_list);
	static CEPtrArray* SFuncRestoreRenormConstants(CEPtrArray* arg_list);

	static CEPtrArray* SFuncGetMassSymPK(CEPtrArray* arg_list);

	// macros
	static CEPtrArray* SFuncInitial(CEPtrArray* arg_list);
	static CEPtrArray* SFuncFinal(CEPtrArray* arg_list);

	// view functions
	static CEPtrArray* SFuncViewFeynmanRules(CEPtrArray* arg_list);
	static CEPtrArray* SFuncViewTensorVITable(CEPtrArray* arg_list);
	static CEPtrArray* SFuncViewScalarVICache(CEPtrArray* arg_list);
	static CEPtrArray* SFuncViewConstants(CEPtrArray* arg_list);

	// kin arranges
	static CEPtrArray* SFuncKinArrange(CEPtrArray* arg_list);
	static CEPtrArray* SFuncKinSimplify(CEPtrArray* arg_list);
	static CEPtrArray* SFuncKinFullSimplify(CEPtrArray* arg_list);
	static CEPtrArray* SFuncSingularArrange(CEPtrArray* arg_list);

	// misc. utilites
	static CEPtrArray* SFuncSubstVecZero(CEPtrArray* arg_list);
	static CEPtrArray* SFuncSubstKinVector(CEPtrArray* arg_list);
	static CEPtrArray* SFuncGetUVTerm(CEPtrArray* arg_list);
	static CEPtrArray* SFuncRange(CEPtrArray* arg_list);

	// expr. operations
	static CEPtrArray* SFuncSym(CEPtrArray* arg_list);
	static CEPtrArray* SFuncNum(CEPtrArray* arg_list);
	static CEPtrArray* SFuncImU(CEPtrArray* arg_list);
	static CEPtrArray* SFuncInfinity(CEPtrArray* arg_list);
	static CEPtrArray* SFuncGetUnique(CEPtrArray* arg_list);
	static CEPtrArray* SFuncVec(CEPtrArray* arg_list);
	static CEPtrArray* SFuncEps(CEPtrArray* arg_list);
	static CEPtrArray* SFuncSqrt(CEPtrArray* arg_list);
	static CEPtrArray* SFuncSqr(CEPtrArray* arg_list);
	static CEPtrArray* SFuncSpr(CEPtrArray* arg_list);
	static CEPtrArray* SFuncPower(CEPtrArray* arg_list);
	static CEPtrArray* SFuncReplace(CEPtrArray* arg_list);
	static CEPtrArray* SFuncSubstVec(CEPtrArray* arg_list);
	static CEPtrArray* SFuncFrac(CEPtrArray* arg_list);
	static CEPtrArray* SFuncGamma(CEPtrArray* arg_list);
	static CEPtrArray* SFuncGellMann(CEPtrArray* arg_list);
	static CEPtrArray* SFuncToAsymmProd(CEPtrArray* arg_list);
	static CEPtrArray* SFuncTr(CEPtrArray* arg_list);
	static CEPtrArray* SFuncTrGellMann(CEPtrArray* arg_list);
	static CEPtrArray* SFuncFiertzTransform(CEPtrArray* arg_list);

	// debug functions
	static CEPtrArray* SFuncDumpDataStorage(CEPtrArray* arg_list);
	static CEPtrArray* SFuncTestCalc(CEPtrArray* arg_list);
	static CEPtrArray* SFuncTestCalcB(CEPtrArray* arg_list);
	static CEPtrArray* SFuncLog(CEPtrArray* arg_list);
	static CEPtrArray* SFuncViewParticleData(CEPtrArray* arg_list);
	static CEPtrArray* SFuncHalt(CEPtrArray* arg_list);
	static CEPtrArray* SFuncTimer(CEPtrArray* arg_list);
	static CEPtrArray* SFuncUseCharBank(CEPtrArray* arg_list);
	static CEPtrArray* SFuncChangeDebugLevel(CEPtrArray* arg_list);
	static CEPtrArray* SFuncDbgExtractVISet(CEPtrArray* arg_list);

	virtual ~CScriptFunctionLib();
};

#endif // !defined(AFX_SRIPTFUNCTIONLIB_H__41C7A45B_4803_4A1D_8F02_7E0FAEFC5B2B__INCLUDED_)
