	////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// SriptProcessor.h: interface for the CScriptProcessor class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SRIPTPROCESSOR_H__76E1AEB0_DD85_457F_9258_37430B74545E__INCLUDED_)
#define AFX_SRIPTPROCESSOR_H__76E1AEB0_DD85_457F_9258_37430B74545E__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CScriptProcessor : public CEAbstractSession  
{
	static CEMap* m_functions_register;
	static bool m_init_OK;
	//static CEMap* m_map_operations;

	CScriptParser* m_parser;
	CEMap* m_map_locals;

	// string execute variables
	CScriptValue* m_whole_value;
	CScriptValue* m_curr_operand_value;
	CEPtrArray* m_last_func_values;

	// static link to working processor
	static CScriptProcessor* m_WorkingProc;

	DWORD m_line_brake_flags; // volatile variable, indicates line break conditions
	static const DWORD LINE_BRAKE_EOF;
	static const DWORD LINE_BRAKE_RIGHT_BRACE;
	static const DWORD LINE_BRAKE_PRIORITY;
	static const DWORD LINE_BRAKE_ANY_FLAG;
	//static const DWORD LINE_FLAG_CURRENT_SKIPED;

	int m_line_priority; // 0 for usual line, >0 for operations, sets by run_script_line(X,...) call

public:
	static CEPtrArray* invoke_function(CScriptFuncInfo *func_info, CEPtrArray* args);
	static CEMap* m_PreDefinedConstants;
	static void finalize();
	static bool init();
	static void addPreDefinedConstant(CExpression* src, long value);
	static CScriptValue* getPreDefinedConstant(CScriptValue* src);
	static CScriptValue* getPreDefinedConstant(CExpression* src);
	static CScriptValue* run_script(CScriptParser* prs);
	static CDebug* dbg;
	static CScriptValue* run_file_script(const char* file_name);
	CScriptProcessor(CScriptParser* _parser);
	virtual ~CScriptProcessor();

protected:
	CScriptValue* run_script_processor();
private:
	void init_variables();

	CScriptValue* run_script_line(int line_priority = 0, CScriptValue* _whole_val = NULL);
	void do_operation_involve(const char* c_op, int op_type);
	void update_string_value();
	int int_operation_binary(const char* c_op, int _whole, int _current);
	int int_operation_unitary(const char* c_op, int _current);
	CExpression* expr_operation_binary(const char* c_op, CExpression* _whole, CExpression* _current);
	CExpression* expr_operation_unitary(const char* c_op, CExpression* _current);
	CEString* str_operation_binary(const char* c_op, CEString* _whole, CEString* _current);
	CEString* strint_operation_binary(const char* c_op, CEString* _str, int _val);
	CEString* intstr_operation_binary(const char* c_op, int _val, CEString* _str);
	CEString* str_operation_unitary(const char* c_op, CEString* _current);
	int getOperandPriority(const char* c_op, int op_type);
	void clear_line_break_flag();
	void lefthand_operation_execute();
	void do_variable_involve(const char* c_op, int op_type);
	bool do_function_cast(const char* c_op, int op_type);
	void declare_script_error(int err_code = 0, int i_param = -1, const char* c_param = NULL);
	
	CEPtrArray* invoke_system_function(CScriptFuncInfo *func_info, CEPtrArray* args);
};

#endif // !defined(AFX_SRIPTPROCESSOR_H__76E1AEB0_DD85_457F_9258_37430B74545E__INCLUDED_)
