////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ScriptValue.h: interface for the CScriptValue class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SCRIPTVALUE_H__59B00DEF_5808_4F27_91B2_5BFE8FE1FB0C__INCLUDED_)
#define AFX_SCRIPTVALUE_H__59B00DEF_5808_4F27_91B2_5BFE8FE1FB0C__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CScriptValue : public CESerializable  
{
public:
	static void finalize();
	static const int VAL_EXPRESSION;
	//static const int VAL_SYMBOL; // Expression | Symbol-check, Note: No ((m&t)==t) checks are presumed!
	static const int VAL_INT;
	static const int VAL_STRING;
	static const int VAL_DIAGRAM_SET;
	static const int VAL_MATRIX_ELEMENT; // S-Matrix element
	static const int VAL_ARRAY; // array of other script-values
	static const int VAL_ARRAY_ELEMENT; // Abstract A[x,y][j,k,l] construction
	static const int VAL_UNDEFINED;
	// Note: CScriptValueImpl::VAL_DEFAULT_IMPL (==0x8000) have the same segment

	virtual bool equals(CScriptValue* other);
	virtual CScriptValue* clone() const = 0;
	virtual int value_type() = 0;
	virtual bool check_value_type(int type_mask) = 0;
	virtual bool check_value(int type_mask, bool check_object) = 0;

	static CScriptValue* newValue(CExpression* val, bool is_cloned = false);
	static CScriptValue* newValue(CEString* str, bool is_cloned = false);
	static CScriptValue* newValue(const char* str);
	static CScriptValue* newValue(CDiagramSet* diags, bool is_cloned = false);
	static CScriptValue* newValue(long value);
	static CScriptValue* newUndefinedValue();
	static CScriptValue* newObjectValue(int type, CESerializable* obj, bool is_cloned = false);
	virtual ~CScriptValue();

	virtual CEString* toString(int mode = 0) = 0;

	static CScriptValue* newArray(CScriptValue* val, bool is_cloned = false);
	virtual void addToArray(CScriptValue* val, bool is_cloned = false); // throws error if type!=VAL_ARRAY
	virtual int getArraySize() const; // throws error if type!=VAL_ARRAY
	virtual CScriptValue* getArrayItem(int i); // throws error if type!=VAL_ARRAY
	virtual void setArrayItem(int i, CScriptValue* val, bool is_cloned = false); // throws error if type!=VAL_ARRAY
	virtual void deleteArrayItem(int i); // throws error if type!=VAL_ARRAY

	virtual long get_int_value();
	virtual CEString* get_str_value();
	virtual CExpression* get_expr_value();
	virtual CExpression* TurnToExprCloned(); // expr, arr->seq, numbers->CENumber, arr-elem->AbstrFunct
	virtual CEObject* get_obj_value() = 0;

public:
	// utilites
	CESymbol* GetValidExprSymbol(); // returns valid (CESymbol*)get_expr_value() or NULL

protected:
	CScriptValue();
};

#endif // !defined(AFX_SCRIPTVALUE_H__59B00DEF_5808_4F27_91B2_5BFE8FE1FB0C__INCLUDED_)
