////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ScriptValueImpl.h: interface for the CScriptValueImpl class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SCRIPTVALUEIMPL_H__92AA95C9_C0F2_4A98_A299_C612B496E4AD__INCLUDED_)
#define AFX_SCRIPTVALUEIMPL_H__92AA95C9_C0F2_4A98_A299_C612B496E4AD__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CScriptValueImpl : public CScriptValue  
{
	int m_type;

	CExpression* m_e_val;
	CEString* m_s_val;
	long m_i_val;

	static const int VAL_DEFAULT_IMPL;

private:
	CScriptValueImpl(int _type, CExpression* val, CEString* str, long value);
public:
	CScriptValueImpl(CExpression* val, bool is_cloned = false);
	CScriptValueImpl(CEString* str, bool is_cloned = false);
	CScriptValueImpl(long value);
	CScriptValueImpl();

	virtual ~CScriptValueImpl();
	virtual int value_type();
	virtual bool check_value_type(int type_mask);
	virtual bool check_value(int type_mask, bool check_object);
	virtual CEString* toString(int mode = 0);
	virtual bool equals(CScriptValue* other);
	virtual CScriptValue* clone() const;

	virtual long get_int_value();
	virtual CEString* get_str_value();
	virtual CExpression* get_expr_value();
	virtual CEObject* get_obj_value();
	virtual CExpression* TurnToExprCloned(); // expr, arr->seq, numbers->CENumber, arr-elem->AbstrFunct

	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
};

class CScriptValueArray : public CScriptValue  
{
	CEPtrArray m_vals;
public:
	CScriptValueArray();
	virtual ~CScriptValueArray();

	virtual CScriptValue* clone() const;
	virtual int value_type();
	virtual bool check_value_type(int type_mask);
	virtual bool check_value(int type_mask, bool check_object);
	virtual CEString* toString(int mode = 0);
	virtual CEObject* get_obj_value(); // returns NULL always
	virtual CExpression* TurnToExprCloned(); // expr, arr->seq, numbers->CENumber, arr-elem->AbstrFunct

	virtual void addToArray(CScriptValue* val, bool is_cloned = false); // throws error if type!=VAL_ARRAY
	virtual int getArraySize() const; // throws error if type!=VAL_ARRAY
	virtual CScriptValue* getArrayItem(int i); // throws error if type!=VAL_ARRAY
	virtual void setArrayItem(int i, CScriptValue* val, bool is_cloned = false); // throws error if type!=VAL_ARRAY
	virtual void deleteArrayItem(int i); // throws error if type!=VAL_ARRAY

	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
};

class CScriptValueArrayElement : public CScriptValue // used as pointer Arr[ind1,ind1][add1,add2,add3]
{
	CScriptValue* m_ArrTitle;
	CScriptValue* m_Indices; // may be CScriptValueArray, CScriptValueArray of CScriptValueArray
public:
	CScriptValueArrayElement(CScriptValue* _Title, CScriptValue* _Indices);
	virtual ~CScriptValueArrayElement();

	virtual CScriptValue* clone() const;
	virtual int value_type();
	virtual bool check_value_type(int type_mask);
	virtual bool check_value(int type_mask, bool check_object);
	virtual CEString* toString(int mode = 0);
	virtual CEObject* get_obj_value(); // returns NULL always
	virtual CExpression* TurnToExprCloned(); // expr, arr->seq, numbers->CENumber, arr-elem->AbstrFunct

	CScriptValue* GetTitle();
	CScriptValue* GetIndices();

	static const char* CLASS_NAME;
	virtual const char* ClassName() const;
};

#endif // !defined(AFX_SCRIPTVALUEIMPL_H__92AA95C9_C0F2_4A98_A299_C612B496E4AD__INCLUDED_)
