////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// ESupport.cpp: implementation of the CESupport class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

CDebug* CESupport::dbg = new CDebug("CESupport");

CEMap* CESupport::m_XML_tag_reference = NULL;

const DWORD CESupport::SERIAL_EXPRESSION	= 0x01;
const DWORD CESupport::SERIAL_VERIFY		= 0x02;

//const int CESupport::FILE_APPEND	= 0x01;
const int CESupport::FILE_TRUNCATE	= 0x01;
const int CESupport::FILE_CLOSE		= 0x02;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CESupport::CESupport() { }
CESupport::~CESupport() { }

//////////////////////////////////////////////////////////////////////
// Library functions
//////////////////////////////////////////////////////////////////////
void CESupport::SaveResults(CExpression *result)
{
	if (result == NULL) {
		Save("Null data pointer.");
		return;
	}
	const char* fn = "results.nb";
	ofstream ofs(fn, ios::out|ios::app);
	if (!ofs) {
		log("<Error creating output file.>");
		return;
	}
	
	//ofs << getPrefix();
	ofs << ",\n\nCell[BoxData[\\(";
	CEString* s = result->toString(STORE_WR_MATHEMATICA);
	ofs << s->getData();
	delete s;
	ofs << "\\)], \"Output\"]";
	//ofs << getPostfix();
	ofs.close();
}

/*
*	(_flag < 0) means FILE_TRUNCATE | FILE_CLOSE (supposed default value).
*/
bool CESupport::SaveExprToNB(const char* fn, CESerializable* item, int _flags)
{
	if (_flags < 0) _flags = FILE_TRUNCATE | FILE_CLOSE;
	if ((fn == NULL) || (item == NULL)) return false;

	bool do_app = ((_flags & FILE_TRUNCATE) == 0);
	ofstream ofs(fn, (do_app)?(ios::out|ios::app):(ios::out|ios::trunc));
	if (!ofs) return false;
	if (_flags & FILE_TRUNCATE) { SaveNBPreface(&ofs); }

	ofs << ",\n\nCell[BoxData[\\(";
	CEString* s = item->toString(STORE_WR_MATHEMATICA);
	ofs << s->getData();
	delete s;
	ofs << "\\)], \"Output\"]";

	if (_flags & FILE_CLOSE) { SaveNBPostfix(&ofs); }
	ofs.close();
	return true;
}

bool CESupport::SaveExprToM(const char* fn, CESerializable* item) {
	if ((fn == NULL) || (item == NULL)) return false;
	ofstream ofs(fn, ios::out|ios::trunc); if (!ofs) return false;
	CEString* s = item->toString(STORE_WR_MATHEMATICA);
	ofs << "\\(";
	ofs << s->getData();
	ofs << "\\)";
	delete s;
	ofs.close();
	return true;
}

/*
*	(_flag < 0) means FILE_TRUNCATE | FILE_CLOSE (supposed default value).
*/
bool CESupport::SaveTextToNB(const char* fn, const char* text, int _flags) {
	if (_flags < 0) _flags = FILE_TRUNCATE | FILE_CLOSE;
	if (fn == NULL) return false;
	bool do_app = ((_flags & FILE_TRUNCATE) == 0);
	ofstream ofs(fn, (do_app)?(ios::out|ios::app):(ios::out|ios::trunc));
	if (!ofs) return false;
	if (_flags & FILE_TRUNCATE) { SaveNBPreface(&ofs); }
	
	ofs << ",\n\nCell[BoxData[\\(";
	if (text != NULL) { ofs << text; }
	else { ofs << "NULL text"; }
	ofs << "\\)], \"Text\"]";

	if (_flags & FILE_CLOSE) { SaveNBPostfix(&ofs); }
	ofs.close();
	return true;
}

/*
*	(_flag < 0) means FILE_TRUNCATE | FILE_CLOSE (supposed default value).
*/
bool CESupport::SaveHeadersToNB(const char* fn, int _flags) {
	if (_flags < 0) _flags = FILE_TRUNCATE | FILE_CLOSE;
	if (fn == NULL) return false;
	bool do_app = ((_flags & FILE_TRUNCATE) == 0);
	ofstream ofs(fn, (do_app)?(ios::out|ios::app):(ios::out|ios::trunc));
	if (!ofs) return false;
	if (_flags & FILE_TRUNCATE) { SaveNBPreface(&ofs); }
	if (_flags & FILE_CLOSE) { SaveNBPostfix(&ofs); }
	ofs.close();
	return true;
}

void CESupport::SaveNBPreface(void* p_ofs) {
	if (p_ofs == NULL) return;
	ofstream* pO = (ofstream*)p_ofs;
	*pO << getNBPrefix();
	*pO << "Cell[\"";
	*pO << CConst::EXPR_CORE_TITLE_SHORT;
	//CEString sVer; CESupport::CreateVersionString(&sVer); ofs << " Created by "<<sVer.getData()<<".\n";
	*pO << " generated file.\", \"Text\"]";
}

void CESupport::SaveNBPostfix(void* p_ofs) {
	if (p_ofs == NULL) return;
	ofstream* pO = (ofstream*)p_ofs;
	*pO << getNBPostfix();
}

// (settings < 0) means FILE_TRUNCATE (supposed default value).
bool CESupport::saveTeX(const char* fn, CESerializable* obj, int iFileSetts){ return saveTeXAdv(fn, obj, NULL, iFileSetts); }
bool CESupport::saveTeX(const char* fn, const char* msg, int iFileSetts, bool bConvertMsgToTex){ return saveTeXAdv(fn, NULL, msg, iFileSetts, bConvertMsgToTex); }
bool CESupport::saveTeXAdv(const char* fn, CESerializable* obj, const char* msg, int iFileSetts, bool bConvertMsgToTex) {
	if (iFileSetts < 0) iFileSetts = FILE_TRUNCATE;
	bool do_app = ((iFileSetts & FILE_TRUNCATE) == 0);
	ofstream ofs_tex(fn, (do_app)?(ios::out|ios::app):(ios::out|ios::trunc));
	if(!ofs_tex) { printf("<Error creating output file.>\n"); return false; }
	if (iFileSetts & FILE_TRUNCATE) {
		CEString sVer; CESupport::CreateVersionString(&sVer);
		ofs_tex<<"% Created by "<<sVer.getData();
		ofs_tex<<"\n\n"; //Line & paragraph break
	}

	//if((obj==NULL)&&(msg==NULL))msg="NULL string";

	if(msg!=NULL) {
		if(bConvertMsgToTex){
			long i=0;
			for(const char* tmp=msg;(*tmp!='\0');tmp++) { // text parsing for non-TeX symbols
				char c=*tmp;
				if((c=='_')||(c=='{')||(c=='}')||(c=='$')||(c=='*')||(c=='%')||(c=='&')){ ofs_tex<<'\\'<<c; }else{ ofs_tex<<c; }
				i++; if (i>10000) { dbg->log(0, "TeX output ERROR: too long string"); break; }
			}
			if(i>0)ofs_tex<<"\n\n"; //Line & paragraph break
		}else{
			ofs_tex<<msg;
			if(*msg != '\0')ofs_tex<<"\n\n"; //Line & paragraph break
		}
	}

	if(obj!=NULL){
		CEString* s=obj->toString(STORE_TEX); if(s==NULL)s=new CEString("<NULL>"); ofs_tex<<s->getData(); delete s;
		ofs_tex<<"\n\n"; //Line & paragraph break
	}

	ofs_tex.close();
	return true;
}

const char* CESupport::getNBPrefix() {
	return "(*NotebookFileLineBreakTest\nNotebookFileLineBreakTest*)\n(*NotebookOptionsPosition[      3005,        101]*)\n(*NotebookOutlinePosition[      3657,        124]*)\n(*  CellTagsIndexPosition[      3613,        120]*)\n(*WindowFrame->Normal*)\n\nNotebook[{\n";
}

const char* CESupport::getNBPostfix()
{
	//return "\n},\nFrontEndVersion->\"4.0 for Microsoft Windows\",\nScreenRectangle->{{0, 1024}, {0, 723}},\nWindowSize->{1016, 696},\nWindowMargins->{{0, Automatic}, {Automatic, 0}}\n]\n\n(*NotebookFileOutline\nNotebook[{\n}\n]\n*)\n\n";

	const char* res = NULL;
	switch (CEProcessor::m_ScreenSize) {
	case 2:
		// 1400x1050 (w,h-45) (w-104,h-72)
		res = "\n},\nFrontEndVersion->\"4.0 for Microsoft Windows\",\nScreenRectangle->{{0, 1400}, {0, 1005}},\nWindowSize->{1296, 978},\nWindowMargins->{{0, Automatic}, {Automatic, 0}}\n]\n\n(*NotebookFileOutline\nNotebook[{\n}\n]\n*)\n\n";
		break;
	case 1:
	default:
		//for 800x600:
		//res = "\n},\nFrontEndVersion->\"4.0 for Microsoft Windows\",\nScreenRectangle->{{0, 800}, {0, 527}},\nWindowSize->{792, 500},\nWindowMargins->{{0, Automatic}, {Automatic, 0}}\n]\n\n(*NotebookFileOutline\nNotebook[{\n}\n]\n*)\n\n";
		// 1024x768 (w,h-45) (w-104,h-72)
		res = "\n},\nFrontEndVersion->\"4.0 for Microsoft Windows\",\nScreenRectangle->{{0, 1024}, {0, 723}},\nWindowSize->{920, 696},\nWindowMargins->{{0, Automatic}, {Automatic, 0}}\n]\n\n(*NotebookFileOutline\nNotebook[{\n}\n]\n*)\n\n";
	}
	return res;
}

bool CESupport::m_resultsNBcreated = false;
bool CESupport::initSupport()
{
	/*const char* fn = "results.nb";
	ofstream ofs(fn);
	if (!ofs) {
		log("<Error creating output file.>");
		return false;
	}
	
	ofs << getPrefix();
	ofs << "Cell[\"CESupport's output list.\", \"Text\"]";
	ofs.close();

	const char* fn_tex = "res.tex";
	ofstream ofs_tex(fn_tex);
	if (!ofs_tex) {
		log("<Error creating output file.>");
		return false;
	}
	ofs_tex.close();*/
	return true;
}

bool CESupport::finalize()
{
	if (m_resultsNBcreated) {
		const char* fn = "results.nb";
		ofstream ofs(fn, ios::out|ios::app);
		if (!ofs) {
			printf("<Error creating output file.>");
			return false;
		}
		ofs << getNBPostfix();
		ofs.close();
	}
	
	if (m_XML_tag_reference != NULL) { delete m_XML_tag_reference; m_XML_tag_reference = NULL; }
	return true;
}

void CESupport::PrintLogoPage() { // to console output
	printf(CConst::EXPR_LOGO_TEXT1); printf("\n");
	printf(CConst::EXPR_LOGO_TEXT2); printf("\n");

	//  ------------------------------  declare version
	CEString sVerNo("Version: "); sVerNo.add_int(CConst::EXPR_VERSION_ID);
	sVerNo.AddString("."); sVerNo.add_int(CConst::EXPR_VERSION_MAJOR);
	sVerNo.AddString("."); sVerNo.add_int(CConst::EXPR_VERSION_MINOR);
	//sVerNo.AddString(", Build "); sVerNo.add_int(CConst::EXPR_VERSION_ID);
	//sVerNo.AddString("build date: "); sVerNo.AddString(CConst::EXPR_CORE_BUILD_DATE);
	char* s_tmp = CEUtil::copyString(CConst::EXPR_LOGO_TEXT_EMPTY);
	int l_tmp = CEUtil::length(s_tmp);
	if(l_tmp>sVerNo.length()) { // copy build-date substring to s_tmp
		int pos = l_tmp/2-sVerNo.length()/2;
		CEUtil::copySymsTo((s_tmp+pos), sVerNo.getData(), -1, 20);
		printf(s_tmp);printf("\n");
	} else {
		printf(sVerNo.getData());
	}
	CMemoryManager::mfree(s_tmp);

	printf(CConst::EXPR_LOGO_TEXT3); printf("\n");
	//printf(CConst::EXPR_LOGO_TEXT3b); printf("\n"); // mirror website
	printf(CConst::EXPR_LOGO_TEXT4); printf("\n");
	printf(CConst::EXPR_LOGO_TEXT5); printf("\n");
}

void CESupport::CreateVersionString(CEString* str) { // "ALHEP 1.1.2"
	if(str!=NULL){
		*str << "ALHEP " << CConst::EXPR_VERSION_ID << "." << CConst::EXPR_VERSION_MAJOR << "." << CConst::EXPR_VERSION_MINOR;
	}
}

void CESupport::Save(const char *data)
{
	const char* fn = "results.nb";
	ofstream ofs(fn, ios::out|ios::app);
	if (!ofs) {
		log("<Error creating output file.>");
		return;
	}
	
	//ofs << getPrefix();
	ofs << ",\n\nCell[\"";
	ofs << data;
	ofs << "\", \"Text\", FormatType->TextForm]";
	//ofs << getPostfix();
	ofs.close();
}

void CESupport::SaveGraphics(CESerializable *obj)
{
	if (obj == NULL) {
		Save("Null graphics object pointer.");
		return;
	}
	const char* fn = "results.nb";
	ofstream ofs(fn, ios::out|ios::app);
	if (!ofs) {
		log("<Error creating output file.>");
		return;
	}
	
	//ofs << getPrefix();
	ofs << ",\n\nCell[GraphicsData[\"PostScript\", \"\\<\\\n";
	CEString* s = obj->toString(STORE_WR_MATHEMATICA);
	ofs << s->getData();
	delete s;
	ofs << "\\>\"], \"Graphics\"]";
	//ofs << getPostfix();
	ofs.close();
}

void CESupport::savetex(CESerializable *obj)
{
	if (obj == NULL) {
		Save("Null graphics object pointer.");
		return;
	}
	const char* fn_tex = "res.tex";
	ofstream ofs_tex(fn_tex, ios::out|ios::app);
	if (!ofs_tex) {
		log("<Error creating output file.>");
		return;
	}
	
	CEString* s = obj->toString(STORE_TEX);
	ofs_tex << s->getData();
	delete s;
	ofs_tex.close();
}

void CESupport::Save(CEString * s)
{
	if (s == NULL) {
		Save("Null data pointer.");
		return;
	}
	const char* fn = "results.nb";
	ofstream ofs(fn, ios::out|ios::app);
	if (!ofs) {
		log("<Error creating output file.>");
		return;
	}
	
	//ofs << getPrefix();
	ofs << ",\n\nCell[\"";
	ofs << s->getData();
	delete s;
	ofs << "\", \"Text\", FormatType->TextForm]";
	//ofs << getPostfix();
	ofs.close();
}

/*
*	Store object to internal format.
*/
void CESupport::store(CESerializable *data, const char *fn, DWORD settings)
{
	ofstream ofs(fn, ios::out|ios::ate);
	if (!ofs) {printf("<Error creating output file.>\n");return;}

	long f_sz = store_stream(data, &ofs, settings);
	printf("Output created: %s; size: %ld\n", fn, f_sz);
	ofs.close();

	bool _Verify=((settings & SERIAL_VERIFY) == SERIAL_VERIFY);
	// Skip verification for diagrams -- it may contain UNIQUE symbols, replaced at reading
	if (CEUtil::equals(data->ClassName(),CDiagramSet::CLASS_NAME)) _Verify=false;
	if (_Verify) {
		CESerializable* read_val = read_file(fn, settings);
		bool are_equal = false;
		if (read_val != NULL) {
			if (settings & SERIAL_EXPRESSION) {
				are_equal = ((CExpression*)data)->equals((CExpression*)read_val);
			}
			else {
				CEString* s1 = data->toString();
				CEString* s2 = read_val->toString();
				are_equal = s1->equals(s2);
				delete s1;
				delete s2;
			}
		}
		if (are_equal) printf("Verification for output file %s successful.\n", fn);
		else {
			dbg->log(0, "Verification FAILED for output file: ", fn);
			dbg->log(0, "stored expression (see debug file)", data);
			dbg->log(0, "read expression (see debug file)", read_val);
		}
		delete read_val;
	}
}

long CESupport::store_stream(CESerializable* data, ofstream* p_ofs, DWORD settings) {
	if ((p_ofs == NULL) || (data==NULL)) return 0;
	const char* tag = data->get_TAG_Name();
	long tag_len = CEUtil::length(tag);
	long res = 0;
	*p_ofs << "<" << tag;
	res += (tag_len+1);

	// store object flags (sign/level settings, & others)
	void* p_flags_map = data->getStorageFlags();
	if (p_flags_map != NULL) {
		CEMap* flags_map = (CEMap*)p_flags_map;
		res += store_flags_map(p_ofs, flags_map);
		delete flags_map;
	}
	*p_ofs << ">";
	res += 1;
	long item_store_len = data->store_stream(p_ofs, settings);
	if (item_store_len < 0) {
		//Error
		*p_ofs << "Error";
		res += 5;
	}
	else {
		res += item_store_len;
	}

	*p_ofs << "</" << tag << ">";
	res += (tag_len+3);
	return res;
}

long CESupport::store_flags_map(ofstream* p_ofs, CEMap* f_map) {
	if ((p_ofs == NULL) || (f_map == NULL)) return 0;
	long res = 0;

	f_map->reset_iterator();
	for(;;) {
		if (!f_map->has_next()) break;
		
		CENamedItem* item = (CENamedItem*)f_map->next();
		if (item == NULL) break; // same check to has_next();
		
		const char* item_id = item->getId();
		if (item_id == NULL) { // declare internal error
			CEException::halt("<Object storage> Internal error: NULL in storage flags map"); }
		
		*p_ofs << " " << item_id << "=\"";
		res += (3 + CEUtil::length(item_id));

		const char* flag_val = NULL;
		// item may be: CEString object or const char* pointer
		if ((item->get_item_type()) & CENamedItem::NI_OBJECT) {
			flag_val = ((CEString*)item->getObject())->getData();
		} else {
			flag_val = (const char*)item->get_ptr();
		}
		if (flag_val == NULL) { CEException::halt("<Object storage> Internal error: NULL in storage flags map"); }

		*p_ofs << flag_val;
		res += CEUtil::length(flag_val);
		
		*p_ofs << '\"';
		res += 1;
	}
	return res;
}

CESerializable* CESupport::read_file(const char *fn, DWORD settings)
{
	char* buf = CAbstractFileParser::LoadFileToMemory(fn);
	if (buf == NULL) { printf("Error loading file"); return NULL; }

	/*
	ifstream ifs(fn, ios::in | ios::nocreate);
	if (!ifs) { printf("<file \"%s\" not found.>\n", fn); return NULL; }
	//printf("ifs.eof(): %ld\n", ifs.eof());

	//	reading file to buffer
	//long buf_size = 0x3fffff;	//  4Mb buffer
	long buf_size = 0xffffff;	//  16Mb buffer
	//long buf_size = 0xffff;	// 64kb buffer (for tests)
	char* buf = (char*)CMemoryManager::mmalloc(buf_size);
	long i = 1;
	char* buf_curr = buf;
	bool loop = true;
	for(;loop;i++) {
		if (i >= buf_size) {
			loop = false;
			*buf_curr = '\0';
		}
		else {
			int c=ifs.get();
			if (c != EOF) {
				*buf_curr = c;
				buf_curr++;
			}
			else {
				loop = false;
				*buf_curr = '\0';
			}
		}
	}
	ifs.close();
	long buf_siz = buf_curr-buf;*/
	
	long buf_siz = CEUtil::length(buf);
	//printf("<file \"%s\" loaded to memory, size: %ld.>\n", fn, buf_siz);
	dbg->log(55, buf_siz, "file loaded to memory (size: %ld)");
	return read_buffer(buf, 0, settings);
}

CESerializable* CESupport::read_buffer(const char *data, long pos, DWORD settings) {
	dbg->log(55, pos, "read_buffer(pos: %ld)");

	char* tag = CUXMLparcer::get_tag_id(data, pos);
	if (*tag == '/') {return NULL;}
	if (m_XML_tag_reference == NULL) m_XML_tag_reference = CEMap::createNewMap();
	void* f_ptr = m_XML_tag_reference->get_ptr(tag);
	if (f_ptr == NULL) {
		//Error: class construction not registered
		printf("Error: Class constructor for TAG <%s> is not registered\n",tag);
		CMemoryManager::mfree(tag);
		return NULL;
	}
	//printf("<%s>", tag);
	dbg->log(75, tag);

	CEMap* params = CUXMLparcer::get_tag_values_map(data, pos);
	pos = CUXMLparcer::skip_tag_header(data, pos);

	//printf(".");

	CESerializable* (*pRestoreFunction)(const char*, long, CEMap*, DWORD);
	//pRestoreFunction = (CESerializable *(__cdecl *)(const char *,long, CEMap *,unsigned long))f_ptr;
	pRestoreFunction = (CESerializable *(*)(const char *,long, CEMap *,unsigned long))f_ptr;
	CESerializable* res = (*pRestoreFunction)(data, pos, params, settings);

	//printf("!");
	if (res != NULL) {
		//parse params for sign and level here and apply them to result
		if (params != NULL) {
			void* v_sl_flags = params->get("f");
			if (v_sl_flags != NULL) {
				if (CEBasic::isExpression(res)) {
					CEString* sl_str = (CEString*)v_sl_flags;
					char* sl_flags = sl_str->getData();
					if (*sl_flags == 's') {
						((CExpression*)res)->unsetFlag();
						sl_flags++;
					}
					if (*sl_flags == 'l') {
						((CExpression*)res)->setLevel(false);
					}
				} else {
					dbg->log(0, "CESupport::read_buffer() ERROR: FLAGS read for NON-EXPRESSION object", res);
				}
			}
		}
	}

	//printf("</%s>", tag);
	if (params != NULL) delete params;
	CMemoryManager::mfree(tag);
	return res;
}

/*
*	returns -1 if there are no item to skip
*/
long CESupport::skip_item_in_buffer(const char *data, long pos, DWORD settings) {
	return CUXMLparcer::skip_tag(data, pos);
}

/*
*	TODO: fix this procedure to return 1 or 2
*/
long CESupport::storeLineBreak(ofstream* p_ofs) { // returns number of bytes stored (1 or 2)
	*p_ofs << '\n'; return 2;
}

long CESupport::storeXMLLong(long val, ofstream* p_ofs, DWORD settings) { // as CENumber
	CENumber* n = new CENumber(val);
	long res = CESupport::store_stream(n, p_ofs, settings);
	delete n;
	return res;
}

bool CESupport::readXMLLong(const char* data,
							long* ppos, // updates pos if success
							long* pres,
							DWORD settings,
							const char* obj_id,
							bool NoObjectOk)
{
	long skip_pos = CESupport::skip_item_in_buffer(data, *ppos, settings);
	if (skip_pos <= 0) {
		if (NoObjectOk) return false;
		dbg->log(0, "Error restoring NUMBER (no NUMBER found), oblect: ", obj_id); return false;
	}

	CESerializable* num_void = CESupport::read_buffer(data, *ppos, settings);
	if ((num_void == NULL) || (!CEUtil::equals(num_void->ClassName(), CENumber::CLASS_NAME))) {
		dbg->log(0, "Error restoring NUMBER (error reding NUMBER), oblect: ", obj_id); return false;
	}

	// check up number value
	CENumber* num_obj = (CENumber*)num_void;
	long res = num_obj->getNum();
	if ((num_obj->getDen() != 1) || (num_obj->getNumIm() != 0)) {
		dbg->log(0, "Error restoring NUMBER (invalid NUMBER read), oblect: ", obj_id);
		delete num_obj;
		return false;
	}
	delete num_obj;

	*ppos = skip_pos;
	*pres = res;
	return true;
}

// obj_id == NULL means NO ERROR MESSAGE are reqiured
CExpression* CESupport::readXMLExpr(const char* data,
									long* ppos, // updates pos if success
									DWORD settings,
									const char* obj_id,
									bool NoObjectOk) {
	long skip_pos = CESupport::skip_item_in_buffer(data, *ppos, settings);
	if (skip_pos <= 0) {
		if (NoObjectOk) return NULL;
		if (obj_id != NULL) { dbg->log(0, "Error restoring EXPRESSION (no OBJECT found), oblect: ", obj_id); }
		return NULL;
	}

	CESerializable* e_void = CESupport::read_buffer(data, *ppos, settings);
	if ((e_void == NULL) || (!CEBasic::isExpression(e_void)) ) {
		if (!NoObjectOk) {
			if (obj_id != NULL) { dbg->log(0, "Error restoring EXPRESSION (UNEXPECTED object read), oblect: ", obj_id); }
			else dbg->log(0, "Error restoring EXPRESSION (NULL object read)");
		}
		if (e_void!=NULL)delete e_void;
		return NULL;
	}

	CExpression* res = (CExpression*)e_void;

	*ppos = skip_pos;
	return res;
}

// caller_id == NULL means NO ERROR MESSAGE are reqiured
CESerializable* CESupport::readXMLObj(const char* data,
								  long* ppos, // updates pos if success
								  const char* CN_check, // check object ClassName() to equal check_CN
								  DWORD settings,
								  const char* caller_id,
								  bool NoObjectOk) {
	long skip_pos = CESupport::skip_item_in_buffer(data, *ppos, settings);
	if (skip_pos <= 0) {
		if (NoObjectOk) return NULL;
		if (caller_id != NULL) { dbg->log(0, "Error restoring OBJECT (no OBJECT found), caller: ", caller_id); }
		return NULL;
	}

	CESerializable* e_void = CESupport::read_buffer(data, *ppos, settings);
	bool read_OK = (e_void != NULL);
	if (read_OK) {
		if (CN_check != NULL) {
			if (!CEUtil::equals(e_void->ClassName(),CN_check)) { read_OK=false; }
		}
	}

	if (!read_OK) {
		if (!NoObjectOk) {
			if (caller_id != NULL) {
				dbg->log(0, "Error restoring OBJECT (NULL or WRONG TYPE), caller: ", caller_id);
				dbg->log(2, "OBJECT restored: ", e_void);
				dbg->log(2, "TYPE expected: ", CN_check);
				delete e_void;
			}
		}
		return NULL;
	}

	*ppos = skip_pos;
	return e_void;
}

bool CESupport::checkAllRead(const char* data, long* ppos, DWORD settings, const char* caller_id) {
	long skip_pos = CESupport::skip_item_in_buffer(data, *ppos, settings);
	if (skip_pos > 0) {
		if (caller_id != NULL) { dbg->log(0, "ERROR reading object, CESupport::checkAllRead() FAILED, object: ", caller_id); }
		return false;
	}
	return true;
}

void CESupport::init_XML_TAG_structure() {
	if (m_XML_tag_reference == NULL) m_XML_tag_reference = CEMap::createNewMap();

	if (m_XML_tag_reference->size() > 0) {
		printf("Multiple XML-TAg structure initialization\n");
		return;
	}
	
	//CEMap* m_XML_tag_reference;
	m_XML_tag_reference->put_ptr(CESum::CLASS_TAG_NAME, (void*)CESum::restore_from_buffer);
	m_XML_tag_reference->put_ptr(CEProduct::CLASS_TAG_NAME, (void*)CEProduct::restore_from_buffer);
	m_XML_tag_reference->put_ptr(CENumber::CLASS_TAG_NAME, (void*)CENumber::restore_from_buffer);
	m_XML_tag_reference->put_ptr(CESymbol::CLASS_TAG_NAME, (void*)CESymbol::restore_from_buffer);
	m_XML_tag_reference->put_ptr(CEPower::CLASS_TAG_NAME, (void*)CEPower::restore_from_buffer);
	m_XML_tag_reference->put_ptr(CSequence::CLASS_TAG_NAME, (void*)CSequence::restore_from_buffer);
	m_XML_tag_reference->put_ptr(CESProduct::CLASS_TAG_NAME, (void*)CESProduct::restore_from_buffer);
	m_XML_tag_reference->put_ptr(CEAntiSymmProd::CLASS_TAG_NAME, (void*)CEAntiSymmProd::restore_from_buffer);
	m_XML_tag_reference->put_ptr(CEVector::CLASS_TAG_NAME, (void*)CEVector::restore_from_buffer);
	m_XML_tag_reference->put_ptr(CEAntisymmetric::CLASS_TAG_NAME, (void*)CEAntisymmetric::restore_from_buffer);
	m_XML_tag_reference->put_ptr(CFLogN::CLASS_TAG_NAME, (void*)CFLogN::restore_from_buffer);
	
	m_XML_tag_reference->put_ptr(CVIDensData::CLASS_TAG_NAME, (void*)CVIDensData::restore_from_buffer);
	m_XML_tag_reference->put_ptr(CVirtInt0::CLASS_TAG_NAME, (void*)CVirtInt0::restore_from_buffer);
	m_XML_tag_reference->put_ptr(CVirtInt1::CLASS_TAG_NAME, (void*)CVirtInt1::restore_from_buffer);
	m_XML_tag_reference->put_ptr(CVirtInt11::CLASS_TAG_NAME, (void*)CVirtInt11::restore_from_buffer);
	m_XML_tag_reference->put_ptr(CVirtIntC::CLASS_TAG_NAME, (void*)CVirtIntC::restore_from_buffer);

	m_XML_tag_reference->put_ptr(CEFunction::CLASS_TAG_NAME, (void*)CEFunction::restore_from_buffer);
	
	m_XML_tag_reference->put_ptr(CQuarkMixMatrix::CLASS_TAG_NAME, (void*)CQuarkMixMatrix::restore_from_buffer);

	m_XML_tag_reference->put_ptr(CDIntLine::CLASS_TAG_NAME, (void*)CDIntLine::restore_from_buffer);
	m_XML_tag_reference->put_ptr(CDExtLine::CLASS_TAG_NAME, (void*)CDExtLine::restore_from_buffer);
	m_XML_tag_reference->put_ptr(CDiagramVertexInfo::CLASS_TAG_NAME, (void*)CDiagramVertexInfo::restore_from_buffer);
	m_XML_tag_reference->put_ptr(CDVertex::CLASS_TAG_NAME, (void*)CDVertex::restore_from_buffer);
	m_XML_tag_reference->put_ptr(CFeynDiagram::CLASS_TAG_NAME, (void*)CFeynDiagram::restore_from_buffer);
	m_XML_tag_reference->put_ptr(CDiagramSet::CLASS_TAG_NAME, (void*)CDiagramSet::restore_from_buffer);
	
	m_XML_tag_reference->put_ptr(CEFuncBondarev::CLASS_TAG_NAME, (void*)CEFuncBondarev::restore_from_buffer);

	m_XML_tag_reference->put_ptr(CENull::CLASS_TAG_NAME, (void*)CENull::restore_from_buffer);
	m_XML_tag_reference->put_ptr(CEInfinity::CLASS_TAG_NAME, (void*)CEInfinity::restore_from_buffer);
	m_XML_tag_reference->put_ptr(CFuncIf::CLASS_TAG_NAME, (void*)CFuncIf::restore_from_buffer);
	m_XML_tag_reference->put_ptr(CEBooleanEquZero::CLASS_TAG_NAME, (void*)CEBooleanEquZero::restore_from_buffer);
	m_XML_tag_reference->put_ptr(CEBooleanAndOr::CLASS_TAG_NAME, (void*)CEBooleanAndOr::restore_from_buffer);
	m_XML_tag_reference->put_ptr(CFuncConjugated::CLASS_TAG_NAME, (void*)CFuncConjugated::restore_from_buffer);
	
	dbg->log(15, "XML-TAG structure initialized", m_XML_tag_reference);
}

long CESupport::fing_item_end_pos(const char *data, const char *item_tag_name, long pos_after_tag)
{
	return CUXMLparcer::find_tag_closing(data, item_tag_name, pos_after_tag, false);
}
