////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// FFortran.cpp: implementation of the CFFortran class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//#include <iostream.h>
//#include <fstream.h>

// environs:
const unsigned CFFortran::ENV_BRACKETS_NEVER	= 0x00;
const unsigned CFFortran::ENV_BRACKETS_FLAG		= 0x01; //need brackets if negative
const unsigned CFFortran::ENV_BRACKETS_SUM		= 0x03;	//need brackets if item is sum
const unsigned CFFortran::ENV_BRACKETS_PROD		= 0x07;	//need brackets if item is product or sum
const unsigned CFFortran::ENV_BRACKETS_SEQ		= 0x0f;	//need brackets if not single
//Note: ENV_NUMBERS_INTEGER flag is currently ignored and used 'always'
const unsigned CFFortran::ENV_NUMBERS_INTEGER	= 0x10; // mean numbers as integer (no 'dot' afterwards)

CSequence* CFFortran::m_IncludeConstsSaved		= NULL;	// set of constants already saved to include file
CEPtrArray* CFFortran::m_IncludeUsedBy			= NULL;	// set of .cpp/.f77 files that use include file

long CFFortran::MAX_LINES_IN_FUNC_DEFAULT = 100;

#define FORTRAN_OBJ_SYMBOLS				0x01
#define FORTRAN_OBJ_2SCALAR_COUPLINGS	0x02
#define FORTRAN_OBJ_4SCALAR_COUPLINGS	0x04
#define FORTRAN_ALL_VECTORS				0x08

#define COMMON_BLOCK_TITLE		"ALHEP1"
#define COMMON_BLOCK_PREFIX		"AL"

#define COMMON_ARRAY_TITLE		"PAR"
#define COMMON_ARRAY_TITLE_C	"par"

#define DEFAULT_INCLUDE_FILENAME_F		"AlConst.fi"
#define DEFAULT_INCLUDE_FILENAME_CPP	"AlConst.h"

#define S_ARR_PARTICLE_ORDER_CPP		"m_PartOrder"

const char* CFFortran::CLASS_NAME = "CFFortran";
CDebug* CFFortran::dbg = new CDebug("CFFortran");
bool CFFortran::m_PAR_CODE_GEN_FAST	= true;//false; //non-approved mode, to be set as default after debug

///////////////////////////////////////////////////////////////////////////// CFuncRegistry
const char* CFFortran::CFuncRegistry::F_STRUCT_F1	= "STRUCTF1";
const char* CFFortran::CFuncRegistry::F_STRUCT_F2	= "STRUCTF2";
const char* CFFortran::CFuncRegistry::F_STRUCT_F3	= "STRUCTF3";
const char* CFFortran::CFuncRegistry::F_STRUCT_F4	= "STRUCTF4";
const char* CFFortran::CFuncRegistry::F_SOFT_BREMS_INT_1	= "SoftBremsInt1";
const char* CFFortran::CFuncRegistry::F_SOFT_BREMS_INT_2	= "SoftBremsInt2";

CFFortran::CFuncRegistry::CFuncRegistry() : CEObject(true) {
	//CFuncRegistryInfo* inf=NULL;
	//Note: parameters of different type are declared like 'inf=new Info(); inf->m_iParTypes.SetAt(i,X); m_Infos.add(inf)'

	//F1-F4
	m_Infos.add(new CFuncRegistryInfo("F1",CEAbstractFunction::PAR_REAL_ABSTRACT,2,"F1",NULL,CCodeProcessImpl::CN_CPP_LIB_CLASS,CEAbstractFunction::PAR_REAL_ABSTRACT));
	m_Infos.add(new CFuncRegistryInfo("F2",CEAbstractFunction::PAR_REAL_ABSTRACT,2,"F2",NULL,CCodeProcessImpl::CN_CPP_LIB_CLASS,CEAbstractFunction::PAR_REAL_ABSTRACT));
	m_Infos.add(new CFuncRegistryInfo("F3",CEAbstractFunction::PAR_REAL_ABSTRACT,2,"F3",NULL,CCodeProcessImpl::CN_CPP_LIB_CLASS,CEAbstractFunction::PAR_REAL_ABSTRACT));
	m_Infos.add(new CFuncRegistryInfo("F4",CEAbstractFunction::PAR_REAL_ABSTRACT,2,"F4",NULL,CCodeProcessImpl::CN_CPP_LIB_CLASS,CEAbstractFunction::PAR_REAL_ABSTRACT));

	//SoftBremsInt1, SoftBremsInt2
	m_Infos.add(new CFuncRegistryInfo(CFuncRegistry::F_SOFT_BREMS_INT_1,CEAbstractFunction::PAR_REAL_ABSTRACT,1,"SoftBremsInt1",NULL,
		CCodeProcessImpl::CN_CPP_LIB_CLASS,CEAbstractFunction::PAR_4VEC_REAL));
	m_Infos.add(new CFuncRegistryInfo(CFuncRegistry::F_SOFT_BREMS_INT_2,CEAbstractFunction::PAR_REAL_ABSTRACT,2,"SoftBremsInt2",NULL,
		CCodeProcessImpl::CN_CPP_LIB_CLASS,CEAbstractFunction::PAR_4VEC_REAL));

	//AsymmProd [C++ only now. TODO: uniform c++ and fXX declarations]
	m_Infos.add(new CFuncRegistryInfo("AsymmProd",CEAbstractFunction::PAR_REAL_ABSTRACT,4,"AsymmProd",NULL,
		CCodeProcessImpl::CN_CPP_LIB_CLASS,CEAbstractFunction::PAR_4VEC_REAL));

	//CAsymmProd [not implemented in C++ now. TODO: uniform c++ and fXX declarations]
	m_Infos.add(new CFuncRegistryInfo("CAsymmProd",CEAbstractFunction::PAR_COMPLEX_ABSTRACT,4,"CAsymmProd",NULL,
		CCodeProcessImpl::CN_CPP_LIB_CLASS,CEAbstractFunction::PAR_4VEC_REAL));
}
void CFFortran::CFuncRegistry::AddFunctionInfo(CFFortran::CFuncRegistry::CFuncRegistryInfo* inf) {
	if(inf!=NULL){
		const char* id = inf->m_sNameAlhep.getData(); int nPars=inf->GetParsNum();
		bool bFound=false;
		for(int i=0; i<m_Infos.size(); i++) {
			CFuncRegistryInfo* infX=(CFuncRegistryInfo*)m_Infos.GetAt(i); if(infX==NULL)continue;
			if((nPars==infX->GetParsNum()) && (CEUtil::equals(infX->m_sNameAlhep.getData(),id))){ bFound=true; break; }
		}
		if(!bFound){ m_Infos.add(inf); }else{ delete inf; }
	}
}
CFFortran::CFuncRegistry::CFuncRegistryInfo* CFFortran::CFuncRegistry::GetInfo(const char* func_id, int nParams) {
	if(func_id==NULL)return NULL;
	for(int i=0; i<m_Infos.size(); i++) {
		CFuncRegistryInfo* inf=(CFuncRegistryInfo*)m_Infos.GetAt(i); if(inf==NULL)continue;
		if((nParams==inf->GetParsNum()) && (CEUtil::equals(inf->m_sNameAlhep.getData(),func_id)))return inf;
	}
	return NULL;
}
int CFFortran::CFuncRegistry::GetReturnType(CEAbstractFunction* func) { //searches both function & registry info
	if(func==NULL)return CEAbstractFunction::PAR_DEFAULT;
	int res=func->GetResultType(); //predefined for some intenal (non CEFunction) functions
	if(res!=CEAbstractFunction::PAR_DEFAULT) return res; //return internal info if available
	CFuncRegistryInfo* inf=GetInfo(func->getTitle(),func->params_num()); if(inf!=NULL)res=inf->m_iReturnType;
	return res;
}
int CFFortran::CFuncRegistry::GetParameterType(CEAbstractFunction* func, int i) { //searches both function & registry info
	if(func==NULL)return CEAbstractFunction::PAR_DEFAULT;
	int res=func->GetParameterType(i); //predefined for some intenal (non CEFunction) functions
	if(res!=CEAbstractFunction::PAR_DEFAULT) return res; //return internal info if available
	CFuncRegistryInfo* inf=GetInfo(func->getTitle(),func->params_num()); if(inf!=NULL)res=inf->m_iParTypes.GetAt(i);
	return res;
}
char* CFFortran::CFuncRegistry::toLine(int mode) {
	CEString sTmp("Function Registry. Size: "); sTmp.add_int(m_Infos.size());
	for(int i=0; i<m_Infos.size(); i++) {
		CEBasic::appendLineBreak(&sTmp,mode);
		CFuncRegistryInfo* inf=(CFuncRegistryInfo*)m_Infos.GetAt(i); if(inf==NULL)continue;
		sTmp.AddString(inf->m_sNameAlhep.getData());
		sTmp.AddString(", c++:"); sTmp.AddString(inf->m_sLibCpp.getData()); sTmp.AddString("::"); sTmp.AddString(inf->m_sNameCpp.getData());
		sTmp.AddString(", F:"); sTmp.AddString(inf->m_sNameFortran.getData());
		sTmp.AddString(", nParams:"); sTmp.add_int(inf->m_iParTypes.GetSize());
	}
	char* res = CEUtil::copyString(sTmp.getData());
	return res;
}
///////////////////////////////////////////////////////////////////////////// CFuncRegistry::CFuncRegistryInfo
CFFortran::CFuncRegistry::CFuncRegistryInfo::CFuncRegistryInfo(const char* sId, int iReturnType, int nParams,
															   const char* sIdCpp, const char* sIdFortran, const char* sLibCpp,
															   int iParamsType
															   ) : CEObject(true),
m_sNameAlhep(sId), m_sNameCpp(sIdCpp), m_sNameFortran(sIdFortran), m_sLibCpp(sLibCpp)
{
	m_iReturnType=iReturnType;
	if(nParams<0)nParams=0; m_iParTypes.AddNItems(nParams,iParamsType);
}

///////////////////////////////////////////////////////////////////////////// CConstRegistry
const int CFFortran::CConstRegistry::C_INTERNAL		= 0;
const int CFFortran::CConstRegistry::C_INCLUDED		= 1;
const int CFFortran::CConstRegistry::C_LOOPTOOLS	= 2;
CFFortran::CConstRegistry::CConstRegistry() : CEObject(true) { m_bInited=false; }
/*
*	Fill basic constants from current model.
*	Uses 'conventions' to determine constants types.
*/
void CFFortran::CConstRegistry::Init(DWORD conventions) {
	if(m_bInited)return;
	CDiagramPhysics* ph=CDiagramPhysics::get_curr_physics();
	CEPtrArray* physConsts = ((ph!=NULL)?(ph->CollectConstantValues()):NULL);
	if(physConsts != NULL){
		for(int i=0; i<physConsts->size(); i++) {
			CExprContainer* cCart = (CExprContainer*)physConsts->GetAt(i);
			CExpression* cName=validate_syms(cCart->get_expr(),NULL); // turn symbols to FORTRAN notation
			// or validate syms via ToFortranString(cCart->get_expr()->toString(mode))?
			if(cName->ClassName() == CESymbol::CLASS_NAME){

				int iTypeConst = CEAbstractFunction::PAR_REAL8;
				//Use abstract type if allowed. E.g. 'pi' must be defined as 'realX'.
				if((conventions & CLFileManager::FT_CPP_TYPE_ABSTRACT)==CLFileManager::FT_CPP_TYPE_ABSTRACT){
					iTypeConst=CEAbstractFunction::PAR_REAL_ABSTRACT;
				}

				CConstInfo* inf = new CConstInfo(((CESymbol*)cName)->get_name(),iTypeConst,CConstRegistry::C_INCLUDED,NULL);
				CExpression* eVal=cCart->get_info_expr();
				if(eVal!=NULL) {
					if(eVal->getProperty(CEProperties::PROP_MASK_IS_NUMERIC_VALUE)){ inf->m_Value=(CEValue*)eVal->clone(); }
					else{ dbg->log(0,"Internal error: NON-NUMERICAL constant value in model",cCart); CEException::halt(); }
				}
				AddConstInfo(inf);
			}else{ dbg->log(0,"Internal error: NON-SYMBOLIC constant in model",cCart); CEException::halt(); }
			if(cName!=cCart->get_expr()){ delete cName; }
		}
		delete physConsts;
	}
	m_bInited=true;
}
void CFFortran::CConstRegistry::AddConstInfo(CFFortran::CConstRegistry::CConstInfo* inf) {
	if(inf==NULL)return;
	CConstInfo* infX=GetInfo(inf->m_sName.getData());
	if(infX==NULL){ m_Infos.add(inf); }
	else{
		if(infX->m_iType!=inf->m_iType){ dbg->log(0,infX->m_iType,inf->m_iType,"Constant variables conflict. Types: %ld and %ld for single variable.",&inf->m_sName); CEException::halt(); }
		if(infX->m_iPath!=inf->m_iPath){ dbg->log(0,infX->m_iPath,inf->m_iPath,"Constant origins conflict. Origins: %ld and %ld for single variable.",&inf->m_sName); CEException::halt(); }
		delete inf;
	}
}
void CFFortran::CConstRegistry::AddConstInfo(const char* sName, int iType, int iPath, const char* sLibCpp){ AddConstInfo(new CConstInfo(sName,iType,iPath,sLibCpp)); }
CFFortran::CConstRegistry::CConstInfo* CFFortran::CConstRegistry::GetInfo(const char* id) {
	if(id==NULL)return NULL;
	for(int i=0; i<m_Infos.size(); i++) {
		CConstInfo* inf=(CConstInfo*)m_Infos.GetAt(i); if(inf==NULL)continue;
		if(CEUtil::equals(inf->m_sName.getData(),id))return inf;
	}
	return NULL;
}
char* CFFortran::CConstRegistry::toLine(int mode) {
	CEString sTmp("Constants Registry. Size: "); sTmp.add_int(m_Infos.size());
	for(int i=0; i<m_Infos.size(); i++) {
		CEBasic::appendLineBreak(&sTmp,mode);
		CConstInfo* inf=(CConstInfo*)m_Infos.GetAt(i); if(inf==NULL)continue;
		if(inf->m_sLibCpp.length()>0){ sTmp.AddString(inf->m_sLibCpp.getData()); sTmp.AddString("::"); }
		sTmp.AddString(inf->m_sName.getData());
		sTmp.AddString(", type:"); sTmp.add_int(inf->m_iType);
		sTmp.AddString(", orig:"); sTmp.add_int(inf->m_iPath);
		if(inf->m_Value!=NULL){ sTmp.AddString(", val: "); sTmp.AddString(inf->m_Value->toString(mode)); }
	}
	char* res = CEUtil::copyString(sTmp.getData());
	return res;

}
CFFortran::CConstRegistry::CConstInfo::CConstInfo(const char* sName, int iType, int iPath, const char* sLibCpp)
: CEObject(true), m_sName(sName), m_sLibCpp(sLibCpp) { m_iType=iType; m_iPath=iPath; m_Value=NULL; }
CFFortran::CConstRegistry::CConstInfo::~CConstInfo() { if(m_Value!=NULL){ delete m_Value; m_Value=NULL; } }

///////////////////////////////////////////////////////////////////////////// CFortranFunctionInfo
CFortranFunctionInfo::CFortranFunctionInfo(const char* _func_name,
										   CEString* _res_value,
										   int iResType,
										   CEPtrArray* _par_list,
										   CEPtrArray* _consts_list,
										   CEPtrArray* _precalc_list,
										   CEPtrArray* _funct_used_list,
										   CEPtrArray* _temp_var_list,
										   DWORD _conventions) : CEObject(true)
{
	CFFortran::dbg->log(0, "[Internal WARNING] CFortranFunctionInfo(unexpected constructor call)");
	if (_func_name!=NULL){ m_func_name = CEUtil::copyString(_func_name); }else{ m_func_name = NULL; }
	m_res_value=_res_value; m_iResType=iResType;
	m_par_list=_par_list; if(m_par_list==NULL)m_par_list=new CEPtrArray();
	m_consts_list=_consts_list; if (m_consts_list==NULL)m_consts_list=new CEPtrArray();
	m_precalc_list = _precalc_list;
	m_funct_used_list = _funct_used_list;
	m_temp_var_list = _temp_var_list;
	m_conventions = _conventions;
}

CFortranFunctionInfo::CFortranFunctionInfo(DWORD _conventions) : CEObject(true) {
	m_func_name=NULL; m_res_value=NULL; m_iResType=CEAbstractFunction::PAR_DEFAULT;
	m_par_list=new CEPtrArray();
	m_consts_list=new CEPtrArray();
	m_precalc_list=NULL;
	m_funct_used_list=NULL;
	m_temp_var_list=NULL;
	m_conventions=_conventions;
}

CFortranFunctionInfo::CFortranFunctionInfo(const char* _func_name,
										   DWORD _conventions) : CEObject(true) {
	if(_func_name!=NULL){ m_func_name=CEUtil::copyString(_func_name); }else{ m_func_name=NULL; }
	m_res_value=NULL; m_iResType=CEAbstractFunction::PAR_DEFAULT;
	m_par_list=new CEPtrArray();
	m_consts_list=new CEPtrArray();
	m_precalc_list=NULL;
	m_funct_used_list=NULL;
	m_temp_var_list=NULL;
	m_conventions=_conventions;
}

CFortranFunctionInfo::~CFortranFunctionInfo() {
	if (m_func_name != NULL) CMemoryManager::mfree(m_func_name);
	if (m_par_list != NULL) delete m_par_list;
	if (m_consts_list != NULL) delete m_consts_list;
	if (m_precalc_list != NULL) delete m_precalc_list;
	if (m_funct_used_list != NULL) delete m_funct_used_list;
	if (m_temp_var_list != NULL) delete m_temp_var_list;
}

const char* CFortranFunctionInfo::CLASS_NAME = "CFortranFunctionInfo";
const char* CFortranFunctionInfo::ClassName() const { return CLASS_NAME; }
bool CFortranFunctionInfo::contain_vector(void * obj) {
	if ((m_func_name == obj) || (m_par_list == obj) || (m_consts_list == obj) || (m_precalc_list == obj) ||
		(m_funct_used_list == obj) || (m_temp_var_list == obj)) return true;
	if (m_par_list != NULL) { if (m_par_list->contain_vector(obj)) return true; }
	if (m_consts_list != NULL) { if (m_consts_list->contain_vector(obj)) return true; }
	if (m_precalc_list!= NULL) { if (m_precalc_list->contain_vector(obj)) return true; }
	if (m_funct_used_list != NULL) { if (m_funct_used_list->contain_vector(obj)) return true; }
	if (m_temp_var_list != NULL) { if (m_temp_var_list->contain_vector(obj)) return true; }
	return false;
}
void CFortranFunctionInfo::init_pre_calc_list() {
	if (m_precalc_list == NULL) m_precalc_list = new CEPtrArray();
}

void CFortranFunctionInfo::set_res_value(CEString* res, int iResType) {
	if (m_res_value != res) {
		if (m_res_value != NULL) { CFFortran::dbg->log(0,"WARNING (CFortranFunctionInfo): previous res_value deleted!"); delete m_res_value; }
		m_res_value=res; m_iResType=iResType;
	}
}

CEString* CFortranFunctionInfo::extract_res_value(){ CEString* res=m_res_value; m_res_value=NULL; return res; }
CEPtrArray* CFortranFunctionInfo::extract_precalc_list(){ CEPtrArray* res=m_precalc_list; m_precalc_list=NULL; return res; }
CEPtrArray* CFortranFunctionInfo::extract_funct_used_list(){ CEPtrArray* res=m_funct_used_list; m_funct_used_list=NULL; return res; }
CEPtrArray* CFortranFunctionInfo::extract_temp_var_list(){ CEPtrArray* res=m_temp_var_list; m_temp_var_list=NULL; return res; }

void CFortranFunctionInfo::set_func_name(const char* _func_name) {
	if (m_func_name != _func_name) {
		if (m_func_name != NULL) CMemoryManager::mfree(m_func_name);
		if (_func_name != NULL) m_func_name = CEUtil::copyString(_func_name);
		else m_func_name = NULL;
	}
}

void CFortranFunctionInfo::add_to_precalc_list(CEObject* obj) {
	if (m_precalc_list == NULL) m_precalc_list = new CEPtrArray();
	m_precalc_list->add(obj);
}

// deletes object
void CFortranFunctionInfo::add_to_func_used_list(int iType, CEString* obj, bool check_previous) {
	if(m_funct_used_list==NULL)m_funct_used_list=new CEPtrArray();
	if (obj == NULL) return;

	// check for existance
	if (check_previous) {
		long s = m_funct_used_list->size();
		for (long i=0; i<s; i++) {
			CEObjIdCart* cart = (CEObjIdCart*)m_funct_used_list->GetAt(i);
			if (obj->equals((CEString*)cart->GetObj())) {
				if(iType != cart->GetId()){ CFFortran::dbg->log(0,iType,cart->GetId(),"ERROR: function names conflict. Different types (%ld,%ld) for single function name.",obj); CEException::halt(); }
				delete obj; return;
			}
		}
	}

	m_funct_used_list->add(new CEObjIdCart(obj,iType));
}

void CFortranFunctionInfo::add_to_temp_var_list(CEString* sym, int iType) {
	if(m_temp_var_list==NULL)m_temp_var_list=new CEPtrArray();
	if(sym==NULL)return;

	m_temp_var_list->add(new CEObjIdCart(sym,iType));
}

void CFortranFunctionInfo::merge_with_other(CFortranFunctionInfo* fi) {
	// other is deleted

	CFFortran::merge_list_with_other(m_par_list, fi->par_list());
	CFFortran::merge_list_with_other(m_consts_list, fi->consts_list());

	CEPtrArray* o_pcl = fi->extract_precalc_list();
	if (o_pcl != NULL) {
		if (m_precalc_list == NULL) m_precalc_list = o_pcl;
		else m_precalc_list->add_array(o_pcl);
	}

	CEPtrArray* o_ful = fi->extract_funct_used_list();
	if (o_ful != NULL) {
		if (m_funct_used_list == NULL) m_funct_used_list = o_ful;
		else {
			// check for similar functions existance
			long s = o_ful->size();
			for(long i=0; i<s; i++) {
				CEObjIdCart* cart=(CEObjIdCart*)o_ful->GetAt(i);
				add_to_func_used_list(cart->GetId(),((CEString*)(cart->GetObj()))->clone(), true);
			}
		}
	}

	CEPtrArray* o_tvl=fi->extract_temp_var_list();
	if(o_tvl!=NULL){
		if(m_temp_var_list==NULL){ m_temp_var_list=o_tvl; }else{ m_temp_var_list->add_array(o_tvl); }
	}

	delete fi;
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFFortran::CFFortran(const char* file_name, DWORD conventions, bool bAppend, CEPtrArray* arrPreIncludes)
: CEObject(true) {
	dbg->log(9,"new code library for file: ",file_name);
	// Settings init
	// Note: 'FT_CPP_TYPE_ABSTRACT' flag is parsed correctly in 'CCodeLanguage::CreateLanguage()'
	m_codeLang = CCodeLanguage::CreateLanguage(conventions); if(m_codeLang==NULL)CEException::halt(dbg,"Can't create CODE object");
	if(file_name==NULL)file_name="default.output";
	m_file_name = CEUtil::copyString(file_name);
	m_header_file_name = NULL;
	m_COMMONBLOCK_NAME = NULL; m_COMMON_ARRAY_TITLE = NULL; m_STORE_EXPR_AS_FLAG=STORE_FORTRAN_SYMBOL;
	m_CppClassName=NULL;
	m_CommonBlockSyms=NULL; m_CommonBlockSyms=0;
	m_conventions = conventions;
	m_general_conventions = conventions;
	m_bAllowConstantsPowers=true;

	m_TmpBuf1kb=(char*)CMemoryManager::mmalloc(1024);

	// include file info
	m_sIncludeFileName.Reset();
	m_IncludedConstantsList.clear_all(); m_bNeedSaveIncludeFile=false;

	m_sub_functions = NULL;
	m_last_temp_num = 0;

	m_string_terminate_pos = 80;

	m_num_strings_in_sum = (MAX_LINES_IN_FUNC_DEFAULT > 3)?MAX_LINES_IN_FUNC_DEFAULT:100000;
	if (conventions & CLFileManager::FT_DONOT_SPLIT_FUNC) m_num_strings_in_sum = 100000;

	// temp variable mane
	m_TEMP_str.Reset();
	if(m_TEMP_str_Predefined != NULL){ m_TEMP_str.AddString(m_TEMP_str_Predefined->getData()); }else{
		if(m_general_conventions & CLFileManager::FT_CPP){ m_TEMP_str.AddString("tmp"); }
		else if((m_general_conventions&CLFileManager::FT_F77) || (m_general_conventions&CLFileManager::FT_F90)){ m_TEMP_str.AddString("TEMP"); }
		else{ m_TEMP_str.AddString("temp"); }
	}
	//**************************************************************** initilization is ended here

	if (m_general_conventions & CLFileManager::FT_CPP) {
		//Compose header file name
		int l=sprintf(m_TmpBuf1kb,"%s",m_file_name);
		int iDotLast=-1; for(;;){int i=CEUtil::findSymbol(m_TmpBuf1kb,'.',((iDotLast>=0)?(iDotLast+1):0)); if(i>=0){ iDotLast=i; }else break; }
		if(iDotLast<0){ *(m_TmpBuf1kb+l)='.'; iDotLast=l; }
		*(m_TmpBuf1kb+iDotLast+1)='h'; *(m_TmpBuf1kb+iDotLast+2)='\0';
		m_header_file_name=CEUtil::copyString(m_TmpBuf1kb);

		//Start header file
		ofstream ofs(m_header_file_name, (bAppend?(ios::out|ios::app):(ios::out|ios::trunc)));
		if(!ofs){ printf("<Error creating output HEADER file.>\n"); return; }
		if(!bAppend){
			m_codeLang->StartCommentLine(&ofs); ofs << "**********************************************************\n";
			m_codeLang->StartCommentLine(&ofs); ofs << " File name: " << m_header_file_name << "\n";
			m_codeLang->StartCommentLine(&ofs); ofs << " Interface to '" << m_file_name << "' file.\n";
			CEString* sVer=new CEString(); CESupport::CreateVersionString(sVer); m_codeLang->StartCommentLine(&ofs); ofs<<" Created by "<<sVer->getData()<<".\n"; delete sVer;

			if(arrPreIncludes != NULL) {
				int cnt=0;
				for(int i=0; i<arrPreIncludes->size(); i++) {
					if(i==0){
						m_codeLang->StartCommentLine(&ofs); ofs << " Pre-includes required: ";
						if(arrPreIncludes->size()>3){ ofs << "\n"; m_codeLang->StartCommentLine(&ofs); ofs<<"    "; }
					}
					CEObject* o=arrPreIncludes->GetAt(i); if(o==NULL)continue; if(o->GetObjectKind() != CEObject::KIND_STRING)continue;
					if(cnt>0){ ofs<<", "; } ofs<<((CEString*)o)->getData();
					cnt++; if(cnt>5){ cnt=0; ofs << "\n"; m_codeLang->StartCommentLine(&ofs); ofs<<"    "; }
				}
			}
		}
		//ofs << "\n";
		ofs.close();
	}

	ofstream ofs(m_file_name, (bAppend?(ios::out|ios::app):(ios::out|ios::trunc)));
	if (!ofs) { dbg->log(0,"<Error creating output CODE file.>"); return; }
	if(!bAppend){
		m_codeLang->StartCommentLine(&ofs); ofs << "**********************************************************\n";
		m_codeLang->StartCommentLine(&ofs); ofs << " File name: " << m_file_name << "\n";
		CEString* sVer=new CEString(); CESupport::CreateVersionString(sVer); m_codeLang->StartCommentLine(&ofs); ofs << " Created by "<<sVer->getData()<<".\n"; delete sVer;

		CEString sDir;
		if(CVMFileUtil::GetCurrentPath(sDir)) {
			m_codeLang->StartCommentLine(&ofs); ofs << " Original location:\n";
			m_codeLang->StartCommentLine(&ofs); ofs << " " << sDir.getData() << "\\" << m_file_name << "\n";
		}

		//m_codeLang->StartCommentLine(&ofs); ofs << "**********************************************************\n";
		//Note: include 'math.h' before 'vmlib.h' to allow inline math-functions declarations in vmlib
		if(m_general_conventions&CLFileManager::FT_CPP){ ofs<<"#include <math.h>\n"; }//for pow, sqrt etc.
		if(m_general_conventions&CLFileManager::FT_CPP){
			ofs<<"#include \""<<CCodeProcessImpl::FN_CPP_LIB_PATH<<CCodeProcessImpl::FN_CPP_LIB_FILE<<"\"\n"; //#include "vmlib.h"
		}

		if((m_general_conventions&CLFileManager::FT_CPP) && (arrPreIncludes != NULL)) {
			for(int i=0; i<arrPreIncludes->size(); i++) {
				CEObject* o=arrPreIncludes->GetAt(i); if(o==NULL)continue;
				if(o->GetObjectKind() == CEObject::KIND_STRING) { ofs<<"#include \""<<((CEString*)o)->getData()<<"\"\n"; }
			}
		}

		if((m_general_conventions&CLFileManager::FT_CPP) && (m_header_file_name!=NULL)){ ofs<<"#include \""<<m_header_file_name<<"\"\n"; }

		if(m_general_conventions&CLFileManager::FT_CPP){
			//defines
			ofs << "\n";
			m_codeLang->StartCommentLine(&ofs); ofs << " Power calculation function: pow() or "<<CCodeProcessImpl::CN_CPP_LIB_CLASS<<"::Pow()\n";
			ofs<<"#define "<<CCodeLanguage::CPP_POW_DEF_SYM<<"  pow\n";
			ofs<<"//#define "<<CCodeLanguage::CPP_POW_DEF_SYM<<"  "<<CCodeProcessImpl::CN_CPP_LIB_CLASS<<"::Pow\n";
		}
	}
	//ofs<<"\n";
	ofs.close();

	m_vec_couplings = NULL; m_syms_replaced=NULL;
	m_Denominators = NULL; m_Dens_functions=NULL;
	//[unused]m_ME2ToDiffCSFunc=NULL;
}

CFFortran::~CFFortran()
{
	if(m_bNeedSaveIncludeFile){
		SaveIncludeConstantsFile();
	}

	//
	ofstream ofs(m_file_name, ios::out|ios::app);
	if (!ofs) {dbg->log(0,"<Error opening output CODE file.>"); return;}
	m_codeLang->StartCommentLine(&ofs); ofs << "**********************************************************\n";
	//m_codeLang->StartCommentLine(&ofs); ofs << " " << m_file_name << "\n";
	//m_codeLang->StartCommentLine(&ofs); ofs << "**********************************************************\n";
	ofs.close();

	CMemoryManager::mfree(m_file_name);
	if(m_header_file_name!=NULL){ CMemoryManager::mfree(m_header_file_name); m_header_file_name=NULL; }
	if (m_COMMONBLOCK_NAME != NULL) { CMemoryManager::mfree(m_COMMONBLOCK_NAME); m_COMMONBLOCK_NAME=NULL; }
	if (m_CppClassName != NULL) { CMemoryManager::mfree(m_CppClassName); m_CppClassName=NULL; }
	if (m_COMMON_ARRAY_TITLE != NULL) { m_COMMON_ARRAY_TITLE=NULL; }
	if (m_CommonBlockSyms != NULL) { delete m_CommonBlockSyms; m_CommonBlockSyms=NULL; }
	if (m_sub_functions != NULL) delete m_sub_functions;
	if (m_vec_couplings != NULL) delete m_vec_couplings;
	if (m_syms_replaced != NULL) delete m_syms_replaced;
	if (m_Denominators != NULL) delete m_Denominators;
	if (m_Dens_functions != NULL) delete m_Dens_functions;
	if (m_TmpBuf1kb!=NULL)CMemoryManager::mfree(m_TmpBuf1kb);
	if (m_codeLang != NULL) delete m_codeLang;
}

const char* CFFortran::ClassName() const { return CLASS_NAME; }
bool CFFortran::contain_vector(void * obj) {
	if ((m_file_name == obj) || (m_sub_functions == obj)) return true;
	if (m_header_file_name == obj) return true;
	if ((m_COMMONBLOCK_NAME == obj) || (m_CommonBlockSyms == obj)) return true;
	if (m_CppClassName == obj) return true;
	if (m_COMMON_ARRAY_TITLE == obj) return true;
	if (m_vec_couplings == obj) return true;
	if (m_syms_replaced == obj) return true;
	if ((m_Denominators == obj) || (m_Dens_functions == obj)) return true;
	if (m_sub_functions != NULL) { if (m_sub_functions->contain_vector(obj)) return true; }
	if (m_CommonBlockSyms != NULL) { if (m_CommonBlockSyms->contain_vector(obj)) return true; }
	if (m_vec_couplings != NULL) { if (m_vec_couplings->contain_vector(obj)) return true; }
	if (m_syms_replaced != NULL) { if (m_syms_replaced->contain_vector(obj)) return true; }
	if (m_Denominators != NULL) { if (m_Denominators->contain_vector(obj)) return true; }
	if (m_Dens_functions != NULL) { if (m_Dens_functions->contain_vector(obj)) return true; }
	return false;
}

char* CFFortran::toLine(int mode) {
	CEString* str = new CEString("{"); str->AddString(m_file_name); str->AddString("}");
	char* res = CEUtil::copyString(str->getData());
	delete str;
	return res;
}

CEPtrArray* CFFortran::m_InternalFunctionsList	= NULL; // DLOG, LOG, DSIN etc.
CEPtrArray* CFFortran::m_LoopToolsFunctionsList	= NULL;
CEString* CFFortran::m_TEMP_str_Predefined		= NULL;
bool CFFortran::m_bInited = false;

void CFFortran::SetTEMPstr(const char* _str) {
	if (_str==NULL) return;
	if (m_TEMP_str_Predefined != NULL) delete m_TEMP_str_Predefined;
	m_TEMP_str_Predefined=new CEString(_str);
}

/*
*	Called once for ALL files & conventions
*
*	TODO: make some non-static, language-specific initialization (e.g. for m_InternalFunctionsList)
*/
void CFFortran::init() {
	if(m_bInited)return;
	// Allow m_TEMP_str_Predefined to be NULL ==> language-specific choise
	//Old: if (m_TEMP_str_Predefined==NULL) m_TEMP_str_Predefined=new CEString("tmp");

	if (m_InternalFunctionsList == NULL) {
		m_InternalFunctionsList = new CEPtrArray();

		// Fortran-specific
		m_InternalFunctionsList->add(new CEString("DLOG"));
		m_InternalFunctionsList->add(new CEString("LOG"));
		m_InternalFunctionsList->add(new CEString("DSIN"));
		m_InternalFunctionsList->add(new CEString("SIN"));
		m_InternalFunctionsList->add(new CEString("DCOS"));
		m_InternalFunctionsList->add(new CEString("COS"));
		m_InternalFunctionsList->add(new CEString("DSQRT"));
		m_InternalFunctionsList->add(new CEString("SQRT"));
		m_InternalFunctionsList->add(new CEString("DSQR"));
		m_InternalFunctionsList->add(new CEString("SQR"));

		// TODO: separate Fortran & C++ internal function lists
		// Or make init() non-static function and fill local lists according to 'm_general_conventions'
	}

	if (m_LoopToolsFunctionsList == NULL) {
		m_LoopToolsFunctionsList = new CEPtrArray();
		m_LoopToolsFunctionsList->add(new CEString("A0")); m_LoopToolsFunctionsList->add(new CEString("CA0"));
		m_LoopToolsFunctionsList->add(new CEString("B0i")); m_LoopToolsFunctionsList->add(new CEString("CB0i"));

		m_LoopToolsFunctionsList->add(new CEString("B0")); m_LoopToolsFunctionsList->add(new CEString("B1"));
		m_LoopToolsFunctionsList->add(new CEString("B00")); m_LoopToolsFunctionsList->add(new CEString("B11"));
		m_LoopToolsFunctionsList->add(new CEString("B001")); m_LoopToolsFunctionsList->add(new CEString("B111"));

		m_LoopToolsFunctionsList->add(new CEString("CB0")); m_LoopToolsFunctionsList->add(new CEString("CB1"));
		m_LoopToolsFunctionsList->add(new CEString("CB00")); m_LoopToolsFunctionsList->add(new CEString("CB11"));
		m_LoopToolsFunctionsList->add(new CEString("CB001")); m_LoopToolsFunctionsList->add(new CEString("CB111"));

		m_LoopToolsFunctionsList->add(new CEString("DB0")); m_LoopToolsFunctionsList->add(new CEString("DB1"));
		m_LoopToolsFunctionsList->add(new CEString("DB00")); m_LoopToolsFunctionsList->add(new CEString("DB11"));
		
		m_LoopToolsFunctionsList->add(new CEString("CDB0")); m_LoopToolsFunctionsList->add(new CEString("CDB1"));
		m_LoopToolsFunctionsList->add(new CEString("CDB00")); m_LoopToolsFunctionsList->add(new CEString("CDB11"));

		m_LoopToolsFunctionsList->add(new CEString("C0")); m_LoopToolsFunctionsList->add(new CEString("CC0"));
		m_LoopToolsFunctionsList->add(new CEString("C0i")); m_LoopToolsFunctionsList->add(new CEString("CC0i"));

		m_LoopToolsFunctionsList->add(new CEString("D0")); m_LoopToolsFunctionsList->add(new CEString("CD0"));
		m_LoopToolsFunctionsList->add(new CEString("D0i")); m_LoopToolsFunctionsList->add(new CEString("CD0i"));

		m_LoopToolsFunctionsList->add(new CEString("E0")); m_LoopToolsFunctionsList->add(new CEString("CE0"));
		m_LoopToolsFunctionsList->add(new CEString("E0i")); m_LoopToolsFunctionsList->add(new CEString("CE0i"));

		m_LoopToolsFunctionsList->add(new CEString("Bget")); m_LoopToolsFunctionsList->add(new CEString("CBget"));
		m_LoopToolsFunctionsList->add(new CEString("Cget")); m_LoopToolsFunctionsList->add(new CEString("CCget"));
		m_LoopToolsFunctionsList->add(new CEString("Dget")); m_LoopToolsFunctionsList->add(new CEString("CDget"));
		m_LoopToolsFunctionsList->add(new CEString("Eget")); m_LoopToolsFunctionsList->add(new CEString("CEget"));

		m_LoopToolsFunctionsList->add(new CEString("getmudim")); m_LoopToolsFunctionsList->add(new CEString("getdelta"));
		m_LoopToolsFunctionsList->add(new CEString("getlambda")); m_LoopToolsFunctionsList->add(new CEString("getmaxdev"));

		m_LoopToolsFunctionsList->add(new CEString("getwarndigits")); m_LoopToolsFunctionsList->add(new CEString("geterrdigits"));
		m_LoopToolsFunctionsList->add(new CEString("getversionkey")); m_LoopToolsFunctionsList->add(new CEString("getdebugkey"));
		m_LoopToolsFunctionsList->add(new CEString("setcachelast")); m_LoopToolsFunctionsList->add(new CEString("getcachelast"));
	}

	m_bInited=true;
}

void CFFortran::finalize() {
	m_bInited=false;
	if (m_InternalFunctionsList != NULL) { delete m_InternalFunctionsList; m_InternalFunctionsList = NULL; }
	if (m_LoopToolsFunctionsList != NULL) { delete m_LoopToolsFunctionsList; m_LoopToolsFunctionsList=NULL; }
	if (m_TEMP_str_Predefined!=NULL) { delete m_TEMP_str_Predefined; m_TEMP_str_Predefined=NULL; }
	if (m_IncludeConstsSaved!=NULL) { delete m_IncludeConstsSaved; m_IncludeConstsSaved=NULL; }
	if (m_IncludeUsedBy!=NULL) { delete m_IncludeUsedBy; m_IncludeUsedBy=NULL; }
}

bool CFFortran::isFuncInternal(CEString* func_name) {
	if(!m_bInited)init();
	if (m_InternalFunctionsList == NULL) return false;
	if (func_name == NULL) return false;

	long s = m_InternalFunctionsList->size(); long i;
	for(i=0; i<s; i++){ if (func_name->equals((CEString*)m_InternalFunctionsList->GetAt(i))) return true; }

	//asdf
	//TODO: adjust this function
	//may cause F77-errors <== used for 'used_functions_list' creation
	//'m_LoopToolsFunctionsList' should be deleted?

	//
	if ((m_conventions & CLFileManager::FT_LOOPTOOLS) && (m_LoopToolsFunctionsList!=NULL)) {
		s = m_LoopToolsFunctionsList->size();
		for(i=0; i<s; i++){ if (func_name->equals((CEString*)m_LoopToolsFunctionsList->GetAt(i))) return true; }
	}

	//TODO: keep internal functions in 'm_fRegistry' instead of 'm_InternalFunctionsList'
	/*CFuncRegistry::CFuncRegistryInfo* inf=m_fRegistry.GetInfo(func_name->getData(),func->params_num());
	if(inf!=NULL) {
		//TODO: check if implementation is available for current language?
		return true;
	}*/

	return false;
}

/*
*	Note: may be (*iObjType != (*pConstInfo)->m_iType) for powers of constants.
*/
bool CFFortran::isSymbolConstant(CExpression* sym, CConstRegistry::CConstInfo** pConstInfo, long* iObjType) { // search by native value
	bool delete_sym=false; if((!sym->getFlag())||(!sym->getLevel())){ sym=sym->clone(); sym->setFlag(); sym->setLevel(true); delete_sym=true; }

	bool bRes = (CFFortran::GetBasicConstSym(sym,NULL,pConstInfo,iObjType)!=NULL);

	if(delete_sym)delete sym;
	return bRes;
}

CEValue* CFFortran::getConstantValue(CExpression* sym) { // non-cloned result, search by FORTRAN value
	if(sym->ClassName() != CESymbol::CLASS_NAME)return NULL;
	CConstRegistry::CConstInfo* inf =m_constRegistry.GetInfo(((CESymbol*)sym)->get_name());
	if(inf==NULL)return NULL;
	return inf->m_Value;
}
// Input: always +signed value, pow may be NULL
// Result: NULL if not constant, ==_SymOrPower if symbol, non-cloned symbol if power
// iObjType: type of '_SymOrPower'. May be different to original constsnt type form 'pConstInfo'.
CExpression* CFFortran::GetBasicConstSym(CExpression* _SymOrPower, long* pow, CConstRegistry::CConstInfo** pConstInfo, long* iObjType) {
	if(_SymOrPower==NULL)return NULL;

	// Search original constants list
	if(_SymOrPower->ClassName() == CESymbol::CLASS_NAME) {
		CConstRegistry::CConstInfo* inf =m_constRegistry.GetInfo(((CESymbol*)_SymOrPower)->get_name());
		if(inf!=NULL){
			if(pow!=NULL)*pow=1; if(pConstInfo!=NULL)*pConstInfo=inf; if(iObjType!=NULL)*iObjType=inf->m_iType;
			return _SymOrPower;
		}
		//no return otherwise
	}

	/*
	*	Parameter 'bAllowPowerTypeOriginal'.
	*	Currently MAY be 'true': [par^x] is extracted from the whole expression and is same throughout the project.
	*	Elsewhere errors are possible from "(realX)(eP2)-(realX)(e)*(realX)(e)".
	*/
	bool bAllowPowerTypeOriginal=false;

	// Search constants-powers list
	long s=m_ConstantsPowersList.size();
	for(long i=0; i<s; i++) {
		CSequence* rec=(CSequence*)m_ConstantsPowersList.GetAt(i); if(rec->size()!=3){ dbg->log(0,"ERROR: invalid m_ConstantsPowersList structure"); continue; }
		if(_SymOrPower->equals(rec->GetAt(2))) {
			if(pow!=NULL) {
				if(rec->GetAt(1)->ClassName()!=CENumber::CLASS_NAME){ dbg->log(0,"ERROR: invalid m_ConstantsPowersList structure. Number expected."); CEException::halt(); }
				*pow=((CENumber*)rec->GetAt(1))->getLong();
			}
			CExpression* res =rec->GetAt(0);
			if(((pConstInfo!=NULL)||(iObjType!=NULL)) && (res->ClassName()==CESymbol::CLASS_NAME)){//also search for constant info
				CConstRegistry::CConstInfo* inf=m_constRegistry.GetInfo(((CESymbol*)res)->get_name());
				if(pConstInfo!=NULL)*pConstInfo=inf;
				if(iObjType!=NULL){
					long iType=((inf!=NULL)?inf->m_iType:0);
					if(!bAllowPowerTypeOriginal){
						if(iType==CEAbstractFunction::PAR_DEFAULT)iType=m_codeLang->GetDefaultTypeId(m_conventions);
						iType=m_codeLang->GetOperatingType(iType);
					}
					*iObjType=iType;
				}
			}
			return res;
		}
	}
	return NULL; //not a constant
}

void CFFortran::AddToCommonBlock(CExpression* sym, bool allow_flags) {
	if (m_CommonBlockSyms == NULL) { m_CommonBlockSyms=new CSequence(); m_CommonArraySize=0; }
	if (sym == NULL) return;
	bool del_src = false;
	if (allow_flags) {
		// uniform parameter
		if ((!sym->getFlag()) || (!sym->getLevel())) { sym=sym->clone(); sym->setFlag(); sym->setLevel(true); del_src=true; }
	} else {
		if (!sym->getFlag()) { CEException::halt("ERROR: Signed item is not allowed in CFFortran::AddToCommonBlock"); }
		if (!sym->getLevel()) { CEException::halt("ERROR: Denominator item is not allowed in CFFortran::AddToCommonBlock"); }
	}
	if (FindInCommonBlock(sym) >= 0) { if(del_src){delete sym;} return; }
	if (!del_src) sym=sym->clone();
	m_CommonBlockSyms->addItem(sym->clone());
}

void CFFortran::DeclareCommonArray() {
	if (m_CommonBlockSyms == NULL) { m_CommonBlockSyms=new CSequence(); m_CommonArraySize=0; return; }
	m_CommonArraySize = m_CommonBlockSyms->size();
}

long CFFortran::FindInCommonBlock(CExpression* sym, bool allow_flags) {
	if ((m_CommonBlockSyms == NULL) || (sym == NULL)) return -1;
	bool del_src = false;
	if (allow_flags) {
		// uniform parameter
		if ((!sym->getFlag()) || (!sym->getLevel())) { sym=sym->clone(); sym->setFlag(); sym->setLevel(true); del_src=true; }
	} else {
		if (!sym->getFlag()) { CEException::halt("ERROR: Search for signed item is not allowed in CFFortran::FindInCommonBlock"); }
		if (!sym->getLevel()) { CEException::halt("ERROR: Search for denominator item is not allowed in CFFortran::FindInCommonBlock"); }
	}
	long s=m_CommonBlockSyms->size();
	long res = -1;
	for(long i=0; i<s; i++) { if(sym->equals(m_CommonBlockSyms->GetAt(i))) { res=i; break; } }
	if (del_src) delete sym;
	return res;
}

void CFFortran::SortCommonBlockItems() { // sort only after m_CommonArraySize position
	if (m_CommonBlockSyms == NULL) return;
	long s = m_CommonBlockSyms->size();
	for (long i=m_CommonArraySize+1; i<s; i++) {
		CExpression* item_i = m_CommonBlockSyms->GetAt(i);
		bool loop = true;
		for (long j=i-1; ((j>=m_CommonArraySize) && (loop)); j--) { // check right order first
			CExpression* item_j = m_CommonBlockSyms->GetAt(j);
			if (!item_i->less_than(item_j)) { // previous pos is right
				if (i > (j+1)) m_CommonBlockSyms->move(i, j+1);
				loop = false;
			}
		}
		if (loop) { // a is less than all previous
			m_CommonBlockSyms->move(i, m_CommonArraySize);
		}
	}
}

CExpression* CFFortran::GetCommonSym(int i) {
	if ((i<0) || (i>=m_CommonBlockSyms->size())) return NULL;
	return m_CommonBlockSyms->GetAt(i);
}
void CFFortran::AddParticleOrderList(ofstream& ofs) { //"p1, p2, k1, k2..."
	CKinematics* kin=CEProcessor::kinematics(); CDiagramPhysics* phys=CDiagramPhysics::get_curr_physics();
	if(kin!=NULL) {
		int nInit=kin->num_init_particles(); int n=nInit+kin->num_final_particles();
		for(int i=0;i<n;i++){
			CEParticle* p=((i<nInit)?(kin->pin(i)):(kin->pfin(i-nInit))); if(i>0)ofs<<", ";
			if(p!=NULL){
				CEString* sTmp=p->get_momentum_symbol()->toString(); sTmp=((sTmp!=NULL)?toFortranSyms(sTmp):new CEString("NULL"));
				bool bIsQuark = p->check_PK(CEParticle::PARTICLE_QUARK,true); // use special notation for antiquarks
				if(phys!=NULL){
					if((p->get_PK()<0)&&(!bIsQuark))ofs<<"-";
					ofs<<phys->get_particle_symbol(p->get_PK(),CDiagramPhysics::SYMBOL_STYLE_TEX);
					if((p->get_PK()<0)&&(bIsQuark))ofs<<"Bar";
					ofs<<"(";
				}
				ofs<<sTmp->getData(); if(phys!=NULL){ ofs<<")"; } delete sTmp;
			}else{ ofs<<"Unknown"; }
		}
	} else { ofs<<"No kinematics information available"; }
}
void CFFortran::AddSetPartOrderFunction(ofstream& ofs, bool bHeaderOnly) { //"void SetPartOrder(int id_p1, int id_p2...)"
	if((m_general_conventions & CLFileManager::FT_CPP)==0)return;
	CKinematics* kin=CEProcessor::kinematics(); CDiagramPhysics* phys=CDiagramPhysics::get_curr_physics();
	if(kin==NULL) { m_codeLang->StartCommentLine(&ofs); ofs<<"ERROR saving SetPartOrder() function. No kinematics available."<<"\n"; return; }
	CEString ln; CEPtrArray sPars;
	if(bHeaderOnly && (m_CppClassName!=NULL) && ((m_conventions & CLFileManager::FT_NON_STATIC_IMPL)==0)){ ln<<"static "; }
	ln<<"void ";
	if((!bHeaderOnly) && (m_CppClassName!=NULL)){ ln<<m_CppClassName<<"::"; } ln<<"SetPartOrder(";
	int i; int nIn=kin->num_init_particles(); int nFin=kin->num_final_particles();
	for(i=0;i<(nIn+nFin);i++){
		if(i>0)ln<<", "; ln<<"int ";
		bool bIn=(i<nIn); CEParticle* p=(bIn?kin->pin(i):kin->pfin(i-nIn));
		//Particle name may be same for 2 particles now ==> use also id-number or momentum
		CEString* sSym=new CEString(""); *sSym<<(bIn?"i":"f")<<(bIn?(i+1):(i+1-nIn));
		if((p!=NULL) && (phys!=NULL)){ // Compose parameter
			*sSym<<"_"<<phys->get_particle_symbol(p->get_PK(),CDiagramPhysics::SYMBOL_STYLE_TEX);
			if(p->get_PK()<0)*sSym<<"Bar";
			//CEString* sPartMom=p->get_momentum_symbol()->toString(); *sSym<<"_"<<sPartMom;
			sSym=toFortranSyms(sSym);
		}
		ln<<sSym->getData(); sPars.add(sSym);
	}
	ln<<")";
	if(bHeaderOnly) {
		m_codeLang->OutCodeLine(&ofs,&ln,-1,false,"set particle positions in the event (first=0)");
	} else {
		ln<<" { ";
		for(i=0;i<(nIn+nFin);i++){ ln<<S_ARR_PARTICLE_ORDER_CPP<<"["<<i<<"]="<<((CEString*)sPars.GetAt(i))->getData()<<"; "; }
		ln<<"}";
		m_codeLang->OutCodeLine(&ofs,&ln,-1,true);
	}
}

/*
*	basic expression parsing: add new fortran-compartible objects here
*	TODO: change 'STORE_PLAIN_TEXT' to 'STORE_FORTRAN_SYMBOL' after ''STORE_F..' is implemented
*
*	Note: function should be automatically splitted into sub-functions ==>
*	F-storing can't be easily realized by toString(STORE_FORTRAN_SYMBOL),
*	STORE_FORTRAN_SYMBOL is used for non-splitting symbols only
*
*	Note: level flags are correctly stored if product-based expressions only
*
*	Note: may (TODO: check if it may!) delete 'fi' and return it's clone
*	Currently returns 'fi' always
*/
CFortranFunctionInfo* CFFortran::to_fortran_string(CFortranFunctionInfo* fi,
												   CExpression* expr,
												   DWORD environs)
{
	bool doSim = (m_ParsingMode==0); // 0: collect commons only, 1: create functions
	dbg->log(35, "to_fortran_string", expr);
	if (fi == NULL) return NULL;
	bool sign = expr->getFlag();
	//bool lev = expr->getLevel();
	bool brace_flag = ((!sign) && ((environs & ENV_BRACKETS_FLAG) == ENV_BRACKETS_FLAG)); // braces are required by 'sign'
	bool bCodeFortran = ((m_general_conventions&CLFileManager::FT_F77) || (m_general_conventions&CLFileManager::FT_F90));
	bool bCodeCpp = ((m_general_conventions & CLFileManager::FT_CPP) == CLFileManager::FT_CPP);

	int iTypeDefault=m_codeLang->GetDefaultTypeId(m_conventions);
	int iTypeRes=iTypeDefault;

	const char* cn = expr->ClassName();
	if (CEUtil::equals(cn, CESymbol::CLASS_NAME)) {
		bool sym_added = false;
		long pos_COMMON = -1;
		if ((m_conventions & CLFileManager::FT_DONOT_INVOLVE_CONSTANTS)==0) { // search for symbol in constants list
			CConstRegistry::CConstInfo* constInfo=NULL; long iConstType;
			if (isSymbolConstant(expr,NULL,&iConstType)) {
				iTypeRes=iConstType;
				add_to_par_list(expr, iTypeRes, fi->consts_list()); sym_added=true;
			}
		}
		if ((!sym_added) && ((m_conventions & CLFileManager::FT_NO_COMMON_PARAMETERS)==0)) {
			pos_COMMON=FindInCommonBlock(expr,true);
			if (pos_COMMON < 0) { AddToCommonBlock(expr,true); }
			sym_added=true;
		}
		if (doSim) return fi;
		if (!sym_added) { add_to_par_list(expr, iTypeRes, fi->par_list()); }

		CEString* res = NULL;

		bool bConvInline=false; //<-- need inline type conversion. Very RARE ==> don't create 'sConvX' locally.
		CEString* sConv1=NULL; CEString* sConv2=NULL; //Used for inline type conversion (very RARE).

		if(((m_conventions & CLFileManager::FT_FORCE_REAL)==CLFileManager::FT_FORCE_REAL) && (CEAbstractFunction::IsTypeComplex(iTypeRes))) {
			int iTypeOrig=iTypeRes; iTypeRes=CEAbstractFunction::GetTypeForComplexComponent(iTypeOrig);
			sConv1=new CEString(); sConv2=new CEString(); bConvInline=true;
			if(!m_codeLang->GetTypeConversion(iTypeOrig,iTypeRes,sConv1,sConv2,m_conventions,expr,true)){dbg->log(0,"Type conversion internal error");CEException::halt();}
		}

		if ((pos_COMMON>=0) && (pos_COMMON<m_CommonArraySize)) {
			if(expr->getFlag()){ res=new CEString(""); }else{ res=new CEString("-"); }
			if(bConvInline && (sConv1!=NULL))res->AddString(sConv1->getData());
			res->AddString(get_CommonItemStr(pos_COMMON));
			if(bConvInline && (sConv2!=NULL))res->AddString(sConv2->getData());
		} else {
			if(bConvInline){ //need to extract sign for this case
				CExpression* eCpy=expr->clone();
				if(eCpy->getFlag()){ res=new CEString(""); }else{ res=new CEString("-"); eCpy->inverseFlag(); }
				if(sConv1!=NULL){ *res<<sConv1->getData(); }
				res->AddString(eCpy->toString(m_STORE_EXPR_AS_FLAG)); delete eCpy;
				if(sConv2!=NULL){ *res<<sConv2->getData(); }
			}else{
				res = expr->toString(m_STORE_EXPR_AS_FLAG);
			}
		}
		if(bConvInline){ if(sConv1!=NULL)delete sConv1; if(sConv2!=NULL)delete sConv2; }
		if (brace_flag) {res->insertString("(",0); res->AddString(")");}
		fi->set_res_value(res,iTypeRes);
		return fi;
	}

	if (expr->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) {
		if (doSim) return fi;
		CENumber* num = (CENumber*)expr;
		if (num->getNumIm() != 0) {
			if ((m_conventions & CLFileManager::FT_COMPLEX_16BIT)==0) { dbg->log(0, "ERROR: COMPLEX number in REAL procedure. Use CODE_COMPLEX16 flag"); }
			if ((environs & ENV_NUMBERS_INTEGER) == ENV_NUMBERS_INTEGER) { dbg->log(0, "ERROR: COMPLEX number when INTEGER-value is expected. Check code for 'i' sym."); }
		}
		CEString* res = NULL;
		//TODO: remove 'ENV_NUMBERS_INTEGER' flag if not required any more!
		//if (((environs & ENV_NUMBERS_INTEGER) == ENV_NUMBERS_INTEGER) && (num->is_integer())) {
		//TODO: check if INTEGER output is allowed for Fortran!
		if (num->is_integer()) {
			res = num->toString(STORE_PLAIN_TEXT); // integer
			iTypeRes=((num->is_integer())?CEAbstractFunction::PAR_INTEGER:CEAbstractFunction::PAR_REAL8);
		} else {
			res = num->toString(m_STORE_EXPR_AS_FLAG); // 5./3. real form
			iTypeRes=((num->IsReal())?CEAbstractFunction::PAR_REAL8:CEAbstractFunction::PAR_COMPLEX);
		}

		if (brace_flag) {res->insertString("(",0); res->AddString(")");}
		fi->set_res_value(res,iTypeRes);
		return fi;
	}
	
	if (CEUtil::equals(cn, CESProduct::CLASS_NAME)) {
		bool sym_added=false;
		long pos_COMMON = -1;
		if ((!sym_added) && ((m_conventions & CLFileManager::FT_NO_COMMON_PARAMETERS)==0)) {
			pos_COMMON=FindInCommonBlock(expr,true);
			if (pos_COMMON < 0) { AddToCommonBlock(expr); }
			sym_added=true;
		}
		if (doSim) return fi;
		if (!sym_added) add_to_par_list(expr, iTypeRes, fi->par_list());
		CEString* res = NULL;
		if ((pos_COMMON>=0) && (pos_COMMON<m_CommonArraySize)) {
			if(expr->getFlag()) { res = new CEString(""); } else { res = new CEString("-"); } res->AddString(get_CommonItemStr(pos_COMMON));
		} else {
			res = expr->toString(m_STORE_EXPR_AS_FLAG);
		}
		if (brace_flag) {res->insertString("(",0); res->AddString(")");}
		fi->set_res_value(res,iTypeRes);
		return fi;
	}

	if (CEUtil::equals(cn, CEPower::CLASS_NAME)) {
		CEPower* pow = (CEPower*)expr; CExpression* pw = pow->power();
		CEString* res=NULL; bool pow_ok=false;
		CFortranFunctionInfo* fi_res = fi;

		if (pow->isPowerInteger()) {
			fi_res = to_fortran_string(fi, pow->basis(), ENV_BRACKETS_SEQ); // TODO: check if fi_res may be != fi
			if (doSim) return fi;
			int iTypeBasis=fi_res->res_type();
			if(iTypeBasis==CEAbstractFunction::PAR_DEFAULT)iTypeBasis=m_codeLang->GetDefaultTypeId(m_conventions);
			iTypeRes=m_codeLang->GetOperatingType(iTypeBasis);
			CEString sTmpBefore,sTmpAfter; if(!m_codeLang->GetTypeConversion(iTypeBasis,iTypeRes,&sTmpBefore,&sTmpAfter,m_conventions,NULL,true)){ dbg->log(0,iTypeBasis,iTypeRes,"[ERROR] Can't cast power basis type (%ld) to required one (%ld). ",pow->basis()); CEException::halt(); }
			if(bCodeCpp) { res = new CEString(CCodeLanguage::CPP_POW_DEF_SYM); *res<<"("; }else{ res = new CEString(""); }
			//Note: we may not write 'a*a' here because the basis may be complicated
			*res<<sTmpBefore.getData()<<fi_res->extract_res_value()<<sTmpAfter.getData();
			if(bCodeCpp){ *res<<","; }else{ res->AddString("**"); }
			fi_res = to_fortran_string(fi_res, pw, ENV_BRACKETS_SEQ | ENV_NUMBERS_INTEGER); // TODO: check if fi_res may be != old fi_res
			*res<<fi_res->extract_res_value();
			if(bCodeCpp) { *res<<")"; }
			pow_ok=true;
		} else {
			if (pw->getProperty(CEProperties::PROP_MASK_IS_NUMBER)) {
				//TODO: allow negative powers here
				CENumber* n_pw = (CENumber*)pw;
				long n_N = n_pw->getNum();
				if ((n_pw->getNumIm() == 0) && (n_pw->getFlag()) && (n_pw->getLevel()) && (n_pw->getDen()>0) && (n_N>=0)) {
					//power is real
					long d_N = CLMath::Log2Int(n_pw->getDen());
					if (d_N>0) { // power is x/2^n
						fi_res = to_fortran_string(fi, pow->basis(),ENV_BRACKETS_NEVER); //ENV_BRACKETS_SEQ// TODO: check if fi_res may be != fi
						if (doSim) return fi;
						int iTypeBasis=fi_res->res_type(); int iTypeBasis2=iTypeBasis;
						if(iTypeBasis2==CEAbstractFunction::PAR_DEFAULT)iTypeBasis2=m_codeLang->GetDefaultTypeId(m_conventions);
						iTypeBasis2=m_codeLang->GetOperatingType(iTypeBasis);
						CEString sTmpBefore,sTmpAfter; if(!m_codeLang->GetTypeConversion(iTypeBasis,iTypeBasis2,&sTmpBefore,&sTmpAfter,m_conventions,NULL,true)){ dbg->log(0,iTypeBasis,iTypeBasis2,"[ERROR] Can't cast power basis type (%ld) to required one (%ld). ",pow); CEException::halt(); }

						res = new CEString("");
						if(n_N!=1) { if(bCodeCpp){*res<<"pow(";} }
						*res<<sTmpBefore.getData()<<fi_res->extract_res_value()<<sTmpAfter.getData();
						if(n_N!=1) { if(bCodeCpp){ *res<<","<<n_N<<")"; }else{ *res<<"**"; res->add_int(n_N); } }

						iTypeRes=m_codeLang->GetDefaultRealTypeId();
						if(!m_codeLang->GetTypeConversion(iTypeBasis2,iTypeRes,&sTmpBefore,&sTmpAfter,m_conventions,NULL,true)){ dbg->log(0,iTypeBasis2,iTypeRes,"[ERROR] Can't cast power basis type (%ld) to required one (%ld). ",pow); CEException::halt(); }
						res->insertString(&sTmpBefore,0,false); *res<<sTmpAfter.getData(); //src -> to real type

						res->insertString((bCodeCpp?"fabs(":"DABS("),0); res->AddString(")"); //fabs/DABS
						for(long SQR_i=0;SQR_i<d_N;SQR_i++) { res->insertString(bCodeCpp?"sqrt(":"DSQRT(",0); res->AddString(")"); }
						pow_ok=true;
					}
				}
			}
		}
		if ((!pow_ok) || (res==NULL)) {
			dbg->log(0, "ERROR: can't create FORTRAN code for 'power' (see debug.nb)", expr);
			dbg->log(0, "Try type FORTRAN code manually instead of 'UNKNOWN' is generated file");
			res = new CEString("UNKNOWN");
		}
		if (!sign) res->insertString("-", 0);
		if (brace_flag || ((environs & ENV_BRACKETS_SEQ) == ENV_BRACKETS_SEQ)) {res->insertString("(",0); res->AddString(")");}
		fi_res->set_res_value(res,iTypeRes);
		return fi_res;
	}

	if (expr->getProperty(CEProperties::PROP_MASK_IS_FUNCTION)) {
		CEAbstractFunction* f_expr = (CEAbstractFunction*)expr;
		CEString sTmpBefore,sTmpAfter,sTmpAfterRes; //used for type conversion calls
		CEString* f_title = f_expr->getTitleAdv(m_conventions, m_STORE_EXPR_AS_FLAG);
		// convert title to FORTRAN standard
		f_title = CFFortran::toFortranSyms(f_title);

		CEString* res = NULL;
		if(!doSim) {
			res=new CEString("");

			// Result type conversion
			iTypeRes=m_fRegistry.GetReturnType(f_expr);
			//TODO: implement 'same-to-parameter' result type if (iTypeRes==CEAbstractFunction::PAR_DEFAULT)

			if(((m_conventions & CLFileManager::FT_FORCE_REAL)==CLFileManager::FT_FORCE_REAL) && (CEAbstractFunction::IsTypeComplex(iTypeRes))) {
				int iTypeOrig=iTypeRes; iTypeRes=CEAbstractFunction::GetTypeForComplexComponent(iTypeOrig);
				if(!m_codeLang->GetTypeConversion(iTypeOrig,iTypeRes,&sTmpBefore,&sTmpAfterRes,m_conventions,expr,true)){dbg->log(0,"Type conversion internal error");CEException::halt();}
				res->AddString(sTmpBefore.getData());
			}

			if(m_general_conventions&CLFileManager::FT_CPP) { // <<"CLibraryClass::"
				CFuncRegistry::CFuncRegistryInfo* inf=m_fRegistry.GetInfo(f_title->getData(),f_expr->params_num());
				if(inf!=NULL){ if(inf->m_sLibCpp.length()>0){ *res<<inf->m_sLibCpp.getData()<<"::"; } }
			}
			res->AddString(f_title->getData());
		}
		CFortranFunctionInfo* fi_curr = fi;
		if (expr->getProperty(CEProperties::PROP_MASK_IS_VIRT_INTEGRAL)) {
			//asdf
			//TODO: select one implementation: this or 'VIToUsualFunctions'!!!
			dbg->log(0,"Warning: deprecated code branch (expression is pure VI).");

			// separated implementation: VI parameters are in dens-data object
			CVirtIntegral* vi = (CVirtIntegral*)expr;
			CVIDensData* dd = vi->get_dens_data();
			// The VI-0 with scalar dens-data are only F-compartible
			bool is_valid = false;
			if (dd->isInvariant()) {
				if((CEUtil::equals(vi->ClassName(),CVirtInt0::CLASS_NAME)) || (CEUtil::equals(vi->ClassName(),CVirtInt1::CLASS_NAME)))is_valid=true;
			}
			if (!is_valid) dbg->log(0, "No valid CODE interface found for virtual integral: ", vi);

			if ((!doSim) && (fi_curr->conventions() & CLFileManager::FT_SEPARATE_VIRT_INTEGRALS)) {
				int arg_num = dd->size();
				int arg_no;
				for(arg_no=0; arg_no<arg_num; arg_no++) {
					fi_curr = to_fortran_string(fi_curr, dd->GetAt(arg_no), ENV_BRACKETS_NEVER);
					res->AddString(fi_curr->extract_res_value());
					// TODO: check if fi_curr may be != fi
				}

				// set new function title
				// remove extra symbols from function title VIm**2m**2()...
				delete f_title;
				f_title = CFFortran::toFortranSyms(res);
				res = f_title->clone();
			}

			int arg_num = dd->size();
			int mass_start_pos = dd->getFirstMassPos(); // used to add **2 for masses
			if (!doSim) {
				res->AddString("(");
				// add ID parameter for LoopTools Xiii-integrals
				if (m_conventions & CLFileManager::FT_LOOPTOOLS) {
					if ((CEUtil::equals(vi->ClassName(),CVirtInt1::CLASS_NAME)) && (dd->getDensNumber() >= 3)) {
						char lt_sym='a' + (dd->getDensNumber()-1); // compose LT-defined parameters like "cc112"
						res->addChar(lt_sym); *res<<vi->GetHEPId()<<",";
					}
					if(mass_start_pos<0){ mass_start_pos=arg_num; } // no add **2 if no masses pos info available (non-invariant DD?)
				}
			}
			// params-> [LoopTools->INT],REAL,REAL... result-> DOUBLE COMPLEX
			for(int arg_no=0; arg_no<arg_num; arg_no++) {
				if(arg_no>0){ if(!doSim) res->AddString(","); }
				CExpression* param = dd->GetAt(arg_no);
				bool del_param = false;
				if (m_conventions & CLFileManager::FT_LOOPTOOLS) { // masses should be **2 for LT
					if((arg_no >= mass_start_pos)&&(!CEBasic::isZero(param))){ param=param->sqra(); del_param=true; }
				}

				// LoopTools VI parameters are real*8/double, non-LoopTools VI we consider as real*8/double too.
				DWORD iParTypeReq=CEAbstractFunction::PAR_REAL8;

				fi_curr = to_fortran_string(fi_curr, param, ENV_BRACKETS_NEVER);
				int iTypePar=fi_curr->res_type();
				if(!m_codeLang->GetTypeConversion(iTypePar,iParTypeReq,&sTmpBefore,&sTmpAfter,m_conventions,param,bCodeFortran)){ dbg->log(0,iTypePar,iParTypeReq,"[ERROR] Can't cast function parameter (type: %ld) to required type (%ld)",f_expr); CEException::halt(); }
				if(!doSim) { *res<<sTmpBefore.getData(); res->AddString(fi_curr->extract_res_value()); *res<<sTmpAfter.getData(); }
				if(del_param){ delete param; param=NULL; }
				// TODO: check if fi_curr may be != fi
			}
			if(doSim)return fi_curr;
			res->AddString(")");
		} else { // Non-VI function
			if(!doSim)res->AddString("(");
			int arg_num = f_expr->params_num();
			int arg_no;
			for(arg_no=0; arg_no<arg_num; arg_no++) {
				if (arg_no>0) { if(!doSim)res->AddString(","); }
				CExpression* param = f_expr->get_param(arg_no); bool del_param=false;
				if(param==NULL){ param=CENumber::GetZero(); del_param=true; }
				DWORD iParTypeReq=m_fRegistry.GetParameterType(f_expr,arg_no);

				//asdf
				bool bSkipParamParsing=false; //TODO: remove after allowing parsing with 'local' type (e.g. 4-vectors)
				if (iParTypeReq==CEAbstractFunction::PAR_4VEC_REAL) { //convert to 4-vec type
					//TODO: implement complicated 4-vector function parameters
					if ((param->ClassName() != CESymbol::CLASS_NAME) || (!param->getFlag()) || (!param->getLevel())) {
						dbg->log(0, "[Internal ERROR] Complicated '4-vector' parameter for function. The case is not implemented.", f_expr, param); CEException::halt();
					}
					if(!doSim){
						//TODO: remove next 'parsing' after implementation of 'SetCurrentType(4-vector)'
						add_to_par_list(param, CEAbstractFunction::PAR_4VEC_REAL, fi_curr->par_list());
						fi_curr->set_res_value(param->toString(m_STORE_EXPR_AS_FLAG),CEAbstractFunction::PAR_4VEC_REAL);
						bSkipParamParsing=true;
					}
				}
				if(!bSkipParamParsing){
					fi_curr = to_fortran_string(fi_curr, param, ENV_BRACKETS_NEVER); int iTypePar=fi_curr->res_type();
					if(!m_codeLang->GetTypeConversion(iTypePar,iParTypeReq,&sTmpBefore,&sTmpAfter,m_conventions,param,bCodeFortran)){ dbg->log(0,iTypePar,iParTypeReq,"[ERROR] Can't cast function parameter (type: %ld) to required type (%ld)",f_expr); CEException::halt(); }
				}else{
					sTmpBefore.Reset(); sTmpAfter.Reset();
				}
				if(!doSim){ *res<<sTmpBefore.getData(); res->AddString(fi_curr->extract_res_value()); *res<<sTmpAfter.getData(); }
				if(del_param){ delete param; param=NULL; }
				// TODO: check if fi_curr may be != fi
			}
			if(doSim)return fi_curr;
			if(!doSim)res->AddString(")");
		}
		//Add result type conversion operand (for 'FT_FORCE_REAL' mode)
		if(sTmpAfterRes.length()>0)res->AddString(sTmpAfterRes.getData());

		if(!doSim){ if(!sign) res->insertString("-", 0); }
		if(!doSim){ if(brace_flag) {res->insertString("(",0); res->AddString(")");} }

		if (!isFuncInternal(f_title)) {
			fi_curr->add_to_func_used_list(iTypeRes, f_title, true);
		} else {
			delete f_title;
		}

		fi_curr->set_res_value(res,iTypeRes);
		return fi_curr;
	}

	if (expr->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
		CEProduct* prod = (CEProduct*)expr;
		CEString* res = (doSim)?NULL:(new CEString(""));
		if (!sign) { if(!doSim)res->AddString("-"); }
		/*
		*	=== Type casting politics ===
		*	Problem:
		*		real8 a,b;
		*		1. (realX)a*(realX)b != (realX)(a*b)
		*		2. (realX)a*(realX)b == a*(realX)b		<== correct calculation!
		*	Solutions:
		*		1. [Not used]Cast second item type to 'operating' one if previous item is less precise.
		*		2. [Not used]Cast every variable to 'operating' type
		*		3. [Used]Cast variable to 'operating' type if 'left' operand is NOT of 'operating' type.
		*			==> generate '(realX)1./2.' instead of '(realX)1./(realX)2.'
		*/

		CFortranFunctionInfo* fi_res = fi;
		long s = prod->size();
		iTypeRes=CEAbstractFunction::PAR_DEFAULT;
		CEString sTmpBefore,sTmpAfter;
		for (long i=0; i<s; i++) {
			CExpression* item = prod->GetAt(i);
			if(!doSim){
				if(item->getLevel()){ if(i!=0)res->AddString("*"); }
				else{ res->AddString( ((i==0)? (((environs & ENV_NUMBERS_INTEGER) == ENV_NUMBERS_INTEGER)?"1/":"1./") : ("/")) ); }
			}
			fi_res = to_fortran_string(fi_res, item, ENV_BRACKETS_PROD);
			int iTypeItem=fi_res->res_type(); int iTypeItemOp=iTypeItem;
			if(iTypeItemOp==CEAbstractFunction::PAR_DEFAULT)iTypeItemOp=m_codeLang->GetDefaultTypeId(m_conventions);
			iTypeItemOp=m_codeLang->GetOperatingType(iTypeItemOp);
			int iTypeLeft=iTypeRes; iTypeRes=CEAbstractFunction::GetMorePreciseType(iTypeRes,iTypeItemOp);
			if(!doSim){
				//Cast item to 'operating' type
				//if left operand type is OK, don't use additional casts
				if((iTypeItemOp!=iTypeItem) && (iTypeItemOp!=iTypeLeft)){
					if(!m_codeLang->GetTypeConversion(iTypeItem,iTypeItemOp,&sTmpBefore,&sTmpAfter,m_conventions,item,true)){ dbg->log(0,iTypeItem,iTypeItemOp,"[ERROR] Can't cast item to 'operating' type (%ld -> %ld). ",item); CEException::halt(); }
				}else{
					sTmpBefore.Reset(); sTmpAfter.Reset();
				}
				*res<<sTmpBefore.getData(); res->AddString(fi_res->extract_res_value()); *res<<sTmpAfter.getData();
			}
		}
		if(!doSim) {
			if (brace_flag || ((environs & ENV_BRACKETS_PROD) == ENV_BRACKETS_PROD)) {res->insertString("(",0); res->AddString(")");}
			fi_res->set_res_value(res,iTypeRes);
		}
		return fi_res;
	}

	if (expr->getProperty(CEProperties::PROP_MASK_IS_SUM)) {
		CESum* sum = (CESum*)expr;
		//CFortranFunctionInfo* fi_res = fi;
		CFortranFunctionInfo* fi_curr = (doSim)?fi:(new CFortranFunctionInfo(fi->conventions()));

		//	1. inline sum
		//	2. precalculated sum
		//	3. sub functions involved
		bool sum_inline = true;
		iTypeRes=CEAbstractFunction::PAR_DEFAULT;
		CEString* temp_sym=NULL; int iTypeTmpSym=CEAbstractFunction::PAR_DEFAULT; // not used if inline;

		CEString* sum_str = (doSim)?NULL:(new CEString(""));
		bool single_and_positive = true; // for whole sum, used to simplify final '-' signs and brackets
		long s = sum->size();
		CEString sTmpBefore,sTmpAfter;
		for(long i=0; i<s; i++) {
			CExpression* item = sum->GetAt(i);
			if (single_and_positive) { if ((i>0) || (!item->getFlag())) single_and_positive = false; }
			//Old: add "+" symbol only ("-" is added to item's view):
			//New: add "+" or "-" here
			bool bSign=item->getFlag();
			if(!doSim){ if(bSign){ if ((i>0)||(!sum_inline))sum_str->AddString("+"); }else{ sum_str->AddString("-"); } }
			if(!doSim){ if (!bSign)item->inverseFlag(); }
			fi_curr = to_fortran_string(fi_curr, item, ENV_BRACKETS_NEVER);
			if(!doSim){ if (!bSign)item->inverseFlag(); }

			int iTypeItem=fi_curr->res_type(); int iTypeItemOp=iTypeItem;
			if(iTypeItemOp==CEAbstractFunction::PAR_DEFAULT)iTypeItemOp=m_codeLang->GetDefaultTypeId(m_conventions);
			iTypeItemOp=m_codeLang->GetOperatingType(iTypeItemOp);
			int iTypeLeftItem=iTypeRes; iTypeRes=CEAbstractFunction::GetMorePreciseType(iTypeRes,iTypeItemOp);
			if(!doSim) {
				//Cast item to 'operating' type
				//if left operand type is OK, don't use additional casts
				if((iTypeItemOp != iTypeItem) && (iTypeItemOp != iTypeLeftItem)){
					if(!m_codeLang->GetTypeConversion(iTypeItem,iTypeItemOp,&sTmpBefore,&sTmpAfter,m_conventions,item,true)){ dbg->log(0,iTypeItem,iTypeItemOp,"[ERROR] Can't cast sum item to 'operating' type (%ld -> %ld). ",item); CEException::halt(); }
					//dbg->log(0,iTypeItem,iTypeItemOp,"[Conversion] %ld -> %ld. ",item);
					//dbg->log(0,iTypeLeftItem,iTypeRes,"[Conversion] iTypeLeftItem: %ld, iTypeRes: %ld.");
				}else{
					sTmpBefore.Reset(); sTmpAfter.Reset();
				}
				*sum_str<<sTmpBefore.getData(); sum_str->AddString(fi_curr->extract_res_value()); *sum_str<<sTmpAfter.getData();
				if (sum_str->length() >= m_string_terminate_pos) {
					//New 'temp' variable: at first loop OR if type is changed
					if ((sum_inline) || (iTypeTmpSym!=iTypeRes)) {
						if(sum_inline){ sum_inline=false; fi_curr->init_pre_calc_list(); }
						if(temp_sym!=NULL){ fi_curr->add_to_temp_var_list(temp_sym,iTypeTmpSym); temp_sym=NULL; }//deletes temp_sym;
						iTypeTmpSym=iTypeRes; temp_sym=get_new_temp_symbol();
					}

					single_and_positive=true;
					if (fi_curr->precalc_list()->size() < m_num_strings_in_sum) {
						// Break line
						sum_str->insertString(" = ", 0); // TEMP = (old TEMP)all previous
						sum_str->insertString(temp_sym, 0, false);
						fi_curr->precalc_list()->add(sum_str);
						sum_str = temp_sym->clone(); // result: TEMP
					} else {
						// Create new function
						// Use 'temp_sym' as a function name
						// Note: 'temp_sym' may be used or not used as local variable in sub-function.
						// ==> Actual sub-function name is changed in 'add_function' procedure for C++ language.

						fi_curr->set_func_name(temp_sym->getData()); fi_curr->set_res_value(sum_str,iTypeRes);
						sum_str=GetSubFuncCallString(fi_curr); CEPtrArray* func_par_list=fi_curr->par_list();
						m_sub_functions->add(fi_curr);

						fi_curr = new CFortranFunctionInfo(fi->conventions());
						merge_list_with_other(fi_curr->par_list(), func_par_list);
						fi_curr->add_to_func_used_list(iTypeRes,temp_sym); temp_sym=NULL;

						// if function call is too long --> break line, else continue inline
						if (sum_str->length() >= m_string_terminate_pos) {
							iTypeTmpSym=iTypeRes; temp_sym=get_new_temp_symbol();
							fi_curr->init_pre_calc_list();
							sum_str->insertString(" = ", 0); sum_str->insertString(temp_sym, 0, false);
							fi_curr->precalc_list()->add(sum_str);
							sum_str=temp_sym->clone();
						} else {
							sum_inline=true; // if(temp_sym!=NULL){ delete temp_sym; temp_sym=NULL; }
						}	
					}
				}
				//else single_and_positive = false; // TODO: remove this
			}
		}
		if (doSim) return fi;

		// turn result to 'fi' function & delete 'fi_curr'
		fi->merge_with_other(fi_curr);

		if (!sign) {
			if (single_and_positive) {sum_str->insertString("-", 0);}
			else {sum_str->insertString("-(", 0); sum_str->AddString(")");}
		}

		bool sum_needs_braces = (brace_flag || ((!single_and_positive) && ((environs & ENV_BRACKETS_SUM) == ENV_BRACKETS_SUM)));
		if (sum_needs_braces) { sum_str->insertString("(",0); sum_str->AddString(")"); }

		if (temp_sym != NULL) { fi->add_to_temp_var_list(temp_sym,iTypeTmpSym); } //deletes temp_sym;
		fi->set_res_value(sum_str,iTypeRes);
		return fi;
	}

	// Error: unexpected object.
	dbg->log(0, "Unexpected object in FORTRAN generator.", expr);
	if (doSim) return fi;

	// try default
	CEString* res = expr->toString(m_STORE_EXPR_AS_FLAG);
	if (brace_flag) {res->insertString("(",0); res->AddString(")");}
	fi->set_res_value(res,iTypeRes);
	return fi;
}

CEString* CFFortran::get_new_temp_symbol() {
	m_last_temp_num++;
	int l=0; if(m_TEMP_str.length()>0){ l=sprintf(m_TmpBuf1kb, m_TEMP_str.getData()); }else{ l=sprintf(m_TmpBuf1kb, "TEMP"); }
	l=CEUtil::addACBNumber(m_TmpBuf1kb, m_last_temp_num, l);
	CEString* res = new CEString(m_TmpBuf1kb);
	return res;
}

/*
*	other remains unchanged;
*/
void CFFortran::merge_list_with_other(CEPtrArray* par_list, CEPtrArray* other) {
	if ((other == NULL) || (par_list == NULL)) return;
	long s = other->size();
	for(long i=0; i<s; i++) {
		CExprContainer* item = (CExprContainer*)other->GetAt(i); if(item==NULL)continue;
		add_to_par_list(item->get_expr(), item->get_info(), par_list);
	}
}

/*void CFFortran::merge_string_array_with_other(CEPtrArray* par_list, CEPtrArray* other)
{
	if (other == NULL) return par_list;
	if (par_list == NULL) {return NULL;}

	int s = par_list->size();
	int o_s = other->size();
	int i;
	for(i=0; i<o_s; i++) {
		CEObject* item = other->GetAt(i);
		if (item->ClassName())
		
		int j;
		for(j=0; j<s; j++) {
			if (item->equals(par_list->GetAt(i))) {
			if (do_delete) delete par_new;
			return;
		}
	}
	if (do_delete) par_list->addItem(par_new);
	else par_list->addItem(par_new->clone());
	}
}*/

// does not affect parameters
void CFFortran::add_to_par_list(CExpression* par_new, CSequence* par_list) {
	if ((par_new == NULL) || (par_list == NULL)) return;

	// uniform parameter
	bool do_delete = false;
	if ((!par_new->getFlag()) || (!par_new->getLevel())) { par_new = par_new->clone(); par_new->setFlag(); par_new->setLevel(true); do_delete=true; }

	long s = par_list->size();
	for(long i=0; i<s; i++) { if (par_new->equals(par_list->GetAt(i))) { if(do_delete)delete par_new; return; } }
	par_list->addItem( ((do_delete)?par_new:(par_new->clone())) );
}
// does not affect parameters
void CFFortran::add_to_par_list(CExpression* par_new, int type, CEPtrArray* par_list) {
	if ((par_new == NULL) || (par_list==NULL)) return;

	// uniform parameter
	bool do_delete = false;
	if ((!par_new->getFlag()) || (!par_new->getLevel())) { par_new = par_new->clone(); par_new->setFlag(); par_new->setLevel(true); do_delete=true; }

	long s = par_list->size();
	for(long i=0; i<s; i++) {
		CExprContainer* item=(CExprContainer*)par_list->GetAt(i); if(item==NULL)continue;
		if(par_new->equals(item->get_expr())){
			if((type != 0) && (item->get_info()!=type)) { //Check types
				if(item->get_info()==0){ item->set_info(0,type); }
				else{ dbg->log(0,"ERROR: variables with different types spotted",par_new); CEException::halt(); }
			}
			if(do_delete)delete par_new; return;
		}
	}
	par_list->add(new CExprContainer(par_new,type)); if(do_delete)delete par_new;
}

/*
*	Warning: par_list may contain SYMBOLS, vector sums, 2- and 4-SCALAR COUPLINGS
*/
void CFFortran::collect_syms(CExpression* expr, CSequence* par_list, unsigned objects_to_collect) {
	if ((expr == NULL) || (par_list == NULL)) return;	
	const char* cn = expr->ClassName();

	if (CEUtil::equals(cn, CESymbol::CLASS_NAME)) {
		if (objects_to_collect & FORTRAN_OBJ_SYMBOLS) add_to_par_list(expr, par_list); return;
	}	
	if (CEUtil::equals(cn, CEPower::CLASS_NAME)) {
		CEPower* pow = (CEPower*)expr;
		collect_syms(pow->basis(),par_list,objects_to_collect); collect_syms(pow->power(),par_list,objects_to_collect);
		return;
	}
	if (expr->getProperty(CEProperties::PROP_MASK_IS_FUNCTION)) {
		CEAbstractFunction* f_expr = (CEAbstractFunction*)expr;
		long s=f_expr->params_num(); long i;
		for(i=0; i<s; i++){ collect_syms(f_expr->get_param(i), par_list, objects_to_collect); }

		if (expr->getProperty(CEProperties::PROP_MASK_IS_VIRT_INTEGRAL)) {
			CVIDensData* dd=((CVirtIntegral*)f_expr)->get_dens_data();
			s=dd->size(); for(i=0; i<s; i++){ collect_syms(dd->GetAt(i), par_list, objects_to_collect); }
		}
		return;
	}
	if (expr->getProperty(CEProperties::PROP_MASK_IS_PRODUCT)) {
		CEProduct* prod = (CEProduct*)expr;
		long s = prod->size(); for(long i=0; i<s; i++){ collect_syms(prod->GetAt(i), par_list, objects_to_collect); }
		return;
	}
	if (expr->getProperty(CEProperties::PROP_MASK_IS_SUM)) {
		CESum* sum=(CESum*)expr; long s=sum->size();
		for(long i=0; i<s; i++){ collect_syms(sum->GetAt(i), par_list, objects_to_collect); }
		return;
	}
	if (CEUtil::equals(cn, CESProduct::CLASS_NAME)) {
		if (objects_to_collect & FORTRAN_OBJ_2SCALAR_COUPLINGS) add_to_par_list(expr, par_list);
		else { // collect symbols
			CESProduct* spr=(CESProduct*)expr;
			if(objects_to_collect & FORTRAN_ALL_VECTORS){
				add_to_par_list(spr->get_a(), par_list); add_to_par_list(spr->get_b(), par_list);
			} else {
				collect_syms(spr->get_a(), par_list, objects_to_collect); collect_syms(spr->get_b(), par_list, objects_to_collect);
			}
		}
		return;
	}
	if (CEUtil::equals(cn, CEAntiSymmProd::CLASS_NAME)) {
		if (objects_to_collect & FORTRAN_OBJ_4SCALAR_COUPLINGS) add_to_par_list(expr, par_list);
		else { // collect symbols
			CEAntiSymmProd* a_pr = (CEAntiSymmProd*)expr;
			if(objects_to_collect & FORTRAN_ALL_VECTORS){ for(int i=0;i<4;i++)add_to_par_list(a_pr->get_v(i),par_list); }
			else{ for(int i=0;i<4;i++)collect_syms(a_pr->get_v(i),par_list,objects_to_collect); }
		}
		return;
	}
	if (CEUtil::equals(cn, CEFuncBondarev::CLASS_NAME)) {
		if (objects_to_collect & FORTRAN_OBJ_2SCALAR_COUPLINGS) add_to_par_list(expr, par_list);
		else { // collect symbols
			CEFuncBondarev* fb = (CEFuncBondarev*)expr;
			CExpression* c1 = fb->GetCouple1(); CExpression* c2 = fb->GetCouple2(); if((c1==NULL)||(c2==NULL)){ dbg->log(0, "ERROR. Output code cannot be created due to vector CEFuncBondarev object", fb); CEException::halt(); }
			if(objects_to_collect & FORTRAN_ALL_VECTORS){
				add_to_par_list(c1, par_list); add_to_par_list(c2, par_list);
			}else{
				collect_syms(c1, par_list, objects_to_collect); collect_syms(c2, par_list, objects_to_collect);
			}
		}
		return;
	}
}

/*
*****************************************
*	MAIN ENTRY
*****************************************
*
*	'conventions' are additional temporary flags for current function.
*	Note: source may (and probably will) be changed, but is NOT deleted.
*/
void CFFortran::add_function(const char* func_name, CExpression *expr, DWORD conventions) {
	if (expr == NULL) return;
	long sExprComplexity = CEBasic::CalcComplexity(expr);
	dbg->log(10, sExprComplexity, "funcion addition (complexity: %ld)");
	//Note: output file is opened LATER (just before saving) to avoid keeping it open too long
	if(!m_bInited)init();
	CKinematics* kin = CEProcessor::kinematics(); //used in C++ header creation

	bool bLogSteps = (sExprComplexity > 300);
	if(bLogSteps)dbg->logperiod(0,"CFFortran::add_function step 1");

	//Init
	DWORD old_conventions = m_conventions;
	m_conventions = m_conventions | conventions;
	//
	//Note:
	//	- 'm_codeLang' must be properly initialized BEFORE first saving to file
	//Elsewhere:
	//	- unable to use same 'm_codeLang' after file re-opening
	//Therefore:
	//	- use 'FT_CPP_TYPE_ABSTRACT' flag in file opening!
	//	- next check is left for compartibility reasons only
	//
	if((m_conventions & CLFileManager::FT_CPP_TYPE_ABSTRACT)==CLFileManager::FT_CPP_TYPE_ABSTRACT) { //deprecated check!
		m_codeLang->SetAbstractAvailable(true);
	}else{
		m_codeLang->SetAbstractAvailable(false);
	}
	long i;

	//Params check
	if (((m_conventions & CLFileManager::FT_COMPLEX_16BIT)==0) && ((m_conventions & CLFileManager::FT_REAL_8BIT)==0)) {
		m_conventions|=CLFileManager::FT_COMPLEX_16BIT;
		dbg->log(0, "COMPLEX type is involved");
	}
	if((m_conventions & CLFileManager::FT_FORCE_REAL)==CLFileManager::FT_FORCE_REAL){
		if(m_conventions & CLFileManager::FT_COMPLEX_16BIT){ m_conventions^=CLFileManager::FT_COMPLEX_16BIT; m_conventions|=CLFileManager::FT_REAL_8BIT; }
	}
	if (((m_conventions & CLFileManager::FT_LOOPTOOLS)!=0)  && ((m_conventions & CLFileManager::FT_COMPLEX_16BIT)==0)) {
		//Old: m_conventions^=CLFileManager::FT_REAL_8BIT; m_conventions|=CLFileManager::FT_COMPLEX_16BIT;
		//Old: dbg->log(0, "LoopTools --> COMPLEX type is set");
	}
	if(m_general_conventions&CLFileManager::FT_CPP) {
		if ((m_conventions & CLFileManager::FT_NO_COMMON_PARAMETERS) == 0) m_conventions |=CLFileManager::FT_NO_COMMON_PARAMETERS;
		//if ((m_conventions & CLFileManager::FT_NO_INCLUDES) == 0) m_conventions |=CLFileManager::FT_NO_INCLUDES;
	}

	//Init (continue)
	m_constRegistry.Init(m_conventions);

	// init
	bool use_COMMON = ((m_conventions & CLFileManager::FT_NO_COMMON_PARAMETERS) == 0);
	bool use_COMMON_arr = false;
	if (use_COMMON) {
		use_COMMON_arr = ((m_conventions & CLFileManager::FT_NO_COMMON_ARRAYS) == 0);
		// compose COMMON block name (unique for this function)
		if (m_COMMONBLOCK_NAME != NULL) { CMemoryManager::mfree(m_COMMONBLOCK_NAME); m_COMMONBLOCK_NAME=NULL; }
		if (m_CommonBlockSyms != NULL) { delete m_CommonBlockSyms; m_CommonBlockSyms=NULL; }
		m_COMMONBLOCK_NAME = CEUtil::concat(COMMON_BLOCK_PREFIX, func_name);
		m_CommonBlockSyms=new CSequence(); m_CommonArraySize=0;
	}
	if (m_general_conventions & CLFileManager::FT_CPP) { m_COMMON_ARRAY_TITLE=COMMON_ARRAY_TITLE_C; } else { m_COMMON_ARRAY_TITLE=COMMON_ARRAY_TITLE; }
	if (m_general_conventions & CLFileManager::FT_CPP) { m_STORE_EXPR_AS_FLAG=STORE_CPP; } else { m_STORE_EXPR_AS_FLAG=STORE_FORTRAN_SYMBOL; }

	//init
	if (m_general_conventions & CLFileManager::FT_CPP) {
		if(m_CppClassName!=NULL){ CMemoryManager::mfree(m_CppClassName); m_CppClassName=NULL; }
		// "C+Name" + validate
		sprintf(m_TmpBuf1kb,"C%s",func_name); //Must be different to any function name
		CESymbol sTmp0(m_TmpBuf1kb); CExpression* sTmp=validate_syms(&sTmp0,NULL);
		if(sTmp!=NULL){ m_CppClassName=sTmp->toLine(m_STORE_EXPR_AS_FLAG); } if(sTmp!=(&sTmp0))delete sTmp;
		if(m_CppClassName==NULL)m_CppClassName=CEUtil::copyString(func_name);
	}

	// init
	m_num_strings_in_sum = (MAX_LINES_IN_FUNC_DEFAULT > 3)?MAX_LINES_IN_FUNC_DEFAULT:100000;
	if (conventions & CLFileManager::FT_DONOT_SPLIT_FUNC) m_num_strings_in_sum = 100000;
	bool del_expr = false;

	if(m_syms_replaced != NULL) { delete m_syms_replaced; m_syms_replaced=NULL; }
	m_syms_replaced = new CEPtrArray();
	
	bool need_vec_entry = ((m_conventions & CLFileManager::FT_DONOT_CREATE_4VECTOR_ENTRY)==0);
	const char* _ScalarFName=func_name; CEString s_ScalarFName;
	if(need_vec_entry){ s_ScalarFName<<func_name<<"Impl"; _ScalarFName=s_ScalarFName.getData(); }
	CExpression* expr_valid=NULL;

	if(bLogSteps)dbg->logperiod(0,"CFFortran::add_function step 2");

	// ampl-ME2-CS selection
	//[unused]if (m_ME2ToDiffCSFunc!=NULL) { delete m_ME2ToDiffCSFunc; m_ME2ToDiffCSFunc=NULL; }
	bool src_is_Ampl = ((m_conventions & CLFileManager::FT_SRC_IS_AMPLITUDE) == CLFileManager::FT_SRC_IS_AMPLITUDE);
	//Note: currently (FT_SRC_SAVE_AS_IS == FT_SRC_IS_DIFFCS) ==> (src_is_DiffCS == bSaveAsIs)
	bool src_is_DiffCS = ((m_conventions & CLFileManager::FT_SRC_IS_DIFFCS) == CLFileManager::FT_SRC_IS_DIFFCS);
	bool bSaveAsIs = ((m_conventions & CLFileManager::FT_SRC_SAVE_AS_IS) == CLFileManager::FT_SRC_SAVE_AS_IS);
	if ((!src_is_DiffCS) && (!bSaveAsIs)) { //Nt: currently (src_is_DiffCS==bSaveAsIs)
		if (!src_is_Ampl) {
			expr_valid = CLHEPBasics::GetDiffCS(expr->clone(),true,((CFFortran::m_PAR_CODE_GEN_FAST)?false:true));
			if (expr_valid==NULL) { dbg->log(0, "ERROR: can't turn axpression to DiffCS form"); expr_valid=expr->clone(); }
			if (del_expr) delete expr; del_expr=true; expr=expr_valid;
		} else {
			CExpression* _CSFactor = CLHEPBasics::GetDiffCS(CENumber::GetUnit(),true,((CFFortran::m_PAR_CODE_GEN_FAST)?false:true));
			if (_CSFactor==NULL) { dbg->log(0, "ERROR: can't turn axpression to DiffCS form"); _CSFactor=CENumber::GetUnit(); }
			expr_valid=expr->clone()->multiply(CEPower::sqrt(_CSFactor));
			if (del_expr) delete expr; del_expr=true; expr=expr_valid;
		}
		dbg->log(25, "DiffCS factor included:", expr);
	}

	if(bLogSteps)dbg->logperiod(0,"CFFortran::add_function step 3");

	// first normalize all the variables.
	// ************************************************** validation objects & symbols
	expr_valid = VIToUsualFunctions(expr);	// turn VI (if exist) to usual functions
	if(expr!=expr_valid){ if(del_expr)delete expr; del_expr=true; } expr=expr_valid;
	expr_valid = validate_objects(expr);	// turn functions to SYMBOLS
	if(expr!=expr_valid){ if(del_expr)delete expr; del_expr=true; } expr=expr_valid;
	if(bLogSteps)dbg->logperiod(0,"CFFortran::add_function step 3a");
	expr_valid = validate_syms(expr, m_syms_replaced); // set non-alowed symbols to '_' + 'complicated vectors'-->symbols
	if(expr!=expr_valid){ if(del_expr)delete expr; del_expr=true; } expr=expr_valid;
	if(bLogSteps)dbg->logperiod(0,"CFFortran::add_function step 3b");

	if (conventions & CLFileManager::FT_POWER_PARAMS) {
		// extract powers as function parameters (replace by symbols here)
		expr_valid = ReplPowersBySymbols(expr);
		if (expr != expr_valid) {if (del_expr) delete expr; del_expr = true;}
		expr = expr_valid;
	}

	if(bLogSteps)dbg->logperiod(0,"CFFortran::add_function step 4");

	// ************************************************** validation scalar couplings
	if(m_vec_couplings != NULL) { delete m_vec_couplings; m_vec_couplings=NULL; }
	if(m_Denominators != NULL) { delete m_Denominators; m_Denominators=NULL; }
	// Note: m_vec_couplings is NULL now & may be leaved as NULL (not filled) if no 4-vec entry are required
	m_vec_couplings = new CEPtrArray();
	expr_valid = scalar_couplings_to_syms(expr, m_vec_couplings);
	if (expr != expr_valid) {if (del_expr) delete expr; del_expr = true;}
	expr = expr_valid;

	/*
	// same to diff-CS factor for amplitude
	if ((src_is_Ampl) && (!src_is_DiffCS) && (!bSaveAsIs)) { //Nt: currently (src_is_DiffCS==bSaveAsIs)
		CExpression* _CSFactor = CLHEPBasics::GetDiffCS(CENumber::GetUnit());
		if (_CSFactor==NULL) { dbg->log(0, "ERROR: can't turn axpression to DiffCS form"); _CSFactor=CENumber::GetUnit(); }
		CExpression* _CSFactor2 = validate_objects(_CSFactor); // turm functions to SYMBOLS
		if (_CSFactor2 != _CSFactor) { delete _CSFactor; _CSFactor=_CSFactor2; }
		_CSFactor2 = validate_syms(_CSFactor, m_syms_replaced); // set non-alowed symbols to '_'.
		if (_CSFactor2 != _CSFactor) { delete _CSFactor; _CSFactor=_CSFactor2; }
	}
	*/

	if(bLogSteps)dbg->logperiod(0,"CFFortran::add_function step 5");

	// init
	if (m_sub_functions!=NULL) { delete m_sub_functions; m_sub_functions=NULL; }
	if (m_Dens_functions!=NULL) { delete m_Dens_functions; m_Dens_functions=NULL; }
	m_sub_functions = new CEPtrArray();

	// ************************************************** common-vars array filling
	if(use_COMMON_arr) {
		m_ParsingMode=0; // 0: collect commons only, 1: create functions

		CFortranFunctionInfo* fi_test = new CFortranFunctionInfo(_ScalarFName, m_conventions);
		fi_test = to_fortran_string(fi_test, expr, ENV_BRACKETS_NEVER);
		delete fi_test;

		SortCommonBlockItems();
		DeclareCommonArray();
		dbg->log(25, m_CommonArraySize, "common variables array declared (%ld items)");
		dbg->log(35, "common variables array", m_CommonBlockSyms);
	} // else filled during expression parsing
	m_ParsingMode=1; // 0: collect commons only, 1: create functions

	if(bLogSteps)dbg->logperiod(0,"CFFortran::add_function step 6");

	// ************************************************** denominators collection (for 0-check)
	long dens_num=0;
	if (m_conventions & CLFileManager::FT_CHECK_DENOMINATORS) {
		CRCollectSymbols* coll_r = new CRCollectSymbols(CRCollectSymbols::COLLECT_DENOMINATORS);
		coll_r->ApplyTo(expr,false);
		m_Denominators = coll_r->getCollectedSymbols(true); delete coll_r;
		// parse extracted denominators for powers
		dens_num=m_Denominators->size();
		for(i=0;i<dens_num;i++) {
			CExpression* d = (CExpression*)m_Denominators->GetAt(i);
			if (!d->getLevel()) d->inverseLevel();
			if (!d->getFlag()) d->inverseFlag();
			if (d->ClassName() == CEPower::CLASS_NAME) {
				d = ((CEPower*)d)->basis()->clone();
				m_Denominators->SetAt(i, d, true);
				i--; // check this item again
			}
		}
		dbg->log(25, "denomonators collected", m_Denominators);
	}

	// ************************************************** parsing expression
	if(bLogSteps)dbg->logperiod(0,"CFFortran::add_function step 7");

	dbg->log(30, "Symbols in the expression are validated.");

	CFortranFunctionInfo* fi = new CFortranFunctionInfo(_ScalarFName, m_conventions);
	fi = to_fortran_string(fi, expr, ENV_BRACKETS_NEVER);

	long sub_func_size = m_sub_functions->size();
	dbg->log(30, sub_func_size, "Expression parsed. %ld sub-functions involved.");

	if (m_conventions & CLFileManager::FT_CHECK_DENOMINATORS) {
		// create F-strings for denominators
		long comm_num=0; if(use_COMMON){comm_num=((m_CommonBlockSyms!=NULL)?(m_CommonBlockSyms->size()):0);}
		m_Dens_functions = new CEPtrArray();
		for(i=0;i<dens_num;i++) {
			CExpression* den_i=(CExpression*)m_Denominators->GetAt(i);
			CEString d_title("DEN_"); d_title<<func_name<<"_"<<i;
			bool drop_COMMON_flag=false;
			if (use_COMMON) { if ((CRCollectSymbols::CountObjects(den_i))*2 < comm_num) { drop_COMMON_flag=true; } }
			if (m_CommonArraySize>0) drop_COMMON_flag=false;
			DWORD cov_old = m_conventions;
			if (drop_COMMON_flag) { m_conventions=m_conventions|CLFileManager::FT_NO_COMMON_PARAMETERS; }
			CFortranFunctionInfo* fi_den = new CFortranFunctionInfo(d_title.getData(), m_conventions);
			fi_den = to_fortran_string(fi_den, den_i, ENV_BRACKETS_NEVER);
			if (drop_COMMON_flag) { m_conventions=cov_old; }
			m_Dens_functions->add(fi_den);
		}
	}

	if(bLogSteps)dbg->logperiod(0,"CFFortran::add_function step 8");
	// ************************************************** common-vars array sorting
	SortCommonBlockItems();

	dbg->log(2,"Registry log",&m_fRegistry,&m_constRegistry);

	// ************************************************** storing
	ofstream ofs(m_file_name, ios::out|ios::app);
	if (!ofs) { dbg->logIns(0,"<Error opening output CODE file (%s).>", m_file_name); }
	if(ofs){
		ofs<<"\n";
		// include looptools.h for C++ code
		if(m_conventions & CLFileManager::FT_LOOPTOOLS){
			if(m_general_conventions & CLFileManager::FT_CPP) {
				if(!m_fIncludes.m_bLoopToolsIncluded) { ofs << "#include \"clooptools.h\"\n"; m_fIncludes.m_bLoopToolsIncluded=true; }
			}
		}
		// ofs<<"\n";
		m_codeLang->StartCommentLine(&ofs); ofs << "**********************************************************\n";
		m_codeLang->StartCommentLine(&ofs); ofs << " Function: " << func_name << "\n";
		m_codeLang->StartCommentLine(&ofs); ofs<<" Temp variable header: "<<m_TEMP_str.getData()<<"\n";
		m_codeLang->StartCommentLine(&ofs); ofs<<" Subfunctions used: " << sub_func_size+(need_vec_entry?1:0)+((m_conventions&CLFileManager::FT_CHECK_DENOMINATORS)?dens_num:0) <<"\n\n";

		//init static (or class-member) variables
		if((m_general_conventions&CLFileManager::FT_CPP) && (m_CppClassName!=NULL)) {
			CEString ln; int n=kin->num_init_particles()+kin->num_final_particles();
			if((m_conventions & CLFileManager::FT_NON_STATIC_IMPL)==CLFileManager::FT_NON_STATIC_IMPL) { //needs constructor
				ln<<m_CppClassName<<"::"<<m_CppClassName<<"() {"; m_codeLang->OutCodeLine(&ofs,&ln,-1,true); m_codeLang->IncIndent();
				//Particles order list (non-static)
				m_codeLang->StartCommentLine(&ofs); ofs<<"Particle order array\n";
				m_codeLang->StartCommentLine(&ofs); ofs<<"The default order is: "; AddParticleOrderList(ofs); ofs<<"\n";
				m_codeLang->AddOpFor(&ln,"i",0,n-1,-1,true); ln<<" "<<S_ARR_PARTICLE_ORDER_CPP<<"[i]=i; "; m_codeLang->AddOpEndFor(&ln);
				m_codeLang->OutCodeLine(&ofs,&ln,-1,true);
				//Number of calls counter (if non-static)
				ln.AddString("m_nCalls = 0"); m_codeLang->OutCodeLine(&ofs,&ln);
				m_codeLang->DecIndent(); ln<<m_codeLang->GetOpEndProc(); m_codeLang->OutCodeLine(&ofs,&ln,-1,true); ofs<<"\n";
			}else{
				//Particles order list (static)
				m_codeLang->StartCommentLine(&ofs); ofs<<"Particle order array\n";
				m_codeLang->StartCommentLine(&ofs); ofs<<"The default order is: "; AddParticleOrderList(ofs); ofs<<"\n";
				ln<<"int "<<m_CppClassName<<"::"<<S_ARR_PARTICLE_ORDER_CPP<<"["<<n<<"] = {"; for(i=0;i<n;i++){ if(i>0)ln<<", "; ln<<i; } ln<<"}"; m_codeLang->OutCodeLine(&ofs,&ln);
			}
			//Set-particle-order function
			AddSetPartOrderFunction(ofs, false); //SetPartOrder() function
			//Number of calls counter (if static)
			if((m_conventions & CLFileManager::FT_NON_STATIC_IMPL)==0) {
				ln<<"int "<<m_CppClassName<<"::m_nCalls = 0"; m_codeLang->OutCodeLine(&ofs,&ln);
			}
			ofs<<"\n";
		}

		if (need_vec_entry) { add_vec_entry(&ofs, func_name, m_vec_couplings, fi); ofs<<"\n"; }
		if(bLogSteps)dbg->logperiod(0,"CFFortran::add_function step 9");

		add_function(&ofs, fi, true);
		//if(bLogSteps)dbg->logperiod(0,"CFFortran::add_function step 10");
		for (i=0; i<sub_func_size; i++) {
			ofs<<"\n"; add_function(&ofs, (CFortranFunctionInfo*)m_sub_functions->GetAt(i));
			//sprintf(m_TmpBuf1kb,"CFFortran::add_function step %ld of %ld",i+11,sub_func_size+11); if(bLogSteps)dbg->logperiod(0,m_TmpBuf1kb);
		}
		// dens-check output
		if (m_conventions & CLFileManager::FT_CHECK_DENOMINATORS) {
			if ((dens_num>0) && (m_Dens_functions!=NULL)) {
				if (dens_num != m_Dens_functions->size()) { dbg->log(0, "Internal ERROR, dens_num != m_Dens_functions->size(), DENOMINATORS CHECK CREATION FAILS",m_Dens_functions); dens_num=m_Dens_functions->size(); }
				for(i=0; i<dens_num; i++) { ofs<<"\n"; add_function(&ofs, (CFortranFunctionInfo*)m_Dens_functions->GetAt(i)); }
			}
		}
		ofs.close();
	}

	// Include file save
	if ((m_conventions & CLFileManager::FT_NO_INCLUDES) == 0) {
		bool bIncOK = SaveIncludeConstantsFile();
		if (!bIncOK) dbg->log(0, "ERROR saving include constants file");
	} else {
		if (m_IncludedConstantsList.size()>0) { dbg->log(0, "INTERNAL ERROR: m_IncludedConstantsList.size()!=0, at CODE_NO_INCLUDES option"); }
	}

	// Header file save (C++ only)
	if(m_general_conventions&CLFileManager::FT_CPP) {
		//TODO: move it to 'AddClassHeaderFile()' function
		bool bClass=(m_CppClassName!=NULL);
		ofstream h_ofs(m_header_file_name, ios::out|ios::app);
		if(h_ofs){
			h_ofs<<"\n";
			int _IndentSizeCpy=m_codeLang->GetIndent(); m_codeLang->ClearIndent(); CEString ln;
			if(bClass){
				h_ofs<<"#if !defined(_ALHEP_CLASS_"<<m_CppClassName<<")\n#define _ALHEP_CLASS_"<<m_CppClassName<<"\n\n";
				h_ofs<<"class "<<m_CppClassName<<" {\n"<<"public:\n"; m_codeLang->IncIndent();
				if((m_conventions & CLFileManager::FT_NON_STATIC_IMPL)==CLFileManager::FT_NON_STATIC_IMPL) { //needs constructor
					ln<<m_CppClassName<<"()"; m_codeLang->OutCodeLine(&h_ofs,&ln);
				}
			}
			if(need_vec_entry){ m_codeLang->StartCommentLine(&h_ofs); h_ofs<<"Event entry\n"; add_vec_entry(&h_ofs,func_name,m_vec_couplings,fi,true); }
			m_codeLang->StartCommentLine(&h_ofs); h_ofs << "Main entry\n";
			add_function_header(&h_ofs, fi);
			if(bClass) { // && (kin!=NULL)){
				m_codeLang->StartCommentLine(&h_ofs); h_ofs<<"Particle order array\n";
				m_codeLang->StartCommentLine(&h_ofs); h_ofs<<"The default order is: "; AddParticleOrderList(h_ofs); h_ofs<<"\n";
				if((m_conventions & CLFileManager::FT_NON_STATIC_IMPL)==0)ln<<"static ";
				ln<<"int "<<S_ARR_PARTICLE_ORDER_CPP<<"["<<(kin->num_init_particles()+kin->num_final_particles())<<"]"; m_codeLang->OutCodeLine(&h_ofs,&ln,-1,false,"particle positions in the event (first=0)");
				AddSetPartOrderFunction(h_ofs, true); //SetPartOrder() function
			}//else if(kin==NULL){ dbg->log(0, "WARNING: NULL kinematics. No partilce order array is available."); }
			h_ofs<<"\n";
			if(bClass){
				m_codeLang->StartCommentLine(&h_ofs); h_ofs<<"Debug variables\n";
				if((m_conventions & CLFileManager::FT_NON_STATIC_IMPL)==0)ln<<"static ";
				ln<<"int m_nCalls"; m_codeLang->OutCodeLine(&h_ofs,&ln);
			}
			if(sub_func_size > 0){ h_ofs<<"\n"; if(bClass){ h_ofs<<"private:\n"; } }
			for (i=0; i<sub_func_size; i++) { add_function_header(&h_ofs, (CFortranFunctionInfo*)m_sub_functions->GetAt(i)); }
			if ((dens_num>0) && (m_Dens_functions!=NULL)) {
				h_ofs<<"\n"; if(bClass){ h_ofs<<"private:\n"; }
				if (dens_num != m_Dens_functions->size()) { dbg->log(0, "Internal ERROR, dens_num != m_Dens_functions->size(), DENOMINATORS CHECK CREATION FAILS",m_Dens_functions); dens_num=m_Dens_functions->size(); }
				for(i=0; i<dens_num; i++) { add_function_header(&h_ofs, (CFortranFunctionInfo*)m_Dens_functions->GetAt(i)); }
			}
			if(bClass){ m_codeLang->ClearIndent(); h_ofs<<"};\n"; }
			if(bClass){ h_ofs<<"\n#endif // !defined(_ALHEP_CLASS_"<<m_CppClassName<<")\n"; }
			m_codeLang->SetIndent(_IndentSizeCpy);
		}else{ dbg->log(0,"Error opening output HEADER file: ", m_header_file_name); }
	}

	if(bLogSteps)dbg->logperiod(0,"CFFortran::add_function step 10");
	dbg->log(30, "Function stored.");
	// ************************************************** finalization
	delete fi;
	delete m_sub_functions; m_sub_functions = NULL;

	m_conventions = old_conventions;
	if (m_vec_couplings != NULL) { delete m_vec_couplings; m_vec_couplings = NULL; }
	if (m_syms_replaced != NULL) { delete m_syms_replaced; m_syms_replaced = NULL; }
	if (m_Denominators != NULL) { delete m_Denominators; m_Denominators=NULL; }
	if (m_Dens_functions != NULL) { delete m_Dens_functions; m_Dens_functions=NULL; }
	if (del_expr) delete expr;
}

void CFFortran::DeclareIncludeFileRequired() { // declare the include file is required
	if(m_sIncludeFileName.length()==0) {
		if((m_conventions & CLFileManager::FT_CPP)==CLFileManager::FT_CPP){
			m_sIncludeFileName<<DEFAULT_INCLUDE_FILENAME_CPP;
		}else{
			m_sIncludeFileName<<DEFAULT_INCLUDE_FILENAME_F;
		}
	}
	//dbg->log(0,m_sIncludeFileName.length(),"DeclareIncludeFileRequired, m_sIncludeFileName.length()=%ld");
}
CExpression* CFFortran::AddConstantToIncludeList(CExpression* var) {
	// add 'var' to 'm_IncludedConstantsList' & set 'm_bNeedSaveIncludeFile'
	// returns 'const' ref to object ==> do never delete/modify result
	CExpression* res = NULL;
	if(var!=NULL) {
		bool bConstAlredyExist=false;
		for(long i=0; i<m_IncludedConstantsList.size(); i++) {
			CExpression* varX=m_IncludedConstantsList.GetAt(i); if(varX==NULL)continue;
			if(varX->equals(var)){ res=varX; bConstAlredyExist=true; break; }
		}
		if (!bConstAlredyExist) {
			res=var->clone(); m_IncludedConstantsList.addItem(res); m_bNeedSaveIncludeFile=true;
		}
	}
	return res;
}
bool CFFortran::IsConstFileCreatedFor(const char* fn) { // if 'include.h' is already created for file (uses 'm_IncludeUsedBy')
	if ((m_IncludeUsedBy==NULL) || (fn == NULL))return false;
	bool bExist=false;
	long s=m_IncludeUsedBy->size();
	for(long i=0; i<s; i++){
		CEString* fnI = (CEString*)m_IncludeUsedBy->GetAt(i); if(fnI==NULL)continue;
		if(CEUtil::equals(fnI->getData(),fn)){ bExist=true; break; }
	}
	return bExist;
}
const char* CFFortran::RequestParticleMassIncluded(long pk) {
	CDiagramPhysics* ph=CDiagramPhysics::get_curr_physics(); if(ph==NULL)return NULL;
	CExpression* m=ph->get_mass_sym(pk); if(m==NULL)return NULL;
	if(m->ClassName() != CESymbol::CLASS_NAME) { return NULL; }
	CExpression* mCode=validate_sym((CESymbol*)m,NULL); if(mCode==NULL)mCode=m; //to FORTRAN notation
	const char* res = NULL;
	if(mCode->ClassName() == CESymbol::CLASS_NAME){
		//[old]const char* mSym=((CESymbol*)mCode)->get_name();	//Note: we may not return this value: it's local
		//[old]CConstRegistry::CConstInfo* inf = m_constRegistry.GetInfo(mSym);

		//check if constant is saved
		bool bConstSaved=false;
		if(m_IncludeConstsSaved!=NULL){
			for (int i=0; i<m_IncludeConstsSaved->size(); i++) {
				CExpression* var=m_IncludeConstsSaved->GetAt(i); if(var==NULL)continue;
				if(var->equals(mCode)) {
					if(var->ClassName()==CESymbol::CLASS_NAME){res=((CESymbol*)var)->get_name();}
					bConstSaved=true; break;
				}
			}
		}
		if(!bConstSaved){
			CExpression* mSymConst=AddConstantToIncludeList(mCode);
			DeclareIncludeFileRequired();
			if(mSymConst->ClassName()==CESymbol::CLASS_NAME){res=((CESymbol*)mSymConst)->get_name();}
		}
	}
	if(mCode!=m)delete mCode;
	return res;
}
bool CFFortran::SaveIncludeConstantsFile() {
	if ((m_conventions & CLFileManager::FT_NO_INCLUDES)==CLFileManager::FT_NO_INCLUDES) {
		m_bNeedSaveIncludeFile=false;
		return true;
	}

	//update registry if required
	if(!m_constRegistry.IsInited()) m_constRegistry.Init(m_conventions);

	bool bSaveFull=false; if ((m_conventions & CLFileManager::FT_FULL_INCLUDE)==CLFileManager::FT_FULL_INCLUDE) bSaveFull=true;
	bool bCpp=((m_conventions & CLFileManager::FT_CPP)==CLFileManager::FT_CPP);
	if(m_IncludeConstsSaved==NULL){
		m_IncludeConstsSaved=new CSequence();						// set of constants already saved to include file
		if(m_IncludeConstsSaved==NULL){ CEException::halt("in CFFortran::SaveIncludeConstantsFile"); }
	}
	if(m_IncludeUsedBy==NULL){ m_IncludeUsedBy=new CEPtrArray(); }	// set of .cpp/.f77 files that use include file

	CSequence* seqConstantsToSave = m_IncludeConstsSaved;			// !=NULL, don't delete
	CIntArray arrConstTypes;
	if (bSaveFull) {
		for(long n=0; n<m_constRegistry.m_Infos.size(); n++) {
			CConstRegistry::CConstInfo* inf =(CConstRegistry::CConstInfo*)m_constRegistry.m_Infos.GetAt(n); if(inf==NULL)continue;
			if(inf->m_iPath == CConstRegistry::C_INCLUDED){
				seqConstantsToSave->addItem(new CESymbol(inf->m_sName.getData())); arrConstTypes.Add(inf->m_iType);
			}
		}
	} else {
		seqConstantsToSave->AddUniquesFrom(&m_IncludedConstantsList);

		//get constant types
		for(long n=0; n<seqConstantsToSave->size(); n++) {
			CExpression* sym=seqConstantsToSave->GetAt(n); CConstRegistry::CConstInfo* inf=NULL;
			if(sym->ClassName()==CESymbol::CLASS_NAME){ inf=m_constRegistry.GetInfo(((CESymbol*)sym)->get_name()); }
			arrConstTypes.Add((inf!=NULL)?inf->m_iType:0);
		}
	}

	long c_num=seqConstantsToSave->size();
	if (((m_sIncludeFileName.length()<=0) && (c_num>0)) ||
		((m_sIncludeFileName.length()>0) && (c_num<=0))) {
		// Note: both cases are wrong
		if (m_sIncludeFileName.length()<=0) { dbg->log(0, c_num, "INTERNAL ERROR: NULL file name for include file (constants-container), c_num=%ld"); }
		if (c_num<=0) { dbg->log(0, m_sIncludeFileName.length(), "INTERNAL ERROR: NO constants but include file is REQUESTED, m_sIncludeFileName.length()=%ld"); }
		m_bNeedSaveIncludeFile=false;
		return false;
	}

	//update 'm_IncludeUsedBy' list
	if((m_file_name!=NULL)&&(m_IncludeUsedBy!=NULL)){
		if(!CFFortran::IsConstFileCreatedFor(m_file_name)){ m_IncludeUsedBy->add(new CEString(m_file_name)); }
	}

	dbg->log(25, "include file creation");
	ofstream ofs(m_sIncludeFileName.getData()); // truncate old
	if (!ofs) {printf("<Error creating include constants file (%s).>\n", m_sIncludeFileName.getData()); return false; }

	m_codeLang->StartCommentLine(&ofs); ofs << "**********************************************************\n";
	m_codeLang->StartCommentLine(&ofs); ofs << " File name: " << m_sIncludeFileName.getData() << "\n";
	if(bSaveFull) {
		CEString* sVer=new CEString(); CESupport::CreateVersionString(sVer);
		m_codeLang->StartCommentLine(&ofs); ofs << " Complete set of constants to be used by ALHEP "<<(bCpp?"C++":"Fortran")<<" output.\n";
		m_codeLang->StartCommentLine(&ofs); ofs << " It fits every "<<(bCpp?"C++":"Fortran")<<" procedure created by "<<sVer->getData()<<".\n";
		delete sVer;
		ofs << "\n";
		m_codeLang->StartCommentLine(&ofs); ofs << "Important note.\n";
		m_codeLang->StartCommentLine(&ofs); ofs << "---------------\n";
		m_codeLang->StartCommentLine(&ofs); ofs << "This file has similar filename for every ALHEP "<<(bCpp?"C++":"Fortran")<<" output (\"" << m_sIncludeFileName.getData() << "\").\n";
		m_codeLang->StartCommentLine(&ofs); ofs << "Use this complete set instead of any other " << m_sIncludeFileName.getData() << " file\n";
		ofs << "\n";
		m_codeLang->StartCommentLine(&ofs); ofs << "WARNING: variable name conflict is possible for unused constants\n";
		m_codeLang->StartCommentLine(&ofs); ofs << "Remove unused constants below in case of a conflict.";
		ofs << "\n";
	} else {
		if((m_IncludeUsedBy!=NULL)&&(m_IncludeUsedBy->size()>1)){
			m_codeLang->StartCommentLine(&ofs); ofs << " These constants are used by following files:\n";
			m_codeLang->StartCommentLine(&ofs); ofs << " "; int nFilesLogged=0;
			for(long n=0; n<m_IncludeUsedBy->size(); n++){
				if(nFilesLogged>=5){ ofs << "\n"; m_codeLang->StartCommentLine(&ofs); ofs << " "; nFilesLogged=0; }
				CEString* fnI = (CEString*)m_IncludeUsedBy->GetAt(n);
				if(fnI!=NULL){ ofs << fnI->getData()<<" "; }else{ ofs << "ERROR "; } nFilesLogged++;
			}
			ofs << "\n";
		}else{
			m_codeLang->StartCommentLine(&ofs); ofs << " Constants used by: " << m_file_name << "\n";
		}
		ofs << "\n";
		m_codeLang->StartCommentLine(&ofs); ofs << "Important notice.\n";
		m_codeLang->StartCommentLine(&ofs); ofs << "-----------------\n";
		m_codeLang->StartCommentLine(&ofs); ofs << "This file has similar filename for every ALHEP "<<(bCpp?"C++":"Fortran")<<" output (\"" << m_sIncludeFileName.getData() << "\").\n";
		m_codeLang->StartCommentLine(&ofs); ofs << "And it may be uniform for every "<<(bCpp?"C++":"Fortran")<<" procedure created.\n";
		m_codeLang->StartCommentLine(&ofs); ofs << "All constants used in a single ALHEP run are combined here.\n";
		m_codeLang->StartCommentLine(&ofs); ofs << "But the include-files produced in different runs may have different sets of constants.\n";
		m_codeLang->StartCommentLine(&ofs); ofs << "Then one should combine all the include files created into single one.\n";
		m_codeLang->StartCommentLine(&ofs); ofs << "Another way:\n";
		m_codeLang->StartCommentLine(&ofs); ofs << "Use \"CODE_COMPLETE_INCLUDE\" option in "<<(bCpp?"C++":"Fortran")<<" generation procedure\n";
		m_codeLang->StartCommentLine(&ofs); ofs << "to generate the include file with full set of constants.\n";
		ofs << "\n";
	}
	m_codeLang->StartCommentLine(&ofs); ofs << "**********************************************************\n";
	ofs << "\n";

	ofstream* p_ofs=&ofs; // uniform calls to other procedures
	CEString* f_line=new CEString(""); long i=0;
	if(!bCpp) { //if fortran
		// declare variables
		// **********************************	REAL*8	TEMP1, TEMP2, ...
		int num_params_in_line=10;
		if(m_conventions&CLFileManager::FT_F77)num_params_in_line=5; // free parameter
		long tmp_1=num_params_in_line-1; bool _start_line=true; int iTypePrev=0;
		if((m_general_conventions&CLFileManager::FT_F77) || (m_general_conventions&CLFileManager::FT_F90)){ //useless check
			for(i=0; i<c_num; i++) {
				CExpression* par=seqConstantsToSave->GetAt(i); if(par==NULL) { CEException::halt("seqConstantsToSave->GetAt(i)==NULL"); continue; }
				int iParType=arrConstTypes.GetAt(i);
				//End line if type is changed
				if((!_start_line) && (iTypePrev!=iParType)) { m_codeLang->OutCodeLine(p_ofs, f_line); tmp_1+=num_params_in_line; _start_line=true; }
				if(_start_line){ *f_line<<m_codeLang->GetTypePrefix(iParType)<<" ";_start_line=false;iTypePrev=iParType; }else{ *f_line<<", "; }
				(*f_line)<<par->toString();
				if ((i>=tmp_1) && (i != (c_num-1))) { m_codeLang->OutCodeLine(p_ofs, f_line); tmp_1+=num_params_in_line; _start_line=true; }
			}
			if (!_start_line) { m_codeLang->OutCodeLine(p_ofs, f_line); }
		}
	}
	// outline values
	// **********************************	PARAMETER (TEMP1 = 91.1875D0) ...
	// **********************************	const double TEMP1=91.1875D0 ...
	// **********************************	#define TEMP1 91.1875D0 ...
	for (i=0; i<c_num; i++) {
		CExpression* par=seqConstantsToSave->GetAt(i); if(par==NULL) { CEException::halt("seqConstantsToSave->GetAt(i)==NULL"); continue; }
		CEString* sym_str = par->toString();
		CEValue* val=getConstantValue(par); bool del_value=false;
		if(val==NULL){ dbg->log(0, "ERROR: constant parameter value not found, ZERO used", par); val = new CERealValue(0); del_value=true; }
		CEString* val_str=val->toString(m_STORE_EXPR_AS_FLAG); if(del_value)delete val;
		bool bOutCodeLine=true;
		if(bCpp) {
			*f_line<<"const "<<m_codeLang->GetTypePrefix(arrConstTypes.GetAt(i))<<" "<<sym_str<<" = "<<val_str;
			//#define
			//*p_ofs<<"#define "<<sym_str->getData()<<" "<<val_str->getData()<<"\n"; bOutCodeLine=false;
		} else {
			*f_line<<"parameter ("<<sym_str<<" = "<<val_str<<")";
		}
		if(bOutCodeLine){ m_codeLang->OutCodeLine(p_ofs, f_line); }else{ f_line->Reset(); }
	}

	ofs << "\n";
	m_codeLang->StartCommentLine(&ofs); ofs << "**********************************************************\n";
	ofs << "\n";
	ofs.close();
	//Nt: don't delete 'seqConstantsToSave' here
	m_bNeedSaveIncludeFile=false;
	return true;
}

/*
*	Func(par1, (int)par2, ....)
*	For internal sub-functions only ==> {required types == parameter types)
*/
CEString* CFFortran::GetSubFuncCallString(CFortranFunctionInfo* func_inf) {
	if (func_inf == NULL) return NULL;
	const char* name = func_inf->func_name();
	if (name == NULL) return new CEString("NULL");
	bool cCodeF =((m_general_conventions & (CLFileManager::FT_F77|CLFileManager::FT_F90)) != 0);

	CEPtrArray* par_list = func_inf->par_list();

	CEString* res = new CEString(); m_codeLang->AddFuncCallName(res,name); *res<<"(";
	bool use_COMMON = ((func_inf->conventions() & CLFileManager::FT_NO_COMMON_PARAMETERS)==0);
	long par_num = par_list->size(); bool need_comma=false;
	for(long i=0; i<par_num; i++) {
		CExprContainer* parInfo=(CExprContainer*)par_list->GetAt(i);
		CESymbol* par =(CESymbol*)parInfo->get_expr(); int iTypePar =parInfo->get_info();
		if (use_COMMON) { if (FindInCommonBlock(par)) continue; }
		if (need_comma) { res->AddString(", "); } else { need_comma=true; }

		if(cCodeF){ //Is cast required for Fortran? Or types are ALWAYS equal here?
			CEString sTmpBefore,sTmpAfter; if(!m_codeLang->GetTypeConversion(iTypePar,iTypePar,&sTmpBefore,&sTmpAfter,func_inf->conventions(),par,true)){ dbg->log(0,iTypePar,"[ERROR] Can't cast variable to type %ld. ",par); CEException::halt(); }
			*res<<sTmpBefore.getData(); res->AddString(par->toString()); *res<<sTmpAfter.getData();
		} else {
			res->AddString(par->toString());
		}
	}
	res->AddString(")");
	return res;
}

//may be called with 'm_IndentSize'
void CFFortran::add_function_header(ofstream* p_ofs, CFortranFunctionInfo* func_inf) { //C++ only
	if ((p_ofs == NULL) || (func_inf == NULL)) return;
	if ((m_general_conventions&CLFileManager::FT_CPP) == 0) return; //C++ only

	// **********************************	REAL*8	FUNCTION FUNC_NAME(PAR1, PAR2, ...)
	DWORD convs = func_inf->conventions(); int iResType = func_inf->res_type();
	//CEString sTmp4("Function: "); sTmp4<<iResType<<" "<<func_inf->func_name(); dbg->log(0,sTmp4.getData());

	bool use_COMMON = ((convs & CLFileManager::FT_NO_COMMON_PARAMETERS)==0);
	CEPtrArray* par_list=func_inf->par_list();
	if(iResType==CEAbstractFunction::PAR_DEFAULT)iResType=m_codeLang->GetDefaultTypeId(convs);
	const char* type_ID=m_codeLang->GetTypePrefix(iResType);
	
	CEString f_line;
	if((m_CppClassName!=NULL) && ((m_conventions & CLFileManager::FT_NON_STATIC_IMPL)==0))f_line<<"static ";
	f_line<<type_ID<<((m_general_conventions&CLFileManager::FT_CPP)?" ":" FUNCTION ");
	m_codeLang->AddFuncCallName(&f_line,func_inf->func_name()); f_line<<"(";

	long par_num = par_list->size(); // save parameters, Nt: par-list should NOT contain COMMON-items
	long i;
	for(i=0; i<par_num; i++) {
		CExprContainer* par=(CExprContainer*)par_list->GetAt(i); CExpression* sym=par->get_expr();
		if ((use_COMMON)&&(FindInCommonBlock(sym)>=0)) { dbg->log(0,"WARNING: COMMON item in par-list", sym); continue; }
		if(m_general_conventions & CLFileManager::FT_CPP){
			int iTypePar=par->get_info();
			if(iTypePar==CEAbstractFunction::PAR_DEFAULT)iTypePar=m_codeLang->GetDefaultTypeId(convs);
			f_line<<m_codeLang->GetTypePrefix(iTypePar);
			if(par->get_info()==CEAbstractFunction::PAR_4VEC_REAL)f_line<<"&"; //all vectors are currently passed as 'refs'
			f_line<<" ";
		}
		f_line.AddString(sym->toString());
		if(i<(par_num-1)) f_line<<", ";
	}
	f_line<<")";
	m_codeLang->OutCodeLine(p_ofs, &f_line);
}

void CFFortran::add_function(ofstream* p_ofs, CFortranFunctionInfo* func_inf, bool bMainEntry) {
	if ((p_ofs == NULL) || (func_inf == NULL)) return;
	int iResType =func_inf->res_type();
	m_codeLang->ClearIndent();

//	*p_ofs << "\n";
	// **********************************	REAL*8	FUNCTION FUNC_NAME(PAR1, PAR2, ...)
	DWORD convs = func_inf->conventions();
	bool use_COMMON = ((convs & CLFileManager::FT_NO_COMMON_PARAMETERS)==0);

	CEPtrArray* par_list = func_inf->par_list();
	if(iResType==CEAbstractFunction::PAR_DEFAULT)iResType=m_codeLang->GetDefaultTypeId(convs);
	const char* type_ID = m_codeLang->GetTypePrefix(iResType);

	CEString* f_line = new CEString("");
	f_line->AddString(type_ID);
	if(m_general_conventions&CLFileManager::FT_CPP){ *f_line<<" "; if(m_CppClassName!=NULL){*f_line<<m_CppClassName<<"::";} }
	else { *f_line<<" FUNCTION "; }
	m_codeLang->AddFuncCallName(f_line,func_inf->func_name()); *f_line<<"(";

	long par_num = par_list->size(); // save parameters, Nt: par-list should NOT contain COMMON-items
	long i;
	for(i=0; i<par_num; i++) {
		CExprContainer* par = (CExprContainer*)par_list->GetAt(i); CExpression* sym = par->get_expr();
		if ((use_COMMON)&&(FindInCommonBlock(sym)>=0)) { dbg->log(0,"WARNING: COMMON item in par-list", sym); continue; }
		if(m_general_conventions & CLFileManager::FT_CPP){
			int iTypePar=par->get_info();
			if(iTypePar==CEAbstractFunction::PAR_DEFAULT)iTypePar=m_codeLang->GetDefaultTypeId(convs);
			*f_line<<m_codeLang->GetTypePrefix(iTypePar);
			if(par->get_info()==CEAbstractFunction::PAR_4VEC_REAL)*f_line<<"&"; //all vectors are currently passed as 'refs'
			*f_line<<" ";
		}
		f_line->AddString(sym->toString());
		if (i < (par_num-1)) *f_line<<", ";
	}
	*f_line<<")"; if (m_general_conventions&CLFileManager::FT_CPP) { *f_line<<" {"; }
	m_codeLang->OutCodeLine(p_ofs, f_line, -1, true);
	if (m_general_conventions&CLFileManager::FT_CPP) { m_codeLang->IncIndent(); }

	// IMPLICIT NONE
	if ((m_general_conventions & CLFileManager::FT_F77) || (m_general_conventions & CLFileManager::FT_F90)) {
		*f_line << "IMPLICIT NONE"; m_codeLang->OutCodeLine(p_ofs,f_line);
	}
	if(m_general_conventions & CLFileManager::FT_CPP){ m_codeLang->StartCommentLine(p_ofs); *p_ofs<<"Result variable: "<<func_inf->func_name()<<"\n"; *f_line<<type_ID<<" "<<func_inf->func_name(); m_codeLang->OutCodeLine(p_ofs, f_line); }

	// some init
	int num_params_in_line = 10; // free parameter
	if (convs & CLFileManager::FT_F77) num_params_in_line = 5;
	long tmp_1=0; bool _start_line=true;
	//	input parameters (may exists together with COMMON-block)
	// **********************************	REAL*8	PAR1, PAR2, ...
	if ((par_num>0) && ((m_general_conventions & CLFileManager::FT_F77) || (m_general_conventions & CLFileManager::FT_F90))) {
		tmp_1=num_params_in_line-1; _start_line=true; int iTypePrev=0;
		for (i=0; i<par_num; i++) {
			CExprContainer* par=(CExprContainer*)par_list->GetAt(i);
			CExpression* sym=par->get_expr(); int iParType=par->get_info();
			if (use_COMMON) { if(FindInCommonBlock(sym)>=0){dbg->log(0,"WARNING: COMMON item in par-list",sym);continue;} }
			CEString* sym_str = sym->toString();
			if((!_start_line) && (iTypePrev!=iParType)) { //End line if type is changed
				m_codeLang->OutCodeLine(p_ofs, f_line); tmp_1+=num_params_in_line; _start_line=true;
			}
			if(_start_line){
				int iParTypeFixed=iParType; if(iParTypeFixed==CEAbstractFunction::PAR_DEFAULT)iParTypeFixed=m_codeLang->GetDefaultTypeId(convs);
				*f_line<<m_codeLang->GetTypePrefix(iParTypeFixed)<<" "; _start_line=false; iTypePrev=iParType;
			}else{ *f_line<<", "; }
			f_line->AddString(sym_str);
			if ((i>=tmp_1) && (i != (par_num-1))) {
				m_codeLang->OutCodeLine(p_ofs, f_line); tmp_1+=num_params_in_line; _start_line=true;
			}
		}
		if (!_start_line) { m_codeLang->OutCodeLine(p_ofs, f_line); }
		*p_ofs<<"\n";
	}

	//	COMMON-block
	if (use_COMMON) { add_CommonBlock(p_ofs, f_line, m_codeLang->GetTypePrefixAdv(convs), num_params_in_line); *p_ofs<<"\n"; }

	//	functions used
	// **********************************	REAL*8	FUNC1, FUNC2, ...
	CEPtrArray* func_used_list = func_inf->funct_used_list();
	if ((func_used_list!=NULL) && ((m_general_conventions&CLFileManager::FT_F77)||(m_general_conventions&CLFileManager::FT_F90))) {
		tmp_1=num_params_in_line-1;
		int func_used_num = func_used_list->size(); _start_line=true; int iTypePrev=0;
		for (i=0; i<func_used_num; i++) {
			CEObjIdCart* fCart = (CEObjIdCart*)func_used_list->GetAt(i);
			CEString* sym_str = (CEString*)fCart->GetObj(); int iParType=fCart->GetId();

			if((!_start_line) && (iTypePrev!=iParType)) { //End line if type is changed
				m_codeLang->OutCodeLine(p_ofs, f_line); tmp_1+=num_params_in_line; _start_line=true;
			}

			if (_start_line) {
				int iParTypeFixed=iParType; if(iParTypeFixed==CEAbstractFunction::PAR_DEFAULT)iParTypeFixed=m_codeLang->GetDefaultTypeId(convs);
				*f_line<<m_codeLang->GetTypePrefix(iParTypeFixed)<<" "; _start_line=false; iTypePrev=iParType;
			}else{ *f_line<<", "; }
			f_line->AddString(sym_str->getData());
			if ((i>=tmp_1) && (i != (func_used_num-1))) {
				m_codeLang->OutCodeLine(p_ofs, f_line); tmp_1 += num_params_in_line; _start_line=true;
			}
		}
		if (!_start_line) { m_codeLang->OutCodeLine(p_ofs, f_line->getData()); f_line->Reset(); }
		*p_ofs << "\n";
	}

	// constants
	CEPtrArray* const_list=func_inf->consts_list(); int c_num=((const_list!=NULL)?const_list->size():0);
	if (c_num>0) {
		if ((convs & CLFileManager::FT_NO_INCLUDES) == CLFileManager::FT_NO_INCLUDES) {
			// **********************************	REAL*8	TEMP1, TEMP2, ...
			// **********************************	PARAMETER (TEMP1 = 91.1875D0) ...
			tmp_1=num_params_in_line-1;_start_line=true; int iTypePrev=0;
			if((m_general_conventions&CLFileManager::FT_F77) || (m_general_conventions&CLFileManager::FT_F90)){
				for (i=0; i<c_num; i++) {
					CExprContainer* sCart= (CExprContainer*)const_list->GetAt(i);
					//asdf: [GetBasicConstSym] is power allowed here?
					CExpression* sym=GetBasicConstSym(sCart->get_expr()); if(sym==NULL){ dbg->log(0,"Internal ERROR: constant expected",const_list->GetAt(i)); }
					if (use_COMMON) { if(FindInCommonBlock(sym)>=0){dbg->log(0,"WARNING: COMMON item in const-list",sym);continue;} }
					long iParType=sCart->get_info();
					if((!_start_line) && (iTypePrev!=iParType)) { //End line if type is changed
						m_codeLang->OutCodeLine(p_ofs, f_line); tmp_1+=num_params_in_line; _start_line=true;
					}
					if(_start_line){
						int iParTypeFixed=iParType; if(iParTypeFixed==CEAbstractFunction::PAR_DEFAULT)iParTypeFixed=m_codeLang->GetDefaultTypeId(convs);
						*f_line<<m_codeLang->GetTypePrefix(iParTypeFixed)<<" "; _start_line=false; iTypePrev=iParType;
					}else{ *f_line<<", "; }
					f_line->AddString(sym->toString());
					if ((i>=tmp_1) && (i != (c_num-1))) { m_codeLang->OutCodeLine(p_ofs, f_line); tmp_1+=num_params_in_line; _start_line=true; }
				}
				if(!_start_line) m_codeLang->OutCodeLine(p_ofs, f_line);
			}

			// outline values
			for (i=0; i<c_num; i++) {
				CExprContainer* sCart= (CExprContainer*)const_list->GetAt(i);
				//asdf: [GetBasicConstSym] is power allowed here?
				CExpression* par=GetBasicConstSym(sCart->get_expr()); if(par==NULL){ dbg->log(0,"Internal ERROR: constant expected",const_list->GetAt(i)); }
				if (use_COMMON) { if(FindInCommonBlock(par)>=0){dbg->log(0,"WARNING: COMMON item in const-list",par);continue;} }
				CEString* sym_str = par->toString();
				CEValue* val = getConstantValue(par); bool del_value=false;
				if (val == NULL) { dbg->log(0, "ERROR: constant parameter value not found, ZERO used", par); val = new CERealValue(0); del_value=true; }
				CEString* val_str = val->toString(m_STORE_EXPR_AS_FLAG);
				if (del_value) delete val;
				if(m_general_conventions&CLFileManager::FT_CPP){
					int iTypeTmp=sCart->get_info(); if(iTypeTmp==CEAbstractFunction::PAR_DEFAULT)iTypeTmp=m_codeLang->GetDefaultTypeId(convs);
					*f_line<<"const "<<m_codeLang->GetTypePrefix(iTypeTmp)<<" "<<sym_str<<" = "<<val_str;
				}
				else{ *f_line<<"parameter ("<<sym_str<<" = "<<val_str<<")"; }
				m_codeLang->OutCodeLine(p_ofs, f_line);
			}
			*p_ofs << "\n";
		} else {
			// Note: (c_num>0) here
			int nConstFromIncludeFile=0;
			for (i=0; i<c_num; i++) {
				CExprContainer* sCart= (CExprContainer*)const_list->GetAt(i); CConstRegistry::CConstInfo* constInfo=NULL;
				//asdf: [GetBasicConstSym] is power allowed here?
				CExpression* par=GetBasicConstSym(sCart->get_expr(),NULL,&constInfo); if(par==NULL){ dbg->log(0,"Internal ERROR: constant expected",const_list->GetAt(i)); }
				if (use_COMMON) { if(FindInCommonBlock(par)>=0){dbg->log(0,"WARNING: COMMON item in const-list",par);continue;} }
				if(constInfo==NULL) { dbg->log(0,"Internal ERROR: Can't find information on constant",sCart); CEException::halt(); continue; }

				//Add value to include file list (if not added previously or declared elsewhere)
				if(constInfo->m_iPath == CConstRegistry::C_INCLUDED) {
					nConstFromIncludeFile++;
					AddConstantToIncludeList(par);
				}
			}

			if(nConstFromIncludeFile>0) {
				DeclareIncludeFileRequired();			// update include file after this procedure
				//[wrong check for in-function includes!]if(!CFFortran::IsConstFileCreatedFor(m_file_name)){
				{
														// add 'include' line if not done before
					m_codeLang->StartCommentLine(p_ofs); *p_ofs << "Include constants (" << nConstFromIncludeFile << " value(s) used)\n";
					if((m_general_conventions&CLFileManager::FT_F77) || (m_general_conventions&CLFileManager::FT_F90)){
						*f_line<<"include \""<<m_sIncludeFileName.getData()<<"\""; m_codeLang->OutCodeLine(p_ofs, f_line);
					} else {
						*p_ofs<<"#include \""<<m_sIncludeFileName.getData()<<"\"\n";
					}
					*p_ofs << "\n";
				}
			}
		}

		if(m_ConstantsPowersList.size() > 0) {
			CEString sTmp1,sTmp2;
			int iConstPwrAdded=0;
			for (i=0; i<c_num; i++) { //Search for 'powered' constants
				CExprContainer* sCart= (CExprContainer*)const_list->GetAt(i);
				CExpression* symX= sCart->get_expr(); CConstRegistry::CConstInfo* constInfo=NULL; long iPow=-1; long iTypeConstPow;
				CExpression* symConst=GetBasicConstSym(symX,&iPow,&constInfo,&iTypeConstPow); if((symConst==NULL)||(symConst==symX))continue;
				CExpression* symPow=symX;
				if(iConstPwrAdded==0) { m_codeLang->StartCommentLine(p_ofs); *p_ofs << "Power of constants used\n"; }
				// **********************************	REAL*8	TEMP1_PN
				CEString* symPow_str = symPow->toString();
				//Note: power's type may be different to original constant type! E.g.: realX eP2=pow((realX)e,2);
				long iTypeConst=sCart->get_info(); //Note: same to =constInfo->m_iType;
				if(iTypeConst==CEAbstractFunction::PAR_DEFAULT)iTypeConst=m_codeLang->GetDefaultTypeId(convs);
				if(iTypeConstPow==CEAbstractFunction::PAR_DEFAULT)iTypeConstPow=m_codeLang->GetDefaultTypeId(convs);

				const char* sType = m_codeLang->GetTypePrefix(iTypeConstPow);

				if((m_general_conventions&CLFileManager::FT_F77) || (m_general_conventions&CLFileManager::FT_F90)){
					*f_line<<sType<<" "<<symPow_str->clone(); m_codeLang->OutCodeLine(p_ofs, f_line);
				}
				// **********************************	TEMP1PN = TEMP1**N...
				CEString* symConst_str = symConst->toString();
				//symPow_str = pow(symConst_str,iPow);
				if(!m_codeLang->GetTypeConversion(iTypeConst,iTypeConstPow,&sTmp1,&sTmp2,convs,symConst,true)) { dbg->log(0,iTypeConst,iTypeConstPow,"Internal ERROR: Can't cast constant value to constant power"); CEException::halt(); }
				if(m_general_conventions&CLFileManager::FT_CPP){
					*f_line<<"const "<< sType <<" "<<symPow_str<<" = "<<CCodeLanguage::CPP_POW_DEF_SYM<<"("<<sTmp1.getData()<<symConst_str<<sTmp2.getData()<<","<<iPow<<")";
				} else {
					*f_line<<"parameter ("<<symPow_str<<" = "<<sTmp1.getData()<<symConst_str<<sTmp2.getData()<<"**"<<iPow<<")";
				}
				m_codeLang->OutCodeLine(p_ofs, f_line); iConstPwrAdded++;
			}
			if(iConstPwrAdded>0)*p_ofs << "\n";
		}
	}

	// temp variables
	// **********************************	REAL*8	TEMP1, TEMP2, ...
	CEPtrArray* temp_var_list = func_inf->temp_var_list();
	if (temp_var_list != NULL) {
		tmp_1=num_params_in_line-1; _start_line=true; int iTypePrev=0;
		int tmp_num = temp_var_list->size();
		for (i=0; i<tmp_num; i++) {
			CEObjIdCart* tCart = (CEObjIdCart*)temp_var_list->GetAt(i);
			CEString* sym_str = (CEString*)tCart->GetObj(); int iParType=tCart->GetId();
			if((!_start_line)&&(iTypePrev!=iParType)){ //End line if type is changed
				m_codeLang->OutCodeLine(p_ofs, f_line); tmp_1+=num_params_in_line; _start_line=true;
			}
			if(_start_line){
				int iParTypeFixed=iParType; if(iParTypeFixed==CEAbstractFunction::PAR_DEFAULT)iParTypeFixed=m_codeLang->GetDefaultTypeId(convs);
				*f_line<<m_codeLang->GetTypePrefix(iParTypeFixed)<<" "; _start_line=false; iTypePrev=iParType;
			}else{ *f_line<<", "; }
			f_line->AddString(sym_str->getData());
			if ((i>=tmp_1) && (i != (tmp_num-1))) { m_codeLang->OutCodeLine(p_ofs, f_line); tmp_1+=num_params_in_line; _start_line=true; }
		}
		if (!_start_line) { m_codeLang->OutCodeLine(p_ofs, f_line); }
		*p_ofs << "\n";
	}

	// include looptools.h to FORTRAN code
	if (convs & CLFileManager::FT_LOOPTOOLS) {
		if((m_general_conventions&CLFileManager::FT_F77)||(m_general_conventions&CLFileManager::FT_F90)){ *p_ofs<<"#include \"looptools.h\"\n\n"; }
	}

	// debug variables
	if((bMainEntry) && (m_general_conventions&CLFileManager::FT_CPP)) {
		*f_line<<"m_nCalls++"; m_codeLang->OutCodeLine(p_ofs, f_line); *p_ofs<<"\n";
	}

	// implementetion
	// ************************************************** precalculations
	CEPtrArray* precalc_list = func_inf->precalc_list();
	if (precalc_list != NULL) {
		int pre_calc_num = precalc_list->size();
		for (i=0; i<pre_calc_num ; i++) {
			CEString* str = (CEString*)precalc_list->get(i);
			m_codeLang->OutCodeLine(p_ofs, str->getData());
		}
		*p_ofs << "\n";
	}

	// ************************************************** result
	CEString* res_val =func_inf->res_value();
	if(res_val==NULL){ dbg->log(0, "Invalid fuinction storage. (res == NULL)."); }
	*f_line<<func_inf->func_name()<<" = "<<((res_val!=NULL)?res_val->getData():"ERROR"); m_codeLang->OutCodeLine(p_ofs, f_line);
	//C++ Note. We should not simply "return res_val->getData()" due to:
	//'func_inf->func_name()' may be & may not be used before ==> unknown if we need to allocate it.
	// ==> Let's declare 'func_inf->func_name()' variable and use it at least here!
	if(m_general_conventions & CLFileManager::FT_CPP){ *f_line<<"return "<<func_inf->func_name(); m_codeLang->OutCodeLine(p_ofs, f_line); }

	if (m_general_conventions & CLFileManager::FT_CPP) { m_codeLang->DecIndent(); }
	*f_line<<m_codeLang->GetOpEndProc(); m_codeLang->OutCodeLine(p_ofs, f_line, -1, true);
	//*p_ofs << "\n";
	delete f_line;
}

void CFFortran::add_CommonBlock(ofstream* p_ofs, CEString* f_line, const char* type_ID, int num_params_in_line) {
	if ((p_ofs == NULL) || (f_line==NULL)) return;
	long comm_num = ((m_CommonBlockSyms!=NULL)?(m_CommonBlockSyms->size()):0);
	if (comm_num>0) {
		long i;
		// **********************************	REAL*8	FUNC1, FUNC2, ...
		long tmp_1=num_params_in_line-1; bool _start_line=true;
		if (m_CommonArraySize>0) { *f_line<<type_ID<<" "<<m_COMMON_ARRAY_TITLE; _start_line=false; }
		for (i=m_CommonArraySize; i<comm_num; i++) {
			CEString* sym_str = (GetCommonSym(i))->toString();
			if(_start_line){ *f_line<<type_ID<<" "; _start_line=false; }else{ *f_line<<", "; }
			f_line->AddString(sym_str);
			if ((i>=tmp_1) && (i != (comm_num-1))) { m_codeLang->OutCodeLine(p_ofs, f_line); tmp_1+=num_params_in_line; _start_line=true; }
		}
		if (!_start_line) { m_codeLang->OutCodeLine(p_ofs, f_line); }

		// **********************************	COMMON/AL.../VscP2
		//tmp_1=num_params_in_line-1; // No line breaking
		_start_line=true;
		if (m_CommonArraySize>0) {
			*f_line<<"COMMON/"<<((m_COMMONBLOCK_NAME!=NULL)?m_COMMONBLOCK_NAME:COMMON_BLOCK_TITLE)<<"/"<<m_COMMON_ARRAY_TITLE<<"("<<m_CommonArraySize<<")";
			_start_line=false;
		}
		for (i=m_CommonArraySize; i<comm_num; i++) {
			CEString* sym_str = (GetCommonSym(i))->toString();
			if (_start_line) {
				*f_line<<"COMMON/"<<((m_COMMONBLOCK_NAME!=NULL)?m_COMMONBLOCK_NAME:COMMON_BLOCK_TITLE)<<"/";
				_start_line=false;
			} else { *f_line<<", "; }
			f_line->AddString(sym_str);
			// No line breaking
			//if ((i>=tmp_1) && (i != (comm_num-1))) { m_codeLang->OutCodeLine(p_ofs, f_line); tmp_1 += num_params_in_line; _start_line=true; }
		}
		if (!_start_line) { m_codeLang->OutCodeLine(p_ofs, f_line); }
	}
}

CEString* CFFortran::get_ItemStr(CExpression* val, bool chk_COMMON) { // possible COMMON value or toString(), never NULL, no sign allowed
	long pos_COMMON=FindInCommonBlock(val);
	CEString* res=NULL;
	if ((pos_COMMON>=0) && (pos_COMMON<m_CommonArraySize)) {
		res = get_CommonItemStr(pos_COMMON);
	} else {
		res = val->toString(m_STORE_EXPR_AS_FLAG);
	}
	return res;
}

CEString* CFFortran::get_CommonItemStr(long i) {
	long comm_num = ((m_CommonBlockSyms!=NULL)?(m_CommonBlockSyms->size()):0);
	if ((i<0)||(i>=comm_num)) return NULL;
	CEString* res=NULL;
	if (i<m_CommonArraySize) {
		res = new CEString(m_COMMON_ARRAY_TITLE); *res<<"("<<(i+1)<<")";
	} else {
		res = (GetCommonSym(i))->toString(m_STORE_EXPR_AS_FLAG);
	}
	return res;
}

void CFFortran::add_vec_entry(ofstream* p_ofs, const char* _EntryTitle, CEPtrArray* cpl_data, CFortranFunctionInfo* func_inf, bool bHeaderOnly) { //ADDVEC
	if ((p_ofs == NULL) || (func_inf == NULL)) CEException::halt(dbg, "add_vec_entry(): invalid data");
	if(!bHeaderOnly)m_codeLang->ClearIndent();
	if (cpl_data == NULL) dbg->log(0, "CFFortran::add_vec_entry WARNING: NULL couplings data");
//	*p_ofs << "\n";
	DWORD convs = func_inf->conventions();
	bool use_COMMON = ((convs & CLFileManager::FT_NO_COMMON_PARAMETERS)==0);
	CEString* f_line = new CEString("");
	bool PY_vectors = ((convs & CLFileManager::FT_PYTHIA_VECTORS)==CLFileManager::FT_PYTHIA_VECTORS);
	bool _Cmplx = ((convs & CLFileManager::FT_COMPLEX_16BIT)==CLFileManager::FT_COMPLEX_16BIT);
	bool lt_calls = ((convs & CLFileManager::FT_LOOPTOOLS) == CLFileManager::FT_LOOPTOOLS);
	CExpression* lt_UV_val=NULL; CExpression* lt_IR_val=NULL;
	CDiagramPhysics* phys = CDiagramPhysics::get_curr_physics(); if (phys==NULL) { dbg->log(0,"NULL physics error"); return; }
	CExpression* DR_mass = phys->get_DR_mass_sym(); //int DR_mass_pow=-1;
	CExpression* DR_mass2 = NULL; CExpression* DR_mass2_F = NULL;
	if (DR_mass!=NULL) {
		DR_mass = DR_mass->clone();
		if (DR_mass->ClassName()==CESymbol::CLASS_NAME) {
			CESymbol* DR_massSym = validate_sym((CESymbol*)DR_mass,NULL);
			CEString* tmp_DR = DR_massSym->toString(m_STORE_EXPR_AS_FLAG); tmp_DR->AddString("P2");
			DR_mass2 = new CESymbol(tmp_DR->getData()); delete tmp_DR;
			if (DR_massSym != DR_mass) delete DR_massSym;
		}
		delete DR_mass;
	}

	//Parameter
	bool bCppEventEntry=true; // (CVMEvtLite* evt) instead of (CVM4Vec* p1, CVM4Vec* p2,...)
	bool bNoKinVecInParams=PY_vectors; // exclude kinematic vectors from parameters
	if ((m_general_conventions&CLFileManager::FT_CPP) && bCppEventEntry) { bNoKinVecInParams=true; }

	// some init
	long num_params_in_line = 10; // free parameter //TODO: used only in COMMON-save now --> unify this procedure to add_function()
	if (convs & CLFileManager::FT_F77) num_params_in_line = 5;
	int iResType = func_inf->res_type();
	if(iResType==CEAbstractFunction::PAR_DEFAULT)iResType=m_codeLang->GetDefaultTypeId(convs);
	const char* type_ID = m_codeLang->GetTypePrefix(iResType);			//function result type
	const char* type_Default = m_codeLang->GetTypePrefixAdv(convs);		//unknown parameters type
	long tmp_1=0; bool _start_line=true;
	long dens_check_num=0;
	if ((m_conventions & CLFileManager::FT_CHECK_DENOMINATORS) && (m_Dens_functions != NULL)) { dens_check_num = m_Dens_functions->size(); }
	CKinematics* kin = CEProcessor::kinematics();
	bool _AllowQuarksSwap=false;							//allowed if ...
	const char* QOrig="kQOrig"; const char* Q1Orig="kQ1Orig"; const char* Q2Orig="kQ2Orig";
	if (((m_conventions & CLFileManager::FT_ALLOW_SWAP_INITIALS) == CLFileManager::FT_ALLOW_SWAP_INITIALS) &&
		(kin != NULL)) {
		if (kin->num_init_particles()==2) {
			_AllowQuarksSwap=true; //: int kQOrig ==1||2
			//[old check replaced by 'FT_ALLOW_SWAP_INITIALS' option:]
			//if (((kin->pin(0)->check_PK(CEParticle::PARTICLE_QUARK,true)) ||
			//     (kin->pin(0)->check_PK(CEParticle::PARTICLE_GLUON,true))) &&
			//    ((kin->pin(1)->check_PK(CEParticle::PARTICLE_QUARK,true)) ||
			//     (kin->pin(1)->check_PK(CEParticle::PARTICLE_GLUON,true))) &&
			//    (kin->pin(0)->get_kind() != kin->pin(1)->get_kind())) {
			//}
		}
	}
	CEString* s_EntryTitle=NULL;
	if ((_EntryTitle!=NULL) && (CEUtil::equals(_EntryTitle,func_inf->func_name()))) { dbg->log(0, "Function title is CHANGED, CHECK CODE!"); _EntryTitle=NULL; }
	if (_EntryTitle == NULL) {
		s_EntryTitle = new CEString(func_inf->func_name()); s_EntryTitle->AddString("_VEC=");
		_EntryTitle=s_EntryTitle->getData();
	}
	CEString s_ReturnVar(_EntryTitle); if(m_general_conventions&CLFileManager::FT_CPP){ s_ReturnVar.AddString("Res"); }
	bool src_ampl = ((m_conventions & CLFileManager::FT_SRC_IS_AMPLITUDE) == CLFileManager::FT_SRC_IS_AMPLITUDE);

	// extract couplings from par_list, compose 4-vectors list
	CEPtrArray* par_list = (CEPtrArray*)func_inf->par_list()->clone();
	CSequence* vec_list = new CSequence();
	//cpl_data = CEPtrArray::get_copy(cpl_data); // clone data & destroy it
	long cpl_num = (cpl_data!=NULL)?(cpl_data->size()):0;
	long i; long j; long s;
	bool _used_sp4=false; bool _used_aPr=false; bool _used_fBond=false;
	dbg->log(45, "cpl_data", cpl_data);
	for(i=0; i<cpl_num; i++) {
		CExprContainer* cart = (CExprContainer*)cpl_data->GetAt(i);
		CExpression* cpl_i = cart->get_expr(); // coupling
		CExpression* r = cart->get_info_expr(); // replacement

		// extract r from params
		bool r_found=false;
		for(long iTmp=0; iTmp<par_list->size(); iTmp++) {
			CExpression* itm = ((CExprContainer*)par_list->GetAt(iTmp))->get_expr();
			if (r->equals(itm)) { par_list->RemoveAt(iTmp,true); r_found=true; break; } // remove the only value
		}
		if (!r_found) { if ((use_COMMON)&&(FindInCommonBlock(r)<0)) {dbg->log(0, "Warning: coupling not found in params or COMMON block", r, par_list); continue;} }

		// add vectors to list
		if(CEUtil::equals(cpl_i->ClassName(), CEAntiSymmProd::CLASS_NAME)) {
			CEAntiSymmProd* a_pr = (CEAntiSymmProd*)cpl_i; _used_aPr=true;
			vec_list->AddExprIfUnique(a_pr->get_v(0));
			vec_list->AddExprIfUnique(a_pr->get_v(1));
			vec_list->AddExprIfUnique(a_pr->get_v(2));
			vec_list->AddExprIfUnique(a_pr->get_v(3));
		} else if(CEUtil::equals(cpl_i->ClassName(), CESProduct::CLASS_NAME)) {
			CESProduct* c = (CESProduct*)cpl_i; _used_sp4=true;
			vec_list->AddExprIfUnique(c->get_a());
			vec_list->AddExprIfUnique(c->get_b());
		} else if(CEUtil::equals(cpl_i->ClassName(), CEFuncBondarev::CLASS_NAME)) {
			CEFuncBondarev* fb = (CEFuncBondarev*)cpl_i; _used_fBond=true;
			CExpression* c=fb->GetCouple1(); if(c!=NULL)vec_list->AddExprIfUnique(c);
			c=fb->GetCouple2(); if(c!=NULL)vec_list->AddExprIfUnique(c);
		} else { dbg->log(0, "CFFortran::add_vec_entry, unexpected coupling spotted", cpl_i); continue; }
	}

	// extract vector parameters from par_list and add them to 4-vectors list
	for(i=0; i<par_list->size(); i++) {
		CExprContainer* par = (CExprContainer*)par_list->GetAt(i);
		if(par->get_info() != CEAbstractFunction::PAR_4VEC_REAL) continue;
		vec_list->AddExprIfUnique(par->get_expr());
		par_list->RemoveAt(i,true); i--; //extract from params
	}

	// vectors list composed
	long vec_num = vec_list->size();
	dbg->log(45, "vec_list", vec_list);

	// sort vectors by kinematics
	CSequence* vec_par_list = new CSequence(); // 0. vector parameters
	CSequence* vec_mom_list = new CSequence(); // 1. external momenta
	CSequence* vec_pol_list = new CSequence(); // 2. external polarizations
	CSequence* ave_pol_list = new CSequence(); // 2.1. external polarizations to be averaged
	CSequence* vec_basisA_list = new CSequence(); // 3.1. basis vectors (Amplitudes)
	CSequence* vec_basisB_list = new CSequence(); // 3.2. basis vectors (Bondarev)
	CSequence* vec_unkn_list = new CSequence(); // N. unknown vectors
	for(i=0; i<vec_num; i++) {
		CExpression* vec = vec_list->GetAt(i);
		if (kin != NULL) {
			// TODO: fix here
			int pID = kin->find_momentum(GetSymOrigin(vec));
			if (pID != 0) {
				vec_mom_list->addItem(vec->clone());
				if (!bNoKinVecInParams) { vec_par_list->addItem(vec->clone()); }
				continue;
			}
			pID = kin->find_polarization_vec(GetSymOrigin(vec));
			if (pID != 0) {
				vec_pol_list->addItem(vec->clone());
				if (kin->get_particle_by_ID(pID)->get_polarization_info() == 0) {
					CEParticle* part = kin->get_particle_by_ID(pID);
					CExpression* mom_orig = part->get_momentum_symbol();
					if (mom_orig->ClassName() != CESymbol::CLASS_NAME) {
						dbg->log(0, "ERROR: NON-SYMBOL momentum found, CHECK CODE", mom_orig);
					} else {
						CESymbol* mom_F = validate_sym((CESymbol*)mom_orig,m_syms_replaced);
						if (vec_list->get_item_pos(mom_F)<0) {vec_list->addItem(mom_F->clone());vec_num++;} // add momentum to vectors list
						if(mom_F != mom_orig) delete mom_F;
					}
					ave_pol_list->addItem(vec->clone());
				} else {
					vec_par_list->addItem(vec->clone());
				}
				continue;
			}
		}
		if (vec->ClassName() == CESymbol::CLASS_NAME) {
			int bas_I = CLHEPBasics::isBasisSymbol((CESymbol*)GetSymOrigin(vec));
			if (bas_I>0) {
				if (bas_I == 1) { vec_basisA_list->addItem(vec->clone()); }
				else if (bas_I == 2) { vec_basisB_list->addItem(vec->clone()); }
				else { vec_par_list->addItem(vec->clone()); }
				continue;
			}
		}
		// check for basis vectors
		vec_unkn_list->addItem(vec->clone());
		vec_par_list->addItem(vec->clone());
	}
	if (vec_unkn_list->size() > 0) {
		dbg->log(0, "WARNING: unknown vectors spotted, treated as parameters", vec_unkn_list);
	}

	// EQU list: a=b items (CExprContainer::get_expr() == CExprContainer::get_info_expr())
	CEPtrArray* EQUList = new CEPtrArray();
	CSequence* TmpList = new CSequence(); // additional temporary variables
	CSequence TmpListReal; // real temporary variables

	// check par-list values for constants
	long par_num = par_list->size(); // Nt: par-list should NOT contain COMMON-items
	for(i=0; i<par_num; i++) {
		CExprContainer* par = (CExprContainer*)par_list->GetAt(i);
		CExpression* sym = par->get_expr();
		CExpression* sym_orig = GetSymOrigin(sym);
		if (sym_orig->equals(CAmplCalcLib::GetBasisMassSym())) {
			CExpression* val= new CERealValue(1000); EQUList->add(new CExprContainer(sym,0,0,val)); delete val;
			TmpList->addItem(sym->clone());
			par_list->RemoveAt(i,true); i--; par_num--;
			continue;
		}
		if (lt_calls) {
			if (sym_orig->equals(CVIScalarCalcLib::getUVSymbol())) {
				lt_UV_val=sym->clone();
				if(m_general_conventions&CLFileManager::FT_CPP){ TmpListReal.addItem(sym->clone()); }else{ TmpList->addItem(sym->clone()); }
				par_list->RemoveAt(i,true); i--; par_num--; continue;
			}
			if (sym_orig->equals(CVIScalarCalcLib::getIRSymbol())) {
				lt_IR_val=sym->clone();
				if(m_general_conventions&CLFileManager::FT_CPP){ TmpListReal.addItem(sym->clone()); }else{ TmpList->addItem(sym->clone()); }
				par_list->RemoveAt(i,true); i--; par_num--; continue;
			}
			if ((DR_mass2!=NULL) && (sym_orig->equals(DR_mass2))) {
				DR_mass2_F=sym->clone();
				if(m_general_conventions&CLFileManager::FT_CPP){ TmpListReal.addItem(sym->clone()); }else{ TmpList->addItem(sym->clone()); }
				par_list->RemoveAt(i,true); i--; par_num--; continue;
			}
		}
	}

	// List items exist in COMMON-block, but absent in couplings-list, and init them via parameters
	long comm_num = ((m_CommonBlockSyms!=NULL)?(m_CommonBlockSyms->size()):0);
	CSequence* NonVecCOMM_list = new CSequence(); // non-vector items in COMMON block (--> to parameters)
	long non_vec_size=0;
	if ((use_COMMON) && (comm_num>0)) {
		for (i=0; i<comm_num; i++) {
			CExpression* sym = GetCommonSym(i);
			bool in_cpls=false; // check existance in "couplings list" 
			for(j=0; j<cpl_num; j++) { if(sym->equals(((CExprContainer*)cpl_data->GetAt(j))->get_info_expr())){in_cpls=true;break;} }
			if (!in_cpls) {
				NonVecCOMM_list->addItem(sym->clone());

				CExpression* sym_X=sym;
				if(i<m_CommonArraySize){ CEString* sym_str=get_CommonItemStr(i); sym_X=new CESymbol(sym_str->getData()); delete sym_str; }
				bool EQU_done=false;

				// check value for constant
				CExpression* sym_orig = GetSymOrigin(sym);
				if (sym_orig->equals(CAmplCalcLib::GetBasisMassSym())) {
					CExpression* val= new CERealValue(1000); EQUList->add(new CExprContainer(sym_X,0,0,val)); delete val; EQU_done=true;
				}

				if (lt_calls) {
					if (sym_orig->equals(CVIScalarCalcLib::getUVSymbol())) {
						lt_UV_val=sym_X->clone(); EQU_done=true;
						//TmpList->addItem(sym->clone());
						//CExpression* val= new CERealValue(1000); EQUList->add(new CExprContainer(sym_X,0,0,val)); delete val; EQU_done=true;
					}
					//else if ((GetSymOrigin(sym))->equals(CVIScalarCalcLib::getIRSymbol())) {
					//	lt_IR_val=sym_X->clone(); EQU_done=true;
					//}
					if ((DR_mass2!=NULL) && (sym_orig->equals(DR_mass2))) {
						DR_mass2_F=sym_X->clone(); EQU_done=true;
					}
				}

				if (!EQU_done) {
					// create a=PAR_a initialization for every NonVecCOMM_list-item
					CEString* sym_str = new CEString("PAR_"); sym_str->AddString(sym->toString());
					CExpression* par = new CESymbol(sym_str->getData()); delete sym_str;
					EQUList->add(new CExprContainer(sym_X,0,0,par)); EQU_done=true;
					par_list->add(new CExprContainer(par,m_codeLang->GetDefaultTypeId(convs)));
					delete par;
				}
				if(sym_X!=sym)delete sym_X;
			}
		}
		non_vec_size=NonVecCOMM_list->size();
	}

	// sort vec_par_list
	for(i=0; i<vec_par_list->size(); i++) {
		CExpression* item_i = vec_par_list->GetAt(i);
		bool loop = true;
		for (long j=i-1; ((j>=0) && (loop)); j--) { // check right order first
			CExpression* item_j = vec_par_list->GetAt(j);
			if (!item_i->less_than(item_j)) { if(i>(j+1))vec_par_list->move(i, j+1); loop=false; }
		}
		if (loop) { vec_par_list->move(i, 0); }
	}

	if(!bHeaderOnly) {
		dbg->log(45, "vec_mom_list", vec_mom_list);
		dbg->log(45, "vec_par_list", vec_par_list);
		dbg->log(45, "vec_pol_list", vec_pol_list);
		dbg->log(45, "ave_pol_list", ave_pol_list);
		dbg->log(45, "vec_basisA_list", vec_basisA_list);
		dbg->log(45, "vec_basisB_list", vec_basisB_list);
		dbg->log(45, "TmpList", TmpList);
		dbg->log(45, "TmpListReal", &TmpListReal);
		dbg->log(45, "EQUList", EQUList);
		dbg->log(45, "NonVecCOMM_list", NonVecCOMM_list);
		dbg->log(45, "par_list", par_list);
		dbg->log(45, "vec_unkn_list", vec_unkn_list);
	}
	// fill kin-ID values for 'vec_mom_list' vectors
	int pID_vec_mom_list[100]; s=vec_mom_list->size();
	for(i=0; i<s; i++) {
		CExpression* vec = vec_mom_list->GetAt(i);
		pID_vec_mom_list[i] = kin->find_momentum(GetSymOrigin(vec)); // >0 means initials
		if (pID_vec_mom_list[i]>2) { dbg->log(0, pID_vec_mom_list[i], "WARNING, more than 2 inilial vectors spotted, CHECK OUTPUT CODE"); }
	}

	//TODO: arrange vectors order according to kinematics
	//TODO: OR: extract vectors from kinematics
	//TODO: SORT cpl_data according to COMMON block sequence

	// **********************************	REAL*8	FUNCTION FUNC_NAME(PAR1, PAR2, ...)
	if(bHeaderOnly && ((m_conventions & CLFileManager::FT_NON_STATIC_IMPL)==0))f_line->AddString("static ");
	f_line->AddString(type_ID);
	if(m_general_conventions&CLFileManager::FT_CPP){ f_line->AddString(" "); if((!bHeaderOnly)&&(m_CppClassName!=NULL)){ f_line->AddString(m_CppClassName); f_line->AddString("::"); } }
	else{ f_line->AddString(" FUNCTION "); }
	*f_line<<_EntryTitle<<"("; bool need_comma=false;
	if((m_general_conventions&CLFileManager::FT_CPP) && bCppEventEntry && bNoKinVecInParams) { //(CHEPEvtLite* evt)
		if(need_comma){f_line->AddString(", ");}else{need_comma=true;} f_line->AddString(CCodeProcessImpl::CN_CPP_EVT_CLASS); f_line->AddString("* evt");
	}
	//add 'QOrig' parameter for quarks if required
	if (_AllowQuarksSwap) {
		if(need_comma){f_line->AddString(", ");}else{need_comma=true;}
		if(m_general_conventions&CLFileManager::FT_CPP){ f_line->AddString(m_codeLang->GetTypePrefixInt()); f_line->AddString(" "); }
		f_line->AddString(QOrig);
	}
	//
	long vec_par_num = vec_par_list->size();
	for(i=0; i<vec_par_num; i++) {
		CEString* sym_str = ((CESymbol*)vec_par_list->GetAt(i))->toString();
		if(need_comma){ *f_line<<", "; }else{ need_comma=true; }
		if(m_general_conventions&CLFileManager::FT_CPP){*f_line<<CCodeProcessImpl::CN_CPP_4VEC_CLASS<<"* ";}//CVM4Vec*
		*f_line<<sym_str;
	}
	par_num = par_list->size(); // Nt: par-list should NOT contain COMMON-items
	for(i=0; i<par_num; i++) {
		CExprContainer* par = (CExprContainer*)par_list->GetAt(i);
		CExpression* sym = par->get_expr(); if((use_COMMON)&&(FindInCommonBlock(sym)>=0)){dbg->log(0,"WARNING: COMMON item in par-list",sym);continue;}
		if(need_comma){ *f_line<<", "; }else{ need_comma = true; }
		int iParType=par->get_info(); if(iParType==CEAbstractFunction::PAR_DEFAULT)iParType=m_codeLang->GetDefaultTypeId(convs);
		if(m_general_conventions & CLFileManager::FT_CPP){ *f_line<<m_codeLang->GetTypePrefix(iParType)<<" "; }
		*f_line<<sym->toString();
	}
	f_line->AddString(")"); if((m_general_conventions&CLFileManager::FT_CPP) && (!bHeaderOnly))f_line->AddString(" {");
	m_codeLang->OutCodeLine(p_ofs, f_line, -1, (!bHeaderOnly));
	if(bHeaderOnly){
		delete f_line;
		if(lt_UV_val!=NULL)delete lt_UV_val; if(lt_IR_val!=NULL)delete lt_IR_val;
		if(DR_mass2!=NULL)delete DR_mass2; if(DR_mass2_F!=NULL)delete DR_mass2_F;
		delete par_list; delete vec_list; delete NonVecCOMM_list; delete EQUList;
		delete vec_par_list; delete vec_mom_list; delete vec_pol_list; delete ave_pol_list; delete vec_basisA_list; delete vec_basisB_list; delete vec_unkn_list;
		if (s_EntryTitle != NULL) { delete s_EntryTitle; }
		return;
	}
	if(m_general_conventions&CLFileManager::FT_CPP) { m_codeLang->IncIndent(); }

	// IMPLICIT NONE
	if((m_general_conventions&CLFileManager::FT_F77) || (m_general_conventions&CLFileManager::FT_F90)) {
		*f_line<<"IMPLICIT NONE"; m_codeLang->OutCodeLine(p_ofs, f_line);
	}

	//	input parameters (may exists together with COMMON-block)
	// **********************************	REAL*8	PAR1, PAR2, ...
	if((m_general_conventions&CLFileManager::FT_F77) || (m_general_conventions&CLFileManager::FT_F90)) {
		if (vec_num>0) {
			tmp_1=num_params_in_line-1; _start_line=true; int iTypePrev=0;
			for (i=0; i<vec_num; i++) {
				CEString* sym_str = vec_list->GetAt(i)->toString();
				int iParType=0;//par->get_info(); //TODO: allow real and complex vectors mixing
				//if((!_start_line) && (iTypePrev!=iParType)) { m_codeLang->OutCodeLine(p_ofs, f_line); tmp_1+=num_params_in_line; _start_line=true; }
				if(_start_line){
					int iParTypeFixed=iParType; if(iParTypeFixed==CEAbstractFunction::PAR_DEFAULT)iParTypeFixed=m_codeLang->GetDefaultTypeId(convs);
					*f_line<<m_codeLang->GetTypePrefix(iParTypeFixed)<<" "; _start_line=false; iTypePrev=iParType;
				}else{ *f_line<<", "; }
				f_line->AddString(sym_str); f_line->AddString("(4)");
				if ((i>=tmp_1) && (i != (par_num-1))) {
					m_codeLang->OutCodeLine(p_ofs, f_line); tmp_1+=num_params_in_line; _start_line=true;
				}
			}
			if(!_start_line){ m_codeLang->OutCodeLine(p_ofs, f_line); }
		}
		if (par_num>0) {
			tmp_1=num_params_in_line-1; _start_line=true; int iTypePrev=0;
			for (i=0; i<par_num; i++) {
				CExprContainer* par = (CExprContainer*)par_list->GetAt(i);
				CExpression* sym = par->get_expr(); int iTypeCurr=par->get_info();
				if (use_COMMON) { if(FindInCommonBlock(sym)>=0){dbg->log(0,"WARNING: COMMON item in par-list",sym);continue;} }
				if((!_start_line)&&(iTypeCurr!=iTypePrev)){ m_codeLang->OutCodeLine(p_ofs,f_line); tmp_1+=num_params_in_line; _start_line=true; }
				if(_start_line){
					int iTypeCurrFixed=iTypeCurr; if(iTypeCurrFixed==CEAbstractFunction::PAR_DEFAULT)iTypeCurrFixed=m_codeLang->GetDefaultTypeId(convs);
					*f_line<<m_codeLang->GetTypePrefix(iTypeCurrFixed)<<" "; _start_line=false; iTypePrev=iTypeCurr;
				}else{ *f_line<<", "; }
				f_line->AddString(sym->toString());
				if ((i>=tmp_1) && (i != (par_num-1))) { m_codeLang->OutCodeLine(p_ofs, f_line); tmp_1+=num_params_in_line; _start_line=true; }
			}
			if(!_start_line){ m_codeLang->OutCodeLine(p_ofs, f_line); }
		}
		if (_AllowQuarksSwap) { *f_line<<m_codeLang->GetTypePrefixInt()<<" "<<QOrig; m_codeLang->OutCodeLine(p_ofs, f_line); }
		if ((vec_num>0)||(par_num>0)||(_AllowQuarksSwap)) { *p_ofs << "\n"; }
	}
	if((m_general_conventions&CLFileManager::FT_CPP) && bCppEventEntry && bNoKinVecInParams && (!PY_vectors)) {
		// Declare local vectors used
		s=vec_mom_list->size();
		if(s>0) {
			tmp_1=num_params_in_line-1; _start_line=true;
			for (i=0; i<s; i++) {
				if(_start_line){ *f_line<<CCodeProcessImpl::CN_CPP_4VEC_CLASS<<" *"; _start_line=false; }else{ *f_line<<", *"; }
				f_line->AddString(vec_mom_list->GetAt(i)->toString());
				if((i>=tmp_1) && (i<(s-1))){ m_codeLang->OutCodeLine(p_ofs, f_line); tmp_1+=num_params_in_line; _start_line=true; }
			}
			if(!_start_line)m_codeLang->OutCodeLine(p_ofs,f_line); *p_ofs<<"\n";
		}
	}

	//	functions used
	// **********************************	REAL*8	FUNC1, FUNC2, ...
	if((m_general_conventions&CLFileManager::FT_F77) || (m_general_conventions&CLFileManager::FT_F90)) {
		m_codeLang->StartCommentLine(p_ofs); *p_ofs << "Functions used" << "\n";
		*f_line<<type_Default<<" "<<func_inf->func_name();
		if (_used_sp4) { *f_line<<", "<<(_Cmplx?("C_SP4"):("R_SP4")); }
		if (_used_aPr) { *f_line<<", "<<(_Cmplx?("C_ASYMM_PROD"):("R_ASYMM_PROD")); }
		if (_used_fBond) { *f_line<<", "<<(_Cmplx?("C_FBONDAREV"):("R_FBONDAREV")); }
		m_codeLang->OutCodeLine(p_ofs, f_line);
		if (dens_check_num>0) {
			m_codeLang->StartCommentLine(p_ofs); *p_ofs << "Denominator 0-check functions" << "\n";
			tmp_1=num_params_in_line-1; _start_line=true;
			for(i=0; i<dens_check_num; i++) {
				CFortranFunctionInfo* d_fi = (CFortranFunctionInfo*)m_Dens_functions->GetAt(i);
				if(_start_line){ *f_line<<type_Default<<" "; _start_line=false; }else{ *f_line<<", "; }
				f_line->AddString(d_fi->func_name());
				if((i>=tmp_1)&&(i!=(par_num-1))){ m_codeLang->OutCodeLine(p_ofs,f_line); tmp_1+=num_params_in_line; _start_line=true; }
			}
			if(!_start_line)m_codeLang->OutCodeLine(p_ofs, f_line);
		}
	}

	// temp variables (scalar couplings if they are not in COMMON block)
	// **********************************	REAL*8	TEMP1, TEMP2, ..
	tmp_1=num_params_in_line-1; _start_line=true; long temp_num=0;
	if(cpl_num>0){ m_codeLang->StartCommentLine(p_ofs); *p_ofs << "local couplings" << "\n"; }
	for (i=0; i<cpl_num; i++) {
		CExpression* sym = ((CExprContainer*)cpl_data->GetAt(i))->get_info_expr();
		if (use_COMMON) { if(FindInCommonBlock(sym)>=0) continue; } // skip coupling
		CEString* sym_str = sym->toString();
		if(_start_line){ *f_line<<type_Default<<" "; _start_line=false; }else{ *f_line<<", "; }
		f_line->AddString(sym_str); temp_num++;
		if ((i>=tmp_1) && (i != (par_num-1))) {
			m_codeLang->OutCodeLine(p_ofs, f_line->getData()); f_line->Reset(); tmp_1+=num_params_in_line; _start_line=true;
		}
	}
	if(!_start_line)m_codeLang->OutCodeLine(p_ofs, f_line); tmp_1=num_params_in_line-1; _start_line=true;
	s=TmpList->size();
	if((s>0) || (TmpListReal.size()>0)){ m_codeLang->StartCommentLine(p_ofs); *p_ofs << "local variables" << "\n"; }
	for (i=0; i<s; i++) {
		CExpression* sym = TmpList->GetAt(i); if (use_COMMON) { if(FindInCommonBlock(sym)>=0) continue; } // skip coupling
		if(_start_line){ *f_line<<type_Default<<" "; _start_line=false; }else{ *f_line<<", "; }
		f_line->AddString(sym->toString()); temp_num++;
		if ((i>=tmp_1) && (i != (par_num-1))) { m_codeLang->OutCodeLine(p_ofs, f_line); tmp_1+=num_params_in_line; _start_line=true; }
	}
	if(!_start_line) { m_codeLang->OutCodeLine(p_ofs, f_line->getData()); f_line->Reset(); }
	for (i=0; i<TmpListReal.size(); i++) {
		CExpression* sym = TmpListReal.GetAt(i); if(use_COMMON){ if(FindInCommonBlock(sym)>=0) continue; } // skip coupling
		if(_start_line){ *f_line<<m_codeLang->GetTypePrefixReal()<<" "; _start_line=false; }else{ *f_line<<", "; }
		f_line->AddString(sym->toString()); temp_num++;
		if ((i>=tmp_1) && (i != (par_num-1))) { m_codeLang->OutCodeLine(p_ofs, f_line); tmp_1+=num_params_in_line; _start_line=true; }
	}
	if(!_start_line) { m_codeLang->OutCodeLine(p_ofs, f_line->getData()); f_line->Reset(); }

	if(temp_num>0) { *p_ofs << "\n"; }

	long basisA_num = vec_basisA_list->size(); // random basis vectors number

	const char* var_DENS_OK;
	const char* var_DENS_LOOP_I;		//used if ((dens_check_num>0) && (basisA_num>0))
	const char* var_DENS_LOOP_MAX;		//used if ((dens_check_num>0) && (basisA_num>0))
	const char* var_DENS_TEMP;
	const char* var_DENS_MIN;
	bool bDensMinIs0=true;				//parameter -- TODO: use 'var_DENS_MIN'~eps*coupling ???
	if((m_general_conventions&CLFileManager::FT_F77)||(m_general_conventions&CLFileManager::FT_F90)) {
		var_DENS_OK="DENS_OK"; var_DENS_LOOP_I="DENS_LOOP_I"; var_DENS_LOOP_MAX="DENS_LOOP_MAX"; var_DENS_TEMP="DENS_TEMP"; var_DENS_MIN="DENS_MIN";
	}else{
		var_DENS_OK="bDensOk"; var_DENS_LOOP_I="iDensLoop"; var_DENS_LOOP_MAX="nDensLoops"; var_DENS_TEMP="tmpDens"; var_DENS_MIN="_DensMin";
	}
	if (dens_check_num>0) {
		if((m_general_conventions&CLFileManager::FT_F77)||(m_general_conventions&CLFileManager::FT_F90)){
			*f_line<<m_codeLang->GetTypePrefixInt()<<" "<<var_DENS_OK<<", "<<var_DENS_LOOP_I<<", "<<var_DENS_LOOP_MAX; m_codeLang->OutCodeLine(p_ofs, f_line);
			*f_line<<type_ID<<" "<<var_DENS_TEMP; m_codeLang->OutCodeLine(p_ofs, f_line);
			if(!bDensMinIs0){ *f_line<<m_codeLang->GetTypePrefixReal()<<" "<<var_DENS_MIN; m_codeLang->OutCodeLine(p_ofs, f_line); }
		}else{
			//[later]*f_line<<CFFortran::TYPE_BOOL_CPP<<" "<<var_DENS_OK; m_codeLang->OutCodeLine(p_ofs, f_line);
			//[later]*f_line<<m_codeLang->GetTypePrefixInt()<<" "<<var_DENS_LOOP_I<<", "<<var_DENS_LOOP_MAX; m_codeLang->OutCodeLine(p_ofs, f_line);
			//[later]*f_line<<type_ID<<" "<<var_DENS_TEMP; m_codeLang->OutCodeLine(p_ofs, f_line);
			//[later]if(!bDensMinIs0){ *f_line<<m_codeLang->GetTypePrefixReal()<<" "<<var_DENS_MIN; m_codeLang->OutCodeLine(p_ofs, f_line); }
		}
		//TODO: make DENS_MIN=1D-12 assign via EQUList, problem: CERealValue->toString() returns double as "...D0" only
	}
	if((m_general_conventions&CLFileManager::FT_F77)||(m_general_conventions&CLFileManager::FT_F90)){ //for VEC(I)=PUP(I,.) cycle
		if(PY_vectors){ *f_line<<m_codeLang->GetTypePrefixInt()<<" I"; m_codeLang->OutCodeLine(p_ofs, f_line); }
	}
	if(_AllowQuarksSwap){ *f_line<<m_codeLang->GetTypePrefixInt()<<" "<<Q1Orig<<", "<<Q2Orig;m_codeLang->OutCodeLine(p_ofs,f_line); }
	if(src_ampl){ *f_line<<type_ID<<" AMPL_VAL"; m_codeLang->OutCodeLine(p_ofs, f_line); }
	long pol_num = ave_pol_list->size(); // polarizations averaging IDs
	tmp_1=num_params_in_line-1; _start_line=true;
	if(pol_num>0){ m_codeLang->StartCommentLine(p_ofs); *p_ofs << "Polarization averaging indices" << "\n"; }
	for (i=0; i<pol_num; i++) {
		if(_start_line){ *f_line<<m_codeLang->GetTypePrefixInt()<<" "; _start_line=false; }else{ *f_line<<", "; }
		*f_line<<"POL_"<<i;
		if ((i>=tmp_1) && (i != (par_num-1))) { m_codeLang->OutCodeLine(p_ofs, f_line); tmp_1+=num_params_in_line; _start_line=true; }
	}
	if(!_start_line)m_codeLang->OutCodeLine(p_ofs, f_line);

	//	COMMON-block
	if (use_COMMON) {
		if((m_general_conventions&CLFileManager::FT_F77) || (m_general_conventions&CLFileManager::FT_F90)) {
			m_codeLang->StartCommentLine(p_ofs); *p_ofs << "Local variables COMMON block" << "\n";
			add_CommonBlock(p_ofs, f_line, type_Default, num_params_in_line);
			// SAVE/COMMONBLOCK/
			*f_line << "SAVE /"<<((m_COMMONBLOCK_NAME!=NULL)?m_COMMONBLOCK_NAME:COMMON_BLOCK_TITLE)<< "/";
			m_codeLang->OutCodeLine(p_ofs, f_line);
			*p_ofs << "\n";
		}
	}
	if ((PY_vectors) && (vec_mom_list->size()>0)) { // PYTHIA COMMON block
		if((m_general_conventions&CLFileManager::FT_F77) || (m_general_conventions&CLFileManager::FT_F90)) {
			m_codeLang->StartCommentLine(p_ofs); *p_ofs << "...User process event common block." << "\n";
			*f_line << "INTEGER MAXNUP"; m_codeLang->OutCodeLine(p_ofs, f_line);
			*f_line << "PARAMETER (MAXNUP=500)"; m_codeLang->OutCodeLine(p_ofs, f_line);
			*f_line << "INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP"; m_codeLang->OutCodeLine(p_ofs, f_line);
			*f_line << "DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP"; m_codeLang->OutCodeLine(p_ofs, f_line);
			*f_line << "COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,IDUP(MAXNUP),"; m_codeLang->OutCodeLine(p_ofs, f_line);
			*f_line << "ISTUP(MAXNUP),MOTHUP(2,MAXNUP),ICOLUP(2,MAXNUP),PUP(5,MAXNUP),"; m_codeLang->OutCodeLine(p_ofs, f_line);
			*f_line << "VTIMUP(MAXNUP),SPINUP(MAXNUP)"; m_codeLang->OutCodeLine(p_ofs, f_line);
			*p_ofs << "\n";
		}
	}

	//C++ return value
	if(m_general_conventions&CLFileManager::FT_CPP){ m_codeLang->StartCommentLine(p_ofs);*p_ofs<<"Return value\n"; *f_line<<type_ID<<" "<<s_ReturnVar.getData(); m_codeLang->OutCodeLine(p_ofs,f_line); *p_ofs<<"\n"; }
	//Include 'looptools.h' to FORTRAN code
	if((lt_calls)&&((lt_UV_val!=NULL)||(lt_IR_val!=NULL)||(DR_mass2_F !=NULL))){ if((m_general_conventions&CLFileManager::FT_F77)||(m_general_conventions&CLFileManager::FT_F90)){ *p_ofs<<"#include \"looptools.h\"\n\n"; } }

	// implementetion
	// ************************************************** precalculations

	/*
	// Unused: use - conflict to 'particle order'
	// Find first iintial/final perticle positions (use 0/2 in FORTRAN mode)
	const char* idIn0 = "idIn0";
	const char* idFin0 = "idFin0";
	if(m_general_conventions&CLFileManager::FT_CPP) {
		*f_line<<type_ID<<" "<<idIn0<<" = "<<CCodeProcessImpl::CN_CPP_EVT_CLASS<<"::GetFirstInitialId()"; m_codeLang->OutCodeLine(p_ofs,f_line,-1,false,"find first final particle");
		*f_line<<type_ID<<" "<<idFin0<<" = "<<CCodeProcessImpl::CN_CPP_EVT_CLASS<<"::GetFirstFinalId()"; m_codeLang->OutCodeLine(p_ofs,f_line,-1,false,"find first final particle");
		*p_ofs<<"\n";
	}
	*/

	// Quarks origins (Note: this case may be used if both particles are quarks only)
	if (_AllowQuarksSwap) {
		m_codeLang->StartCommentLine(p_ofs); *p_ofs << "Define quark origins" << "\n";
		*f_line<<m_codeLang->GetOpIf()<<"("<<QOrig<<m_codeLang->GetOpEqu()<<"1) "<<m_codeLang->GetOpThen();m_codeLang->OutCodeLine(p_ofs,f_line,-1,true);m_codeLang->IncIndent();
		int pos1=0; int pos2=1; if((m_general_conventions&CLFileManager::FT_F77) || (m_general_conventions&CLFileManager::FT_F90)) { pos1++; pos2++;  }
		*f_line<<Q1Orig<<" = "<<pos1;m_codeLang->OutCodeLine(p_ofs,f_line);
		*f_line<<Q2Orig<<" = "<<pos2;m_codeLang->OutCodeLine(p_ofs,f_line); m_codeLang->DecIndent();
		*f_line<<m_codeLang->GetOpElse();m_codeLang->OutCodeLine(p_ofs,f_line,-1,true); m_codeLang->IncIndent();
		*f_line<<Q1Orig<<" = "<<pos2;m_codeLang->OutCodeLine(p_ofs,f_line);
		*f_line<<Q2Orig<<" = "<<pos1;m_codeLang->OutCodeLine(p_ofs,f_line); m_codeLang->DecIndent();
		*f_line<<m_codeLang->GetOpEndIf();m_codeLang->OutCodeLine(p_ofs,f_line,-1,true);
	}

	// PYTHIA vectors copying
	if ((PY_vectors) && (vec_mom_list->size()>0)) { // PYTHIA COMMON block
		if((m_general_conventions&CLFileManager::FT_F77) || (m_general_conventions&CLFileManager::FT_F90)) {
			m_codeLang->StartCommentLine(p_ofs); *p_ofs << "Init internal momenta 4-vectors from PUP(X,Y)" << "\n";
		}else{
			m_codeLang->StartCommentLine(p_ofs); *p_ofs << "Init internal momenta 4-vectors from PYTHIA user array" << "\n";
		}
		s=vec_mom_list->size();
		if((m_general_conventions&CLFileManager::FT_F77) || (m_general_conventions&CLFileManager::FT_F90)) {
			m_codeLang->AddOpFor(f_line,"I",1,4,10); m_codeLang->OutCodeLine(p_ofs,f_line,-1,true); m_codeLang->IncIndent(); //VECTOR ASSIGN cycle NO: 10,11,12,13..
		}
		for(i=0; i<s; i++) {
			CExpression* vec = vec_mom_list->GetAt(i); int pID=pID_vec_mom_list[i]; // >0 means initials
			if(pID==0){ dbg->log(0,"Warning: no PID found for momentum", vec); continue; }
			if((m_general_conventions&CLFileManager::FT_F77)||(m_general_conventions&CLFileManager::FT_F90)){
				f_line->AddString(vec->toString()); f_line->AddString("(I) = "); if(_Cmplx)f_line->AddString("DCMPLX("); f_line->AddString("PUP(I,");
				if((_AllowQuarksSwap)&&(pID>0)&&(pID<=2)){ f_line->AddString(((pID==1)?Q1Orig:Q2Orig)); }else{ f_line->add_int(((pID>0)?pID:(2-pID))); } //(pID<0)==>3,4,5...
				f_line->AddString(")"); if(_Cmplx){ f_line->AddString(",0D0)"); }
			} else if(m_general_conventions&CLFileManager::FT_CPP) {
				f_line->AddString(vec->toString()); f_line->AddString(" = "); if(_Cmplx){ *f_line<<"("<<m_codeLang->GetTypePrefixComplex()<<")"; }
				//f_line->AddString(((pID==1)?Q1Orig:Q2Orig));
				f_line->AddString("UNIMPLEMENTED!!!");
			}
			m_codeLang->OutCodeLine(p_ofs,f_line);
		}
		if((m_general_conventions&CLFileManager::FT_F77)||(m_general_conventions&CLFileManager::FT_F90)){ m_codeLang->DecIndent(); m_codeLang->AddOpEndFor(f_line); m_codeLang->OutCodeLine(p_ofs, f_line,10,true); }
		*p_ofs<<"\n";
	}
	if((m_general_conventions&CLFileManager::FT_CPP) && bCppEventEntry && bNoKinVecInParams && (!PY_vectors)) {
		m_codeLang->StartCommentLine(p_ofs); *p_ofs << "Init momenta vectors (some vectors may be unused)" << "\n";
		s=vec_mom_list->size();
		for(i=0; i<s; i++) {
			CExpression* vec = vec_mom_list->GetAt(i); int pID=pID_vec_mom_list[i]; // >0 means initials
			if(pID==0){ dbg->log(0,"Warning: no PID found for momentum", vec); continue; }
			f_line->AddString(vec->toString()); *f_line<<" = &evt->m_Particles["<<S_ARR_PARTICLE_ORDER_CPP<<"[";
			if((_AllowQuarksSwap)&&(pID>0)&&(pID<=2)){ f_line->AddString(((pID==1)?Q1Orig:Q2Orig)); }else{ f_line->add_int(((pID>0)?(pID-1):(1-pID))); } //(pID>0)==>0,1;(pID<0)==>2,3,4...
			f_line->AddString("]].m_Mom"); m_codeLang->OutCodeLine(p_ofs,f_line);
		}
		*p_ofs<<"\n";
	}

	// EQU list
	long EQU_size = EQUList->size();
	for(i=0; i<EQU_size; i++) {
		CExprContainer* cart = (CExprContainer*)EQUList->GetAt(i);
		CExpression* sym = cart->get_expr();
		CExpression* val = cart->get_info_expr(); // sym=val
		f_line->AddString(sym->toString()); f_line->AddString(" = "); f_line->AddString(val->toString(m_STORE_EXPR_AS_FLAG));
		m_codeLang->OutCodeLine(p_ofs, f_line);
	}
	//TODO: make DENS_MIN=1D-12 assign via EQUList, problem: CERealValue->toString() returns double as "...D0" only
	if (dens_check_num>0) {
		if(!bDensMinIs0){
			m_codeLang->StartCommentLine(p_ofs); *p_ofs << "Minimal allowed denominator value:" << "\n";
			if(m_general_conventions&CLFileManager::FT_CPP){ *f_line<<m_codeLang->GetTypePrefixReal()<<" "; }
			*f_line<<var_DENS_MIN<<"="; f_line->AddString(((m_general_conventions&CLFileManager::FT_CPP)?"1D-14":"1.e-14")); m_codeLang->OutCodeLine(p_ofs, f_line);
		}
		if (basisA_num>0) {
			m_codeLang->StartCommentLine(p_ofs); *p_ofs << "Maximum attempts of basis selection (to fit DENS!=0 condition):" << "\n";
			if(m_general_conventions&CLFileManager::FT_CPP){*f_line<<m_codeLang->GetTypePrefixInt()<<" ";}
			*f_line<<var_DENS_LOOP_MAX<<"=100"; m_codeLang->OutCodeLine(p_ofs, f_line);
		}
	}
	if (EQU_size>0) { *p_ofs << "\n"; }

	// LoopTools-init
	if (lt_calls) {
		bool lt_call_exist=false;
		if (lt_UV_val!=NULL) {
			m_codeLang->StartCommentLine(p_ofs); *p_ofs << "LoopTools UV-regulator:\n";
			f_line->AddString(lt_UV_val->toString(m_STORE_EXPR_AS_FLAG)); f_line->AddString("=getdelta()"); m_codeLang->OutCodeLine(p_ofs,f_line); //f_line->AddString(get_ItemStr(lt_UV_val, true));
			lt_call_exist=true;
		}
		if (lt_IR_val!=NULL) {
			m_codeLang->StartCommentLine(p_ofs); *p_ofs << "LoopTools IR-regulator [Bardin, Passarino (2.258)]:\n";
			//TODO: check sign + definition. (Find old notes!).
			//Xi=ln(Mu^2/La^2)
			f_line->AddString(lt_IR_val->toString(m_STORE_EXPR_AS_FLAG)); f_line->AddString("=");
			if((m_general_conventions&CLFileManager::FT_F77)||(m_general_conventions&CLFileManager::FT_F90)){ f_line->AddString("DLOG("); }
			else if(m_general_conventions&CLFileManager::FT_CPP){ f_line->AddString("log("); }else{CEException::halt("Unexpected code language.");}
			f_line->AddString("getmudim()/getlambda())"); m_codeLang->OutCodeLine(p_ofs,f_line); //f_line->AddString(get_ItemStr(lt_UV_val, true));
			lt_call_exist=true;
		}
		if (DR_mass2_F != NULL) {
			m_codeLang->StartCommentLine(p_ofs); *p_ofs << "LoopTools dimension regularization mass parameter:\n";
			f_line->AddString(DR_mass2_F->toString(m_STORE_EXPR_AS_FLAG)); f_line->AddString("=getmudim()"); m_codeLang->OutCodeLine(p_ofs,f_line);
			lt_call_exist=true;
		}
		if (lt_call_exist) { *p_ofs << "\n"; }
	}

	// basis vectors (non-random) init
	s = vec_basisB_list->size();
	if (s>0) {
		m_codeLang->StartCommentLine(p_ofs); *p_ofs << "Bondarev functions basis init" << "\n";
		for (int i=0; i<s; i++) {
			CExpression* sym = vec_basisB_list->GetAt(i);
			int _id = CEFuncBondarev::FindBondarevBaseSyms(sym); if (_id<0) { dbg->log(0,"Internal WARNING: Bondarev symbol is not found, CHECK CODE!",sym); continue; }
			// GEN_BONDAREV_BASIS_VECTOR(VEC,_ID)
			if(m_general_conventions&CLFileManager::FT_CPP){ f_line->AddString("GenBondarevBasisVector("); }
			else{ f_line->AddString("CALL GEN_BONDAREV_BASIS_VECTOR("); }
			*f_line<<sym->toString()<<", "<<_id<<")";m_codeLang->OutCodeLine(p_ofs,f_line);
		}
		*p_ofs << "\n";
	}

	// polarization averaging
	if (pol_num>0) { // FUNC_RES_VEC=0
		*f_line<<s_ReturnVar.getData()<<"=0"; m_codeLang->OutCodeLine(p_ofs,f_line); *p_ofs << "\n";
		m_codeLang->StartCommentLine(p_ofs); *p_ofs << "Average polarizations for unpolarized particles" << "\n";
	}
	for(i=0; i<pol_num; i++) { // POL_I cycle NO: 20,21,22,23..
		CExpression* vec = ave_pol_list->GetAt(i);
		int pID = kin->find_polarization_vec(GetSymOrigin(vec)); // >0 means initials
		CEParticle* part = kin->get_particle_by_ID(pID); if (part->get_polarization_info() != 0) continue;
		CExpression* mom_orig = part->get_momentum_symbol();
		if (mom_orig->ClassName() != CESymbol::CLASS_NAME) { dbg->log(0, "ERROR: NON-SYMBOL momentum found, CHECK CODE", mom_orig); continue;  }
		CESymbol* mom_F = validate_sym((CESymbol*)mom_orig,m_syms_replaced);
		if (vec_list->get_item_pos(mom_F) < 0) { dbg->log(0, "Internal ERROR. No momentum declared for particle, CHECK CODE!",mom_F); if(mom_F!=mom_orig)delete mom_F; continue; }
		// average polarization
		int _PolNum = part->GetPolarizationsNumber();
		sprintf(m_TmpBuf1kb,"POL_%ld",i);m_codeLang->AddOpFor(f_line,m_TmpBuf1kb,1,_PolNum,20+i);m_codeLang->OutCodeLine(p_ofs,f_line,-1,true);m_codeLang->IncIndent();
		bool _part_boson = CEParticle::is_bosonic_PK(part->get_PK());
		// C_GEN_BOSON_POLARIZATION|C_GEN_FERMION_POLARIZATION
		if(m_general_conventions&CLFileManager::FT_CPP){ f_line->AddString((_part_boson)?"CGenBosonPolarization(":"CGenFermionPolarization("); }
		else{ f_line->AddString(((_part_boson)?"CALL C_GEN_BOSON_POLARIZATION(":"CALL C_GEN_FERMION_POLARIZATION(")); }
		*f_line<<vec->toString()<<", "<<mom_F->toString()<<", "<<"POL_"<<i<<")"; m_codeLang->OutCodeLine(p_ofs,f_line);
		CEString* s_tmp1 = vec->toString(); CEString* s_tmp2 = mom_F->toString();
		m_codeLang->StartCommentLine(p_ofs,true);*p_ofs<<((m_general_conventions&CLFileManager::FT_CPP)?"std::cout<<":"print *,\"")<<s_tmp1->getData()<<","<<s_tmp2->getData()<<"\","<<s_tmp1->getData()<<","<<s_tmp2->getData()<<"\n";
		delete s_tmp1; delete s_tmp2;
		if(mom_F != mom_orig) delete mom_F;
	}

	// random basis vectors (non-random) init
	if (basisA_num>0) {
		if (pol_num>0) { *p_ofs << "\n"; }

		 // RANDOM BASIS GOTO label: (dens_check_num>0)?39:-1
		m_codeLang->StartCommentLine(p_ofs); *p_ofs << "Generate RANDOM basis vector for spinors projection" << "\n";
		// 1. vector base
		CExpression* p_vec=NULL; bool _massive=true; bool del_p_vec=false;
		for (i=0; i<basisA_num; i++) {
			CExpression* sym = vec_basisA_list->GetAt(i);
			CExpression* sym_orig = GetSymOrigin(sym);
			if (sym_orig->equals(CAmplCalcLib::GetBasisVecSym(true,true))) {
				if (p_vec!=NULL) { dbg->log(0,"Internal WARNING: Different(!!) vector bases ase found, CHECK CODE!",vec_basisA_list); break; }
				p_vec=sym; _massive=true;
			} else if (sym_orig->equals(CAmplCalcLib::GetBasisVecSym(true,false))) {
				if (p_vec!=NULL) { dbg->log(0,"Internal WARNING: Different(!!) vector bases ase found, CHECK CODE!",vec_basisA_list); break; }
				p_vec=sym; _massive=false;
			}
		}
		if (p_vec == NULL) {
			dbg->log(0,"Internal WARNING: Can't found amplitude vector base in saving expression, Massive one is introduced",vec_basisA_list);
			CESymbol* p_orig = CAmplCalcLib::GetBasisVecSym(true,true);
			p_vec = validate_sym(p_orig,m_syms_replaced);
			if (p_vec != p_orig) { del_p_vec=true; }
		}
		//DENS_LOOP_I=0
		if (dens_check_num>0) {
			if(m_general_conventions&CLFileManager::FT_CPP){ *f_line<<m_codeLang->GetTypePrefixInt()<<" "; }
			*f_line<<var_DENS_LOOP_I<<"=0"; m_codeLang->OutCodeLine(p_ofs, f_line);
		}
		// GEN_RANDOM_VECTOR(VEC,SQR)
		const char* FGEN_title=((m_general_conventions&CLFileManager::FT_CPP)? (_Cmplx?"CGenRandomVector":"RGenRandomVector") : (_Cmplx?"C_GEN_RANDOM_VECTOR":"R_GEN_RANDOM_VECTOR") );
		if((m_general_conventions&CLFileManager::FT_F77)||(m_general_conventions&CLFileManager::FT_F90)){ f_line->AddString("CALL "); }
		*f_line<<FGEN_title<<"("<<p_vec->toString()<<", ";
		if(_massive){ f_line->AddString(((m_general_conventions&CLFileManager::FT_CPP)?"1000D0":"1000.")); }else{ f_line->AddString(((m_general_conventions&CLFileManager::FT_CPP)?"0D0":"0.")); } // 1000GeV or 0
		f_line->AddString(")"); m_codeLang->OutCodeLine(p_ofs,f_line,((dens_check_num>0)?39:-1)); // label: (dens_check_num>0)?39:-1

		// 2. polarization vector for random base
		CExpression* e_vec=NULL;
		for (i=0; i<basisA_num; i++) {
			CExpression* sym = vec_basisA_list->GetAt(i);
			CExpression* sym_orig = GetSymOrigin(sym);
			if (sym_orig->equals(CAmplCalcLib::GetBasisVecSym(false,true))) {
				if (e_vec!=NULL) { dbg->log(0,"Internal WARNING: Different(!!) polarization vector bases ase found, CHECK CODE!",vec_basisA_list); break; }
				e_vec=sym;
			}
		}
		if (e_vec != NULL) {
			// GEN_RANDOM_VECTOR_POLARIZATION(POL,VEC) or C_GEN_BOSON_POLARIZATION(E,P,I)
			*f_line<<((m_general_conventions&CLFileManager::FT_CPP)?"CGenFermionPolarization(":"CALL C_GEN_FERMION_POLARIZATION(");
			*f_line<<e_vec->toString()<<", "<<p_vec->toString()<<",1)"; m_codeLang->OutCodeLine(p_ofs,f_line);
		}
		if (del_p_vec) { delete p_vec; }
		*p_ofs << "\n";
	}

	// couplings list
	if (cpl_num>0) { m_codeLang->StartCommentLine(p_ofs); *p_ofs << "Pre-calculate scalar couplings" << "\n"; }
	for(i=0; i<cpl_num; i++) {
		CExprContainer* cart = (CExprContainer*)cpl_data->GetAt(i);
		CExpression* cpl_i = cart->get_expr(); // coupling
		CExpression* r = cart->get_info_expr(); // replacement

		*f_line<<get_ItemStr(r, true)<<" = ";

		if(m_general_conventions&CLFileManager::FT_CPP) {
			if(CEUtil::equals(cpl_i->ClassName(), CEAntiSymmProd::CLASS_NAME)) {
				CEAntiSymmProd* a_pr = (CEAntiSymmProd*)cpl_i;
				*f_line<<CCodeProcessImpl::CN_CPP_LIB_CLASS<<"::"<<(_Cmplx?("CAsymmProd(*"):("AsymmProd(*"));
				*f_line<<a_pr->get_v(0)->toString()<<", *"<<a_pr->get_v(1)->toString()<<", *";
				*f_line<<a_pr->get_v(2)->toString()<<", *"<<a_pr->get_v(3)->toString()<<")";
			} else if(CEUtil::equals(cpl_i->ClassName(), CESProduct::CLASS_NAME)) {
				if(_Cmplx){ dbg->log(0,"ERROR! Complex vectors are not implemented for C++ ouptut"); CEException::halt(); }
				CESProduct* c = (CESProduct*)cpl_i; // coupling

				int iVec1=-1; int iVec2=-1; int pid1=0; int pid2=0;	
				for(int j=0; j<vec_mom_list->size(); j++) {
					CExpression* vec = vec_mom_list->GetAt(j);
					if((iVec1<0) && (c->get_a()->equals(vec))){ pid1=pID_vec_mom_list[j]; iVec1=j; }
					if((iVec2<0) && (c->get_b()->equals(vec))){ pid2=pID_vec_mom_list[j]; iVec2=j; }
					if((iVec1>=0) && (iVec2>=0))break;
				}
				if((pid1!=0)&&(pid2!=0)) { // Event->GetCpl() may be MORE precise than vec1->GetCpl(vec2)
					*f_line<<"evt->GetCpl("<<S_ARR_PARTICLE_ORDER_CPP<<"[";
					//Note: pid1,pid2 -- IDs of particles in kinematics, >0 means initials
					if((_AllowQuarksSwap)&&(pid1>0)&&(pid1<=2)){ *f_line<<((pid1==1)?Q1Orig:Q2Orig); }else{ *f_line<<((pid1>0)?(pid1-1):(1-pid1)); } //(pid1>0)==>0,1;(pid1<0)==>2,3,4...
					*f_line<<"], "<<S_ARR_PARTICLE_ORDER_CPP<<"[";
					if((_AllowQuarksSwap)&&(pid2>0)&&(pid2<=2)){ *f_line<<((pid2==1)?Q1Orig:Q2Orig); }else{ *f_line<<((pid2>0)?(pid2-1):(1-pid2)); } //(pid2>0)==>0,1;(pid2<0)==>2,3,4...
					*f_line<<"])";
				} else {
					if(c->get_a()->equals(c->get_b())){ *f_line<<c->get_a()->toString()<<"->GetSqr()"; }
					else{ *f_line<<c->get_a()->toString()<<"->GetCpl(*"<<c->get_b()->toString()<<")"; }
				}
			} else if(CEUtil::equals(cpl_i->ClassName(), CEFuncBondarev::CLASS_NAME)) {
				dbg->log(0,"ERROR! Bondarev Complex vectors are not implemented for C++ ouptut");
				f_line->AddString("ERROR");
			}
		} else {
			if(CEUtil::equals(cpl_i->ClassName(), CEAntiSymmProd::CLASS_NAME)) {
				CEAntiSymmProd* a_pr = (CEAntiSymmProd*)cpl_i;
				*f_line<<(_Cmplx?("C_ASYMM_PROD("):("R_ASYMM_PROD("));
				*f_line<<a_pr->get_v(0)->toString()<<", "<<a_pr->get_v(1)->toString()<<", ";
				*f_line<<a_pr->get_v(2)->toString()<<", "<<a_pr->get_v(3)->toString()<<")";
			} else if(CEUtil::equals(cpl_i->ClassName(), CESProduct::CLASS_NAME)) {
				CESProduct* c = (CESProduct*)cpl_i; // coupling
				*f_line<<(_Cmplx?("C_SP4("):("R_SP4("))<<c->get_a()->toString()<<", "<<c->get_b()->toString()<<")";
			} else if(CEUtil::equals(cpl_i->ClassName(), CEFuncBondarev::CLASS_NAME)) {
				CEFuncBondarev* fb = (CEFuncBondarev*)cpl_i; CExpression* c1=fb->GetCouple1(); CExpression* c2=fb->GetCouple2();
				if ((_Cmplx) && (c1!=NULL) && (c2!=NULL)) {
					*f_line<<"C_FBONDAREV("<<fb->GetNo()<<", "<<c1->toString()<<", "<<c2->toString()<<")";
				} else {
					if (!_Cmplx) { dbg->log(0, "ERROR. FORTRAN output for CEFuncBondarev object requires COMPLEX VALUE. Please allow COMPLEX_NUMBERS & restart task"); }
					else { dbg->log(0, "ERROR. FORTRAN output cannot be created due to vector CEFuncBondarev object", fb); }
					f_line->AddString("ERROR");
				}
			}
		}
		m_codeLang->OutCodeLine(p_ofs, f_line);
	}
	if (cpl_num>0) { *p_ofs << "\n"; }

	if(m_general_conventions&CLFileManager::FT_CPP) {
		m_codeLang->StartCommentLine(p_ofs,true); *p_ofs<<m_codeLang->GetTypePrefixBool()<<" bLog = (m_nCalls<10); // debug\n";
		*p_ofs << "\n";
	}

	bool need_ENDIF=false;
	// denominators 1/0 check
	if (dens_check_num>0) {
		m_codeLang->StartCommentLine(p_ofs); *p_ofs << "Denominators==0 check" << "\n";
		if(m_general_conventions&CLFileManager::FT_CPP){ *f_line<<CCodeLanguage::TYPE_BOOL_CPP<<" "; }
		*f_line<<var_DENS_OK<<" = "<<((m_general_conventions&CLFileManager::FT_CPP)?"true":"1"); m_codeLang->OutCodeLine(p_ofs, f_line);
		if(m_general_conventions&CLFileManager::FT_CPP){ //done previously for FORTRAN
			*f_line<<type_ID<<" "<<var_DENS_TEMP; m_codeLang->OutCodeLine(p_ofs, f_line);
		}
		for(i=0; i<dens_check_num; i++) {
			CFortranFunctionInfo* d_fi = (CFortranFunctionInfo*)m_Dens_functions->GetAt(i);
			CEPtrArray* d_pars = d_fi->par_list(); long d_pars_num=d_pars->size();
			*f_line<<var_DENS_TEMP<<" = "; m_codeLang->AddFuncCallName(f_line,d_fi->func_name()); *f_line<<"(";
			need_comma=false;
			for(j=0; j<d_pars_num; j++) {
				if(need_comma){ *f_line<<", "; }else{ need_comma = true; }
				CExprContainer* par=(CExprContainer*)d_pars->GetAt(j);
				*f_line<<par->get_expr()->toString();
			}
			*f_line<<")"; m_codeLang->OutCodeLine(p_ofs, f_line);
			if(m_general_conventions&CLFileManager::FT_CPP){
				*f_line<<"if(fabs(("<<m_codeLang->GetTypePrefixReal()<<")"<<var_DENS_TEMP;
				if(bDensMinIs0){ *f_line<<")==0."; }else{ *f_line<<")<"<<var_DENS_MIN; }
				*f_line<<") "<<var_DENS_OK<<"=false";
			}else{
				*f_line<<"IF(DABS(DBLE("<<var_DENS_TEMP;
				if(bDensMinIs0){ *f_line<<")).EQ.0D0"; }else{ *f_line<<")).LT."<<var_DENS_MIN; }
				*f_line<<") "<<var_DENS_OK<<"=0";
			}
			m_codeLang->OutCodeLine(p_ofs, f_line);
		}
		if(m_general_conventions&CLFileManager::FT_CPP){
			m_codeLang->StartCommentLine(p_ofs,true); *p_ofs<<"if(!"<<var_DENS_OK<<") std::cout<<\""<<var_DENS_OK<<"==false\";\n";
		}else{
			m_codeLang->StartCommentLine(p_ofs,true); *p_ofs<<"IF("<<var_DENS_OK<<".EQ.0) print *,\""<<var_DENS_OK<<"=0\"\n";
		}

		//DENS_LOOP_I, DENS_LOOP_MAX
		if (basisA_num>0) { //if random basis vectors are generated
			m_codeLang->StartCommentLine(p_ofs); *p_ofs << "If denominators check failed.." << "\n";
			*f_line<<m_codeLang->GetOpIf()<<" ("; if(m_general_conventions&CLFileManager::FT_CPP){ *f_line<<"!"<<var_DENS_OK; }else{ *f_line<<var_DENS_OK<<m_codeLang->GetOpEqu()<<"0"; }
			*f_line<<")"<<m_codeLang->GetOpThen();m_codeLang->OutCodeLine(p_ofs, f_line,-1,true);
			m_codeLang->IncIndent();
			if(m_general_conventions&CLFileManager::FT_CPP){
				*f_line<<var_DENS_LOOP_I<<"++"; m_codeLang->OutCodeLine(p_ofs, f_line);
			}else{
				*f_line<<var_DENS_LOOP_I<<"="<<var_DENS_LOOP_I<<"+1"; m_codeLang->OutCodeLine(p_ofs, f_line);
			}
			m_codeLang->StartCommentLine(p_ofs); *p_ofs << "... generate another RANDOM basis vector" << "\n";
			if(m_general_conventions&CLFileManager::FT_CPP){
				*f_line<<"if("<<var_DENS_LOOP_I<<" >= "<<var_DENS_LOOP_MAX<<") break"; m_codeLang->OutCodeLine(p_ofs, f_line);
			}else{
				*f_line<<"IF("<<var_DENS_LOOP_I<<".LT."<<var_DENS_LOOP_MAX<<")GOTO 39"; m_codeLang->OutCodeLine(p_ofs, f_line);
			}
			m_codeLang->StartCommentLine(p_ofs); *p_ofs << "... or stop the loop ad declare result as 0" << "\n";
			if (pol_num==0) {
				f_line->AddString("ERROR 1/0 devision, ZERO value returned"); m_codeLang->OutConsoleWriteLine(p_ofs, f_line);
				*f_line<<s_ReturnVar.getData()<<"=0"; m_codeLang->OutCodeLine(p_ofs, f_line);
				f_line->AddString("RETURN"); m_codeLang->OutCodeLine(p_ofs, f_line);
				m_codeLang->DecIndent();
				f_line->AddString(m_codeLang->GetOpEndIf()); m_codeLang->OutCodeLine(p_ofs, f_line, -1, true); //(!bDensOk)
			} else {
				f_line->AddString("ERROR 1/0 devision, ZERO value invloved"); m_codeLang->OutConsoleWriteLine(p_ofs, f_line);
				m_codeLang->DecIndent(); f_line->AddString(m_codeLang->GetOpElse()); m_codeLang->OutCodeLine(p_ofs, f_line, -1, true); m_codeLang->IncIndent();
				need_ENDIF=true;
			}
		} else {
			*f_line<<m_codeLang->GetOpIf()<<" ("; if(m_general_conventions&CLFileManager::FT_CPP){ *f_line<<"!"<<var_DENS_OK; }else{ *f_line<<var_DENS_OK<<m_codeLang->GetOpEqu()<<"0"; }
			*f_line<<")"<<m_codeLang->GetOpThen();m_codeLang->OutCodeLine(p_ofs, f_line,-1,true);
			m_codeLang->IncIndent();
			if (pol_num==0) {
				f_line->AddString("ERROR 1/0 devision, ZERO value returned"); m_codeLang->OutConsoleWriteLine(p_ofs, f_line);
				*f_line<<s_ReturnVar.getData()<<"=0"; m_codeLang->OutCodeLine(p_ofs, f_line);
				if(m_general_conventions&CLFileManager::FT_CPP){ *f_line<<"return "<<s_ReturnVar.getData(); }else{ *f_line<<"RETURN"; }
				m_codeLang->OutCodeLine(p_ofs, f_line);
				m_codeLang->DecIndent(); f_line->AddString(m_codeLang->GetOpEndIf()); m_codeLang->OutCodeLine(p_ofs, f_line, -1, true);
			} else {
				f_line->AddString("ERROR 1/0 devision, ZERO value returned"); m_codeLang->OutConsoleWriteLine(p_ofs, f_line);
				m_codeLang->DecIndent(); f_line->AddString(m_codeLang->GetOpElse()); m_codeLang->OutCodeLine(p_ofs, f_line, -1, true); m_codeLang->IncIndent();
				need_ENDIF=true;
			}
		}
		*p_ofs << "\n";
	}

	// ************************************************** main function call
	m_codeLang->StartCommentLine(p_ofs); *p_ofs << "Main function call" << "\n";
	CEPtrArray* pars_origin = func_inf->par_list(); long par_orig_num = pars_origin->size(); need_comma=false;
	if (src_ampl) {
		*f_line<<"AMPL_VAL="; m_codeLang->AddFuncCallName(f_line,func_inf->func_name()); *f_line<<"(";
		for(i=0; i<par_orig_num; i++) {
			CExprContainer* par= (CExprContainer*)pars_origin->GetAt(i);
			CExpression* sym = par->get_expr(); if((use_COMMON)&&(FindInCommonBlock(sym)>=0)){dbg->log(0,"WARNING: COMMON item in par-list",sym);continue;}
			if (need_comma) { f_line->AddString(", "); } else { need_comma = true; }
			if(par->get_info()==CEAbstractFunction::PAR_4VEC_REAL)f_line->AddString("*"); //all vectors are pointers here
			f_line->AddString(sym->toString());
		}
		f_line->AddString(")"); m_codeLang->OutCodeLine(p_ofs, f_line);
		*f_line<<s_ReturnVar.getData()<<" = ";
		if (pol_num>0) { *f_line<<s_ReturnVar.getData()<<"+"; } // averaging
		*f_line<<"AMPL_VAL*DCONJG(AMPL_VAL)";
	} else {
		*f_line<<s_ReturnVar.getData()<<" = ";
		if (pol_num>0) { *f_line<<s_ReturnVar.getData()<<"+"; } // averaging
		m_codeLang->AddFuncCallName(f_line,func_inf->func_name()); *f_line<<"(";
		for(i=0; i<par_orig_num; i++) {
			CExprContainer* par= (CExprContainer*)pars_origin->GetAt(i);
			CExpression* sym = par->get_expr(); if((use_COMMON)&&(FindInCommonBlock(sym)>=0)){dbg->log(0,"WARNING: COMMON item in par-list",sym);continue;}
			if (need_comma) { f_line->AddString(", "); } else { need_comma = true; }
			if(par->get_info()==CEAbstractFunction::PAR_4VEC_REAL)f_line->AddString("*"); //all vectors are pointers here
			f_line->AddString(sym->toString());
		}
		f_line->AddString(")");
	}
	m_codeLang->OutCodeLine(p_ofs, f_line);
	if (need_ENDIF) { m_codeLang->DecIndent(); f_line->AddString(m_codeLang->GetOpEndIf()); m_codeLang->OutCodeLine(p_ofs, f_line, -1, true); }

	if (pol_num>0) { // polarization averaging
		m_codeLang->StartCommentLine(p_ofs); *p_ofs << "End of \"sum-of-polarizations\" cycle" << "\n";
		for(i=pol_num-1; i>=0; i--) { // POL_I cycle NO: 20,21,22,23..
			CExpression* vec = ave_pol_list->GetAt(i);
			int pID = kin->find_polarization_vec(GetSymOrigin(vec)); // >0 means initials
			CEParticle* part = kin->get_particle_by_ID(pID); if (part->get_polarization_info() != 0) continue;
			m_codeLang->DecIndent(); m_codeLang->AddOpEndFor(f_line); m_codeLang->OutCodeLine(p_ofs, f_line, 20+i, true);
		}
		
		bool bNeesComt=false;
		for(i=pol_num-1; i>=0; i--) { // POL_I cycle NO: 20,21,22,23..
			CExpression* vec = ave_pol_list->GetAt(i);
			int pID = kin->find_polarization_vec(GetSymOrigin(vec)); // >0 means initials
			CEParticle* part = kin->get_particle_by_ID(pID); if (part->get_polarization_info() != 0) continue;
			if (pID>0) {
				if (bNeesComt) { bNeesComt=false; m_codeLang->StartCommentLine(p_ofs); *p_ofs << "Average polarizations" << "\n"; }
				int _PolNum = part->GetPolarizationsNumber();
				*f_line<<s_ReturnVar.getData()<<"="<<s_ReturnVar.getData()<<"/"<<_PolNum; m_codeLang->OutCodeLine(p_ofs, f_line);
			}
		}
	}

	if(m_general_conventions & CLFileManager::FT_CPP){
		*p_ofs << "\n";

		//debug output template
		m_codeLang->StartCommentLine(p_ofs,true);
		*f_line<<m_codeLang->GetOpIf()<<"(bLog)std::cout<<\"["; if(m_CppClassName!=NULL){ *f_line<<m_CppClassName<<"::"; }
		*f_line<<_EntryTitle<<"] result"; if (_AllowQuarksSwap) { *f_line<<" (\"<<"<<QOrig<<"<<\")"; }
		*f_line<<": \"<<"<<s_ReturnVar.getData()<<"<<std::endl; // debug\n"; *p_ofs<<f_line->getData(); f_line->Reset();
		m_codeLang->StartCommentLine(p_ofs,true);
		*f_line<<m_codeLang->GetOpIf()<<"(bLog && (m_nCalls>=10)) exit(0); // debug\n"; *p_ofs<<f_line->getData(); f_line->Reset();

		//return value
		*f_line<<"return "<<s_ReturnVar.getData(); m_codeLang->OutCodeLine(p_ofs, f_line);
	}

	if(m_general_conventions&CLFileManager::FT_CPP) { m_codeLang->DecIndent(); }
	*f_line << m_codeLang->GetOpEndProc(); m_codeLang->OutCodeLine(p_ofs, f_line, -1, true);
	if (lt_UV_val != NULL) { delete lt_UV_val; lt_UV_val=NULL; }
	if (lt_IR_val != NULL) { delete lt_IR_val; lt_IR_val=NULL; }
	if (DR_mass2 != NULL) { delete DR_mass2; DR_mass2=NULL; }
	if (DR_mass2_F != NULL) { delete DR_mass2_F; DR_mass2_F=NULL; }
	delete par_list; delete vec_list; delete NonVecCOMM_list; delete EQUList;
	delete vec_par_list; delete vec_mom_list; delete vec_pol_list; delete ave_pol_list; delete vec_basisA_list; delete vec_basisB_list; delete vec_unkn_list;
	if (s_EntryTitle != NULL) { delete s_EntryTitle; }
	//*p_ofs << "\n";
	delete f_line;
}

// no check are performed here
void CFFortran::addExprString(ofstream* p_ofs, CExpression* val) {
	CEString* str = val->toString();
	*p_ofs << str->getData();
	delete str;
}

CExpression* CFFortran::GetSymOrigin(CExpression* _sym) {
	CExpression* res = _sym;
	if (m_syms_replaced!=NULL) {
		int s=m_syms_replaced->size();
		for(int i=0;i<s;i++) {
			CExprContainer* cart = (CExprContainer*)m_syms_replaced->GetAt(i);
			if (_sym->equals(cart->get_expr())) { res=cart->get_info_expr(); break; }
		}
	}
	return res;
}

/*
*	Turns some complicated objects to FORTRAN-storable SYMBOLS
*/
CExpression* CFFortran::validate_objects(CExpression* expr) {
	dbg->log(10, "validate_objects", expr);

	//1. CKM matrix --> symbols
	CExpression* res = CQuarkMixMatrix::QMMConvertToSymbols(expr, false);

	//2. Complicated vectors (like 'p1-p2') -> symbols
	//Note: the code moved to 'validate_syms()' to avoid extra (slow!) 'collect_syms()' call

	return res;
}

/*
*	Called for SOURCE expression, all symbols turns to F-notation.
*	TODO: "toFortranSyms" function also turns SYMBOL to F-style, hence it should be used here!
*	This implementation seems to be replaceble by for (i=0; i<s; i++) {... toFortranSyms(); ...}
*	Note: validate_sym is also called from other points.
*/
CExpression* CFFortran::validate_syms(CExpression* expr, CEPtrArray* _sym_origins) {
	dbg->log(30, "validate_syms ", expr);
	CSequence sym_list; collect_syms(expr, &sym_list, FORTRAN_OBJ_SYMBOLS|FORTRAN_ALL_VECTORS);

	CEReplList r_list; CEReplList r_list_vec; //Note: (p\_1-p\_2)->p1mp2 rule must be used prior to {p\_i->p_i} rules
	long s = sym_list.size();
	for(long i=0; i<s; i++) {
		// Note: sym_list may contain 'CESymbol' or 'CESum'(due to FORTRAN_ALL_VECTORS)
		CExpression* eSym=sym_list.GetAt(i);
		if(eSym->ClassName() == CESymbol::CLASS_NAME) {
			//ver 1: via 'validate_sym' (DEPRECATED)
			//CESymbol* sym_repl = validate_sym((CESymbol*)eSym,_sym_origins);
			//if (sym_repl != ((CESymbol*)eSym)) { r_list.add(eSym, sym_repl, false); delete sym_repl; }
			//ver 2: via 'toFortranSyms'
			CEString* sSymOrig=eSym->toString(); CEString* sSym=CFFortran::toFortranSyms(sSymOrig);
			if(sSym != sSymOrig) { // sSym is never NULL, sSymOrig is already deleted by 'toFortranSyms()'
				CESymbol sym_repl(sSym->getData()); dbg->log(2, "<CODE> new symbol involved:", sSym->getData());
				r_list.add(eSym, &sym_repl, false);
				if(_sym_origins!=NULL){ _sym_origins->add(new CExprContainer(&sym_repl, 0, 0, eSym)); }
			}
			delete sSym;
		} else { // Complicated vector
			//dbg->log(0,"Unexpected object in SYMBOLs collection: ",eSym); CEException::halt();
			CEString* sObj=eSym->toString();
			for(int iTmp=0; iTmp<sObj->length(); iTmp++) { //Modify 'sObj' to separate 'p1-p2' vector and 'p1_p2' coupling
				char c=sObj->GetChar(iTmp); if(c=='-')sObj->SetChar(iTmp,'m');else if(c=='+')sObj->SetChar(iTmp,'p');else if(c=='*')sObj->SetChar(iTmp,'t');else if(c=='/')sObj->SetChar(iTmp,'d');
			}
			sObj=CFFortran::toFortranSyms(sObj); //sObj is never NULL
			dbg->log(2, "<CODE> new symbol involved:", sObj->getData());
			CESymbol sRepl(sObj->getData()); r_list_vec.add(eSym,&sRepl,false); delete sObj;
			//TODO: check if the only SYMBOLs are expected in '_sym_origins'
			//if(_sym_origins!=NULL){ _sym_origins->add(new CExprContainer(&sRepl, 0, 0, eSym)); }
		}
	}

	CExpression* res=expr;
	if(r_list_vec.size()>0){
		dbg->log(2, "validate complicated vectors replace list: ", &r_list_vec);
		CExpression* res2=r_list_vec.ApplyTo(res, false); if((res2!=res)&&(res!=expr)&&(res!=NULL)){delete res;} res=res2;
	}
	if(r_list.size()>0){
		dbg->log(2, "validate_syms replace list: ", &r_list);
		CExpression* res2=r_list.ApplyTo(res, false); if((res2!=res)&&(res!=expr)&&(res!=NULL)){delete res;} res=res2;
	}
	return res;
}

/*
*	Old implementation is DEPRECATED. It had behaviour DIFFERENT to the newest one!
*	[old]validate_sym(m\_1)-->m_1
*	toFortranSyms(m\_1)-->m1
*	toFortranSyms($Unique)-->_Unique
*/
CESymbol* CFFortran::validate_sym(CESymbol* sym, CEPtrArray* _sym_origins) { // does not affect input, but may return it
	if (sym==NULL) return NULL;
	CESymbol* res = sym;
	CEString* sSymOrig=sym->toString(); CEString* sSym=CFFortran::toFortranSyms(sSymOrig);
	if(sSym != sSymOrig) { // sSym is never NULL, sSymOrig is already deleted by 'toFortranSyms()'
		res = new CESymbol(sSym->getData());
		dbg->log(2, "<FORTRAN> new symbol involved:", res);
		if(_sym_origins!=NULL){ _sym_origins->add(new CExprContainer(res, 0, 0, sym)); }
	}
	delete sSym;
	return res;

	/*[old implementation]
	if (sym==NULL) return NULL;
	CEString* sym_str = sym->toString();
	char* data=sym_str->getData(); bool bDelData=false;
	
	bool is_valid = true;
	bool loop = true;
	int data_len = CEUtil::length(data);
	for(;loop;) {
		int remove_syms = 0; // num syms to extract
		long pos = CEUtil::findSymbol(data, '\\');
		if (pos >= 0) {
			if ((pos+1 < data_len) && (*(data+pos+1) == '_')) remove_syms=1;
			else remove_syms=1;
		}
		
		int truncate_brackets = 1;
		if (pos < 0) { pos = CEUtil::findSymbol(data, '+'); if (pos>=0) remove_syms=1; }
		if (pos < 0) { pos = CEUtil::findSymbol(data, '-'); if (pos>=0) remove_syms=1; }
		if (pos < 0) { pos = CEUtil::findSymbol(data, '*'); if (pos>=0) remove_syms=1; }
		if (pos < 0) { pos = CEUtil::findSymbol(data, '/'); if (pos>=0) remove_syms=1; }
		if (pos < 0) { pos = CEUtil::findSymbol(data, '('); if (pos>=0) remove_syms=truncate_brackets; }
		if (pos < 0) { pos = CEUtil::findSymbol(data, ')'); if (pos>=0) remove_syms=truncate_brackets; }
		if (pos < 0) { pos = CEUtil::findSymbol(data, '['); if (pos>=0) remove_syms=truncate_brackets; }
		if (pos < 0) { pos = CEUtil::findSymbol(data, ']'); if (pos>=0) remove_syms=truncate_brackets; }
		
		if (pos >= 0) {
			if(!bDelData){ data = CEUtil::copyString(data); bDelData=true; } //first time
			if(is_valid){ dbg->log(2, "<FORTRAN> non-valid symbol:", data); is_valid=false; } //first time

			if (remove_syms > 0) {
				char* a = CEUtil::removeSubString(data, pos, remove_syms);
				if(bDelData){ CMemoryManager::mfree(data); }
				data=a; bDelData=true;
			} else {
				*(data + pos)='_';
			}
		}
		else{loop = false;}
	}

	CESymbol* sym_repl = sym;
	if (!is_valid) {
		sym_repl = new CESymbol(data); dbg->log(2, "<FORTRAN> new symbol involved:", data);
		if(bDelData){ CMemoryManager::mfree(data); }
		if (_sym_origins!=NULL) { _sym_origins->add(new CExprContainer(sym_repl, 0, 0, sym)); }
	}
	delete sym_str;
	return sym_repl;
	*/
}

/*
*	deletes input string
*	used for FUNCTION titles, scalar souplings.
*	TODO: should be used also in validate_syms() function
*	Note. old was: 'return original string of delete it & return new' ==> Check new to be at DIFFERENT pointer!
*/
CEString* CFFortran::toFortranSyms(CEString* str) {
	char* src = str->getData();

	//Replase '$'-symbols by '_' (unique operands)
	for(int i=0; i<str->length(); i++) { if(*(src+i)=='$') { *(src+i)='_'; } }

	//Skip all other valid symbols
	char* res = CEUtil::copyStringSyms(str->getData(),
		CEUtil::UTL_COPY_SYMBOLS | CEUtil::UTL_COPY_NUMBERS | CEUtil::UTL_COPY_NO_FIRST_NUMBER | CEUtil::UTL_COPY_SKIP_BSLASH_CMD,
		"_");
	if((res==NULL) || (CEUtil::length(res) == 0)){
		// Return original symbol --> error will be visible as compiller error
		dbg->log(0, "ERROR: toFortranString() ==> empty string", str); if(res!=NULL)CMemoryManager::mfree(res); return str;
	}
	if(CEUtil::equals(res,str->getData())) {
		// Return original symbol if unchanged
		CMemoryManager::mfree(res); return str;
	}
	CEString* res_str=new CEString(res); CMemoryManager::mfree(res);
	delete str; //delete after 'res_str' creation to allow (old != new) check!
	return res_str;
}

/*
*	fills out _vec_couplings array by CExprContainer(src: coupling; info: replaced symbol);
*	NULL array means no couplings data are reqiured
*/
CExpression* CFFortran::scalar_couplings_to_syms(CExpression* expr, CEPtrArray* _vec_couplings) {
	CSequence* scalar_couplings_list = new CSequence();
	collect_syms(expr, scalar_couplings_list, FORTRAN_OBJ_2SCALAR_COUPLINGS|FORTRAN_OBJ_4SCALAR_COUPLINGS);
	CEReplList r_list;
	long s = scalar_couplings_list->size();
	dbg->log(10, s, "different couplings found: %ld");
	long i;
	for (i=0; i<s; i++) {
		CExpression* c_item = scalar_couplings_list->GetAt(i);
		CEString* data = NULL;

		if(CEUtil::equals(c_item->ClassName(), CEAntiSymmProd::CLASS_NAME)) {
			CEAntiSymmProd* a_pr = (CEAntiSymmProd*)c_item;
			data = a_pr->toString(m_STORE_EXPR_AS_FLAG);
		} else if(CEUtil::equals(c_item->ClassName(), CESProduct::CLASS_NAME)) {
			CESProduct* scc = (CESProduct*)c_item;
			data=scc->get_a()->toString(); *data<<"_"<<scc->get_b()->toString();
		} else if(CEUtil::equals(c_item->ClassName(), CEFuncBondarev::CLASS_NAME)) {
			data = ((CEFuncBondarev*)c_item)->toString(m_STORE_EXPR_AS_FLAG);
		} else { dbg->log(0, "CFFortran::scalar_couplings_to_syms, unexpected symbol collected", c_item); continue; }

		data = toFortranSyms(data);

		CESymbol* sym_repl = new CESymbol(data->getData());
		dbg->log(2, "<FORTRAN> new symbol involved:", data->getData());
		delete data;
		r_list.add(c_item, sym_repl, false);
		if (_vec_couplings != NULL) { _vec_couplings->add(new CExprContainer(c_item, 0, 0, sym_repl)); }
		delete sym_repl;
	}

	if (r_list.size() > 0) { return expr->applyRule(&r_list); } //don't delete source
	return expr;
}

CExpression* CFFortran::ReplPowersBySymbols(CExpression* src) { // changes powers to symbols; does not deletes input, but may return it
	// find MAXIMAL possible powers to involve as single symbol
	//printf("\n--------------------- ReplPowersBySymbols ------------------------\n");

	dbg->log(10, "ReplPowersBySymbols", src);
	CSequence sym_list; collect_syms(src, &sym_list, FORTRAN_OBJ_SYMBOLS);
	long i;

	// exclude constants
	if ((!m_bAllowConstantsPowers) && ((m_conventions & CLFileManager::FT_DONOT_INVOLVE_CONSTANTS)==0)) {
		for (i=0; i<sym_list.size(); i++) {
			if (isSymbolConstant(sym_list.GetAt(i))) { sym_list.RemoveAt(i); i--; }
		}
	}

	bool del_src = false;
	long s = sym_list.size();
	CEPtrArray arrReplSyms; int iReplNumExpected=0;
	for (i=0; i<s; i++) {
		CESymbol* sym = (CESymbol*)sym_list.GetAt(i);
		int pow_found = 0;
		CExpression* src_new = CRMaxPowerToSym::MaxPowerToSym(src, sym, &pow_found, &arrReplSyms, "P", false);
		if ((src_new != src) && (src_new != NULL)) { if(del_src)delete src; src=src_new; }
		dbg->log(45, pow_found, "pow_found: %ld, ", sym);

		if(pow_found<1) { dbg->log(0, pow_found, "ERROR: Unexpected minimal power found (%ld).", sym); CEException::halt(); }
		// check src_new for symbol remaimings
		if (pow_found > 1) {
			iReplNumExpected++; if(arrReplSyms.size()!=iReplNumExpected) { dbg->log(0, arrReplSyms.size(), iReplNumExpected, "Internal ERROR: Unexpected size of replaced symbols list (%ld, expected %ld).",&arrReplSyms); CEException::halt(); }
			if(m_bAllowConstantsPowers && (isSymbolConstant(sym))) {
				CSequence* rec=new CSequence(); rec->addItem(sym->clone()); rec->addItem(CENumber::GetNumber(pow_found)); rec->addItem(((CExpression*)arrReplSyms.GetAt(arrReplSyms.size()-1))->clone());
				m_ConstantsPowersList.addItem(rec);
			}
			if (CRContainCheck::search(src, sym)) { dbg->log(0, "INTERNAL ERROR: symbol remains after power substitute", src, sym); CEException::halt(); }
		}
	}
	if(arrReplSyms.size()>0) { dbg->log(2, "ReplPowersBySymbols() list of new symbols:", &arrReplSyms); }
	if(m_ConstantsPowersList.size() > 0) { dbg->log(2, "Constants powers list:", &m_ConstantsPowersList); }
	return src;
}

/************************************************************
***					 Library procedures
************************************************************/
/*
*	Note: VI parameters are in dens-data object ==> separated implementation is required
*/
CExpression* CFFortran::VIToUsualFunctions(CExpression* src) { //may delete input
	//return src; //Involve old implementation of VI

	CEPtrArray data_arr;
	data_arr.add(CENumber::GetNumber(m_conventions));
	data_arr.add(CENumber::GetNumber(m_STORE_EXPR_AS_FLAG));
	data_arr.add(new CEObjRef(&m_fRegistry)); //to avoid cloning/deleting
	data_arr.add(new CEObjRef(&m_constRegistry)); //to avoid cloning/deleting

	CESocketRule r((void*)CFFortran::VIToUsualFunctionsImpl, &data_arr, CESocketRule::USE_EXTERNAL_DATA_ARRAY);
	CExpression* res = r.ApplyTo(src);
	return res;
}
CExpression* CFFortran::VIToUsualFunctionsImpl(CExpression* src, CEPtrArray* data_arr) {
	if ((src == NULL) || (data_arr == NULL)) return src;
	if(!src->getProperty(CEProperties::PROP_MASK_IS_VIRT_INTEGRAL)) return src;

	//parameters
	long iConvs=((CENumber*)data_arr->GetAt(0))->getLong();
	bool bUseLoopTools=((iConvs & CLFileManager::FT_LOOPTOOLS) == CLFileManager::FT_LOOPTOOLS);
	bool bSeparateVI=((iConvs & CLFileManager::FT_SEPARATE_VIRT_INTEGRALS) == CLFileManager::FT_SEPARATE_VIRT_INTEGRALS);	
	int iStoreAsFlag=((CENumber*)data_arr->GetAt(1))->getLong();
	CFFortran::CFuncRegistry* fRegistry=(CFFortran::CFuncRegistry*)(((CEObjRef*)data_arr->GetAt(2))->GetRef());
	CFFortran::CConstRegistry* constRegistry=(CFFortran::CConstRegistry*)(((CEObjRef*)data_arr->GetAt(3))->GetRef());

	CVirtIntegral* vi = (CVirtIntegral*)src;
	CVIDensData* dd = vi->get_dens_data();
	int arg_num = dd->size(); int i;

	// The VI-0 with scalar dens-data are only compartible here
	bool is_valid = ((dd->isInvariant()) && ((vi->ClassName()==CVirtInt0::CLASS_NAME) || (vi->ClassName()==CVirtInt1::CLASS_NAME)));
	if (!is_valid) { dbg->log(0, "ERROR: No valid CODE interface exist for virtual integral: ", vi); CEException::halt(); }

	DWORD tRes=fRegistry->GetReturnType(vi); // ==vi->GetResultType();
	//LoopTools: CEAbstractFunction::PAR_COMPLEX
	//Non-LoopTools: CEAbstractFunction::PAR_REAL_ABSTRACT or CEAbstractFunction::PAR_COMPLEX_ABSTRACT

	CEString* f_title = vi->getTitleAdv(iConvs, iStoreAsFlag);
	f_title = CFFortran::toFortranSyms(f_title); // convert title to FORTRAN standard
	if (bSeparateVI) { //Adjust function title to parameters set
		for(int i=0; i<arg_num; i++) {
			//Old: fi_curr = to_fortran_string(fi_curr, dd->GetAt(i), ENV_BRACKETS_NEVER);
			//Old: res->AddString(fi_curr->extract_res_value());
			f_title->AddString(dd->GetAt(i)->toString(iStoreAsFlag));
		}
		f_title = CFFortran::toFortranSyms(f_title); // remove extra symbols from function title VIm**2m**2()...
	}

	CFFortran::CFuncRegistry::CFuncRegistryInfo* fInfo = new CFFortran::CFuncRegistry::CFuncRegistryInfo(
		f_title->getData(),tRes,0,
		f_title->getData(),	//Correct for 'C++' language, not used otherwise
		f_title->getData(), //Correct for 'F' language, not used otherwise
		NULL,
		CEAbstractFunction::PAR_REAL_ABSTRACT
		);
	CEPtrArray fPars;
	// params-> [LoopTools->INT],REAL,REAL... result-> DOUBLE COMPLEX

	if (bUseLoopTools) { // add 'ID' parameter for LoopTools Xiii-integrals
		if ((vi->ClassName()==CVirtInt1::CLASS_NAME) && (dd->getDensNumber() >= 3)) { //c,d,e
			char lt_sym='a'+(dd->getDensNumber()-1); // compose LT-defined parameters like "cc112"
			CEString sTmp; sTmp.addChar(lt_sym); sTmp.addChar(lt_sym); sTmp<<vi->GetHEPId();

			fPars.add(new CESymbol(sTmp.getData()));
			fInfo->m_iParTypes.Add(CEAbstractFunction::PAR_INTEGER);
			constRegistry->AddConstInfo(sTmp.getData(),CEAbstractFunction::PAR_INTEGER,CConstRegistry::C_LOOPTOOLS);
		}
	}

	int iMassFirstPos = dd->getFirstMassPos();	//used to add **2 for masses
	if (iMassFirstPos<0){
		iMassFirstPos=arg_num;	//no add **2 if no masses pos info available (non-invariant DD?)
		if(bUseLoopTools) { dbg->log(0,"ERROR: can't implement LoopTools call. The position of mass parameter is unavailable.",vi); CEException::halt(); }
	}
	for(i=0; i<arg_num; i++) {
		CExpression* param=dd->GetAt(i); bool del_param=false;
		if (bUseLoopTools && (i>=iMassFirstPos)) { // masses should be squared for LoopTools
			if(!CEBasic::isZero(param)){ param=param->sqra(); del_param=true; }
		}

		// LoopTools VI parameters are real*8/double, non-LoopTools VI we consider as real*8/double too.
		fPars.add(del_param?param:(param->clone()));
		fInfo->m_iParTypes.Add(CEAbstractFunction::PAR_REAL8);
	}

	fRegistry->AddFunctionInfo(fInfo);
	CEFunction* res=new CEFunction(f_title->getData(),&fPars,true,false);
	delete f_title;

	src->copyAttributesTo(res,true);
	return res;
}
